// CPlotWindow.cpp
//
// Copyright 1996--2000 Krishna Myneni 
// Creative Consulting for Research & Education
//
// This software is provided under the terms of the GNU General Public
// License.
//
// Last Revised: 4-15-2000

char* RELEASE = "Rls. 04-15-2000  (kForth Rls. 03-07-2000)";

#include "consts.h"
#include <math.h>
#include <strstream.h>
#include <vector>
#include "CPlotWindow.h"
#include "ColorTable.h"
#include "fbc.h"
#include "ForthCompiler.h"
#include "ForthVM.h"

#define PRINT_TEMP_FILE "xyplot-output.ps"

// Functions and data defined externally

extern void SortRect (CRect *);
extern int NumberParse (float*, char*);
extern int CompileAE (vector<byte>*, char* exp);
extern void strupr (char*);
extern int AddToHeader (char*, char*, bool);
extern int LoadForthFile (char*);

extern void FileMenuCB (Widget, caddr_t, XmAnyCallbackStruct*);
extern void DataMenuCB (Widget, caddr_t, XmAnyCallbackStruct*);
extern void PlotMenuCB (Widget, caddr_t, XmAnyCallbackStruct*);
extern void MathMenuCB (Widget, caddr_t, XmAnyCallbackStruct*);

extern Widget TopLevel;

// Global data

char* SymbolNames[7] =
{
  "LINE", "DASHED", "POINT", "BIG_POINT", 
  "LINE_PLUS_POINT", "STICK", "HISTOGRAM"};

// Global menu data

char* FileOptions[16] = 
  {"New", "Open", "Save", "Save Options", 
   "Print", "Exit", "", "",
   "", "", "", "",
   "", "", "", ""};
Widget FileWidgets[16];

char* EditOptions[16] = 
  {"Copy", "", "", "",
    "", "", "", "",
    "", "", "", "",
    "", "", "", ""};
Widget EditWidgets[16];

char* DataOptions[16] = 
  {"Header", "Delete", "Duplicate", "",
    "", "", "", "",
    "", "", "", "",
    "", "", "", "" };
Widget DataWidgets[16];

char* PlotOptions[16] = 
  { ">View", "Extrema", "Pick", "Drop", 
    "Grid", "Color", ">Symbol", "",
    "", "", "", "",
    "", "", "", "" };
Widget PlotWidgets[16];

char* MathOptions[16] = 
  { 
    "Expression", "", "", "", 
    "", "", "", "",
    "", "", "", "",
    "", "", "", ""};
Widget MathWidgets[16];

char* HelpOptions[16] = 
  {"About", "", "", "",
   "", "", "", "",
   "", "", "", "",
   "", "", "", "" };
Widget HelpWidgets[16];

// Widget ViewWidgets[5];
// char* ViewOptions[] = {"Cartesian", "Polar", "Aspect",
//  "Back", "Forward"};
// Widget SymbolWidgets[5];
// char* SymbolOptions[] = {"Line", "Point", "Line+Pt",
//  "Stick", "Toggle"};
// Widget ArithmeticWidgets[4];
// char* ArithmeticOptions[] = {"Add", "Subtract", "Multiply",
//			     "Divide"};
// Widget FitWidgets[1];
// char* FitOptions[] = {"Polynomial"};
// Widget FftWidgets[2];
// char* FftOptions[] = {"Forward", "Inverse"};
// Widget ComplexWidgets[5];
// char* ComplexOptions[] = {"Magnitude", "Phase", "Conjugate",
//  "Real(Y)", "Imag(Y)"};

//---------------------------------------------------------------

CPlotWindow::CPlotWindow(int argc, char* argv[])
{
  Arg al[10];
  int ac;
  m_pFileName = new char[256];
  *m_pFileName = '\0';

  CxDC* pDc = new CxDC (DEV_X_WINDOW, 600, 640);

  // Create a form widget
  
  ac = 0;
  m_nForm = XtCreateManagedWidget("form", xmFormWidgetClass,
    TopLevel, al, ac);
  ac = 0;
  XtSetArg (al[ac], XmNheight, 480);
  ++ac;
  XtSetArg (al[ac], XmNwidth, 600);
  ++ac;
  XtSetValues (TopLevel, al, ac);

  m_pDb = NULL;
  m_pDi = NULL;

  // Create a menu bar and attach it to the form.
  ac = 0;
  XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); 
  ac++;
  XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); 
  ac++;
  XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); 
  ac++;
  m_nMenuBar = XmCreateMenuBar(m_nForm, "menu_bar", al, ac);
  XtManageChild(m_nMenuBar);


  // Create the Forth shell dialog

  ac = 0;
  XtSetArg(al[ac], XmNeditMode, XmMULTI_LINE_EDIT);
  ac++;
  XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNheight, 100);
  ac++;

  m_nForthShell = XmCreateScrolledText(m_nForm, "Forth_Shell",
    al, ac);
  XtManageChild(m_nForthShell);

  // Create the status message label widget and attach it below the
  //   drawing area (plot window)

  ac = 0;
  XtSetArg(al[ac], XmNlabelString,
    XmStringCreate(" ", XmSTRING_DEFAULT_CHARSET));
  ac++;
  XtSetArg(al[ac], XmNalignment, XmALIGNMENT_BEGINNING);
  ac++;
  XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_WIDGET);
  ac++;
  XtSetArg(al[ac], XmNbottomWidget, XtParent(m_nForthShell));
  ac++;

  m_nStatusBar = XtCreateManagedWidget("status_bar",
    xmLabelWidgetClass, m_nForm, al, ac);

  // Create coordinates label widget and attach it to the right of the
  //  status bar

  ac = 0;
  XtSetArg(al[ac], XmNlabelString,
 	   	 XmStringCreate(" ",XmSTRING_DEFAULT_CHARSET)); 
  ac++;
  XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_WIDGET);
  ac++;
  XtSetArg(al[ac], XmNbottomWidget, XtParent(m_nForthShell));
  ac++;

  m_nCoordinates = XtCreateManagedWidget("coordinates",
    xmLabelWidgetClass, m_nForm, al, ac);

  // Create the drawing area and attach it below the menu bar
  // and above the status bar

  ac = 0;
  XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); 
  ac++;
  XtSetArg(al[ac], XmNtopWidget, m_nMenuBar); 
  ac++;
  XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_WIDGET);
  ac++;
  XtSetArg(al[ac], XmNbottomWidget, m_nStatusBar);
  ac++;
  XtSetArg(al[ac], XmNheight, 400);
  ac++;
  XtSetArg(al[ac], XmNwidth, 600);
  ac++;

  m_nPlotWindow = XtCreateManagedWidget ("plot_window",
     xmDrawingAreaWidgetClass, m_nForm, al, ac);

  pDc->OpenDisplay(XtDisplay(m_nPlotWindow), XtWindow(m_nPlotWindow));
  m_pDc = pDc;


  // Make the menus

  for (int j = 0; j < 16; j++)
    {
      FileWidgets[j] = 0;
      EditWidgets[j] = 0;
      DataWidgets[j] = 0;
      PlotWidgets[j] = 0;
      MathWidgets[j] = 0;
      HelpWidgets[j] = 0;
    }
  m_nFileMenu = MakeMenu (m_nMenuBar, "File", FileOptions, FileWidgets, 6);
  m_nEditMenu = MakeMenu (m_nMenuBar, "Edit", EditOptions, EditWidgets, 1);
  m_nDataMenu = MakeMenu (m_nMenuBar, "Data", DataOptions, DataWidgets, 3);
  m_nPlotMenu = MakeMenu (m_nMenuBar, "Plot", PlotOptions, PlotWidgets, 7);
  m_nMathMenu = MakeMenu (m_nMenuBar, "Math", MathOptions, MathWidgets, 1);
  m_nHelpMenu = MakeMenu (m_nMenuBar, "Help", HelpOptions, HelpWidgets, 1);

  // Make the submenus

  //  m_nViewMenu = MakeMenu (PlotWidgets[ID_VIEW], "View", ViewOptions,
  //   ViewWidgets, 5); 

  // Create the file open dialog

  ac = 0;
  m_nFileOpenDialog = XmCreateFileSelectionDialog(TopLevel,
     "open_dialog", al, ac);
  XtUnmanageChild(XmSelectionBoxGetChild(m_nFileOpenDialog,
    XmDIALOG_HELP_BUTTON));

  // Create the file save dialog

  ac = 0;
  m_nFileSaveDialog = XmCreateFileSelectionDialog(TopLevel,
     "save_dialog", al, ac);
  XtUnmanageChild(XmSelectionBoxGetChild(m_nFileSaveDialog,
    XmDIALOG_HELP_BUTTON));

  // Create a dual purpose About/Message box

  m_nAboutBox = XmCreateInformationDialog(TopLevel, "about_box",
    NULL, 0);
  Widget temp = XmMessageBoxGetChild (m_nAboutBox, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(temp);
  temp = XmMessageBoxGetChild (m_nAboutBox, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(temp);

  // Create the input dialog

  ac = 0;
  XtSetArg(al[ac], XmNselectionLabelString, XmStringCreateLtoR
		("Enter input", XmSTRING_DEFAULT_CHARSET));  
  ac++;
  m_nInputDialog = XmCreatePromptDialog(TopLevel, "input_dialog", al, ac);
  XtUnmanageChild(XmSelectionBoxGetChild(m_nInputDialog, 
    XmDIALOG_HELP_BUTTON));

  // Create the header view/edit dialog

  ac = 0;
  m_nHeaderDialog = XmCreateFormDialog(TopLevel, "header_dialog", al, ac);

  ac = 0;
  XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); 
  ac++;
  XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM);
  ac++;
  m_nHeaderMessage = XmCreateMessageBox (m_nHeaderDialog, "header_message",
    al, ac);
  XtUnmanageChild(XmMessageBoxGetChild(m_nHeaderMessage, 
    XmDIALOG_HELP_BUTTON));
  XtManageChild(m_nHeaderMessage);

  ac = 0;
  XtSetArg(al[ac], XmNeditMode, XmMULTI_LINE_EDIT); 
  ac++;
  XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); 
  ac++;
  XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM);
  ac++;
  XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_WIDGET);
  ac++;
  XtSetArg(al[ac], XmNbottomWidget, m_nHeaderMessage);
  ac++;
  XtSetArg(al[ac], XmNheight, 200);
  ac++;
  XtSetArg(al[ac], XmNwidth, 400);
  ac++;
  m_nHeaderText = XmCreateScrolledText(m_nHeaderDialog, "header_text",
    al, ac);
  XtManageChild(m_nHeaderText);

  // Create the Pick dialog

  ac = 0;
  m_nPickDialog = XmCreateSelectionDialog (TopLevel, "pick_data",
    al, ac);
  XtUnmanageChild(XmSelectionBoxGetChild(m_nPickDialog, 
    XmDIALOG_HELP_BUTTON));

  // Setup the plot colors

  m_pDc->SetColors(color_list, MAX_COLORS);
  SetBackgroundColor("DarkGrey");
  SetForegroundColor("black");

  OnFileNew();
}
//---------------------------------------------------------------

CPlotWindow::~CPlotWindow()
{
  // Cleanup

  delete m_pDb;
  delete m_pDi;
  m_pDc->CloseDisplay();
  delete m_pDc;
  delete [] m_pFileName;
}
//---------------------------------------------------------------

void CPlotWindow::OnPaint ()
{
  CRect r = m_pDc->GetClientRect();
  m_pDi->SetPlotRect (r, m_pDc);
  m_pDi->Draw(m_pDc);
}
//---------------------------------------------------------------

void CPlotWindow::OnPrint ()
{
  CpsDC printDC (612, 792);
  printDC.OpenDisplay (PRINT_TEMP_FILE);
  printDC.SetColors (color_list, MAX_COLORS);
  printDC.SetForeground (printDC.GetColor("black"));

  CRect r = printDC.GetClientRect();
  m_pDi->CreateView (CARTESIAN_INVERTED, m_pDi->GetExtrema());
  m_pDi->SetPlotRect (r, &printDC);
  m_pDi->Draw(&printDC);
  m_pDi->DeleteView();
  printDC.CloseDisplay();

  // Allow user to print or save to file from GhostView

  char print_string[256];
  strcpy (print_string, "gv ");
  strcat (print_string, PRINT_TEMP_FILE);
  strcat (print_string, " &");
  system (print_string);
}
//---------------------------------------------------------------

void CPlotWindow::OnResize ()
{
  CRect r = m_pDc->GetClientRect();
  m_pDc->SetClientRect(r);
  Invalidate();
}
//---------------------------------------------------------------

void CPlotWindow::OnReset ()
{
    m_pDi->ResetExtrema();
    m_pDi->GoHome();
    Invalidate ();
}
//--------------------------------------------------------------
void CPlotWindow::OnRefresh()
{
    Invalidate();
}

void CPlotWindow::Invalidate()
{
  m_pDc->ClearDisplay();
}

void CPlotWindow::OnBack ()
{
    m_pDi->GoBack();
    Invalidate();
}

void CPlotWindow::OnForward ()
{
    m_pDi->GoForward();
    Invalidate();
}

void CPlotWindow::OnDeleteView ()
{
    m_pDi->DeleteView();
    Invalidate();
}
//---------------------------------------------------------------

Widget CPlotWindow::AddMenuItem (Widget wMenu, char* name,
  Widget buttons[])
{
  // Add a new button widget in wMenu, with specified name.

  Arg al[3];
  int i = 0, ac = 0;
  while (buttons[i]) i++; // Find first non-zero entry
  XtSetArg(al[ac], XmNlabelString, XmStringCreateLtoR(name,
		XmSTRING_DEFAULT_CHARSET)); 
  // ac++;
  // XtSetArg (al[ac], XmNbackground, 2);
  ac++;
  buttons[i] = XtCreateManagedWidget(name, xmPushButtonWidgetClass,
	    wMenu, al, ac);  
  return buttons[i];
}
//---------------------------------------------------------------

Widget CPlotWindow::MakeMenu (Widget wAttach, char* menu_name, 
  char* options[], Widget buttons[], int n)
{
  char submenu_name [32];
  Widget menu, cascade, b;
  Arg al[10];
  int i, ac = 0;

  menu = XmCreatePulldownMenu (wAttach, menu_name, al, ac);
  XtSetArg (al[ac], XmNsubMenuId, menu);  
  ac++;
  XtSetArg(al[ac], XmNlabelString, 
    XmStringCreateLtoR(menu_name, XmSTRING_DEFAULT_CHARSET)); 
  ac++;
  cascade = XmCreateCascadeButton (m_nMenuBar, menu_name, al, ac);
  XtManageChild (cascade);

  // Create the menu button widgets

  for (i = 0; i < n; i++)
    {
      if (*options[i] == '>')
	{
	  // Create a submenu cascade button
	 
	    strcpy (submenu_name, options[i]+1);
	    ac = 0;
	    XtSetArg(al[ac], XmNlabelString, XmStringCreateLtoR(submenu_name,
	      XmSTRING_DEFAULT_CHARSET));
	    ac++;
	    buttons[i] = XtCreateManagedWidget(submenu_name,
	      xmCascadeButtonWidgetClass, menu, al, ac);
	}
      else
	{
	  AddMenuItem (menu, options[i], buttons);
	}
    }
  return menu;
}
//---------------------------------------------------------------

void CPlotWindow::SetBackgroundColor (char* color_name)
{
  unsigned bkg = m_pDc->GetColor(color_name);
  m_pDc->SetBackground(bkg);
  strcpy (m_szBackgroundColor, color_name);
  Invalidate();
}

void CPlotWindow::SetForegroundColor (char* color_name)
{
  unsigned fg = m_pDc->GetColor(color_name);
  m_pDc->SetForeground(fg);
  strcpy (m_szForegroundColor, color_name);

  if (m_pDi) 
    {
      CGrid* pGrid = m_pDi->m_pGrid;
      if (pGrid) pGrid->SetColor(fg);
    }
  Invalidate();
}
//---------------------------------------------------------------

Symbol CPlotWindow::GetSymbol (char* symbol_name)
{
  // Return the plot symbol value associated with symbol_name
  // See the file CPlot.h

  strupr (symbol_name);
  Symbol s = sym_LINE;

  for (int i = 0; i < 7; i++)
    {
      if (strcmp(SymbolNames[i], symbol_name) == 0) 
	{
	  s = (Symbol) i;
	  break;
	}
    }
  return s;
}
//---------------------------------------------------------------

CPlotMessage* CPlotWindow::GetPlotMessage()
{
  return m_pDc->GetPlotMessage();
}
//---------------------------------------------------------------

void CPlotWindow::OnMouseMove(unsigned int mf, CPoint p)
{
    char cur_x [24], cur_y [24], cur_xy[48];
    vector<float> x;
    
    m_pDi->m_nMousePt = p;
    x = m_pDi->Logical(p);
    sprintf (cur_x, m_pDi->m_szXform, x[0]);
    sprintf (cur_y, m_pDi->m_szYform, x[1]);
    strcpy (cur_xy, cur_x);
    strcat (cur_xy, "  ");
    strcat (cur_xy, cur_y);

    int ac = 0;
    Arg al[2];

    XtSetArg(al[ac], XmNlabelString,
 	   	 XmStringCreate(cur_xy, XmSTRING_DEFAULT_CHARSET)); 
    ac++;
    XtSetValues(m_nCoordinates, al, ac);
}
//---------------------------------------------------------------

void CPlotWindow::OnLButtonDown(unsigned int mf, CPoint p)
{
    XEvent report;
    Display* display = XtDisplay(m_nPlotWindow);
    CRect rect, ExpRect;

    ExpRect.TopLeft() = p;
    ExpRect.BottomRight() = p;
    rect = ExpRect;

    m_pDc->SetForeground (-1);  // white 
    m_pDc->SetDrawingMode (GXxor);
    m_pDc->Rectangle (rect);

    while (1)
    {
      XNextEvent(display, &report);
      if (report.type == MotionNotify)
      {
	  m_pDc->Rectangle (rect);
	  p.x = report.xmotion.x;
	  p.y = report.xmotion.y;
	  OnMouseMove(0, p);
          ExpRect.BottomRight() = m_pDi->m_nMousePt;
          rect = ExpRect;
          SortRect (&rect);
          m_pDc->Rectangle (rect);
      }
      else if (report.type == ButtonRelease)
	{
	  m_pDc->SetDrawingMode (GXcopy);
	  break;
	}
      else
	{	
	   ;
	}      
    }

    SortRect (&ExpRect);

    if ((ExpRect.Width() > 9) && (ExpRect.Height() > 9))
      {
	vector<float> x = m_pDi->Logical(ExpRect);
        m_pDi->CreateView(CARTESIAN, x);
        Invalidate ();
     }
    
}
//---------------------------------------------------------------

void CPlotWindow::OnRButtonDown(unsigned int mf, CPoint p)
{

}
//---------------------------------------------------------------

void CPlotWindow::MessageBox (char* s)
{
// Display a message
  
  Arg a;
  XtSetArg(a, XmNmessageString, XmStringCreateLtoR(s,
    XmSTRING_DEFAULT_CHARSET));
  XtSetValues (m_nAboutBox, &a, 1);
  XtManageChild(m_nAboutBox);

}
//---------------------------------------------------------------

void CPlotWindow::OnExtremaInput ()
{
  Arg al[4];
  int ac = 0;
  XtSetArg(al[ac], XmNselectionLabelString, 
  XmStringCreateLtoR("Enter xmin, ymin, xmax, ymax:", 
    XmSTRING_DEFAULT_CHARSET));  
  ac++;
  XtSetArg(al[ac], XmNtextString, XmStringCreateLtoR("",
    XmSTRING_DEFAULT_CHARSET));
  ac++;
  XtSetValues(m_nInputDialog, al, ac);
  XtAddCallback (m_nInputDialog, XmNokCallback, 
    PlotMenuCB, PL_EXTREMA);
  XtAddCallback (m_nInputDialog, XmNcancelCallback, 
    PlotMenuCB, PL_EXTREMA_CANCEL);      
  XtManageChild (m_nInputDialog);
}
//----------------------------------------------------------------

void CPlotWindow::OnExtrema (char* s)
{
  float x[6];
  vector<float> e = m_pDi->GetExtrema();
  x[0] = e[0];
  x[1] = e[2];
  x[2] = e[1];
  x[3] = e[3];

  int n = NumberParse (x, s);

  e[0] = x[0];
  e[1] = x[2];
  e[2] = x[1];
  e[3] = x[3];

  m_pDi->CreateView(CARTESIAN, e);
    
  Invalidate();

}
//---------------------------------------------------------------

void CPlotWindow::WriteStatusMessage(char* msg)
{
  Arg al[2];
  XtSetArg(al[0], XmNlabelString,
    XmStringCreate(msg, XmSTRING_DEFAULT_CHARSET));
  XtSetValues(m_nStatusBar, al, 1);
}
//---------------------------------------------------------------

void CPlotWindow::OnCopyBitmap ()
{
	;
}
//------------------------------------------------------------------

void CPlotWindow::OnCopyMetafile ()
{
	;
}
//---------------------------------------------------------------

void CPlotWindow::OnSymbolLine ()
{
    m_pDi->GetActivePlot()->SetSymbol (sym_LINE);
    Invalidate();
}

void CPlotWindow::OnSymbolPoint ()
{
    m_pDi->GetActivePlot()->SetSymbol (sym_POINT);
    Invalidate();
}

void CPlotWindow::OnSymbolLinePt ()
{
    m_pDi->GetActivePlot()->SetSymbol (sym_LINE_PLUS_POINT);
    Invalidate();
}

void CPlotWindow::OnSymbolStick ()
{
    m_pDi->GetActivePlot()->SetSymbol (sym_STICK);
    Invalidate();
}

void CPlotWindow::OnSymbolToggle ()
{
    if (m_pDi->GetActivePlot()->GetSymbol() == sym_LINE)
        m_pDi->GetActivePlot()->SetSymbol (sym_POINT);
    else
        m_pDi->GetActivePlot()->SetSymbol (sym_LINE);

    Invalidate();
}
//---------------------------------------------------------------

void CPlotWindow::OnTemplate()
{
    char tmpl[256], prompt[256];

    strcpy (prompt, "Enter x1, x2, dx:");
    if (GetInput(prompt, tmpl) == 0) return;

    float x[6], temp;
    vector<float> e = m_pDi->GetExtrema();
    x[0] = e[0];
    x[1] = e[1];
    x[2] = (x[1] - x[0])/100.f;

    char s[255];
    strcpy (s, tmpl);

    NumberParse (x, s);
    if (x[0] > x[1])
    {
        temp = x[1];
        x[1] = x[0];
        x[0] = temp;
    }

    int npts;

    if (x[2] == 0.)
    {
        npts = 1;
    }
    else
    {
        npts = 1 + (x[1] - x[0])/x[2] ;
    }

    CReal* d = new CReal (2, npts, "Template", "\0");

    if (d == NULL)
    {
// AfxMessageBox ("Unable to create template data set.");
        return;
    }

    float* data = d->begin();
    temp = x[0];
    for (int i = 0; i < npts; i++)
    {
        *data++ = temp;
        *data++ = temp;
        temp += x[2];
    }
    d->SetExtrema();

    m_pDb->AddSet(d);
    m_pDi->MakePlot (d, 0);

    OnReset();

}
//---------------------------------------------------------------

void CPlotWindow::OnExpressionInput ()
{
  Arg al[4];

  int ac = 0;
  XtSetArg(al[ac], XmNselectionLabelString, 
    XmStringCreateLtoR("Enter the math expression:", 
    XmSTRING_DEFAULT_CHARSET));  
  ac++;
  XtSetArg(al[ac], XmNtextString, XmStringCreateLtoR("",
    XmSTRING_DEFAULT_CHARSET));
  ac++;
  XtSetValues(m_nInputDialog, al, ac);
  XtAddCallback (m_nInputDialog, XmNokCallback, 
    MathMenuCB, PL_EXPRESSION);
  XtAddCallback (m_nInputDialog, XmNcancelCallback, 
    MathMenuCB, PL_EXPRESSION_CANCEL);      
  XtManageChild (m_nInputDialog);
}
//---------------------------------------------------------------

void CPlotWindow::OnExpression(char* exp)
{
  char s[256], emsg[256];
  *s = 0;
  int i, *StackPtr;
  BYTE* TypePtr;
  ostrstream* pSS = NULL;
  
  if (*exp)
    {
      vector<BYTE> opcodes, prefix;
      pSS = new ostrstream(emsg, 256);
      strupr(exp);

      int ecode = CompileAE (&opcodes, exp);

// OutputForthByteCode (&opcodes);
// MessageBox(emsg);
 
      if (ecode)
	{
	  sprintf (emsg, "%s %d", "Expression Compiler Error:", ecode);
	  MessageBox (emsg);
	}
      else
	{
            // Perform the expression operations

	  CDataset* d = m_pDi->GetActiveSet();
	  int npts = d->NumberOfPoints();
	  int nsize = d->SizeOfDatum();
	  float *p = d->begin();
	  BYTE* bp = (BYTE*) &p;
	  prefix.push_back(OP_ADDR);
	  for (i = 0; i < sizeof(void*); i++)
	    prefix.push_back(*(bp + i));
	  bp = (BYTE*) &nsize;
	  prefix.push_back(OP_IVAL);
	  for (i = 0; i < sizeof(int); i++)
	    prefix.push_back(*(bp + i));
	  bp = (BYTE*) &npts;
	  prefix.push_back(OP_IVAL);
	  for (i = 0; i < sizeof(int); i++)
	    prefix.push_back(*(bp + i));
// prefix.push_back(OP_DOTS);
	  prefix.push_back(OP_RET);

	  SetForthInputStream (cin);
	  SetForthOutputStream (*pSS);
	  ecode = ForthVM (&prefix, &StackPtr, &TypePtr);
	  if (ecode)
	    {
	      sprintf (emsg, "%s %d", "Execution Error:", ecode);
	      MessageBox (emsg);
	    }
	  else
	    {
	      ecode = ForthVM (&opcodes, &StackPtr, &TypePtr);
	      if (ecode)
		{
		  sprintf (emsg, "%s %d", "Execution Error:", ecode);
		  MessageBox (emsg);
	        }
	      else
		{
		  // Revise the dataset extrema and header

		  d->SetExtrema();
		  sprintf (emsg, "%s\n", exp);
		  if (AddToHeader (emsg, d->m_szHeader, True))
		    MessageBox ("Header not updated.\nLength exceeded.");
		  
		  m_pDi->ResetExtrema();
		  WriteStatusMessage (exp);
		  Invalidate();
		}
	    }
	}
    }
}
//--------------------------------------------------------------

void CPlotWindow::OnArithmetic()
{
  ;
}
//--------------------------------------------------------------

void CPlotWindow::OnSwap()
{
  ;
}
//---------------------------------------------------------------

void CPlotWindow::OnSmooth()
{
  ;
}
//---------------------------------------------------------------

void CPlotWindow::OnGrid()
{
    // Allow user to setup grid style

	;
}
//--------------------------------------------------------------

int CPlotWindow::GetInput (char* prompt, char* response)
{
  int i = 0;
  /*
  CInputDialog getinp(prompt);
  i = getinp.DoModal();
  strcpy (response, getinp.GetResponse());

  return i;
  */
}
//---------------------------------------------------------------

void CPlotWindow::OnDelete()
{
    CDataset* d = m_pDi->GetActiveSet();
    int i;
    char s[256];

    if (d)
    {
        // Remove the dataset from the database

        m_pDb->RemoveSet(d);

        // Delete all plots of set d

        m_pDi->DeletePlotsOf(d);

        sprintf (s, "Dataset %s deleted from workspace", d->m_szName);
	WriteStatusMessage(s);

        // Delete the actual data set

        delete d;

        // Refresh the display

        m_pDi->ResetExtrema();
        Invalidate();
    }


}
//---------------------------------------------------------------

void CPlotWindow::OnDuplicate()
{
    // Duplicate the active data set and make a new plot

    CDataset* d = m_pDi->GetActiveSet();
    if (d)
    {
        CDataset* d2 = d->Duplicate();
        if (d2)
        {
            m_pDb->AddSet(d2);
            m_pDi->MakePlot(d2, 0);
            Invalidate();
        }
        else
        {
	  MessageBox ("Dataset Duplication Failed!");
        }
    }
}
//---------------------------------------------------------------

void CPlotWindow::OnPick()
{

  int nsets = m_pDb->Nsets();
  if (nsets == 0)
    {
      MessageBox ("There are no datasets in the database.");
    }
  else
    {
      char** sp = new char* [nsets];
      int i;

      for (i = 0; i < nsets; i++) sp[i] = new char [256];

      m_pDb->MakeListOfNames(sp);

      Widget pick_list = XmSelectionBoxGetChild(m_nPickDialog, 
	XmDIALOG_LIST);

      XmListDeleteAllItems (pick_list);

      for (i = 0; i < nsets; i++)
	{
	  XmListAddItem(pick_list, XmStringCreate(sp[i], 
	    XmSTRING_DEFAULT_CHARSET), 0);
	}

      XtManageChild(m_nPickDialog);
    }
}
//---------------------------------------------------------------

void CPlotWindow::OnPickData()
{

  Widget pick_list = XmSelectionBoxGetChild(m_nPickDialog,
    XmDIALOG_LIST);

  int* selections = new int[16];
  int nsel, i;

  if (XmListGetSelectedPos(pick_list, &selections, &nsel))
    {
      i = selections[0];
      CDataset* d =  (*m_pDb)[i-1];
      if (d)
	{
	  m_pDi->MakePlot(d, 0);
	  OnReset();
	}
    }
  delete [] selections;

  XtUnmanageChild(m_nPickDialog);
}
//----------------------------------------------------------------

void CPlotWindow::OnDrop()
{
	m_pDi->DeletePlot();
	m_pDi->ResetExtrema();
	Invalidate();
}
//---------------------------------------------------------------

void CPlotWindow::OnColor()
{
  ;
}
//---------------------------------------------------------------


void CPlotWindow::OnView2DCartesian()
{
//
	OnReset();
}
//---------------------------------------------------------------

void CPlotWindow::OnView2DPolar()
{
  ;
}
//---------------------------------------------------------------
/*
void CPlotWindow::OnView3DCartesian()
{
	m_pDi->Set3D (CARTESIAN);
	OnReset();
}
*/
//---------------------------------------------------------------

void CPlotWindow::OnAspect()
{
  ;
}
//---------------------------------------------------------------
/*
void CPlotWindow::OnRotate()
{

}
*/
//---------------------------------------------------------------

void CPlotWindow::OnHeader ()
{
  CDataset* d = m_pDi->GetActiveSet();
  if (d)
    {
      char s[256];;
      Arg al[5];
      int ac = 0;
      // strcpy (s, d->m_szName);
      // XtSetArg(al[ac], XmNselectionLabelString, 
      //	XmStringCreateLtoR(s, XmSTRING_DEFAULT_CHARSET));  
      // ac++;

      XmTextSetString(m_nHeaderText, d->m_szHeader);      
      XtManageChild (m_nHeaderDialog);     

    }
}
//---------------------------------------------------------------

void CPlotWindow::OnSetHeader (char* s)
{
  CDataset* d = m_pDi->GetActiveSet();
  if (d)
    {
      strcpy (d->m_szHeader, s);
    }
}
//---------------------------------------------------------------

BOOL CPlotWindow::OnFileNew ()
{
	delete m_pDi;
	delete m_pDb;

	m_pDb = new CDatabase();	// create database and display
	m_pDi = new CPlotDisplay();

	WriteStatusMessage("Workspace cleared.");

	OnReset();
	return TRUE;
}
//---------------------------------------------------------------

bool CPlotWindow::OnFileOpen ()
{
  XtManageChild(m_nFileOpenDialog); 
}
//--------------------------------------------------------------

bool CPlotWindow::OnFileSave ()
{
  XtManageChild(m_nFileSaveDialog);
}
//---------------------------------------------------------------

void CPlotWindow::OnAbout()
{
  char s[512];
  strcpy (s, "XYPLOT v1.0 for Linux/X Windows\n");
  strcat (s, RELEASE);
  strcat (s, "\nCopyright (c) 1999--2000 Krishna Myneni and John Kielkopf\n");
  strcat (s, "Creative Consulting for Research and Education\n");
  strcat (s, "e-mail: krishnamyneni@compuserve.com\n\n");
  strcat (s, "Provided under the GNU General Public License\n\n");
  MessageBox(s);
}
//--------------------------------------------------------------

void CPlotWindow::SelectColumns (char* fname, int nCols)
{
  // Open up a dialog box to allow user to specify columns to load

  char s[128];
  Arg al[4];
  int ac = 0;
  XtSetArg(al[ac], XmNtextString, XmStringCreateLtoR("",
    XmSTRING_DEFAULT_CHARSET));
  ac++;
  sprintf (s, "File contains %d columns.\n", nCols);
  strcat (s, "Select columns to load:");
  XtSetArg(al[ac], XmNselectionLabelString, 
    XmStringCreateLtoR(s, XmSTRING_DEFAULT_CHARSET));  
  ac++;
  //  XtSetArg(al[ac], XmNinitialFocus, XmDIALOG_TEXT);
  // ac++;
  XtSetValues(m_nInputDialog, al, ac);
  XtAddCallback (m_nInputDialog, XmNokCallback, 
    FileMenuCB, PL_FILE_LOAD);
  XtAddCallback (m_nInputDialog, XmNcancelCallback, 
    FileMenuCB, PL_LOAD_CANCEL);      
  XtManageChild (m_nInputDialog);
}
//---------------------------------------------------------------

vector<int> CPlotWindow::ParseColumns (char* cs)
{
// Parse a column specifier string into a vector of column numbers

  float x[8];
  int i, nSel = 0;
  char s[64];
  vector<int> Cols;

  if (*cs)
  {
    strcpy (s, cs);
    nSel = NumberParse(x, s);
    for (i = 0; i < nSel; i++) Cols.push_back((int) x[i] - 1);
  }

  return Cols;

}
//---------------------------------------------------------------

void CPlotWindow::LoadDatasetFile (char* fname, char* col_spec)
{
  // Load a dataset file into the database, with the columns
  //   to load specified in the col_spec

  char s[128];
  CDataset* d;

  int ecode = m_pDb->LoadDataset(&d, fname, ParseColumns(col_spec), 0);
  if (ecode <= 0)
    {
      MessageBox ("Error reading file.");
    }
  else
    {
      sprintf (s, "%d points read from %s", ecode, fname);
      WriteStatusMessage(s);
      m_pDi->MakePlot(d, 0);
      OnReset();
    }
  *m_pFileName = '\0';
}
  
//---------------------------------------------------------------

bool CPlotWindow::LoadFile(char* fname)
{
    int i, ecode, ns;
    char s[128];

    if (*fname)
    {
        CDataset* d;

       	if (strstr(fname, ".xsp"))
       	{
       	    CWorkspace41* pWs;
            ecode = m_pDb->LoadWorkspace (&pWs, fname);
            if (ecode)
            {
                sprintf (s, "XSP FILE CORRUPT");
                MessageBox (s);
            }
            else
            {
                for (i = 0; i < pWs->di.nplots; i++)
                {
                    ns = pWs->pv[i].set - 1;  // array origin is now 0
                    d = m_pDb->FindInList(pWs->ds[ns].fname);
                    if (d) m_pDi->MakePlot(pWs, d, i);
                }
                OnReset();
   	        delete pWs;     // delete the workspace (we're done with it)

            }

        }
        else if (strstr(fname, ".4th"))
        {
          // Load a Forth source file into the built-in Forth environment

	  ecode = LoadForthFile (fname);
        }
        else
        {
	  int nCols = m_pDb->GetColumnCount(fname);
          if (nCols < 0)
	    {
	      MessageBox ("Cannot open input file.");	    
	    }
	  else if (nCols == 0)
	    {
	      MessageBox ("Cannot find first line of data in file.");
	    }
	  else if (nCols <= 2)
	    {
	      LoadDatasetFile (fname, "");
	    }
	  else
	    {
	      SelectColumns (fname, nCols);
	    }

        }
    }

    return FALSE;
}
//---------------------------------------------------------------

bool CPlotWindow::SaveFile (char* fname)
{
    bool success = False;
    char s[256];

    CDataset* d = m_pDi->GetActiveSet();
    if (d)
    {
      if (strstr(fname, ".xsp"))
	{
	  // Create an xyplot workspace file

	  success = SaveWorkspace(fname);
	}
      else
	{
	  // Save the active data set

	  vector<float> e = m_pDi->GetExtrema();
	  int bOverlap;
	  vector<int> lim = d->IndexLimits(e[0], e[1], bOverlap);
	  if (bOverlap)
	    {
	      sprintf (s, "Source file is %s\n", d->m_szName);
	      AddToHeader (s, d->m_szHeader, True);

	      if ((lim[0] > 0) || (lim[1] < (d->NumberOfPoints()-1)))
		{
		  /*
		    CVerifyDialog::m_szPromptString =
                    "Save only data in window domain?";
		    CVerifyDialog vd;
		    int result = vd.DoModal();
		    if (result == IDC_NO)
		  */
		  // {
                    lim[0] = 0;
                    lim[1] = d->NumberOfPoints()-1;
		    // }

		    //  if (result != IDCANCEL)
                    success = m_pDb->SaveDataset (d, lim, fname);
		    //else
                    // WriteStatusMessage ("Save cancelled.");
		}
	      else
                success = m_pDb->SaveDataset (d, lim, fname);

	      if (success)
		{
		  strcpy (d->m_szName, fname);
		  sprintf (s, "%d points written to %s", lim[1]-lim[0]+1,
			   fname);
		  WriteStatusMessage (s);
		}
	      else
		{
		  MessageBox ("Error writing output file.");
		}
	    }
	  else
	    {
	      MessageBox("Active dataset not visible in plot window.");
	    }
	}
    }
    return success;
}
//----------------------------------------------------------------

bool CPlotWindow::SaveWorkspace (char* fname)
{
  // Save an xyplot workspace file

  bool success = False;

  ofstream outfile (fname);
  if (!outfile.fail())
    {
      char hdr[64];

      // Write the header

      strcpy (hdr, "XYPLOT_WORKSPACE_VER10X");
      outfile.write(hdr, 64);
      
      // Write database info

      m_pDb->Write(outfile);

      // Write plot display info
      
      m_pDi->Write(outfile);

      success = True;
    }
  return success;
}
//-----------------------------------------------------------------












