'
'
'  DOS Library
'
'  Copyright (c) 1990-1993 Creative Consulting for Research & Education
'
'  Last Revision: 6 Nov 1993
'
' $INCLUDE: 'c:\qb45\include\qb.bi'
' $INCLUDE: 'c:\xyplot\devqb\include\doslib.inc'

DEFINT I-N
'
' ---------------------------------------------------------------
'
FUNCTION FileExists% (FileName$)
'
'  Input Parameters:
'
'               FileName$  -- filename string.
'
'  Returns:
'
'               0  -- no such file exists
'              -1  -- file exists
'
	DIM InRegs AS RegTypeX, OutRegs AS RegTypeX
'
'
	FL$ = FileName$ + CHR$(0)
	IDTA$ = SPACE$(128)
	SetDTA IDTA$

        InRegs.ax = &H4E00
	InRegs.cx = 0
	InRegs.ds = VARSEG(FL$)
	InRegs.dx = SADD(FL$)
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
	IFLG = -(OutRegs.flags AND 1)
	FileExists = NOT IFLG
'
END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION OpenFile% (FL$, Acc%)
'
' Open file FL$ with access type specified by Acc%:
'               0 = read; 1 = write; 2 = read/write
' Returns the file handle as a 2-byte integer.
'
	DIM InRegs AS RegTypeX, OutRegs AS RegTypeX
'
'
	FL$ = FileName$ + CHR$(0)

        InRegs.ax = &H3D00 + Acc%
	InRegs.ds = VARSEG(FL$)
	InRegs.dx = SADD(FL$)
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
        OpenFile = OutRegs.ax

END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION CloseFile% (Fhan%)
'
'  Close the file specified with handle Fhan%.
'    Returns error code.
'
        DIM InRegs AS RegTypeX, OutRegs AS RegTypeX

        InRegs.ax = &H3E00
        InRegs.bx = Fhan%
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
        CloseFile = OutRegs.ax

END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION ReadFile% (Fhan%, N, JS, JP)
'
'  Read N bytes from file specified by Fhan%, into the buffer at
'    segment:offset specified by JS and JP. Returns the
'    number of bytes transferred. File must have previously been
'    opened.
'
	DIM InRegs AS RegTypeX, OutRegs AS RegTypeX

        InRegs.ax = &H3F00
        InRegs.bx = Fhan%
        InRegs.cx = N
        InRegs.ds = JS
        InRegs.dx = JP
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
        ReadFile = OutRegs.ax

END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION WriteFile% (Fhan%, N, JS, JP)
'
'  Write N bytes to file specified by Fhan%, from the buffer at
'    segment:offset specified by JS and JP. Returns the
'    number of bytes transferred. File must have previously been
'    opened.
'
	DIM InRegs AS RegTypeX, OutRegs AS RegTypeX

        InRegs.ax = &H4000
        InRegs.bx = Fhan%
        InRegs.cx = N
        InRegs.ds = JS
        InRegs.dx = JP
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
        WriteFile = OutRegs.ax

END FUNCTION
'
' ---------------------------------------------------------------
'
SUB FindFiles (FileSpec$, FileList$, NFILES)
'
'  **************************************************************
'  **                                                          **
'  **  Module: FindFiles                                       **
'  **                                                          **
'  **  Purpose: Retrieve list of all files that match          **
'  **           the string FileSpec$                           **
'  **                                                          **
'  **  Syntax: FindFiles FileSpec$, FileList$                  **
'  **                                                          **
'  **  Input Parameters:                                       **
'  **                                                          **
'  **           FileSpec$ --   file specification;             **
'  **                          may include wildcards.          **
'  **  Output Parameters:                                      **
'  **                                                          **
'  **           FileList$ -- string containing names of        **
'  **                          matching files separated by     **
'  **                          CRs                             **
'  **           NFILES      -- number of matching files        **
'  **                                                          **
'  **  Author: K. Myneni                                       **
'  **  Date: 28 Feb 90                                         **
'  **                                                          **
'  **************************************************************
'
'
'
	FSpec$ = FileSpec$
        FileList$ = SPACE$(4096)
	IF INSTR(FSpec$, "?") = 0 AND INSTR(FSpec$, "*") = 0 THEN
	       FSpec$ = "*.*"
	END IF
'
'
	FL$ = FSpec$ + CHR$(0)
'
'  Set Disk Transfer Area
'
	IDTA$ = SPACE$(128)
	SetDTA IDTA$
	FindFirstFile FL$, IFLG

	IF IFLG THEN
	   NFILES = 0                   ' no files found
	   EXIT SUB
	END IF


'
'  Find remaining files which match FileSpec$
'
      J = 1

      DO
	FM$ = SPACE$(12)
	FM$ = MID$(IDTA$, 31, 12)               ' get file name
	K = INSTR(FM$, CHR$(0))                 ' keep file name until
        IF K <> 0 THEN                          '   null character
          FM$ = MID$(FM$, 1, K - 1)
        END IF
	IATTR = ASC(MID$(IDTA$, 22, 1))         ' get file attribute
        IF (IATTR AND 16) <> 0 THEN
	    FM$ = RTRIM$(FM$)
            FM$ = "[" + FM$ + "]"
        ELSEIF INSTR(FM$, ".") = 0 THEN
          FM$ = RTRIM$(FM$) + "."
        ELSE
          '
        END IF

        MID$(FileList$, (J - 1) * 16 + 1) = FM$
        MID$(FileList$, J * 16, 1) = CHR$(13)
	J = J + 1
        MID$(IDTA$, 31, 12) = SPACE$(12)         ' clear file name in DTA
	FindNextFile IFLG
      LOOP WHILE NOT IFLG

      NFILES = J - 1

END SUB
'
' ---------------------------------------------------------------
'
SUB FindFirstFile (FileSpec$, IFLG)
'
'
'  Input Parameters:
'
'               FileSpec$  -- pathname string.
'               IFLG  -- success flag.
'
	DIM InRegs AS RegTypeX, OutRegs AS RegTypeX
'
'  Find first file
'
	InRegs.ax = &H4E00
	InRegs.cx = 16
	InRegs.ds = VARSEG(FileSpec$)
	InRegs.dx = SADD(FileSpec$)
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
	IFLG = -(OutRegs.flags AND 1)
'
END SUB
'
' ---------------------------------------------------------------
'
SUB FindNextFile (IFLG)
'
'  Call DOS Interrupt function to find next file in directory matching
'    filespec of previous search with FindFirstFile subroutine.
'
	DIM InRegs AS RegTypeX, OutRegs AS RegTypeX
'
	InRegs.ax = &H4F00
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
	IFLG = -(OutRegs.flags AND 1)
'
END SUB
'
' ---------------------------------------------------------------
'
SUB GetDirectory (DIR$)
'
'  Get current directory
'
  DIM InRegs AS RegTypeX, OutRegs AS RegTypeX
'
        DIR$ = SPACE$(64)
        InRegs.ax = &H4700
        InRegs.dx = 0
        InRegs.ds = VARSEG(DIR$)
        InRegs.si = SADD(DIR$)
        CALL INTERRUPTX(&H21, InRegs, OutRegs)
        DIR$ = RTRIM$(DIR$)
        DIR$ = "\" + DIR$
'
END SUB
'
' ---------------------------------------------------------------
'
SUB SetDirectory (DIR$)
'
'  Set current directory
'
	DIM InRegs AS RegTypeX, OutRegs AS RegTypeX
'
	D$ = DIR$ + CHR$(0)

	InRegs.ax = &H3B00
	InRegs.ds = VARSEG(D$)
	InRegs.dx = SADD(D$)
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
'
END SUB
'
' ---------------------------------------------------------------
'
SUB GetDrive (ID)
'
'  Get current disk, return in ID.
'
	DIM InRegs AS RegTypeX, OutRegs AS RegTypeX
'
	InRegs.ax = &H1900
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
        ID = OutRegs.ax AND 255
'
END SUB
'
' ---------------------------------------------------------------
'
SUB SetDrive (ID)
'
'  Set default drive (ID = 0 for A, 1 for B, 2 for C, etc).
'
	DIM InRegs AS RegTypeX, OutRegs AS RegTypeX
'
	InRegs.ax = &HE00
	InRegs.dx = ID
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
'
END SUB
'
' ---------------------------------------------------------------
'
SUB SetDTA (IDTA$)
'
'  Set disk transfer area to string IDTA$
'
	DIM InRegs AS RegTypeX, OutRegs AS RegTypeX
'
	InRegs.ax = &H1A00
	InRegs.ds = VARSEG(IDTA$)
	InRegs.dx = SADD(IDTA$)
	CALL INTERRUPTX(&H21, InRegs, OutRegs)
'
END SUB
