DECLARE FUNCTION ASIN# (X AS DOUBLE)
DECLARE FUNCTION ACOS# (X AS DOUBLE)
DECLARE FUNCTION COSH# (X AS DOUBLE)
DECLARE SUB ClearTopLine ()
DECLARE SUB EqnErr (Message AS STRING)
DECLARE SUB EqnHelp ()
DECLARE SUB Equation (X() AS DOUBLE, Y() AS DOUBLE, J1 AS INTEGER, J2 AS INTEGER, Status AS INTEGER)
DECLARE FUNCTION FACTRL# (X AS DOUBLE)
DECLARE FUNCTION GetExpr$ (Prompt AS STRING, Default AS STRING)
DECLARE FUNCTION GetKey$ ()
DECLARE SUB Parse (Expr$, Op$(), SN AS INTEGER)
DECLARE SUB ParsErr (Expr AS STRING, K AS INTEGER, Message AS STRING)
DECLARE SUB Polar (X AS DOUBLE, Y AS DOUBLE, R AS DOUBLE, Theta AS DOUBLE)
DECLARE FUNCTION SINH# (X AS DOUBLE)
DECLARE FUNCTION TANH# (X AS DOUBLE)
DIM X(1000) AS DOUBLE
DIM Y(1000, 2) AS DOUBLE
DIM I AS INTEGER
DIM J AS INTEGER
DIM J1 AS INTEGER
DIM J2 AS INTEGER
DIM N AS INTEGER
DIM Status AS INTEGER
CONST LTBLUE = 9
CONST RED = 4
CONST WHITE = 7

CLS
PRINT
Template:
COLOR WHITE
PRINT
PRINT "Specify parameters for y = x template. "
COLOR LTBLUE
INPUT "x1, x2, step size : "; X1, X2, dX
COLOR WHITE
IF X2 < X1 THEN
   COLOR RED
   PRINT "  No, x2 must be >= x1.  Try again.  "
   GOTO Template
END IF
N = (X2 - X1) / dX + 1
IF N > (1000) THEN
   COLOR RED
   PRINT "  Too many points.  Try a larger step size or shorter domain. "
   GOTO Template
END IF
FOR J = 1 TO N
   X(J) = X1 + (J - 1) * dX
   Y(J, 1) = X(J)
   Y(J, 2) = -X(J)                          ' imaginary part
NEXT J

Expand:
COLOR WHITE
PRINT
PRINT "Specify region to modify within [x1, x2]. "
COLOR LTBLUE
INPUT "x3, x4 : "; X3, X4
COLOR WHITE
IF (X3 < X1 OR X3 > X2) OR (X4 < X1 OR X4 > X2) THEN
    COLOR RED
    PRINT "  Out of range!  Try again.  "
    GOTO Expand
END IF
J1 = X3 / dX
J2 = X4 / dX
CLS
Equation X(), Y(), J1, J2, Status

COLOR LTBLUE
PRINT "Press a key to continue. "
DO
LOOP WHILE INKEY$ = ""

COLOR WHITE
CLS
FOR J = J1 TO J2
   PRINT " X("; J; ") = "; X(J); ";   ";
   PRINT TAB(20); " Y("; J; ") = "; Y(J, 1); " + "; Y(J, 2); "i"
NEXT J

COLOR LTBLUE
PRINT "Save to a file? (y/n) : ";
Q$ = GetKey$
IF UCASE$(Q$) = "Y" THEN
   PRINT "Yes"
   PRINT
   INPUT "File name> "; SaveFile$
   COLOR WHITE
   IF LEN(SaveFile$) > 0 THEN
      OPEN SaveFile$ FOR OUTPUT AS #11
      PRINT #11, "/*"
      PRINT #11, "Data created by the test program for the equation parser."
      PRINT #11, "*/"
      Fmt$ = "#.#######^^^^^ #.#######^^^^^ #.#######^^^^^"
      FOR J = 1 TO N
         PRINT #11, USING Fmt$; X(J); Y(J, 1); Y(J, 2)
      NEXT J
      CLOSE 11
   ELSE
      COLOR RED
      PRINT "No data saved. "
   END IF
ELSE
   PRINT "No"
END IF
COLOR WHITE

END

FUNCTION ACOS# (X AS DOUBLE)
   ACOS = ATN(SQR(1 - X * X) / X)
END FUNCTION

FUNCTION ASIN# (X AS DOUBLE)
   ASIN = ATN(X / SQR(1 - X * X))
END FUNCTION

SUB ClearTopLine
   LOCATE 1, 1, 0               ' Home; no cursor
   PRINT STRING$(80, 32);       ' Print line of spaces
   LOCATE 1, 1
END SUB

FUNCTION COSH# (X AS DOUBLE)
   COSH = (EXP(X) + EXP(-X)) / 2!
END FUNCTION

SUB EqnErr (Message AS STRING)
' Displays an error message from Equation.
' The message should be no more than 60 characters.
   CONST WHITE = 7
   CONST RED = 4
 
   IF LEN(Message) > 60 THEN
      Message = LEFT$(Message, 60)
   END IF
   ClearTopLine
   COLOR RED                                    ' <--
   PRINT "ERROR: "; Message; "; press a key.";
   Junk$ = GetKey$
   COLOR WHITE                                  ' <--
END SUB

SUB EqnHelp
' Help screen for Equation
   '
   ' ( open a window )
   CLS
   PRINT
   PRINT "Enter an algebraic equation, or an expression in one variable (X or Y). "
   PRINT "You may use any of the following: "
   PRINT
   PRINT " NUMBERS, e.g.:  3, (2.3026,1.905), 6.024E23, 1E-6, &H64, &O144 "
   PRINT
   PRINT " CONSTANTS:  I or J, PI"
   PRINT
   PRINT " ARRAY VARIABLES:  X, Y "
   PRINT
   PRINT " BINARY OPERATIONS:  +, -, *, /, MOD, ^, AND, OR, NOR, XOR"
   PRINT
   PRINT " UNARY OPERATIONS:  -, NOT (before a quantity);  %, ! (after a quantity)"
   PRINT
   PRINT " FUNCTIONS:  ABS or MAG, ACOS, ASC, ASIN, ATAN or ATN, CONJ, COS, COSH,"
   PRINT "             DEG (radians to degrees), EXP, IM or IMAG, INT, INV, LN or"
   PRINT "             LOG, LGT (common log), PHASE or ARG, RAD (degrees to radians),"
   PRINT "             RAN or RAND, RE or REAL, RND or CINT (round), SIN, SINH, SQR"
   PRINT "             or SQRT, TAN, TANH "
   LOCATE 24, 28
   'COLOR BLUE
   PRINT "Press a key to continue. "
   DO
   LOOP WHILE INKEY$ = ""
   'COLOR WHITE
   CLS
   ' (redraw the screen)
END SUB

SUB Equation (X() AS DOUBLE, Y() AS DOUBLE, J1 AS INTEGER, J2 AS INTEGER, Status AS INTEGER)
' Input an equation or one-variable expression and calculate the result
' using arrays X() and Y().
STATIC OldExpr AS STRING                ' old Expr
CONST FALSE = 0
CONST TRUE = NOT FALSE
CONST EXPLIM = 709.7821#
'CONST EXPLIM = 88.722838#                ' just under ln(2^128)
CONST FACLIM = 169
DIM Op$(50)                             ' operation list
DIM Stack(20, 2) AS DOUBLE              ' numeric stack
DIM Result(J1 TO J2, 2) AS DOUBLE       ' result of calculation
DIM A AS DOUBLE, B AS DOUBLE, C AS DOUBLE, D AS DOUBLE
DIM R AS DOUBLE, S AS DOUBLE, Sq AS DOUBLE, Phi AS DOUBLE, Theta AS DOUBLE
DIM U AS DOUBLE, V AS DOUBLE
DIM L10 AS DOUBLE, PI AS DOUBLE
DIM I AS INTEGER, J AS INTEGER, K AS INTEGER, L AS INTEGER, M AS INTEGER
DIM Complex AS INTEGER, XChanged AS INTEGER, YChanged AS INTEGER
DIM SN AS INTEGER                       ' step number
DIM SP AS INTEGER                       ' numeric stack pointer
DIM Expr AS STRING
DIM Prompt AS STRING

PI = 4! * ATN(1)
L10 = LOG(10#)
Seed = TAN(VAL(RIGHT$(TIME$, 2)))
RANDOMIZE Seed
Prompt = "Equation>> "

LOCATE 2, 1
Expr = GetExpr$(Prompt, OldExpr)
Expr = UCASE$(Expr)
L = LEN(Expr)
IF L = 0 THEN
   ClearTopLine
   EXIT SUB
END IF

PE = INSTR(Expr, "=")
IF PE > 5 THEN
   EqnErr "This equation can't be calculated."
   EXIT SUB
ELSEIF PE = 0 THEN              ' not an equation
   LHS$ = ""
ELSE
   LHS$ = MID$(Expr, 1, PE - 1)
END IF

SN = 0
E$ = RIGHT$(Expr, L - PE)

Parse E$, Op$(), SN

 PRINT : PRINT "There are"; SN; " steps. "                      ' <----<<
 FOR M = 1 TO SN                                                ' <----<<
    PRINT M; ":  "; Op$(M)                                      ' <----<<
 NEXT M                                                         ' <----<<

FOR J = J1 TO J2
   SP = 0
   FOR M = 1 TO SN
   IF SP > 1 THEN
      A = Stack(SP - 1, 1)
      B = Stack(SP - 1, 2)
   ELSE
      A = 0
      B = 0
   END IF
   C = Stack(SP, 1)
   D = Stack(SP, 2)
   SELECT CASE Op$(M)
   ' constants and functions with no argument:
      CASE "0" TO "9E:", "." TO ".9E:", "&H" TO "&O8"   ' numbers
         SP = SP + 1
         A = VAL(Op$(M))
      CASE "I"
         SP = SP + 1
         A = 0
         B = 1
      CASE "PI"
         SP = SP + 1
         A = PI
      CASE "RAN", "RAND"
         SP = SP + 1
         A = RND
      ' variables:
      CASE "X"
         XChanged = TRUE
         SP = SP + 1
         A = X(J)
         B = 0
      CASE "Y"
         YChanged = TRUE
         SP = SP + 1
         A = Y(J, 1)
         B = Y(J, 2)
      ' binary operations:
      CASE "+"
         IF SP < 2 THEN
            A = C               ' no change
            B = D
         ELSE
            A = A + C
            B = B + D
            SP = SP - 1
         END IF
      CASE "-"
         IF SP < 2 THEN
            A = -C
            B = -D
         ELSE
            A = A - C
            B = B - D
            SP = SP - 1
         END IF
      CASE "*"
         IF SP < 2 THEN
            EqnErr "Missing number or variable"
            EXIT SUB
         END IF
         IF LOG(ABS(A) + 1) + LOG(ABS(C) + 1) < EXPLIM AND LOG(ABS(B) + 1) + LOG(ABS(D) + 1) < EXPLIM THEN
            A = A * C - B * D
            B = A * D + B * C
            SP = SP - 1
         ELSE
            EqnErr "Product is too large. "
            EXIT SUB
         END IF
      CASE "/"
         IF SP < 2 THEN
            EqnErr "Missing number or variable"
            EXIT SUB
         END IF
         IF C = 0 AND D = 0 THEN
            EqnErr " Division by zero."
            EXIT SUB
         ELSE
            Sq = C * C + D * D
            A = (A * C + B * D) / Sq
            B = (B * C - A * C) / Sq
            SP = SP - 1
         END IF
      CASE "MOD"
         IF SP < 2 THEN
            EqnErr "Missing number or variable"
            EXIT SUB
         END IF
         IF C = 0 AND D = 0 THEN
            EqnErr " Division by zero."
            EXIT SUB
         ELSEIF B = 0 AND D = 0 THEN
            A = A MOD C
            SP = SP - 1
         ELSE
            Sq = C * C + D * D
            A = (A * C + B * D) MOD Sq          ' This may not be right.
            B = (B * C - A * C) MOD Sq
            SP = SP - 1
         END IF
      CASE "^"
         IF SP < 2 THEN
            EqnErr "Missing number or variable"
            EXIT SUB
         END IF
         IF A = 0 AND B = 0 THEN
            SP = SP - 1                 ' leave them zero
         ELSEIF (B = 0 AND D = 0) AND C * LOG(ABS(A) + 1) < EXPLIM THEN
            A = A ^ C
            SP = SP - 1
         ELSEIF 2 * LOG(ABS(A) + 1) < EXPLIM AND 2 * LOG(ABS(B) + 1) < EXPLIM THEN
            Polar A, B, R, Theta
            IF ABS(C) * LOG(R) < EXPLIM AND ABS(D * Theta) < EXPLIM THEN
               S = R ^ C * EXP(-D * Theta)
               Phi = C * Phi + D * LOG(R)
               A = S * COS(Phi)
               B = S * SIN(Phi)
               SP = SP - 1
            ELSE
               EqnErr "Exponentiation too large"
               EXIT SUB
            END IF
         ELSE
            Msg$ = "Unable to raise (" + STR$(A) + "," + STR$(B) + ") to the power (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE "AND"
         IF SP < 2 THEN
            EqnErr "Missing number or variable"
            EXIT SUB
         END IF
         A = INT(A) AND INT(C)
         B = INT(B) AND INT(C)
         SP = SP - 1
      CASE "NOR"
         IF SP < 2 THEN
            EqnErr "Missing number or variable"
            EXIT SUB
         END IF
         A = NOT (INT(A) OR INT(C))
         B = NOT (INT(C) OR INT(D))
         SP = SP - 1
      CASE "OR"
         IF SP < 2 THEN
            EqnErr "Missing number or variable"
            EXIT SUB
         END IF
         A = INT(A) OR INT(C)
         B = INT(C) OR INT(D)
         SP = SP - 1
      CASE "XOR"
         IF SP < 2 THEN
            EqnErr "Missing number or variable"
            EXIT SUB
         END IF
         A = INT(A) XOR INT(C)
         B = INT(C) XOR INT(D)
         SP = SP - 1
      ' special operations:
      CASE "CMPL"
         IF SP < 2 THEN
            A = C                       ' Cmpl(real #)
            B = 0
         ELSE
            'A = A
            B = C
         END IF
         SP = SP - 1
      CASE "%"
         IF SP < 1 THEN
            EqnErr "Missing number or variable"
            EXIT SUB
         ELSEIF SP < 2 THEN
            A = C / 100#
            B = D / 100#
         ELSE
            A = A * C / 100#
            B = B * D / 100#
            ' DON'T drop the stack.
         END IF
      ' functions of one variable:
      CASE "!", "FAC"
         IF SP < 1 THEN
            EqnErr "Missing number or variable"
            EXIT SUB
         END IF
         IF C = INT(C) AND C > 0 AND C < FACLIM AND D = 0 THEN
            A = FACTRL(C)
            B = 0
         ELSE
            Msg$ = "Unable to take the factorial of " + STR$(C)
            EqnErr Msg$
         END IF
      CASE "ABS", "MAG"
         A = SQR(C * C + D * D)
         B = 0
      CASE "ACOS"
         IF ABS(C) < 1 AND D = 0 THEN
            A = ACOS(C)
            B = 0
         ELSE
            ' arccos z = -i*log[z + sqr(z^2 - 1)]
            IF LOG(ABS(C + 1)) < EXPLIM AND LOG(ABS(D) + 1) < EXPLIM THEN
               Polar C, D, R, Theta
               U = R * R * COS(2# * Theta) - 1
               V = R * R * SIN(2# * Theta)
               Polar U, V, R, Theta
               U = SQR(R) * COS(Theta / 2#) + C
               V = SQR(R) * COS(Theta / 2#) + D
               Polar U, V, R, Theta
               A = Theta
               B = -LOG(R)
            ELSE
               Msg$ = "Unable to take the arc cosine of (" + STR$(C) + "," + STR$(D) + ")"
               EqnErr Msg$
            END IF
         END IF
      CASE "ASIN"
         IF ABS(C) < 1 AND D = 0 THEN
            A = ASIN(C)
            B = 0
         ELSE
            ' arcsin z = -i*log[i*z + sqr(1 - z^2)]
            IF LOG(ABS(C + 1)) < EXPLIM AND LOG(ABS(D) + 1) < EXPLIM THEN
               Polar C, D, R, Theta
               U = 1 - R * R * COS(2# * Theta)
               V = R * R * SIN(2# * Theta)
               Polar U, V, R, Theta
               U = SQR(R) * COS(Theta / 2#) - D
               V = SQR(R) * COS(Theta / 2#) + C
               Polar U, V, R, Theta
               A = Theta
               B = -LOG(R)
            ELSE
               Msg$ = "Unable to take the arc sine of (" + STR$(C) + "," + STR$(D) + ")"
               EqnErr Msg$
            END IF
         END IF
      CASE "ATAN", "ATN"
         IF D = 0 THEN
            A = ATN(C)
            B = 0
         ELSE
            ' arctan z = (i/2)*log[(i + z)/(i - z)]
            IF LOG(ABS(C + 1)) < EXPLIM AND LOG(ABS(D) + 1) < EXPLIM THEN
               Sq = C * C + (D - 1) ^ 2
               U = (C * C + (D + 1) * (D - 1)) / Sq
               V = ((D + 1) * C - C * (D - 1)) / Sq     ' check this
               Polar U, V, R, Theta
               A = Theta / 2#
               B = -LOG(R) / 2#
            ELSE
               Msg$ = "Unable to take the arc tangent of (" + STR$(C) + "," + STR$(D) + ")"
               EqnErr Msg$
            END IF
         END IF
      CASE "CONJ"
         A = C
         B = -D
      CASE "COS"
         IF D = 0 THEN
            A = COS(C)
            B = 0
         ELSEIF D < EXPLIM THEN
            A = COS(C) * COSH(D)
            B = -SIN(C) * SINH(D)
         ELSE
            Msg$ = "Unable to take the cosine of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE "COSH"
         IF C < EXPLIM AND D = 0 THEN
            A = COSH(C)
            B = 0
         ELSEIF C < EXPLIM AND D < EXPLIM THEN
            A = COSH(C) * COS(D)
            B = -SINH(C) * SIN(D)
         ELSE
            Msg$ = "Unable to take the cosh of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE "DEG"
         A = 180# * C / PI
         B = 180# * D / PI                      '  ?
      CASE "EXP"
         IF C < EXPLIM THEN
            A = EXP(C) * COS(D)
            B = EXP(C) * SIN(D)
         ELSE
            Msg$ = "Unable to take the exp of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE "IM", "IMAG"
         A = D
         B = 0
      CASE "INT"
         A = INT(C)
         B = INT(D)
      CASE "INV"
         Sq = C * C + D * D
         IF Sq > 0 THEN
            A = C / Sq
            B = D / Sq
         ELSE
            Msg$ = "Unable to take the inverse of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
         END IF
      CASE "LGT"   ' Log base 10  ... maybe this should be LOG
         IF C = 0 AND D = 0 THEN
            EqnErr "Logarithm of zero"
            EXIT SUB
         ELSEIF C > 0 AND D = 0 THEN
            A = LOG(C) / L10
         ELSEIF 2 * LOG(ABS(C) + 1) < EXPLIM AND 2 * LOG(ABS(D) + 1) < EXPLIM THEN
            R = SQR(C * C + D * D)
            Theta = ATN(D / C)
            A = LOG(R) / L10
            B = Theta
         ELSE
            Msg$ = "Unable to take the logarithm of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE "LN", "LOG"  ' Ln
         IF C = 0 AND D = 0 THEN
            EqnErr "Logarithm of zero"
            EXIT SUB
         ELSEIF C > 0 AND D = 0 THEN
            A = LOG(C)
         ELSEIF 2 * LOG(ABS(C) + 1) < EXPLIM AND 2 * LOG(ABS(D) + 1) < EXPLIM THEN
            R = SQR(C * C + D * D)
            Theta = ATN(D / C)
            A = LOG(R)
            B = Theta
         ELSE
            Msg$ = "Unable to take the logarithm of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE "NEG"
         A = -C
         B = -D
      CASE "NOT"
         A = NOT INT(C)
         B = NOT INT(D)
      CASE "PHASE", "ARG"
         IF C = 0 THEN
            A = PI / 2#
            B = 0
         ELSE
            A = ATN(D / C)
            B = 0
         END IF
      CASE "POLAR"
         Polar C, D, A, B
      CASE "RAD"
         A = PI * C / 180#
         B = PI * D / 180#                      '  ?
      CASE "RE", "REAL"
         A = C
         B = 0
      CASE "RECT"
         ' assume C = R and D = Theta
         A = C * COS(D)
         B = C * SIN(D)
      CASE "RND", "CINT"
         A = CINT(C)
         B = CINT(D)
      CASE "SIN"
         IF D = 0 THEN
            A = SIN(C)
            B = 0
         ELSEIF D < EXPLIM THEN
            A = SIN(C) * COSH(D)
            B = COS(C) * SINH(D)
         ELSE
            Msg$ = "Unable to take the sine of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE "SINH"
         IF C < EXPLIM AND D = 0 THEN
            A = SINH(C)
            B = 0
         ELSEIF C < EXPLIM AND D < EXPLIM THEN
            A = SINH(C) * COS(D)
            B = COSH(C) * SIN(D)
         ELSE
            Msg$ = "Unable to take the sinh of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE "SQR", "SQRT"
         IF C = 0 AND D = 0 THEN
            A = 0
            B = 0
         ELSEIF C > 0 AND D = 0 THEN
            A = SQR(C)
            B = 0
         ELSEIF 2 * LOG(ABS(C) + 1) < EXPLIM AND 2 * LOG(ABS(D) + 1) < EXPLIM THEN
            Polar C, D, R, Theta
            A = SQR(R) * COS(Theta / 2)
            B = SQR(R) * SIN(Theta / 2)
         ELSE
            Msg$ = "Unable to take the square root of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE "TAN"
         IF D = 0 THEN
            A = TAN(C)
            B = 0
         ELSEIF D < EXPLIM THEN
            ' denominator:
            U = COS(C) * COSH(D)
            V = -SIN(C) * SINH(D)
            Sq = U * U + V * V
            ' numerator:
            A = SIN(C) * COSH(D)
            B = COS(C) * SINH(D)
            ' quotient:
            C = (A * U + B * V) / Sq
            D = (B * U - A * U) / Sq
            A = C
            B = D
         ELSE
            Msg$ = "Unable to take the tangent of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE "TANH"
         IF C < EXPLIM AND D = 0 THEN
            A = TANH(C)
            B = 0
         ELSEIF C < EXPLIM AND D < EXPLIM THEN
            ' denominator:
            U = COSH(C) * COS(D)
            V = SINH(C) * SIN(D)
            Sq = U * U + V * V
            ' numerator:
            A = SINH(C) * COS(D)
            B = COSH(C) * SIN(D)
            ' quotient:
            C = (A * U + B * V) / Sq
            D = (B * U - A * U) / Sq
            A = C
            B = D
         ELSE
            Msg$ = "Unable to take the tanh of (" + STR$(C) + "," + STR$(D) + ")"
            EqnErr Msg$
            EXIT SUB
         END IF
      CASE ELSE
         PRINT "Programming error!"
         Msg$ = "The operation " + CHR$(34) + Op$(SN) + CHR$(34) + " is presently undefined."
         PRINT Msg$
'         EqnErr Msg$
         EXIT SUB
      END SELECT
      Stack(SP, 1) = A
      Stack(SP, 2) = B
   NEXT M
   Result(J, 1) = Stack(1, 1)
   Result(J, 2) = Stack(1, 2)
' PRINT "Stack at X="; X(J); ": "                               ' <----<<
' PRINT "3:"; Stack(3, 1); Stack(3, 2)                          ' <----<<
' PRINT "2:"; Stack(2, 1); Stack(2, 2)                          ' <----<<
' PRINT "1:"; Stack(1, 1); Stack(1, 2)                          ' <----<<
NEXT J

IF INSTR(LHS$, "X") OR (XChanged AND NOT YChanged) THEN
   FOR J = J1 TO J2
      X(J) = Result(J, 1)
   NEXT J
   Status = 1
ELSEIF INSTR(LHS$, "Y") OR (YChanged AND NOT XChanged) THEN
   FOR J = J1 TO J2
      Y(J, 1) = Result(J, 1)
      Y(J, 2) = Result(J, 2)
   NEXT J
   Status = 1
ELSE
   EqnErr "No data changed"
   Status = 0
END IF
' Store Expr for next time.
OldExpr = Expr

END SUB

FUNCTION FACTRL# (X AS DOUBLE)
' based on routine from Numerical Recipes
   DIM F AS DOUBLE
   STATIC Top AS DOUBLE
   DIM I AS INTEGER
   DIM N AS INTEGER
   STATIC NTop AS INTEGER

   IF X = INT(X) THEN
      N = X
      IF N > NTop AND NTop > 0 THEN
         I = NTop
         F = Top
      ELSE
         I = 1
         F = 1
      END IF

      FOR I = I TO N
         F = F * I
      NEXT I
      IF N > NTop THEN
         NTop = N
         Top = F
      END IF
   ELSE
      'F = GAMMA(X)
      PRINT "Gamma function not available."
      F = 0
   END IF
   FACTRL# = F
END FUNCTION

FUNCTION GetExpr$ (Prompt AS STRING, Default AS STRING)
' Read a line of text from the keyboard, after displaying a prompt.
' The string Default is used to fill in characters when the right
' arrow key is pressed, or to become the current expression if
' the up arrow is pressed.
DIM LD AS INTEGER
DIM Expr AS STRING
CONST BS = 8
CONST CR = 13
CONST DEL = 127
CONST ESC = 27
CONST LF = 10

KEYF1$ = CHR$(0) + CHR$(&H3B)
KeyUp$ = CHR$(0) + CHR$(&H48)
KeyDown$ = CHR$(0) + CHR$(&H50)
KeyLeft$ = CHR$(0) + CHR$(&H4B)
KeyRight$ = CHR$(0) + CHR$(&H4D)

LD = LEN(Default)
LP = LEN(Prompt)
Temp$ = ""

ClearTopLine
PRINT "Enter an equation in X and Y, or an expression in one variable. "
FOR K = 1 TO 80
   PRINT " ";
NEXT K
LOCATE , 1, 1                  ' Start of current line, cursor on
'INPUT Prompt; Expr
PRINT Prompt;
DO WHILE POS(0) < 80
   A$ = GetKey$
   SELECT CASE A$
   CASE KEYF1$, "?"
      EqnHelp
   CASE KeyUp$
      Temp$ = Egn$
      Expr = Default
      LOCATE , 1
      PRINT Prompt; Expr; STRING$(80 - LD - LP, 32);
      LOCATE , LP + LD + 1
   CASE KeyDown$
      Expr = Temp$
      L = LEN(Expr)
      LOCATE , 1
      PRINT Prompt; Temp$
      PRINT STRING$(80 - LP - L, 32);
      LOCATE , LP + L + 1
   CASE KeyLeft$
      IF L > 0 THEN
         MID$(Default, L, 1) = MID$(Expr, L, 1)
         Expr = LEFT$(Expr, L - 1)
         L = L - 1
      END IF
   CASE KeyRight$
      IF L < LD THEN
         Expr = Expr + MID$(Default, L + 1, 1)
      ELSE
         Expr = Expr + " "
      END IF
      L = L + 1
      LOCATE , 1
      PRINT Prompt; Expr;
   CASE CHR$(BS), CHR$(DEL)
      IF L > 0 THEN
         Expr = LEFT$(Expr, L - 1)
         L = LEN(Expr)
         LOCATE , 1
         PRINT Prompt; Expr; " ";
         LOCATE , LP + L + 1
      END IF
   CASE CHR$(LF), CHR$(CR)
      EXIT DO
   CASE CHR$(0) TO CHR$(31)
      BEEP
   CASE ELSE
      Expr = Expr + A$
      L = L + 1
      PRINT A$;
   END SELECT
LOOP
GetExpr$ = Expr
END FUNCTION

FUNCTION GetKey$
   DO
      Temp$ = INKEY$
   LOOP WHILE Temp$ = ""
   GetKey$ = Temp$
END FUNCTION

SUB Parse (Expr AS STRING, Op$(), SN AS INTEGER)
' Parse an algebraic expression into a reverse-Polish stack.
' If SN is to be correct, it must be passed as itself.
' (SN, Expr, E$ must NOT be static variables.)
DIM Complex AS INTEGER, DecFlag AS INTEGER, ExpFlag AS INTEGER
DIM FuncFlag AS INTEGER
DIM K AS INTEGER, K1 AS INTEGER, L AS INTEGER, P AS INTEGER
DIM DelimSet AS STRING
CONST FALSE = 0
CONST TRUE = NOT FALSE

E$ = Expr
L = LEN(E$)
'  PRINT "E$ ="; CHR$(34); E$; CHR$(34); ". "            ' <----<<
K = 0
P = 0
E1$ = ""
DO WHILE K < L
   IF SN > 45 THEN
      ParsErr E$, K, "That formula is too complex!"
      EXIT SUB
   END IF

   K = K + 1
   A$ = MID$(E$, K, 1)
' PRINT : PRINT CHR$(34); A$; CHR$(34); "[";                ' <----<<
   SELECT CASE A$
   CASE " ", "", CHR$(0)
      ' continue
   CASE "(", "[", "{"
      P = 1
      FOR K1 = K + 1 TO L
         A1$ = MID$(E$, K1, 1)
         SELECT CASE A1$
         CASE "(", "[", "{"
            P = P + 1
         CASE ","
            IF Complex THEN
               ParsErr E$, K1, "Too many commas within a pair of parentheses"
               EXIT SUB
            ELSE
               Complex = TRUE
               E1$ = MID$(E$, K + 1, K1 - K - 2)
               Parse (E1$), Op$(), SN
            END IF
         CASE ")", "]", "}"
            P = P - 1
            IF P = 0 THEN
               IF Complex THEN
                  E1$ = MID$(E$, K + 1, K1 - K - 2)
                  Parse (E1$), Op$(), SN
                  SN = SN + 1
                  Op$(SN) = "CMPL"
                  EXIT FOR
               ELSE
                  EXIT FOR
               END IF
            END IF
         CASE ELSE
           ' continue
         END SELECT
      NEXT K1
      IF K1 > L THEN
         ParsErr E$, L, "There is a right parenthesis missing"
'         EXIT SUB
      END IF
      E1$ = MID$(E$, K + 1, K1 - K - 1)
      Parse (E1$), Op$(), SN
      K = K1 + 1
   CASE ")", "]", "}"
      ParsErr E$, K, "There is a right parenthesis out of place"
'      EXIT SUB
   CASE "&"
      A1$ = MID$(E$, K + 1, 1)
      IF A1$ = "H" OR A1$ = "O" THEN    ' hexadecimal or octal numbers
         FOR K1 = K + 1 TO L
            A1$ = MID$(E$, K1, 1)
            IF NOT INSTR("0123456789ABCDEF", A1$) THEN EXIT FOR
         NEXT K1
         SN = SN + 1
         Op$(SN) = MID$(E$, K, K1 - K)
         K = K1 - 1
      ELSE
'        SN = SN + 1
'        Op$(SN) = "AND"
        Msg$ = "The character " + CHR$(34) + "&" + CHR$(34) + " makes no sense here"
        ParsErr E$, K, Msg$
      END IF
   CASE "0" TO "9", "."
      DecFlag = FALSE
      ExpFlag = FALSE
      K1 = K
      DO WHILE K1 <= L
         K1 = K1 + 1
         A1$ = MID$(E$, K1, 1)
         SELECT CASE A1$
         CASE "0" TO "9"
         CASE "."
            IF DecFlag = TRUE THEN
               ParsErr E$, K1, "Too many decimal points in this number"
               EXIT SUB
            ELSE
               DecFlag = TRUE
            END IF
         CASE "E", "D"
            IF ExpFlag THEN
               ParsErr E$, K1, "Too many exponents in this number"
               EXIT SUB
            ELSE
               IF INSTR("+-", MID$(E$, K1 + 1)) THEN
                  K1 = K1 + 1
               END IF
            END IF
         CASE ELSE
            EXIT DO
         END SELECT
      LOOP
      SN = SN + 1
      Op$(SN) = MID$(E$, K, K1 - K)
      K = K1 - 1
   CASE "+", "-"
      DelimSet = "+-"
      GOSUB Delimit
      E1$ = MID$(E$, K + 1, K1 - K - 1)
      Parse (E1$), Op$(), SN
      IF SN = 0 OR K = 1 THEN    ' unary operation
         IF A$ = "-" THEN
            SN = SN + 1
            Op$(SN) = "NEG"
         'ELSE ignore the +
         END IF
      ELSE
         SN = SN + 1
         Op$(SN) = A$
      END IF
      K = K1 - 1
   CASE "*", "/"
      DelimSet = "+-*/"
      GOSUB Delimit
      E1$ = MID$(E$, K + 1, K1 - K - 1)
      Parse (E1$), Op$(), SN
      SN = SN + 1
      Op$(SN) = A$
      K = K1 - 1
   CASE "^"
      DelimSet = "+-*/^"
      GOSUB Delimit
      E1$ = MID$(E$, K + 1, K1 - K - 1)
      Parse (E1$), Op$(), SN
      SN = SN + 1
      Op$(SN) = A$
      K = K1 - 1
   CASE "%"
      SN = SN + 1
      Op$(SN) = "%"
   CASE "!"
      IF SN > 0 THEN
         SN = SN + 1
         Op$(SN) = A$
      ELSE
         ParsErr E$, K, "No number to take factorial of"
         EXIT SUB
      END IF
      'ParsErr E$, K, "Sorry, factorials are not presently supported"
   CASE "A" TO "Z"
      FOR K1 = K + 1 TO L
         A1$ = MID$(E$, K1, 1)
         IF A1$ < "A" OR A1$ > "Z" OR K1 > 4 THEN EXIT FOR
      NEXT K1
      FuncFlag = TRUE
      C$ = MID$(E$, K, K1 - K)
      SELECT CASE C$
      CASE "PHASE", "POLAR"
         K = K + 4
      CASE "ACOS", "ASIN", "ATAN", "CINT", "CMPL", "CONJ", "COSH", "IMAG", "REAL", "RECT", "SINH", "SQRT", "TANH"
         K = K + 3
      CASE "RAND"
         FuncFlag = FALSE
         K = K + 3
      CASE "ABS", "ARG", "ATN", "COS", "DEG", "EXP", "FAC", "INT", "INV", "LOG", "LGT", "MAG", "NOT", "RAD", "RND", "SIN", "SQR", "TAN"
         K = K + 2
      CASE "AND", "MOD", "NOR", "XOR"
         K = K + 2
      CASE "NOT"
         K = K + 2
      CASE "RAN"
         FuncFlag = FALSE
         K = K + 2
      CASE "IM", "LN", "OR", "RE"
         K = K + 1
      CASE "PI"
         FuncFlag = FALSE
         K = K + 1
      CASE "I", "J"
         FuncFlag = FALSE
      CASE "X", "Y"
         FuncFlag = FALSE
         A1$ = MID$(E$, K + 1, 1)
         IF A1$ >= "0" AND A1$ <= "9" THEN
            ParsErr E$, K + 1, "This routine cannot perform operations on two data sets"
            EXIT SUB
         ELSE
         END IF
      CASE ELSE
         Msg$ = "The function " + CHR$(34) + C$ + CHR$(34) + " is unfamiliar"
         ParsErr E$, K, Msg$
         EXIT SUB
      END SELECT
      IF FuncFlag THEN
         DelimSet = "+-"                 ' e.g., "COS 3*Y^2" --> COS(3*Y^2)
         GOSUB Delimit
         E1$ = MID$(E$, K + 1, K1 - K - 1)
         Parse (E1$), Op$(), SN
         K = K1 - 1
      ELSE
      END IF
      SN = SN + 1
      Op$(SN) = C$
   CASE ELSE
       Msg$ = "The character " + CHR$(34) + A$ + CHR$(34) + " makes no sense here"
       ParsErr E$, K, Msg$
       EXIT SUB
   END SELECT
' PRINT "]";                                                     ' <----<<
LOOP

EXIT SUB

Delimit:
' Find the end of the current operator's (second) operand.
   P = 0
   FOR K1 = K + 1 TO L
      A1$ = MID$(E$, K1, 1)
      SELECT CASE A1$
      CASE "(", "[", "{"
         P = P + 1
      CASE ")", "]", "}"
         P = P - 1
         IF P < 0 THEN
            ParsErr E$, K1, "Extra right parenthesis in an operation's argument"
            EXIT FOR
         END IF
      CASE "A" TO "W", "Z"
         IF FuncFlag AND P = 0 THEN
           ' operation is a function; another function must end its argument.
            EXIT FOR
         END IF
      CASE ELSE
         IF INSTR(DelimSet, A1$) AND P = 0 THEN
            EXIT FOR
         END IF
      END SELECT
   NEXT K1
RETURN

END SUB

SUB ParsErr (Expr AS STRING, K AS INTEGER, Message AS STRING)
' Displays an error message from Parse.
' The message should be no more than 60 characters.
   CONST RED = 4
   CONST WHITE = 7
   CONST YELLOW = 14
  
   IF LEN(Message) > 60 THEN
      Message = LEFT$(Message, 60)
   END IF
   COLOR RED                                    ' <-- may need to be modified
   ClearTopLine
   PRINT "ERROR: "; Message; "; press a key."
   PRINT LEFT$(Expr, K - 1);
   COLOR YELLOW                                 ' <--
   PRINT MID$(Expr, K, 1);
   COLOR RED                                    ' <--
   PRINT MID$(Expr, K + 1);
   Junk$ = GetKey$
   COLOR WHITE
END SUB

SUB Polar (X AS DOUBLE, Y AS DOUBLE, R AS DOUBLE, Theta AS DOUBLE)
CONST PI = 3.14159265#
   R = SQR(X * X + Y * Y)
   IF X = 0 THEN
      Theta = PI / 2
   ELSE
      Theta = ATN(Y / X)
   END IF
END SUB

FUNCTION SINH# (X AS DOUBLE)
   SINH = (EXP(X) - EXP(-X)) / 2!
END FUNCTION

FUNCTION TANH# (X AS DOUBLE)
   DIM em, ep AS DOUBLE
   ep = EXP(X)
   em = EXP(-X)
   TANH = (ep - em) / (ep + em)
END FUNCTION

