DECLARE FUNCTION PGAUSS! (X AS SINGLE, MU AS SINGLE, SIGMA AS SINGLE)
DECLARE FUNCTION PLOREN! (X AS SINGLE, MU AS SINGLE, GAMMA AS SINGLE)
DECLARE FUNCTION LEGENDRE! (THETA AS SINGLE, L AS INTEGER)

FUNCTION LEGENDRE! (THETA AS SINGLE, L AS INTEGER)
'
'  Evaluate the L th Legendre polynomial at the angle THETA
'    cf Bevington, p. 179.
'
        COSINE = COS(.0174533 * THETA)
        P1 = 1!
        P2 = COSINE
        IF L = 0 THEN
          LEGENDRE = P1
        ELSE
          IF L = 1 THEN
            LEGENDRE = P2
          ELSE
            FOR K = 2 TO L
              FL = K
              P3 = ((2! * FL - 1!) * COSINE * P2 - (FL - 1!) * P1) / FL
              P1 = P2
              P2 = P3
            NEXT K
            LEGENDRE = P2
          END IF
       END IF
'
END FUNCTION

FUNCTION PGAUSS! (X AS SINGLE, MU AS SINGLE, SIGMA AS SINGLE)
'
'  Evaluate Gaussian probability function
'    cf Bevington, p. 45.
'
        Z = (X - MU) / SIGMA
        PGAUSS = .3989423 / SIGMA * EXP(-(Z ^ 2) / 2!)
'
END FUNCTION

FUNCTION PLOREN! (X AS SINGLE, MU AS SINGLE, GAMMA AS SINGLE)
'
'  Evaluate Lorentzian probability distribution
'    cf Bevington, p. 51.
'
        PLOREN = .1591549431# * GAMMA / ((X - MU) ^ 2 + (GAMMA / 2) ^ 2)
'
END FUNCTION

