'
'  Math Library
'
'  Copyright (c) 1990-1993 Creative Consulting for Research & Education
'  Last Revision: 6 Nov 93
'
'  $INCLUDE: 'c:\xyplot\devqb\include\mathlib.inc'
'
' ---------------------------------------------------------------
'
DEFINT I-N
'
' ---------------------------------------------------------------
'
SUB NumberParse (A() AS SINGLE, N AS INTEGER, V$)
'
'  Parse V$ into an arbitrary number of values which are separated by
'  commas, spaces, or tabs. This subroutine returns the values in A() and
'  the number of values found in N.
'
        DIM D1 AS STRING*1
        DIM D2 AS STRING*1
        DIM D3 AS STRING*1

        IF V$ = "" OR V$ = CHR$(13) OR V$ = CHR$(27) THEN
	  N = 0
	  EXIT SUB
	END IF

	V$ = LTRIM$(RTRIM$(V$))
        IF V$ = "" THEN
          N = 0
          EXIT SUB
        END IF

        L = LEN(V$)
	N = 1
	J = 1
        D1 = CHR$(44)  ' comma
        D2 = CHR$(9)   ' tab
        D3 = CHR$(32)  ' space

	DO
          K = INSTR(J, V$, D1)
          IF K = 0 THEN
            K = INSTR(J, V$, D2)
            IF K = 0 THEN
              K = INSTR(J, V$, D3)
            END IF
          END IF
	  IF K = 0 THEN
            A(N) = VAL(MID$(V$, J))
            EXIT DO
	  END IF
          IF J <> K THEN
	    NV$ = LTRIM$(MID$(V$, J, K - J))
	  ELSE
	    NV$ = LTRIM$(MID$(V$, J, 1))
	  END IF
	  SELECT CASE NV$
	    CASE ""
	      J = J + 1
            CASE D1, D2
	      N = N + 1
	      J = K + 1
	    CASE ELSE
	      A(N) = VAL(NV$)
	      N = N + 1
	      J = K + 1
	  END SELECT
	LOOP WHILE (J <= L)

END SUB
'
' ---------------------------------------------------------------
'
SUB ReScale (X(), Y(), JSr, JEr, IC, E$, IERR)
'
'  ReScale values in X, Y arrays according to expression E$.
'  (JSr, JEr) is the index interval for the real part of the data.
'  IC is an integer flag: = -1 for complex data, 0 for real data.
'  IERR = 0 for no error; otherwise set to appropriate error code.
'
        DIM XOP AS INTEGER
        DIM YOP AS INTEGER
        DIM CMPLX AS INTEGER
        DIM A(10) AS SINGLE

        IERR = 0
        XOP = 0 : YOP = 0 : CMPLX = 0
        E$ = RTRIM$(LTRIM$(UCASE$(E$)))
        L = LEN(E$)
        I1 = INSTR(E$, "X")
        I2 = INSTR(E$, "Y")
        NPTS = JEr - JSr + 1
        JSi = JEr + 1

'  Determine whether rescale expression is a function expression
'    or an algebraic expression.

'  The functions may be:
'
'       SIN     ATN    COS     LOG    TAN     EXP    L10
'       D>R     R>D
'
        I3 = INSTR("SIN/COS/TAN/ATN/LOG/EXP/L10/D>R/R>D/POL/", MID$(E$, 1, 3))
        IF I3 > 0 THEN

'  Evaluate functional expression

          I3 = (I3 - 1) / 4 + 1
          I4 = (I3 = 4) OR (I3 = 5) OR (I3 = 7) OR (I3 = 10)
          I5 = INSTR(MID$(E$, 4), "X")
          I6 = INSTR(MID$(E$, 4), "Y")
          IF I5 > 0 THEN XOP = -1 ELSE YOP = -1

          CMPLX = IC AND YOP
          IF I4 AND CMPLX THEN
            IERR = 240
            EXIT SUB
          END IF

          V$ = MID$(E$, MAX(4, MAX(I5, I6)+4))
          IF I3 = 10 THEN
            V$ = LTRIM$(V$)
            IF MID$(V$, 1, 1) = "," THEN V$ = MID$(V$, 2)
            FOR K = 1 TO 9
              A(K) = 0!
            NEXT K
            NumberParse A(), NTERMS, V$
            IF (NTERMS < 1) OR (NTERMS > 10) THEN
              IERR = 229
              EXIT SUB
            END IF
          END IF

          FOR K = 1 TO NPTS
            Jr = JSr + K - 1 : Ji = JSi + K - 1
            IF NOT CMPLX THEN C0i = 0!
            IF XOP THEN
              C1r = X(Jr) : C1i = 0!
	    ELSE
              C1r = Y(Jr)
              IF CMPLX THEN C1i = Y(Ji) ELSE C1i = 0!
	    END IF
            SELECT CASE I3
	      CASE 1
                IF CMPLX THEN
                  E1 = EXP(-C1i) : E2 = EXP(C1i)
                  C0r = 0.5*(E1 + E2)*SIN(C1r)
                  C0i = 0.5*(E2 - E1)*COS(C1r)
                ELSE
                  C0r = SIN(C1r)
                END IF
	      CASE 2
                IF CMPLX THEN
                  E1 = EXP(-C1i) : E2 = EXP(C1i)
                  C0r = 0.5*(E1 + E2)*COS(C1r)
                  C0i = 0.5*(E1 - E2)*SIN(C1r)
                ELSE
                  C0r = COS(C1r)
                END IF
	      CASE 3
                C0r = TAN(C1r)
                C0i = 0!
              CASE 4
                C0r = ATN(C1r)
                C0i = 0!
	      CASE 5
                IF C1r <= 0 THEN
                  IERR = 220
                  EXIT SUB
                END IF
                C0r = LOG(C1r)
                C0i = 0!
	      CASE 6
                IF CMPLX THEN
                  E1 = EXP(C1r)
                  C0r = E1*COS(C1i)
                  C0i = E1*SIN(C1i)
                ELSE
                  C0r = EXP(C1r)
                END IF
              CASE 7
                IF C1r <= 0 THEN
                  IERR = 220
                  EXIT SUB
                END IF
                C0r = LOG(C1r)*0.43429448
                C0i = 0
              CASE 8
                C0r = C1r * 0.017453293
                IF CMPLX THEN C0i = C1i * 0.017453293
              CASE 9
                C0r = C1r * 57.2957795
                IF CMPLX THEN C0i = C1i * 57.2957795
              CASE 10
                C0r = Horner (A(), NTERMS, C1r)
	      CASE ELSE
		'
	    END SELECT

            IF XOP THEN
              X(Jr) = C0r
	    ELSE
              Y(Jr) = C0r
              IF CMPLX THEN Y(Ji) = C0i
	    END IF
          NEXT K
	ELSE

'  Evaluate algebraic expression

	  I = I1 + I2

          IF I = 0 THEN
            IERR = 225
            EXIT SUB
          END IF
          IF I1 <> 0 AND I2 <> 0 THEN
            IERR = 225
            EXIT SUB
          END IF
          IF I <> 1 AND I <> L THEN
            IERR = 225
            EXIT SUB
          END IF

          B$ = MID$(E$, I, 1)

	  IF B$ = "X" THEN
            XOP = -1
	  ELSE
            YOP = -1
	  END IF

          CMPLX = IC AND YOP

'  Parse out operator and operand strings

          IF I = 1 THEN
            C$ = LTRIM$(RTRIM$(MID$(E$, 2, L - 1)))
	    L = LEN(C$)
	    O$ = MID$(C$, 1, 1)
	    D$ = MID$(C$, 2, L - 1)
	  ELSE
            C$ = LTRIM$(RTRIM$(MID$(E$, 1, L - 1)))
	    L = LEN(C$)
	    O$ = MID$(C$, L, 1)
	    D$ = MID$(C$, 1, L - 1)
	  END IF

	  I4 = INSTR("*/+-^", O$)

          IF I4 = 0 THEN
            IERR = 225
            EXIT SUB
          END IF
          IF I4 = 5 AND CMPLX THEN
            IERR = 240
            EXIT SUB
          END IF

'  Determine constant operand

          I5 = INSTR(D$, "(")
          IF I5 <> 0 THEN
            I6 = INSTR(D$, ",")
            IF I6 = 0 THEN
              IERR = 225
              EXIT SUB
            END IF
            C3r = VAL(LTRIM$(RTRIM$(MID$(D$, I5 + 1)))) ' complex numeric operand
            C3i = VAL(LTRIM$(RTRIM$(MID$(D$, I6 + 1))))
          ELSE
            C3r = VAL(LTRIM$(RTRIM$(D$)))                ' real numeric operand)
            C3i = 0!
          END IF

          IF I5 AND I6 AND NOT CMPLX THEN
            IERR = 241
            EXIT SUB
          END IF

          IF C3r = 0! AND C3i = 0! THEN
            IERR = 244
            EXIT SUB
          END IF

          FOR K = 1 TO NPTS
            Jr = JSr + K - 1 : Ji = JSi + K - 1
            C1i = 0!
            IF XOP THEN
              C1r = X(Jr)
	    ELSE
              C1r = Y(Jr)
              IF CMPLX THEN C1i = Y(Ji)
	    END IF
            C2r = C3r
            C2i = C3i

            IF I <> 1 THEN
              SWAP C1r, C2r
              SWAP C1i, C2i
            END IF

            IF NOT CMPLX THEN C0i = 0!

	    SELECT CASE I4
	      CASE 1
                IF CMPLX THEN
                  C0r = C1r*C2r - C1i*C2i
                  C0i = C1i*C2r + C1r*C2i
                ELSE
                  C0r = C1r * C2r
                END IF
	      CASE 2
                IF CMPLX THEN
                  D = C2r*C2r + C2i*C2i
                  C0r = (C1r*C2r + C1i*C2i)/D
                  C0i = (C1i*C2r - C1r*C2i)/D
                ELSE
                  C0r = C1r / C2r
                END IF
              CASE 3
                C0r = C1r + C2r
                C0i = C1i + C2i
	      CASE 4
                C0r = C1r - C2r
                C0i = C1i - C2i
	      CASE 5
                IFL = ((C2r - INT(C2r)) <> 0!)
                IF C1r < 0! AND IFL THEN
                  IERR = 245
                  EXIT SUB
                END IF
                C0r = C1r ^ C2r
	      CASE ELSE
                IERR = 225
                EXIT SUB
	    END SELECT

            IF XOP THEN
              X(Jr) = C0r
	    ELSE
              Y(Jr) = C0r
              IF CMPLX THEN Y(Ji) = C0i
	    END IF
          NEXT K
	END IF

END SUB
'
' ---------------------------------------------------------------
'
SUB Derivative (X(), Y(), J1, J2, J3, CMPLX%, IFLG)
'
'  **************************************************************
'  **                                                          **
'  **  Module: Derivative                                      **
'  **                                                          **
'  **  Purpose: Compute the numerical derivative of a data set.**
'  **                                                          **
'  **  Syntax: Derivative X(), Y(), J1, J2, J3, XC%, YC%, IFLG **
'  **                                                          **
'  **  Input Parameters:                                       **
'  **           X(), Y()   -- data arrays.                     **
'  **           J1, J2     -- starting and ending indices of   **
'  **                         X values.                        **
'  **           J3         -- starting index to store          **
'  **                         derivative.                      **
'  **           CMPLX%     -- integer flags set to true when   **
'  **                         data set is complex.             **
'  **  Output Parameters:                                      **
'  **           X(), Y()   -- contain derivative starting at   **
'  **                         index J3.                        **
'  **           IFLG       -- flag indicating success.         **
'  **                                                          **
'  **  Author: Krishna Myneni                                  **
'  **  Date: 30 Dec 89                                         **
'  **  Updated: 23 Jan 92 for complex values.                  **
'  **************************************************************
'
'
        NVALS = J2 - J1 + 1

        JSr = J1 : JEi = J2

        IF CMPLX% THEN
           NPTS = NVALS / 2
           JEr = J1 + NPTS - 1
           JSi = JEr + 1
        ELSE
           NPTS = NVALS
           JEr = J2
        END IF

        FOR J = 1 TO NVALS
	   X(J3 + J - 1) = X(J1 + J - 1)
	NEXT J

        J4 = J3 + NPTS
'
'  Calculate derivative at endpts with just two points.
'
        DXr = X(JSr + 1) - X(JSr)
        IF CMPLX% THEN
          DXi = X(JSi + 1) - X(JSi)
          DX2 = DXr*DXr + DXi*DXi
        ELSE
          DXi = 0!
          DX2 = DXr
        END IF

        IF DX2 = 0! THEN
	   IFLG = 0
	   EXIT SUB
	END IF

        DYr = Y(JSr + 1) - Y(JSr)
        IF CMPLX% THEN
          DYi = Y(JSi + 1) - Y(JSi)
        ELSE
          DYi = 0!
        END IF

        IF CMPLX% THEN
          YPr = (DYr*DXr + DYi*DXi) / DX2
          YPi = (DYi*DXr - DYr*DXi) / DX2
        ELSE
          YPr = DYr / DXr
        END IF

        Y(J3) = YPr
        IF CMPLX% THEN
          Y(J4) = YPi
        END IF

        DXr = X(JEr) - X(JEr - 1)

        IF CMPLX% THEN
          DXi = X(JEi) - X(JEi - 1)
          DX2 = DXr*DXr + DXi*DXi
        ELSE
          DXi = 0!
          DX2 = DXr
        END IF

        IF DX2 = 0! THEN
	   IFLG = 0
	   EXIT SUB
	END IF

        DYr = Y(JEr) - Y(JEr - 1)
        IF CMPLX% THEN
          DYi = Y(JEi) - Y(JEi - 1)
        ELSE
          DYi = 0!
        END IF

        IF CMPLX% THEN
          YPr = (DYr*DXr + DYi*DXi) / DX2
          YPi = (DYi*DXr - DYr*DXi) / DX2
        ELSE
          YPr = DYr / DXr
        END IF

        Y(J4 - 1) = YPr
        IF CMPLX% THEN
          Y(J4 + NPTS - 1) = YPi
        END IF

'
'  Calculate derivative everywhere else with forward and backward points.
'
	FOR J = 2 TO NPTS - 1
           Jr = JSr + J - 1
           Ji = JSi + J - 1
           DXr1 = X(Jr + 1) - X(Jr)
           DXr2 = X(Jr) - X(Jr - 1)
           IF CMPLX% THEN
             DXi1 = X(Ji + 1) - X(Ji)
             DXi2 = X(Ji) - X(Ji - 1)
             DX12 = DXr1*DXr1 + DXi1*DXi1
             DX22 = DXr2*DXr2 + DXi2*DXi2
           ELSE
             DXi1 = 0!
             DXi2 = 0!
             DX12 = DXr1
             DX22 = DXr2
           END IF

           IF (DX12 = 0!) OR (DX22 = 0!) THEN
	      IFLG = 0
	      EXIT SUB
	   END IF

           DYr1 = Y(Jr + 1) - Y(Jr)
           DYr2 = Y(Jr) - Y(Jr - 1)
           IF CMPLX% THEN
             DYi1 = Y(Ji + 1) - Y(Ji)
             DYi2 = Y(Ji) - Y(Ji - 1)
           ELSE
             DYi1 = 0!
             DYi2 = 0!
           END IF

           IF CMPLX% THEN
             YPr1 = (DYr1*DXr1 + DYi1*DXi1) / DX12
             YPr2 = (DYr2*DXr2 + DYi2*DXi2) / DX22
             YPi1 = (DYi1*DXr1 - DYr1*DXi1) / DX12
             YPi2 = (DYi2*DXr2 - DYr2*DXi2) / DX22
             YPr = (YPr1 + YPr2) / 2!
             YPi = (YPi1 + YPi2) / 2!
           ELSE
             YPr = (DYr1/DXr1 + DYr2/DXr2) / 2!
             YPi = 0!
           END IF

           JNr = J3 + J - 1
           JNi = J4 + J - 1
           Y(JNr) = YPr
           IF CMPLX% THEN
             Y(JNi) = YPi
           END IF
	NEXT J
'
'  Derivative computed successfully.
'
	IFLG = -1
'
END SUB
'
' ---------------------------------------------------------------
'
FUNCTION DETERM# (ARR#(), NORDER)
'
'  Compute the determinant of a square matrix.
'
'
        DIM DTERM AS DOUBLE

        DTERM = 1#
        FOR K = 1 TO NORDER
'
'    Interchange columns if diagonal element is zero.
'
          IF (ARR#(K, K) = 0#) THEN
            FOR J = K TO NORDER
              IF ARR#(K, J) = 0# THEN EXIT FOR
            NEXT J

            IF ARR#(K, J) = 0# THEN
              DETERM = 0#
              EXIT FUNCTION
            END IF

            FOR I = K TO NORDER
              SWAP ARR#(I, J), ARR#(I, K)
            NEXT I

            DTERM = -DTERM

          END IF
'
'  Subtract row K from lower rows to get diagonal matrix
'
          DTERM = DTERM * ARR#(K, K)

          IF K < NORDER THEN
            K1 = K + 1

            FOR I = K1 TO NORDER
              FOR J = K1 TO NORDER
                ARR#(I, J) = ARR#(I, J) - ARR#(I, K) * ARR#(K, J) / ARR#(K, K)
              NEXT J
            NEXT I

          END IF

        NEXT K

        DETERM = DTERM
'
END FUNCTION
'
' ---------------------------------------------------------------
'
SUB MATINV (ARRAY() AS DOUBLE, NORDER, DET AS DOUBLE)
'
'  **************************************************************
'  **                                                          **
'  **  MODULE: MATINV                                          **
'  **                                                          **
'  **  PURPOSE: Invert a symmetric matrix and calculate        **
'  **           its determinant.                               **
'  **                                                          **
'  **  SYNTAX: MATINV ARRAY(), NORDER, DET                     **
'  **                                                          **
'  **  INPUT PARAMETERS:                                       **
'  **                                                          **
'  **           ARRAY() -- two dimensional square matrix       **
'  **                      to be inverted (double precision).  **
'  **           NORDER  -- order of matrix (integer).          **
'  **                                                          **
'  **  OUTPUT PARAMETERS:                                      **
'  **                                                          **
'  **           ARRAY() -- original matrix is replaced by      **
'  **                      the inverse matrix.                 **
'  **           DET     -- determinant of matrix (double).     **
'  **                                                          **
'  **************************************************************
'
'
	DIM AMAX AS DOUBLE
	DIM IK(10) AS INTEGER
	DIM JK(10) AS INTEGER

	DET = 1#

	FOR K = 1 TO NORDER
' 
'          Find largest element ARRAY(I,J) in rest of matrix
'
FINDMAX:
		AMAX = 0#

		FOR I = K TO NORDER
		   FOR J = K TO NORDER
		      IF ((ABS(AMAX) - ABS(ARRAY(I, J))) <= 0#) THEN
			 AMAX = ARRAY(I, J)
			 IK(K) = I
			 JK(K) = J
		      END IF
		   NEXT J
		NEXT I
'
'               Interchange rows and columns to put AMAX in ARRAY(K,K)
'
		IF (AMAX = 0#) THEN
		   DET = 0#
		   EXIT SUB
		END IF

		I = IK(K)

		SELECT CASE (I - K)
		   CASE IS > 0
		      FOR J = 1 TO NORDER
			 SWAP ARRAY(K, J), ARRAY(I, J)
			 ARRAY(I, J) = -ARRAY(I, J)
		      NEXT J
		   CASE IS < 0
		      GOTO FINDMAX
		   CASE ELSE
		      '
		END SELECT
		J = JK(K)
		SELECT CASE (J - K)
		   CASE IS > 0
		      FOR I = 1 TO NORDER
			 SWAP ARRAY(I, K), ARRAY(I, J)
			 ARRAY(I, J) = -ARRAY(I, J)
		      NEXT I
		   CASE IS < 0
		      GOTO FINDMAX
		   CASE ELSE
		      '
		END SELECT
'
'               Accumulate elements of inverse matrix
'
		FOR I = 1 TO NORDER
		   IF ((I - K) <> 0) THEN
		      ARRAY(I, K) = -ARRAY(I, K) / AMAX
		   END IF
		NEXT I

		FOR I = 1 TO NORDER
		   FOR J = 1 TO NORDER
		      IF ((I - K) <> 0) THEN
			 IF ((J - K) <> 0) THEN
			    ARRAY(I, J) = ARRAY(I, J) + ARRAY(I, K) * ARRAY(K, J)
			 END IF
		      END IF
		   NEXT J
		NEXT I

		FOR J = 1 TO NORDER
		   IF ((J - K) <> 0) THEN
		      ARRAY(K, J) = ARRAY(K, J) / AMAX
		   END IF
		NEXT J

		ARRAY(K, K) = 1# / AMAX
		DET = DET * AMAX
	NEXT K
'
'       Restore ordering of matrix
'
	FOR L = 1 TO NORDER
	   K = NORDER - L + 1
	   J = IK(K)

	   IF ((J - K) > 0) THEN
	      FOR I = 1 TO NORDER
		 SWAP ARRAY(I, K), ARRAY(I, J)
		 ARRAY(I, K) = -ARRAY(I, K)
	      NEXT I
	   END IF

	   I = JK(K)

	   IF ((I - K) > 0) THEN
	      FOR J = 1 TO NORDER
		 SWAP ARRAY(K, J), ARRAY(I, J)
		 ARRAY(K, J) = -ARRAY(K, J)
	      NEXT J
	   END IF
	NEXT L
'
END SUB
'
' ---------------------------------------------------------------
'
SUB LinearLeastSquares (Y(), JA(), A(), JS, NPTS, ARR#(), Norder,_
                        ErrType%, Sigma, IERR)
'
'  Solve the general linear least squares problem through matrix
'    inversion.
'       Array JA contains the starting indices in Y for the fitting
'       functions. JS is the starting index of the data to be fitted.
'

        DIM SUM AS DOUBLE
        DIM PROD AS DOUBLE
        DIM WEIGHT AS DOUBLE
        DIM BETA(10) AS DOUBLE
        DIM DET AS DOUBLE

        Sigma2 = Sigma * Sigma
        IERR = 0
'
'  Set up the alpha matrix (also known as the curvature matrix); only
'    need to fill in lower half since matrix is symmetric.

        FOR J = 1 TO Norder
          FOR K = 1 TO J
            SUM = 0#
            FOR I = 1 TO NPTS
              IJ = JA(J) + I - 1
              IK = JA(K) + I - 1
              XJ = Y(IJ)
              XK = Y(IK)
              PROD = XJ * XK
              IF ErrType% = 0 THEN
                WEIGHT = CDBL(1!/Y(JS + I - 1))
              ELSEIF ErrType% = 1 THEN
                WEIGHT = CDBL(Sigma2)
              ELSE
                WEIGHT = 1#
              END IF
              PROD = PROD / WEIGHT
              SUM = SUM + PROD
            NEXT I
            ARR#(J, K) = SUM
          NEXT K
        NEXT J
'
'  Fill in upper half of matrix.
'
        FOR J = 1 TO Norder
          FOR K = J + 1 TO Norder
            ARR#(J, K) = ARR#(K, J)
          NEXT K
        NEXT J
'
'  Invert symmetric matrix alpha
'
        MATINV ARR#(), Norder, DET

        IF DET = 0# THEN
          IERR = 1
          EXIT SUB
        END IF
'
'  Compute vector BETA
'
        FOR J = 1 TO Norder
          SUM = 0#
          FOR I = 1 TO NPTS
            IJ = JA(J) + I - 1
            XJ = Y(IJ)
            YI = Y(JS + I - 1)
            PROD = CDBL(YI*XJ)
            IF ErrType% = 0 THEN
              WEIGHT = CDBL(1!/YI)
            ELSEIF ErrType% = 1 THEN
              WEIGHT = CDBL(Sigma2)
            ELSE
              WEIGHT = 1#
            END IF
            PROD = PROD / WEIGHT
            SUM = SUM + PROD
          NEXT I
          BETA(J) = SUM
        NEXT J
'
'  Compute coefficient vector A()
'
        FOR J = 1 TO Norder
          SUM = 0#
          FOR K = 1 TO Norder
            SUM = SUM + ARR#(J, K)*BETA(K)
          NEXT K
          A(J) = CSNG(SUM)
        NEXT J

END SUB
'
' ---------------------------------------------------------------
'
SUB POLFIT (X(), Y(), J1, J2, NTERMS, ARR#(), A(), EType%, Sigma, CHISQR)
'
'
	DIM SUMX(19) AS DOUBLE, SUMY(10) AS DOUBLE
	DIM XTERM AS DOUBLE
	DIM YTERM AS DOUBLE
	DIM CHISQ AS DOUBLE
	DIM DELTA AS DOUBLE
'
'  Accumulate weighted sums
'
	NPTS = J2 - J1 + 1
	NMAX = 2 * NTERMS - 1

	FOR N = 1 TO NMAX
	   SUMX(N) = 0#
	NEXT N

	FOR J = 1 TO NTERMS
	   SUMY(J) = 0#
	NEXT J

	CHISQ = 0#

	FOR I = J1 TO J2
	   XI = X(I): YI = Y(I)
'
'   NO WEIGHTING IS USED
'
	   XTERM = 1#

	   FOR N = 1 TO NMAX
	      SUMX(N) = SUMX(N) + XTERM
	      XTERM = XTERM * CDBL(XI)
	   NEXT N

	   YTERM = YI

	   FOR N = 1 TO NTERMS
	      SUMY(N) = SUMY(N) + YTERM
	      YTERM = YTERM * CDBL(XI)
	   NEXT N

	   CHISQ = CHISQ + CDBL(YI) ^ 2
	NEXT I
'
'  CONSTRUCT MATRICES AND CALCULATE COEFFICIENTS
'
	FOR J = 1 TO NTERMS
	   FOR K = 1 TO NTERMS
	      N = J + K - 1
              ARR#(J, K) = SUMX(N)
	   NEXT K
	NEXT J

        DELTA = DETERM(ARR#(), NTERMS)

	IF (DELTA = 0#) THEN
	   CHISQR = 0!

	   FOR J = 1 TO NTERMS
	      A(J) = 0!
	   NEXT J
	ELSE
	   FOR L = 1 TO NTERMS
	      FOR J = 1 TO NTERMS
		 FOR K = 1 TO NTERMS
		    N = J + K - 1
                    ARR#(J, K) = SUMX(N)
		 NEXT K
                 ARR#(J, L) = SUMY(J)
	      NEXT J
              A(L) = DETERM(ARR#(), NTERMS) / DELTA
	   NEXT L
'
'   CALCULATE CHI SQUARE
'
	   FOR J = 1 TO NTERMS
	      CHISQ = CHISQ - 2# * CDBL(A(J)) * SUMY(J)

	      FOR K = 1 TO NTERMS
		 N = J + K - 1
		 CHISQ = CHISQ + CDBL(A(J)) * CDBL(A(K)) * SUMX(N)
	      NEXT K
	   NEXT J

	   FREE = NPTS - NTERMS

	   IF FREE = 0! THEN
	      CHISQR = 0!
	   ELSE
	      CHISQR = CHISQ / FREE
	   END IF
	END IF
'
END SUB
'
' ---------------------------------------------------------------
'
FUNCTION FMAX! (X1, X2)
'
'  Return maximum of x1 and x2
'
	FMAX = X1
	IF X2 > X1 THEN FMAX = X2
'
END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION FMIN! (X1, X2)
'
'  Return minimum of x1 and x2
'
	FMIN = X1
	IF X2 < X1 THEN FMIN = X2
'
END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION Horner (A(), NTERMS, XX)
'
'  COMPUTE POLYNOMIAL WITH NTERMS COEFFICIENTS STORED IN A
'    USING HORNER'S ALGORITHM
'
	P = A(NTERMS)

	FOR J = NTERMS - 1 TO 1 STEP -1
	   P = A(J) + P * XX
	NEXT J

	Horner = P
'
END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION EquallySpaced% (X(), J1, J2, TOL)
'
' Return TRUE (-1) if X values are equally spaced between
'   X(J1) and X(J2); return FALSE (0) otherwise. TOL sets
'   the maximum allowed difference between intervals.
'
        JS = J1 : JE = J2
        IF JE < JS THEN SWAP JS, JE
        XS0 = X(JS + 1) - X(JS)
        JS1 = JS + 1
        JE1 = JE - 1
        FOR J = JS1 TO JE1
          XS = X(J + 1) - X(J)
          IF ABS(XS-XS0) > TOL THEN
            EquallySpaced = 0
            EXIT FUNCTION
          END IF
        NEXT J
        EquallySpaced = -1
END FUNCTION
'
' ---------------------------------------------------------------
'
SUB IndexLimits (X(), X1, X2, JS, JE, J1, J2, IERR)
'
'       IndexLimits
'
'       Purpose: Determine the indices of points falling in
'         the domain (x1, x2).
'       Syntax: IndexLimits X(), X1, X2, JS, JE, J1, J2, IERR
'       Input Parameters:
'         X     -- array of X values
'         X1    -- lower limit of domain
'         X2    -- upper limit of domain
'         JS    -- lower index limit in X array
'         JE    -- upper index limit in X array
'       Output Parameters:
'         J1    -- lower index limit of domain
'         J2    -- upper index limit of domain
'         IERR  -- error code has the following meaning:
'                    0 -- no error
'                    1 -- (X1, X2) interval does not overlap
'                         interval (X(JS), X(JE))
'       Author: K. Myneni
'       Date: 16 August 90
'       Notes:
'         The X array values between indices JS and JE may be in
'         increasing or decreasing order. The output indices, J1
'         and J2 will always be returned with J1 less than J2,
'         regardless of the ordering of x values.
'         Translated to QuickBasic on 25 Aug 90.
'


	IERR = 0

'      Make sure x limits are used in order of minimum, maximum.

	IF X2 < X1 THEN
	   XX1 = X2
	   XX2 = X1
	ELSE
	   XX1 = X1
	   XX2 = X2
	END IF

'       Search for index of lower limit

	IF X(JE) > X(JS) THEN
           FOR J = JS TO JE
	      IF X(J) >= XX1 THEN
		 J1 = J
		 GOTO C1
	      END IF
	   NEXT J
	ELSE
           FOR J = JE TO JS STEP - 1
	      IF X(J) >= XX1 THEN
		 J1 = J
		 GOTO C1
	      END IF
	   NEXT J
	END IF

	' x limits outside domain of data

	IERR = 1
	EXIT SUB

C1:

'       Search for index of upper limit

	IF X(JE) > X(JS) THEN
           FOR J = JE TO JS STEP -1
	      IF X(J) <= XX2 THEN
		 J2 = J
		 GOTO C2
	      END IF
	   NEXT J
	ELSE
           FOR J = JS TO JE
	      IF X(J) <= XX2 THEN
		 J2 = J
		 GOTO C2
	      END IF
	   NEXT J
	END IF

	' x limits outside domain of data

	IERR = 1
	EXIT SUB

C2:

'       Put indices in order

	IF J2 < J1 THEN SWAP J1, J2

END SUB
'
' ---------------------------------------------------------------
'
FUNCTION INTERVAL (I, J, K)
'
'  Return yes if K is in the closed interval [I, J]
'
	INTERVAL = (K >= MIN(I, J)) AND (K <= MAX(I, J))
'
END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION JFIND (X(), XX, J1, J2)
'
'  Find index of XX in array X, with starting and ending indices
'    at J1 and J2.
'
	J0 = (X(J2) > X(J1))
	JL = J1 - 1
	JU = J2 + 1

	DO WHILE ((JU - JL) > 1)
	   JM = (JU + JL) / 2

	   IF (J0 EQV (XX > X(JM))) THEN
	      JL = JM
	   ELSE
	      JU = JM
	   END IF

	LOOP

	IF JL < J1 THEN
	   JL = JL + 1
	END IF

	JFIND = JL
'
END FUNCTION
'
' ---------------------------------------------------------------
'
SUB LinearInterp (X1, Y1, X2, Y2, X, Y, JFL)
'
'  Perform linear interpolation to obtain Y, given X, and the endpoint
'    coordinates X1, Y1, and X2, Y2. JFL is an error flag for x1 = x2.
'
	IF X2 = X1 THEN
	  JFL = -1
	ELSE
	  Y = (Y2 * X - Y2 * X1 - Y1 * X + Y1 * X2) / (X2 - X1)
	  JFL = 0
	END IF
'
END SUB
'
' ---------------------------------------------------------------
'
FUNCTION MAX% (I, J)
'
'  Return max of I and J
'
	IF I < J THEN
	   MAX = J
	ELSE
	   MAX = I
	END IF
'
END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION MIN% (I, J)
'
'  Return minimum of I and J
'
	IF I < J THEN
	   MIN = I
	ELSE
	   MIN = J
	END IF
'
END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION ParabArea# (X(), Y(), JS, JE)
'
'  Find area for points between X(JS) and X(JE) using parabolic
'  fit to data. Return the area as double precision number.
'
        DIM Ar AS DOUBLE

        Ar = 0#
        ParabArea = 0#

        X1 = X(JS)
        Y1 = Y(JS)
        X2 = X(JS + 1)
        Y2 = Y(JS + 1)
        X3 = X(JS + 2)
        Y3 = Y(JS + 2)

	J = JS + 2

	DO

           XMIN = X1
           IF X3 < XMIN THEN XMIN = X3
           XP1 = X1 - XMIN
           XP2 = X2 - XMIN
           XP3 = X3 - XMIN
'
'          Perform parabolic fit to 3 points.
'
           XP1S = XP1 * XP1
           XP2S = XP2 * XP2

           A11 = XP2 - XP1
           A12 = XP2S - XP1S
           A21 = XP3 - XP2
           A22 = XP3*XP3 - XP2S
           YY1 = Y2 - Y1
           YY2 = Y3 - Y2
           DET = A11*A22 - A21*A12
           B = (A22*YY1 - A12*YY2) / DET
           C = (A11*YY2 - A21*YY1) / DET
           A = Y1 - B*XP1 - C*XP1S

	   IF J = JE THEN
              XM = XP3
	   ELSE
              XM = (XP3 + XP2) / 2!
	   END IF

           XMS = XM*XM

           AreaElement# = A * (XM - XP1)
           AreaElement# = AreaElement# + B * (XMS - XP1S) / 2!
           AreaElement# = AreaElement# + C * (XM*XMS - XP1*XP1S) / 3!
'
'          Accumulate area
'
           Ar = Ar + AreaElement#
'
'          Choose next 3 points for parabolic fit; first point is the
'            midpoint between last two fitting points.
'
           X1 = XM + XMIN
           Y1 = A + B*XM + C*XMS

           X2 = X(J)
           Y2 = Y(J)

	   IF J < JE THEN
              X3 = X(J + 1)
              Y3 = Y(J + 1)
	   END IF

	   J = J + 1
	LOOP WHILE J <= JE

        IF X(JE) < X(JS) THEN Ar = -Ar

        ParabArea = Ar
'
END FUNCTION
'
' ---------------------------------------------------------------
'
FUNCTION Simpson# (X(), Y(), J1, J2)
'
'  Compute area under points Y(JS) to Y(JE). This routine is only
'    valid for equally spaced abscissas. Return double precision
'    area.
'
        DIM Ar AS DOUBLE

        Ar = 0#
        Simpson = 0#
        JS = J1
        JE = J2
        IF JE < JS THEN SWAP JS, JE
        XS = ABS (X(JS + 1) - X(JS))
        Ar = CDBL(Y(JS)/3!)
        JS1 = JS + 1
        JE1 = JE - 2
        K = 0
        FOR J = JS1 TO JE1
          K = NOT K
          S = 2*ABS(K) + 2
          Ar = Ar + CDBL(S*Y(J)/3!)
        NEXT J
'
' Handle endpoints; For even number of points, use trapezoid rule for last
'   two points.
'
        IF K THEN
          Ar = Ar + CDBL(Y(JE - 1)/3!)
          Ar = Ar + CDBL((Y(JE - 1) + Y(JE))/2!)
        ELSE
          Ar = Ar + CDBL((4!*Y(JE - 1) + Y(JE))/3!)
        END IF

        Simpson = CDBL(XS) * Ar

END FUNCTION
'
' ---------------------------------------------------------------
'
SUB CopyXImag (X(), JS, NPTS)
'
'  Copy real part of x array into the imaginary part of x in
'    a complex data set -- this is useful for certain functions.
'
'    JS is the starting index for the real x values, and NPTS
'    is the number of points.
'
        JF = JS + NPTS - 1
        FOR J = 1 TO NPTS
          JN = JF + J
          JO = JS + J - 1
          X(JN) = X(JO)
        NEXT J
'
END SUB
'
' ---------------------------------------------------------------
'
SUB ZeroXImag (X(), JS, NPTS)
'
'  Zero the imaginary x values of a complex data set. Note that
'    JS is the index for the starting point of the real x values.
'
        JF = JS + NPTS - 1
        FOR J = 1 TO NPTS
          JN = JF + J
          X(JN) = 0!
        NEXT J
'
END SUB
'
' ---------------------------------------------------------------
'
SUB SortComplex1 (X(), Y(), J1, J2)
'
'  Resort a complex data sorted as y1r, y1i, y2r, y2i, ...etc.
'    so that the new order is y1r, y2r, ... , y1i, y2i, ...etc.
'    X is assumed to have only real values which are contiguous.
'    J1% and J2% are the indices of the first and last values of
'    real part of X. The lower half of X array is used for sorting.
'
        NPTS = J2 - J1 + 1

        FOR J = 1 TO NPTS        ' move Im(y) into lower half of x
          Ji = J1 + 2*J - 1
          JX = J2 + J
          X(JX) = Y(Ji)
        NEXT J

        FOR J = 2 TO NPTS         ' move Re(y) to make contiguous
          JOr = J1 + 2*J - 2
          JNr = J1 + J - 1
          Y(JNr) = Y(JOr)
        NEXT J

        FOR J = 1 TO NPTS          ' move Im(y) back into y array
          Ji = J2 + J              ' and set Im(x) = 0.
          Y(Ji) = X(Ji)
          X(Ji) = 0!
        NEXT J

END SUB
'
' ---------------------------------------------------------------
'
SUB Sort2 (X(), Y(), J1, J2)
'
'  Sort x, y values between indices J1 and J2 in order of
'  increasing X.
'
        N = J2 - J1 + 1
        L = N/2 + 1 : LJ = J1 + L - 1
        IR = N : IRJ = J1 + IR - 1

        DO
          IF L > 1 THEN
            L = L - 1: LJ = LJ - 1
            RRA = X(LJ)
            RRB = Y(LJ)
          ELSE
            RRA = X(IRJ)
            RRB = Y(IRJ)
            X(IRJ) = X(J1)
            Y(IRJ) = Y(J1)
            IR = IR - 1 : IRJ = IRJ - 1
            IF IR = 1 THEN
              X(J1) = RRA
              Y(J1) = RRB
              EXIT DO
            END IF
          END IF

          I = L : IJ = LJ
          J = L + L : JJ = J1 + J - 1

          DO WHILE (J <= IR)
            IF J < IR THEN
              IF X(JJ) < X(JJ + 1) THEN J = J + 1 : JJ = JJ + 1
            END IF
            IF RRA < X(JJ) THEN
              X(IJ) = X(JJ)
              Y(IJ) = Y(JJ)
              I = J : IJ = JJ
              J = J + J : JJ = J1 + J - 1
            ELSE
              J = IR + 1 : JJ = IRJ + 1
            END IF
          LOOP

          X(IJ) = RRA
          Y(IJ) = RRB
        LOOP
'
END SUB
'
' ---------------------------------------------------------------
'
SUB SMOOTH (Y(), J1, J2)
'
'  Smooth a set of data by three point convolution.
'
	IMAX = J2 - 1
	IMIN = J1 + 1
	YIM1 = Y(J1)
	Y(J1) = (3 * Y(J1) + Y(IMIN)) / 4

	FOR I = IMIN TO IMAX
	   YNEW = (YIM1 + 2 * Y(I) + Y(I + 1)) / 4
	   YIM1 = Y(I)
	   Y(I) = YNEW
	NEXT I

	Y(J2) = (YIM1 + 3 * Y(J2)) / 4
'
END SUB
'
' ---------------------------------------------------------------
'
SUB Four1 (Y(), J1, J2, ISIGN)
'
'  Replace data values between Y(J1) to Y(J2) with its discrete
'    Fourier transform (ISIGN =  1), or with its inverse discret
'    Fourier transform (ISIGN = -1) . The real and imaginary
'    parts of each point must be in alternating order.
'
'  Routine from Numerical Recipes, by Press et al.
'
        DIM WR AS DOUBLE
        DIM WI AS DOUBLE
        DIM WPR AS DOUBLE
        DIM WPI AS DOUBLE
        DIM WTEMP AS DOUBLE
        DIM THETA AS DOUBLE

        N = J2 - J1 + 1
        J = J1

        FOR I0 = 1 TO N STEP 2
          I = J1 + I0 - 1

          IF J > I THEN
            SWAP Y(J), Y(I)
            SWAP Y(J + 1), Y(I + 1)
          END IF

          M = N / 2

          DO WHILE (M >= 2 AND (J - J1 + 1) > M)
            J = J - M
            M = M / 2
          LOOP

          J = J + M
        NEXT I0

        MMAX = 2
        DO WHILE (N > MMAX)
          ISTEP = 2 * MMAX
          THETA = 6.28318530717959# / CDBL(ISIGN*MMAX)
          WPR = -2# * SIN(0.5# * THETA)^2
          WPI = SIN(THETA)
          WR = 1#
          WI = 0#
          FOR M = 1 TO MMAX STEP 2
            FOR I0 = M TO N STEP ISTEP
              I = J1 + I0 - 1
              J = I + MMAX
              TEMPr = CSNG(WR)*Y(J) - CSNG(WI)*Y(J + 1)
              TEMPi = CSNG(WR)*Y(J + 1) + CSNG(WI)*Y(J)
              Y(J) = Y(I) - TEMPr
              Y(J + 1) = Y(I + 1) - TEMPi
              Y(I) = Y(I) + TEMPr
              Y(I + 1) = Y(I + 1) + TEMPi
            NEXT I0
            WTEMP = WR
            WR = WR*WPR - WI*WPI + WR
            WI = WI*WPR + WTEMP*WPI + WI
          NEXT M
          MMAX = ISTEP
        LOOP
END SUB
'
' ---------------------------------------------------------------
'
FUNCTION Pad2% (NPTS)
'
'  If NPTS is an integer power of 2, return NPTS; otherwise, return
'    next highest integer power of 2.
'
        M = 1
        DO WHILE (M < NPTS)
          M = M * 2
        LOOP
        Pad2 = M

END FUNCTION
'
' ---------------------------------------------------------------
'
