'  Utilities Module 2 for XYPLOT
'
'  Last Revision: 24 Oct 1993
'
'       $INCLUDE: 'c:\XYPLOT\DEVQB\INCLUDE\MATHLIB.INC'
'       $INCLUDE: 'c:\XYPLOT\DEVQB\INCLUDE\GRAPHLIB.INC'
'       $INCLUDE: 'c:\XYPLOT\DEVQB\INCLUDE\DOSLIB.INC'
'       $INCLUDE: 'c:\XYPLOT\DEVQB\INCLUDE\XYPLOT.INC'
'
'
' ---------------------------------------------------------------
'
DEFINT I-N
'
' ---------------------------------------------------------------
'
SUB DisplayDirectory (FileSpec$, IR1, NLINES, IFC, IBC, IAC) STATIC
'
'  **************************************************************
'  **                                                          **
'  **  Module: DisplayDirectory                                **
'  **  Purpose: Display the directory of files specified by    **
'  **           the string FileSpec$                           **
'  **  Syntax: DisplayDirectory FileSpec$, IR1, NLINES         **
'  **  Input Parameters:                                       **
'  **                                                          **
'  **           FileSpec$ -- file specification;               **
'  **                        may include wildcards.            **
'  **                        Set FileSpec$ to "PgDn" for       **
'  **                        next display screen, or "PgUp"    **
'  **                        for first display screen.         **
'  **           IR1  -- first row on screen to display.        **
'  **           NLINES -- number of rows on screen for display **
'  **           IFC, IBC, IAC -- fore, back, aux colors        **
'  **                                                          **
'  **  Output Parameters: NONE                                 **
'  **  Author: K. Myneni                                       **
'  **  Date: 28 Dec 89; modified 12/6/92                       **
'  **                                                          **
'  **************************************************************
'
'
'  Display directory area and dir name
'
        DIM FileCount AS INTEGER
        DIM TopFile AS INTEGER

        IR2 = IR1 + NLINES
        TextBox IR1, 1, IR2, 80, IFC, IBC
        GetDrive ID%
        GetDirectory DIR$
        DIR$ = MID$(DIR$, 1, LEN(DIR$) - 1)
        DIR$ = CHR$(ID% + 65) + ":" + DIR$
        IC = (80 - LEN(DIR$)) / 2
        LOCATE IR1, IC - 1
        COLOR IAC
        PRINT " "; DIR$; " ";

'  Check to see if FileSpec$ is PgUp, or PgDn keys,
'    else retrieve file list for current directory

        IF FileSpec$ = "PgDn" THEN
          ' display next page
          IF FileCount >= NFILES THEN
            SOUND 500, 3
          ELSE
            TopFile = FileCount
          END IF
        ELSEIF FileSpec$ = "PgUp" THEN
          ' display previous page
          IF TopFile = 1 THEN
            SOUND 500, 3
          ELSE
            TopFile = TopFile - (NLINES - 2) * 4
          END IF
          IF TopFile < 1 THEN TopFile = 1
        ELSEIF FileSpec$ = "<HOME>" THEN
          ' first page
          TopFile = 1
        ELSE
          ' obtain directory listing and display from top
          FindFiles FileSpec$, FileList$, NFILES
          TopFile = 1
        END IF

        COLOR IAC
        IF TopFile > 1 THEN
          LOCATE IR1, 65
          PRINT "PgUp"
        END IF

        IR = IR1 + 2            ' row counter
        J = 0                   ' column counter
        FileCount = TopFile     ' first file to display

	DO
	  IF J > 3 THEN
	    J = 0
	    IR = IR + 1
	  END IF
          IF IR >= IR2 THEN
            IF FileCount < NFILES THEN
              LOCATE IR2, 65
              COLOR IAC
              PRINT "PgDn";
            END IF
            EXIT DO
          END IF
          FM$ = MID$(FileList$, (FileCount - 1) * 16 + 1, 15)
          IF MID$(FM$, 1, 1) = "[" THEN
            COLOR IAC
          ELSE
            COLOR IFC
          END IF
          LOCATE IR, J * 20 + 4
          PRINT FM$;
          FileCount = FileCount + 1
          J = J + 1
        LOOP WHILE FileCount <= NFILES

'
END SUB
'
' ---------------------------------------------------------------
'            XYPLOT FILE ROUTINES
' ---------------------------------------------------------------
'
SUB ReadIniFile (Pcolors$, Wcolors$)
'
'  Read xyplot.ini if it exists. Check environment string table
'    for path.
'
        D$ = LTRIM$(RTRIM$(ENVIRON$("XYPINI")))
        IF D$ <> "" THEN
          L = LEN(D$)
          IF MID$(D$, L, 1) <> "\" THEN
            D$ = D$ + "\"
          END IF
        END IF
        D$ = D$ + "XYPLOT.INI"
        IF FileExists (D$) THEN
          OPEN D$ FOR INPUT AS #1
          DO WHILE NOT(EOF(1))
            LINE INPUT #1, FL$
            FL$ = UCASE$(LTRIM$(FL$))
            K = INSTR(FL$, "=")
            O$ = LTRIM$(MID$(FL$, K + 1))
            IF MID$(FL$, 1, 1) = "!" THEN
              ' ignore comment line
            ELSEIF INSTR(FL$, "SCR") <> 0 THEN
              ' set screen mode
              IF INSTR(O$, "CGALO") <> 0 THEN
                SI.N = 1
              ELSEIF INSTR(O$, "CGA") <> 0 THEN
                SI.N = 2
              ELSEIF INSTR(O$, "EGALO") <> 0 THEN
                SI.N = 7
              ELSEIF INSTR(O$, "EGAMED") <> 0 THEN
                SI.N = 8
              ELSEIF INSTR(O$, "EGAMON") <> 0 THEN
                SI.N = 10
              ELSEIF INSTR(O$, "EGA") <> 0 THEN
                SI.N = 9
              ELSEIF INSTR(O$, "MCGALO") <> 0 THEN
                SI.N = 13
              ELSEIF INSTR(O$, "MCGA") <> 0 THEN
                SI.N = 11
              ELSEIF INSTR(O$, "VGA") <> 0 THEN
                SI.N = 12
              ELSE
                '
              END IF
            ELSEIF INSTR(FL$, "SYM") <> 0 THEN
              ' set default symbol
              IF INSTR(O$, "+") <> 0 THEN
                Dsym$ = "+"
              ELSEIF INSTR(O$, "P") <> 0 THEN
                Dsym$ = "P"
              ELSEIF INSTR(O$, "S") <> 0 THEN
                Dsym$ = "S"
              ELSE
                '
              END IF
            ELSEIF INSTR(FL$, "PLOT COL") <> 0 THEN
              ' set plot colors
              Pcolors$ = O$
            ELSEIF INSTR(FL$, "WINDOW COL") <> 0 THEN
              ' set window colors
              Wcolors$ = O$
            ELSEIF INSTR(FL$, "MONO") <> 0 THEN
              ' set monochrome monitor
              SI.MFLAG = YES
            ELSEIF INSTR(FL$, "SAVE") <> 0 THEN
              ' set save options
              IF INSTR(O$, "NO HEAD") <> 0 THEN VI.HS = NO
              IF INSTR(O$, "FLOAT") <> 0 THEN
                VI.FM = "F"
              ELSEIF INSTR(O$, "INTEG") <> 0 THEN
                VI.FM = "I"
              ELSE
                '
              END IF
              IF INSTR(O$, "COMMA") <> 0 THEN VI.DL = ","
            ELSEIF INSTR(FL$, "MOUSE") <> 0 THEN
              IF INSTR(O$, "OFF") <> 0 THEN DI.CMODE = NO
            ELSEIF INSTR(FL$, "GRID") <> 0 THEN
              DI.Grid = YES
            ELSE
              ' unrecognized setting
            END IF
          LOOP
          CLOSE #1
        END IF
'
END SUB
'
' ---------------------------------------------------------------
'
SUB XYREAD (X(), Y(), NPTS, DataType%, JS, JE, IERR, FL$, HDR$)
'
'
'    ++++++++        Read file in X, Y [, Im(Y)] format     ++++++++++
'
'       Variables:
'
'         F$ = Filename
'       HDR$ = File header
'          X = x array
'          Y = y array
'       NPTS = number of points read
'  DataType% = integer flag for complex data (-1 for complex, 0 for real)
'         JS = starting index for storing data
'         JE = ending index for stored (x, y) values
'      IERR = error number for read failure
'
' Modified to be able to read multi-column files -- 8/10/91 KM
'
        DIM A(20) AS SINGLE

	JE = JS
	OPEN FL$ FOR INPUT AS #1
        ReadHeader FL$, HDR$, V$

'  Parse first non-blank line and count number of columns
'    Recognized delimiters are a space, comma, or tab
'
        NumberParse A(), NCOLS, V$

        NC = NCOLS

        SELECT CASE NCOLS
          CASE 0
            IERR = 223         ' no valid data in first non-blank line
            EXIT SUB
          CASE 1
            IYRC = 1
          CASE 2
            IXC = 1
            IYRC = 2
          CASE IS > 2
            SelectColumns NCOLS, NC, IXC, IYRC, IYIC
            IF NC < 1 OR NC > 3 THEN
              IERR = 248
              EXIT SUB
            END IF
        END SELECT

'  Read file of 1, 2, or 3 columns of data

        SELECT CASE NC
          CASE 1
	    DataType% = NO                      ' real x, real y
	    K = 1
	    X(JE) = K
            Y(JE) = A(IYRC)

	    DO
              LINE INPUT #1, V$
              NumberParse A(), N, V$
              IF N >= IYRC THEN
                JE = JE + 1
                K = K + 1
                X(JE) = K
                Y(JE) = A(IYRC)
              END IF
	    LOOP WHILE NOT EOF(1)

	  CASE 2
	    DataType% = NO                      ' real x, real y
            X(JE) = A(IXC)
            Y(JE) = A(IYRC)

	    DO
              LINE INPUT #1, V$
              NumberParse A(), N, V$
              IF N >= IYRC THEN
                JE = JE + 1
                X(JE) = A(IXC)
                Y(JE) = A(IYRC)
              END IF
	    LOOP WHILE NOT EOF(1)

	  CASE 3
	     DataType% = YES                     ' real x, complex y
	    
	     ' Read in point re(x), re(y), im(y) as one row of x, and
	     ' and two rows of y values: (re(x1), re(y1)), and (x2, im(y1))
	    
             JX = JE
             JY = JE
             X(JX) = A(IXC)
             Y(JY) = A(IYRC)
	     JY = JY + 1
             Y(JY) = A(IYIC)

	     DO
               LINE INPUT #1, V$
               NumberParse A(), N, V$
               IF N >= IYIC THEN
                 JX = JX + 1
                 JY = JY + 1
                 X(JX) = A(IXC)
                 Y(JY) = A(IYRC)
                 JY = JY + 1
                 Y(JY) = A(IYIC)
               END IF
	     LOOP WHILE NOT EOF(1)

	     JE = JY

            ' Reorganize so that real parts of x and y are adjacent,
            '   and imaginary parts are adjacent.

            SortComplex1 X(), Y(), JS, JX

          CASE ELSE
            '
        END SELECT

	CLOSE #1

	NPTS = JE - JS + 1
	IF DataType% THEN NPTS = NPTS / 2

END SUB
'
' ---------------------------------------------------------------
'
SUB ReadHeader (FL$, HDR$, A$)
'
'  This subroutine reads the header (HDR$) of a file (F$), and
'    returns the first non-blank line after the header in A$.
'
'  The file must be opened for input and closed by the calling routine as
'    #1.
'

	HDR$ = ""

'  Ignore blank lines at beginning of file

	DO
	  LINE INPUT #1, A$
	LOOP WHILE BlankLine(A$)

'  Check to see if first non-blank line is a comment line

	K = INSTR(A$, "/*")
	IF K <> 0 THEN
	  HDR$ = MID$(A$, K + 2, LEN(A$) - (K + 1)) + CHR$(13)

	  DO
	    LINE INPUT #1, A$
	    K = INSTR(A$, "*/")

	    IF K <> 0 THEN
	      A$ = MID$(A$, 1, K - 1)
	    ELSE
	      A$ = A$ + CHR$(13)
	    END IF

	    HDR$ = HDR$ + A$
	  LOOP WHILE K = 0

	  LINE INPUT #1, A$
	END IF

'  Check for blank lines at end of comment line

	DO WHILE BlankLine(A$)
	  LINE INPUT #1, A$
	LOOP

END SUB
'
' ---------------------------------------------------------------
'
SUB SelectColumns (N, NC, IXC, IYRC, IYIC)
'
'  Prompt user to select input columns for multicolumn file.
'  N is number of columns in file;
'
'
        DIM A(10) AS SINGLE

        NC = 0

        SOUND 500, 1
        LOCATE 2, 2
        PRINT SPACE$(78);
        LOCATE 2, 3
        PRINT "FILE CONTAINS ";N;" COLUMNS -- ";
        PRINT "Please specify columns to load: ";
        GetInput V$
        LOCATE 2, 2
        PRINT SPACE$(78);
        LOCATE 2, 18
        PRINT "READING FILE ..."
        IF V$ = CHR$(ESC) THEN EXIT SUB
        IF V$ = CHR$(RET) THEN
          NC = 3
          IXC = 1
          IYRC = 2
          IYIC = 3
        ELSE
          NumberParse A(), NC, V$
          IF NC > 3 THEN NC = 3
          SELECT CASE NC
            CASE 1
              IXC = 0
              IYRC = INT(A(1))
            CASE 2
              IXC = INT(A(1))
              IYRC = INT(A(2))
            CASE 3
              IXC = INT(A(1))
              IYRC = INT(A(2))
              IYIC = INT(A(3))
            CASE ELSE
              NC = 0
          END SELECT
        END IF
END SUB
'
' ---------------------------------------------------------------
'
SUB StoreData (X(), Y(), NS)
'
'  Store specified data set
'
        JSr = DS(NS).JSr
        JEr = DS(NS).JEr
        JSi = DS(NS).JSi
        JEi = DS(NS).JEi
        NPTS = DS(NS).NPTS

        ITRS = 0                        ' truncated save flag
        INF = 0                         ' take integer flag
	HDR$ = RTRIM$(DS(NS).HDR)

        J1 = JSr
        J2 = JEr

	IF (DI.XMIN > DS(NS).XMIN) OR (DI.XMAX < DS(NS).XMAX) THEN
	   ClearTopLine
           PRINT "OK to store ONLY displayed data? "
           ButtonMenu 1, KeyPress$
           IF INSTR(KeyPress$, "CANC") <> 0 THEN EXIT SUB
           IF KeyPress$ = CHR$(RET) OR KeyPress$ = CHR$(ESC) THEN EXIT SUB

           IF INSTR(KeyPress$, "Y") <> 0 THEN
	      ITRS = -1                    ' flag to indicate truncated store

              IndexLimits X(), DI.XMIN, DI.XMAX, JSr, JEr, J1, J2, IERR
              IF IERR <> 0 THEN
                ErrorDisplay 233
                EXIT SUB
              END IF
              HDR$ = "==XYPLOT==> TRUNCATED DATA SET" + CHR$(RET) + HDR$
	   END IF
	END IF

	ClearTopLine
	PRINT "Output Filename >> ";
	GetInput FL$

        IF FL$ = CHR$(RET) OR FL$ = CHR$(ESC) THEN EXIT SUB
	IF INSTR(FL$, ".") = 0 THEN FL$ = FL$ + ".DAT"

        IF FileExists(FL$) THEN
          ClearTopLine
          PRINT "File already exists! Overwrite? "
          ButtonMenu 1, KeyPress$
          IF INSTR(KeyPress$, "Y") = 0 THEN EXIT SUB
        END IF

        FS$ = RTRIM$(DS(NS).FNAME)
        HDR$ = "==XYPLOT==> SOURCE FILE IS " + FS$ + CHR$(RET) + HDR$

	ClearTopLine
        PRINT "Storing Data Set "; NS; " - One Moment Please. . . "

        IF VI.FM = "F" THEN
          NF1$ = FormFSF$ (DS(NS).XMIN, DS(NS).XMAX)   ' floating point format
          NF2$ = FormFSF$ (DS(NS).YMIN, DS(NS).YMAX)
          IF DS(NS).CMPLX THEN
            NF3$ = FormFSF$ (DS(NS).iYMIN, DS(NS).iYMAX)
          END IF
        ELSEIF VI.FM = "I" THEN
          INF = -1
          NF1$ = FormISF$ (DS(NS).XMIN, DS(NS).XMAX)    ' integer format
          NF2$ = FormISF$ (DS(NS).YMIN, DS(NS).YMAX)
          IF DS(NS).CMPLX THEN
            NF3$ = FormISF$ (DS(NS).iYMIN, DS(NS).iYMAX)
          END IF
        ELSE
          NF1$ = "##.######^^^^"                ' exponential format
          NF2$ = NF1$
          NF3$ = NF1$
        END IF

        OutForm$ = NF1$ + VI.DL + " " + NF2$
        IF DS(NS).CMPLX THEN
          OutForm$ = OutForm$ + VI.DL + " " + NF3$
	END IF

	OPEN FL$ FOR OUTPUT AS #2
'
' Write header unless options set to no header.
'
        IF VI.HS THEN
          PRINT #2, "/*"
          L = LEN(HDR$)
          K = 0

          DO
            J = INSTR(K + 1, HDR$, CHR$(RET))
            IF J = 0 THEN
              LN$ = MID$(HDR$, K + 1, L - K)
            ELSE
	      IF J - K = 1 THEN
                LN$ = ""
              ELSE
                M = J - K - 1
                LN$ = MID$(HDR$, K + 1, M)
              END IF
            END IF
            K = J
            PRINT #2, LN$
          LOOP WHILE ((J <> 0) OR (J = L))

          PRINT #2, "*/"
        END IF

	IF DS(NS).CMPLX THEN
	  FOR JX = J1 TO J2
            JYr = JX
            JYi = JX - JSr + JSi
            IF INF THEN
              PRINT #2, USING OutForm$; CLNG(X(JX)); CLNG(Y(JYr)); CLNG(Y(JYi))
            ELSE
              PRINT #2, USING OutForm$; X(JX); Y(JYr); Y(JYi)
            END IF
	  NEXT JX
	ELSE
	  FOR JX = J1 TO J2
            IF INF THEN
              PRINT #2, USING OutForm$; CLNG(X(JX)); CLNG(Y(JX))
            ELSE
              PRINT #2, USING OutForm$; X(JX); Y(JX)
            END IF
	  NEXT JX
	END IF

	CLOSE #2

	IF NOT ITRS THEN
	   DS(NS).HDR = HDR$
	END IF

	DS(NS).FNAME = FL$
	ClearTopLine

END SUB
'
' ---------------------------------------------------------------
'
SUB SaveWork (X(), Y())
'
'  Save entire XYPLOT workspace.
'
	ClearTopLine
        PRINT "Workspace Filename (*.XSP) >> ";
	GetInput FL$
        IF FL$ = CHR$(RET) OR FL$ = CHR$(ESC) THEN EXIT SUB
        IF INSTR(FL$, ".") = 0 THEN FL$ = FL$ + ".XSP"

        IF FileExists(FL$) THEN
          ClearTopLine
          PRINT "Workspace already exists! Overwrite?"
          ButtonMenu 1, KeyPress$
          IF INSTR(KeyPress$, "Y") = 0 THEN EXIT SUB
        END IF

        ClearTopLine
        PRINT "Saving Workspace . . ."

        OPEN FL$ FOR BINARY AS #2
        A$ = SPACE$(64)
        MID$(A$, 1) = "XYPLOT_WORKSPACE_VERSION_"+ "4.00"
        PUT #2, , A$
        PUT #2, , DB
        PUT #2, , DI
        FOR J = 1 TO DB.NSETS
          PUT #2, ,DS(J)
        NEXT J
        FOR J = 1 TO DI.NPLOTS
          PUT #2, ,PV(J)
        NEXT J
        Nbytes = 4 * DB.NVALS
        Nb2 = WriteFile(FILEATTR(2, 2), Nbytes, VARSEG(X(1)), VARPTR(X(1)))
        Nb2 = WriteFile(FILEATTR(2, 2), Nbytes, VARSEG(Y(1)), VARPTR(Y(1)))

        CLOSE #2

        WKSaved = YES

END SUB
'
' ---------------------------------------------------------------
'
SUB ReadWork (X(), Y(), FL$)
'
'  Read an XYPLOT workspace file.
'
        M = DI.CMODE

        OPEN FL$ FOR BINARY AS #1
        A$ = SPACE$(64)
        GET #1, , A$
        J = INSTR (25, A$, "4.")
        IF J = 0 THEN
          CLOSE #1
          ErrorDisplay 236
          EXIT SUB
        END IF
        GET #1, , DB
        GET #1, , DI
        FOR J = 1 TO DB.NSETS
          GET #1, , DS(J)
        NEXT J
        FOR J = 1 TO DI.NPLOTS
          GET #1, , PV(J)
        NEXT J
        Nbytes = 4 * DB.NVALS
        Nb2 = ReadFile(FILEATTR(1, 2), Nbytes, VARSEG(X(1)), VARPTR(X(1)))
        Nb2 = ReadFile(FILEATTR(1, 2), Nbytes, VARSEG(Y(1)), VARPTR(Y(1)))
        CLOSE #1

        DI.CMODE = M
        IF DI.LEVEL <> 1 THEN
          DI.LEVEL = 2
          CORNERS(2).XMIN = DI.XMIN
          CORNERS(2).XMAX = DI.XMAX
          CORNERS(2).YMIN = DI.YMIN
          CORNERS(2).YMAX = DI.YMAX
        END IF
        DI.IX = (SI.IX1 + SI.IX2) / 2
        DI.IY = (SI.IY1 + SI.IY2) / 2

END SUB
'
' ---------------------------------------------------------------
'
SUB SaveWK1 (X(), Y())
'
' Write workspace to a Lotus 1-2-3 worksheet file
'
'
        DIM Wbof AS WksBOF
        DIM Weof AS WksEOF
        DIM Wrange AS WksRange
        DIM Wnhead AS WksNumHeader
        DIM Wnum AS WksNumber

' Initialize worksheet structures

        Wbof.TypeCode = 0
        Wbof.BodyLength = 2
        Wbof.Revision = 1030
        Weof.TypeCode = 1
        Weof.BodyLength = 0
        Wrange.TypeCode = 6
        Wrange.BodyLength = 8
        Wrange.StartCol = 0
        Wrange.StartRow = 0
        Wrange.EndCol = 0
        Wrange.EndRow = 0
        Wnhead.TypeCode = 14
        Wnhead.BodyLength = 13
        Wnhead.Fmt = CHR$(241)
        Wnum.Col = 0
        Wnum.Row = 0
        Wnum.Value = 0#

        ClearTopLine
        PRINT "Worksheet Filename (*.WK1) >> ";
	GetInput FL$
        IF FL$ = CHR$(RET) OR FL$ = CHR$(ESC) THEN EXIT SUB
        IF INSTR(FL$, ".") = 0 THEN FL$ = FL$ + ".WK1"

        IF FileExists(FL$) THEN
          ClearTopLine
          PRINT "Worksheet already exists! Overwrite?"
          ButtonMenu 1, KeyPress$
          IF INSTR(KeyPress$, "Y") = 0 THEN EXIT SUB
        END IF

        ClearTopLine
        PRINT "Saving Worksheet . . ."

' Determine number of rows and columns for worksheet range

        NCOLS = 0
        FOR J = 1 TO DI.NPLOTS
          NS = PV(J).SET
          IF (DS(NS).CMPLX) THEN
            NCOLS = NCOLS + 3
          ELSE
            NCOLS = NCOLS + 2
          END IF
        NEXT J

        NROWS = 0
        FOR J = 1 TO DI.NPLOTS
          NS = PV(J).SET
          IF DS(NS).NPTS > NROWS THEN NROWS = DS(NS).NPTS
        NEXT J

        Wrange.EndRow = NROWS - 1
        Wrange.EndCol = NCOLS - 1

        OPEN FL$ FOR BINARY AS #2
        Fhan% = FILEATTR(2, 2)

' Write BOF code

        Nb2 = WriteFile (Fhan%, 4 + Wbof.BodyLength,_
                VARSEG(Wbof), VARPTR(Wbof))

' Write active worksheet range

        Nb2 = WriteFile (Fhan%, 4 + Wrange.BodyLength,_
                VARSEG(Wrange), VARPTR(Wrange))

' Write each row to worksheet; wnum.fmt = 23 (scientific, 7 places)

        FOR J = 1 TO NROWS
          Wnum.Row = J - 1
          Wnum.Col = 0
          FOR K = 1 TO DI.NPLOTS
            NS = PV(K).SET
            IF J <= DS(NS).NPTS THEN
              I = DS(NS).JSr + J - 1
              Wnum.Value = CDBL(X(I))
              Nb2 = WriteFile (Fhan%, 5, VARSEG(Wnhead), VARPTR(Wnhead))
              Nb2 = WriteFile (Fhan%, 12, VARSEG(Wnum), VARPTR(Wnum))
              Wnum.Col = Wnum.Col + 1
              Wnum.Value = CDBL(Y(I))
              Nb2 = WriteFile (Fhan%, 5, VARSEG(Wnhead), VARPTR(Wnhead))
              Nb2 = WriteFile (Fhan%, 12, VARSEG(Wnum), VARPTR(Wnum))
              Wnum.Col = Wnum.Col + 1
              IF DS(NS).CMPLX THEN
                I = DS(NS).JSi + J - 1
                Wnum.Value = CDBL(Y(I))
                Nb2 = WriteFile (Fhan%, 5, VARSEG(Wnhead), VARPTR(Wnhead))
                Nb2 = WriteFile (Fhan%, 12, VARSEG(Wnum), VARPTR(Wnum))
                Wnum.Col = Wnum.Col + 1
              END IF
            ELSE
              IF DS(NS).CMPLX THEN
                Wnum.Col = Wnum.Col + 3
              ELSE
                Wnum.Col = Wnum.Col + 2
              END IF
            END IF
          NEXT K
        NEXT J

' Write EOF code

        Nb2 = WriteFile (Fhan%, 4 + Weof.BodyLength,_
                VARSEG(Weof), VARPTR(Weof))

        CLOSE #2

END SUB
'
' ---------------------------------------------------------------
'
SUB GetKey (KeyPress$, BS%)
'
'  Get single key press or return word pointed to by mouse
'
        KeyPress$ = ""
        IF DI.CMODE AND TextScreen% THEN
          SetTextCursor
          ShowMouse
          BS% = 0
          WHILE ((BS% = 0) AND (KeyPress$ = ""))
            GetMousePosition IX%, IY%, BS%
            KeyPress$ = INKEY$
          WEND

          IF BS% <> 0 THEN
            ' button has been pressed; wait for release
            ButtonRelease
            ' get word on screen pointed to by mouse
            ConvertToRC IX%, IY%
            JROW = IY%
            JCOL = IX%
            GetWord JROW, JCOL, KeyPress$
          END IF
          HideMouse
        ELSE
          KeyWait KeyPress$
        END IF
END SUB
'
' ---------------------------------------------------------------
'
SUB GetInput (REQ$)
REM
REM  Get keyboard input or current word pointed to by mouse
REM
	REQ$ = ""
	PRINT "_";

' Wait for key to be pressed or mouse button to be depressed.

        GetKey A$, BS%
        REQ$ = A$
        IF BS% <> 0 THEN EXIT SUB
        IF A$ = CHR$(ESC) OR A$ = CHR$(RET) THEN EXIT SUB
        IF LEN(A$) = 2 THEN EXIT SUB
        LOCATE , MAX(1, POS(0) - 1): PRINT A$;

        DO
          PRINT "_";
          KeyWait A$
          IF A$ = CHR$(ESC) OR LEN(A$) = 2 THEN REQ$ = A$: EXIT SUB
          IF A$ = CHR$(8) THEN
            LOCATE , MAX(1, POS(0) - 2)
            PRINT "  ";
            LOCATE , MAX(1, POS(0) - 2)
            IF REQ$ <> "" THEN
              REQ$ = MID$(REQ$, 1, LEN(REQ$) - 1)
            END IF
          ELSE
            IF A$ <> CHR$(RET) THEN 
              REQ$ = REQ$ + A$
              LOCATE , POS(0) - 1
              PRINT A$;
            END IF
          END IF
        LOOP WHILE A$ <> CHR$(RET)

END SUB
'
' ---------------------------------------------------------------
'
SUB DisplayFile (FL$)
'
' Display contents of an ASCII file.
'

'  Make sure file exists

        IF NOT FileExists (FL$) THEN EXIT SUB

        VIEW PRINT 4 TO 25
        COLOR CI.FWF, CI.FWB
        CLS
        COLOR CI.FWA
        LOCATE 4, 1
        PRINT STRING$(80, 205)
        LOCATE 23, 1
        PRINT STRING$(80, 205)
        LOCATE 4, 70
        PRINT "PgUp";
        LOCATE 23, 70
        PRINT "PgDn";
        FL$ = UCASE$(FL$)
        IF INSTR(FL$, ".") = 0 THEN FL$ = FL$ + ".DAT"
        COLOR CI.FWA, 0
        LOCATE 4, (80 - LEN(FL$)) / 2
        PRINT FL$
        IF DI.CMODE THEN
          CANCbutn.NROWS = 2: CANCbutn.TopRow = 23: CANCbutn.LeftCol = 36
          ShowButton CANCbutn
        ELSE
          LOCATE 24, 1
          PRINT TAB(31); "<<Press ESC to exit>>";
        END IF
        COLOR CI.FWF, CI.FWB
        VIEW PRINT 5 TO 22
        ViewFile FL$
        VIEW PRINT 4 TO 25
        COLOR CI.FWF, 0
        CLS
        VIEW PRINT 1 TO 25

END SUB
'
' ---------------------------------------------------------------
'
SUB ViewFile (F$)
'
'  Display contents of file F$, one screen at a time
'
        DIM IPAGE(30) AS LONG
        OPEN F$ FOR INPUT AS #1
	JPAGE = 1

	DO WHILE NOT EOF(1)
	  IPAGE(JPAGE) = SEEK(1)
	  J = 1

          DO WHILE ((J < 18) AND NOT EOF(1))
	    LINE INPUT #1, A$
            PRINT A$
	    J = J + 1
	  LOOP
          IF NOT EOF(1) THEN
            LINE INPUT #1, A$
            PRINT A$;
          END IF

	  DO
            GetKey KeyPress$, BS%
            SpecialKey KeyPress$

            SELECT CASE KeyPress$
              CASE "PgUp"
		IF JPAGE > 1 THEN
		  JPAGE = JPAGE - 1
		END IF
		SEEK #1, IPAGE(JPAGE)
                CLS 2
              CASE "PgDn"
		IF EOF(1) THEN
                  CLS 2
		  SEEK #1, IPAGE(JPAGE)
		ELSE
		  JPAGE = JPAGE + 1
                  PRINT
		END IF
	      CASE "<HOME>"
		JPAGE = 1
		SEEK #1, IPAGE(JPAGE)
                CLS 2
              CASE CHR$(27), "CANCEL"
		CLOSE #1
		EXIT SUB
	      CASE ELSE
                KeyPress$ = ""
	    END SELECT
          LOOP WHILE KeyPress$ = ""
	LOOP

	CLOSE #1

END SUB
