\ arithmetic.4th
\
\ xyplot module for dataset arithmetic functions
\
\ (c) 1999--2000 Krishna Myneni
\
\ This software is released under the terms of the GNU
\ General Public License
\ 
\ Last Revised: 1-18-2000
\
\ Requires:
\
\	matrix.4th
\
\
create ds1 8 cells allot
create ds2 8 cells allot
create ds3 8 cells allot

create result 2 8000 * dfloats cell+ cell+ allot
2 8000 result mat_size!

fvariable fxmin
fvariable fxmax
fvariable fymin
fvariable fymax
fvariable fx1
fvariable fx2
fvariable fdel1



: ds_extrema ( addr1 -- | determine the dataset extrema )
	\ results are stored in the variables fxmin, fxmax, fymin, fymax
	dup 0 swap @xy fdup fymax f! fymin f! fdup fxmax f! fxmin f!
	dup @npts 1 do
	  i over @xy
	  fdup fymin f@ fmin fymin f!
	  fymax f@ fmax fymax f!
	  fdup fxmin f@ fmin fxmin f!
	  fxmax f@ fmax fxmax f!
	loop
	drop ;

: overlap ( addr1 addr2 -- flag | determine the overlap of two datasets )
	\ store the overlap region in fx1 and fx2
	\ flag is true if the regions overlap
	ds_extrema fxmin f@ rot fxmax f@ rot
	ds_extrema fxmax f@ fmin fx2 f!
	fxmin f@ fmax fx1 f!
	fx1 f@ fx2 f@ f<= ;


variable rcntr
variable dstemp
variable npts
variable jtemp
variable ordering


: findx ( fx addr -- n | return index of point with closest x to fx )
	\ addr is the address of a dataset info structure
	dup dstemp ! @npts npts !	
	0 dstemp a@ @xy fdrop
	npts @ 1- dstemp a@ @xy fdrop
	f< ordering !
	npts @ 0
	begin
	  2dup - 1 >
	while
	  2dup + 2/ -rot
	  >r >r 
	  dup dstemp a@ @xy fdrop rot 
	  >r fover f<= ordering @ =
	  r> swap r> r> rot
	  if drop swap else nip then 
	repeat
	nip dup jtemp !
	dstemp a@ @xy fdrop fover f- fabs fdel1 f!
	jtemp @ npts @ 1- < if
	  jtemp @ 1+ dstemp a@ @xy fdrop f- fabs
	  fdel1 f@ f< if 1 jtemp +! then
	else
	  fdrop
	then
	jtemp @ ;  

:

: index_range ( -- n1 n2 | return starting and ending indices of overlap region in active set )
	?active
	dup 0 >= if
	  ds1 get_ds drop
	  ?operand
	  dup 0 >= if
	    ds2 get_ds drop
	    ds1 ds2 overlap		\ determine if sets overlap
	    if
	      fx1 f@ ds1 findx		\ find starting and ending indices of
	      fx2 f@ ds1 findx		\ overlap region in active dataset
	      1+
	      2dup > if swap then	\ swap if descending order
	      2dup = if 1+ then		\ adjust to prevent infinite loop
	      exit
	    then
	  then
	  drop
	then
	drop
	0 0 ;
	

: make_result ( ^name  -- | make dataset resulting from an arithmetic operation )
	
	\ Set up the info structure for a new dataset

	1+ ds3 DNAME + !
	"  " 1+ ds3 DHEADER + !
	256 ds3 DTYPE + !
	result mat_size@ drop ds3 DNPTS + !
	2 ds3 DSIZE + !
	result cell+ cell+ ds3 DDATA + !
		 	      
	ds3 make_ds ;

variable ar_operator 

: do_arithmetic ( n1 n2 addr -- flag | loop over index range n1 to n2 and apply operator )
	ar_operator !			\ store execution address of operator
	1 rcntr !
	index_range
	2dup <>
	if
	  swap
	  2dup - 2 result mat_size!	\ set size of result matrix	
	  do
	    i ds1 @xy
	    fover ds2 findx		\ find nearest point in ds2
	    ds2 @xy
	    fswap fdrop
	    ar_operator a@ execute
	    2 rcntr @ result frow!
	    1 rcntr +!
	  loop
	  true
	else
	  2drop
	  false
	then ;

 
: add_ds ( -- | add the operand set and the active set )
	['] f+ do_arithmetic
	if " Sum" make_result then ;

   
: sub_ds ( -- | add the operand set and the active set )
	['] f- do_arithmetic
	if " Difference" make_result then ;
   	   
     	        
: mul_ds ( -- | add the operand set and the active set )
	['] f* do_arithmetic
	if " Product" make_result then ;

   
: div_ds ( -- | add the operand set and the active set )
	['] f/ do_arithmetic
	if " Quotient" make_result then ;
   

\ add xyplot math menu items

MN_MATH " Add"		" add_ds" add_menu_item
MN_MATH " Subtract"	" sub_ds" add_menu_item
MN_MATH " Multiply"	" mul_ds" add_menu_item
MN_MATH " Divide"	" div_ds" add_menu_item
