# barnsley.R
#
# Barnsley's Fern, an iterated function system which creates
# a fractal fern leaf.
#
# See http://en.wikipedia.org/wiki/Barnsley_fern
#
# R program by K. Myneni, 18 March 2014
#

## The transformation matrices:

f1_A = matrix(c(0,0,0,0.16), nrow=2, ncol=2, byrow=TRUE)
f1_B = c(0, 0)

f2_A = matrix(c(0.85,0.04,-0.04,0.85), nrow=2, ncol=2, byrow=TRUE)
f2_B = c(0,1.6)

f3_A = matrix(c(0.2,-0.26,0.23,0.22), nrow=2, ncol=2, byrow=TRUE)
f3_B = c(0,1.6)

f4_A = matrix(c(-0.15,0.28,0.26,0.24), nrow=2, ncol=2, byrow=TRUE)
f4_B = c(0,0.44)


## The probabilities for each transformation
prob = c(0.01, 0.85, 0.07, 0.07)


## Generate N iterations of the function system
N = 50000

## The sequence of transformations, randomly drawn using the
## probabilities given above.
tx_s = sample(1:4, N, replace=TRUE, prob)

## The starting point
x0 = c(0, 0)

xs = rep(0, N)
ys = xs

xn = x0
for (i in 1:N) {
	xs[i] = xn[1]
	ys[i] = xn[2]
	xn = switch( tx_s[i], 
		f1_A %*% xn + f1_B,
		f2_A %*% xn + f2_B,
		f3_A %*% xn + f3_B,
		f4_A %*% xn + f4_B )
}

plot(xs, ys, type="p", cex=0.2, col="forestgreen")


	
