# odesolve-example.R
#
# Illustrate the use of the extra R package, deSolve, to solve a
# system of two linear, ordinary differential equations:
#
#  	du/dt = -u + alpha*v
#	
#	dv/dt = beta*u - v
#

derivs <- function(t, x, p) {
	udot = -x[1] + p[1]*x[2]
	vdot = p[2]*x[1] - x[2]	
	return (list(c(udot, vdot)))
}

params <- c( 2, 0.1 )              # alpha = 2, beta = 0.1
x0 <- c(0, 0.05)                   # initial conditions
times <- c(0, 0.1*(1:100))        # time steps at which to compute the solution

require("deSolve")

# Solve the differential equations for this example
out <- lsoda( x0, times, derivs, params)

# Plot the solutions, u(t) and v(t)
plot(out[,1], out[,2], type='l', col="blue", xlab="Time (s)", ylab="u(t), v(t)", ylim=c(0, .05))
lines(out[,1], out[,3], col="red")

