\ ceva4.fs: Solve quadratic equation
\ a*x^2 + b*x + c = 0
\ Handle b=c=0 case
\ DO NOT HANDLE: a=0 case
\ uses following artifice:
\ a*x^2 - 2*(-b/2)*x + c = 0 ;  a*x^2 - 2*'b*x + c = 0
\
\ Posted to comp.lang.forth on 6 September 2009 by Humpty Dumpty
\ Test code added by KM
\
cr .( CEVA4             v1.0       6 September  2009   HD )
fvariable cc
\ ****************************************
\             d <--> 'b^2 - a*c
\ ****************************************
: discrim(cc@)                  \ F: a 'b -- 'b^2-ac
                fdup f* fswap cc f@ f* f-
;
\ [ 'b + sgn('b)*sqrt(d) ] / a   <--> big
\ ****************************************
\                                  d >= 0
\ ****************************************
: real-quad(cc!@)               \ F: a 'b d -- little big
                fsqrt
                fover f0< IF fnegate THEN
                f+                                      \ F: a  big*a
                fdup f0= IF fnip 0e0 exit THEN          \ b=c=0 case
                fdup                            \ F: a  big*a  big*a
                cc f@ fswap f/          \ F: a  big*a  little
                cc f!                           \ F: a  big*a   ; cc as a temporary
                fswap f/ cc f@          \ F: big  little
                fswap
;
\ ****************************************
\                                   d < 0
\ ****************************************
: complex-quad(cc!@)    \ F: a 'b d -- re +im re -im
                fabs fsqrt
                frot ftuck                              \ F: 'b a  sqrt|d| a
                f/ fabs                                 \ imaginary part positive!
                cc f!                                   \ ; cc as a temporary
                f/
                cc f@
                fover fover fnegate
;
\ ****************************************
\               solve-quad
\ ****************************************
: solve-quad            \ F: a b c -- re1 im1  re2 im2
                cc f!
                2e0 f/ fnegate                                  \ F: a 'b
                fover fover discrim(cc@)                \ F: a 'b d
                fdup f0<
                IF        complex-quad(cc!@)
                ELSE  real-quad(cc!@) 0e0 ftuck THEN
; 

: solve_quadratic solve-quad ;

TEST-CODE? [IF]   \ test code ==============================================
[undefined] T{      [IF]  include ttester.fs  [THEN]    
BASE @ DECIMAL

1e-15  rel-near F!
1e-256 abs-near F!
set-near

\ Examples from:
\
\    http://www.purplemath.com/modules/quadform.htm

-2e 3e F/       fconstant -2/3
-3e 2e F/       fconstant -3/2
 5e fsqrt       fconstant sqrt{5}
 2e fsqrt 3e F/ fconstant sqrt{2}/3
 3e fsqrt 2e F/ fconstant sqrt{3}/2
10e fsqrt 2e F/ fconstant sqrt{10}/2

: zz}t  rrrr}t ;


CR
TESTING SOLVE_QUADRATIC
t{ 1e  0e  0e solve_quadratic ->  0e               0e    0e               0e   zz}t
t{ 1e  3e -4e solve_quadratic ->  1e               0e   -4e               0e   zz}t
t{ 2e -4e -3e solve_quadratic ->  1e sqrt{10}/2 F- 0e    1e sqrt{10}/2 F+ 0e   zz}t
t{ 1e -2e -4e solve_quadratic ->  1e sqrt{5} F-    0e    1e sqrt{5} F+    0e   zz}t
t{ 9e 12e  4e solve_quadratic ->  -2/3             0e   -2/3              0e   zz}t
t{ 3e  4e  2e solve_quadratic ->  -2/3      sqrt{2}/3   -2/3 sqrt{2}/3 fnegate zz}t
t{ 1e  3e  3e solve_quadratic ->  -3/2      sqrt{3}/2   -3/2 sqrt{3}/2 fnegate zz}t
t{ 1e  0e  1e solve_quadratic ->  0e               1e    0e              -1e   zz}t

\ Test case which loses accuracy with ordinary quadratic formula:
\ 
\        x^2 + x + c = 0
\
\ when c << 1, the approximate solution is  x1 = -c,  x2 = -1 + c
\
t{ 1e 1e 1e-17 solve_quadratic ->     -1e-17  0e    -1e 1e-17 f+           0e    zz}t


BASE !
[THEN]

