\ zmatrix.fs
\
\ Complex matrix words: for Forth systems which use a separate fp stack.
\
\ Copyright (c) 2002 Krishna Myneni, April 5, 2002
\ Provided under the GNU General Public License
\
\ Requires:
\	matrix.fs
\	complex.fs
\
\ Revisions:
\
\	2002-12-26  kForth code was adapted to Forths which use
\		      a separate fp stack.  KM
\

: zmatrix ( nrows ncols -- | allot space for complex matrix and initialize)
	create 2dup * 2* floats 2 cells + ?allot mat_size! ;

: zmat_addr ( i j a -- a2 | returns address of the i j element of a )
	>r 1- 2* floats swap 1- r@ @ * 2* floats + r> + 
	[ 2 cells ] literal + ;

: zmat@ ( i j a --  | returns the i j element of a )
	( F: -- z )
	zmat_addr z@ ;

: zmat! ( i j a -- | store z as the i j element of a )
	( F: z -- )
	zmat_addr z! ; 

: zmat_zero ( a -- | zero all entries in complex matrix )
	dup mat_size@ * >r 1 1 rot zmat_addr r>
	0 ?do dup z=0 z! [ 2 floats ] literal + loop drop ; 

: zrow@ ( i a -- n | fetch row i of zmatrix a as an zrc )
	( F: -- z1 ... zn )
	dup @ >r 1 swap zmat_addr r> swap over
	0 ?do dup z@ float+ float+ loop drop ;

: zcol@ ( j a -- n | fetch column j of zmatrix a )
	( F: -- z1 ... zn )
	dup mat_size@ 2* floats 2>r 1 -rot zmat_addr 2r>
	rot 2 pick 0 ?do dup z@ over + loop 2drop ;

: zrow! ( n i a -- | store zrc as row i of zmatrix a )
	( F: z1 ... zn -- )
	dup @ dup >r swap zmat_addr swap r> min
	0 ?do dup z! [ 2 floats ] literal - loop drop ;	

: zcol! ( n j a -- | store zrc as column j of zmatrix a )
	( F: z1 ... zn -- )
	rot over mat_size@ >r min r> 2* floats >r dup >r -rot zmat_addr 
	r> r> -rot 0 ?do dup z! over - loop 2drop ; 

: zrow_swap ( i j a -- | interchange rows i and j for zmatrix a )
	tuck 2dup 2>r 2over 2>r
	2>r zrow@ 2r> zrow@
	2r> zrow! 2r> zrow! ;

: zcol_swap ( i j a -- | interchange columns i and j for a )
	tuck 2dup 2>r 2over 2>r
	2>r zcol@ 2r> zcol@
	2r> zcol! 2r> zcol! ; 
	
: zmat. ( a -- | print out the complex matrix )
	dup mat_size@ 1+
	swap 1+
	1 do
	  dup
	  1 do
	    over j i rot zmat@ z. 9 emit 
	  loop
	  cr
	loop
	2drop
;


