\ hp-scope.4th
\
\  Provides class HpScope, which is a base class for 
\  HP/Agilent scope communications across GPIB
\
\ Requires:
\   modules
\   mini-oof
\   gpib
\   gpib-device
\
\ Revisions:
\   2011-11-24  km  convert hpcommon module to class module
\   2012-02-16  km  revised for modules v0.5.0

Module: hp-scope

ALSO gpib-device

Begin-Module

GpibDevice class
Public:
	method  Reset              ( o -- )
	method  AutoScale          ( o -- )
	method  ClearDisplay       ( o -- )
	method  SetNumAverages     ( u o -- )
	method  GetRecordLength    ( o -- u )
	method  GetTimePosition    ( o -- r )
	method  GetTimeRange       ( o -- r )
	method  GetTimeIncrement   ( o -- r )
	method  GetChannelOffset   ( u o -- r )
	method  GetChannelScale    ( u o -- r )
	method  GetXInc            ( o -- r )
	method  GetXOrigin         ( o -- r )
	method  GetYInc            ( o -- r )
	method  GetYOrigin         ( o -- r )

	\ virtual methods
	method  SetRecordLength    ( u o -- )
	method  SetAveraging       ( flag o -- )
	method  DownloadChannel    ( u o -- )

end-class HpScope

\ Helper words

: >f ( -- r | convert gpib buffer contents to float )
    	in_buf 19 10 scan	\ scan for end of line char
    	drop in_buf - 		\ number of chars in line
    	in_buf swap strpck string>f ;

\ Send command to object, read response, and convert to fp number
: QueryF ( o ^str -- r )  over Send  19 swap Read >f ;

Private:

: noname ( o -- ) c" *RST" swap Send ;
' noname HpScope defines Reset

: noname ( o -- ) c" :AUT" swap Send ;
' noname HpScope defines AutoScale

: noname ( o -- ) c" :CDIS" swap Send ;
' noname HpScope defines ClearDisplay

: noname ( u o -- )            
    	>r s>string count s" :ACQ:COUN "  
	2swap strcat strpck r> Send ;
' noname HpScope defines SetNumAverages

: noname ( o -- u | get number of points )         
    	c" :WAV:POIN?" QueryF fround>s ; 
' noname HpScope defines GetRecordLength

: noname ( o -- r | return x offset )  c" :TIM:POS?"  QueryF ;    
' noname HpScope defines GetTimePosition

: noname ( o -- r | return full scale time in sec ) 
	c" :TIM:RANGE?" QueryF ;
' noname HpScope defines GetTimeRange

: noname ( o -- r | delta t between points in sec )   
	>r 
	r@ GetTimeRange
	r> GetRecordLength  s>f f/ ;
' noname HpScope defines GetTimeIncrement

: noname ( u o -- r | return ch u vertical offset in volts )
	s>string s" CHAN" rot count strcat 
	s" :OFFS?" strcat strpck  QueryF ;
' noname HpScope defines GetChannelOffset

: noname ( u o -- r | return ch u vertical scale in volts/div )
	s>string count s" CHAN" 2swap strcat 
	s" :SCALE?" strcat strpck  QueryF ;
' noname HpScope defines GetChannelScale

: noname ( o -- r | return the time between points in sec )
	c" :WAV:XINC?" QueryF ;
' noname HpScope defines GetXInc

: noname ( o -- r | return x value in sec for first point )
	in_buf 14 32 fill  c" :WAV:XOR?" QueryF ;
' noname HpScope defines GetXOrigin

: noname ( o -- r | return the volts/unit scale factor for the waveform )
	c" :WAV:YINC?" QueryF ;
' noname HpScope defines GetYInc

: noname ( o -- r | return vertical y value for data count of zero )
	in_buf 14 32 fill  c" :WAV:YOR?" QueryF ;
' noname HpScope defines GetYOrigin

End-Module

