\ ==============================================================================
\
\                  config - the config in the ffl
\
\             Copyright (C) 2005-2007  Dick van Oudheusden
\  
\ This library is free software; you can redistribute it and/or
\ modify it under the terms of the GNU General Public
\ License as published by the Free Software Foundation; either
\ version 2 of the License, or (at your option) any later version.
\
\ This library is distributed in the hope that it will be useful,
\ but WITHOUT ANY WARRANTY; without even the implied warranty of
\ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\ General Public License for more details.
\
\ You should have received a copy of the GNU General Public
\ License along with this library; if not, write to the Free
\ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
\
\ ==============================================================================
\ 
\  $Date: 2009-05-23 05:37:24 $ $Revision: 1.20 $
\
\ ==============================================================================
\
\ This file is for kforth.
\
\ ==============================================================================


s" ffl.version" forth-wordlist search-wordlist 0= [IF]


( config = Forth system specific words )
( The config module contains the extension and missing words for a forth system.)


000800 constant ffl.version

\ kforth defs
false   constant nil
1 cells constant cell
: sgn ( n -- sgn[n] ) dup 0< swap 0> if drop 1 then ;
: 0>= ( n -- flag ) 0< invert ;
: 0<= ( n -- flag ) 0> invert ;
: u<= ( u -- flag ) u> invert ;
: bounds  ( str len -- str+len str )  OVER + SWAP ;
: rdrop ( r: a -- ) postpone r> postpone drop ; immediate 
[undefined] ptr [IF] : ptr create 1 cells ?allot ! does> a@ ; [THEN]

( Private words )
  
variable ffl.endian   1 ffl.endian !


( System Settings )

create end-of-line    ( -- c-addr = Counted string for the end of line for the current system )
2 ?allot  1 over c! 10 swap 1+ c!  
\   newline string,    \ All hosts except dos  (gforth 0.6.2)
\ 2 c, 13 c, 10 c,   \ dos:  cr lf


\ s" ADDRESS-UNIT-BITS" environment? 0= [IF] 8 [THEN] 
8  constant #bits/byte   ( -- n = Number of bits in a byte )
  
#bits/byte 1 chars *
  constant #bits/char   ( -- n = Number of bits in a char )
  
#bits/byte 1 cells *
  constant #bits/cell   ( -- n = Number of bits in a cell )  

ffl.endian c@ 0=             
  constant bigendian?   ( -- flag = Check for bigendian hardware )


( Extension words )

0 [IF]
: ms@  ( -- u = Fetch milliseconds timer )
  utime 1 1000 m*/ drop ;
[THEN]

( s" MAX-U" environment? drop) -1 constant max-ms@   ( -- u = Maximum value of the milliseconds timer )


\ No Command line arguments in kforth
0 [IF]
argc @ 1- constant #args  ( -- n = Get the number of command line arguments )

0 argc !            \ tell gforth not to process any more arguments

: arg@  ( n -- c-addr u = Get the nth command line argument )
  dup #args u< IF
    1+ cells argv @ + @ cstring>sstring
  ELSE
    drop 0 0 
  THEN
;
[THEN]


: lroll   ( u1 u2 -- u3 = Rotate u1 u2 bits to the left )
  2dup lshift >r
  #bits/cell swap - rshift r>
  or
;


: rroll   ( u1 u2 -- u3 = Rotate u1 u2 bits to the right )
  2dup rshift >r
  #bits/cell swap - lshift r>
  or
;


: 0!   ( a-addr -- = Set address to zero )
  0 swap !
;


: nil!   ( a-addr -- = Set address to nil )
  nil swap !
;


: nil=   ( addr -- flag = Check for nil )
  nil =
;


: nil<>   ( addr -- flag = Check for unequal to nil )
  nil <>
;


0 nil= [IF]
: nil<>?    ( addr -- false | addr true = If addr is nil, then return false, else return address with true )
  state @ IF
    postpone ?dup
  ELSE
    ?dup
  THEN
; immediate nondeferred
[ELSE]
: nil<>?
  dup nil<> IF
    true
  ELSE
    drop
    false
  THEN
;
[THEN]  


: ?free   ( addr -- ior = Free the address if not nil )
  dup nil<> IF
    free
  ELSE
    drop 0
  THEN
;


: 1+!   ( a-addr -- = Increase contents of address by 1 )
  1 swap +!
;


: 1-!   ( a-addr -- = Decrease contents of address by 1 )
  -1 swap +!
;


: @!   ( x1 a-addr -- x2 = First fetch the contents x2 and then store value x1 )
  dup a@ -rot !
;

: toupper ( char -- CHAR ) 95 and ;

: icompare   ( c-addr1 u1 c-addr2 u2 -- n = Compare case-insensitive two strings and return the result [-1,0,1] )
  rot swap 2swap 2over
  min 0 ?DO
    over c@ toupper over c@ toupper - sgn ?dup IF
      >r 2drop 2drop r>
      unloop 
      exit
    THEN
    1 chars + swap 1 chars + swap
  LOOP
  2drop
  - sgn
;


: <=>   ( n1 n2 -- n = Compare the two numbers and return the compare result [-1,0,1] )
  2dup = IF 
    2drop 0 EXIT 
  THEN
  < 2* 1+
;

      
: index2offset   ( n1 n2 -- n3 = Convert the index n1 range [-n2..n2> into offset n3 range [0..n2>, negative values of n1 downward length n2 )
  over 0< IF
    +
  ELSE
    drop
  THEN
;


: r'@              ( R: x1 x2 -- x1 x2; -- x1 = Fetch the second cell on the return stack )
  postpone 2r@ postpone drop
; immediate


[DEFINED] sw@ [IF]
' sw@ alias <w@   ( w-addr -- n = Fetch a word, 16 bit, sign extend )
[THEN]

[DEFINED] sl@ [IF]
' sl@ alias <l@   ( l-addr -- n = Fetch a long word, 32 bit, sign extend )
[THEN]


[DEFINED] float [IF]

( Float extension constants )

0E+0 fconstant 0e+0  ( F: -- r = Float constant 0.0 )
1E+0 fconstant 1e+0  ( F: -- r = Float constant 1.0 )
2E+0 fconstant 2e+0  ( F: -- r = Float constant 2.0 )


( Float extension words )

: f-rot            ( F: r1 r2 r3 -- r3 r1 r2 = Rotate counter clockwise three floats )
  frot frot
;


[UNDEFINED] f2dup [IF]
: f2dup            ( F: r1 r2 -- r1 r2 r1 r2 = Duplicate two floats )
  fover fover
;
[THEN]

: f>r              ( F: r -- ; R: -- r = Push float on the return stack )
  2>r
;

: fr>              ( F: -- r ; R: r -- = Pop float from the return stack )
  2r>
;

: fr@              ( F: -- r ; R: r -- r = Get float from top of return stack )
  2r@
;

[THEN]


( Exceptions )

0 [IF]
s" Index out of range" exception constant exp-index-out-of-range ( -- n = Index out of range exception number )
s" Invalid state"      exception constant exp-invalid-state      ( -- n = Invalid state exception number )
s" No data available"  exception constant exp-no-data            ( -- n = No data available exception number )
s" Invalid parameters" exception constant exp-invalid-parameters ( -- n = Invalid parameters on stack )
s" Wrong file type"    exception constant exp-wrong-file-type    ( -- n = Wrong file type )
s" Wrong file version" exception constant exp-wrong-file-version ( -- n = Wrong file version )
s" Wrong file data"    exception constant exp-wrong-file-data    ( -- n = Wrong file data )
s" Wrong checksum"     exception constant exp-wrong-checksum     ( -- n = Wrong checksum )
s" Wrong length"       exception constant exp-wrong-length       ( -- n = Wrong length )
s" Invalid data"       exception constant exp-invalid-data       ( -- n = Invalid data exception number )
[ELSE]

2256 constant exp-index-out-of-range
2257 constant exp-invalid-state
2258 constant exp-no-data
2259 constant exp-invalid-parameters
2260 constant exp-wrong-file-type
2261 constant exp-wrong-file-version
2262 constant exp-wrong-file-data
2263 constant exp-wrong-checksum
2264 constant exp-wrong-length
2265 constant exp-wrong-length
2266 constant exp-invalid-data

[THEN]


[ELSE]
  drop
[THEN]

\ ==============================================================================

