\ folding.4th
\
\ Iteratively replace binary operations between constants 
\ with the result of the calculation in a word definition
\
\ K. Myneni. 2010-07-12; rev. 2010-07-13, 2010-07-14
\
\ Notes:
\
\ 1. This code is NOT portable --- it is specific to kForth.
\ 2. Folding cannot be applied to words with branching.
\ 3. Recursion is not used. The word FOLD is not re-entrant.

include dump
: ctable ( ... n <name> -- | create a table of byte values)
    dup >r create ?allot dup r> + 1-
    ?do	 i c! -1 +loop ;

1 cells constant WSIZE
WSIZE 1+ constant OFFS_CD

65 constant OP_ADDR
68 constant OP_DEFINITION
70 constant OP_FVAL
71 constant OP_CALLADDR
73 constant OP_IVAL

238 constant OP_RET

16 constant N_BIN_OPS

\ Binary operators which result in single value
 38  \ AND
 94  \ XOR
124  \ OR
164  \ MIN
165  \ MAX
 60  \ LT
 61  \ EQ
 62  \ GT
241  \ GE
242  \ LE
243  \ NE
 37  \ MOD
 42  \ MUL
 43  \ ADD
 45  \ SUB
 47  \ DIV
N_BIN_OPS ctable binary-ops

: op-has-arg? ( u -- flag )
    >r
    r@ OP_ADDR =     r@ OP_DEFINITION =  or 
    r@ OP_FVAL =  or r@ OP_CALLADDR   =  or
    r> OP_IVAL =  or
;

: binary-op? ( u -- flag )
    N_BIN_OPS 0 DO 
      binary-ops I + c@ 
      over = IF drop true UNLOOP EXIT THEN 
    LOOP
    drop false
;

\ advance pointer in byte code to next op
: offset-to-nextop ( a1 -- a2 )
    dup c@ op-has-arg? IF OFFS_CD + ELSE 1+ THEN ;

\ Move the Forth byte code from a2 to end, up to a1
: move-up ( a1 a2 -- )
    BEGIN  
      dup c@ OP_RET <>
    WHILE
      dup c@ op-has-arg? IF OFFS_CD ELSE 1 THEN >r
      2dup swap r@ cmove
      r@ + swap r> + swap
    REPEAT
    drop OP_RET swap c! ;

create arithmetic-code-buf 16 allot
OP_RET arithmetic-code-buf 11 + c!

variable adjacent_vals

: fold-binary ( a -- a' )
    >r 
    r@ 10 - arithmetic-code-buf 11 cmove
    arithmetic-code-buf execute 
    r@ 9 - ! 
    r@ OFFS_CD - dup r> 1+ move-up
;

\ Replace constant arithmetic with computed values
: fold ( xt -- )
    0 adjacent_vals !
    BEGIN
      dup c@ OP_RET <>
    WHILE
      dup c@ >r
      r@ OP_IVAL = IF 
	offset-to-nextop 
	1 adjacent_vals +!
      ELSE
        r@ binary-op?  IF
	  adjacent_vals @ 2 >= 
	  IF 
	    fold-binary 
	    -1 adjacent_vals +!
	  ELSE 
	    offset-to-nextop 
	    0 adjacent_vals !
	  THEN
        ELSE
          offset-to-nextop 
	  0 adjacent_vals !
        THEN
      THEN
      r> drop
    REPEAT
    drop ;

\ Examples:

: add-ten 2 3 * 4 + + ;

cr .( Byte EE marks RETURN ) cr
cr .( Original byte code for add-ten : )
' add-ten 20 dump

' add-ten fold
cr .( Folded byte code for add-ten : )
' add-ten 8 dump

: example2  2 3 * 4 + 8 max * ;

' example2 fold
cr .( Folded byte code for example2 : )
' example2 8 dump

: example3  2 3 * 4 + 8 min * ;

' example3 fold
cr .( Folded byte code for example3 : )
' example3 8 dump

: example4 5 3 + 8 4 * * ;

' example4 fold
cr .( Folded byte code for example4 : )
' example4 8 dump

\ illustrate folding with symbolic constants
: example5 5 3 + 8 WSIZE * * ;

' example5 fold
cr .( Folded byte code for example5 : )
' example5 8 dump


