\ zeigen22.4th
\
\ Compute the eigenvalues and eigenvectors of a general, 
\ complex 2x2 matrix.
\
\ Requires:
\   fsl-util  (2.0 or later)
\   complex   (1.0.3 or later)
\   lagroots  (1.0.2 or later)
\
\ Revisions:
\   2012-02-24  km; use KM/DNW's modules library.
\
\ Copyright (c) 2010--2012 Krishna Myneni
\
\ Permission is granted to use this code for any purpose, provided
\ the above copyright notice is retained.
\
BASE @
DECIMAL

Begin-Module

Private:

3 COMPLEX ARRAY coeffs{
2 COMPLEX ARRAY roots{

Public:

\ Change eps for  accuracy eigenvalues
fvariable zeig_eps
1e-15 zeig_eps f!

\ Return the complex eigenvalues of a real 2x2 matrix
: }}zeigenvalues22 ( 'mat -- z1 z2 )
    >r
    1e 0e  coeffs{ 2 } z!
    r@ 0 0 }} z@  r@ 1 1 }} z@  z+ znegate  coeffs{ 1 } z!
    r@ 0 0 }} z@  r@ 1 1 }} z@  z* 
    r@ 0 1 }} z@  r> 1 0 }} z@  z* z- coeffs{ 0 } z!
    1e 0e zeig_eps f@ coeffs{ roots{ 2 zroots
    roots{ 0 } z@  roots{ 1 } z@
;

Private:

2 2 COMPLEX MATRIX ev{{  \ eigenvector matrix
fvariable norm

: }}zcopy22 ( 'a 'b -- ) 4 COMPLEXES move ;

Public:

\ Compute and place the two normalized eigenvectors in the 
\ columns of ev{{ ;  use only after finding eigenvalues
: }}zeigenvectors22 ( 'mat  -- ) 
	>r
	\ Compute eigenvector for lambda1
	\ r@ 0 1 }} z@  r@ 0 0 }} z@ roots{ 0 } z@ z- z/ znegate
	\ ev{{ 0 0 }} z! 	1e 0e ev{{ 1 0 }} z!
	1e 0e ev{{ 0 0 }} z!
        roots{ 0 } z@  r@ 0 0 }} z@ z-  r@ 0 1 }} z@ z/ ev{{ 1 0 }} z!
	
	\ Compute eigenvector for lambda2
	\ r@ 0 1 }} z@  r> 0 0 }} z@ roots{ 1 } z@ z- z/ znegate
	\ ev{{ 0 1 }} z!  1e 0e ev{{ 1 1 }} z!
        1e 0e ev{{ 0 1 }} z!
	roots{ 1 } z@  r@ 0 0 }} z@ z-  r> 0 1 }} z@ z/ ev{{ 1 1 }} z!

	\ Normalize the vectors
	ev{{ 1 0 }} z@ |z|^2  1e f+ fsqrt norm f!
	ev{{ 0 0 }} z@ norm f@ z/f  ev{{ 0 0 }} z!
	ev{{ 1 0 }} z@ norm f@ z/f  ev{{ 1 0 }} z!
	ev{{ 1 1 }} z@ |z|^2  1e f+ fsqrt norm f!
	ev{{ 0 1 }} z@ norm f@ z/f  ev{{ 0 1 }} z!
	ev{{ 1 1 }} z@ norm f@ z/f  ev{{ 1 1 }} z!
;

\ Return the eigenvalues and compute the eigenvectors;
\   input matrix columns are replaced by its eigenvectors.
: }}zeigen22 ( 'mat -- z1 z2  )
      dup >r }}zeigenvalues22  r@ }}zeigenvectors22
      ev{{ r> }}zcopy22
;


End-Module
BASE !

TEST-CODE? [IF]  \ ---------------------------------------------
[undefined] T{ [IF] s" ttester.4th" included [THEN]
[undefined] }}zput [IF]
\ store z11 ... z_nm into nxm matrix 
: }}zput ( z11 z12 ... z_nm  n m 'A -- )
      -ROT 2DUP * >R 1- SWAP 1- SWAP }} R> 
      0 ?DO  DUP >R z! R> COMPLEX -  LOOP  drop ;

[THEN]
BASE @
DECIMAL

: zz}t  rrrr}t ;
: }}22ev1 ( 'mat -- r1 r2 ) dup >r 0 0 }} z@ r> 1 0 }} z@ ;
: }}22ev2 ( 'mat -- r1 r2 ) dup >r 0 1 }} z@ r> 1 1 }} z@ ;
 
3e  fsqrt       fconstant  sqrt{3}
13e fsqrt       fconstant  sqrt{13} 
1e 2e f/ fsqrt  fconstant  1/sqrt{2}
3e sqrt{3} f*   fconstant  3*sqrt{3}


2 2 COMPLEX MATRIX m{{

1e-15 rel-near F!
zeig_eps f@ abs-near f!
set-near

TESTING  }}ZEIGENVALUES22
 -1e 0e   0e 0e
  0e 0e   1e 0e
2 2 m{{ }}zput
t{  m{{ }}zeigenvalues22 ->  1e 0e -1e 0e zz}t

  0e 0e   1e 0e
  1e 0e   0e 0e
2 2 m{{ }}zput
t{  m{{ }}zeigenvalues22 -> 1e 0e -1e 0e  zz}t

  2e 0e   1e 0e
  1e 0e   2e 0e
2 2 m{{ }}zput
t{  m{{ }}zeigenvalues22  ->  3e 0e  1e 0e  zz}t 

TESTING }}ZEIGEN22
  0e 0e   1e 0e
  1e 0e   0e 0e
2 2 m{{ }}zput
t{  m{{ }}zeigen22  ->  1e 0e -1e 0e zz}t
t{  m{{ }}22ev1  ->  1/sqrt{2} 0e  zdup  zz}t
t{  m{{ }}22ev2  ->  1/sqrt{2} fnegate 0e  1/sqrt{2} 0e zz}t

  2e 0e   1e 0e
  1e 0e   2e 0e
2 2 m{{ }}zput
t{  m{{ }}zeigen22  ->  3e 0e  1e 0e  zz}t
t{  m{{ }}22ev1  ->  1/sqrt{2} 0e  zdup  zz}t
t{  m{{ }}22ev2  ->  1/sqrt{2} fnegate 0e 1/sqrt{2} 0e zz}t

  2e 0e  -3e 0e
  4e 0e  -5e 0e
2 2 m{{ }}zput
t{  m{{ }}zeigen22  ->  -1e 0e  -2e 0e  zz}t
t{  m{{ }}22ev1  -> 1/sqrt{2} 0e  zdup  zz}t
t{  m{{ }}22ev2  -> 0.6e 0e  0.8e 0e  zz}t

  3e 0e  -9e 0e
  4e 0e  -3e 0e
2 2 m{{ }}zput
t{  m{{ }}zeigen22  ->  0e 3*sqrt{3} zdup conjg  zz}t
t{  m{{ }}22ev1  ->  3e 0e sqrt{13} z/f  1e sqrt{3} fnegate sqrt{13} z/f zz}t
t{  m{{ }}22ev2  ->  3e 0e sqrt{13} z/f  1e sqrt{3} sqrt{13} z/f  zz}t

BASE !
[THEN]


