#! /usr/local/bin/kforth
\ hp.4th
\
\ Loader for hp546x oscilloscope words
\ --  also performs GPIB setup to use the scope
\
\ K. Myneni, 2002-05-10
\
\ Revisions:
\
\	2002-11-14  added text menu code  KM
\	2003-11-03  extensive revision to use new common functions
\		    and header structure declaration in separate files.  KM
\       2005-07-28  change prefix from hp54624 to hp546x; integrate math
\                   function data download words from experimental version  km
\       2005-11-25  test for presence of required modules and load if needed  km
\       2006-08-09  updated for use with linux-gpib interface  km
include ans-words
include strings
include files
include ansi
include dump

[undefined] hp546x_digitize [IF]
include hp546x
[THEN]

7 to SCOPE

: ag_message. ( a u -- )
	black background
	7 foreground 
	1 1 at-xy type ;

: ag_message_clear ( -- )
	1 1 at-xy
	black background 79 spaces ;

: ag_reset ( -- | set up starting config )
	reset_scope		\ reset to factory defaults
	1e hp546x_set_time_range	\ set duration for waveform acquisition
	4000 set_record_length		\ set to 100, 250, 500, 1000, 2000; >2000 gives MAX
	FALSE set_averaging             \ no averaging (NORMAL mode acquisition)
	1 1 hp546x_set_channels         \ select channel 1 for acquisition
	1 1 hp546x_set_realtime         \ set real-time acquisition
;

create user_input 80 allot

: ag_select ( -- | permit user to select channels )
	s" Select range of channels to digitize, e.g. 1 4: "
	ag_message.
	depth >r
	user_input 80 accept
	user_input swap evaluate
	depth r> - 1 = if dup then hp546x_set_channels 
	ag_message_clear ;

: ag_digitize ( -- ) 
	page hp546x_digitize ;

: ag_download ( -- )
	s" Downloading waveforms from scope ..." ag_message.
	hp546x_download_waveforms
	ag_message_clear 
	7 emit page ;

: ag_info ( -- )
	1 2 at-xy
	black background 7 foreground
	whdr .wfmhdr ;

: ag_save ( -- )
	s" Enter the binary file name: " ag_message.
	user_input 80 accept
	user_input swap strpck hp546x_write_waveforms
	ag_message_clear
	?dup IF s" Error writing waveforms: " ag_message. . THEN ;
	
: ag_func ( -- )
	s" Downloading math function data ..." ag_message.
	hp546x_download_func
	ag_message_clear
	7 emit page ;

: ag_save_func ( -- )
	s" Enter the ascii file name: " ag_message.
	user_input 80 accept
	user_input swap strpck write_func drop
	ag_message_clear ;

\ cr
\
\ .( Usage: ) cr cr
\ .( 1 4 hp546x_set_channels		==> select channels 1 through 4 ) cr
\ .( hp546x_digitize			==> perform the acquisition ) cr
\ .( hp546x_download_waveforms		==> transfer waveform data to host ) cr
\ .( whdr .wfmhdr			==> display acquisition information ) cr
\ .( " temp.bin" hp546x_write_waveforms ==> write waveforms data to binary file )
\ cr cr


: ag_menu1 ( -- )                                              
    55 4 at-xy ."  Reset Config " ;                             
                                                                
: ag_menu2 ( -- )                                              
    55 5 at-xy ."  Select Channels " ;                            
                                                                
: ag_menu3 ( -- )                                              
    55 6 at-xy ."  Digitize " ;                              
                                                                
: ag_menu4 ( -- )                                              
    55 7 at-xy ."  Download " ;                       
                                                               
: ag_menu5 ( -- )
    55 8 at-xy  ."  Waveforms Info ";

: ag_menu6 ( -- )                                              
    55 9 at-xy  ."  Save Waveforms " ;                              

: ag_menu7 ( -- )
    55 10 at-xy ."  Download Math Func " ;

: ag_menu8 ( -- )
    55 11 at-xy ."  Save Func Data " ;
                                                             
: ag_menu_item. ( n -- | display n th menu item )              
   case
    1 of  ag_menu1  endof                                        
    2 of  ag_menu2  endof                                        
    3 of  ag_menu3  endof                                        
    4 of  ag_menu4  endof                                        
    5 of  ag_menu5  endof
    6 of  ag_menu6  endof
    7 of  ag_menu7  endof
    8 of  ag_menu8  endof                                        
   endcase ;                                                    
                                                                
VARIABLE ag_menu_choice
2 ag_menu_choice !

: AG_MENU. ( -- | display all menu options )                   
   \ prints currently selected item in reverse                  
   BLACK BACKGROUND
   55 2 AT-XY GREEN FOREGROUND
   ." AGILENT 546x MENU"
   55 3 AT-XY WHITE FOREGROUND
   ." GPIB ADDR: " SCOPE . 
   CYAN FOREGROUND                        
   9 1 DO                                                       
     I AG_MENU_CHOICE @ =                                      
     IF                                                         
       ( TEXT_REVERSE) 
       BLACK FOREGROUND CYAN BACKGROUND 
       I AG_MENU_ITEM. 
       ( TEXT_REVERSE)
       CYAN FOREGROUND BLACK BACKGROUND
     ELSE
       I ag_menu_item.                                         
     THEN                                                       
   LOOP                                                         
   ;                                                            

: MKEY ( -- n | wait for key and return code, handle extended keys )
	KEY DUP 27 = IF
	  KEY? IF 
	    DROP KEY
	    91 = IF
	      KEY
	    THEN
	  THEN
	THEN ;
	

: ag_menu ( -- | display menu and poll for input )
    BLACK BACKGROUND PAGE                                            
    BEGIN                                                       
      AG_MENU.                                                 
                                                               
     MKEY 				\ wait for key
     CASE                               \ consider each on   
       65 OF                            \   cursor up                        
         AG_MENU_CHOICE @ 1- 1 MAX            
         AG_MENU_CHOICE !                                      
         ENDOF                                                  
       66 OF                                                    
         AG_MENU_CHOICE @ 1+ 8 MIN      \  cursor down         
         AG_MENU_CHOICE !                                      
         ENDOF                                                  
                                                              
       10 OF AG_MENU_CHOICE @             \ the return key     
          case                             \ actuates a case    
            1 of ag_reset	endof     \ reset configuration         
            2 of ag_select endof     \                    
            3 of ag_digitize endof
            4 of ag_download endof
            5 of ag_info  endof
	    6 of ag_save  endof
	    7 of ag_func  endof
	    8 of ag_save_func endof
          endcase                                               
                                                               
          ENDOF                                                 
       27 OF PAGE EXIT ENDOF               \ or esc to exit     
     ENDCASE                             \ end of cases         
   AGAIN                                   \ repeat             
   ;                                                            
                                                             
: ag ( -- )                                                    
    open_gpib if
      red foreground 5 5 at-xy 7 emit                          
      ." Error opening GPIB!" cr        
      15 foreground ." <<Press any key to continue>> "           
      key drop exit                                             
    then
    init_gpib abort" Error initializing GPIB!" 
    10000000 ibtmo drop	\ set 100 sec timeout for GPIB I/O
   ag_reset
   ag_menu ;

ag
