\ hp546x.4th
\
\ GPIB interface to Agilent 54621A/22A/24A and 54621D/22D/41D/42D oscilloscopes
\
\ Requires the following source files:
\
\	ans-words.4th
\	strings.4th
\	files.4th
\	struct.4th
\	struct-ext.4th
\       ioctl.4th
\	gpib.4th or nigpib.4th
\	hpwfmhdr.4th
\	hpcommon.4th
\
\ Revisions:
\	10-10-2001  ported from hp54750.4th, modified several words  KM
\	05-10-2002  extensively revised  KM
\	2003-10-31  moved waveform header defns to separate file,
\			hpwfmhdr.4th for multiple use  KM
\	2003-11-03  updated code to use the waveform_header structure
\			and common hp functions in hpcommon.4th  KM
\       2005-07-28  use deferred common words SET_RECORD_LENGTH, SET_AVERAGING,
\                       and DOWNLOAD_WAVEFORM  km
\       2005-11-25  added ms delays in hp546x_download_waveform;
\                       test for presence of required modules and load if needed  km
\       2006-06-22  modified hp546x_set_wfmhdr to set new header fields for
\                       x increment, x origin, y increment, and y origin  km
\       2006-08-09  updated for use with linux-gpib driver interface  km
\       2007-02-23  merge changes from forked version  km
\
\ SCOPE is a value declared in hpcommon.4th. Use "n to SCOPE" to
\ select which HP/Agilent scope on the GPIB is being commanded,
\ where n is the primary GPIB address of the scope.

[undefined] waveform-header% [IF]
include hpwfmhdr
[THEN]

[undefined] SCOPE [IF]
include hpcommon
[THEN]

4 constant MAX_CHANNELS
2000000 constant MAX_SAMPLES

create whdr waveform-header% %allot drop

\ buffer to hold waveform data

MAX_SAMPLES MAX_CHANNELS * 2* constant WFMBUFSIZE

create hpbuf WFMBUFSIZE allot
variable hpbuf_ptr

: hp546x_set_record_length ( n -- | set the number of points )
    \ valid n values are 100, 250, 500, 1000, 2000, >2000 gives MAX
    	dup 2000 > if drop s" MAX" else s>string count then
    	s" :WAV:POIN " 2swap strcat strpck                      
    	SCOPE send_command ;

' hp546x_set_record_length  IS  set_record_length


: hp546x_set_averaging ( n -- | averaging on/off if n true/false )     
    	if c" :ACQ:TYPE AVER" else c" :ACQ:TYPE NORM" then                 
    	SCOPE send_command ;


' hp546x_set_averaging  IS  set_averaging


: hp546x_set_time_range ( f -- | set timebase seconds per division )
    	2 f>string count s" :TIM:RANGE " 
    	2swap strcat strpck SCOPE send_command ;

: hp546x_set_realtime ( -- | set real-time acquisition mode )
	c" :ACQ:MODE RTIM" SCOPE send_command ;

: hp546x_get_num_averages ( -- n | get the number of averages )
	c" :ACQ:TYPE?" SCOPE send_command
	5 SCOPE read_bytes
	gpib_in_buf c@ [char] A = if
	  c" :ACQ:COUN?" SCOPE send_command
	  19 SCOPE read_bytes
	  get_gpib_fnumber fround>s 
	else
	  0	\ not in averaging mode
	then ;


: hp546x_get_channel_coupling ( n -- a u | return string address and count )
	s" CHAN" rot s>string count strcat
	s" :COUP?" strcat strpck SCOPE send_command
	4 SCOPE read_bytes
	gpib_in_buf 3 ;

\ Make sure hpbuf_ptr is set to valid address prior to callin hp546x_download_waveform
: hp546x_download_waveform ( n -- | download raw waveform data for channel n )
	s>string count s" :WAV:SOUR CHAN"                      
	2swap strcat strpck                                         
	SCOPE send_command
	c" :WAV:FORM WORD" SCOPE send_command
	c" :WAV:DATA?" SCOPE send_command
	SCOPE 64 + 32 2 ibcmd drop \ set talker and listener
	1 ms
	gpib_in_buf 10 ibrd drop   \ read 10 byte header which contains data length
	gpib_in_buf 2+ 8 strpck string>s  \ number of bytes in the scope output
	hpbuf_ptr a@ swap 1+ ibrd drop    \ read the data bytes and terminator
	1 ms
	63 95 2 ibcmd drop            \ untalk and unlisten
	get_record_length 2* hpbuf_ptr +!  \ update ptr past recorded data
;

' hp546x_download_waveform  IS  download_waveform


variable nchannels
create channels MAX_CHANNELS allot

: hp546x_set_channels ( n1 n2 -- | n1 and n2 specify range of measurement channels )
	2dup > if swap then
	1+ swap 2dup - 
	dup MAX_CHANNELS > if drop 2drop 0 nchannels ! exit then
	nchannels !
	0 -rot do dup channels + i swap c! 1+ loop drop ;


: hp546x_clear_waveforms \ -- | erase the waveforms buffer 
	hpbuf WFMBUFSIZE erase ;



: hp546x_digitize ( -- | digitize the range of selected channels )
	hp546x_clear_waveforms
	nchannels @ if
	  s" :DIGITIZE "
	  nchannels @ 0 do
	    s" CHAN" i channels + c@ s>string count strcat
	    s" ," strcat strcat
	  loop
	  1- strpck SCOPE send_command
	then ;


: hp546x_set_wfmhdr ( -- | fill in the waveform header structure )
	get_record_length  	   whdr  wh_nsamples  !
	hp546x_get_num_averages    whdr  wh_naverages !
	get_time_range     	   whdr  wh_time_range f!
        get_time_position  	   whdr  wh_time_offset f!
	get_xinc		   whdr  wh_xincrement f!
	get_xorigin		   whdr  wh_xorigin f!
	get_yinc		   whdr  wh_yincrement f!
	get_yorigin		   whdr  wh_yorigin f!    
	nchannels @        	   whdr  wh_nchannels !
	channels           	   whdr  wh_channels MAX_CHANNELS cmove
	whdr wh_vscales
	nchannels @ 0 ?do 
	  dup channels i + c@ get_channel_scale rot f! dfloat+
	loop drop
	whdr wh_voffsets 
	nchannels @ 0 ?do
	  dup channels i + c@ get_channel_offset rot f! dfloat+
	loop drop
	whdr wh_couplings 
	nchannels @ 0 ?do
	  dup channels i + c@ hp546x_get_channel_coupling rot swap cmove 4 +
	loop drop
;



: hp546x_download_waveforms ( -- | download the measured channel waveform data )
	hpbuf hpbuf_ptr !
	nchannels @ if
	  nchannels @ 0 do
	    i channels + c@ hp546x_download_waveform
	  loop
	then 
	hp546x_set_wfmhdr ;
	     
variable wfm_fd

: hp546x_write_waveforms ( ^str -- error | write waveform hdr and data to file )
	\ error codes: 0 = no error, 1 = error opening file, 2 = error writing
	count W/O create-file 
	if 2drop 1 cr ." Error opening file." cr  exit then
	wfm_fd !
	whdr waveform-header% %size  wfm_fd @ write-file
	waveform-header% %size <> if wfm_fd @ close-file drop 2 exit then
	whdr wh_nsamples  @
	whdr wh_nchannels @
	* 2*	\ total number of bytes to write
	dup
	hpbuf swap wfm_fd @ write-file
	<> if 2 else 0 then
	wfm_fd @ close-file drop ;
	

fvariable func_range
fvariable func_offset

: hp546x_get_func_offset ( -- f | return the math function vertical offset )
	c" :FUNC:OFFS?" SCOPE send_command
	19 SCOPE read_bytes 
	get_gpib_fnumber ;

: hp546x_get_func_range ( -- f | return the math function vertical range )
	c" :FUNC:RANG?" SCOPE send_command
	19 SCOPE read_bytes 
	get_gpib_fnumber ;

: hp546x_download_func ( -- | download data for math function ) 
	hpbuf hpbuf_ptr !
	hp546x_get_func_offset  func_offset  f!
	hp546x_get_func_range   func_range   f!
	c" :WAV:SOUR FUNC"                                         
	SCOPE send_command
	c" :WAV:FORM WORD" SCOPE send_command
	c" :WAV:DATA?" SCOPE send_command

	10 SCOPE 64 + 32 2 ibcmd drop \ set talker and listener
	1 ms
	gpib_in_buf swap ibrd drop \ read the header which contains data length
	gpib_in_buf 2+ 8 strpck string>s  \ number of bytes in the scope output
	hpbuf_ptr a@ swap 1+ ibrd drop    \ read the data bytes and terminator
	1 ms
	63 95 2 ibcmd drop            \ untalk and unlisten
	get_record_length 2* hpbuf_ptr +!  \ update ptr past recorded data
	hp546x_set_wfmhdr ;
	     

fvariable nyquist_freq \ Nyquist frequency for FFT data
fvariable freq_step    \ frequency step between points for FFT data 	

: fft_val>dBV ( n -- fdBV | convert fft raw value to dBV value )
	s>f 256e f/ func_range f@ f* func_offset f@ f+ ;
 
: write_func ( ^str -- error | write function data to ascii file )
	\ error codes: 0 = no error, 1 = error opening file, 2 = error writing
	count W/O create-file 
	if 2drop 1 cr ." Error opening file." cr  exit then
	wfm_fd !
	whdr  wh_time_range f@
	whdr  wh_nsamples @ dup >R s>f f/ 2e f*
	1e fswap f/ nyquist_freq f!
	nyquist_freq f@ R@ s>f f/ freq_step f!
	R> 0 ?do
	  i s>f freq_step f@ f* 7 f>string count
	  s"  " strcat
	  i 2* dup hpbuf + c@ 8 lshift
	  swap 1+ hpbuf + c@ or 
	  fft_val>dBV 4 f>string count strcat	  
	  wfm_fd @ write-line drop
	loop
	0 wfm_fd @ close-file drop ;

