\ hp54750.4th
\
\ GPIB interface to HP54750 digitizing scope
\
\ Requires the following source files:
\
\	ans-words.4th
\	matrix.4th
\	strings.4th
\	files.4th
\	matfiles.4th
\	gpib.4th or nigpib.4th
\	hpcommon.4th
\
\ Revisions:
\
\	3-23-1999  ported from UR/FORTH  KM
\	3-26-1999  completed port  KM
\	5-7-1999   added waveform to file storage KM
\	2003-11-03  major revision to separate common code
\		    and allow multiple scopes to be interfaced
\		    simultaneously.  KM
\	2004-06-18  minor string constant cleanup. KM

\ SCOPE is a value declared in hpcommon.4th. Use "n to SCOPE" to
\ select which HP/Agilent scope on the GPIB is being commanded,
\ where n is the primary GPIB address of the scope.

7 constant HP54750		\ primary GPIB address of scope
HP54750 to SCOPE  		\ set default scope to HP54750

variable nwfmpts                                                
fvariable hp_yoffset
fvariable hp_yscale

2 4096 fmatrix hp54750_waveform
                                                                

: hp54750_set_record_length ( n -- | set the number of points )         
    s>string count                                              
    s" ACQ:POIN " 2swap strcat strpck                      
    SCOPE send_command ;

' hp54750_set_record_length  IS  set_record_length
 
: hp54750_set_averaging ( n -- | averaging on/off if n true/false )     
    if c" ACQ:AVER ON" else c" ACQ:AVER OFF" then                 
    SCOPE send_command ;

' hp54750_set_averaging  IS set_averaging

: hp54750_download_waveform ( ch -- | download waveform data )          
    s>string count s" :WAV:SOUR CHAN"                      
    2swap strcat strpck                                         
    SCOPE send_command                                          
    c" :WAV:FORM WORD" SCOPE send_command                        
    get_record_length dup nwfmpts !                                           
    2* 10 +                      \ number of bytes in record     
    c" :WAV:DATA?" SCOPE send_command                            
    SCOPE read_bytes ;

' hp54750_download_waveform  IS  download_waveform

: hp54750_get_waveform ( n -- rc | read ch n waveform onto stack )      
    hp54750_download_waveform                                           
    iberr if cr ." SCOPE GPIB TIMEOUT" cr then                        
    gpib_in_buf 9 +      \ copy waveform onto stack            
    nwfmpts @ 0 do dup w@ swap 2+ loop 
    drop nwfmpts @ ;


: hp54750_record_waveform  ( n -- | download ch n waveform into fmatrix hp54750_waveform )
	hp54750_download_waveform
	iberr if
	  cr ." SCOPE GPIB TIMEOUT" cr
	  exit
	then
	nwfmpts @ 2 hp54750_waveform mat_size!

	\ fill first column of hp54750_waveform with time in seconds

	get_time_position
	get_time_increment
	nwfmpts @ 0 do 
	  fover i 1+ 1 hp54750_waveform fmat!
	  fdup 2>r f+ 2r>
	loop
	fdrop fdrop

	\ fill second column of hp54750_waveform with waveform data in volts

	get_yinc hp_yscale f!
	get_yorigin hp_yoffset f!

	gpib_in_buf 9 +
	nwfmpts @ 0 do
	  dup w@
	  s>f hp_yscale f@ f* hp_yoffset f@ f+	   
	  i 1+ 2 hp54750_waveform fmat!
	  2+	  
	loop
	drop ;
	     

: hp54750_write_waveform ( ^str -- | write the waveform to specified file )
	hp54750_waveform swap write_fmat_ascii_file ;



