\ hpcommon.4th
\
\ Common GPIB commands for Hewlett-Packard/Agilent digital oscilloscopes
\
\ Revisions:
\
\	2003-10-31  created by Krishna Myneni
\       2005-11-25  test for presence of GPIB module and load if needed  km
\	2006-06-22  added get_xinc and get_xorigin  km
\       2006-08-09  updated for use with linux-gpib driver  km
\       2007-02-23  merge changes from forked version  km
\
\ Requires:
\
\	ans-words.4th
\	strings.4th
\       ioctl.4th
\	gpib.4th or nigpib.4th
\

[undefined] gpib_driver [IF]
    include ioctl
    include gpib
    \ include nigpib
[THEN]

0 value SCOPE  	\ GPIB primary address of oscilloscope

\ Model Specific Words

DEFER  set_record_length
DEFER  set_averaging
DEFER  download_waveform

\ Common GPIB commands for HP/Agilent scopes

: get_gpib_fnumber ( -- f | convert gpib ascii to float )     
    	gpib_in_buf 19 10 scan	\ scan for end of line char
    	drop gpib_in_buf - 		\ number of chars in line
    	dup gpib_in_buf fnumber_buf 1+ rot cmove               
    	dup fnumber_buf c!                                          
    	1+ 0 fnumber_buf rot + c!                                   
    	fnumber_buf string>f ; 



: reset_scope ( -- )
   	c" *RST" SCOPE send_command ;                                 

: autoscale ( -- )
   	c" :AUT" SCOPE send_command ;                                 

: clear_display ( -- )
	c" :CDIS" SCOPE send_command ;    

: set_num_averages ( nave -- | set number of averages )            
    	s>string count s" :ACQ:COUN "                          
    	2swap strcat strpck SCOPE send_command ;

: get_record_length ( -- npoints | get number of points )             
    	c" :WAV:POIN?"  SCOPE send_command                            
    	19 SCOPE read_bytes                                          
    	get_gpib_fnumber fround>s ;        

: get_time_position ( -- f | return the x offset )              
    	c" :TIM:POS?" SCOPE send_command                   
    	19 SCOPE read_bytes                                         
    	get_gpib_fnumber ;    

: get_time_range ( -- f | return full scale time in sec )       
    	c" :TIM:RANGE?" SCOPE send_command                      
    	19 SCOPE read_bytes                                         
    	get_gpib_fnumber ;

: get_time_increment ( -- f | delta t between points in sec )   
	get_time_range                                              
	get_record_length                                           
	s>f f/ ; 

: get_channel_offset ( n -- f | return ch n vertical offset in volts )
	s>string s" CHAN" rot count strcat
	s" :OFFS?" strcat strpck SCOPE send_command
	19 SCOPE read_bytes
	get_gpib_fnumber ;

: get_channel_scale ( n -- f | return ch n vertical scale in volts/div )
	s>string s" CHAN" rot count strcat
	s" :SCALE?" strcat strpck SCOPE send_command
	19 SCOPE read_bytes
	get_gpib_fnumber ;

: get_xinc ( -- f | return the time between points in sec )
	c" :WAV:XINC?" SCOPE send_command
	19 SCOPE read_bytes
	get_gpib_fnumber ;

: get_xorigin ( -- f | return x value in sec for first point )
	gpib_in_buf 14 32 fill
	c" :WAV:XOR?" SCOPE send_command
	19 SCOPE read_bytes
	get_gpib_fnumber ;

: get_yinc ( -- f | return the volts/unit scale factor for the waveform )
	c" :WAV:YINC?" SCOPE send_command
	19 SCOPE read_bytes
	get_gpib_fnumber ;

: get_yorigin ( -- f | return vertical y value for data count of zero )
	gpib_in_buf 14 32 fill
	c" :WAV:YOR?" SCOPE send_command
	19 SCOPE read_bytes
	get_gpib_fnumber ;

