\ hpwfmhdr.4th
\
\ Common header information for HP/Agilent oscilloscope waveforms
\
\ Usage: To make a waveform header structure,
\
\	CREATE hdr waveform-header% %allot
\
\ Examples of accessing individual member elements:
\
\	hdr  hdr_nchannels  @
\	hdr  hdr_time_range F@
\	
\ Revisions:
\
\	2003-10-31  created by Krishna Myneni
\	2003-11-03  used struct extension words  KM
\	2005-11-25  test for presence of required structure defs  KM
\	2006-06-22  added four new header fields: X increment, X origin,
\	              Y increment, and Y origin. Reduced size of padding 
\		      to maintain header size of 256 bytes  KM
\
\ Requires:
\
\	ans-words.4th
\	struct.4th
\	struct-ext.4th
\
\ The waveform header structure contains the following fields:
\
\	Number of samples			4-byte int
\	Number of averages			4-byte int
\	Number of channels			4-byte int
\	List of channel numbers			8 bytes
\	Channel vertical scales (volts/div)	64 bytes (8 dfloats)
\	Channel offsets (volts)			64 bytes (8 dfloats)
\	Channel couplings (AC, DC, GND)		32 bytes (8 4-byte strings)
\	Timebase range				8-byte dfloat
\	Timebase offset				8-byte dfloat
\	X increment (sec)			8-byte dfloat
\	X origin (sec)				8-byte dfloat
\	Y increment (volts)			8-byte dfloat
\	Y origin (volts at y=0)			8-byte dfloat
\	Padding					28 bytes
\

[undefined] struct [IF]
include struct.4th
[THEN]

[undefined] int32: [IF]
include struct-ext.4th
[THEN]

struct

	 int32:	 wh_nsamples
	 int32:	 wh_naverages
	 int32:	 wh_nchannels
	 8 buf:	 wh_channels
	64 buf:	 wh_vscales
	64 buf:	 wh_voffsets
	32 buf:	 wh_couplings
	dfloat:	 wh_time_range
	dfloat:  wh_time_offset
	dfloat:  wh_xincrement
	dfloat:  wh_xorigin
	dfloat:  wh_yincrement
	dfloat:  wh_yorigin
	28 buf:  wh_padding		\ pad to 256 bytes
end-struct waveform-header%


\ printing utility for waveform header structure

 
: .wfmhdr ( addr -- | annotated display of the waveform header )
	cr
	." # samples  = "  dup  wh_nsamples @ . cr
	." # averages = "  dup  wh_naverages @ . cr
	." # channels = "  dup  wh_nchannels @ dup >r . cr
	." Channels       : "  dup  wh_channels 
	r@ 0 ?do dup c@ . 1+ loop drop cr
	." Vscales (V/div): "   dup  wh_vscales 
	r@ 0 ?do dup f@ f. dfloat+ loop drop cr
	." Voffsets    (V): "   dup  wh_voffsets
	r@ 0 ?do dup f@ f. dfloat+ loop drop cr
	." Couplings      : "   dup  wh_couplings
	r@ 0 ?do dup 2 + c@ [char] D <> 
	  if 2 else 3 then over swap type bl emit 4 +
	loop drop cr
	." Time range  (s): "   dup  wh_time_range  f@ f. cr
	." Time pos    (s): "   dup  wh_time_offset f@ f. cr
	." X increment (s): "   dup  wh_xincrement  f@ f. cr
	." X origin    (s): "   dup  wh_xorigin     f@ f. cr
	." Y increment (s): "   dup  wh_yincrement  f@ f. cr
	." Y origin    (s): "   dup  wh_yorigin     f@ f. cr
	cr 
	drop r> drop
;
