(       Title:  Complex Word Set Tests
         File:  complex-test.fs
       Author:  David N. Williams
      Version:  0.9.3
      License:  LGPL
 Last Revised:  August 20, 2007
)
\ Copyright (C) 2002, 2003 David N. Williams
(
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or at your option any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA 02111-1307 USA.

Version 0.9.3
20Aug07  * Replaced test harness, ftester.fs, with ttester.fs.
           Changed the tests accordingly.

Version 0.9.2
 5Mar03  * Removed signed zero and branch cut tests.  Now
           they're in complex-szero-test.fs.

Version 0.9.1
21Feb03  * Added principal branch cut definitions and tests.
28Feb03  * Rearranged conditional includes.

Version 0.9.0
12Dec02  * Start.
18Feb03  * Release.

This code tests our modifications to Julian V. Noble's complex
arithmetic lexicon, and its port to pfe.

It is intended to test for formal correctness, not high
accuracy.

"Gauge functions" are functions that we test against.  They are
defined independently here, sometimes in terms of already tested
functions.

Except for DEFER and IS, this code is compatible with ANS Forth,
with an environmental dependence on lower case.
)

include ans-words.4th
include ttester.4th
: z}t   rr}t ;
: zz}t  rrrr}t ;
: zzz}t  6 0 DO ftester LOOP ...}t ;
DECIMAL
true verbose !

0 [IF]
s" COMPLEX-EXT" environment? [IF] ( version) drop
true constant PRINCIPAL-ARG
cr
testing pfe complex words
[ELSE]
cr .( COMPLEX-EXT not available.) cr
[THEN]
[THEN]

1 [IF]
include complex.4th  \ defines PRINCIPAL-ARG
cr
testing complex.fs
[THEN]

0 [IF]
s" complex-kahan.fs" included
true constant PRINCIPAL-ARG
cr
testing complex-kahan.fs
[THEN]

decimal

s" [UNDEFINED]" pad c! pad char+ pad c@ move 
pad find nip 0=
[IF]
: [UNDEFINED]  ( "name" -- flag )
  bl word find nip 0= ; immediate
[THEN]

s" [DEFINED]" pad c! pad char+ pad c@ move 
pad find nip 0=
[IF]
: [DEFINED]  postpone [UNDEFINED] 0= ; immediate
[THEN]

(
[UNDEFINED] \\ [IF]
  : \\   BEGIN  -1 parse  2drop  refill 0= UNTIL ; [THEN]
)

: near-defaults  ( -- )
  1e-14 abs-near f!
  1e-14 rel-near f! ;

near-defaults    \ set default tolerances

 0.7853981633974483096157e0 fconstant  pi/4
-0.7853981633974483096157e0 fconstant -pi/4
 0.5497787143782138167310e1 fconstant  7pi/4
[UNDEFINED] pi/2 [IF]
 0.1570796326794896619231e1 fconstant  pi/2
[THEN]
-0.1570796326794896619231e1 fconstant -pi/2
 0.4712388980384689857694e1 fconstant  3pi/2

 0.2356194490192344928847e1 fconstant  3pi/4
-0.2356194490192344928847e1 fconstant -3pi/4
 0.3926990816987241548078e1 fconstant  5pi/4

[UNDEFINED] pi [IF]
 0.3141592653589793238463e1 fconstant  pi
[THEN]
[UNDEFINED] -pi [IF]
-0.3141592653589793238463e1 fconstant -pi
[THEN]

 0.2718281828459045235360e1 fconstant  e
-0.2718281828459045235360e1 fconstant -e
 0.3678794411714423215955e0 fconstant  1/e
-0.3678794411714423215955e0 fconstant -1/e
[UNDEFINED] ln2 [IF]
 0.6931471805599453094172e0 fconstant  ln2
[THEN]
-0.6931471805599453094172e0 fconstant -ln2
 0.1414213562373095048802e1 fconstant  rt2
-0.1414213562373095048802e1 fconstant -rt2
 0.7071067811865475244008e0 fconstant  1/rt2
-0.7071067811865475244008e0 fconstant -1/rt2

\ only two choices
cr PRINCIPAL-ARG [IF]
testing uses arg output with -pi < arg <= pi
  -3pi/4 fconstant 225arg
  -pi/2  fconstant 270arg
  -pi/4  fconstant 315arg
[ElSE]
testing uses arg output with 0 <= arg < 2pi
   5pi/4 fconstant 225arg
   3pi/2 fconstant 270arg
   7pi/4 fconstant 315arg
[THEN]


defer gauge
defer func
defer inverse

\ ZVARIABLE, Z!, and Z@ have to be tested before the next 3
\ words are used.

zvariable zatemp   zvariable zbtemp

: ?gauge  ( f: z -- )
(
Compare the functions whose xt's are in FUNC and GAUGE.
) 
  zdup zatemp z! gauge   ->   zatemp z@ func   ;

: ?2gauge  ( f: z1 z2 -- )
(
Same as above with 2 complex arguments.
) 
  zbtemp z! zatemp z!
  zatemp z@ zbtemp z@ gauge -> zatemp z@ zbtemp z@ func ;

: ?inverse  ( f: z -- )
(
Check that INVERSE FUNCT, i.e., func[inverse], is the identity
mapping.
)
  zdup zatemp z! inverse func   ->   zatemp z@   ;

: -z   znegate ;

\ *** NONSTANDARD FP WORDS

testing  S>F  F-ROT or -FROT  FNIP  FTUCK  1/F  F^2  F2*  F2/

set-exact 
t{    0 s>f  ->    0e  r}t
t{  137 s>f  ->  137e  r}t
t{ -137 s>f  -> -137e  r}t

[DEFINED] f-rot [IF]
t{ 1e1 2e1 3e1 f-rot  -> 3e1 1e1 2e1  rrr}t
[THEN]
[DEFINED] -frot [IF]
t{ 1e1 2e1 3e1 -frot  -> 3e1 1e1 2e1 rrr}t
[THEN]
t{ 1e1 2e1     fnip   -> 2e1         r}t
t{ 1e1 2e1     ftuck  -> 2e1 1e1 2e1 rrr}t

t{  2e 1/f  ->  0.5e r}t
t{ -2e 1/f  -> -0.5e r}t

t{  0e f^2  -> 0e r}t
t{  2e f^2  -> 4e r}t
t{ -2e f^2  -> 4e r}t

t{     0e   f2*  ->     0e r}t
t{   128e   f2*  ->   256e r}t
t{ -12.8e   f2*  -> -25.6e r}t
set-near
t{  1/rt2   f2*  ->  rt2  r}t
t{ -1/rt2   f2*  -> -rt2  r}t

set-exact
t{     0e f2/  ->     0e   r}t
t{   256e f2/  ->   128e   r}t
t{ -25.6e f2/  -> -12.8e   r}t
set-near
t{  rt2   f2/  ->  1/rt2 r}t
t{ -rt2   f2/  -> -1/rt2 r}t

\ testing  FPI  F0.0  F1.0
\ testing  F0.0  F1.0

set-exact 
\ t{   fpi  -> pi  r}t
\ t{  f0.0  -> 0e  r}t
\ t{  f1.0  -> 1e  r}t

\ *** COMPLEX CONSTANTS AND VARIABLES

testing  ZCONSTANT  ZVARIABLE  Z@  Z!

set-exact
t{ 1e 2e zconstant 1+i2  ->       }t
t{ 1+i2                  -> 1e 2e z}t
t{ : equ zconstant ;     ->       }t
t{ 1+i2 equ z=(1+i2)     ->       }t
t{ z=(1+i2)             -> 1+i2   z}t

t{ zvariable zv1  ->      }t
t{ 1+i2 zv1 z!    ->      }t
t{ zv1 z@         -> 1+i2 z}t

\ *** COMPLEX STACK MANIPULATION

testing  Z=0  Z=1  Z=I

set-exact
t{  z=0  -> 0e 0e rr}t
t{  z=1  -> 1e 0e rr}t
t{  z=i  -> 0e 1e rr}t

testing  ZDROP  ZDUP  ZSWAP  ZOVER  ZNIP  ZTUCK  ZROT  Z-ROT

set-exact
t{ z=0         zdrop  ->                }t
t{ z=1         zdup   -> z=1 z=1      zz}t
t{ z=0 z=1     zswap  -> z=1 z=0      zz}t
t{ z=0 z=1     zover  -> z=0 z=1 z=0  zzz}t
t{ z=0 z=1     znip   -> z=1          z}t
t{ z=0 z=1     ztuck  -> z=1 z=0 z=1  z}t
[DEFINED] zrot [IF]
t{ z=0 z=1 z=i zrot  -> z=1 z=i z=0   zzz}t
[THEN]
[DEFINED] z-rot [IF]
t{ z=0 z=1 z=i z-rot -> z=i z=0 z=1   zzz}t
[THEN]

\ *** COMPLEX ALGEBRA

testing  REAL IMAG  CONJG  Z*F   Z/F  Z*  Z/  Z+  Z-

set-exact
t{ z=1     real   -> 1e      r}t
t{ z=i     imag   -> 1e      r}t
t{ 1e  2e  conjg  -> 1e -2e  z}t

set-near  \ true also works in pfe
t{ 1e 2e 3e    z*f   ->  3e  6e  z}t
t{ 3e 6e 3e    z/f   ->  1e  2e  z}t
t{ 1e 2e 3e 4e z*    -> -5e 10e  z}t
t{ 1e 2e z=1   z*    ->  1e  2e  z}t
t{ 1e 2e z=i   z*    -> -2e  1e  z}t
t{ 1e 1e 3e 4e z/    ->  7e 25e f/ -1e 25e f/  z}t
t{ 1e 1e 4e 3e z/    ->  7e 25e f/  1e 25e f/  z}t
t{ 1e 2e 3e 4e z+    ->  4e  6e  z}t
t{ 1e 2e 3e 4e z-    -> -2e -2e  z}t

testing  |Z|  |Z^2|  1/Z  Z^2  Z^N  ZNEGATE  Z2*  Z2/  I*  (-I)*

set-exact
t{   z=0        |z|     ->   0e         r}t
t{   z=0        |z|^2   ->   0e         r}t
t{   z=0        z^2     ->   z=0        z}t
t{   z=0        -z      ->  -0e   -0e   z}t
t{   1e   -2e   -z      ->  -1e    2e   z}t
t{   z=0        z2*     ->   z=0        z}t
t{  40e1 -20e1  z2*     ->  80e1 -40e1  z}t
t{ -40e1  20e1  z2*     -> -80e1  40e1  z}t
t{   z=0        z2/     ->   z=0        z}t
t{  50e1 -30e1  z2/     ->  25e1 -15e1  z}t
t{ -50e1  30e1  z2/     -> -25e1  15e1  z}t
t{   z=0        i*      ->  -0e    0e   z}t
t{  40e1 -20e1  i*      ->  20e1  40e1  z}t
t{ -40e1  20e1  i*      -> -20e1 -40e1  z}t
t{   z=0        (-i)*   ->   0e   -0e   z}t
t{  40e1 -20e1  (-i)*   -> -20e1 -40e1  z}t
t{ -40e1  20e1  (-i)*   ->  20e1  40e1  z}t

set-near
t{  3e  4e |z|   ->  5e  r}t
t{ -3e  4e |z|   ->  5e  r}t
t{  3e -4e |z|   ->  5e  r}t
t{  3e  4e |z|^2 -> 25e  r}t
t{ -3e  4e |z|^2 -> 25e  r}t
t{  3e -4e |z|^2 -> 25e  r}t
t{  3e  4e 1/z   ->  3e  25e f/ -4e 25e f/  z}t
t{ -3e  4e 1/z   -> -3e  25e f/ -4e 25e f/  z}t
t{  3e -4e 1/z   -> -3e -25e f/  4e 25e f/  z}t
t{  3e  4e z^2   -> -7e  24e  z}t
t{ -3e  4e z^2   -> -7e -24e  z}t
t{  3e -4e z^2   -> -7e -24e  z}t
t{ -3e -4e z^2   -> -7e  24e  z}t
t{ z=1     1/z   ->  1e -0e   z}t  \ sign of 0 ignored here

set-exact
t{  z=0      0 z^n  ->  z=1      z}t
t{  z=1      0 z^n  ->  z=1      z}t
t{ -1e   0e  0 z^n  ->  z=1      z}t
t{  z=i      0 z^n  ->  z=1      z}t
t{  0e  -1e  0 z^n  ->  z=1      z}t
t{  rt2  rt2 0 z^n  ->  z=1      z}t
t{  rt2 -rt2 0 z^n  ->  z=1      z}t
t{ -rt2  rt2 0 z^n  ->  z=1      z}t
t{ -rt2 -rt2 0 z^n  ->  z=1      z}t

t{  z=0      1 z^n  ->  z=0      z}t
t{  z=1      1 z^n  ->  z=1      z}t
t{ -1e   0e  1 z^n  -> -1e   0e  z}t
t{  z=i      1 z^n  ->  z=i      z}t
t{  0e  -1e  1 z^n  ->  0e  -1e  z}t
t{  rt2  rt2 1 z^n  ->  rt2  rt2 z}t
t{  rt2 -rt2 1 z^n  ->  rt2 -rt2 z}t
t{ -rt2  rt2 1 z^n  -> -rt2  rt2 z}t
t{ -rt2 -rt2 1 z^n  -> -rt2 -rt2 z}t

t{  z=0      2 z^n  ->  z=0      z}t
t{  z=1      2 z^n  ->  z=1      z}t
t{ -1e   0e  2 z^n  ->  z=1      z}t
t{  z=i      2 z^n  -> -1e   0e  z}t
set-near \ avoid signed zero discrepancy
t{  0e  -1e  2 z^n  -> -1e   0e  z}t
set-exact
t{  3e   4e  2 z^n  -> -7e   24e z}t
t{ -3e   4e  2 z^n  -> -7e  -24e z}t
t{  3e  -4e  2 z^n  -> -7e  -24e z}t
t{ -3e  -4e  2 z^n  -> -7e   24e z}t

t{  z=0      5 z^n  ->  z=0        z}t
t{  z=1      5 z^n  ->  z=1        z}t
t{ -1e   0e  5 z^n  -> -1e    0e   z}t
t{  z=i      5 z^n  ->  z=i        z}t
t{  0e  -1e  5 z^n  ->  0e   -1e   z}t
t{  2e   2e  5 z^n  -> -128e -128e z}t
t{  2e  -2e  5 z^n  -> -128e  128e z}t
t{ -2e   2e  5 z^n  ->  128e -128e z}t
t{ -2e  -2e  5 z^n  ->  128e  128e z}t

testing  ARG  >POLAR  POLAR>  ZSQRT  ZLN  ZEXP  Z^

set-exact
t{  z=0      arg  -> 0e     r}t
t{  z=1      arg  -> 0e     r}t
t{  z=i      arg  -> pi/2   r}t
t{  0e  -1e  arg  -> 270arg r}t
t{ -1e   0e  arg  -> pi     r}t
PRINCIPAL-ARG [IF]
t{ -1e  -0e  arg  -> -pi    r}t
[THEN]
set-near
t{  2e   2e  arg  -> pi/4   r}t
t{  3e  -3e  arg  -> 315arg r}t
set-exact
t{ -rt2  rt2 arg  -> 3pi/4  r}t
t{ -rt2 -rt2 arg  -> 225arg r}t

rt2 f2*   fconstant 2rt2
rt2 3e f* fconstant 3rt2

set-exact
t{  z=0      >polar  -> 0e   0e     z}t
t{  z=1      >polar  -> 1e   0e     z}t
t{  z=i      >polar  -> 1e   pi/2   z}t
t{  0e  -1e  >polar  -> 1e   270arg z}t
t{ -1e   0e  >polar  -> 1e   pi     z}t
set-near
t{  2e   2e  >polar  -> 2rt2 pi/4   z}t
t{  3e  -3e  >polar  -> 3rt2 315arg z}t
t{ -rt2  rt2 >polar  -> 2e   3pi/4  z}t
t{ -rt2 -rt2 >polar  -> 2e   225arg z}t

set-exact
t{ 0e    0e    polar>  ->  z=0      z}t
t{ 1e    0e    polar>  ->  z=1      z}t
set-near
t{ 1e    pi/2  polar>  ->  z=i      z}t
t{ 1e   -pi/2  polar>  ->  0e  -1e  z}t
t{ 1e    pi    polar>  -> -1e   0e  z}t
t{ 2rt2  pi/4  polar>  ->  2e   2e  z}t
t{ 3rt2 -pi/4  polar>  ->  3e  -3e  z}t
t{ 2e    3pi/4 polar>  -> -rt2  rt2 z}t
t{ 2e   -3pi/4 polar>  -> -rt2 -rt2 z}t

: gsqrt  ( f: z -- exp[[ln|z|+iarg[z]]/2] )
  zln z2/ zexp ; 

' zsqrt is func   ' gsqrt is gauge
set-near
t{ z=0  zsqrt  ->  z=0 z}t
t{  2e   0e  ?gauge  z}t
t{ -2e   0e  ?gauge  z}t
t{  0e   2e  ?gauge  z}t
t{  0e  -2e  ?gauge  z}t
t{  rt2  rt2 ?gauge  z}t
t{  rt2 -rt2 ?gauge  z}t
t{ -rt2  rt2 ?gauge  z}t
t{ -rt2 -rt2 ?gauge  z}t

set-exact
t{  z=1      zln  -> z=0        z}t
t{  z=i      zln  -> 0e  pi/2   z}t
t{  0e  -1e  zln  -> 0e  270arg z}t
t{ -1e   0e  zln  -> 0e  pi     z}t
set-near
t{ -2e   0e  zln  -> ln2 pi     z}t
t{  rt2  rt2 zln  -> ln2 pi/4   z}t
t{  rt2 -rt2 zln  -> ln2 315arg z}t
t{ -rt2  rt2 zln  -> ln2 3pi/4  z}t
t{ -rt2 -rt2 zln  -> ln2 225arg z}t

 1/rt2 f2/      fconstant  1/2rt2
 1/2rt2 fnegate fconstant -1/2rt2

set-exact
t{  z=0        zexp  ->  z=1            z}t
t{  ln2  0e    zexp  ->  2e      0e     z}t
set-near
t{ -ln2  0e    zexp  ->  0.5e    0e     z}t
t{  0e   pi    zexp  -> -1e      0e     z}t
t{  0e   pi/2  zexp  ->  z=i            z}t
t{  0e  -pi/2  zexp  ->  z=i conjg      z}t
t{  0e   pi/4  zexp  ->  1/rt2   1/rt2  z}t
t{  0e  -pi/4  zexp  ->  1/rt2  -1/rt2  z}t
t{  0e   3pi/4 zexp  -> -1/rt2   1/rt2  z}t
t{  0e  -3pi/4 zexp  -> -1/rt2  -1/rt2  z}t
t{  ln2  pi/4  zexp  ->  rt2     rt2    z}t
t{  ln2 -pi/4  zexp  ->  rt2    -rt2    z}t
t{ -ln2  3pi/4 zexp  -> -1/2rt2  1/2rt2 z}t
t{ -ln2 -3pi/4 zexp  -> -1/2rt2 -1/2rt2 z}t

set-exact
\ t{  z=0      z=0  z^  ->  z=0     }t
t{  z=1      z=0  z^  ->  z=1     z}t
t{ -1e   0e  z=0  z^  ->  z=1     z}t
t{  z=i      z=0  z^  ->  z=1     z}t
t{  0e  -1e  z=0  z^  ->  z=1     z}t
t{  rt2  rt2 z=0  z^  ->  z=1     z}t
t{  rt2 -rt2 z=0  z^  ->  z=1     z}t
t{ -rt2  rt2 z=0  z^  ->  z=1     z}t
t{ -rt2 -rt2 z=0  z^  ->  z=1     z}t

: identical  ( f: z -- z )  ;

: z^(z=1)   z=1 z^ ;

' z^(z=1) is func   ' identical is gauge
set-near
\ t{  z=0     ?gauge z}t
t{  z=1      ?gauge  z}t
t{ -1e   0e  ?gauge  z}t
t{  z=i      ?gauge  z}t
t{  0e  -1e  ?gauge  z}t
t{  rt2  rt2 ?gauge  z}t
t{  rt2 -rt2 ?gauge  z}t
t{ -rt2  rt2 ?gauge  z}t
t{ -rt2 -rt2 ?gauge  z}t

: noname  ( f: z -- z^(1+i2)  1+i2 z^ ; ' noname  is func
: noname  ( f: z -- z^(1+i2)  zln 1+i2 z* zexp ; ' noname is gauge
t{  z=1      ?gauge  z}t
t{ -1e   0e  ?gauge  z}t
t{  z=i      ?gauge  z}t
t{  0e  -1e  ?gauge  z}t
t{  rt2  rt2 ?gauge  z}t
t{  rt2 -rt2 ?gauge  z}t
t{ -rt2  rt2 ?gauge  z}t
t{ -rt2 -rt2 ?gauge  z}t

testing  ZCOSH  ZSINH  ZTANH  ZCOTH  ZCOS  ZSIN  ZTAN  ZCOT

e 1/e f+ f2/ fconstant ch1
e 1/e f- f2/ fconstant sh1
      ch1 0e zconstant zch1
      sh1 0e zconstant zsh1
  sh1 ch1 f/ fconstant th1
  ch1 sh1 f/ fconstant cth1
      th1 0e zconstant zth1
     cth1 0e zconstant zcth1

ch1 sh1 rt2 z/f zconstant zC1
      zC1 conjg zconstant zC2
sh1 ch1 rt2 z/f zconstant zC3
      zC3 conjg zconstant zC4 

set-exact
t{  z=0      zcosh  -> z=1  z}t
set-near
t{  z=1      zcosh  -> zch1 z}t
t{ -1e  0e   zcosh  -> zch1 z}t
t{  0e  pi/2 zcosh  -> z=0  z}t
t{  0e -pi/2 zcosh  -> z=0  z}t
t{  1e  pi/4 zcosh  -> zC1  z}t
t{  1e -pi/4 zcosh  -> zC2  z}t
t{ -1e  pi/4 zcosh  -> zC2  z}t
t{ -1e -pi/4 zcosh  -> zC1  z}t

set-exact
t{  z=0      zsinh  -> z=0        z}t
set-near
t{  z=1      zsinh  -> zsh1       z}t
t{ -1e  0e   zsinh  -> sh1 fnegate 0e z}t
t{  0e  pi/2 zsinh  -> z=i        z}t
t{  0e -pi/2 zsinh  -> z=i conjg  z}t
t{  1e  pi/4 zsinh  -> zC3        z}t
t{  1e -pi/4 zsinh  -> zC4        z}t
t{ -1e  pi/4 zsinh  -> zC4 -z     z}t
t{ -1e -pi/4 zsinh  -> zC3 -z     z}t

1e  pi/4 zdup zsinh zswap zcosh z/ zconstant ztanhA
1e -pi/4 zdup zsinh zswap zcosh z/ zconstant ztanhB
ztanhA 1/z zconstant zcothA
ztanhB 1/z zconstant zcothB

set-exact
t{  z=0      ztanh  -> z=0        z}t
set-near
t{  z=1      ztanh  -> zth1       z}t
t{ -1e  0e0  ztanh  -> zth1 -z    z}t
t{  0e  pi/4 ztanh  -> z=i        z}t
t{  0e -pi/4 ztanh  -> z=i conjg  z}t
t{  1e  pi/4 ztanh  -> ztanhA     z}t
t{  1e -pi/4 ztanh  -> ztanhB     z}t
t{ -1e  pi/4 ztanh  -> ztanhB -z  z}t
t{ -1e -pi/4 ztanh  -> ztanhA -z  z}t

set-near
t{  z=1      zcoth  -> zcth1      z}t
t{ -1e  0e0  zcoth  -> zcth1 -z   z}t
t{  0e  pi/4 zcoth  -> z=i conjg  z}t
t{  0e -pi/4 zcoth  -> z=i        z}t
t{  1e  pi/4 zcoth  -> zcothA     z}t
t{  1e -pi/4 zcoth  -> zcothB     z}t
t{ -1e  pi/4 zcoth  -> zcothB -z  z}t
t{ -1e -pi/4 zcoth  -> zcothA -z  z}t

: noname  (-i)* zcos ; ' noname is func
' zcosh is gauge
set-exact
t{  z=0      ?gauge  z}t
t{  z=1      ?gauge  z}t
t{ -1e  0e   ?gauge  z}t
t{  0e  pi/2 ?gauge  z}t
t{  0e -pi/2 ?gauge  z}t
t{  1e  pi/4 ?gauge  z}t
t{  1e -pi/4 ?gauge  z}t
t{ -1e  pi/4 ?gauge  z}t
t{ -1e -pi/4 ?gauge  z}t

: noname  i* zsin ;  ' noname is func
: noname  zsinh i* ; ' noname is gauge
set-near
t{  z=0      ?gauge  z}t
t{  z=1      ?gauge  z}t
t{ -1e  0e   ?gauge  z}t
t{  0e  pi/2 ?gauge  z}t
t{  0e -pi/2 ?gauge  z}t
t{  1e  pi/4 ?gauge  z}t
t{  1e -pi/4 ?gauge  z}t
t{ -1e  pi/4 ?gauge  z}t
t{ -1e -pi/4 ?gauge  z}t

: noname  i* ztan ;  ' noname is func
: noname  ztanh i* ; ' noname is gauge
set-near
t{  z=0      ?gauge  z}t
t{  z=1      ?gauge  z}t
t{ -1e  0e   ?gauge  z}t
t{  0e  pi/4 ?gauge  z}t
t{  0e -pi/4 ?gauge  z}t
t{  1e  pi/4 ?gauge  z}t
t{  1e -pi/4 ?gauge  z}t
t{ -1e  pi/4 ?gauge  z}t
t{ -1e -pi/4 ?gauge  z}t

: noname  i* zcot ;  ' noname is func
: noname  zcoth (-i)* ; ' noname is gauge
set-near
t{  z=1      ?gauge  z}t
t{ -1e  0e   ?gauge  z}t
t{  0e  pi/4 ?gauge  z}t
t{  0e -pi/4 ?gauge  z}t
t{  1e  pi/4 ?gauge  z}t
t{  1e -pi/4 ?gauge  z}t
t{ -1e  pi/4 ?gauge  z}t
t{ -1e -pi/4 ?gauge  z}t


\ *** INVERSE FUNCTIONS

PRINCIPAL-ARG 0= [IF]
cr .( Skipping inverse function tests.  To do those,)
cr .(   set PRINCIPAL-ARG to true in complex.fs.)
cr

[ELSE]
testing  ZASINH  ZACOSH  ZATANH  ZACOTH

\ Inverse hyperbolic gauges.  Note that in the principal
\ expressions for the gauges here, and in GACOS in the next
\ section, it is important to use "1E x+" instead of "z=1 z+"
\ to preserve the sign of zero on the branch cuts.  That is
\ not tested in this file (see complex-szero-test.fs).

: gasinh   ( z -- [ln[z+sqrt[z^2+1]]] )
  zdup z^2 1E x+ zsqrt z+ zln ;

: gacosh   ( z -- 2ln[sqrt[[z+1]/2]+sqrt[[z-1]/2] )
  zdup 1E x- z2/ zsqrt   zswap 1E x+ z2/ zsqrt
  z+ zln z2* ;

: gatanh   ( z -- [ln[1+z]-ln[1-z]]/2 )
  zdup 1E x+ zln   zswap znegate 1E x+ zln   z- z2/ ;

: gacoth  ( z = [ln[-1-z]-ln[1-z]]/2 )
(
Use -1e 0e Z+ instead of Z=1 Z- so -0 doesn't give the wrong
value on the ZLN principal branch cut.
)
  -z zdup -1e 0e z+ zln   zswap z=1 z+ zln   z- z2/ ;

\ Check that the gauges are inverses.  The order func(inverse)
\ used here, e.g., ZACOSH COSH in Forth reverse polish, should
\ work for all branches of the inverse when func is meromorphic.

' gasinh is inverse   ' zsinh is func
set-near
t{ z=0        ?inverse  z}t
t{ zsh1       ?inverse  z}t
t{ zsh1 -z    ?inverse  z}t
t{ z=i        ?inverse  z}t
t{ z=i conjg  ?inverse  z}t
t{ zC3        ?inverse  z}t
t{ zC4        ?inverse  z}t
t{ zC4 -z     ?inverse  z}t
t{ zC3 -z     ?inverse  z}t

' zasinh is func   ' gasinh is gauge
set-near
t{ z=0        ?gauge  z}t
t{ zsh1       ?gauge  z}t
t{ zsh1 -z    ?gauge  z}t
t{ z=i        ?gauge  z}t
t{ z=i conjg  ?gauge  z}t
t{ zC3        ?gauge  z}t
t{ zC4        ?gauge  z}t
t{ zC4 -z     ?gauge  z}t
t{ zC3 -z     ?gauge  z}t

' gacosh is inverse   ' zcosh  is func
set-near
t{ z=1  ?inverse  z}t
t{ zch1 ?inverse  z}t
t{ z=0  ?inverse  z}t
t{ zC1  ?inverse  z}t
t{ zC2  ?inverse  z}t

' zacosh is func   ' gacosh is gauge   
set-near
t{ z=1  ?gauge  z}t
t{ zch1 ?gauge  z}t
t{ z=0  ?gauge  z}t
t{ zC1  ?gauge  z}t
t{ zC2  ?gauge  z}t

' gatanh is inverse   ' ztanh is func
set-near
t{  z=0       ?inverse  z}t
t{ zth1       ?inverse  z}t
t{ zth1 -z    ?inverse  z}t
t{ z=i        ?inverse  z}t
t{ z=i conjg  ?inverse  z}t
t{ ztanhA     ?inverse  z}t
t{ ztanhB     ?inverse  z}t
t{ ztanhB -z  ?inverse  z}t
t{ ztanhA -z  ?inverse  z}t

' zatanh is func   ' gatanh is gauge
set-near
t{ z=0        ?gauge  z}t
t{ z=i        ?gauge  z}t
t{  1e  1e    ?gauge  z}t
t{  1e -1e    ?gauge  z}t
t{ -1e  1e    ?gauge  z}t
t{ -1e -1e    ?gauge  z}t
t{ z=i conjg  ?gauge  z}t
t{ zth1       ?gauge  z}t
t{ zth1 -z    ?gauge  z}t
t{ ztanhA     ?gauge  z}t
t{ ztanhB     ?gauge  z}t
t{ ztanhB -z  ?gauge  z}t
t{ ztanhA -z  ?gauge  z}t

' gacoth is inverse   ' zcoth is func
set-near
t{ zcth1      ?inverse  z}t
t{ zcth1 -z   ?inverse  z}t
t{ z=i conjg  ?inverse  z}t
t{ z=i        ?inverse  z}t
t{ zcothA     ?inverse  z}t
t{ zcothB     ?inverse  z}t
t{ zcothB -z  ?inverse  z}t
t{ zcothA -z  ?inverse  z}t

[DEFINED] zacoth [IF]
' zacoth is func   ' gacoth is gauge
set-near
t{ zcth1      ?gauge  z}t
t{ zcth1 -z   ?gauge  z}t
t{ z=i conjg  ?gauge  z}t
t{ z=i        ?gauge  z}t
t{ zcothA     ?gauge  z}t
t{ zcothB     ?gauge  z}t
t{ zcothB -z  ?gauge  z}t
t{ zcothA -z  ?gauge  z}t
[THEN]

testing  ZASIN  ZACOS  ZATAN  ZACOT

\ Inverse trigonometric gauges.  GACOS is uncouth in the sense
\ of Corless, Davenport, Jeffrey, and Watt, i.e., not related to
\ the inverse hyperbolic counterpart by the naive identity.

: gacos   ( f: z -- -2iln[sqrt[[1+z]/2]+isqrt[[1-z]/2]] )
  zdup 1E x+ z2/ zsqrt   zswap -z 1E x+ z2/ zsqrt
  i* z+ zln z2* (-i)*  ;

: gasin   i*    gasinh (-i)* ;
: gatan   i*    gatanh (-i)* ;
: gacot   (-i)* gacoth (-i)* ;

\ We've checked that the inverse hyperbolic gauges are inverses,
\ so where they're couthly related, it's sufficient to check one
\ case of each inverse trigonometric gauge, where both input and
\ output are full complex numbers.  We checked by hand that zC1
\ works.

' gacos is inverse   ' zcos is func
set-near
t{ zC1 ?inverse  z}t
t{ zC2 ?inverse  z}t
t{ zC3 ?inverse  z}t
t{ zC4 ?inverse  z}t
t{ zC1 gasin zsin -> zC1 z}t
t{ zC1 gatan ztan -> zC1 z}t
t{ zC1 gacot zcot -> zC1 z}t

: noname   (-i)* zcos zacos ; ' noname is func
: noname   (-i)* zcos gacos ; ' noname is gauge
set-near
t{  z=0      ?gauge  z}t
t{  z=1      ?gauge  z}t
t{ -1e  0e   ?gauge  z}t
t{  0e  pi/2 ?gauge  z}t
t{  0e -pi/2 ?gauge  z}t
t{  1e  pi/4 ?gauge  z}t
t{  1e -pi/4 ?gauge  z}t
t{ -1e  pi/4 ?gauge  z}t
t{ -1e -pi/4 ?gauge  z}t

: noname   (-i)* zsin zasin ; ' noname is func
: noname   (-i)* zsin gasin ; ' noname is gauge
set-near
t{  z=0      ?gauge  z}t
t{  z=1      ?gauge  z}t
t{ -1e  0e   ?gauge  z}t
t{  0e  pi/2 ?gauge  z}t
t{  0e -pi/2 ?gauge  z}t
t{  1e  pi/4 ?gauge  z}t
t{  1e -pi/4 ?gauge  z}t
t{ -1e  pi/4 ?gauge  z}t
t{ -1e -pi/4 ?gauge  z}t

: noname   (-i)* ztan zatan ; ' noname is func
: noname   (-i)* ztan gatan ; ' noname is gauge
set-near
t{  z=0      ?gauge  z}t
t{  z=1      ?gauge  z}t
t{ -1e  0e   ?gauge  z}t
t{  0e  pi/2 ?gauge  z}t
t{  0e -pi/2 ?gauge  z}t
t{  1e  pi/4 ?gauge  z}t
t{  1e -pi/4 ?gauge  z}t
t{ -1e  pi/4 ?gauge  z}t
t{ -1e -pi/4 ?gauge  z}t

[DEFINED] zacot [IF]
: noname   (-i)* zcot zacot ; ' noname is func
: noname   (-i)* zcot gacot ; ' noname is gauge
set-near
t{  z=1      ?gauge  z}t
t{ -1e  0e   ?gauge  z}t
t{  0e  pi/2 ?gauge  z}t
t{  0e -pi/2 ?gauge  z}t
t{  1e  pi/4 ?gauge  z}t
t{  1e -pi/4 ?gauge  z}t
t{ -1e  pi/4 ?gauge  z}t
t{ -1e -pi/4 ?gauge  z}t
[THEN]

[THEN] \ inverse functions

testing done
