(   Title:  FATAN2 tests
     File:  fatan2-test.fs
   Author:  David N. Williams
  Version:  0.1.0
  License:  Public Domain
  Revised:  March 19, 2009
  
Changes:
Version 0.1.0
14-Mar-09 * Started.
19-Mar-09    revised for kForth

This file tests the ISO C / Single UNIX 3 specification for
the ANS Forth Floating-Ext word:
)
\ FATAN2 ( f: y x -- radians ) or ( y x -- radians )
(
It expects the return value to be the principal angle, and
includes a number of tests with +/-0, +/-NAN, and +/-INF that
are not required by the ANS Forth 94 standard.

)

\ for pfe:
\ s" FLOATING-EXT" environment? [IF] drop [THEN]

\ for kforth
include ans-words
: PARSE   WORD COUNT ;		  \ not exactly correct, but ok for use here

[UNDEFINED]  NAN [IF]  0e 0e f/ fconstant  NAN [THEN]
[UNDEFINED]  INF [IF]  1e 0e f/ fconstant  INF [THEN]
[UNDEFINED] -INF [IF] -1e 0e f/ fconstant -INF [THEN]

\ s" ttester.fs" included
include ttester
true verbose !
decimal

\ The ttester default for EXACT? is TRUE.  Uncomment the
\ following line if your system needs it to be FALSE:
\ SET-NEAR

variable #errors    0 #errors !

: noname  ( c-addr u -- )
(
Display an error message followed by the line that had the
error.
)
  1 #errors +! error1 ; 
' noname error-xt !

[UNDEFINED] \\ [IF]
  : \\  ( -- )  -1 parse 2drop BEGIN refill 0= UNTIL ; [THEN]

 0.7853981633974483096157E0 fconstant  pi/4
-0.7853981633974483096157E0 fconstant -pi/4
 0.1570796326794896619231E1 fconstant  pi/2
-0.1570796326794896619231E1 fconstant -pi/2
 0.4712388980384689857694E1 fconstant  3pi/2

 0.2356194490192344928847E1 fconstant  3pi/4
-0.2356194490192344928847E1 fconstant -3pi/4

[UNDEFINED] pi [IF]
 0.3141592653589793238463E1 fconstant  pi
[THEN]
[UNDEFINED] -pi [IF]
-0.3141592653589793238463E1 fconstant -pi
[THEN]

\ These tests are from Ed, comp.lang.forth, "Re: Ambiguity in
\ FATAN2", 14-Mar-09:

t{ -1e  1e fatan2 -> -pi/4 }t
t{ -1e  0e fatan2 -> -pi/2 }t
t{ -1e -1e fatan2 -> -3pi/4 }t

\ ISO C / Single UNIX Specification Version 3:
\   http://www.unix.org/single_unix_specification/
\ Select "Topic", then "Math Interfaces", then "atan2()":
\   http://www.opengroup.org/onlinepubs/009695399/functions/atan2f.html

\ If y is +/-0 and x is < 0, +/-pi shall be returned.
t{  0e -1e fatan2 ->  pi }t
t{ -0e -1e fatan2 -> -pi }t

\ If y is +/-0 and x is > 0, +/-0 shall be returned.
t{  0e  1e fatan2 ->  0e }t
t{ -0e  1e fatan2 -> -0e }t

\ If y is < 0 and x is +/-0, -pi/2 shall be returned.
t{ -1e  0e fatan2 -> -pi/2 }t
t{ -1e -0e fatan2 -> -pi/2 }t

\ If y is > 0 and x is +/-0, pi/2 shall be returned.
t{  1e  0e fatan2 -> pi/2 }t
t{  1e -0e fatan2 -> pi/2 }t

\ Optional ISO C / single UNIX specs:

\ If either x or y is NaN, a NaN shall be returned.
t{ NAN  1e fatan2 -> NAN }t
t{  1e NAN fatan2 -> NAN }t
t{ NAN NAN fatan2 -> NAN }t

\ If y is +/-0 and x is -0, +/-pi shall be returned.
t{  0e -0e fatan2 ->  pi }t
t{ -0e -0e fatan2 -> -pi }t

\ If y is +/-0 and x is +0, +/-0 shall be returned.
t{  0e  0e fatan2 -> +0e }t
t{ -0e  0e fatan2 -> -0e }t

\ For finite values of +/-y > 0, if x is -Inf, +/-pi shall be returned.
t{  1e -INF fatan2 ->  pi }t
t{ -1e -INF fatan2 -> -pi }t

\ For finite values of +/-y > 0, if x is +Inf, +/-0 shall be returned.
t{  1e  INF fatan2 -> +0e }t
t{ -1e  INF fatan2 -> -0e }t

\ For finite values of x, if y is +/-Inf, +/-pi/2 shall be returned.
t{  INF  1e fatan2 ->  pi/2 }t
t{  INF -1e fatan2 ->  pi/2 }t
t{  INF  0e fatan2 ->  pi/2 }t
t{  INF -0e fatan2 ->  pi/2 }t
t{ -INF  1e fatan2 -> -pi/2 }t
t{ -INF -1e fatan2 -> -pi/2 }t
t{ -INF  0e fatan2 -> -pi/2 }t
t{ -INF -0e fatan2 -> -pi/2 }t

\ If y is +/-Inf and x is -Inf, +/-3pi/4 shall be returned.
t{  INF -INF fatan2 ->  3pi/4 }t
t{ -INF -INF fatan2 -> -3pi/4 }t

\ If y is +/-Inf and x is +Inf, +/-pi/4 shall be returned.
t{  INF  INF fatan2 ->  pi/4 }t
t{ -INF  INF fatan2 -> -pi/4 }t

cr .( #ERRORS: ) #errors @ . cr
