\ h2XJ0-x11.4th
\
\ Compute the bound vibrational levels of the H_2 molecule
\ in its ground electronic state (see h2XJ0.4th) and provide
\ a graphical display of the computed energy levels and
\ probability densities obtained from the wavefunctions.
\
\ Copyright (c) 2011 Krishna Myneni
\
\ This code may be used for any purpose as long as the copyright
\ notice above is preserved.
\
\ Requires:
\
\   simple-plot.4th
\   h2XJ0.4th
\
\ Revisions:
\   2011-08-28  km  first version

include simple-plot
include h2XJ0.4th
also X11

[undefined] fnip  [IF] : fnip  fswap fdrop ; [THEN]
[undefined] ftuck [IF] : ftuck fswap fover ; [THEN]

XPoint% %size constant XPT_SIZE
: XPoint! ( nx ny apoint -- )  rot over XPoint->x w! XPoint->y w! ;

fvariable fheight
fvariable fwidth

7e      Rmin f@ f- fconstant delR
-0.95e  Vmin f@ f- fconstant delV

: scale-point ( F: r x -- ) ( -- ux uy )
	Vmin f@ f- delV f/ 1e f- fnegate fheight f@ f* fround>s >r
	Rmin f@ f- delR f/ fwidth  f@ f* fround>s r> ;
 

\ Find the endpoints of r on the potential curve, corresponding to
\   specified energy.
: find-rlims ( F: e -- r1 r2 )
    Nmesh @ 0 DO
      fdup V_mesh{ I } f@ f> IF I leave THEN
    LOOP
    >r r_mesh{ r@ } f@ fswap 
    Nmesh @ r> 1+ DO
      fdup V_mesh{ I } f@ f< IF r_mesh{ I } f@ leave THEN
    LOOP
    fnip ;

variable Nlevels
create xpl XPT_SIZE 2* allot
    
: draw-levels ( -- )
    black foreground
    Nlevels @ 0 DO
      Ev{ I } f@  fdup find-rlims frot ftuck 
      scale-point xpl XPT_SIZE + XPoint!
      scale-point xpl XPoint!
      xpl 2 draw-lines
    LOOP
;

\ Set up storage for the wavefunctions
Nmesh @ MAX-LEVELS FLOAT matrix Ps{{
0 value vt
: save-soln ( v -- )
   to vt
   Nmesh @ 0 DO
     Pl{ I } F@  Pr{ Nmesh @ I - 1- } F@ 
     I mo @ > IF  fswap THEN  fdrop Ps{{ I vt }} f!
   LOOP
       
;

\ Draw the probability distributions for each wavefunction
Nmesh @ float array pd{ 

: calc-pd  ( v -- ) 
    to vt 
    Nmesh @ 0 DO  Ps{{ I vt }} f@ fsquare pd{ I } f!  LOOP
;

create xpd Nmesh @ XPT_SIZE * allot

: scale-pd ( v -- )
    to vt
    xpd Nmesh @ 0 DO
      dup r_mesh{ I } f@ pd{ I } f@ 
      100e f/ Ev{ vt } f@ f+
      scale-point  rot XPoint!
      XPT_SIZE +
    LOOP drop ;   

: draw-pd ( v -- )
    dup 2 mod IF blue ELSE red THEN foreground
    dup calc-pd  scale-pd  xpd Nmesh @ 4 / draw-lines ;
        
\ Draw the potential curve

create xpotnl XPT_SIZE Nmesh @ * allot

: scale-potnl ( -- )
	xpotnl Nmesh @ 0 DO
	   dup r_mesh{ I } f@ V_mesh{ I } f@ scale-point rot Xpoint!
	   XPT_SIZE +
        LOOP
        drop ;

: draw-potnl ( -- )
    black foreground
    scale-potnl
    xpotnl Nmesh @ draw-lines
;

: draw-h2X ( -- )
    get-window-size  s>f fheight f! s>f fwidth f!
    clear-window
    draw-potnl
    draw-levels
    Nlevels @ 0 DO  I draw-pd  LOOP
;

' draw-h2X IS redraw-window

\ A version of find-levels which saves the calculated wave 
\ functions, and does not deallocate arrays at the end of the calc.
: findlev2 ( -- )
    MAX-LEVELS Ev{ }fzero
    \ Find the energy of the lowest level (v=0)
    0 to v
    Vmin F@ start_dE solve ABORT" find-levels: Unable to find the lowest level!" 
    v save-soln
    fdrop fdup Ev{ v } F!
    fdup  Vmin F@ F- F+ 
    BEGIN
      start_dE solve
      0= IF
        v 1+ to v
        v save-soln
        \ The following energy increment works well to determine
        \   the trial value for the next eigenvalue
	fdrop fdup Ev{ v } F! 
        fdup  Ev{ v 1- } F@ F- 2e F/  
        F+
      ELSE
        fdrop
      THEN
      fdup V_inf F>
    UNTIL
    v Nlevels !
    fdrop
;

cr .( Finding bound levels of X state ... )
findlev2
2 1 simple-plot
end-solve


