\ henon-graphics.4th
\
\ Graphical demonstration of dynamics of the Henon Map near
\ a fixed point.
\
\ Copyright (c) 2012 Krishna Myneni, Creative Consulting for
\ Research and Education, http://ccreweb.org
\
\ Acknowledgements:
\   Thanks to J. N. Blakely for helpful comments and corrections
\   on this demo.
\
\ Revisions:
\   2012-04-19  km  first version.
\   2012-04-22  km  cleaned up frame9 through frame12 words.

include henon.4th
include asm
include strings
include lib-interface
include libs/x11/libX11
include simple-graphics-x11
include simple-frames-x11

Also simple-graphics-x11
Also simple-frames-x11

10000 float array x{
10000 float array y{

variable niterates

: gen-henon ( F: x y -- ) ( u -- )  \ ( rx ry u -- )
    niterates !
    niterates @ 0 do
      f2dup y{ I } f!  x{ I } f!
      H
    loop
    f2drop
;

fvariable ptsize

: draw-attractor ( F: ptsize -- ) ( color -- )  \ ( rptsize color -- )
    foreground
    fdup f0> IF
      ptsize f!
      niterates @ 0 do
        x{ I } f@ y{ I } f@
        f2dup in-window? IF
	  ptsize f@ put-filled-circle  
	ELSE  f2drop  
	THEN
      loop
    ELSE
      fdrop
      x{ 0 } y{ 0 } niterates @ point-plot
    THEN
;

: zoom1 ( -- )  -2e -2e 2e 2e set-window-limits ;
: frame1 ( -- )
    zoom1
    -1e -1e 6000 gen-henon
    0e red draw-attractor
    black foreground
    0.5e 1.7e s" Dynamics of The Henon Attractor Near the Fixed Point" put-text
    1.0e 1.5e s" x_n+1 = B*y_n - (x_n)^2 + A" put-text
    1.0e 1.3e s" y_n+1 = x_n" put-text 
;

: zoom2 ( -- ) xf f@ 1e f- fdup xf f@ 1e f+ fdup set-window-limits ;
: frame2 ( -- )
    solve_fixed_point set_fixed_point
    zoom2
    0.01e red draw-attractor
    green foreground
    xf f@ yf f@ 0.01e put-filled-circle
    blue foreground
    xf f@ yf f@ 0.05e put-circle
    xf f@ yf f@ 0.1e f+ 
    s" The fixed point on the Henon Map returns to itself after one iteration." put-text
;

: draw-zoomed-fixedpt ( -- )
    green foreground
    xf f@ yf f@ 0.003e put-filled-circle  \ identify the fixed point
    blue foreground
    xf f@ yf f@ 0.02e put-circle
;

: zoom3 ( -- )  xf f@ 0.2e f- fdup xf f@ 0.2e f+ fdup set-window-limits ;
: frame3 ( -- )
    zoom3
    -1e -1e 6000 gen-henon
    0.003e red draw-attractor
    draw-zoomed-fixedpt
    black foreground
    1.0e 1.0e s" Fixed Point:" put-text
    1.0e 0.98e s" xf = yf = " xf f@ 8 f>string count strcat put-text
    xf f@ yf f@ 0.99e 0.97e put-line
;

: frames4-common ( u -- )
    >r
    zoom3
    draw-zoomed-fixedpt
    black foreground
    xf f@ 0.01e f+ fdup s" radius = 0.02" put-text
    -1e -1e 6000 gen-henon
    0.003e grey draw-attractor
    red foreground
    xf f@ yf f@ r> gen-henon
    0.003e red draw-attractor
    black foreground
    xf f@ 0.04e f- fdup 0.2e f+ niterates @ s>string count 
    s"  iterates starting from fixed point" strcat put-text
;
    
: frame4 ( -- )   20 frames4-common ;
: frame5 ( -- )   50 frames4-common ;
: frame6 ( -- )  100 frames4-common ;
: frame7 ( -- ) 6000 frames4-common ;

: draw-fixedpt-evectors ( -- )
    xf f@ yf f@ Jacobian  eigenvectors
    2 1 set-line-type
    green foreground
    xf f@ yf f@  fover e{{ 0 0 }} f@ f+  fover e{{ 1 0 }} f@ f+ put-line
    xf f@ yf f@  fover e{{ 0 0 }} f@ f-  fover e{{ 1 0 }} f@ f- put-line
    red foreground
    xf f@ yf f@  fover e{{ 0 1 }} f@ f+  fover e{{ 1 1 }} f@ f+ put-line
    xf f@ yf f@  fover e{{ 0 1 }} f@ f-  fover e{{ 1 1 }} f@ f- put-line
    1 0 set-line-type
    black foreground
    xf f@ 0.03e f- yf f@ 0.10e f+ s" Stable Direction"     put-text
    xf f@ 0.11e f+ yf f@ 0.04e f- s" Unstable Direction"   put-text
;
    
: frame8 ( -- )
    zoom3
    -1e -1e 6000 gen-henon
    0.003e grey draw-attractor
    draw-zoomed-fixedpt
    draw-fixedpt-evectors
;

: draw-ball ( -- )
    red foreground
    360 0 do  ellipse{{ I 0 }} f@  ellipse{{ I 1 }} f@ 8e-4 put-filled-circle  loop
;

: frame9-common ( u -- )
    >r zoom3
    0.003e grey draw-attractor
    green foreground
    xf f@ yf f@ 0.003e put-filled-circle  \ identify the fixed point
    draw-fixedpt-evectors
    xf f@ yf f@ setup-ball
    r> 
    dup 0 ?do evolve-ball loop  draw-ball
    black foreground
    dup 0= if  drop s" Initial ball of points"  else  
      s>string count s" Ball after " 2swap strcat 
      s"  iteration(s)" strcat
    then
    2>r xf f@ 0.01e f+ fdup 2r> put-text
;

: frame9  ( -- )  0 frame9-common ;
: frame10 ( -- )  1 frame9-common ;
: frame11 ( -- )  2 frame9-common ;
: frame12 ( -- )  3 frame9-common ;


' frame1  ' frame2  ' frame3  ' frame4  ' frame5  ' frame6  ' frame7  ' frame8
' frame9  ' frame10 ' frame11 ' frame12
12 set-frames
start-frames


