\ henon-graphics.4th
\
\ Graphical demonstration of dynamics of the Henon Map near
\ a fixed point.
\
\  Copyright (c) 2012 Krishna Myneni, Creative Consulting for
\  Research and Education, http://ccreweb.org
\
\  This code may be used for any purpose, provided the copyright
\  notice above is included.
\
\ Acknowledgements:
\   Thanks to J. N. Blakely for helpful comments and corrections
\   on this demo.
\
\ Revisions:
\   2012-04-19  km  first version.
\   2012-04-22  km  cleaned up frame9 through frame12 words.
\   2012-04-26  km  added coordinate display
\   2012-04-28  km  loaded font support modules and set fonts for output
\   2012-04-29  km  provide pointer coordinate display in zoom3;
\                   cleaned up placement of text; 

include henon.4th
include asm
include strings
include lib-interface
include libs/x11/libX11
include font-strings-x11
include simple-graphics-x11
include simple-fonts-x11
include simple-frames-x11

Also simple-graphics-x11
Also simple-fonts-x11
Also simple-frames-x11

10000 float array x{
10000 float array y{

variable niterates

: gen-henon ( F: x y -- ) ( u -- )  \ ( rx ry u -- )
    niterates !
    niterates @ 0 do
      f2dup y{ I } f!  x{ I } f!
      H
    loop
    f2drop
;

fvariable ptsize

: draw-attractor ( F: ptsize -- ) ( color -- )  \ ( rptsize color -- )
    foreground
    fdup f0> IF
      ptsize f!
      niterates @ 0 do
        x{ I } f@ y{ I } f@
        f2dup in-window? IF
	  ptsize f@ put-filled-circle  
	ELSE  f2drop  
	THEN
      loop
    ELSE
      fdrop
      x{ 0 } y{ 0 } niterates @ point-plot
    THEN
;

: title-font   ( -- ) bold   regular 140 TextFonts1 select-font ;   
: content-font ( -- ) medium regular 140 TextFonts1 select-font ;
: coords-font  ( -- ) medium regular 120 TextFonts2 select-font ;

: zoom1 ( -- )  
    -2e -2e 2e 2e set-window-limits
    ['] default-pointermove-handler IS on-pointermove
;

: frame1 ( -- )
    zoom1
    -1e -1e 6000 gen-henon
    0e red draw-attractor
    black foreground
    title-font
    0e 1.8e uc>wc
    2dup s" Dynamics of The Henon Map"   draw-text
    current-font-height + s" Near the Fixed Point" draw-text
    medium italic 140 TextFonts1 select-font
    -1.4e 0.0e uc>wc
    2dup s" x_{n+1} = B*y_n - (x_n)^2 + A" draw-text
    current-font-height + s" y_{n+1} = x_n"    draw-text
    content-font 
;

: zoom2 ( -- ) 
    xf f@ 1e f- fdup xf f@ 1e f+ fdup set-window-limits
    ['] default-pointermove-handler IS on-pointermove
;

: frame2 ( -- )
    solve_fixed_point set_fixed_point
    zoom2
    0.01e red draw-attractor
    green foreground
    xf f@ yf f@ 0.01e put-filled-circle
    blue foreground
    xf f@ yf f@ 0.05e put-circle
    black foreground
    xf f@ yf f@ 0.3e f+ uc>wc
    2dup s" The fixed point on the Henon Map returns" draw-text
    current-font-height + s" to itself after one iteration." draw-text
;

: draw-zoomed-fixedpt ( -- )
    green foreground
    xf f@ yf f@ 0.003e put-filled-circle  \ identify the fixed point
    blue foreground
    xf f@ yf f@ 0.02e put-circle
;

\ Alternate pointer move handler
: show-coords ( -- )
    coords-font
    get-pointerinfo  ( -- x y state )
    drop wc>uc
    3 f>string >r 3 f>string count
    s" x: " 2swap strcat s"  y: " strcat r> count strcat  2>r
    white foreground  10 0 220 current-font-height 2+ draw-filled-rectangle
    black foreground  10 current-font-height 2r> draw-text
    content-font
;

: zoom3 ( -- )  
    xf f@ 0.2e f- fdup xf f@ 0.2e f+ fdup set-window-limits
    ['] show-coords IS on-pointermove
;

: frame3 ( -- )
    zoom3
    -1e -1e 6000 gen-henon
    0.003e red draw-attractor
    draw-zoomed-fixedpt
    black foreground
    0.9e 1.0e uc>wc 2dup s" Fixed Point:" draw-text
    medium regular 140 TextFonts2 select-font
    current-font-height + s" xf = yf = " 
    xf f@ 8 f>string count strcat draw-text
    xf f@ yf f@ 0.9e 0.97e put-line
    medium regular 140 TextFonts1 select-font
;

: frames4-common ( u -- )
    >r
    zoom3
    draw-zoomed-fixedpt
    black foreground
    xf f@ 0.01e f+ fdup s" radius = 0.02" put-text
    -1e -1e 6000 gen-henon
    0.003e grey draw-attractor
    red foreground
    xf f@ yf f@ r> gen-henon
    0.003e red draw-attractor
    black foreground
    xf f@ 0.04e f- fdup 0.2e f+ niterates @ s>string count 
    s"  iterates starting from fixed point" strcat put-text
;
    
: frame4 ( -- )   20 frames4-common ;
: frame5 ( -- )   50 frames4-common ;
: frame6 ( -- )  100 frames4-common ;
: frame7 ( -- ) 6000 frames4-common ;

: draw-evectors ( F: x y -- )
    f2dup Jacobian  eigenvectors
    2 1 set-line-type
    green foreground
    f2dup  fover e{{ 0 0 }} f@ f+  fover e{{ 1 0 }} f@ f+ put-line
    f2dup  fover e{{ 0 0 }} f@ f-  fover e{{ 1 0 }} f@ f- put-line
    red foreground
    f2dup  fover e{{ 0 1 }} f@ f+  fover e{{ 1 1 }} f@ f+ put-line
    f2dup  fover e{{ 0 1 }} f@ f-  fover e{{ 1 1 }} f@ f- put-line
    1 0 set-line-type
    black foreground
    f2drop
;

: label-eigenvectors ( -- )
    xf f@ 0.06e f- yf f@ 0.10e f+ s" Stable Direction"     put-text
    xf f@ 0.10e f+ yf f@ 0.04e f- s" Unstable Direction"   put-text
;
    
: frame8 ( -- )
    zoom3
    -1e -1e 6000 gen-henon
    0.003e grey draw-attractor
    draw-zoomed-fixedpt
    xf f@ yf f@ draw-evectors
    label-eigenvectors
;

fvariable ball_radius
: draw-ball ( F: radius -- )
    ball_radius f!
    red foreground
    360 0 do  
      ellipse{{ I 0 }} f@  ellipse{{ I 1 }} f@  
      ball_radius f@ put-filled-circle  
    loop
;

: frame9-common ( u -- )
    >r
    0.003e grey draw-attractor
    green foreground
    xf f@ yf f@ 0.003e put-filled-circle 
    xf f@ yf f@ draw-evectors
    label-eigenvectors
    xf f@ yf f@ setup-ball
    r> 
    dup 0 ?do evolve-ball loop  8e-4 draw-ball
    black foreground
    dup 0= if  drop s" Initial ball of points"  else  
      s>string count s" Ball after " 2swap strcat 
      s"  iteration(s)" strcat
    then
    2>r xf f@ 0.01e f+ fdup 2r> put-text
;

: frame9  ( -- )  zoom3  0 frame9-common ;
: frame10 ( -- )  zoom3  1 frame9-common ;
: frame11 ( -- )  zoom3  2 frame9-common ;
: frame12 ( -- )  zoom3  3 frame9-common ;

\ user-specified point
fvariable xs
fvariable ys

variable nevolve
0 nevolve !

\ Alternate button press handler
: bph1 ( -- )
  get-buttoninfo  wc>uc ys f! xs f!  drop
  0 nevolve !
  \ ." Selected coords: " xs f@ f. ys f@ f. cr
  clear-window
  redraw-window ;
' bph1 IS on-buttonpress    


: frame13-key-handler ( -- )
    get-keyinfo
    CASE
      XK_Left  OF  devolve-ball -1 nevolve +! ENDOF
      XK_Right OF  evolve-ball   1 nevolve +! ENDOF
    ENDCASE
    frame-nav
    get-current-frame 13 <> IF
      ['] frame-nav IS on-keypress
    THEN
;
    
: frame13 ( -- )
    zoom1 0.01e grey draw-attractor
    ['] frame13-key-handler IS on-keypress
    blue foreground
    xs f@ ys f@ 0.01e put-filled-circle
    xs f@ ys f@ draw-evectors
    nevolve @ 0 = IF xs f@ ys f@ setup-ball THEN
    1e-2 draw-ball
    black foreground
    -1.6e 0.4e uc>wc
    2dup s" Select a new point. Then press left or right" draw-text
    current-font-height + 
    s" arrows to evolve the ball" draw-text
;


' frame1  ' frame2  ' frame3  ' frame4  ' frame5  ' frame6  ' frame7  ' frame8
' frame9  ' frame10 ' frame11 ' frame12 ' frame13
13 set-frames
start-frames


