\ lkspec.4th
\
\ Calculate the time averaged optical spectrum for a Lang-Kobayashi system.
\
\ Copyright (c) 2002 Krishna Myneni, 2002-11-15
\
\ Requires:
\
\	matrix.4th
\	complex.4th
\	zmatrix.4th
\	sl.4th
\	lk.4th
\	fft1d.4th
\	noise.4th
\
\ Revisions:
\
\ Notes:
\
\  1. To compute and print the spectrum with the default
\     L-K parameters (see lk.4th), execute lkspec. The frequency,
\     centered about the optical frequency, is output in GHz
\     and the spectral intensity is an arbitrary scale. To
\     direct the output to a file, e.g. spec.dat, type
\
\	>file spec.dat lkspec console
\
\  2. To increase the frequency resolution, increase NPTS.
\     Always use a power of 2 (4096, 8192, etc.) for NPTS.
\     The frequency range is controlled by the time step, ds.
\
\  3. You may also increase the number of averages (nave) to 
\     see the results of measurement for different integration times.
\

include lk
include fft1d
include noise

8192 constant NPTS
  
NPTS 1 zmatrix E	\ array for recent E and fft
NPTS 2 fmatrix spec	\ averaged power spectrum
NPTS 2/ 1 fmatrix tmp   \ use for reordering spectrum

fvariable min_freq	\ minumum of the spectrum frequency
fvariable freq_step	\ frequency step for the spectrum

variable nave		\ number of averages
100 nave !
variable sindex


: lkspec ( -- )
	E  zmat_zero
	lk_init
	1.9e 1.1e 0.1e V sv!      \ initialize the state vector
	V z@ !Epast

	\ Compute 100000 normalized time steps to go past the transient;
	\ change flag to TRUE for output of data

	100000 FALSE lksteps  

	spec fmat_zero

	\ Compute NAVE sets of NPTS time steps and the average power spectrum

	nave @ 0 do
	  1 sindex !
	  NPTS 10 * 1+ 1 do
	    V dV_lk/ds Vdot sv!
	    V ds f@ Vdot rk4 V sv!
	    V z@ !Epast
	    i 1- 10 mod 0= if		\ sample at 10*ds 
	      V z@ sindex @ 1 E zmat! 1 sindex +!
	    then
	  loop

	  \ Compute the complex fft of the set of NPTS points.

	  NPTS 2* 1 E mat_size!   \ we must resize the matrix for fft 

	  \ The inverse FFT is used because of our choice of sign in
	  \   factoring the optical frequency out of the laser equations.

	  E inv_fft 

	  NPTS 1 E mat_size!	\ restore the matrix size
	  
	  \ Accumulate time averaged spectrum

	  NPTS 1+ 1 do 
	    i 1 E zmat@ |z|^2
	    i 2 spec fmat@ f+ i 2 spec fmat!	
	  loop
	loop

	\ Re-order the spectrum from min frequency to max frequency

	NPTS 2/ 1+ 1       do  i 2 spec fmat@  i 1 tmp fmat!  loop
	NPTS 1+ NPTS 2/ 1+ do  i 2 spec fmat@  i NPTS 2/ - 2 spec fmat!  loop  
	NPTS 2/ 1+ 1       do  i 1 tmp  fmat@  i NPTS 2/ + 2 spec fmat!  loop 

	\ Compute a frequency scale and print out the spectrum

	dt f@ 10e f* 2e f* -1e fswap f/ min_freq f!
	dt f@ 10e f* NPTS s>f f* 1e fswap f/ freq_step f!

	min_freq f@ freq_step f@ f-
	NPTS 1+ 1 do 
	  freq_step f@ f+ fdup 1e9 f/ i 1 spec fmat!    \ freq in GHz
	  i 2 spec fmat@ nave @ s>f f/  i 2 spec fmat!  \ power is arbitrary
	loop fdrop
	
	spec fmat.
;

lk_init
separator cr
.( Spectral calculation parameters: ) cr
separator cr
.( Number of averages [nave] = ) tab nave @ . cr
.( Time segment              = ) tab 
   dt f@ 10e f* NPTS s>f f* 1e-9 f/ f. .(  ns) cr
.( Sampling interval         = ) tab dt f@ 10e f* 1e-9 f/ f. .( ns) cr
cr

