\ poetry-reader.4th
\
\ Displays poems from sequentially numbered text files.
\
\  Copyright (c) 2012 Krishna Myneni, Creative Consulting for
\  Research and Education, http://ccreweb.org
\
\  This code may be used for any purpose, provided the copyright
\  notice above is included.
\
\ Notes:
\
\ 1) Poem file names start with a prefix, configurable using
\    $FILE_PREFIX (see below), concatenated with a 3-digit 
\    index number starting at 001.
\
\ 2) All poem files are located in a specified directory, 
\    $POEMS_DIR (see below).
\
\ 3) The output fonts are configurable -- see POETRY_FONT and
\    the list of common fonts in font-strings-x11.4th
\
\ 4) Currently, the program doesn't handle poems longer than
\    will fit in the window height. It may be generalized to
\    do so, easily.
\
\ Revisions:
\   2012-04-30  km  first version.

include ans-words
include modules
include asm
include strings
include files
include utils
include lib-interface
include libs/x11/libX11
include font-strings-x11
include simple-graphics-x11
include simple-fonts-x11
include simple-typeset-x11
include simple-frames-x11

Also simple-graphics-x11
Also font-strings-x11
Also simple-fonts-x11
Also simple-typeset-x11
Also simple-frames-x11

s" poems/" $constant $POEMS_DIR
s" poem"   $constant $FILE_PREFIX

\ Set the desired text font family below

FONT_TIMES_NEW_ROMAN constant POETRY_FONT

\ Load the desired poetry font and check to see if
\ we were successful. If not, fall back to the
\ default text fonts.

0 ptr myFonts

: set-poetry-font ( -- )
    POETRY_FONT  ExtraFonts  load-font-table
    0 ExtraFonts @font-entry drop
    IF  ExtraFonts  ELSE  TextFonts1  THEN
    to myFonts
    
    \ perform other graphics setup
    extra-graphics-setup
;

: title-font   ( -- ) bold   regular 240 myFonts select-font ;   
: author-font  ( -- ) bold   italic  180 myFonts select-font ;
: poem-font    ( -- ) medium regular 140 myFonts select-font ;

10 constant EOL
create title    256 allot
create author   256 allot
create poemline 256 allot

\ Parse subsequent poem text
: poem
    clear-window 
    EOL parse title pack  refill drop
    EOL parse author pack
    title-font  1 title count place-centered-text
    author-font 3 author count place-centered-text
    poem-font
    5      \ start line (in new font) from top of window
    BEGIN
      dup
      refill drop
      EOL parse poemline pack
      poemline count s" fin" compare
    WHILE
      poemline count place-centered-text
      1+
    REPEAT
    2drop
;

variable poem-number
1 poem-number !

: previous-poem ( -- ) poem-number @ 1- 1 max poem-number ! ;
: next-poem     ( -- ) 1 poem-number +! ;

\ Return the current poem file name
: poem-file-name  ( -- caddr u )
    $POEMS_DIR $FILE_PREFIX strcat
    poem-number @ 0 <# # # # #> strcat s" .txt" strcat
;

: poem-file-exists? ( -- flag )  poem-file-name strpck file-exists ;

: key-handler ( -- )
    get-keyinfo
    CASE
      XK_Home      OF  1 poem-number !      ENDOF
      XK_Page_Up   OF  previous-poem        ENDOF
      XK_Page_Down OF  next-poem poem-file-exists?
                       0= IF  previous-poem  THEN
                       ENDOF
      XK_Escape    OF  exit-simple-graphics EXIT ENDOF
    ENDCASE
    redraw-window
;

: frame1 ( -- ) 
    poem-file-exists? IF poem-file-name included  ELSE  2drop  THEN
;


' frame1
1 set-frames

\ override default handlers provided by simple-frames-x11
' key-handler IS on-keypress
' set-poetry-font IS user-graphics-init
2 1 simple-graphics



