\ simple-frames-x11.4th
\
\ A minimal framework for the presentation of sequential x11 graphics,
\ generated as individual frames by user-specified code, and traversable
\ by the user.
\
\ Copyright (c) 2012, K. Myneni, Creative Consulting for
\   Research and Education
\
\ Notes:
\
\ 0) The user app provides a series of words, e.g. frame1 frame2 ... ,
\    each of which performs the graphics to draw a single frame. To
\    present these frames in the desired sequence, the application
\    first uses SET-FRAMES :
\
\       ' frame1  ' frame2  ' frame3 ...
\       n SET-FRAMES
\
\    where n is the number of frames to be presented. To start the
\    presentation, the user application executes START-FRAMES
\
\       START-FRAMES
\
\    The simple-frames module allows the user to navigate between
\    the frames using the [Page Up] and [Page Down] keys. Also, the
\    [Home] and [End] keys may be used to go to the first and last
\    frames, and the [Escape] key is used to exit the presentation.
\
\ 1) To replace or chain key handling from the user application,
\    define a key-handler in the app, e.g.
\
\      : key-handler ( -- )
\          get-keyinfo
\          CASE
\            XK_A  OF ... ENDOF
\            \ etc.
\          ENDCASE
\       
\          frame-nav  ( optional, to chain the handlers)
\      ;
\
\    Set the key handler in the user app, after SET-FRAMES
\    and before START-FRAMES , e.g.
\
\        ... n SET-FRAMES
\        ' key-handler IS on-keypress
\        START-FRAMES
\
\
\ Revisions:
\   2012-04-19  km  created
\   2012-04-20  km  defined !FRAME and @FRAME
\   2012-04-25  km  made FRAME-NAV a public member to allow
\                     for key handler chaining; added notes.

true value  XK_MISCELLANY
include libs/x11/keysymdef

Module: simple-frames-x11

Also simple-graphics-x11

Begin-Module

1 value first_frame
1 value last_frame

variable current_frame
first_frame current_frame !

variable pFrames  \ the frames list

: !frame ( xt u -- )
    1- cells pFrames a@ + ! ;

: @frame ( u -- xt )
    dup first_frame last_frame 1+ within
    IF  1- cells pFrames a@ + a@  ELSE  drop 0  THEN
;

: free-frame-list ( -- )
    pFrames a@ free drop 
    0 pFrames ! ;

Public:

: frame-nav ( -- )
    get-keyinfo
    CASE
      XK_Page_Up    OF  -1 current_frame +!  ENDOF
      XK_Page_Down  OF   1 current_frame +!  ENDOF
      XK_Home       OF   first_frame current_frame !  ENDOF
      XK_End        OF   last_frame  current_frame !  ENDOF
      XK_Escape     OF   free-frame-list  exit-simple-graphics  EXIT  ENDOF
    ENDCASE
    current_frame @ first_frame max last_frame min 
    dup current_frame ! @frame IS redraw-window
    clear-window
    redraw-window
;

: set-frame-range ( first last -- )
    to last_frame 1 max to first_frame ;

: set-frames ( xt1 xt2 ... xtn n -- )
    dup cells allocate ABORT" Unable to allocate frames list!"
    pFrames !
    1 over set-frame-range
    1 swap ?do I !frame -1 +loop
    1 @frame IS redraw-window
    ['] frame-nav IS on-keypress
;

: start-frames ( -- )
    pFrames @ 0= ABORT" No frames setup!"
    2 2 simple-graphics
;

End-Module

