\ simple-plot.4th
\
\ A simple X11 plot window utility, for making simple point and line plots.
\
\ Provides:
\
\  redraw-window  vector which must be initialized by the user program
\  clear-window      ( -- )
\  get-window-size   ( -- uwidth uheight )
\  foreground        ( pixval -- ) 
\  draw-points       ( apoints u -- )
\  draw-lines        ( apoints u -- )
\  simple-plot       ( widthscale heightscale -- )
\  flush-window      ( -- )
\
\  Defined pixvals: red green blue black white
\
\ K. Myneni, 2011-08-26
\
\ Revisions:
\   2011-08-27  km  added draw-lines
\
include ans-words
include asm
include strings
include lib-interface
include libX11


0 value screen_num     \ number of screen to place the window on.
0 value win            \ window ID of the newly created window.
0 value root_win       \ parent window of new window
0 value gc             \ GC (graphics context) used for drawing in our window.
0 value width
0 value height         \ height and width for the new window.

0 value colormap
variable display       \ pointer to X Display structure.
variable gcvalues
0 value colormap

XColor col_exact
0 value red
0 value green
0 value blue
0 value black
0 value white

: get-window-size ( -- u ) width height ;

\ Allocate a color
: get-color ( addr u -- pixel )
    $>zstr display @ colormap rot col_exact XParseColor
    IF
      display @ colormap col_exact XAllocColor drop
      col_exact XColor->pixel @
    ELSE 
      cr ." Failed to lookup color"
      black
    THEN
;

\ Set the foreground color
: foreground ( pixelval -- )
    >r display @ gc r> XSetForeground drop ;

XEvent event


\ Default window drawing word
\
\ redraw-window must be re-vectored by the calling application.

defer redraw-window

: clear-window ( -- ) display @ win XClearWindow drop ;
' clear-window is redraw-window  \ default vector

: flush-window ( -- ) display @ XFlush drop ;

: draw-points ( apoints u -- )
	2>r display @ win gc 2r> CoordModeOrigin XDrawPoints drop ;

: draw-lines ( apoints u -- )
	2>r display @ win gc 2r> CoordModeOrigin XDrawLines drop ;

\ ws and hs are integer width and height scale factors for the plot
\ window: 1 means full width of the display, 2 means half, etc.

: simple-plot ( ws hs -- )
    2>r 			
    \ open connection with the X server.
    s" :0.0" $>zstr XOpenDisplay dup display !
    0= IF
      cr ." cannot connect to X server" cr
      ABORT
    THEN

    \ get the default colormap and window size for our display.
    display @ XDefaultScreen to screen_num
    display @ screen_num
    2dup  XDisplayWidth  2r@ drop / to width 
    2dup  XDisplayHeight 2r> nip  / to height
    2dup  XBlackPixel to black
    2dup  XWhitePixel to white
    2dup  XDefaultColormap to colormap
    2dup  XRootWindow to root_win
    2drop
    s" blue"   get-color to  blue
    s" red"    get-color to  red
    s" green"  get-color to  green

    display @  root_win  0 0  width height  0  black  white
    XCreateSimpleWindow  to win

    \ Obtain MapNotify, Key press, Button press, Window resize, and
    \ Window destroy events
    display @ win
    ExposureMask KeyPressMask or StructureNotifyMask or
    XSelectInput drop

    \ make the window actually appear on the screen. 
    display @ win XMapWindow drop

    \ allocate a new GC (graphics context) for drawing in the window.
    display @ win 0 gcvalues XCreateGC to gc
    gc 0< ABORT" Unable to create the graphics context!"
    display @ gc FillSolid  XSetFillStyle drop

    \ flush all pending requests to the X server.
    display @ XFlush drop
    display @ false XSync drop

    \ Event loop
    BEGIN
      display @ event XNextEvent drop
      event @ 
      CASE
        Expose        OF  event XExposeEvent->count @ 0= IF redraw-window THEN
	                   ENDOF
        ConfigureNotify OF 
                     event XConfigureRequestEvent->width @ to width
                     event XConfigureRequestEvent->height @ to height
                  ENDOF
      ENDCASE
      event @ Keypress =
    UNTIL

    \ close the connection to the X server.
    display @ gc XFreeGC drop
    display @ XCloseDisplay drop
;


