\ simple-plot.4th
\
\ A simple X11 plot window utility, for making simple point and line plots.
\
\ Provides:
\
\ General:
\  redraw-window  vector which must be initialized by the user program
\  clear-window      ( -- )
\  get-window-size   ( -- uwidth uheight )
\  get-window-limits ( F: -- xmin ymin xmax ymax )
\  set-window-limits ( F: xmin ymin xmax ymax -- )
\  foreground        ( pixval -- )
\  set-line-type     ( thickness style -- )
\ 
\ Window Coordinate Drawing Words: 
\  draw-points       ( apoints u -- )
\  draw-lines        ( apoints u -- )
\  draw-point        ( x y -- )
\  draw-line         ( x1 y1 x2 y2 -- )
\  draw-text         ( x y caddr u -- )
\
\ User Coordinate Drawing Words:
\  point-plot        ( ax ay u -- )
\  line-plot         ( ax ay u -- )
\  put-point         ( F: x y -- )
\  put-line          ( F: x1 y1 x2 y2 -- )
\  put-text          ( F: x y -- ) ( caddr u -- ) or ( x y caddr u -- )
\
\  simple-plot       ( widthscale heightscale -- )
\  flush-window      ( -- )
\
\  Defined pixvals: red green blue black white
\
\ Requires:
\   ans-words.4th
\   modules.fs
\   asm.4th
\   strings.4th
\   lib-interface.4th
\   libs/x11/libX11.4th
\
\ K. Myneni, 2011-08-26
\
\ Revisions:
\   2011-08-27  km  added draw-lines
\   2012-04-09  km  ported to modules framework; added words
\                   to provide scaling from user coords to window coords;
\                   added:  GET-WINDOW-LIMITS  SET-WINDOW-LIMITS
\                   POINT-PLOT  LINE-PLOT  PUT-POINT  PUT-LINE
\   2012-04-12  km  added DRAW-POINT  DRAW-LINE  DRAW-TEXT  PUT-TEXT
\                   SET-LINE-TYPE


Module: simple-plot-x11
Begin-Module

0 value screen_num     \ number of screen to place the window on.
0 value win            \ window ID of the newly created window.
0 value root_win       \ parent window of new window
0 value gc             \ GC (graphics context) used for drawing in our window.
0 value width
0 value height         \ height and width for the new window.

0 value colormap
variable display       \ pointer to X Display structure.
variable gcvalues
0 value colormap

XColor col_exact

Public:

0 value red
0 value green
0 value blue
0 value black
0 value white

: get-window-size ( -- u ) width height ;

\ Allocate a color
: get-color ( addr u -- pixel )
    $>zstr display @ colormap rot col_exact XParseColor
    IF
      display @ colormap col_exact XAllocColor drop
      col_exact XColor->pixel @
    ELSE 
      cr ." Failed to lookup color"
      black
    THEN
;

Private:

fvariable xmin
fvariable ymin
fvariable xmax
fvariable ymax

fvariable xdel
fvariable ydel

-1e xmin f!
-1e ymin f!
 1e xmax f!
 1e ymax f!
 2e xdel f!
 2e ydel f!

fvariable fheight
fvariable fwidth

: scale-to-window ( F: x y -- ) ( -- ux uy )
    ymin f@ f-  ydel f@ f/ 1e fswap f- fheight f@ f* fround>s >r
    xmin f@ f-  xdel f@ f/             fwidth  f@ f* fround>s r> 
;

Public:

\ Get the current window coordinates
: get-window-limits ( F: -- xmin ymin xmax ymax )
    xmin f@  ymin f@  xmax f@  ymax f@ ;

\ Set the window coordinates
: set-window-limits ( F: xmin ymin xmax ymax -- )
    ymax f!  xmax f!  ymin f!  xmin f!
    ymax f@  ymin f@  f-  ydel f!
    xmax f@  xmin f@  f-  xdel f!
;

\ Set the foreground color
: foreground ( pixelval -- )
    >r display @ gc r> XSetForeground drop ;

\ Set the line attributes
\ style: 0 = solid, 1 = dash, 2 = double dash
: set-line-type ( thickness style -- )
    2 min 2>r
    display @ gc 2r> CapButt JoinBevel XSetLineAttributes drop ;

Private:
XEvent event

Public:

\ Default window drawing word
\
\ redraw-window must be re-vectored by the calling application.

defer redraw-window

: clear-window ( -- ) display @ win XClearWindow drop ;
' clear-window is redraw-window  \ default vector

: flush-window ( -- ) display @ XFlush drop ;

\ Drawing in window coordinates; apoints is a pointer to a
\ series of u XPoint structures.
: draw-points ( apoints u -- )
    2>r display @ win gc 2r> CoordModeOrigin XDrawPoints drop ;

: draw-lines ( apoints u -- )
    2>r display @ win gc 2r> CoordModeOrigin XDrawLines drop ;

: draw-point ( x y -- )
    2>r display @ win gc 2r> XDrawPoint drop ;

: draw-line ( x1 y1 x2 y2 -- )
    2>r 2>r display @ win gc 2r> 2r> XDrawLine drop ;

: draw-text ( x y caddr u -- )
    2>r 2>r display @ win gc 2r> 2r> XDrawString drop ;
     
\ Drawing in user coordinates

Private:
XPoint% %size constant XPT_SIZE
: XPoint! ( nx ny apoint -- )  rot over XPoint->x w! XPoint->y w! ;

variable np
variable pPoints

: @point ( ax ay i -- ) ( F: -- ax[i]  ay[i] )
    floats dup >r + swap r> + >r f@ r> f@ fswap ;

\ Transform user coordinates to window coordinates for a set of points.
: transform-points ( ax ay u -- )
    np !
    np @ 0= IF  2drop EXIT  THEN
    np @ XPT_SIZE * allocate ABORT" allocate error!"
    pPoints !
    get-window-size s>f fheight f!  s>f fwidth f!
    np @ 0 DO 
      2dup I @point  scale-to-window
      pPoints a@ I XPT_SIZE * + XPoint!
    LOOP
    2drop
;

Public:

: point-plot ( ax ay u -- )
    transform-points
    pPoints a@ np @ draw-points
    pPoints a@ free drop
;

: line-plot ( ax ay u -- )
    transform-points
    pPoints a@ np @ draw-lines
    pPoints a@ free drop
;

: put-point ( F: x y -- )
    scale-to-window 2>r
    display @ win gc 2r> XDrawPoint drop
;

: put-line ( F: x1 y1 x2 y2 -- ) 
    scale-to-window 2>r  scale-to-window 2>r
    display @ win gc 2r> 2r> XDrawLine drop
;

: put-text ( F: x y -- ) ( caddr u -- )  \ ( x y caddr u -- )
    2>r scale-to-window 2>r 
    display @ win gc 2r> 2r> XDrawString  drop ;

\ ws and hs are integer width and height scale factors for the plot
\ window: 1 means full width of the display, 2 means half, etc.

: simple-plot ( ws hs -- )
    2>r 			
    \ open connection with the X server.
    s" :0.0" $>zstr XOpenDisplay dup display !
    0= IF
      cr ." cannot connect to X server" cr
      ABORT
    THEN

    \ get the default colormap and window size for our display.
    display @ XDefaultScreen to screen_num
    display @ screen_num
    2dup  XDisplayWidth  2r@ drop / to width 
    2dup  XDisplayHeight 2r> nip  / to height
    2dup  XBlackPixel to black
    2dup  XWhitePixel to white
    2dup  XDefaultColormap to colormap
    2dup  XRootWindow to root_win
    2drop
    s" blue"   get-color to  blue
    s" red"    get-color to  red
    s" green"  get-color to  green

    display @  root_win  0 0  width height  0  black  white
    XCreateSimpleWindow  to win

    \ Obtain MapNotify, Key press, Button press, Window resize, and
    \ Window destroy events
    display @ win
    ExposureMask KeyPressMask or StructureNotifyMask or
    XSelectInput drop

    \ make the window actually appear on the screen. 
    display @ win XMapWindow drop

    \ allocate a new GC (graphics context) for drawing in the window.
    display @ win 0 gcvalues XCreateGC to gc
    gc 0< ABORT" Unable to create the graphics context!"
    display @ gc FillSolid  XSetFillStyle drop

    \ flush all pending requests to the X server.
    display @ XFlush drop
    display @ false XSync drop

    \ Event loop
    BEGIN
      display @ event XNextEvent drop
      event @ 
      CASE
        Expose        OF  event XExposeEvent->count @ 0= IF redraw-window THEN
	                   ENDOF
        ConfigureNotify OF 
                     event XConfigureRequestEvent->width @ to width
                     event XConfigureRequestEvent->height @ to height
                  ENDOF
      ENDCASE
      event @ Keypress =
    UNTIL

    \ close the connection to the X server.
    display @ gc XFreeGC drop
    display @ XCloseDisplay drop
;

End-Module

