\ fft1d.4th
\
\ Compute Fourier transform of a 1 dimensional matrix of complex data
\
\ Notes:
\
\	The number of rows in the input matrix must be a power of 2.
\	Assumes the use of matrix.4th to create the 1D fmatrix.
\
\ Requires:
\
\	matrix.4th
\
\ Revisions:
\	8-10-2001 created from fft.4th to optimize fft calculation
\	            for 1D arrays. Required mods to @a_n and !a_n only. KM
\
\	12-14-2002 ported to Forths with a separate fp stack. Also
\	             cleaned up the code with use of VALUEs  KM


2 cells constant DOFFSET
0 value nvals
0 value fft_arr

\ Note: origin is 1 for @a_n and !a_n

: @a_n ( n -- | fetch the n^th element in the matrix fft_arr )
       ( F: -- f )
	1- floats fft_arr + DOFFSET + f@ ; 

: !a_n ( n -- | store f in the n^th element of matrix fft_arr )
       ( F: f -- )
 	1- floats fft_arr + DOFFSET + f! ;

0 value jj
0 value mm
0 value mmax
0 value istep

fvariable wr
fvariable wi
fvariable wpr
fvariable wpi

: fft ( a -- | replace a, which is a 1-D fmatrix, with FFT )
	dup to fft_arr 
	mat_size@ drop to nvals

	1 to jj

	nvals 1+ 1 do

	  jj i > if		\ exchange two complex numbers
	    jj @a_n
	    jj 1+ @a_n
	    i @a_n jj !a_n
	    i 1+ @a_n jj 1+ !a_n
	    i 1+ !a_n
	    i !a_n	
	  then

	  nvals 2/ to mm

	  begin
	    mm 2 >= jj mm > and
	  while
	    mm negate jj + to jj
	    mm 2/ to mm
	  repeat

	  mm jj + to jj
	2 +loop

	2 to mmax

	begin
	  nvals mmax >
	while
	  mmax dup 2* to istep
	  6.28318530717959e0 s>f f/
	  fdup
	  0.5e f* fsin fdup f* -2e f* wpr f!
	  fsin wpi f!
	  1e wr f!
	  0e wi f!
	  
	  mmax 1+ 1 do
	    nvals 1+ i do
	      i mmax + to jj
	      jj @a_n wr f@ f* 
	      jj 1+ @a_n wi f@ f*
	      f-
	      jj 1+ @a_n wr f@ f*
	      jj @a_n wi f@ f*
	      f+			
	      fover
	      i @a_n fswap f- jj !a_n
	      i 1+ @a_n fover f- jj 1+ !a_n
	      i 1+ @a_n f+ i 1+ !a_n
	      i @a_n f+ i !a_n
	    istep +loop
	    
	    wr f@
	    fdup fdup wpr f@ f* f+ wi f@ wpi f@ f* f- wr f!
	    wpi f@ f* wi f@ wpr f@ f* f+ wi f@ f+ wi f!
	  2 +loop
	  
	  istep to mmax
	repeat
;


 

: inv_fft ( a -- | replace a with inverse FFT )
	dup to fft_arr
	mat_size@ drop to nvals

	1 to jj

	nvals 1+ 1 do

	  jj i > if		\ exchange two complex numbers
	    jj @a_n
	    jj 1+ @a_n
	    i @a_n jj !a_n
	    i 1+ @a_n jj 1+ !a_n
	    i 1+ !a_n
	    i !a_n	
	  then

	  nvals 2/ to mm

	  begin
	    mm 2 >= jj mm > and
	  while
	    mm negate jj + to jj
	    mm 2/ to mm
	  repeat

	  mm jj + to jj
	2 +loop

	2 to mmax

	begin
	  nvals mmax >
	while
	  mmax dup 2* to istep
	  6.28318530717959e0 negate s>f f/
	  fdup
	  0.5e f* fsin fdup f* -2e f* wpr f!
	  fsin wpi f!
	  1e wr f!
	  0e wi f!
	  
	  mmax 1+ 1 do
	    nvals 1+ i do
	      i mmax + to jj
	      jj @a_n wr f@ f* 
	      jj 1+ @a_n wi f@ f*
	      f-
	      jj 1+ @a_n wr f@ f*
	      jj @a_n wi f@ f*
	      f+			
	      fover
	      i @a_n fswap f- jj !a_n
	      i 1+ @a_n fover f- jj 1+ !a_n
	      i 1+ @a_n f+ i 1+ !a_n
	      i @a_n f+ i !a_n
	    istep +loop
	    
	    wr f@
	    fdup fdup wpr f@ f* f+ wi f@ wpi f@ f* f- wr f!
	    wpi f@ f* wi f@ wpr f@ f* f+ wi f@ f+ wi f!
	  2 +loop
	  
	  istep to mmax
	repeat
;
 	  
	
