\ polyfit.4th   
\                                                               
\  Polynomial fitting routine in Forth (for Forths with a separate fp stack)
\
\ Adapted from the routine polfit in P.R. Bevington,
\ "Data Reduction and Error Analysis for the Physical Sciences"
\
\ Requires the following source files:
\
\	matrix.4th
\
\ Revisions:
\              
\	10-02-98 Adapted for LabForth KM             
\	03-29-99 Adapted for kForth KM  
\       01-01-02 Cleaned up the code (no functional changes)  KM
\	12-15-02 Ported to Forths with a separate fp stack  KM
\ 
0 value nterms     \ number of terms (order - 1)               
0 value params     \ address of parameter matrix         
0 value nmax                                                   
0 value npts
                                                   
fvariable chisq                                                 
fvariable xterm                                                 
fvariable yterm                                                 
fvariable delta                                                 
                                                                
\ Floating point matrices used by polfit                        
                                                                
10 10 fmatrix apfit
19  1 fmatrix sumx
10  1 fmatrix sumy


\ x = address of fmatrix containing x values                    
\ y = address of fmatrix containing y values                    
\ a = address of fmatrix for receiving fitted parameters        
\ n = order of fitting polynomial                               
                                                                
: polfit ( x y a n -- | perform polynomial fit )     
	 ( F: -- chi-square )
    1+ to nterms                                                 
    to params                                                    
    nterms dup apfit mat_size!                               
    nterms 1 sumy mat_size!                                  
    nterms 1 params mat_size!                              
    nterms 2* 1- to nmax                                       
    nmax 1 sumx mat_size!                                    
    dup mat_size@ drop to npts                                 
    sumx fmat_zero                                              
    sumy fmat_zero                                              
    params fmat_zero                                          
    0e chisq f!                                                 

    npts 1+ 1 do                                                 
      2dup                                                      
      i 1 rot fmat@
      i 1 rot fmat@                                          
      1e xterm f!                                               

      nmax 1+ 1 do                                               
        xterm f@                                                
        i 1 sumx fmat_addr                                 
        dup f@ f+ f!                                          
        fdup xterm dup                                           
        f@ f* f!                                          
      loop                                                      

      fover yterm f!                                            

      nterms 1+ 1 do                                             
        yterm f@                                                
        i 1 sumy fmat_addr                                
        dup f@ f+ f!                                          
        fdup yterm dup                                           
        f@ f* f!                                          
      loop                                                      

      fover fdup f*
      chisq dup f@ f+ f!                                   
      fdrop fdrop                                               
    loop                                                        

    2drop
                                                       
    nterms 1+ 1 do                                               
      nterms 1+ 1 do                                             
        i j + 1- 1 sumx fmat@                                   
        j i apfit fmat!                                   
      loop                                                      
    loop                                                        

    apfit determ 

    fdup delta f!                                  
    f0= if 0e exit then
                                   
    nterms 1+ 1 do                                               
      nterms 1+ 1 do                                             
        nterms 1+ 1 do                                           
          i j + 1- 1 sumx fmat@                                 
          j i apfit fmat!                                 
        loop                                                    
        i 1 sumy fmat@                                       
        i j apfit fmat!                                   
      loop                                                      
      apfit determ
      delta f@ f/                                  
      i 1 params fmat!                                     
    loop                                                        

\ Calculate chi-squared                                         

    nterms 1+ 1 do                                               
      chisq f@ i 1 params fmat@                            
      i 1 sumy fmat@ f* 2e f*                                
      f- chisq f!                                               
      nterms 1+ 1 do                                             
        j i + 1- 1 sumx fmat@                                   
        i 1 params fmat@ f*                                
        j 1 params fmat@ f*                                
        chisq f@ f+ chisq f!                                    
      loop                                                      
    loop
                                      
    chisq f@                                                    
    npts nterms - s>f                                       
    f/ ;
