/*
xyplot.c

XYPLOT for X Windows

Copyright (c) 1995 Creative Consulting for Research & Education
All Rights Reserved.

Last Revised: 9/11/95

*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "xyplot.h"


char* color_list[] = {"blue", "red", "dark green", "cyan", "magenta",
  "brown", "yellow", "grey", "black"};


/* X Windows global vars */

Display* display;
Window win, term_win;
GC gc;					/* graphics context */
Cursor cursor;

DeviceContext gdc;
DisplayContext di;
Database db;
PlotMessage mb [MAX_MESSAGES];

int nMessages;
int WKSaved;



int main (int argc, char** argv)
{
	DeviceContext sc;	/* window/screen device context */
	PlotMessage pm, pm2;
	Dataset* ds [MAX_SETS];
	Plot* pv [MAX_PLOTS];
	WindowLimits wl [MAX_LEVELS];

	int j, npts, ns, np, width, height;
	int ix, iy;
	int ResetBase, ResetWin, ReDraw;
	int* buf;
	char fname [255];
	Plot* p1;
	Plot* p2;
	Dataset* d1;
	Dataset* d2;
	

	CopyrightNotice();
		
	/* Initialization */
	
	pm.pBuf = NULL;
	db.nSets = 0;
	db.pSets = ds;
	di.pPlots = pv;
	di.pWl = wl;
	NewWorkspace();	

	/* Set up the display device context, open it, and make it the global dc */

	sc.dev_type = DEV_X_WINDOW;
	j = OpenDisplay (&sc);
	if (j) {
	  ErrorDisplay(j);
	  return 0;
	}
	
	/* Input file specified on command line? */
	
	if (argc > 1) {
	  pm.message = PL_OPEN;
	  pm.pBuf = malloc(255);
	  strcpy ((char *) pm.pBuf, argv[1]);
	  pm.nParam = 1;
	  PostPlotMessage (&pm);
	}
	
	

/*---------------Message loop----------------------*/

	while (GetPlotMessage (&pm)) {
	  if (pm.pBuf) buf = (int *) pm.pBuf;
	  if (pm.message == PL_KEYPRESS) pm.message = TranslateKey (*buf);
          
          p1 = di.ap;       	/* active plot */
          p2 = di.op;          	/* operand plot */
          
          if (p1) 
            d1 = p1->set;    	/* selected data set 1 */
          else
            d1 = NULL;
          
          if (p2)    
            d2 = p2->set;     	/* second set for two-set operations */
	  else
	    d2 = NULL;
	    
          ResetBase = NO;
          ResetWin = NO;
          ReDraw = NO;
	  
	  switch (pm.message) {
	    case PL_NONE:
	      break;
	    case PL_CURSORMOVE:
	      UpdateCursorCoords (&sc, pm.x, pm.y);
	      break;
	    case PL_RESIZE:
	      width = *buf++;
	      height = *buf;
	      ResizeWin (&sc, width, height);
	      ReDraw = YES;
	      break;
	    case PL_BUTTONPRESS:
	      ExpansionBox (pm.x, pm.y, &ix, &iy);
	      /* Expand (pm.x, pm.y, ix, iy); */
	      ReDraw = YES;
	      break;
	    case PL_HELP:
	      RaiseWindow(term_win);
	      ShowHelp ();
	      break;
	    case PL_RESET:
	      ResetBase = YES;	/* force extrema and window reset */
	      ResetWin = YES;
	      break;
	    case PL_REDRAW:
	      ReDraw = YES;	/* force redraw */
	      break;
	    case PL_PICK:
	      RaiseWindow(term_win);
	      /* DatasetMenu (); */
	      break;
	    case PL_DELETESET:
	      DeleteSet (d1);
	      ResetBase = YES;
	      ResetWin = YES;
	      if (db.nSets == 0) {
	        /* PostPlotMessage (PL_NEW); */
	       ;
	      }
	      break;
	    case PL_DUP:
	      /* Duplicate (d1); */
	      break;
	    case PL_JOIN:
	      /* JoinSets (d1, d2); */
	      ResetBase = YES;
	      break;
	    case PL_PRUNE:
	      /* PruneSet (d1); */
	      ResetBase = YES;
	      break;
	    case PL_PALETTE:
	      /* select color */
	      break;
	    case PL_TEMPLATE:
	      /* Template (); */
	      ResetBase = YES;
	      ResetWin = YES;
	      break;
	    case PL_AREA:
	      /* Area (d1); */
	      ReDraw = NO;
	      break;
	    case PL_BIN:
	      /* Bin (d1); */
	      ResetBase = YES;
	      ResetWin = YES;
	      break;
	    case PL_DERIV:
	      /* Deriv (d1); */
	      ResetBase = YES;
	      ResetWin = YES;
	      break;
	    case PL_POLY:
	      /* PolynomialFit (d1); */
	      break;
	    case PL_SORT:
	      /* Sort (d1); */
	      break;
	    case PL_SWAPXY:
	      /* SwapXY (d1); */
	      ResetBase = YES;
	      ResetWin = YES;
	      break;
	    case PL_MAG:
	      /* Mag (d1); */
	      break;
	    case PL_PHASE:
	      /* Phase (d1); */
	      break;
	    case PL_CONJ:
	      /* Conj (d1); */
	      break;
	    case PL_REALY:
	      /* CmpToReal (d1, -1); */
	      break;
	    case PL_IMAGY:
	      /* CmpToReal (d1, 0); */
	      break;
	    case PL_CMPLY:
	      /* RealToCmp (d1); */
	      break;
	    case PL_ZERO:
	      /* Zero (d1); */
	      break;
	    case PL_ESCAPE:          
              if (di.level > 0) {
                --di.level;
	      }
              else {
                ResetBase = YES;
              }
              ReDraw = YES;
	      break;
	    case PL_BASE:
              ResetBase = YES;
	      break;
	    case PL_EXIT:
              CloseDisplay (&sc);
	      FreeWorkspace ();
	      exit (0);
	      break;
	    case PL_EXPAND:
	      RaiseWindow (term_win);
	      Expand ();
	      break;
	    case PL_FILE:
	      RaiseWindow(term_win);
	      if (db.nSets == MAX_SETS) {
	        ErrorDisplay (PL_E_MAXSETS); 
	      }
	      else {
	        printf ("\nEnter the file name >> ");
	        gets (fname);
	        pm2.pBuf = malloc(255);
	        strcpy ((char *) pm2.pBuf, fname);
	        pm2.message = PL_OPEN;
	        pm2.nParam = 1;
	        PostPlotMessage (&pm2);
	      }
	      break;
	    case PL_OPEN:
	      j = OpenFile (&d1, (char *) pm.pBuf);
	      if (j) {
	        ErrorDisplay (j);
	      }
	      else {
	        InsertSet (d1);
	        DisplayHeader (d1);
	        p1 = CreatePlot (d1, 'L');
	        InsertPlot(p1);
	        ResetBase = YES;
	        ReDraw = YES;
	      }
	      break;  
	    case PL_GRID:
              if (di.grid) {
                di.grid = NO;
                ReDraw = YES;
              }
	      else {
                di.grid = YES;
                Grid (&gdc);
              }
	      break;
	    case PL_ASPECT:
	      break;
	    case PL_MOUSETOGGLE:
	      break;
	    case PL_DROP:
	      DestroyPlot (p1);
	      if (di.nPlots == 0) {
	        pm2.message = PL_PICK;
	        pm2.pBuf = NULL;
	        pm2.nParam = 0;
	        PostPlotMessage (&pm2);
	      }
	      else {
	        ResetBase = YES;
	        ReDraw = YES;
	      }
	      break;
	    case PL_SCALE:
              if ((di.scale == 1) && (di.pWl[di.level].ymin <= 0)) {
                ErrorDisplay (PL_E_LOG);
              }
	      else {
                if (di.scale == 1) {
	          di.scale = 0;
	        }
	        else {
	          di.scale = 1;
	        } 
              } 
	      break;
	    case PL_FFT:
	      /* FFT (d1, wParam); */
	      ResetBase = YES;
	      ResetWin = YES;
	      break;
	    case PL_RESCALE:
	      /* Scale (d1); */
	      ResetBase = YES;
	      break;
	    case PL_SMOOTH:
	      /* Smooth (d1); */
	      ResetBase = YES;
	      break;
	    case PL_ARITHMETIC:
	      /* PlotArithmetic (d1, d2, wParam); */
	      ResetBase = YES;
	      break;
	    case PL_HEADER:
	      DisplayHeader (d1);
	      RaiseWindow (term_win);
	      break;
	    case PL_SAVEWORK:
	      /* SaveWork (); */
	      break;
	    case PL_SAVEWK1:
	      /* SaveWK1 (); */
	      break;
	    case PL_SAVETEX:
	      /* SaveTeX (); */
	      break;
	    case PL_SAVEPS:
	      /* SavePS (); */
	      break;
	    case PL_SAVEDATA:
	      /* SaveData (d1); */
	      break;
	    case PL_SETSAVEOPTS:
	      /* SetSaveOptions (); */
	      break;
	    case PL_VIEWFILE:
	      /* FileView (); */
	      break;
	    case PL_MENU:
	      /* ShowMenu ((char *) pm->pBuf); */
	      break;
	    case PL_POINT:
	      p1->sym = 'p';
	      break;
	    case PL_LINE:
	      p1->sym = 'l';
	      break;
	    case PL_DATACURSOR:
	      /* DataCursor (p1); */
	      break;
	    case PL_NEW:
	      NewWorkspace ();
	      ResetWin = YES;
	      break;
	    case PL_ERR:
	      ResetBase = YES;
	      ResetWin = YES;
	      break;
	    default:
	      ;
	  }
	  
	  /* Perform redraw and reset functions */

	  if (ResetBase) FullWindow();
	  
	  if (ResetWin) {
	    di.level = 0;
	    ReDraw = YES;
	  }
	  
	  if (ReDraw) {
	     MakePlotDisplay (&gdc);
	     /* RaiseWindow (win); */
	  }
	        
	  /* done with current plot message; destroy it */
	  
	  DestroyPlotMessage (&pm);
	}
	
}
/*-----------------end of main----------------------------------*/

void CopyrightNotice ()
{
	printf ("\nXYPLOT for X Windows v%s", VERSION);
	printf ("\nCopyright (c) 1995 Creative Consulting for ");
	printf ("Research & Education");
	printf ("\nAll Rights Reserved.\n");
}
/*--------------------------------------------------------------*/

int TranslateKey (int keycode)
{
/* Map a key code into a plot message code */

	switch (keycode) {
	  case XK_D:
	  case XK_d:
	    return PL_DROP;
	  case XK_E:
	  case XK_e:
	    return PL_EXPAND;
	  case XK_F:
	  case XK_f:
	    return PL_FILE;
	  case XK_G:
	  case XK_g:
	    return PL_GRID;
	  case XK_H:
	  case XK_h:
	    return PL_HEADER;  
	  case XK_I:
	  case XK_i:
	    return PL_PICK;    
	  case XK_L:
	  case XK_l:
	    return PL_SCALE;    
	  case XK_N:
	  case XK_n:
	    return PL_NEW;
	  case XK_Q:
	  case XK_q:
	    return PL_EXIT;
	  case XK_X:
	  case XK_x:
	    return PL_DELETESET;
	  case XK_F1:
	    return PL_HELP;
	  case XK_Escape:
	    return PL_ESCAPE;
	  default:
	    return PL_NONE;
	}  	              
}
/*------------------------------------------------------------*/

int FreeWorkspace ()
{
/* Free all dynamically allocated objects */

	PlotMessage pm;
	
	while (db.nSets > 0) DeleteSet (db.pSets[0]);
	while (di.nPlots > 0) DestroyPlot (di.pPlots[0]);
	while (nMessages) {
	  GetPlotMessage (&pm);
	  DestroyPlotMessage (&pm);
	}
	return 0;
}
/*-----------------------------------------------------------*/

int NewWorkspace ()
{
	FreeWorkspace ();

	di.level = 0;
	di.scale = 0;
	di.ap = NULL;
	di.op = NULL;
	di.pWl[0].xmin = -1.;
	di.pWl[0].xmax = 1.;
	di.pWl[0].ymin = -1.;
	di.pWl[0].ymax = 1.;
	
	return 0;
}
/*-----------------------------------------------------------*/

void RaiseWindow (Window w)
{
	if (gdc.dev_type == DEV_X_WINDOW) {
	  XRaiseWindow (display, w);
	  XSetInputFocus (display, w, RevertToParent, CurrentTime);
	  XFlush (display);
	}

}
/*-----------------------------------------------------------*/

void ErrorDisplay (int ecode)
{
	printf ("\nXYPLOT ERROR: %s", PL_ERROR_MESSAGE[ecode]);
	RaiseWindow(term_win);
}
/*-----------------------------------------------------------*/

void DisplayHeader (Dataset* d)
{
	printf ("\nName: %s  Points: %d  Type: %s", d->name,
	  d->npts, "Real");
	printf ("\n=============================================");
	printf ("%s", d->hdr);
	printf ("\n=============================================\n");
}
/*-----------------------------------------------------------*/

double dmin (double x1, double x2)
{
/* Return minimum of x1 and x2 */

	if (x1 < x2)
	  return x1;
	else
	  return x2;
}

double dmax (double x1, double x2)
{
/* Return maximum of x1 and x2 */
	
	if (x1 > x2)
	  return x1;
	else
	  return x2;
}


int IndexLimits (double* x, double x1, double x2, int n, int* j1, int* j2)
{
/*
  IndexLimits
  Purpose: Determine the indices of values falling in the interval (x1, x2).
  Syntax: i = IndexLimits (x, x1, x2, npts, &j1, &j2);
  Parameters:
    x     -- ptr to array of values
    x1    -- lower bound
    x2    -- upper bound
    n     -- number of vals in array
    j1    -- ptr to lower index
    j2    -- ptr to upper index
  Return:        
	0 -- no error
        1 -- (x1, x2) interval does not overlap interval (x[js], x[je])
  Author: K. Myneni
  Date: 16 August 90
  Notes:
         The x array values may be in
         increasing or decreasing order. The indices, *j1
         and *j2 will always be returned with *j1 less than *j2,
         regardless of the ordering of x values.
         Translated to QuickBasic on 25 Aug 90. Subsequently
         translated to C.
*/

	double xx1, xx2;
	int j;

/* Make sure x limits are used in order of minimum, maximum. */

	if (x2 < x1) {
	   xx1 = x2;
	   xx2 = x1;
	}
	else {
	   xx1 = x1;
	   xx2 = x2;
	}

/* Search for index of lower limit */

	if (x[n-1] > x[0]) {
           for (j = 0; j < n; j++) {
	      if (x[j] >= xx1) {
		 *j1 = j;
		 goto c1;
	      }
	   }
	}   
	else {
           for (j = n-1; j >= 0; j--) {
	      if (x[j] >= xx1) {
		 *j1 = j;
		 goto c1;
	      }
	   }
	}

/* x limits outside domain of data */

	return 1;

c1:

/* Search for index of upper limit */

	if (x[n-1] > x[0]) {
           for (j = n-1; j >= 0; j--) {
	      if (x[j] <= xx2) {
		 *j2 = j;
		 goto c2;
	      }
	   }
	}   
	else {
           for (j = 0; j < n; j++) {
	      if (x[j] <= xx2) {
		 *j2 = j;
		 goto c2;
	      }
	   }
	}

/* x limits outside domain of data */

	return 1;

c2:

/* Put indices in order */

	if (j2 < j1) {
	  j = *j1;
	  *j1 = *j2;
	  *j2 = j;
	}
	return 0;

}
/*-------------------------------------------------------------------*/

int OpenDisplay (DeviceContext* dc)
{
	
	switch (dc->dev_type) {
	  case DEV_NULL:
            break;
          case DEV_VGA:
	    break;
	  case DEV_X_WINDOW:
	    return (xOpenDisplay (dc));
	    break;  
          case DEV_LATEX:
	    break;
          case DEV_PS:
	    break;
          default:
            return PL_E_GDUNKNOWN;
        }
	return 0;
}
/*--------------------------------------------------------------------*/
int CloseDisplay (DeviceContext* dc)
{
	switch (dc->dev_type) {
	  case DEV_NULL:
	    break;
	  case DEV_VGA:
	    break;
	  case DEV_X_WINDOW:
	    xCloseDisplay(dc);
	    break;
	  case DEV_LATEX:
	  case DEV_PS:
	    break;
	  default:
	    return PL_E_GDUNKNOWN;
	}
	return 0;              
}
/*-------------------------------------------------------------*/

void SetDisplayContext ()
{
	int l, xd, yd;
	double xmin, xmax, ymin, ymax;

        GetDisplayLimits (&xmin, &ymin, &xmax, &ymax);

        if (di.scale == 2) {               /* polar scale */
          xmax = fabs (ymax);
          xmin = -xmax;
          ymin = xmin;
        }

        xd = gdc.x2 - gdc.x1;
        yd = gdc.y2 - gdc.y1;
        di.xsf = ((double) xd) / (xmax - xmin);
        di.ysf = ((double) yd) / (ymax - ymin);

        
        if ((di.pWl[0].ymin > 0.) && (ymin > 0.)) {
          di.ylsf = ((double) yd) / (log(ymax) - log(ymin));
          di.lymin = log(ymin);
	}
        else {
          if (di.scale == 1) di.scale = 0;
        }
	
}
/*--------------------------------------------------------------*/

void SetDeviceContext (DeviceContext *dc)
{
	dc->ncpix = 8;
	dc->nrpix = 8;
	dc->x1 = 8*dc->ncpix;
	dc->y1 = 5*dc->nrpix/2;
	dc->x2 = dc->xres - 3*dc->ncpix;
	dc->y2 = dc->yres - 3*dc->nrpix;
	dc->xd = dc->x2 - dc->x1;
	dc->yd = dc->y2 - dc->y1;
	dc->yinv = dc->y2;
	
	/* Update the global device context */
	
	gdc = *dc;
}
/*-------------------------------------------------------------*/

int OpenGraphicsDevice (DeviceContext* dc)
{
	
	switch (dc->dev_type) {
	  case DEV_NULL:
            break;
          case DEV_VGA:
	    /* vga_setmode(dc->mode); */
	    break;
	  case DEV_X_WINDOW:
	    XClearWindow (display, win);
	    break;  
          case DEV_LATEX:
            dc->fp = fopen (dc->dev_name, "w");
	    if (dc->fp == NULL) return (1);
            fprintf (dc->fp, "\% LaTeX picture generated by XYPLOT\n");
            fprintf (dc->fp, "\\setlength{\\unitlength}{.25pt}\n");
            fprintf (dc->fp, "\\begin{picture}(1200,1200)\n");
	    break;
          case DEV_PS:
            dc->fp = fopen (dc->dev_name, "w");
	    if (dc->fp == NULL) return (1);
            fprintf (dc->fp, "\%!PS-Adobe-2.0 EPSF-1.2\n");
            fprintf (dc->fp, "%%BoundingBox: %u %u %u %u", 0, 0, 612, 792);
            fprintf (dc->fp, "%%EndComments\n");
            fprintf (dc->fp, "%%EndProlog\n");
            fprintf (dc->fp, "%%Page: %u %u", 1, 1);
            fprintf (dc->fp, "/Times-Roman findfont 50 scalefont setfont\n");
            fprintf (dc->fp, "0.25 0.25 scale\n");
            fprintf (dc->fp, "1 setlinewidth\n");
            fprintf (dc->fp, "0 setgray\n");
	    break;
          default:
            return (2);
        }
	return (0);
}
/*------------------------------------------------------------*/

int xOpenDisplay (DeviceContext *dc)
{
	char* display_name = NULL;
	char* window_name = "xyplot";
	XTextProperty windowName;
	XGCValues values;
	int revert;
	
	/* Connect to X server */

	if ((display = XOpenDisplay(display_name)) == NULL) {
	  return PL_E_GDCANTOPEN;
	}
	XGetInputFocus (display, &term_win, &revert);
	dc->mode = DefaultScreen(display);
	dc->xres = DisplayWidth(display, dc->mode)/2;
	dc->yres = DisplayHeight(display, dc->mode)/3;
	dc->invert = YES;
	
	SetDeviceContext (dc); 

	/* Create opaque window */
	
	win = XCreateSimpleWindow(display, RootWindow(display, dc->mode),
	  0, 0, dc->xres, dc->yres, 4, BlackPixel(display,
	  dc->mode), WhitePixel(display, dc->mode));
	
	XStringListToTextProperty (&window_name, 1, &windowName);
	XSetWMName (display, win, &windowName);
	
	/* Select event types wanted */
	
	XSelectInput (display, win, ExposureMask | KeyPressMask |
	  ButtonPressMask | ButtonReleaseMask | 
	  StructureNotifyMask | PointerMotionMask);
	  
	/* Create graphics context for text and drawing */
	
	gc = XCreateGC (display, win, 0L, &values);
	
	/* Display window */
	
	XMapWindow (display, win);
	    
	/* Use the pre-defined crosshair cursor */
	    
	cursor = XCreateFontCursor (display, XC_crosshair);
	XDefineCursor (display, win, cursor);
	    
	return 0;
}
/*------------------------------------------------------------*/

int xCloseDisplay ()
{
	XFreeGC (display, gc);
	XCloseDisplay (display);
	
	return 0;
}
/*------------------------------------------------------------*/

int CloseGraphicsDevice (DeviceContext* dc)
{
	XRectangle r;
	
        switch (dc->dev_type) {
          case DEV_NULL:
	    break;
	  case DEV_VGA:
	    break;  
	  case DEV_X_WINDOW:
	    XSetForeground (display, gc, BlackPixel(display, dc->mode));
	    r.x = 0;
	    r.y = 0;
	    r.width = dc->xres;
	    r.height = dc->yres;
	    XSetClipRectangles (display, gc, 0, 0, &r, 1, 0);
	    break;
          case DEV_LATEX:
            fprintf (dc->fp, "\\end{picture}\n");
            fclose (dc->fp);
	    break;
          case DEV_PS:
            fprintf (dc->fp, "showpage\n");
            fclose (dc->fp);
	    break;
          default:
            fclose (dc->fp);
	}
	return 0;
}
/*----------------------------------------------------------------*/

int GetPlotMessage (PlotMessage* pm)
{

/*
Retrieve (and remove) a message from the message queue; if no
message is present, wait for it in the appropriate device context
*/
 
	int j;
	
	if (nMessages > 0) {
	  *pm = mb[0];
	  for (j = 0; j < nMessages - 1; j++) mb[j] = mb[j + 1];
	  --nMessages;
	}
	else {
	  switch (gdc.dev_type) {
	    case DEV_NULL:
	      pm->message = PL_NONE;
	      break;
	    case DEV_X_WINDOW:
	      xGetPlotMessage (pm);
	      break;
	    default:
	      pm->message = PL_NONE;
	  }
	}  
	return TRUE;
}
/*---------------------------------------------------------------*/

int PostPlotMessage (PlotMessage* pm)
{
/* Place the plot message into the message queue */

	if (nMessages < MAX_MESSAGES) {
	  mb[nMessages] = *pm;
	  ++nMessages;
	  return 0;
	}
	return TRUE;
}
/*--------------------------------------------------------------*/

int DestroyPlotMessage (PlotMessage* pm)
{
/* free the message parameter buffer and reset plot message vals */

	if (pm->pBuf) free (pm->pBuf);
	pm->pBuf = NULL;
	pm->nParam = 0;
	pm->message = PL_NONE;
	
	return TRUE;
}
/*--------------------------------------------------------------*/

int xGetPlotMessage (PlotMessage* pm)
{
    XEvent report;
    KeySym key;
    XComposeStatus compose;
    char cl [255];
    int* buf;

    DestroyPlotMessage (pm);
    
    XNextEvent (display, &report);
    
    pm->x = report.xmotion.x;
    pm->y = report.xmotion.y;
        
    switch (report.type) {
      case Expose:
        return pm->message = PL_REDRAW;
      case DestroyNotify:
        return pm->message = PL_EXIT;
      case ConfigureNotify:
        pm->nParam = 2;
        pm->pBuf = malloc(2*sizeof(int));
        buf = (int *) pm->pBuf;
        *buf++ = report.xconfigure.width;
        *buf = report.xconfigure.height;
        return pm->message = PL_RESIZE;
      case MotionNotify:
        return pm->message = PL_CURSORMOVE;
      case ButtonPress:
        return pm->message = PL_BUTTONPRESS;
      case KeyPress:
        pm->nParam = 1;
        pm->pBuf = malloc(2*sizeof(int));
        buf = (int *) pm->pBuf;
        XLookupString (&report.xkey, cl, 64, &key, &compose);
        *buf = (int) key;
	return pm->message = PL_KEYPRESS;
      default:
        return pm->message = PL_NONE;	
    }
}
/*---------------------------------------------------------------*/

void UpdateCursorCoords (DeviceContext* dc, int x, int y)
{
    double xx, yy;
    char cl [255];
    	
    Coordinates (&xx, &yy, &x, &y, -1); 
    sprintf (cl, "%10.4lf %10.4lf", xx, yy);

    switch (dc->dev_type) {
      case DEV_NULL:
	break;
      case DEV_X_WINDOW:
    	XClearArea (display, win, dc->x2 - 16*dc->ncpix, 0,
      	  21*dc->ncpix, 3*dc->nrpix/2, 0);
    	XDrawString (display, win, gc, dc->x2 - 16*dc->ncpix,
      	  3*dc->nrpix/2, cl, strlen(cl));
	break;
      default:
	;
    }  

}
/*---------------------------------------------------------------*/

int ResizeWin (DeviceContext* dc, int width, int height)
{
	dc->xres = width;
	dc->yres = height;
	SetDeviceContext (dc);	
	return 0;
}

/*---------------------------------------------------------------*/
void GetDisplayLimits (double* xmin, double* ymin, double* xmax,
  double* ymax)
{
    int l = di.level;
    *xmin = di.pWl[l].xmin;
    *ymin = di.pWl[l].ymin;
    *xmax = di.pWl[l].xmax;
    *ymax = di.pWl[l].ymax;
}
/*---------------------------------------------------------------*/

Dataset* CreateSet (double* x, double* y, long npts, int dtype,
  char* name, char* hdr)
{
/*
Create a new data set based on the x, y buffers; Return
a pointer to the new set; return NULL if not successful
*/
	long nvals, nbytes;
	double* dptr;
	Dataset* d;
	char* name_buf;
	char* hdr_buf;
	
        d = (Dataset*) malloc (sizeof(Dataset));
        
        switch (dtype) {
          case DS_2DREAL:
            nvals = 2L*npts;
            break;
          case DS_2DCOMPLEX:
            nvals = 3L*npts;
            break;
          case DS_3DREAL:
            nvals = 3L*npts;
            break;
          case DS_3DCOMPLEX:
            nvals = 5L*npts;
            break;
          case DS_2DVECTOR:
            nvals = 4L*npts;
            break;
          case DS_3DVECTOR:
            nvals = 6L*npts;
            break;
          default:
            return NULL;
            break;
        }
        nbytes = nvals*((long)sizeof(double));
        dptr = (double *) malloc (nbytes);
	if (dptr == NULL) return NULL;
	name_buf = (char *) malloc (NAME_LENGTH);
	hdr_buf = (char *) malloc (HEADER_LENGTH);
	
	if (hdr_buf == NULL) {
	  free (name_buf);
	  free (dptr);
	  free (d);
	  return NULL;
	}
	
        d->pData = dptr;
        d->data_type = dtype;
        d->npts = npts;
        d->name = name_buf;
        d->hdr = hdr_buf;
	d->d.x = NULL;
	d->d.y = NULL;
	d->d.z = NULL;
	d->d.ix = NULL;
	d->d.iy = NULL;
	d->d.iz = NULL;
	d->d.Ax = NULL;
	d->d.Ay = NULL;
	d->d.Az = NULL;

	d->d.x = dptr;
	d->d.y = dptr + npts;
	nbytes = npts*((long)sizeof(double));
	memcpy (d->d.x, x, nbytes);
	memcpy (d->d.y, y, nbytes);
	
        switch (dtype) {
	  case DS_2DCOMPLEX:
	    d->d.iy = d->d.y + npts;
	    memcpy (d->d.iy, y + npts, nbytes);
	    break;
	  case DS_3DREAL:
	    d->d.z = d->d.y + npts;
	    memcpy (d->d.z, y + npts, nbytes);
	    break;
	  case DS_3DCOMPLEX:
	    d->d.z = d->d.y + npts;
	    d->d.iz = d->d.z + npts;
	    break;
	  case DS_2DVECTOR:
	    d->d.Ax = d->d.y + npts;
	    d->d.Ay = d->d.Ax + npts;
	    break;
	  case DS_3DVECTOR:
	    d->d.Ax = d->d.z + npts;
	    d->d.Ay = d->d.Ax + npts;
	    d->d.Az = d->d.Ay + npts;
	    break;
	  default:
	    ;
	}
	
        strcpy (d->name, name);
        strcpy (d->hdr, hdr);

        Extrema (d);
        WKSaved = FALSE;

        return d;
}
/*-------------------------------------------------------------*/

int InsertSet (Dataset* d)
{
/*
Insert a new data set into the database. Return the new
set number, or -1 if error;
*/

	int ns;

        if (d == NULL) return -1;
        
        if (db.nSets == MAX_SETS) {
          ErrorDisplay (PL_E_MAXSETS);		/* too many data sets */
          return -1;
        }
	
	ns = db.nSets;
	db.pSets[ns] = d;
	++db.nSets;
	

	return ns;

}
/*---------------------------------------------------------------*/

void DeleteSet (Dataset* d)
{
/* Delete data set ns from database */

	/* Free the dataset and any associated plots */
	
	int i, j, nsets, nplots;
	
	if (d == NULL) return;
	
	nsets = db.nSets;
	
	for (i = 0; i < db.nSets; i++) {
	  if (db.pSets[i] == d) {
            for (j = i; j < db.nSets - 1; j++) {
              db.pSets[j] = db.pSets[j+1];
            }
	    --nsets;
	  }  
	}
	
	i = 0;
	do {
	  if (di.pPlots[i]->set == d) {
	    DestroyPlot (di.pPlots[i]);
	  }
	  else {
	    ++i;
	  }  
	} while (i < di.nPlots) ;
	 
	free (d->pData);
	free (d->name);
	free (d->hdr);
	free (d);
	
        db.nSets = nsets;
	printf ("\nData set deleted!");
        WKSaved = FALSE;

}
/*-------------------------------------------------------------*/

int OpenFile (Dataset** d, char* name)
{
/*
 Open a data file, read x, y data, and return error code;
 0 = no error, in which case d will point to the dataset
*/

	char hdr[HEADER_LENGTH];
	int npts, dtype;
	double *x, *y;
	
	x = malloc (2*MAX_POINTS*sizeof(double));
	if (x == NULL) return PL_E_MEMALLOC;
	y = x + MAX_POINTS;
	
	if (strlen(name) == 0) return PL_E_BADFILE;
	
	npts = xyread (x, y, &dtype, name, hdr);
	
	if (npts < 0) {
	  free (x);
	  return -npts;
	}
	
	*d = CreateSet (x, y, npts, dtype, name, hdr);
	Extrema (*d);
	
	free (x);
	return 0;
	
}
/*------------------------------------------------------------------------*/

int xyread (double* x, double* y, int* dtype, char* fname, char* hdr)
{
/*
Reads data from an ASCII data file.  Accepts 1-, 2- or 3-column data,
or calls SelectColumns to let user choose 1, 2 or 3 columns out of
4 or more columns.
*/

	double a [32];
   	FILE* fp; 		
   	int nc; 	/* number of columns to be used */
   	int ncols; 	/* number of columns in data file */
   	int npts; 	/* number of data points read */
   	int c [6], k, j;
   	char ln [255];
   	int x_col, y_R_col, y_I_col;
	int x_end, y_end;
	
   	if ((fp = fopen (fname, "r")) == NULL) return -PL_E_FILENOTFOUND;

   	j = 0;
	
   	ReadHeader (fp, hdr, ln);
	
	ncols = NumberParse (a, ln);
   	nc = ncols;
	
   	switch (nc) {
      	  case 0:
	    /* There's no valid data in this line. */
	    return -PL_E_NODATA;
      	  case 1:
	    y_R_col = 1;
	    break;
      	  case 2:
	    x_col = 1;
	    y_R_col = 2;
	    break;
      	  default:
	    /* Ask user which columns to use:  */
	    
	    nc = SelectColumns (c, ncols);
	      
	    if (nc < 1 || nc > 3) return -PL_E_NODATA;
	      
	    x_col = c [0];
	    y_R_col = c [1];
	    y_I_col = c [2];
	     
   	}

/* Read 1, 2, or 3 columns of data from data file. */

   	switch (nc) {
      	  case 1:
	    *dtype = DS_2DREAL;
	    k = 1;
	    x [j] = k;
	    y [j] = a [y_R_col - 1];
	    ++j;
	    while (! feof(fp)) {
	      fgets (ln, MAX_LEN, fp);
	      if (! BlankLine (ln)) {
	       	ncols = NumberParse (a, ln);
	       	if (ncols < y_R_col) goto l1;
		x [j] = (double) ++k;
		y [j] = a [y_R_col - 1];
		++j;
	      }
	    }
	    break;

      	  case 2:
	    *dtype = DS_2DREAL;
	    x [j] = a [x_col - 1];
	    y [j] = a [y_R_col - 1];
	    ++j;
	    while (! feof (fp)) {
	      fgets (ln, MAX_LEN, fp);
	      if (! BlankLine (ln)) {
	        ncols = NumberParse (a, ln);
	        if (ncols < y_R_col) goto l1;
		x [j] = a [x_col - 1];
		y [j] = a [y_R_col - 1];
		++j;
	      }
	    }
	    break;

      	  case 3:
	    *dtype = DS_2DCOMPLEX;
	    x_end = j;
	    y_end = j;
	    x [x_end] = a [x_col - 1];
	    y [y_end] = a [y_R_col - 1];
	    ++y_end;
	    y [y_end] = a [y_I_col - 1];
	    ++x_end;
	    ++y_end;
	    while (! feof (fp)) {
	      fgets (ln, MAX_LEN, fp);
	      if (! BlankLine (ln)) {
	        ncols = NumberParse (a, ln);
	        if (ncols < y_R_col) goto l2;
		x [x_end] = a [x_col - 1];
		y [y_end] = a [y_R_col - 1];
		++y_end;
		y [y_end] = a [y_I_col - 1];
		++x_end;
		++y_end;
	      }
	    }

/*
Reorganize (this part of) the array so real parts of x and y
are adjacent, and imaginary parts are adjacent.
*/
l2:
	    SortComplex1 (x, y, 0, x_end);

	    j = y_end;
	    break;

          default:
	    break;
   	}
l1:
   	fclose (fp);
	
   	npts = j;
   	if (*dtype == DS_2DCOMPLEX) npts = npts/2;

   	return npts;

}

/*--------------------------------------------------------------*/

int BlankLine (char* ln)
{
/* Return TRUE if the line is blank. */

	int l;
	
	l = strlen(ln);

	if (l == 0) return TRUE;

	if (l == 1 && *ln == '\n') return TRUE;
	if (strspn (ln, " ") >= l) return TRUE;  /* all spaces */

    	return FALSE;
}

/*-------------------------------------------------------------*/
int NumberParse (double* a, char* s)
{
/*
This is a C routine to parse a string (s) into an array (a) of
double numbers. Returns the number of values found.
*/
	int n;
	static char* delim = "\t, ";
	char *substr;

	n = 0;
	substr = s;
	
	while (*substr) {
	  if (*substr == ' ' || *substr == '\t' || *substr == '\n') {
	    ;
	  }
	  else {
	    a[n] = strtod (substr, NULL);
	    ++n;
	    substr = strpbrk (substr, delim);
	    if (substr == NULL) return n;
	  }
	  ++substr;
	}

   	return n;
}

/*------------------------------------------------------------------------*/

int ReadHeader (FILE* fp, char* hdr, char* ln)
{
/*
Reads the header of a data file.  The header must be enclosed by
HEADER_BEGIN and HEADER_END.
*/

   	char *p;
   	*hdr = '\0';

/* Find beginning of file in stream.  */

   	fseek (fp, 0L, SEEK_SET);

/* Ignore blank lines at beginning of file.  */

   	do {
      	  fgets (ln, MAX_LEN, fp);
   	} while (BlankLine (ln) && (!feof(fp)));

/* Check whether first non-blank line is the beginning of a comment.  */ 
   
   	p = strstr (ln, HEADER_BEGIN);
   
   	if (p) {
      	  strcpy (ln, p + 2);
      	  strcpy (hdr, ln);
      	  strcat (hdr, "\n");
      
      	  do {
	    fgets (ln, MAX_LEN, fp);
	    p = strstr (ln, HEADER_END);
	    
	    if (p)
	      *p = '\0';
	    else
	      strcat (ln, "\n");
	 
	    strcat (hdr, ln);
      	  } while ((!p) && (!feof(fp)));
      
      	  fgets (ln, MAX_LEN, fp);
   	}
   	else {

/* The first line wasn't the beginning of a header.  */
/* Return the_line as it is.  */
      	  return FALSE;
   	}

/*
Read blank lines after the header to position at first
line of data
*/
      
   	while (BlankLine (ln) && (!feof(fp))) fgets (ln, MAX_LEN, fp);
    
   	return TRUE;
}

/*------------------------------------------------------------------------*/

int SelectColumns (int* c, int n)
{
/*
Prompts user to choose which of 4 or more columns to use as
input data.

Parameters:
  n 	--  number of columns in the data file
  c	--  columns chosen for [x,] Re(y) [, Im(y)]
Returns the number of columns selected.
 */
   
   	double a[MAX_NUMS];
   	char v[MAX_LEN];
   	int nc;

   	printf ("\nFile contains %d columns; ", n);
   	printf (" please specify columns to load: ");
   	
   	gets (v);
   
   	if (*v == CR) return 0;
   
   	nc = NumberParse (a, v);
   	if (nc > 3) {
      	  nc = 3;
      	  printf ("Three columns will be used.\n");
   	}
   	
   	switch (nc) {
      	  case 1:
	    c [0] = 0;
	    c [1] = (int) a [0];
	    break;
      	  case 2:
	    c [0] = (int) a [0];
	    c [1] = (int) a [1];
	    break;
      	  case 3:
	    c [0] = (int) a [0];
	    c [1] = (int) a [1];
	    c [2] = (int) a [2];
	    break;
      	  default:
	    nc = 0;
   	}
   
   	return nc;
}

/*------------------------------------------------------------------------*/

int SortComplex1 (double* x, double* y, int j0, int j1)
{
/*
Sorts complex data so that real values of y are adjacent and complex
values of y are adjacent:

	x1  Re(y1)               x1  Re(y1)
	x2  Im(y1)               x2  Re(y2)
	x3  Re(y2)               x3  Re(x3)
	x4  Im(y2)      --->     x4  Re(x4)
	  ...                      ...
	 ?  Re(y_n)               0  Im(y_(n-1))
	 ?  Im(y_n)               0  Im(y_n)

Parameters:
  j0	-- starting index for (x, y) data in this data set
  j1    -- ending index for real part of data
  x	-- array of x coordinate values
  y  	-- array of y coordinate values
Return number of data points sorted
*/

   int j; 			/* index in x[] and y[] */
   int jt2; 			/* see below */
   int n; 		/* number of (complex) data points in this data set */

   n = j1 - j0 + 1;

   if (n < 1)
      return 0;
   else if (n < 2)		/* No sorting necessary.  */
   {
      x[j1 + 1] = 0.;
      return 1;
   }
   for (j = j0 + 1; j <= j1; j++)
   {
      jt2 = 2 * j - j0;  	/*  =  2*(j - j0) + j0  */
      x[j + n - 1] = y[jt2 - 1]; /* Store Im(y) values in unused part of x[]. */
      y[j] = y[jt2];  		/* Collapse Re(y) values into lower part.  */
      if (jt2 > j1)
      {
	 y[jt2] = x[jt2];  	/* Put Im(y) value in proper place.  */
	 x[jt2] = 0.;  		/* Set Im(x) to zero.  */
      }
      if (jt2 > j1 + 1)
      {
	 y[jt2 - 1] = x[jt2 - 1];
	 x[jt2 - 1] = 0.;
      }
   } /* for */
   x[jt2 + 1] = 0;  		/* jt2 + 1 should = 2*j1 - j0  */

   n = j - j0 + 1;

   return n;

}

/*--------------------------------------------------------------*/

Plot* CreatePlot (Dataset* d, char sym)
{
/*
Create a plot for specified data set. Plotting
  symbol is set to sym and pointer to new plot is returned;
*/
	Plot* p;

	if (d == NULL) return NULL;
	
        if (di.nPlots == MAX_PLOTS)  {
          ErrorDisplay (PL_E_MAXPLOTS);
          return NULL;
        }

        p = (Plot *) malloc (sizeof(Plot));
        
        p->sym = sym;
        p->set = d;
        p->vis = 1;
	SetPlotColor (p, di.nPlots);
	          
        WKSaved = FALSE;

	return p;
}
/*---------------------------------------------------------------*/

void SetPlotColor (Plot* p, int n)
{
/* Set plot color in manner appropriate for the global device context */

	XColor c;
	Colormap map;
	
	switch (gdc.dev_type) {
	  case DEV_X_WINDOW:
	    map = DefaultColormap (display, gdc.mode);
	    
	    if (! XParseColor (display, map, color_list[n], &c)) {
	      c.pixel = BlackPixel(display, gdc.mode);
	    }
	    else {
	      XAllocColor (display, map, &c);
	    }
	    p->color = c.pixel; 
	    break;
	  default:
	    p->color = n;
	}

}
/*---------------------------------------------------------------*/

void DestroyPlot (Plot* p)
{
/*
Free the plot and remove it from the plot list.
*/

	int i, j, nplots;

	nplots = di.nPlots;
	
        for (i = 0; i < di.nPlots; i++) {
          if (di.pPlots[i] == p) {
            --nplots;
            for (j = i; j < di.nPlots - 1; j++) {
              di.pPlots[j] = di.pPlots[j+1];
            }
          }
        }
        
        di.nPlots = nplots;
        
        if (nplots) {
	  if (di.ap == p) di.ap = di.pPlots[nplots - 1];
	  if (di.op == p) di.op = di.pPlots[nplots - 1];
	}
	else {
	  di.ap = NULL;
	  di.op = NULL;
	}         
        
        free (p);
        WKSaved = FALSE;

}
/*--------------------------------------------------------------*/
int InsertPlot (Plot* p)
{
/*
Insert plot p into the plot list and return the new plot number,
  or -1 if error
*/

	int np;

        if (p == NULL) return -1;
        
        if (di.nPlots == MAX_PLOTS) {
          ErrorDisplay (PL_E_MAXPLOTS);		/* too many plots */
          return -1;
        }
	
	np = di.nPlots;
	di.pPlots[np] = p;
	di.ap = p;
	++di.nPlots;
	
	return np;
}

/*---------------------------------------------------------------*/

int PlotVisible (Plot* p)
{
/* Return true if plot is visible in current display window. */

	int j, k, l;
	double xmin, xmax, ymin, ymax;
	Dataset* d = p->set;
	
/* Determine plot bounds */
        
        xmin = d->e.min.x;
        xmax = d->e.max.x;

        if (p->vis > 1) {
          ymin = d->e.min.iy;
          ymax = d->e.max.iy;
        }
        else {
          ymin = d->e.min.y;
          ymax = d->e.max.y;
        }
        l = di.level;
        j = ((xmin > di.pWl[l].xmax) || (xmax < di.pWl[l].xmin));
        k = ((ymin > di.pWl[l].ymax) || (ymax < di.pWl[l].ymin));
        return (! (j || k));
}

/*----------------------------------------------------------------*/

void MakePlotDisplay (DeviceContext* dc)
{
	/* Render the graphics */

	int j;

        SetDisplayContext ();
        OpenGraphicsDevice (dc);
        Grid (dc);
        Labels (dc);
        PlotList (dc);

        if (di.nPlots != 0) {
          for (j = 0; j < di.nPlots; ++j) {
            Pxy (di.pPlots[j], dc);
          }
        }

        CloseGraphicsDevice (dc);

}
/*------------------------------------------------------------*/
	   
void Grid (DeviceContext* dc)
{

	/* Create grid on output device */

	int x1, y1, x2, y2, xd, yd, xl, yl, k;
	double yl1, yld; 
        int xt [11];
        int yt [11];

        x1 = dc->x1;
        y1 = dc->y1;
        x2 = dc->x2;
        y2 = dc->y2;
        
        xd = x2 - x1;
        yd = y2 - y1;
        xl = xd/50;
        yl = yd/50;
        yl1 = log ((double) y1);
        yld = log ((double) y2) - yl1;

	/* Make frame */

        switch (dc->dev_type) {
          /*
          case DEV_VGA:
            if (di.scale < 1) {
              Box (x1, y1, x2, y2, 15);
	    }
            else {
              circle ((x1+x2)/2, (y1+y2)/2, yd/2, 15);
            }
	    break;
	  */
	  case DEV_X_WINDOW:
	    xBox (dc);
	    break;
	  /*    
          case DEV_LATEX:
            TexBox (x1, y1, x2, y2);
	    break;
          case DEV_PS:
            PsBox (x1, y1, x2, y2);
	    break;
          */
          default:
            ;
        }

/* Make Tic Marks */

        for (k = 0; k < 10; ++k) {
          xt[k] = x1 + k*xd/10;
          if (di.scale == 0) {
            yt[k] = y1 + k*yd/10;
	  }
          else if (di.scale == 1) {
            yt[k] = y1 + (int) ((log(yt[k]) - yl1)*yd/yld);
	  }
          else if (di.scale == 2) {
            xt[k] = k*yd/20;
            yt[k] = y1 + k*yd/10;
	  }
          else {
            ;
          }
        }

        switch (dc->dev_type) {
          /*
          case DEV_VGA:
            Tics (xt, yt, xl, yl, x1, y1, x2, y2);
	    break;
	  */  
	  case DEV_X_WINDOW:
	    xTics (xt, yt, xl, yl, dc);
	    break;  
          /*
          case DEV_LATEX:
            TexTics (xt, yt, xl, yl, x1, y1, x2, y2);
	    break;
          case DEV_PS:
            PsTics (xt, yt, xl, yl, x1, y1, x2, y2);
	    break;
	  */  
          default:
            ;
        }

/* Make grid lines if selected */

        if (di.grid) {
          switch (dc->dev_type) {
            /*
            case DEV_VGA:
              GridLines (xt, yt, x1, y1, x2, y2);
	      break;
            case DEV_LATEX:
              TexGridLines (xt, yt, x1, y1, x2, y2);
	      break;
            case DEV_PS:
              PsGridLines (xt, yt, x1, y1, x2, y2);
	      break;
	    */
	    case DEV_X_WINDOW:
	      xGridLines (xt, yt, dc);
	      break;  
            default:
              ;
          }
        }

}
/*---------------------------------------------------------------*/

void xBox (DeviceContext* dc)
{

	/* Draw the frame */
	
	XSetForeground (display, gc, BlackPixel(display, dc->mode));
	XSetLineAttributes (display, gc, 0, LineSolid, CapButt, JoinMiter);
	XDrawRectangle (display, win, gc, dc->x1, dc->y1,
	  dc->xd, dc->yd);

}
/*----------------------------------------------------------------*/

void xTics (int* xt, int* yt, int xl, int yl, DeviceContext* dc)
{
	/* Draw the tics */
	
	int j;
	
	XSetLineAttributes (display, gc, 0, LineSolid, CapButt, JoinMiter);
	for (j = 0; j < 10; ++j) {
	  XDrawLine (display, win, gc, xt[j], dc->y1, xt[j], dc->y1 + yl);
	  XDrawLine (display, win, gc, xt[j], dc->y2, xt[j], dc->y2 - yl);
	  XDrawLine (display, win, gc, dc->x1, yt[j], dc->x1 + xl, yt[j]);
	  XDrawLine (display, win, gc, dc->x2, yt[j], dc->x2 - xl, yt[j]); 
	}  	
}
/*----------------------------------------------------------------*/

void xGridLines (int* xt, int* yt, DeviceContext* dc)
{
	/* Draw the grid lines */

	int j;
	
	XSetLineAttributes (display, gc, 0, LineDoubleDash, CapButt,
	  JoinMiter);
	for (j = 0; j < 10; ++j) {
	  XDrawLine (display, win, gc, xt[j], dc->y1, xt[j], dc->y2);
	  XDrawLine (display, win, gc, dc->x1, yt[j], dc->x2, yt[j]);
	}

}

/*--------------------------------------------------------------*/

void Labels (DeviceContext* dc)
{
	/* Draw the axes labels */
	
	switch (dc->dev_type) {
	  case DEV_NULL:
	    break;
	  case DEV_X_WINDOW:
	    xLabels (dc);
	    break;
	  default:
	    ;
	}        

}
/*--------------------------------------------------------------*/

void xLabels (DeviceContext* dc)
{
	int j, ix, iy;
	char label [16];
	double xmin, ymin, xmax, ymax, xdel, ydel;

	GetDisplayLimits (&xmin, &ymin, &xmax, &ymax);
	xdel = xmax - xmin;
	ydel = ymax - ymin;
		
	/* Y axis labels */
	
	ix = dc->x1 - 8*dc->ncpix;
	if (ix < 0) ix = 0;
	
	for (j = 0; j <= 10; j += 2) {
	  iy = dc->y2 - j*dc->yd/10 + dc->nrpix/2;
	  sprintf (label, "%8.4f", ymin + j*ydel/10.);
	  XDrawString (display, win, gc, ix, iy, label, 8); 
	}
	
	/* X axis labels */
	
	iy = dc->y2 + 2*dc->nrpix;
	if (iy > dc->yres) iy = dc->yres;
	
	for (j = 0; j <= 10; j += 2) {
	  ix = dc->x1 + j*dc->xd/10 - 4*dc->ncpix;
	  sprintf (label, "%9.4f", xmin + j*xdel/10.);
	  XDrawString (display, win, gc, ix, iy, label, 9);
	}	
}
/*----------------------------------------------------------------*/

int ExpansionBox (int x1, int y1, int* x2, int* y2)
{
	/* Expansion box user interface */
	
	switch (gdc.dev_type) {
	  case DEV_NULL:
	    break;
	  case DEV_X_WINDOW:
	    xExpansionBox (x1, y1, x2, y2);
	    break;
	  default:
	    ;
	}
	return 0;        

}
/*----------------------------------------------------------------*/

int xExpansionBox (int x1, int y1, int* x2, int* y2)
{
    XEvent report;
    int x3, y3, w, h;
	
    XSetForeground (display, gc, BlackPixel(display, gdc.mode));
    XSetFunction (display, gc, GXxor);
	
    *x2 = x1;
    *y2 = y1;
    x3 = x1;
    y3 = y1;
    w = 0;
    h = 0;
    
    XDrawRectangle (display, win, gc, x3, y3, w, h);
	
    while (1) {
      XNextEvent(display, &report);
      switch (report.type) {
	case MotionNotify:
	  XDrawRectangle (display, win, gc, x3, y3, w, h);
	  *x2 = report.xmotion.x;
	  *y2 = report.xmotion.y;
	  if (*x2 < x1) x3 = *x2; else x3 = x1;
	  if (*y2 < y1) y3 = *y2; else y3 = y1;
	  w = abs(*x2 - x1);
	  h = abs(*y2 - y1);
	  XDrawRectangle (display, win, gc, x3, y3, w, h);
	  break;
	case ButtonRelease:
	  XSetFunction (display, gc, GXcopy);
	  XSetForeground (display, gc, BlackPixel(display, gdc.mode));
	  return 0;
	default:
	  ;
	}      
    }
}
/*----------------------------------------------------------------*/
void PlotList (DeviceContext* dc)
{
	switch (dc->dev_type) {
	  case DEV_X_WINDOW:
	    xPlotList (dc);
	    break;
	  default:
	    return;
	}
}
/*----------------------------------------------------------------*/

void xPlotList (DeviceContext* dc)
{
/* Draw the plot list */

	int j, x, y, w, h;
	
	w = dc->ncpix;
	h = dc->nrpix;
	y = dc->y1 - 3*dc->nrpix/2;
	
	for (j = 0; j < di.nPlots; j++) {
	  x = dc->x1 + j*3*dc->ncpix/2;
	  XSetForeground (display, gc, di.pPlots[j]->color);
	  if (di.pPlots[j] == di.ap) {
	    XFillArc (display, win, gc, x, y, w, h, 0, 360*64);
	  }
	  else {
	    XDrawArc (display, win, gc, x, y, w, h, 0, 360*64);
	  }  
	}
}
/*----------------------------------------------------------------*/

void Pxy (Plot* p, DeviceContext* dc)
{
	/* Render plot np */

	Dataset* ds;
	double* x;
	double* y;
	double xmin, ymin, xmax, ymax;
	int n, i, jx1, jx2, ic, xd, DataType;
	char sym;

        if (p->vis == 0) return;

        ds = p->set;
        DataType = ds->data_type;

/* If plot not visible in display window, then exit */

        if (! PlotVisible(p)) return;

/* Determine pointers in data set for beginning and end of visible region */

	x = ds->d.x;
	GetDisplayLimits (&xmin, &ymin, &xmax, &ymax);
        i = IndexLimits (x, xmin, xmax, ds->npts, &jx1, &jx2);

	/*
        if (jx1 > 0) --jx1;
        if (jx2 < ds->npts - 1) ++jx2;
        
        if (p->vis > 1) {
          jy1 = jx1 - jsr + jsi;
	}
        else {
          jy1 = jx1;
        }
        */

        sym = p->sym;
	ic = p->color;
        if (dc->mflag) ic = 7;

        switch (dc->dev_type) {
          case DEV_NULL:
            break;
          /*  
          case DEV_VGA:  
            if (sym == 'B') {
              PointPlot (x, y, jx1, jx2, 3, ic);
	    }
            else if (sym == 'P') {
              PointPlot (x, y, jx1, jx2, 1, ic);
	    }
            else if (sym == 'L') {
              LinePlot (x, y, jx1, jx2, 0, ic);
	    }
            else if (sym == 'R') {
              LinePlot (x, y, jx1, jx2, -1, ic);
	    }
            else if (sym == '+') {
              PointPlot (x, y, jx1, jx2, 1, ic);
              LinePlot (x, y, jx1, jx2, 0, ic);
	    }
            else if (sym == 'S') {
              StickPlot (x, y, jx1, jx2, ic);
	    }
            else if (sym == 'H') {
              HistPlot (x, y, jx1, jx2, ic);
	    }
            else {
              ;
            }
	    break;
	  */
	  case DEV_X_WINDOW:
	    xLinePlot (p, dc);
	    break;  
	  /*
          case DEV_LATEX:
            xd = dc->x2 - dc->x1;
            switch (sym) {
              case 'P':
                TexPointPlot (x, y, jx1, jx2, -ixd/100, ic);
	        break;
              case 'B':
                TexPointPlot (x, y, jx1, jx2, ixd/50, ic);
	        break;
              case 'S':
                TexStickPlot (x, y, jx1, jx2, ic);
	        break;
              case '+':
                TexPointPlot (x, y, jx1, jx2, -ixd/100, ic);
                TexLinePlot (x, y, jx1, jx2, 0, ic);
		break;
              case 'H':
                TexHistPlot (x, y, jx1, jx2, ic);
		break;
              default:
                TexLinePlot (x, y, jx1, jx2, 0, ic);
            }
	    break;
          case DEV_PS:
            switch (sym) {
              case 'P':
                PsPointPlot (x, y, jx1, jx2, 12, ic);
		break;
              case 'B':
                PsPointPlot (x, y, jx1, jx2, 25, ic);
		break;
              case 'S':
                PsStickPlot (x, y, jx1, jx2, ic);
		break;
              case '+':
                PsPointPlot (x, y, jx1, jx2, 5, ic);
                PsLinePlot (x, y, jx1, jx2, 0, ic);
		break;
              case 'H':
                PsHistPlot (x, y, jx1, jx2, ic);
		break;
              default:
                PsLinePlot (x, y, jx1, jx2, 0, ic);
            }
	    break;
          */
          default:
            ;
        }

}
/*---------------------------------------------------------------*/

void xLinePlot (Plot* p, DeviceContext* dc)
{
	/* Render a line plot for plot np in an X Window*/
	
	int j, npts, x1, y1, x2, y2;
	XRectangle r;
	Dataset* ds;
	
	ds = p->set;
	npts = ds->npts;
	
	if (npts <= 0) return;
	
	XSetForeground (display, gc, p->color);
	XSetLineAttributes (display, gc, 0, LineSolid, CapButt, JoinMiter);
	
	r.x = dc->x1;
	r.y = dc->y1;
	r.width = dc->x2 - dc->x1 + 1;
	r.height = dc->y2 - dc->y1 + 1;
	XSetClipRectangles (display, gc, 0, 0, &r, 1, 0);
		
	Coordinates (ds->d.x, ds->d.y, &x1, &y1, 0);			
	
	for (j = 0; j < npts; ++j) {
	  Coordinates (ds->d.x+j, ds->d.y+j, &x2, &y2, 0); 
	  XDrawLine(display, win, gc, x1, y1, x2, y2);
	  x1 = x2;
	  y1 = y2;
	}
}
/*---------------------------------------------------------------*/

void Extrema (Dataset* ds)
{

/* Find extrema for data set ds */

        DataExtrema e;
        DataPtr p;
        Datum d;
	long k;

	if (ds->npts <= 0L) return;

	p = ds->d;
        e.min.x = *p.x;
        e.min.y = *p.y;
        e.max.x = e.min.x;
        e.max.y = e.min.y;
	
/* Determine extrema for x */

        for (k = 0L; k < ds->npts; k++) {
          d.x = *p.x++;
          if (d.x < e.min.x) e.min.x = d.x;
          if (d.x > e.max.x) e.max.x = d.x;
        }

/* Determine extrema for y */

        for (k = 0L; k < ds->npts; ++k) {
	  d.y = *p.y++;
	  if (d.y < e.min.y) e.min.y = d.y;
	  if (d.y > e.max.y) e.max.y = d.y;
        }

/* Determine other extrema according to data set type */

	switch (ds->data_type) {
          case DS_2DCOMPLEX:
            e.min.iy = *p.iy;
            e.max.iy = e.min.iy;
            for (k = 0L; k < ds->npts; ++k) {
              d.iy = *p.iy++;
              if (d.iy < e.min.iy) e.min.iy = d.iy;
              if (d.iy > e.max.iy) e.max.iy = d.iy;
            }
            break;
          default:
            ;
        }

        ds->e.min = e.min;
        ds->e.max = e.max;

}
/*---------------------------------------------------------------*/

void Expand ()
{
/* Expand the plot display */

	char v[255];
	int j, m, n;
        double a[10], dx, dy, rx, ry;
	double xmin, ymin, xmax, ymax;
	
        GetDisplayLimits (&xmin, &ymin, &xmax, &ymax);
        
        a[0] = xmin;
        a[1] = ymin;
        a[2] = xmax;
        a[3] = ymax;
        dx = a[2] - a[0];
        dy = a[3] - a[1];
        j = di.level + 1;

        printf ("\nEnter new window limits x1, y1, x2, y2: ");
        gets (v);
        if (*v == '\r' || *v == '\n' || *v == ESC) return;
        n = NumberParse (a, v);
        
        /*
        if ((dmin(a[1], a[3]) <= 0) && (di.scale == 1)) {
           ErrorDisplay (PL_E_LOG);
           return;
         }
         
         
         if (a[2] < a[0]) swap(a[2], a[0]);
         if (a[3] < a[1]) swap(a[3], a[1]);
         */
         
         if ((a[2] == a[0]) || (a[3] == a[1])) return;  /* error */
            
         di.pWl[j].xmin = a[0];
         di.pWl[j].ymin = a[1];
         di.pWl[j].xmax = a[2];
         di.pWl[j].ymax = a[3];

        /* Check for too small an expansion region to prevent crash! */

        
        rx = fabs((di.pWl[j].xmax - di.pWl[j].xmin)/dx);
        ry = fabs((di.pWl[j].ymax - di.pWl[j].ymin)/dy);

        if ((rx < 0.03) || (ry < 0.03)) {
          ErrorDisplay (PL_E_WINTOOSMALL);
          return;
        }

        if (j < MAX_LEVELS) di.level = j;

	
}
/*----------------------------------------------------------------*/

void FullWindow ()
{
/*
Set base expansion level to the full window, i.e. min and max
values for all visible plots.
*/
	int i, j, k;
	double xmin, xmax, ymin, ymax;
	Dataset* d;

/* Find first visible plot */

        j = 0;

        while ((di.pPlots[j]->vis == NO) && (j < di.nPlots)) ++j;
        d = di.pPlots[j]->set;
        xmin = d->e.min.x;
	xmax = d->e.max.x;

        switch (di.pPlots[j]->vis) {
          case 1:
            ymin = d->e.min.y;
	    ymax = d->e.max.y;
	    break;
          case 2:
            ymin = d->e.min.iy;
	    ymax = d->e.max.iy;
	    break;
          default:
            ;
        }

        for (k = j; k < di.nPlots; ++k) {
           d = di.pPlots[k]->set;
           switch (di.pPlots[k]->vis) {
             case 0:
               /* plot not visible */
	       break;
             case 1:
               xmin = dmin (xmin, d->e.min.x);
               xmax = dmax (xmax, d->e.max.x);
               ymin = dmin (ymin, d->e.min.y);
               ymax = dmax (ymax, d->e.max.y);
	       break;
             case 2:
               xmin = dmin (xmin, d->e.min.x);
               xmax = dmax (xmax, d->e.max.x);
               ymin = dmin (ymin, d->e.min.iy);
               ymax = dmax (ymax, d->e.max.iy);
	       break;
             default:
               ;
           }
	}

        if (xmin == xmax) {		/* prevent divide by zero */
          xmin -= 1.;
	  xmax += 1.;
        }
        if (ymin == ymax) {
          ymin -= 1.;
	  ymax += 1.;
        }

     	di.pWl[0].xmin = xmin;
        di.pWl[0].xmax = xmax;
        di.pWl[0].ymin = ymin;
        di.pWl[0].ymax = ymax;

}
/*-------------------------------------------------------------------*/

void Coordinates (double* xx, double* yy, int* ix, int* iy, int fl)
{
/*
Compute the logical coordinates xx and yy from the physical coordinates
x and y. Logical->Physical if fl == 0; otherwise Physical->Logical
*/
	int iyy;
	int l = di.level;
	
        if (fl) {

          *xx = di.pWl[l].xmin + (*ix - gdc.x1)/di.xsf;

          if (gdc.invert) {
            iyy = gdc.yinv - *iy;
	  }
          else {
            iyy = *iy - gdc.y1;
          }

          if (di.scale == 0) {
            *yy = iyy/di.ysf + di.pWl[l].ymin;
	  }
          else if (di.scale == 1) {
            *yy = exp(iyy/di.ylsf + di.lymin);
	  }
          else if (di.scale == 2) {
            *xx = 0.;
            *yy = (((double)*ix)*((double)*ix) +
              ((double)*iy)*((double)*iy));
	  }
          else {
            *yy = 0.;
          }
	}
        else {

/* Compute the physical coordinates from the logical coordinates */

          *ix = gdc.x1 + (*xx - di.pWl[l].xmin)*di.xsf;

          if (di.scale == 0) {
            *iy = (*yy - di.pWl[l].ymin)*di.ysf;
          }
          else if (di.scale == 1) {
            if (*yy > 0.) {
              *iy = (log(*yy) - di.lymin)*di.ylsf;
            }
            else {
              *iy = 0;
            }
          }
          else if (di.scale == 2) {
            *ix = gdc.x1 + (*yy*cos(*xx) - di.pWl[l].xmin)*di.xsf;
            *iy = (*yy*sin(*xx) - di.pWl[l].ymin)*di.ysf;
          }
          else {
            *ix = 0;
            *iy = 0;
          }

          if (gdc.invert) {
            *iy = gdc.yinv - *iy;
          }
          else {
            *iy += gdc.y1;
          }
        }

}

/*------------------------------------------------------------------*/

void ShowHelp ()
{
	puts ("\n\nXYPLOT Key Commands:\n");
	puts ("F1	--- display this list");
	puts ("N	--- new workspace");
	puts ("F	--- open and plot a file");
	puts ("G	--- toggle grid lines");
	puts ("H	--- display header");
	puts ("D	--- drop active plot");
	puts ("E	--- expand window");
	puts ("ESC	--- unexpand");
	puts ("X	--- delete dataset"); 
	puts ("Q	--- quit program");
}
/*-------------------------------------------------------------------*/

