/*
  xyplot.h

 Header file for xyplot and associated files.
 Copyright (c) 1990-1995 Creative Consulting for Research & Education
 Last Update: 9/9/95
*/



/*--------------------------------------------------------------------*/
/*                      GLOBAL CONSTANTS			      */
/*--------------------------------------------------------------------*/

#define YES 	-1
#define NO 	0
#define RET 	13
#define DEFAULT_SYMBOL  'L'
#define COMMA 	44
#define CR 	13
#define ESC 	27
#define SPACE 	32
#define TAB 	9
#define FALSE 	0
#define TRUE 	! FALSE

#define PL_NONE		0
#define PL_ERR		1
#define PL_EXIT		2
#define PL_RESET	3
#define PL_HELP		4
#define PL_MENU		5

#define PL_CURSORMOVE	16
#define PL_BUTTONPRESS	17
#define PL_KEYPRESS	18
#define PL_MOUSETOGGLE	19
#define PL_DATACURSOR	20
#define PL_RESIZE	21
#define PL_REDRAW	22

#define PL_NEW		32
#define PL_FILE		33
#define PL_OPEN		34
#define PL_SAVEDATA	35
#define PL_SAVEWORK	36
#define PL_SAVEWK1	37
#define PL_SAVETEX	38
#define PL_SAVEPS	39
#define PL_SETSAVEOPTS	40
#define PL_VIEWFILE	41
#define PL_HEADER	42

#define PL_BASE		64
#define PL_EXPAND	65
#define PL_ESCAPE	66

#define PL_DROP		80
#define PL_PICK		81
#define PL_ASPECT	82
#define PL_SCALE	83

#define PL_PALETTE	96
#define PL_GRID		97
#define PL_POINT	98
#define PL_LINE		99

#define PL_DELETESET	128
#define PL_DUP		129
#define PL_JOIN		130
#define PL_PRUNE	131

#define PL_ZERO		160
#define PL_AREA		161
#define PL_DERIV	162
#define PL_INTEGRATE	163
#define PL_BIN		164
#define PL_POLY		165
#define PL_SORT		166
#define PL_SWAPXY	167
#define PL_MAG		168
#define PL_PHASE	169
#define PL_CONJ		170
#define PL_REALY	171
#define PL_IMAGY	172
#define PL_CMPLY	173
#define PL_RESCALE	174
#define PL_ARITHMETIC	175
#define PL_SMOOTH	176
#define PL_FFT		177
#define PL_TEMPLATE	178

/* Error codes */

#define PL_E_NONE		0
#define PL_E_UNKNOWN		1
#define PL_E_ILLEGALFUNCCALL	2
#define PL_E_OVERFLOW		3
#define PL_E_DIVBYZERO		4
#define PL_E_BADFILENAME	5
#define PL_E_FILEEXISTS		6
#define PL_E_FILENOTFOUND	7
#define PL_E_DISKFULL		8
#define PL_E_DISKERROR		9
#define PL_E_PATHNOTFOUND	10
#define PL_E_MEMALLOC		11
#define PL_E_GDUNKNOWN		12
#define PL_E_GDCANTOPEN		13

#define PL_E_LOG		16
#define PL_E_NOPLOT		17
#define PL_E_OVERLAP		18
#define PL_E_BADFILE		19
#define PL_E_BADCLOPT		20
#define PL_E_BADEXP		21
#define PL_E_CORNER		22
#define PL_E_NOMOUSE		23
#define PL_E_POLORDER		24
#define PL_E_TOOFEWPTS		25
#define PL_E_MAXPLOTS		26
#define PL_E_SAMEABSCISSA	27
#define PL_E_NODATA		28
#define PL_E_MAXSETS		29
#define PL_E_NEW		30
#define PL_E_XSPVER		31
#define PL_E_FFTUNABSC		32
#define PL_E_MAXPTS		33
#define PL_E_PHASE		34
#define PL_E_NOCOMPLEX		35
#define PL_E_CMPLXRES		36
#define PL_E_NOSPACE		37
#define PL_E_DBFUNC		38
#define PL_E_ZERONA		39
#define PL_E_FRACPWR		40
#define PL_E_WINTOOSMALL	41
#define PL_E_DATACURMODE	42

char* PL_ERROR_MESSAGE [64] =
{
	"No error",
	"Unknown error occured",
	"Illegal function call",
	"Overflow",
	"Divide by zero",
	"Bad file name",
	"File already exists",
	"File was not found",
	"Disk is full",
	"Disk is not ready",
	"Path was not found",
	"Memory allocation failed",
	"Graphics device unknown",
	"Cannot open graphics device",
	"",
	"",
	"Log function is invalid:  y <= 0",
	"Plot does not exist",
	"Data sets do not overlap",
	"Cannot find first line of data in file",
	"Command line option not recognized",
	"Rescale expression not recognized",
	"Mark opposite corner",
	"Mouse is not present",
	"Polynomial order must be from 0 TO 9",
	"Insufficient number of points for fit",
	"Too many plots",
	"Two points have the same abscissa",
	"No data for plot in window domain",
	"Too many data sets",
	"Select NEW before loading workspace",
	"Incorrect workspace version",
	"FFT requires uniformly spaced abscissas",
	"Too many points -- Exceeding storage limit",
	"Phase = 0 for real data set",
	"Function not available for complex data",
	"Complex rescale only allowed for complex Y data",
	"Insufficient space in database for operation",
	"Invalid database function",
	"Zero operand not allowed",
	"Fractional power not allowed for negative operand",
	"Expansion window too small",
	"Set Data Cursor mode first",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", ""
};

/*--------------------------------------------------------------------*/
/*              CONFIGURABLE CONSTANTS				      */
/*--------------------------------------------------------------------*/

#define MAX_LEN 	255
#define MAX_NUMS 	20

#define MAX_POINTS 	32768	
#define MAX_MESSAGES	32
#define MAX_SETS  	32          /* number of allowed data sets */
#define MAX_PLOTS 	32          /* number of plots */
#define MAX_TERMS 	32          /* max terms in fitting matrix */
#define MAX_OPTS  	32          /* max menu options */
#define MAX_LEVELS 	32
#define HEADER_LENGTH  	4096        /* length of header text */
#define VERSION  	"1.0"       /* current version number */
#define HEADER_BEGIN 	"/*"
#define HEADER_END 	"*/"
#define HDR_PREFACE  	"==XYPLOT==> " /* header preface */
#define NAME_LENGTH	255

#define DEV_NULL	0
#define DEV_VGA		1
#define DEV_X_WINDOW	2
#define DEV_PS		3
#define DEV_LATEX	4

#define DS_2DREAL	0
#define DS_2DCOMPLEX	1
#define DS_3DREAL	2
#define DS_3DCOMPLEX	3
#define DS_2DVECTOR	4
#define DS_3DVECTOR	5

/*--------------------------------------------------------------------*/
/*              XYPLOT DATA STRUCTURES				      */
/*--------------------------------------------------------------------*/

/* Define the datum type */

typedef struct tagDatum {
	double x;
	double y;
	double z;
	double ix;
	double iy;
	double iz;
	double Ax;
	double Ay;
	double Az;
} Datum;
	
/* Define the data pointer type */

typedef struct tagDataPtr {
	double* x;		/* ptr to x coordinate */
	double* y;		/* ptr to y coordinate */
	double* z;		/* ptr to z coordinate */
	double* ix;		/* ptr to imaginary part of x */
	double* iy;		/* ptr to imaginary part of y */
	double* iz;		/* ptr to imaginary part of z */
	double* Ax;		/* ptr to vector x component */
	double* Ay;		/* ptr to vector y component */
	double* Az;		/* ptr to vector z component */
} DataPtr;


/* Define the data extrema type */

typedef struct tagDataExtrema {
	Datum min;
	Datum max;
} DataExtrema;
	

/* Define data set type */

typedef struct tagDataset {
	double* pData;		/* pointer to the data block */
	DataPtr d;		/* pointers to the data components */
	DataExtrema e;		/* extrema */
	int data_type;		/* specifies the data set type */
	int npts;		/* number of points */
	char* name;
	char* hdr;
} Dataset;


/*  Define data base type */

typedef struct tagDatabase {
	Dataset** pSets;
	int nSets;
	char* hdr;
} Database;

/*  Define type for containing output device information */

typedef struct tagDeviceContext {
 	int dev_type;           /* type of device (0=NULL) */
        int mode;               /* device mode */
        char* dev_name;
        FILE* fp;		/* file ptr */
        int nrows;              /* number of rows */
        int ncols;              /* number of columns */
        int ncolors;            /* number of colors */
        int xres;               /* resolution in x */
        int yres;               /* resolution in y */
        int ncpix;              /* number of pixels per column */
        int nrpix;              /* number of pixels per row */
        int x1;                 /* min. x physical coordinate */
        int x2;                 /* max. x physical coordinate */
        int y1;                 /* etc. */
        int y2;                 /* etc. */
        int xd;
        int yd;
        int yinv;               /* inversion coordinate */
        int invert;             /* y coordinate inversion flag */
        int mflag;              /* monochrome/color flag */
} DeviceContext;


/*  Define type for plot information */

typedef struct tagPlot {
	Dataset* set;
	char sym;
	int color;
	int vis;
} Plot;


/* Define window limits structure */

typedef struct tagWindowLimits {
          double xmin;
          double xmax;
          double ymin;
          double ymax;
} WindowLimits;


/* Define type for display information */

typedef struct tagDisplayContext {
	Plot** pPlots;
	int nPlots;
	WindowLimits* pWl;
	Plot* ap;		/* ptr to the active plot */
	Plot* op;		/* ptr to the operand plot */
	int scale;
	int csize;
	int cmode;
	int level;
	int grid;
	int mouse_button;
	int cx;
	int cy;
	double xsf;
	double ysf;
	double ylsf;
	double lymin;
} DisplayContext;


/* Define data structure to keep track of color information */


struct ColorInfo {
          int dwb;       /* directory background */
          int dwf;       /* directory foreground */
          int dwa;       /* directory alternate */
          int fwb;       /* file view background */
          int fwf;       /* file view foreground */
          int fwa;       /* file view alternate */
          int hwb;       /* header view background */
          int hwf;       /* header view foreground */
          int hwa;       /* header view alternate */
          int gwb;       /* graphics window background */
          int gbc;       /* grid box color */
          int clc;       /* coordinate labels color */
          int ccc;       /* cursor coordinates color */
          int mic;       /* menu items color */
};

/* Data type to store math operations info */

struct MathInfo {
          double mean;
          double variance;
          double area;
          double sigma;    	/* specified uncertianty */
          double corr;       	/* correlation coefficient */
          double chisq_r;      	/* chi-square for fit (real) */
          double chisq_i;      	/*    "  " (imaginary) */
  	  int ErrType;    	/* type of error (0 = statistical, 1 = specified) */
};

/* Data type to store file save options */

struct SaveInfo {
          int hs;  		/* save header flag */
          char fm;        	/* number format ('E', 'F', or 'I') */
          char dl;        	/* character delimiter (' ', or ',') */
};
        


/* Lotus 1-2-3 worksheet data types */

struct WksBOF {
	short int TypeCode;
	short int BodyLength;
	short int Revision;
};

struct WksEOF {
	short int TypeCode;
	short int BodyLength;
};

struct WksRange {
	short int TypeCode;
	short int BodyLength;
	short int StartCol;
	short int StartRow;
	short int EndCol;
	short int EndRow;
};

struct WksNumHeader {
	short int TypeCode;
	short int BodyLength;
	char Fmt;
};

struct WksNumber {
	short int Col;
	short int Row;
	double Value;
};

typedef struct tagPlotMessage {
	int wid;	/* window identifier */
	int x;		/* pointer x */
	int y;		/* pointer y */
	int message;	/* message id */
	int nParam;	/* number of parameters */
	long dLen;	/* buffer length in bytes */
	void* pBuf;	/* ptr to parameter buffer */
} PlotMessage;

/*--------------------------------------------------------------*/
/*		Function Prototypes				*/
/*--------------------------------------------------------------*/

/*
void ViewFile (char* fname);
void ScreenTitle (char* title, int tc);
void DisplayFile (char* fname);
void Message1 ();
void Message2 ();
void KeyHelp (char* key);
void DirectorySet (char* dir);

void SelectFile (char* fspec, int mode, char* p, char* fl,
	int ix, int iy, int bs);
void FileView ();
void ShowDir (char* fspec);
char* GetFileListName (char* flist, int n);
void SetFileListName (char* flist, int n, char* fname);
void SortFileList (char* flist, int nfiles);
void DisplayDirectory (char* fspec, int ir, int nlines,
	int ifc, int ibc, int iac);
void ViewHeader ();
void Color (int col);

void MenuBox (void* gm, char** opt);
void MenuSelection (char* key);
void HighlightOption (void* gm, int opt);
void ShowMenu (char* key);
void FileMenu (char* key);
void SetMenu (char* key);
void DataSetMenu (char* key);
void SaveMenu (char* key);
void PlotMenu (char* key);
void SymbolMenu (char* key);
void ScaleMenu (char* key);
void MathMenu (char* key);
void ArithmeticMenu (char* key);
void FitMenu (char* key);
void MathMenu2 (char* key);
void FFTMenu (char* key);
void ComplexMenu (char* key);
void ExitMenu (char* key);
void ButtonMenu (int n, char* key);
void SetSaveOptions ();

int ibound (int x, int y);
int ibound2 (int x, int y);
void CmpToReal (Dataset* d, int jp);
void RealToCmp (Dataset* d);
void SwapXY (Dataset* d);
Dataset* PolynomialFit (Dataset* d);
Dataset* PlotArithmetic (Dataset* d1, Dataset* d2, char* key);
Dataset* Template ();
void Zero (Dataset* d);
void Scale (Dataset* d);
void Smooth (Dataset* d);
void Area (Dataset* d);
Dataset* Deriv (Dataset* d);
Dataset* Fft (Dataset* d, char* key);
Dataset* Mag (Dataset* d);
Dataset* Phase (Dataset* d);
void Conj (Dataset* d);
Dataset* Bin (Dataset* d);
void Sort (Dataset* d);

void SpecialKey (char* key);
void GetKey (char* key, int bs);
void GetInput (char* req);
void KMouse (char* key);
void KeypadMouse (char* key);
void SetCursor ();
void DataCursor (Plot* p);

void Overlap (int ns1, int ns2, double* x1, double* x2);

void StoreData (Dataset* d);
int ReadWork (char* fname);
void SaveWork ();
void SaveWK1 ();
void SaveTeX ();
void SavePS ();

int BlankLine (char*);
int ReadHeader (FILE*, char*, char*);
int SelectColumns (int*, int);
int SortComplex1 (double*, double*, int, int);

void WriteHeader (char* hdr);
void ReadIniFile (void* sc, char* Pcolors, char* Wcolors);

int Pick ();
void DeletePoint (Dataset* d, int j);
void InsertPoint (Dataset* d, double xx, double yy);
void ShiftValues (double* x, double* y, int j1, int m);
Dataset* Duplicate (Dataset* d);
Dataset* CopySet (Dataset* d);
Dataset* JoinSets (Dataset* d1, Dataset* d2);
void PruneSet (Dataset* d);

void SetColors (void* sc, char* p, char* w);
void InitParameters (void* sc);

void SetScnParms (DeviceContext* dc, int aspect);
void SelectColor (int n);

void Tics (int* xt, int* yt, int xl, int yl,
	int x1, int y1, int x2, int y2);
void GridLines (int* xt, int* yt, int x1, int y1, int x2, int y2);
void PointPlot (double* x, double* y, int x1, int x2, int y1,
	int size, int c);
void LinePlot (double* x, double* y, int x1, int x2, int y1,
	int style, int c);
void StickPlot (double* x, double* y, int x1, int x2, int y1, int c);
void HistPlot (double* x, double* y, int x1, int x2, int y1, int c);
void TeXPointPlot (double* x, double* y, int x1, int x2, int y1,
	int size, int c);
void TeXLinePlot (double* x, double* y, int x1, int x2, int y1,
	int style, int c);
void TeXStickPlot (double* x, double* y, int x1, int x2, int y1, int c);
void TeXHistPlot (double* x, double* y, int x1, int x2, int y1, int c);
void TeXLine (int x1, int y1, int x2, int y2, char* ln);
void TeXBox (int x1, int y1, int x2, int y2);
void TeXTics (int* xt, int* yt, int xl, int yl,
	int x1, int y1, int x2, int y2);
void TeXGridLines (int* xt, int* yt, int x1, int y1, int x2, int y2);
void PsLinePlot (double* x, double* y, int x1, int x2, int y1,
	int style, int c);
void PsPointPlot (double* x, double* y, int x1, int x2, int y1,
	int size, int c);
void PsStickPlot (double* x, double* y, int x1, int x2, int y1, int c);
void PsHistPlot (double* x, double* y, int x1, int x2, int y1, int c);
void PsBox (int x1, int y1, int x2, int y2);
void PsTics (int* xt, int* yt, int xl, int yl, int x1, int y1,
	int x2, int y2);
void PsGridLines (int* xt, int* yt, int x1, int y1, int x2, int y2);

void SetCorners ();

*/

int GetPlotMessage (PlotMessage* pm);
int xGetPlotMessage (PlotMessage* pm);
int PostPlotMessage (PlotMessage* pm);
int DestroyPlotMessage (PlotMessage* pm);

void CopyrightNotice ();

int FreeWorkspace ();
int NewWorkspace ();
void ShowHelp();  
void ErrorDisplay (int number);
void DisplayHeader (Dataset* d);

int OpenFile (Dataset** ds, char* fname);
int xyread (double* x, double* y, int* dtype,
  char* name, char* hdr);
int NumberParse (double*, char*);
int BlankLine (char* ln);
int ReadHeader (FILE* fp, char* hdr, char* ln);
int SelectColumns (int* c, int n);
int SortComplex1 (double* x, double* y, int j1, int j2);
int IndexLimits (double*, double, double, int, int*, int*);
double dmin (double, double);
double dmax (double, double);
int TranslateKey (int);

void RaiseWindow (Window w);
Plot* CreatePlot (Dataset* d, char sym); 
void DestroyPlot (Plot* p);
int InsertPlot (Plot* p);
void SetPlotColor (Plot* p, int n);
int PlotVisible (Plot* p);

int AllocateArray (int n, int* j1, int* j2);
Dataset* CreateSet (double* x, double* y, long npts, int data_type,
  char* name, char* hdr);
int InsertSet (Dataset* d);
void DeleteSet (Dataset* d);
int ExpansionBox (int x1, int y1, int* x2, int* y2);
void Expand ();
void Extrema (Dataset* d);
void FullWindow ();
int ResizeWin (DeviceContext* dc, int width, int height);
void UpdateCursorCoords (DeviceContext* dc, int x, int y);

void SetDeviceContext (DeviceContext* dc);
void SetDisplayContext ();
int OpenGraphicsDevice (DeviceContext* dc);
int CloseGraphicsDevice (DeviceContext* dc);
int OpenDisplay (DeviceContext* dc);
int CloseDisplay (DeviceContext* dc);
int xOpenDisplay (DeviceContext* dc);
int xCloseDisplay ();
void GetDisplayLimits (double* xmin, double* ymin, double* xmax,
  double* ymax);

void Labels (DeviceContext* dc);
void xLabels (DeviceContext* dc);
void Grid (DeviceContext* dc);
void xBox (DeviceContext* dc);
void xTics (int* xt, int* yt, int xl, int yl, DeviceContext* dc);
void xGridLines (int* xt, int* yt, DeviceContext* dc);
int xExpansionBox (int x1, int y1, int* x2, int* y2);

void MakePlotDisplay (DeviceContext* dc);
void Pxy (Plot* p, DeviceContext* dc);
void PlotList (DeviceContext* dc);
void xPlotList (DeviceContext* dc);

void Coordinates (double* xx, double* yy, int* ix, int* iy, int fl);

void xLinePlot (Plot* p, DeviceContext* dc);

/*---------------------------------------------------------------*/
/*      XYPLOT GLOBAL VARIABLES 				 */
/*---------------------------------------------------------------*/


/*
        COMMON SHARED /XYVARS/ DS() AS DataSet, PV() AS PlotInfo, _
          DI AS DisplayInfo, GC AS DeviceContext, MI AS MathInfo,_
          CI AS ColorInfo, VI AS SaveInfo, CORNERS() AS CornerInfo,_
          DB AS DataBase, AlphaR() AS DOUBLE, AlphaI() AS DOUBLE,_
          Ar() AS SINGLE, Ai() AS SINGLE, StartFile$, Dsym$,_
          IGC() AS INTEGER, IAREA() AS INTEGER, MaxVals AS INTEGER, WKSaved

        DIM AlphaR (MaxTerms, MaxTerms) AS DOUBLE
        DIM AlphaI (MaxTerms, MaxTerms) AS DOUBLE
        DIM Ar (MaxTerms) AS SINGLE
        DIM Ai (MaxTerms) AS SINGLE
        DIM IGC (500) AS INTEGER
        DIM IAREA (500) AS INTEGER
*/

/* end of xyplot.h */
