\ chaos.4th
\
\ Words to create a 3-dimensional embedding for discrete time series data
\ and to obtain a Poincare' surface section in xyplot.
\
\ K. Myneni
\
\ Required files:
\ 
\	matrix.4th
\	strings.4th
\
\ Revisions:
\
\	02-19-1999 created
\	03-06-2001 updated with user interface  and error checking for max size
\
variable rcntr	\ row counter
variable nembed	\ number of points in embedding
variable embedding_step \ number of points to step for embedding
fvariable embedding_time



DatasetInfo ds1 
DatasetInfo dsem 	\ dataset info structure for embedding

8192 constant MAX_EMBED_SIZE

3 MAX_EMBED_SIZE fmatrix embedding 	\ matrix to hold embedding data
embedding 8 + dsem DDATA + !		\ store the data pointer

: embed ( fdeltat -- | perform the embedding on the active dataset )
\ fdeltat is the embedding time
	embedding_time f!
	?active dup
	0 >= if
	  ds1 get_ds
	  0 >= if

	    \ Determine embedding step

	    embedding_time f@ 
	    0 ds1 @xy fdrop 1 ds1 @xy fdrop f- fabs
	    f/ fround f>s embedding_step ! 	    

	    ds1 ->npts embedding_step @ 2* - nembed ! \ number of points in embedding
	    nembed @ MAX_EMBED_SIZE > if
	      ." Required size of embedding array exceeds "
	      MAX_EMBED_SIZE .
	      exit
	    then

	    " Embedding" 1+ dsem DNAME + !
	    " 3D embedding" 1+ dsem DHEADER + !

	    256 dsem DTYPE + !		\ double precision fp type
	    nembed @ dsem DNPTS + !
	    3 dsem DSIZE + !
	    nembed @ 3 embedding mat_size!
 	 
	    1 rcntr !
	    nembed @ embedding_step @ 2* 
	    do
	      i ds1 @xy fswap fdrop
	      i embedding_step @ - ds1 @xy fswap fdrop
	      i embedding_step @ 2* - ds1 @xy fswap fdrop
	      3 rcntr @ embedding frow!
	      1 rcntr +!
	    loop
	
	    dsem make_ds
	  then
	else
	  drop
	then ;


fvariable psvalue
2e-3 fconstant epsilon	\ instrumental noise in signal
			\ here we assume 2 millivolts

: psection ( f -- | print out the poincare' section for z = f )
	psvalue f!
	1 rcntr !
	begin
	  rcntr @ 3 embedding fmat@   \ z is the 3rd coordinate in embedding
	  psvalue f@ f- fabs
	  epsilon f< if
	    cr rcntr @ .
	    rcntr @ 1 embedding fmat@ f.
	    rcntr @ 2 embedding fmat@ f.
	    embedding_step @ 2 - 1 max rcntr +!
	  else
	    1 rcntr +!
	  then
	  rcntr @ nembed @ >
	until
;

: xyembed ( -- | prompt user for embedding time and perform embedding )
	?active 0 >=
	if
	  c" Enter the embedding step: " get_input
	  if
	    string>f embed
	  else
	   drop
	  then
	then ;

\ add "Embed" to the math menu

MN_MATH c" Embed3D" c" xyembed draw_window" add_menu_item

