\ s2000.4th
\
\  kforth interface to the S2000 spectrometer via the Ocean Optics 
\  ADC500 interface card. This Forth code interfaces with the S2000 
\  driver for Linux written by Jochen Karrer. For details on
\  on the driver, see
\
\    http://cip.physik.uni-wuerzburg.de/~cip307/s2000/menu.html
\
\ Copyright (c) 1999--2005 David Wallace and Krishna Myneni
\ Provided under the GNU General Public License
\
\ Requires:
\
\	strings
\	files
\	dump
\
\ Revisions:
\   12-27-1999  changed include file order, 
\               added wavelength calibration, fixed file output  KM
\   01-18-2005  minor code cleanup and new comments  KM
\   06-15-2006  changed device driver name to "/dev/adc500_0" to be
\               consistent with installation from source package  KM
\   06-16-2006  created string ptr for driver name and removed excess error checking  KM


\ Wavelength calibration coefficients for the S2000

create s2000_coefficients 3 dfloats allot
745.1227315e  s2000_coefficients  f!
0.144910053e  s2000_coefficients  dfloat+ f!
-1.39067e-5   s2000_coefficients  dfloat+ dfloat+ f!


: ptr CREATE 1 CELLS ?allot ! DOES> a@ ;

c" /dev/adc500_0"  ptr  ^adc_driver

variable ADC500_fp
0 adc500_fp !

create s2000_spectrum 2048 cells allot


: s2000_open ( -- b |  open the adc500 device driver; return TRUE if success)
    0 ADC500_fp !
    ^adc_driver R/W open dup 
    0> IF ADC500_fp ! ELSE  drop THEN
    ADC500_fp @ 0> ;
	

: s2000_close ( -- b | close the adc500 device driver)
     ADC500_fp @ close 0= ;


: s2000_ascii ( -- b | put the adc500 in ascii mode)
     s" ascii" ADC500_fp @ write-line 0> ;


: s2000_bin ( -- b | put the adc500 in binary mode)
     s" bin" ADC500_fp @ write-line 0> ;
  

: s2000_set_int ( n -- b | set the integration time in milliseconds)
     dup 5 >= IF
       s>string count ADC500_fp @ write-line 0>
    ELSE  drop false
    THEN ;


: s2000_set_frames ( n -- b | set the number of added frames)
     dup 0> IF
       s" add " rot s>string count strcat ADC500_fp @ write-line 0>
     ELSE  drop false
     THEN ;
	

: s2000_start ( -- b | start spectrometer)
     s" start" ADC500_fp @ write-line 0> ;

	
: s2000_stop ( -- b | stop spectrometer)
     s" stop" ADC500_fp @ write-line 0> ;

	
: s2000_get_spectrum ( -- b | get the spectrum)
      BEGIN
 	ADC500_fp @ s2000_spectrum 2048 4 * read
	2048 4 * >=
      UNTIL
      true ;


: s2000_lambda ( n -- flambda | compute wavelength at pixel n )
	\ n is 0 for the first pixel
	\ flambda is the calibrated wavelength for pixel n
	s>f
	fdup fdup f* s2000_coefficients 2 dfloats + f@ f*
	fswap s2000_coefficients dfloat+ f@ f* f+
	s2000_coefficients f@ f+ ;
	  
	 
variable outfile_id

: write_spectrum ( ^filename -- )
	count W/O create-file
	if 
	  ." Cannot open output file!" cr
	  drop
	else 
	  outfile_id !
	  2048 0 do
	    i s2000_lambda 7 f>string count s"   " strcat
	    s2000_spectrum i cells + @ s>string count strcat
	    outfile_id @ write-line drop
	  loop
	  outfile_id @ close-file drop 
	then ;


: gs S2000_get_spectrum drop ;
: cs S2000_open s2000_bin 2drop ;
\ : ds gs s2000_spectrum 512 dump ;	
: ws write_spectrum ;
	
	
