\ s2000x.4th
\ 
\ xyplot interface to the Ocean Optics S2000 spectrometer
\
\ Copyright (c) 2000-2005  Krishna Myneni and David P Wallace
\ Provided under the GNU General Public License
\
\ Revisions:
\	1-18-2000  KM
\	1-24-2001  KM; added two more menu items; moved items to data menu
\       1-18-2005  KM; updated for xyplot 2.0.3 data structures; use signals
\                        based interface for live update of spectra
\       1-24-2005  KM; fixed signal handler to drop signal number
\       3-11-2005  KM; use separate menu for S2000 (requires xyplot >= 2.1.2)
\	6-16-2006  KM; add error messages on initialization

include s2000.4th    \ driver interface

DatasetInfo dsspec 
PlotInfo    plspec 
create spec_xydata 2048 2* sfloats allot

\ Fill out a dataset info structure on the new set

c" S2000_Spectrum" 1+ dsspec DNAME ! \ name of new dataset
c"  " 1+ dsspec DHEADER !
0 dsspec DTYPE !		\ real dataset
2048 dsspec DNPTS !		\ 200 points in the set
2 dsspec DSIZE !		\ dimension is 2 (x, y)
spec_xydata dsspec DDATA !	\ pointer to data

\ create a new data set in xyplot

dsspec make_ds
0 dsspec get_ds drop		\ get xyplot dataset info

\ Fill out a plot info structure on the new plot

0 plspec PDSET !		\ dataset number (assume it is first set)
0 plspec PTYPE !		\ plot type
1 plspec PSYM  !		\ plot symbol
1 plspec PCOLOR !		\ plot color

\ create a new plot in xyplot

plspec make_plot


: set_spec_data ( -- | initialize x-y data buffer with latest spectrum )
	\ x goes from 0 to 2047 
	2048 0 do
	  i s2000_lambda			\ x value
	  s2000_spectrum i 2* 2* + @ s>f	\ y value
	  i dsspec !xy
	loop ?active set_ds_extrema ;


\   Old method of live update in xyplot required key press in open console
\   window to halt the update and other xyplot functions were inactive
\   during this time. The replacement code below uses the signals
\   interface in kForth 1.2.10 to make the real-time display a background
\   process. All xyplot functions continue to be active during the
\   real-time display and a menu function is used to halt the update.
\
\ : show_spec ( -- | real-time update of spectrum in xyplot )
\	false false set_grid_lines
\	draw_window
\	begin
\	  false draw_plot	\ erase previous plot
\	  gs			\ get the latest spectrum
\	  set_spec_data		\ send data to xyplot dataset
\	  \ reset_window
\	  true draw_plot
\	  500 ms
\	  key?
\	until
\	key drop 
\ ;


\ Signal handler to obtain and show the spectrum in xyplot

: update_spec ( n -- | refresh spectrum plot )
	drop              \ drop the signal number
        false draw_plot   \ erase previous plot
        gs                \ get the latest spectrum
        set_spec_data     \ send data to xyplot dataset
        true draw_plot    \ refresh display
;

\ show_spec starts the signal handler with 500 ms period

: show_spec ( -- | set up real-time update of spectrum in xyplot )
	draw_window
	['] update_spec  SIGALRM  forth-signal drop
	300 300 set-timer    \ send SIGALRM to xyplot every 300 ms
;

: halt_spec ( -- | stop real-time update )
	SIG_IGN SIGALRM  forth-signal  DROP  \ stop timer signal
;


: set_integ ( -- | set the integration time for the spectrum )
	c" Enter the integration time in ms:" get_input
	if
	  string>s s2000_set_int 
	  if ." New S2000 integration time is set."
	  else ." Error setting S2000 integration time."
	  then
	  gs gs gs
	else
	  drop
	then ;


\ make an S2000 menu and add following items to this menu

c" S2000" make_menu constant MN_S2000

MN_S2000  c" Get  Spectrum"   c" gs set_spec_data draw_window"  add_menu_item
MN_S2000  c" Show Spectrum"   c" show_spec"                     add_menu_item
MN_S2000  c" Halt"            c" halt_spec"                     add_menu_item
MN_S2000  c" Set Integration" c" set_integ gs set_spec_data draw_window" 
  add_menu_item
 

\ initialization

s2000_open 0= [IF] s" Unable to open " ^adc_driver count strcat message_box [THEN]
s2000_bin  0= [IF] s" Unable to put S2000 in binary mode." message_box [THEN]
s2000_get_spectrum 0= [IF] s" Unable to get spectrum from S2000." message_box [THEN]
set_spec_data
" white" set_background
" blue" set_plot_color
reset_window
false false set_grid_lines
\ show_spec

	
	
