\
\  noise.fs
\
\  Pseudo-random noise generation routines:
\
\    ran0	generates a random number between 0e and 1e with a
\		uniform distribution. Translated from Numerical Recipes
\		in C, by Press, et. al. 
\
\     gauss 	generates statistical noise with a normal distribution
\    		having mean of 0e and variance of 1e. Uses the output
\		of ran0 to generate a uniform deviate and
\ 		transforms this number into a normally distributed
\ 		deviate.
\
\ Notes:
\
\ 1.  The formula for the inverse cumulative distribution function (CDF)
\     of a normal distribution is taken from Abramowitz and Stegun, p. 933.
\
\ 2.  Requires the FSL modules fsl-util.fs and  horner.fs
\
\
\ Copyright (c) 2001 Krishna Myneni
\ Original code for ran0 is Copyrighted by Numerical Recipes Software
\
\ Revisions:
\   2001-07-30  km; fixed gauss (flog -> fln)
\   2007-11-06  km; use FSL routine }Horner to simplify GAUSS

CR .( NOISE             V1.0b         06 November  2007   KM )
BASE @ DECIMAL
variable idum

16807 constant IA
2147483647 constant IM
1e IM s>f f/ fconstant AM
127773 constant IQ
2836 constant IR
123459876 constant MASK

\ The word ran0 returns an observation from a uniform distribution
\   between 0. and 1.
\
\   Initialize the variable idum to any integer value prior to calling
\   ran0. Do not alter idum between calls for successive deviates in
\   a sequence.

: ran0 ( -- f )
    idum @ MASK xor dup idum !
    IQ / 
    dup IR * swap
    IQ * idum @ swap - IA *
    swap - idum !
    idum @ 0< IF  IM idum +!  THEN
    idum @ s>f AM f*
    idum @ MASK xor idum !	
;


variable iflag

3 FLOAT array C{
2.515517e  0.802853e  0.010328e  3 C{ }fput
4 FLOAT array D{
1e 1.432788e  0.189269e  0.001308e  4 D{ }fput

: gauss ( -- f | generate a pseudo-random number with a Gaussian distribution )

    false iflag !

    BEGIN  ran0  fdup  F0=  WHILE  fdrop  REPEAT

    fdup 0.5e F> dup iflag !
    IF  1e fswap F-  THEN			\ X

\ Compute the inverse of the CDF
\    
\ Equivalent FORTRAN code:
\
\ 	T = SQRT(LOG(1./X**2.))
\ 	XP = T - (C0+C1*T+C2*T**2.)/(1.+D1*T+D2*T**2.+D3*T**3.)
\ 	IF (IFLAG) XP = -XP

    fdup F* 1e fswap F/ fln fsqrt
    fdup C{ 2 }Horner  fover D{ 3 }Horner F/ F-
    iflag @ IF fnegate THEN
;

BASE !
