\ quad-random-test.fs
\
\ Generate random coefficients a, b, c for quadratic solver
\ and test the roots for consistency.
\
\ From an idea posted to comp.lang.forth by Marcel Hendrix,
\ 20 August 2009
\
\ K. Myneni, krishna.myneni@ccreweb.org
\
\ Notes:
\ 
\ Last Revised: 2009-09-07

include fsl-util.fs
include complex.fs
\ include quadratic-v1.2c.fs
include ceva4.fs

[UNDEFINED] fsquare [IF] : fsquare fdup f* ; [THEN]
[UNDEFINED] z2dup   [IF] : z2dup   zover zover ; [THEN]
[UNDEFINED] z2drop  [IF] : z2drop  zdrop zdrop ; [THEN]

1e 0e f/      fconstant +inf
+inf fnegate  fconstant -inf
0e 0e f/      fconstant +nan
+nan fnegate  fconstant -nan


defer random-val

1 [IF]
include horner.fs
include noise.fs

: random-sign ( F: x --  x | -x )
    ran0 0.5e f< IF fnegate THEN ;

\ Generate a random value between -1, 1
: random-val-uniform ( F: -- x )
    ran0 random-sign ;

[ELSE]

\ You may substitute your own random number generator here, 
\ if you wish; just invert the flag.
\
\ : random-sign ( F: x -- x | -x )
\ : random-val-uniform ( F: -- x | generate a random value between -1, 1)

[THEN]

\ A wider dynamic range of coefficients will result from a
\ distribution which is logarithmic
: random-val-log ( F: --x | random value between -1e10, 1e10 )
    random-val-uniform 10e f* falog 
    random-sign ;

\ ' random-val-log is random-val  cr .( Using Logarithmic Sampling ) cr
' random-val-uniform is random-val  cr .( Using Uniform Sampling ) cr

fvariable aa
fvariable bb
fvariable cc

: gen-random-coeffs ( F: -- )
    BEGIN
      random-val 
      fdup f0=
    WHILE
    REPEAT         \ don't allow a=0
    random-val
    random-val
    cc f! bb f! aa f!
;

[UNDEFINED] (q_discriminant) [IF]
: (q_discriminant) ( F: -- d )
    bb f@ fsquare 4e aa f@ f* cc f@ f* f- ;
[THEN]

[UNDEFINED] q_root_error [IF]

\ Return the complex residual for the given root
\ zq = a*z^2 + b*z + c ; computed with Horner's algorithm
: zq_residual ( F: zroot -- zres )
    zdup aa f@ z*f bb f@ 0e z+ z* cc f@ 0e z+ ;

\ Return the error in the magnitude of the root, given the root
: q_root_error ( F: zroot -- rerror )
    zq_residual |z|
    (q_discriminant) fabs fsqrt f/ ;

[THEN]

fvariable tol
fvariable max-residual
fvariable max-abs-error
fvariable max-rel-error

: root-relative-error ( F: magerror zroot -- relerror )
    |z| f/ ;

: update-max-abs-error ( F: r -- )
    max-abs-error f@ fmax max-abs-error f! ;

: update-max-rel-error ( F: r -- )
    max-rel-error f@ fmax max-rel-error f! ;
 
: update-max-residual ( F: r -- )
    max-residual f@ fmax max-residual f! ;

\ Check whether or not residuals for both roots are within tolerance.
\ Return true if within tolerance.
: check-residuals ( F: z1 z2 -- flag )
     zq_residual |z| fdup update-max-residual tol f@ f< >r
     zq_residual |z| fdup update-max-residual tol f@ f< r> and
;

\ Calculate the absolute error and estimated relative error of the
\ roots.
: calc-root-errors ( F: z1 z2 -- )
    zdup q_root_error fdup update-max-abs-error 
    -frot root-relative-error update-max-rel-error 
    zdup q_root_error fdup update-max-abs-error
    -frot root-relative-error update-max-rel-error
;

\ Check whether roots contain nan or inf
: check-roots-nan ( F: z1 z2 -- flag )
    fabs +nan 0e f~    fabs +nan 0e f~  or 
    fabs +nan 0e f~ or fabs +nan 0e f~  or
;

: check-roots-inf ( F: z1 z2 -- flag )
    fabs +inf f=    fabs +inf f=  or 
    fabs +inf f= or fabs +inf f=  or
;

variable failures
variable n_infs
variable n_nans
variable verbose

: print-failure-case ( z1 z2 -- ) 
    cr aa f@ fs. 2 spaces bb f@ fs. 2 spaces cc f@ fs. 5 spaces
    cr zs. 2 spaces zs. 
;

: tol-test ( F: tol -- ) ( uvals -- )
    tol f! 
    0e max-residual  f!
    0e max-abs-error f!
    0e max-rel-error f!
    0  failures  !
    0  n_infs !
    0  n_nans !
    0 ?DO
      gen-random-coeffs 
      aa f@ bb f@ cc f@ solve_quadratic
      \ z2dup check-roots-nan IF 1 n_nans +!  THEN
      \ z2dup check-roots-inf IF 1 n_infs +!  THEN 
      z2dup check-residuals invert IF 
       verbose @ IF z2dup print-failure-case THEN
       1 failures +! 
      THEN
      calc-root-errors
    LOOP  
;

: test ( uvals -- )
   false verbose !
   1e-16 tol f!
   cr ." Tolerance    Failures    Max. Residual   Max. Abs.  Max. Rel."
   cr ." -------------------------------------------------------------"
   10 0 DO
      dup tol f@ 
      fdup cr fs. 5 spaces
      tol-test 
      failures      @  7 .r 5 spaces
      \ n_infs      @  5 .r 5 spaces
      \ n_nans      @  5 .r 5 spaces
      max-residual  f@  fs. 5 spaces
      max-abs-error f@  fs. 5 spaces
      max-rel-error f@  fs.  
      tol f@ 10e f* tol f!
   LOOP
   drop 
   cr ." -----------------------------------------------------------"
   cr
;

1 [IF]
2 set-precision
cr
cr .( Testing solve_quadratic with 10 million random sets of coeffs. )
cr
10000000 test
[ELSE]

\ Display the coefficients which fail the residual tolerance test at 1E-11
 true verbose !
 1e-11 10000000 tol-test
[THEN]
