\ quad-random-test.fs
\
\ Generate random coefficients a, b, c for quadratic solver
\ and test the roots for consistency.
\
\ From an idea posted to comp.lang.forth by Marcel Hendrix,
\ 20 August 2009
\
\ K. Myneni, krishna.myneni@ccreweb.org
\ Last Revised: 2009-08-26

include fsl-util.fs
include complex.fs
\ include quadratic-hdkm-v1.3.fs
include quadratic-v1.2b.fs

1 [IF]
include horner.fs
include noise.fs

: random-sign ( F: x --  x | -x )
    ran0 0.5e f< IF fnegate THEN ;


: random-val ( F: -- x | generate a random value between -1, 1)
    ran0 random-sign ;
[ELSE]

\ You may substitute your own random number generator here, 
\ if you wish; just invert the flag.
\
\ : random-val ( F: -- x | generate a random value between -1, 1)
[THEN]

fvariable aa
fvariable bb
fvariable cc

: gen-random-coeffs ( F: -- )
    BEGIN
      random-val 
      fdup f0=
    WHILE
    REPEAT         \ don't allow a=0
    random-val
    random-val
    cc f! bb f! aa f!
;

fvariable tol

\ zq = a*z^2 + b*z + c ; computed with Horner's algorithm
: zquadratic ( F: z -- zq )
    zdup aa f@ z*f bb f@ 0e z+ z* cc f@ 0e z+ ;

\ Check whether or not both roots are within tolerance.
\ Return true if within tolerance.
: check-roots ( F: z1 z2 -- flag )
     zquadratic |z| tol f@ f< >r
     zquadratic |z| tol f@ f< r> and
;


: tol-test ( F: tol -- ) ( uvals -- failures)
    tol f! 
    0 swap
    0 ?DO
      gen-random-coeffs 
      aa f@ bb f@ cc f@ solve_quadratic
      check-roots invert IF 1+ THEN
    LOOP  
;

: test ( uvals -- )
   1e-16 tol f!
   cr ." Tolerance    Failures "
   cr ." ----------------------"
   7 0 DO
      dup tol f@ 
      fdup cr fs. 4 spaces
      tol-test .
      tol f@ 10e f* tol f!
   LOOP
   drop 
   cr ." ----------------------"
   cr
;

2 set-precision
cr
cr .( Testing solve_quadratic with 10 million random sets of coeffs. )
cr .( The tolerance is the distance from zero in the complex plane, so )
cr .( it represents an absolute error rather than a relative error. )
cr
10000000 test



