// kForth.cpp
//
// The kForth environment
//
// Copyright (c) 1998--2004 Krishna Myneni
//   Creative Consulting for Research and Education
// 
// This software is provided under the terms of the GNU General Public License.
//
// Contributions by (source code, bug fixes, documentation, packaging, misc):
//
//    David P. Wallace          input line history, default directory, cygwin
//                                port, misc.
//    Matthias Urlichs          maintains Debian package
//    Guido Draheim             maintains RPM packages
//    Brad Knotwell             interpreter Ctrl-D handling, dictionary
//                                initialization, input handling for outer
//                                interpreter.
//    Alaric B. Snell           command line parsing
//    Todd Nathan               ported kForth to BeOS
//    Bdale Garbee              originally created Debian kForth package
//    Christopher M. Brannon    bug alert for default-directory handling
//
// Usage from console prompt:
//
//      kforth [name[.4th]] [-D] [-e string]
//
char* version = "1.2.2";
char* Rls_Date = "2004-04-08";

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>
using std::istream;
using std::ostream;
using std::cout;
using std::endl;
using std::stringstream;
using std::string;
using std::vector;
extern "C" {
#include <stdio.h>
#include <readline/readline.h>
#include <readline/history.h>
}
#include "fbc.h"
#include "ForthCompiler.h"
#include "ForthVM.h"

extern vector<DictionaryEntry> Dictionary;
extern char* C_ErrorMessages[];

extern "C" int* JumpTable;
extern "C" int* BottomOfStack;
extern "C" int* BottomOfReturnStack;
extern "C" char TIB[];
bool debug = false;

int main(int argc, char *argv[])
{
    char name[256], InFileName[256], OutFileName[256], *cp, ch;
    char *input_line;
    stringstream inputstream("");
    char* prompt = " ok\n";
    vector<byte> op;
    int nWords, i, ec;

    if (argc < 2)
      {
	cout << "kForth v " << version << "\t (Rls. " << Rls_Date << ")" << endl;
	cout << "Copyright (c) 1998--2004 Krishna Myneni and David P. Wallace" 
	     << endl;
        cout << "Contributions by: gd mu bk abs tn cmb bg" << endl;
	cout << "Provided under the GNU General Public License." << endl << endl;
      }

    nWords = OpenForth();

    i = 1;

    while (i < argc)
      {
	if (!strcmp(argv[i], "-D"))
	  {
	    debug = true;
	  }
	else if (!strcmp(argv[i], "-e"))
	  {
	    if (argc > i)
	      {
		  inputstream << argv[i+1] << endl;
	      }
	    ++i;
	  }
	else
	  {
	      inputstream << "include " << argv[i] << endl;
	  }
	++i;
      }

    if (debug) cout << '\n' << nWords << " words defined.\n";

    int* sp;
    byte* tp;
    int line_num = 0;

    if (debug) 
    {
	cout << "Jump Table address:  " << JumpTable << endl;
	cout << "Bottom of Stack:     " << BottomOfStack << endl;
	cout << "Bottom of Ret Stack: " << BottomOfReturnStack << endl;
    }

    SetForthInputStream (inputstream);
    SetForthOutputStream (cout);

    if (inputstream.str() == string(""))
	cout << "\nReady!\n";
    else
	goto skip_readline;

//----------------  the interpreter main loop

    while (1)
    {

        // Obtain commands and execute

        do
        {
            if ((input_line = readline(prompt)) == NULL) CPP_bye();
            if (strlen(input_line)) add_history(input_line);
            inputstream.str(input_line);
            inputstream.str()[255] = 0;
            free(input_line);
skip_readline:	    
            ec = ForthCompiler (&op, &line_num);

	    inputstream.clear();
        } while (ec == E_C_ENDOFSTREAM) ;   // test for premature end of input
                                            //   that spans multiple lines
        if (ec && ec < MAX_ERR_MESSAGES)
        {
	    cout << "Line " << line_num << ": " << C_ErrorMessages[ec] << endl;
	    cout << TIB << endl;
        }
        op.erase(op.begin(), op.end());
    }
}
//---------------------------------------------------------------

