\ Dets      Determinant of a matrix in LU form

\ Forth Scientific Library Algorithm #34

\ det       The determinant of a matrix in LU form
\ det-i     The determinant of a matrix in LU form, result returned as
\           as a factor and a power of 10 (useful for very large and
\           very small determinants).

\ Presumes that the matrix has been converted in LU form (using LUFACT)
\ before being called.

\ This code is an ANS Forth program requiring:
\      1. The Floating-Point word set
\      2. Uses words 'Private:', 'Public:' and 'Reset_Search_Order'
\         to control the visibility of internal code.
\      3. Uses the words 'FLOAT' and 'DARRAY' to create floating point arrays
\         plus 'INTEGER' to create integer arrays.
\      4. The word '}' to dereference a one-dimensional array, and '}}' to
\         dereference two dimensional arrays.
\      5. Uses the words 'DARRAY' and '&!' to set array pointers.
\      6. The compilation of the test code is controlled by the VALUE TEST-CODE?
\         and the conditional compilation words in the Programming-Tools wordset
\      7. The test code uses 'HILBERT' and 'HILBERT-DET' for generating the testt

\ see,
\ Baker, L., 1989; C Tools for Scientists and Engineers,
\ McGraw-Hill, New York, 324 pages,   ISBN 0-07-003355-2


\  (c) Copyright 1994 Everett F. Carter.  Permission is granted by the
\  author to use this software for any application provided this
\  copyright notice is preserved.


CR .( DETS              V1.0           13 October 1994   EFC )


FLOAT  DMATRIX a{{            \ pointer to users matrix
INTEGER DARRAY pivot{          \ pointer to users array of LU pivots

: ?odd ( n -- t/f )
     DUP 2/ 2* -
;

0 value nexp

: large-det ( fdet -- fdet' ) 

    BEGIN
	0.10e F*
	nexp 1+ to nexp
	10.0e FOVER FABS F< 0=
    UNTIL

;

: small-det ( fdet -- fdet' ) 

    BEGIN
	10.0e F*
	nexp 1- to nexp
	FDUP FABS 0.10e F< 0=
    UNTIL

;

0 ptr a{{
0 ptr pivot{
0 value npiv

: det ( 'lu -- fdet ) 

    DUP ->matrix{{ a@ to a{{ 
    DUP ->pivot{   a@ to pivot{ 
    0 to npiv
    
    1e 
    ROT ->N @ 0 DO
	a{{ I I }} F@ F*
	pivot{ I } @ I = 0= IF npiv 1+ to npiv THEN
    LOOP

    npiv ?odd IF FNEGATE THEN
    
;


: deti ( 'lu -- nexp fx )     \ det = fx * 10^nexp

    DUP ->matrix{{ a@ to a{{ 
    DUP ->pivot{   a@ to pivot{
    0 to npiv  0 to nexp
    
    1e
    ROT ->N @ 0 DO
	a{{ I I }} F@ F*
	pivot{ I } @ I = 0= IF npiv 1+ to npiv THEN
	
	10e FOVER FABS F< IF
	    large-det
        ELSE
	    FDUP FABS 0.10e F< IF
		small-det
	    THEN
	THEN

    LOOP

    npiv ?odd IF FNEGATE THEN
    nexp -ROT
               
;


TEST-CODE? [IF]   \ test code ==============================================

[undefined] hilbert [IF] include hilbert.4th [THEN]
    
\ test code, creates a finite segment of a Hilbert matrix of the specified
\ size and gets its determinant.  Uses the known form for the determinant
\ of these matrices to calculate the comparison value.

\ Dynamically allocated array space
FLOAT DMATRIX mat{{

LUMATRIX lmat
0 value N

: det-test ( n -- )
    to N
    & mat{{ N N }}malloc  malloc-fail? ABORT" malloc failure (1) "

    mat{{ N  hilbert

    CR ." A: " CR
    N N mat{{ }}fprint

    lmat N lu-malloc 

    mat{{ lmat lufact
    lmat det

    CR ." DET(A): " F.
        
    lmat deti

    F. ." X 10^ " . CR

    \ now calculate the determinant directly, since it is a Hilbert matrix

    N hilbert-det 

    CR ." DET-Hilbert: " F. CR

    & mat{{ }}free
    lmat lu-free

;

[THEN]

