\    File:  divtest.4th
\   Title:  Multiple cell division tests
\  Author:  David N. Williams
\ License:  Public Domain
\ Started:  March 14, 2006
\ Revised:  March 15, 2006
\ Revised:  March 16, 2006
\ Revised:  March 20, 2006
\ Revised:  April 6, 2006
\ Revised:  April 8, 2006

include tester.4th
include ans-words.4th  \  for [IF] [THEN]

\ Stack effects:

\ UTS/MOD   ( num.lo num.mi num.hi denom -- rem quo.lo quo.mi quo.hi )
\ STS/REM   ( num.lo num.mi num.hi denom -- rem quo.lo quo.mi quo.hi )
\ UTM/      ( num.lo num.mi num.hi denom -- quo.lo quo.mi )
\ UM/MOD    ( num.lo num.hi denom -- rem quo.lo )

\ These tests are an elaboration of the UM/MOD tests in John
\ Hayes' core.4th.  They were written to test the author's
\ PowerPC assembly language division subroutines, written
\ initially for Krishna Myneni's kForth.

\ Four underlying, restoring shift and subtract division
\ subroutines are used in the implementation, depending on the
\ number of significant bits in the numerator.  Each of these
\ subroutines also has two subcases, depending on whether the
\ numerator has full 32-bit significance in its most significant
\ cell or not.

\ The four basic routines are called by four driver routines
\ that handle signs and analyze cases, including whether to use
\ the normal ppc divwu assembly language op.

\ These tests attempt to cover the corners, plus the normal
\ stuff for the nonstandard words UTS/MOD and STS/REM.

\ Dependences:

\ UTS/MOD  udiv96by32
\ UTM/     udiv96by32
\ UM/MOD   udiv64by32
\ STS/REM  sdiv96by32
\ SM/REM   sdiv64by32
\ FM/MOD   sdiv64by32
\ */       sdiv64by32
\ */MOD    sdiv64by32

\ sdiv96by32   udiv96by32
\ sdiv64by32   udiv64by32
\ udiv96by32   udiv96by32to96, udiv96by32to64,
\              udiv64by32to64, udiv64by32to32
\ udiv64by32   udiv64by32to64, udiv64by32to32


\ from core.4th:
0 INVERT                 CONSTANT  maxuint
0 INVERT 1 RSHIFT        CONSTANT  maxint
0 INVERT 1 RSHIFT INVERT CONSTANT  minint

hex

TESTING UTS/MOD
\ driver: udiv96by32

\ ppc divwu

TESTING num.hi = num.mi = 0

{  0  0  0       1 uts/mod -> 0  0  0  0 }
{  1  0  0       1 uts/mod -> 0  1  0  0 }
{  1  0  0       2 uts/mod -> 1  0  0  0 }
{  3  0  0       2 uts/mod -> 1  1  0  0 }
{ -1  0  0 1 uts/mod -> 0 -1  0  0 }

\ udiv64by32to64

TESTING num.mi.sd >= denom.sd

{ -1 -1  0  1 uts/mod -> 0 -1 -1  0 }
{ -2 1 0          1 uts/mod -> 0 -2 1 0 }
{ -2 3 0          2 uts/mod -> 0 -1 1 0 }
{ -1 3 0          2 uts/mod -> 1 -1 1 0 }

TESTING   num.mi > denom, denom.sd = 32

{ 1 -1 0        -2 uts/mod -> 3 1 1 0 }
{ 0 -1 0        -2 uts/mod -> 2 1 1 0 }

TESTING   num.mi = denom, denom.sd = 32

{ -1 -2 0 -2 uts/mod -> 1 1 1 0 }
{ -2 -2 0 -2 uts/mod -> 0 1 1 0 }
{ fffffffd -2 0 -2 uts/mod -> fffffffd 0 1 0 }

TESTING   num.mi < denom, denom.sd = 32

{ 2 -2 0        -1 uts/mod -> 1 -1 0 0 }
{ 1 -2 0        -1 uts/mod -> 0 -1 0 0 }
{ 0 -2 0        -1 uts/mod -> -2 -2 0 0 }

\ udiv64by32to32

TESTING num.mi.sd < denom.sd

{ -2 1 0   2 uts/mod -> 0 -1 0 0 }
{ -1 1 0   2 uts/mod -> 1 -1 0 0 }
{ fffffff0 3 0  10 uts/mod -> 0 3fffffff 0 0 }
{ fffffff8 3 0  10 uts/mod -> 8 3fffffff 0 0 }
{ -2 1 0   2 uts/mod -> 0 -1 0 0 }

TESTING   num.mi < denom, denom.sd = 32

{ -2 1 0        -1 uts/mod -> 0  2  0  0 }
{ 10000001 efffffff 0 -1 uts/mod -> 1 f0000000 0 0 }
{ 10000000 efffffff 0 -1 uts/mod -> 0 f0000000 0 0 }
{ 0fffffff efffffff 0 -1 uts/mod -> -2 efffffff 0 0 }

\ udiv96by32to96

TESTING num.hi.sd >= denom.sd

{ -1 -1 -1 1 uts/mod -> 0 -1 -1 -1 }
{ -2 -1 1        1 uts/mod -> 0 -2 -1 1 }
{ -2 -1 3        2 uts/mod -> 0 -1 -1 1 }
{ -1 -1 3        2 uts/mod -> 1 -1 -1 1 }
{ 0 1 -2        -1 uts/mod -> 0  0 -1  0 }

\ udiv96by32to64

TESTING num.hi.sd < denom.sd

{ -2 -1 1  2 uts/mod -> 0 -1 -1 0 }
{ -1 -1 1  2 uts/mod -> 1 -1 -1 0 }
{ fffffff0 -1 3 10 uts/mod -> 0 -1 3fffffff 0 }
{ fffffff8 -1 3 10 uts/mod -> 8 -1 3fffffff 0 }
{ -2 -1 1  2 uts/mod -> 0 -1 -1  0 }
{ 0 -2 1  -1 uts/mod -> 0  0  2  0 }

TESTING STS/REM

\ Because it's based on the same underlying unsigned division
\ code as UTS/MOD, we just adapt the core.4th SM/REM tets, and
\ check most positive number overflow.

{  0  0  0  1 sts/rem ->  0  0  0  0 }
{  1  0  0  1 sts/rem ->  0  1  0  0 }
{  2  0  0  1 sts/rem ->  0  2  0  0 }
{ -1 -1 -1  1 sts/rem ->  0 -1 -1 -1 }
{ -2 -1 -1  1 sts/rem ->  0 -2 -1 -1 }
{  0  0  0 -1 sts/rem ->  0  0  0  0 }
{  1  0  0 -1 sts/rem ->  0 -1 -1 -1 }
{  2  0  0 -1 sts/rem ->  0 -2 -1 -1 }
{ -1 -1 -1 -1 sts/rem ->  0  1  0  0 }
{ -2 -1 -1 -1 sts/rem ->  0  2  0  0 }
{  2  0  0  2 sts/rem ->  0  1  0  0 }
{ -1 -1 -1 -1 sts/rem ->  0  1  0  0 }
{ -2 -1 -1 -2 sts/rem ->  0  1  0  0 }
{  7  0  0  3 sts/rem ->  1  2  0  0 }
{  7  0  0 -3 sts/rem ->  1 -2 -1 -1 }
{ -7 -1 -1  3 sts/rem -> -1 -2 -1 -1 }
{ -7 -1 -1 -3 sts/rem -> -1  2  0  0 }
{ -1 maxint  1  1 sts/rem -> 0 -1 maxint 1 }
{  0 minint  0  1 sts/rem -> 0 0 minint 0 }
{  0 maxint  0  1 sts/rem -> 0 0 maxint 0 }
{  minint   -1 -1 minint sts/rem -> 0 1 0 0 }
{ -1  1  0  4 sts/rem ->  3 maxint  0  0 }
{  2  0  minint ds* 2 sts/rem ->  0 minint -1 -1 }
{  2  0  minint ds* minint sts/rem ->  0  2  0  0 }
{  2  0  maxint ds* 2 sts/rem ->  0  maxint  0  0 }
{  0 minint minint ds* minint sts/rem ->  0  0 minint -1 }
{  0 minint maxint ds* minint sts/rem ->  0  0 maxint  0 }
{  0 minint maxint ds* maxint sts/rem ->  0  0 minint -1 }
{ -1 maxint maxint ds* maxint sts/rem ->  0 -1 maxint  0 }

\ Uncomment a line to test divide by zero.
\ {  0 0 0 0 sts/rem -> 0 }
\ { -1 0 0 0 sts/rem -> 0 }
\ {  0 -1  0  0 sts/rem -> 0 }
\ {  0  0 -1  0 sts/rem -> 0 }
\ {  1  2  3  0 sts/rem -> 0 }

\ Uncomment the line to test postive number overflow.
\ { 0 0 minint -1 sts/rem -> 0 }

TESTING UTM/
\ driver: udiv96by32

\ ppc divwu

TESTING num.hi = num.mi = 0

{  0  0  0       1 utm/ -> 0  0 }
{  1  0  0       1 utm/ -> 1  0 }
{  1  0  0       2 utm/ -> 0  0 }
{  3  0  0       2 utm/ -> 1  0 }
{ -1  0  0 1 utm/ -> -1  0 }

\ udiv64by32to64

TESTING num.mi.sd >= denom.sd

{ -1 -1  0  1 utm/ -> -1 -1 }
{ -2 1 0          1 utm/ -> -2 1 }
{ -2 3 0          2 utm/ -> -1 1 }
{ -1 3 0          2 utm/ -> -1 1 }

TESTING   num.mi > denom, denom.sd = 32

{ 1 -1 0        -2 utm/ -> 1 1 }
{ 0 -1 0        -2 utm/ -> 1 1 }

TESTING   num.mi = denom, denom.sd = 32

{ -1 -2 0 -2 utm/ -> 1 1 }
{ -2 -2 0 -2 utm/ -> 1 1 }
{ fffffffd -2 0 -2 utm/ -> 0 1 }

TESTING   num.mi < denom, denom.sd = 32

{ 2 -2 0        -1 utm/ -> -1 0 }
{ 1 -2 0        -1 utm/ -> -1 0 }
{ 0 -2 0        -1 utm/ -> -2 0 }

\ udiv64by32to32

TESTING num.mi.sd < denom.sd

{ -2 1 0   2 utm/ -> -1 0 }
{ -1 1 0   2 utm/ -> -1 0 }
{ fffffff0 3 0  10 utm/ -> 3fffffff 0 }
{ fffffff8 3 0  10 utm/ -> 3fffffff 0 }
{ -2 1 0   2 utm/ -> -1 0 }

TESTING   num.mi < denom, denom.sd = 32

{ -2 1 0        -1 utm/ ->  2  0 }
{ 10000001 efffffff 0 -1 utm/ -> f0000000 0 }
{ 10000000 efffffff 0 -1 utm/ -> f0000000 0 }
{ 0fffffff efffffff 0 -1 utm/ -> efffffff 0 }

\ udiv96by32to96

TESTING num.hi.sd >= denom.sd

{ 0 1 -2        -1 utm/ -> 0 -1 }

\ Uncomment individual lines to test divide by zero.
\ { 0 0 0  0 utm/ -> 0 }
\ { 1 0 0  0 utm/ -> 0 }
\ { 0 1 0  0 utm/ -> 0 }
\ { 0 0 1  0 utm/ -> 0 }

\ Uncomment individual lines to test division overflow into the
\ third cell:
\ { -1 -1 -1  1 utm/ -> -1 -1 } \ -1
\ { -2 -1  1  1 utm/ -> -2 -1 } \ 1
\ { -2 -1  3  2 utm/ -> -1 -1 } \ 1
\ { -1 -1  3  2 utm/ -> -1 -1 } \ 1

\ udiv96by32to64

TESTING num.hi.sd < denom.sd

{ -2 -1 1  2 utm/ -> -1 -1 }
{ -1 -1 1  2 utm/ -> -1 -1 }
{ fffffff0 -1 3 10 utm/ -> -1 3fffffff }
{ fffffff8 -1 3 10 utm/ -> -1 3fffffff }
{ -2 -1 1  2 utm/ -> -1 -1 }
{ 0 -2 1  -1 utm/ -> 0  2 }

\ TESTING UM/MOD FM/REM
comment Use core.4th for UM/MOD FM/REM nonexception tests.
\ drivers: udiv64by32 and sdiv64by32

\ Copied from core.4th:
0 [IF]
{ 0 0 1 UM/MOD -> 0 0 }
{ 1 0 1 UM/MOD -> 0 1 }
{ 1 0 2 UM/MOD -> 1 0 }
{ 3 0 2 UM/MOD -> 1 1 }
{ -1 2 UM*               2 UM/MOD -> 0 -1 }
{ -1 2 UM*        -1 UM/MOD -> 0 2 }
{ -1 -1 UM* -1 UM/MOD -> 0 -1 }
[THEN]

\ Uncomment a line to test divide by zero.
\ { 0 0 0 um/mod -> 0 }
\ { 1 0 0 um/mod -> 0 }
\ { 0 1 0 um/mod -> 0 }

\ Uncomment a line to test division overflow into the second
\ cell.
\ { -1 -1 1 UM/MOD -> 0 } \ -1 -1
\ { -2 1        1 UM/MOD -> 0 } \ -2 1
\ { -2 3        2 UM/MOD -> 0 } \ -1 1
\ { -1 3        2 UM/MOD -> 1 } \ -1 1

\ Because there are so many, we do not copy the core.4th FM/MOD
\ tests.

\ Uncomment a line to test divide by zero.
\ { 0 0 0 fm/mod -> 0 }
\ { 1 0 0 fm/mod -> 0 }
\ { 0 1 0 fm/mod -> 0 }

\ Uncomment the line to test postive number overflow.
\ { 0 minint -1 fm/mod -> 0 }

\ Uncomment a line to test division overflow into the second
\ cell.
\ { -1  0  1 fm/mod -> 0 } \ -1 0
\ { 3fffffff  1  1 fm/mod -> 0 } \ 3fffffff 1
\ { 3fffffff -1  1 fm/mod -> 0 } \ 3fffffff -1
\ { -2  3 -2 fm/mod -> 0 } \ 1 -2
\ {  0  3 -2 fm/mod -> 1 } \ minint -2

testing SM/REM

{  0  0  1 sm/rem ->  0  0 }
{  1  0  1 sm/rem ->  0  1 }
{  2  0  1 sm/rem ->  0  2 }
{ -1 -1  1 sm/rem ->  0 -1 }
{ -2 -1  1 sm/rem ->  0 -2 }
{  0  0 -1 sm/rem ->  0  0 }
{  1  0 -1 sm/rem ->  0 -1 }
{  2  0 -1 sm/rem ->  0 -2 }
{ -1 -1 -1 sm/rem ->  0  1 }
{ -2 -1 -1 sm/rem ->  0  2 }
{  2  0  2 sm/rem ->  0  1 }
{ -1 -1 -1 sm/rem ->  0  1 }
{ -2 -1 -2 sm/rem ->  0  1 }
{  7  0  3 sm/rem ->  1  2 }
{  7  0 -3 sm/rem ->  1 -2 }
{ -7 -1  3 sm/rem -> -1 -2 }
{ -7 -1 -3 sm/rem -> -1  2 }
{ maxint  0  1 sm/rem -> 0 maxint }
{ minint -1  1 sm/rem -> 0 minint }
{ minint -1 minint sm/rem -> 0 1 }
{ -1  1  4 sm/rem ->  3 maxint }
{  2  minint m* 2 sm/rem ->  0 minint }
{  2  minint m* minint sm/rem ->  0  2 }
{  2  maxint m* 2 sm/rem ->  0  maxint }
{  minint minint m* minint sm/rem ->  0 minint }
{  minint maxint m* minint sm/rem ->  0 maxint }
{  minint maxint m* maxint sm/rem ->  0 minint }
{  maxint maxint m* maxint sm/rem ->  0 maxint }

\ Uncomment a line to test divide by zero.
\ { 0 0 0 sm/rem -> 0 }
\ { 1 0 0 sm/rem -> 0 }
\ { 0 1 0 sm/rem -> 0 }

\ Uncomment the line to test postive number overflow.
\ { 0 minint -1 sm/rem -> 0 }

\ Uncomment a line to test division overflow into the second
\ cell.
\ { -1  0  1 sm/rem ->  0 } \ -1 0
\ { 3fffffff  1  1 sm/rem ->  0 } \ 3fffffff 1
\ { 3fffffff -1  1 sm/rem ->  0 } \ 3fffffff -1
\ { -2  3 -2 sm/rem ->  0 } \ 1 -2
\ { -1  0  1 sm/rem -> -1 } \ -1 -1

comment Uncomment lines in divtest.4th to test error reports.
