\     File:  div-test.4th
\    Title:  Multiple cell division tests
\   Author:  David N. Williams
\  License:  Public Domain
\  Started:  March 14, 2006
\  Revised:  March 15, 2006
\  Revised:  March 16, 2006
\  Revised:  March 20, 2006

include tester.4th

\ Stack effects:

\ UTS/MOD   ( num.lo num.mi num.hi denom -- rem quo.lo quo.mi quo.hi )
\ STS/REM   ( num.lo num.mi num.hi denom -- rem quo.lo quo.mi quo.hi )
\ UTM/      ( num.lo num.mi num.hi denom -- quo.lo quo.mi )
\ UM/MOD    ( num.lo num.hi denom -- rem quo.lo )

\ These tests are an elaboration of the UM/MOD tests in John
\ Hayes' core.4th.  They were written to test the author's
\ PowerPC assembly language division subroutines, written
\ initially for Krishna Myneni's kForth.

\ Four underlying, restoring shift and subtract division
\ subroutines are used in the implementation, depending on the
\ number of significant bits in the numerator.  Each of these
\ subroutines also has two subcases, depending on whether the
\ numerator has full 32-bit significance in its most significant
\ cell or not.

\ The four basic routines are called by four driver routines
\ that handle signs and analyze cases, including whether to use
\ the normal ppc divwu assembly language op.

\ These tests attempt to cover the corners, plus the normal
\ stuff for the nonstandard words UTS/MOD and STS/REM.

\ Dependences:

\ UTS/MOD  udiv96by32
\ UTM/     udiv96by32
\ UM/MOD   udiv64by32
\ STS/REM  sdiv96by32
\ SM/REM   sdiv64by32
\ FM/MOD   sdiv64by32
\ */       sdiv64by32
\ */MOD    sdiv64by32

\ sdiv96by32   udiv96by32
\ sdiv64by32   udiv64by32
\ udiv96by32   udiv96by32to96, udiv96by32to64,
\              udiv64by32to64, udiv64by32to32
\ udiv64by32   udiv64by32to64, udiv64by32to32

\ From core.4th:

\ { 0 0 1 UM/MOD -> 0 0 }
\ { 1 0 1 UM/MOD -> 0 1 }
\ { 1 0 2 UM/MOD -> 1 0 }
\ { 3 0 2 UM/MOD -> 1 1 }
\ { MAX-UINT 2 UM* 2 UM/MOD -> 0 MAX-UINT }
\ { MAX-UINT 2 UM* MAX-UINT UM/MOD -> 0 2 }
\ { MAX-UINT MAX-UINT UM* MAX-UINT UM/MOD -> 0 MAX-UINT }

hex
COMMENT SKIPPING UTS/MOD TESTS
(
TESTING UTS/MOD
\ driver: udiv96by32

\ ppc divwu

TESTING num.hi = num.mi = 0
{  0  0  0       1 uts/mod -> 0  0  0  0 }
{  1  0  0       1 uts/mod -> 0  1  0  0 }
{  1  0  0       2 uts/mod -> 1  0  0  0 }
{  3  0  0       2 uts/mod -> 1  1  0  0 }
{ ffffffff  0  0 1 uts/mod -> 0 ffffffff  0  0 }

\ udiv64by32to64

TESTING num.mi.sd >= denom.sd
{ ffffffff ffffffff  0  1 uts/mod -> 0 ffffffff ffffffff  0 }
{ fffffffe 1 0          1 uts/mod -> 0 fffffffe 1 0 }
{ fffffffe 3 0          2 uts/mod -> 0 ffffffff 1 0 }
{ ffffffff 3 0          2 uts/mod -> 1 ffffffff 1 0 }
TESTING   num.mi > denom, denom.sd = 32
{ 1 ffffffff 0        fffffffe uts/mod -> 3 1 1 0 }
{ 0 ffffffff 0        fffffffe uts/mod -> 2 1 1 0 }
TESTING   num.mi = denom, denom.sd = 32
{ ffffffff fffffffe 0 fffffffe uts/mod -> 1 1 1 0 }
{ fffffffe fffffffe 0 fffffffe uts/mod -> 0 1 1 0 }
{ fffffffd fffffffe 0 fffffffe uts/mod -> fffffffd 0 1 0 }
TESTING   num.mi < denom, denom.sd = 32
{ 2 fffffffe 0        ffffffff uts/mod -> 1 ffffffff 0 0 }
{ 1 fffffffe 0        ffffffff uts/mod -> 0 ffffffff 0 0 }
{ 0 fffffffe 0        ffffffff uts/mod -> fffffffe fffffffe 0 0 }

\ udiv64by32to32

TESTING num.mi.sd < denom.sd
{ fffffffe 1 0   2 uts/mod -> 0 ffffffff 0 0 }
{ ffffffff 1 0   2 uts/mod -> 1 ffffffff 0 0 }
{ fffffff0 3 0  10 uts/mod -> 0 3fffffff 0 0 }
{ fffffff8 3 0  10 uts/mod -> 8 3fffffff 0 0 }
{ fffffffe 1 0   2 uts/mod -> 0 ffffffff 0 0 }
TESTING   num.mi < denom, denom.sd = 32
{ fffffffe 1 0        ffffffff uts/mod -> 0  2  0  0 }
{ 10000001 efffffff 0 ffffffff uts/mod -> 1 f0000000 0 0 }
{ 10000000 efffffff 0 ffffffff uts/mod -> 0 f0000000 0 0 }
{ 0fffffff efffffff 0 ffffffff uts/mod -> fffffffe efffffff 0 0 }

\ udiv96by32to96

TESTING num.hi.sd >= denom.sd
{ ffffffff ffffffff ffffffff 1 uts/mod -> 0 ffffffff ffffffff ffffffff }
{ fffffffe ffffffff 1        1 uts/mod -> 0 fffffffe ffffffff 1 }
{ fffffffe ffffffff 3        2 uts/mod -> 0 ffffffff ffffffff 1 }
{ ffffffff ffffffff 3        2 uts/mod -> 1 ffffffff ffffffff 1 }
{ 0 1 fffffffe        ffffffff uts/mod -> 0  0 ffffffff  0 }

\ udiv96by32to64

TESTING num.hi.sd < denom.sd
{ fffffffe ffffffff 1  2 uts/mod -> 0 ffffffff ffffffff 0 }
{ ffffffff ffffffff 1  2 uts/mod -> 1 ffffffff ffffffff 0 }
{ fffffff0 ffffffff 3 10 uts/mod -> 0 ffffffff 3fffffff 0 }
{ fffffff8 ffffffff 3 10 uts/mod -> 8 ffffffff 3fffffff 0 }
{ fffffffe ffffffff 1  2 uts/mod -> 0 ffffffff ffffffff  0 }
{ 0 fffffffe 1  ffffffff uts/mod -> 0  0  2  0 }
)
TESTING UTM/
\ driver: udiv96by32

\ ppc divwu

TESTING num.hi = num.mi = 0
{  0  0  0       1 utm/ -> 0  0 }
{  1  0  0       1 utm/ -> 1  0 }
{  1  0  0       2 utm/ -> 0  0 }
{  3  0  0       2 utm/ -> 1  0 }
{ ffffffff  0  0 1 utm/ -> ffffffff  0 }

\ udiv64by32to64

TESTING num.mi.sd >= denom.sd
{ ffffffff ffffffff  0  1 utm/ -> ffffffff ffffffff }
{ fffffffe 1 0          1 utm/ -> fffffffe 1 }
{ fffffffe 3 0          2 utm/ -> ffffffff 1 }
{ ffffffff 3 0          2 utm/ -> ffffffff 1 }
TESTING   num.mi > denom, denom.sd = 32
{ 1 ffffffff 0        fffffffe utm/ -> 1 1 }
{ 0 ffffffff 0        fffffffe utm/ -> 1 1 }
TESTING   num.mi = denom, denom.sd = 32
{ ffffffff fffffffe 0 fffffffe utm/ -> 1 1 }
{ fffffffe fffffffe 0 fffffffe utm/ -> 1 1 }
{ fffffffd fffffffe 0 fffffffe utm/ -> 0 1 }
TESTING   num.mi < denom, denom.sd = 32
{ 2 fffffffe 0        ffffffff utm/ -> ffffffff 0 }
{ 1 fffffffe 0        ffffffff utm/ -> ffffffff 0 }
{ 0 fffffffe 0        ffffffff utm/ -> fffffffe 0 }

\ udiv64by32to32

TESTING num.mi.sd < denom.sd
{ fffffffe 1 0   2 utm/ -> ffffffff 0 }
{ ffffffff 1 0   2 utm/ -> ffffffff 0 }
{ fffffff0 3 0  10 utm/ -> 3fffffff 0 }
{ fffffff8 3 0  10 utm/ -> 3fffffff 0 }
{ fffffffe 1 0   2 utm/ -> ffffffff 0 }
TESTING   num.mi < denom, denom.sd = 32
{ fffffffe 1 0        ffffffff utm/ ->  2  0 }
{ 10000001 efffffff 0 ffffffff utm/ -> f0000000 0 }
{ 10000000 efffffff 0 ffffffff utm/ -> f0000000 0 }
{ 0fffffff efffffff 0 ffffffff utm/ -> efffffff 0 }

\ udiv96by32to96

TESTING num.hi.sd >= denom.sd
{ 0 1 fffffffe        ffffffff utm/ -> 0 ffffffff }
\ Uncomment individual lines to test division overflow into the
\ third cell:
\ { ffffffff ffffffff ffffffff 1 utm/ -> ffffffff ffffffff } \ ffffffff
\ { fffffffe ffffffff 1        1 utm/ -> fffffffe ffffffff } \ 1
\ { fffffffe ffffffff 3        2 utm/ -> ffffffff ffffffff } \ 1
\ { ffffffff ffffffff 3        2 utm/ -> ffffffff ffffffff } \ 1

\ udiv96by32to64

TESTING num.hi.sd < denom.sd
{ fffffffe ffffffff 1  2 utm/ -> ffffffff ffffffff }
{ ffffffff ffffffff 1  2 utm/ -> ffffffff ffffffff }
{ fffffff0 ffffffff 3 10 utm/ -> ffffffff 3fffffff }
{ fffffff8 ffffffff 3 10 utm/ -> ffffffff 3fffffff }
{ fffffffe ffffffff 1  2 utm/ -> ffffffff ffffffff }
{ 0 fffffffe 1  ffffffff utm/ -> 0  2 }

TESTING UM/MOD
\ driver: udiv64by32

\ Copied from core.4th:
{ 0 0 1 UM/MOD -> 0 0 }
{ 1 0 1 UM/MOD -> 0 1 }
{ 1 0 2 UM/MOD -> 1 0 }
{ 3 0 2 UM/MOD -> 1 1 }
{ ffffffff 2 UM*               2 UM/MOD -> 0 ffffffff }
{ ffffffff 2 UM*        ffffffff UM/MOD -> 0 2 }
{ ffffffff ffffffff UM* ffffffff UM/MOD -> 0 ffffffff }
\ Uncomment individual lines to test division overflow into the
\ second cell:
\ { ffffffff ffffffff 1 UM/MOD -> 0 ffffffff } \ ffffffff
\ { fffffffe 1        1 UM/MOD -> 0 fffffffe } \ 1
\ { fffffffe 3        2 UM/MOD -> 0 ffffffff } \ 1
\ { ffffffff 3        2 UM/MOD -> 1 ffffffff } \ 1
