\ gfit-fsl2.4th
\
\ Fit a double Gaussian function to data using the FSL version of curvefit 
\ fitting routines (see curvefit-fsl.4th). 
\
\ The fitting function is:
\
\	y = A*exp{-(x-mu1)^2/(2*sigma1^2)} + B*exp{-(x-mu2)^2/(2*sigma2^2)} + C
\
\ The parameters are stored in the array a{ in the following sequence:
\
\	A, mu1, sigma1, B, mu2, sigma2, C
\
\ Copyright (c) 2006 Krishna Myneni, Provided under the GNU General Public 
\   License
\
\ Revisions:
\
\	2006-03-17  modified from gfit-fsl.4th 

include ans-words
include fsl-util
include dynmem
include gaussj

\ Define the fitting function.

: functn ( fx a -- fy )
	>r
	fdup 
	r@ 1 } f@ f- fdup f*
	r@ 2 } f@ fdup f* 2e f* f/
	fnegate fexp
	r@ 0 } f@ f*
	fswap
	r@ 4 } f@ f- fdup f*
	r@ 5 } f@ fdup f* 2e f* f/
	fnegate fexp
	r@ 3 } f@ f*
	f+
	r> 6 } f@ f+
;

	
include curvefit-fsl   ( functn must be defined prior to loading curvefit-fsl)

FLOAT DARRAY  x{
FLOAT DARRAY  y{

0 value np	\ number of data points
7 value npar	\ number of parameters

: put_data ( y1 ... yn -- )
	& x{ np }malloc
	& y{ np }malloc

	np 0 do
	  s>f y{ np i - 1- } F!
	  i 1+ s>f x{ i } F!	\ x is a running index
	loop

;

include wfms01-1.dat	\ put the data from the file onto the stack
513 to np               \ number of data points in the file
put_data		\ move the data from the stack to the y matrix


npar FLOAT ARRAY  a{
npar FLOAT ARRAY  deltaa{

\ Setup initial values for the function parameters
\   ( see func_2gauss.4th for meaning of parameters) 

20000e  a{ 0 }  F!	\ A
206e    a{ 1 }  F!	\ mu1
100e    a{ 2 }  F!	\ sigma1
15000e  a{ 3 }  F!	\ B
390e    a{ 4 }  F!	\ mu2
100e    a{ 5 }  F!	\ sigma2
-15000e a{ 6 }  F!	\ C

\ The following are the initial increments for the
\   parameters, i.e. the amount each parameter may be
\   varied to try to improve the fit. Any function
\   parameter may be fixed to its initial value by
\   using 0e for its corresponding increment.

1e      deltaa{ 0 }  F!
0.1e    deltaa{ 1 }  F!
0.1e    deltaa{ 2 }  F!
1e      deltaa{ 3 }  F!
0.1e    deltaa{ 4 }  F!
0.1e    deltaa{ 5 }  F!
1e      deltaa{ 6 }  F!


: params. ( -- | display the current values of the parameters )
	cr npar 0 DO a{ i } f@ f. cr loop ;

: iterate ( -- | perform one iteration of curvefit and display results )
        x{ y{ a{ deltaa{ npar np curfit
	." Fitted Parameters:" cr params. cr 
	CR ." reduced chi-squared = " f. ;


: genfit ( -- | display the fitted data values )
	\ You can redirect the output to a file to plot it with another program
	\   e.g. '>file fit.dat genfit console'
	np 0 do
	  i s>f a{ functn         \ compute the fitted y value
	  i 1+ . 2 spaces f. cr   \ print the x and y values
	loop ; 

cr cr
.( Initial Parameters are: ) cr params. cr cr
.( Type 'iterate' to execute curfit once and print the results.) cr
.( Continue to 'iterate' until chi-square converges.) cr cr
.( When the fit converges, you may write the fitted data to a file) cr
.(   by typing, '>file fit.dat genfit console') cr cr




