# vm-fast.s
#
# The kForth Virtual Machine (fast version)
#
# Copyright (c) 1998--2006 Krishna Myneni, Creative Consulting for
#   Research and Education
#
# This software is provided under the terms of the General Public License.
#
# Usage from C++
#
#       extern "C" int vm (byte* ip);
#       ecode = vm(ip);
#
# Originally written for the A386 assembler
# Revisions:
#	3-21-98
#	8-25-1998 ported to GNU assembler under Linux
#	9-8-1998  additional functions added: c@, c!, f=, f<>
#	9-10-1998 added: and, or, not, xor
#	9-11-1998 added: = , <>, <, >, <=, >=
#	9-15-1998 added: +!, fsqrt, fsin, fcos, -rot, sp@, rp@
#	9-17-1998 added: a@
#	9-27-1998 added: mod, 2*, 2/
#	10-1-1998 added: pick
#	10-4-1998 fixed signed integer division and mod; added /MOD
#	10-6-1998 added ?dup
#	10-14-1998 added count
#	10-16-1998 fixed L_div error
#	10-19-1998 added 0<, 0=, 0>, true, false
#	10-20-1998 added 2+, 2-
#	02-09-1999 added execute
#	03-01-1999 added open, lseek, close, read, write
#	03-02-1999 added ioctl
#	03-03-1999 added usleep	
#	03-07-1999 added fill, cmove
#	03-09-1999 interchanged meaning of execute and call
#		   to be consistent with ANS Forth
#	03-27-1999 added +loop, unloop
#	03-29-1999 added roll
#	03-31-1999 added cmove>, key
#	05-05-1999 fixed +loop
#	05-06-1999 added fround
#	05-15-1999 added floor
#	05-26-1999 added fatan2, lshift, rshift
#	05-27-1999 added u<, quit, base
#	06-02-1999 added */, */mod
#	06-09-1999 call CPP functions from vm
#	07-18-1999 added find
//	09-06-1999 added pTIB, word, tick
//	09-12-1999 added system
#	10-04-1999 added create, variable, fvariable as intrinsic words
#	10-06-1999 added constant, fconstant as intrinsic words
#	10-07-1999 added chdir
#       10-08-1999 added erase, brackettick
#	10-09-1999 added time&date, ms, question, bl
#       10-10-1999 added char, forget, cold
#       10-20-1999 added >file, console
#	10-28-1999 added key?
#	12-24-1999 added hooks for f0=, f0<, f0>, u>, s>d, d>f, f>d, 
#	              um*, um/mod, m*, m+, m/, m*/
#	12-25-1999 added cells, cell+, dfloats, dfloat+
#	12-27-1999 added bye
#	01-08-2000 fixed f0<, increased vm loop efficiency
#	01-13-1999 added ?allot, fixed f0=
#	01-22-2000 modified + to remove ordering sensitivity for address arithmetic
#	01-23-2000 added 0<>, [char], .r, u.r, changed opcodes for 
#	             relational operators
#	01-24-2000 added CPP_literal, CPP_cquote, CPP_squote, CPP_dotquote
#	02-04-2000 implemented m*, um*
#	02-26-2000 fixed fm/mod, added sm/rem, um/mod
#	03-02-2000 modified QUIT to clear the return stacks, added CPP_do
#	03-05-2000 added CPP_begin, CPP_while, CPP_repeat, CPP_until, CPP_again,
#	             CPP_leave, CPP_if, CPP_else, CPP_then, CPP_lparen
#	05-17-2000 added CPP_does
#	05-18-2000 fix L_plusloop for negative increments
#	06-04-2000 fix L_roll to roll the typestack as well
#	06-11-2000 added CPP_case, CPP_endcase, CPP_of, and CPP_endof
#	06-15-2000 added CPP_querydo, CPP_abortquote
//	09-05-2000 handled overflow condition on um/mod; added code for m/;
#		     added m+	;  hooks for CPP_lbracket, CPP_rbracket, CPP_ddot
#	11-30-2000 added L_dabs, L_dnegate, L_udmstar, L_mstarslash
#	01-26-2001 changed jumptable address for usleep to C_usec
#	04-01-2001 changed jmpl to jmp for L_fzerolt
#	04-22-2001 added L_dplus, L_dminus
#	04-24-2001 added L_twopush, L_twopop, L_tworfetch
#       05-07-2001 added L_definition
#	05-08-2001 improved vm execution efficiency by about 10%
#	05-13-2001 added CPP_dotparen, L_dlt, L_dzeroeq, L_deq
#	05-20-2001 added CPP_sharp, CPP_bracketsharp, CPP_sharpbracket, 
#	             CPP_sharps, CPP_sign, CPP_hold; fixed problem with L_dnegate
#	07-12-2001 fixed type-stack alignment problem with L_utmslash
//	08-26-2001 updated L_fval to avoid routing fp through NDP, changed all
#	             typestack settings of OP_FVAL to OP_IVAL
//	09-02-2001 added L_tobody
#       12-08-2001 added CPP_evaluate
#       07-29-2002 fixed L_ret to set GlobalRp and GlobalRtp to bottom upon
#                    return from vm.
#	07-31-2002 fixed L_cmovefrom for zero count argument; added
#		     CPP_backslash; fixed bug in L_fmslashmod for divisor < 0
//	08-01-2002 added C_msfetch, C_search, C_compare
//      09-11-2002 modified vm and L_ret to make vm re-entrant
//	09-29-2002 restored L_usleep since it provides a different function 
#	             than L_ms, added CPP_immediate
#	04-11-2003 renamed L_ftos to L_froundtos
#	04-15-2003 added L_ftrunc and L_ftrunctos
#	04-18-2003 fixed L_ftrunctos typestack pointer update
#	06-15-2003 recoded L_rot, L_swap, and L_over for increased efficiency
#	01-01-2004 modified threading mode to a hybrid of indirect jump 
#                    threading and indirect call threading ;  results in
#		     significant speedup of kForth (about 1.7--2x)
#	01-31-2004 extended JumpTable past 256 entries,
#                    added CPP_include, CPP_source, CPP_refill, and
#                    CPP_nondeferred, CPP_state, CPP_colon, CPP_semicolon
#	02-09-2004 added CPP_allocate and CPP_free
#	03-18-2004 added L_fsincos, C_fsinh, C_fcosh, C_ftanh, C_fasinh,
#                    C_facosh, C_fatanh
#       03-21-2004 implemented L_fsincos
#       03-30-2004 modified L_ret to remove unneeded instructions
#       04-08-2004 added CPP_postpone and CPP_compilecomma
#       04-09-2004 added CPP_semicolon
#	04-26-2004 exported L_udmstar and L_utmslash for use by ForthVM.cpp
#       06-20-2004 modified L_call to prevent loss of return error code
//      09-05-2004 added C_forth_signal, C_raise, C_setitimer, C_getitimer
//      09-10-2004 moved CPP_spstore and CPP_rpstore to opcodes below 256
//      09-28-2005 changed some comment markers to // to prevent GNU as from quirky errors
//	02-10-2006 The EBP register is now used to hold GlobalIp for the jump-threaded portion
//                   of the VM. This results in 10--20% speedup for typical apps.
//                   Any additions to code in vm.s which needs to make use of the EBP
//	             register should save and restore this register using "pushl %ebp" at 
//	             the beginning and "popl %ebp" before returning (ret) or using NEXT.
//	             Also, calls from vm.s to external words (C/C++) that reference
//		     GlobalIp should update GlobalIp prior to the call:	"movl %ebp, GlobalIp"
//      04-11-2006 added stubs for L_dult, L_utsslashmod, L_stsslashrem, L_udmstar, L_cputest,
//                   L_dsstar, L_dmax, and L_dmin
//	04-19-2006 fixed bug in L_mslash, and added C_syscall
//	04-23-2006 recoded relational operators L_eq, L_neq, L_ult, L_ugt, L_lt, L_gt, L_le, L_ge,
//                   L_zeroeq, L_zerone, L_zerolt, L_zerogt. Added macros REL_DYADIC and REL_ZERO.
//                   Added macro LOGIC_DYADIC and used it for recoding _AND, _OR, and _XOR macros.
.equ WSIZE,	4

.equ OP_ADDR,	65
.equ OP_FVAL,	70
.equ OP_IVAL,	73
.equ OP_RET,	238
.equ SIGN_MASK,	0x80000000
	
# Error Codes

.equ E_NOT_ADDR,	1
.equ E_DIV_ZERO,	4
.equ E_RET_STK_CORRUPT,	5
.equ E_UNKNOWN_OP,	6

.data
FCONST_180: .double 180.
JumpTable: .int L_false, L_true, L_cells, L_cellplus # 0 -- 3
           .int L_dfloats, L_dfloatplus, CPP_case, CPP_endcase  # 4 -- 7
           .int CPP_of, CPP_endof, C_open, C_lseek     # 8 -- 11
           .int C_close, C_read, C_write, C_ioctl # 12 -- 15
           .int L_usleep, L_ms, C_msfetch, C_syscall  # 16 -- 19
           .int L_fill, L_cmove, L_cmovefrom, CPP_dotparen # 20 -- 23
           .int CPP_bracketsharp, CPP_tofile, CPP_console, CPP_sharpbracket  # 24 -- 27
           .int CPP_sharps, CPP_squote, CPP_cr, L_bl    # 28 -- 31
           .int CPP_spaces, L_store, CPP_cquote, CPP_sharp # 32 -- 35
           .int CPP_sign, L_mod, L_and, L_tick       # 36 -- 39
           .int CPP_lparen, CPP_hold, L_mul, L_add  # 40 -- 43
           .int L_nop, L_sub, CPP_dot, L_div  # 44 -- 47
           .int L_dabs, L_dnegate, L_umstar, L_umslashmod   # 48 -- 51
           .int L_mstar, L_mplus, L_mslash, L_mstarslash # 52 -- 55
           .int L_fmslashmod, L_smslashrem, CPP_colon, CPP_semicolon # 56 -- 59
           .int L_lt, L_eq, L_gt, L_question      # 60 -- 63
           .int L_fetch, L_addr, L_base, L_call   # 64 -- 67
           .int L_definition, L_erase, L_fval, CPP_forget # 68 -- 71
           .int L_tobody, L_ival, CPP_evaluate, C_key     # 72 -- 75
           .int L_lshift, L_slashmod, C_numberquery, CPP_dotr # 76 -- 79
           .int CPP_ddot, C_keyquery, L_rshift, CPP_dots  # 80 -- 83
           .int C_accept, CPP_char, CPP_bracketchar, CPP_word  # 84 -- 87
           .int L_starslash, L_starslashmod, CPP_udotr, CPP_lbracket  # 88 -- 91
           .int CPP_backslash, CPP_rbracket, L_xor, CPP_literal  # 92 -- 95
           .int CPP_queryallot, CPP_allot, L_binary, L_count # 96 -- 99
           .int L_decimal, CPP_emit, CPP_fdot, CPP_cold # 100 -- 103
           .int L_hex, L_i, L_j, CPP_brackettick         # 104 -- 107
           .int CPP_fvariable, C_timeanddate, CPP_find, CPP_constant # 108 -- 111
           .int CPP_immediate, CPP_fconstant, CPP_create, CPP_dotquote  # 112 -- 115
           .int CPP_type, CPP_udot, CPP_variable, CPP_words # 116 -- 119
           .int CPP_does, C_system, C_chdir, C_search   # 120 -- 123
           .int L_or, C_compare, L_not, L_nop     # 124 -- 127
           .int L_fsin, L_fcos, C_ftan, C_fasin   # 128 -- 131
           .int C_facos, C_fatan, C_fexp, C_fln   # 132 -- 135
           .int C_flog, L_fatan2, L_ftrunc, L_ftrunctos    # 136 -- 139
           .int C_fmin, C_fmax, L_floor, L_fround # 140 -- 143
           .int L_dlt, L_dzeroeq, L_deq, L_twopush  # 144 -- 147
           .int L_twopop, L_tworfetch, L_stod, L_stof # 148 -- 151
           .int L_dtof, L_froundtos, L_ftod, L_degtorad  # 152 -- 155
           .int L_radtodeg, L_dplus, L_dminus, L_dult   # 156 -- 159
           .int L_inc, L_dec, L_abs, L_neg        # 160 -- 163
           .int L_min, L_max, L_twostar, L_twodiv # 164 -- 167
           .int L_twoplus, L_twominus, L_cfetch, L_cstore  # 168 -- 171
           .int L_wfetch, L_wstore, L_dffetch, L_dfstore  # 172 -- 175
           .int L_sffetch, L_sfstore, L_spfetch, L_plusstore # 176 -- 179
           .int L_fadd, L_fsub, L_fmul, L_fdiv    # 180 -- 183
           .int L_fabs, L_fneg, C_fpow, L_fsqrt   # 184 -- 187
           .int CPP_spstore, CPP_rpstore, L_feq, L_fne  # 188 -- 191
           .int L_flt, L_fgt, L_fle, L_fge        # 192 -- 195
           .int L_fzeroeq, L_fzerolt, L_fzerogt, L_nop # 196 -- 199
           .int L_drop, L_dup, L_swap, L_over     # 200 -- 203
           .int L_rot, L_minusrot, L_nip, L_tuck  # 204 -- 207
           .int L_pick, L_roll, L_2drop, L_2dup   # 208 -- 211
           .int L_2swap, L_2over, L_2rot, L_depth # 212 -- 215
           .int L_querydup, CPP_if, CPP_else, CPP_then # 216 -- 219
           .int L_push, L_pop, L_puship, L_rfetch # 220 -- 223
           .int L_rpfetch, L_afetch, CPP_do, CPP_leave # 224 -- 227
           .int CPP_querydo, CPP_abortquote, L_jz, L_jnz  # 228 -- 231
           .int L_jmp, L_loop, L_plusloop, L_unloop  # 232 -- 235
           .int L_execute, CPP_recurse, L_ret, L_abort  # 236 -- 239
           .int L_quit, L_ge, L_le, L_ne          # 240 -- 243
           .int L_zeroeq, L_zerone, L_zerolt, L_zerogt # 244 -- 247
           .int L_ult, L_ugt, CPP_begin, CPP_while    # 248 -- 251
           .int CPP_repeat, CPP_until, CPP_again, CPP_bye  # 252 -- 255
	   .int L_utmslash, L_utsslashmod, L_stsslashrem, L_udmstar   # 256 -- 259	
	   .int CPP_include, CPP_source, CPP_refill, CPP_nondeferred # 260--263
	   .int CPP_state, CPP_allocate, CPP_free, L_nop  # 264--267
	   .int L_cputest, L_dsstar, CPP_compilecomma, L_nop     # 268--271
	   .int CPP_postpone, L_nop, L_nop, C_forth_signal # 272--275
	   .int C_raise, C_setitimer, C_getitimer, L_nop   # 276--279
	   .int L_nop, L_fsincos, C_facosh, C_fasinh  # 280--283
	   .int C_fatanh, C_fcosh, C_fsinh, C_ftanh   # 284--287
	   .int L_dmax, L_dmin, L_nop, L_nop	      # 288--291
.text
	.align 4
.global JumpTable
.global L_depth, L_tick, L_quit, L_abort, L_ret, L_dabs
.global L_dminus, L_mstarslash, L_udmstar, L_utmslash

.macro NEXT
	incl %ebp		 # increment the Forth instruction ptr
	movl %ebp, GlobalIp
	movb (%ebp), %al         # get the opcode
	movl JumpTable(,%eax,4), %ebx	# machine code address of word
	xor %eax, %eax
	jmpl *%ebx		# call the word
.endm

.macro SWAP
	pushl %ebp
	movl GlobalSp, %ebx
        addl $WSIZE, %ebx
	movl %ebx, %ebp
	movl (%ebp), %ecx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	movl %ecx, (%ebx)
	movl %edx, (%ebp)
	popl %ebp
.endm

.macro DUP
        movl GlobalSp, %ebx
        movl WSIZE(%ebx), %eax
        movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
        xor %eax, %eax
.endm
	
.macro DROP
        addl $WSIZE, GlobalSp
.endm

.macro OVER
	movl GlobalSp, %ebx
        movl 2*WSIZE(%ebx), %eax
        movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
        xor %eax, %eax
.endm 

.macro FOVER
	movl GlobalSp, %ebx
	movl %ebx, %ecx
	addl $3*WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	movl %ecx, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax
.endm
	
.macro _PUSH
	movl $WSIZE, %eax
	movl GlobalSp, %ebx
	addl %eax, %ebx	
        movl %ebx, GlobalSp
	movl (%ebx), %ecx
	movl GlobalRp, %ebx
	movl %ecx, (%ebx)
	subl %eax, %ebx
	movl %ebx, GlobalRp
        xor %eax, %eax
.endm

.macro _POP
	movl $WSIZE, %eax
	movl GlobalRp, %ebx
	addl %eax, %ebx
	movl %ebx, GlobalRp
	movl (%ebx), %ecx
	movl GlobalSp, %ebx
	movl %ecx, (%ebx)
	subl %eax, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax
.endm

.macro FETCH
	movl GlobalSp, %edx	
	addl $WSIZE, %edx
        movl (%edx), %ebx	
        movl (%ebx), %eax
	movl %eax, (%edx)
	xor %eax, %eax
.endm
	
.macro STORE
	movl $WSIZE, %eax
	movl GlobalSp, %ebx
        addl %eax, %ebx
        movl (%ebx), %ecx	# address to store to in ecx
	addl %eax, %ebx
	movl (%ebx), %edx	# value to store in edx
	movl %ebx, GlobalSp
	movl %edx, (%ecx)
	xor %eax, %eax
.endm

// Dyadic Logic operators 
	
.macro LOGIC_DYADIC op
	movl $WSIZE, %ecx
	movl GlobalSp, %ebx
	addl %ecx, %ebx
	movl %ebx, GlobalSp
	movl (%ebx), %eax
	addl %ecx, %ebx
	\op (%ebx), %eax
	movl %eax, (%ebx)
	xorl %eax, %eax 
.endm
	
.macro _AND
	LOGIC_DYADIC andl
.endm

.macro _OR
	LOGIC_DYADIC orl
.endm

.macro _XOR
	LOGIC_DYADIC xorl
.endm
	
.macro _NOT
	movl GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	notl %eax
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
.endm

// Dyadic relational operators (single length numbers) 
	
.macro REL_DYADIC setx
	movl $WSIZE, %ecx
	movl GlobalSp, %ebx
	addl %ecx, %ebx
	movl %ebx, GlobalSp
	movl (%ebx), %eax
	addl %ecx, %ebx
	cmpl %eax, (%ebx)
	movl $0, %eax
	\setx %al
	negl %eax
	movl %eax, (%ebx)
	xorl %eax, %eax
.endm

// Relational operators for zero (single length numbers)
	
.macro REL_ZERO setx
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	cmpl $0, %eax
	movl $0, %eax
	\setx %al
	negl %eax
	movl %eax, (%ebx)
	xorl %eax, %eax
.endm

.macro DNEGATE
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, %ecx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	notl %eax
	clc
	addl $1, %eax
	movl %eax, (%ebx)
	movl %ecx, %ebx
	movl (%ebx), %eax
	notl %eax
	adcl $0, %eax
	movl %eax, (%ebx)
	xor %eax, %eax
.endm
	
.macro DPLUS
	movl GlobalSp, %ebx
	addl $2*WSIZE, %ebx
	movl (%ebx), %eax
	clc
	addl 2*WSIZE(%ebx), %eax
	movl %eax, 2*WSIZE(%ebx)
	movl WSIZE(%ebx), %eax
	adcl -WSIZE(%ebx), %eax
	movl %eax, WSIZE(%ebx)
	movl %ebx, GlobalSp
	xor %eax, %eax
.endm

.macro DMINUS
	movl GlobalSp, %ebx
	addl $2*WSIZE, %ebx
	movl 2*WSIZE(%ebx), %eax
	clc
	subl (%ebx), %eax
	movl %eax, 2*WSIZE(%ebx)
	movl WSIZE(%ebx), %eax
	sbbl -WSIZE(%ebx), %eax
	movl %eax, WSIZE(%ebx)
	movl %ebx, GlobalSp
	xor %eax, %eax
.endm

.macro STARSLASH
	movl $WSIZE, %eax
	sall $1, %eax
        addl %eax, GlobalSp
        movl GlobalSp, %ebx
        movl WSIZE(%ebx), %eax
        imull (%ebx)
	idivl -WSIZE(%ebx)
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
.endm
	
					
.global vm
	.type	vm,@function
vm:	
        pushl %ebp
        pushl %ebx
        pushl %ecx
        pushl %edx
	pushl GlobalIp
	pushl vmEntryRp
        movl %esp, %ebp
        movl 28(%ebp), %ebp     # load the Forth instruction pointer
        movl %ebp, GlobalIp
	movl GlobalRp, %eax
	movl %eax, vmEntryRp
	xor %eax, %eax
next:
        movb (%ebp), %al         # get the opcode
	movl JumpTable(,%eax,4), %ebx	# machine code address of word
	xor %eax, %eax          # clear error code
	call *%ebx		# call the word
	movl GlobalIp, %ebp
	incl %ebp		 # increment the Forth instruction ptr
	movl %ebp, GlobalIp
	cmpb $0, %al		 # check for error
	jz next        
exitloop:
        cmpl $OP_RET, %eax         # return from vm?
        jnz vmexit
        xor %eax, %eax            # clear the error
vmexit:
	pop vmEntryRp
	pop GlobalIp
	pop %edx
        pop %ecx
        pop %ebx
        pop %ebp
        ret
L_cputest:
	ret
L_nop:
        movl $E_UNKNOWN_OP, %eax   # unknown operation
        ret
L_quit:
	movl BottomOfReturnStack, %eax	# clear the return stacks
	movl %eax, GlobalRp
	movl %eax, vmEntryRp
	movl $8, %eax		# exit the virtual machine
	ret
L_abort:
	movl BottomOfStack, %eax
	movl %eax, GlobalSp
	jmp L_quit
L_base:	
	movl GlobalSp, %ebx
	movl $Base, %eax
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax
	NEXT
L_binary:
	movl $Base, %ebx
	movl $2, (%ebx)
	NEXT
L_decimal:	
	movl $Base, %ebx
	movl $10, (%ebx)
	NEXT
L_hex:	
	movl $Base, %ebx
	movl $16, (%ebx)
	NEXT	
L_false:
	movl GlobalSp, %ebx
	movl $0, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	NEXT
L_true:
	movl GlobalSp, %ebx
	movl $-1, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	NEXT
L_cells:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	sall $2, %eax
	movl %eax, (%ebx)
	xor %eax, %eax
	NEXT
L_cellplus:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %eax
	movl %eax, (%ebx)
	xor %eax, %eax
	NEXT
L_dfloats:	
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	sall $3, %eax
	movl %eax, (%ebx)
	xor %eax, %eax
	NEXT
L_dfloatplus:	
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %eax
	addl $WSIZE, %eax
	movl %eax, (%ebx)
	xor %eax, %eax
	NEXT				
L_bl:
	movl GlobalSp, %ebx
	movl $32, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	NEXT
L_ret:
	movl vmEntryRp, %eax		# Return Stack Ptr on entry to VM
	movl GlobalRp, %ecx
	cmpl %eax, %ecx
	jl ret1
        movl $OP_RET, %eax             # exhausted the return stack so exit vm
        ret
ret1:
	addl $WSIZE, %ecx
        movl %ecx, GlobalRp
ret2:   movl (%ecx), %eax
	movl %eax, GlobalIp		# reset the instruction ptr
        xor %eax, %eax
retexit:
        ret
L_tick:
	movl GlobalSp, %ebx
	movl $32, (%ebx)
	movl $WSIZE, %eax
	subl %eax, GlobalSp
	call CPP_word
	call CPP_find
	DROP
	ret
L_tobody:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx	# code address
	incl %ecx		# the data address is offset by one
	movl (%ecx), %ecx
	movl %ecx, (%ebx)
	ret
#
# For precision delays, use MS instead of USLEEP
# Use USLEEP when task can be put to sleep and reawakened by OS
#
L_usleep:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	movl (%ebx), %eax
	pushl %eax
	call usleep
	popl %eax
	xor %eax, %eax
	ret
L_ms:
	movl GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	imull $1000, %eax
	movl %eax, WSIZE(%ebx)
	call C_usec
	ret
L_fill:
	SWAP
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	addl %eax, GlobalSp
fill2:	movl GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	call memset
	addl $12, %esp
	xor %eax, %eax
fillexit:	
	ret
L_erase:
	movl GlobalSp, %ebx
	movl $0, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp

	call L_fill
	ret	
L_cmove:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	SWAP
	movl $WSIZE, %eax
	addl %eax, GlobalSp
cmove2:	movl GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	movl $WSIZE, %eax
	addl %eax, GlobalSp
cmove3:	movl GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	call memcpy
	addl $12, %esp
	xor %eax, %eax				
	ret		
L_cmovefrom:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	movl (%ebx), %ecx	# load count register
	addl %eax, GlobalSp
cmovefrom2:
	movl GlobalSp, %ebx
	movl (%ebx), %ebx
	movl %ecx, %eax
	decl %eax
	addl %eax, %ebx
	movl %ebx, %edx		# dest addr in %edx
	movl $WSIZE, %eax
	addl %eax, GlobalSp
cmovefrom3:
	movl GlobalSp, %ebx
	movl (%ebx), %ebx
	movl %ecx, %eax
	cmpl $0, %eax
	jnz cmovefrom4
	ret
cmovefrom4:	
	decl %eax
	addl %eax, %ebx		# src addr in %ebx
cmovefromloop:	
	movb (%ebx), %al
	decl %ebx
	xchgl %ebx, %edx
	movb %al, (%ebx)
	decl %ebx
	xchgl %ebx, %edx
	loop cmovefromloop	
	xor %eax, %eax
	ret
L_call:	
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	xor %eax, %eax
	movl GlobalSp, %ebx
	call *(%ebx)
	ret 		
L_push:
	_PUSH
        NEXT
L_pop:
	_POP
	NEXT
L_twopush:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	movl %ebx, GlobalSp
	movl GlobalRp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalRp
	xor %eax, %eax
	NEXT
L_twopop:
	movl GlobalRp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	movl %ebx, GlobalRp
	movl GlobalSp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax				
	NEXT
L_puship:
        movl %ebp, %eax
        movl GlobalRp, %ebx
        mov %eax, (%ebx)
	movl $WSIZE, %eax
        subl %eax, GlobalRp
        xor %eax, %eax
        NEXT
L_execute:	
        movl %ebp, %ecx
        movl GlobalRp, %ebx
        movl %ecx, (%ebx)
	movl $WSIZE, %eax 
        subl %eax, %ebx 
	movl %ebx, GlobalRp
	movl GlobalSp, %ebx
        addl %eax, %ebx
	movl %ebx, GlobalSp
        movl (%ebx), %eax
	decl %eax
	movl %eax, %ebp
        xor %eax, %eax
        NEXT
L_definition:
        movl %ebp, %ebx
	movl $WSIZE, %eax
	incl %ebx
	movl (%ebx), %ecx # address to execute
	addl $3, %ebx
	movl %ebx, %edx
	movl GlobalRp, %ebx
	movl %edx, (%ebx)
	subl %eax, %ebx
	movl %ebx, GlobalRp
	decl %ecx
	movl %ecx, %ebp
        xor %eax, %eax	
	NEXT
L_rfetch:
        movl GlobalRp, %ebx
        addl $WSIZE, %ebx
        movl (%ebx), %eax
        movl GlobalSp, %ebx
        movl %eax, (%ebx)
	movl $WSIZE, %eax
        subl %eax, GlobalSp
        xor %eax, %eax
	NEXT
L_tworfetch:
	movl GlobalRp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	movl GlobalSp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax				
	NEXT
L_rpfetch:
	movl GlobalRp, %eax
	addl $WSIZE, %eax
	movl GlobalSp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax
	NEXT
L_spfetch:
	movl GlobalSp, %eax
	movl %eax, %ebx
	addl $WSIZE, %eax
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax 
	NEXT
L_i:
        movl GlobalRp, %ebx
        movl 3*WSIZE(%ebx), %eax
        movl GlobalSp, %ebx
        movl %eax, (%ebx)
	movl $WSIZE, %eax
        subl %eax, %ebx 
	movl %ebx, GlobalSp
        xor %eax, %eax
        NEXT
L_j:
        movl GlobalRp, %ebx
        movl 6*WSIZE(%ebx), %eax
        movl GlobalSp, %ebx
        movl %eax, (%ebx)
	movl $WSIZE, %eax
        subl %eax, %ebx
	movl %ebx, GlobalSp
        xor %eax, %eax
        NEXT
L_loop:
        movl GlobalRp, %ebx	
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %ecx
	addl %eax, %ebx
        movl (%ebx), %eax
        incl %eax
	cmpl %ecx, %eax	
        jz L_unloop
loop1:	
        movl %eax, (%ebx)	# set loop counter to next value
	movl %edx, %ebp		# set instruction ptr to start of loop
        xor %eax, %eax
        NEXT
L_unloop:  
	movl $WSIZE, %eax
	shll $1, %eax
	addl $WSIZE, %eax
#	imul $3, %eax
	addl %eax, GlobalRp  # terminal count reached, discard top 3 items
	movl $3, %eax
        xor %eax, %eax
        NEXT
loopbad:
        movl $E_RET_STK_CORRUPT, %eax
        ret	
L_plusloop:
	pushl %ebp
	movl $WSIZE, %eax
	movl GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %ebp	# get loop increment 
	movl %ebx, GlobalSp		
        movl GlobalRp, %ebx
	addl %eax, %ebx		# get ip and save in edx
	movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %ecx	# get terminal count in ecx
	addl %eax, %ebx
	movl (%ebx), %eax	# get current loop count 
	addl %ebp, %eax		# increment loop count
	cmpl $0, %ebp
	jl plusloop1
	popl %ebp
	cmpl %ecx, %eax
	jge L_unloop				
	movl %eax, (%ebx)	# set loop counter to incremented value
        movl %edx, %ebp		# set instruction ptr to start of loop
        xor %eax, %eax
        NEXT
plusloop1:
	popl %ebp
	cmpl %ecx, %eax
	jl L_unloop
	movl %eax, (%ebx)
	movl %edx, %ebp
	xor %eax, %eax
	NEXT
L_jz:
	movl $WSIZE, %eax
        movl GlobalSp, %ebx
	addl %eax, %ebx
	movl %ebx, GlobalSp
        movl (%ebx), %eax
        cmpl $0, %eax
        jz jz1
	movl $4, %eax
        addl %eax, %ebp       # do not jump
	xor %eax, %eax
        NEXT
jz1:    movl %ebp, %ebx
        incl %ebx
        movl (%ebx), %eax          # get the relative jump count
        decl %eax
        addl %eax, %ebp
	xor %eax, %eax
        NEXT
L_jnz:				# not implemented
	ret
L_jmp:
        movl %ebp, %ebx
        incl %ebx
        movl (%ebx), %eax       # get the relative jump count
        addl %eax, %ebx
        subl $2, %ebx
        movl %ebx, %ebp		# set instruction ptr
	xor %eax, %eax
        NEXT
L_count:
	movl GlobalSp, %ebx
	movl WSIZE(%ebx), %ebx
	xor %eax, %eax
	movb (%ebx), %al
	movl GlobalSp, %ebx
	incl WSIZE(%ebx)
	movl %eax, (%ebx)
	movl $WSIZE, %eax
	subl %eax, GlobalSp
	xor %eax, %eax
	ret
L_ival:
        movl %ebp, %ebx
        incl %ebx
        movl (%ebx), %eax
	addl $WSIZE-1, %ebx
	movl %ebx, %ebp
	movl GlobalSp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax
	NEXT
L_addr:
	movl %ebp, %ebx
	incl %ebx
	movl (%ebx), %eax
	addl $WSIZE-1, %ebx
	movl %ebx, %ebp
	movl GlobalSp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax
	NEXT
L_fval:
        movl %ebp, %ebx
        incl %ebx
        movl GlobalSp, %ecx
        subl $WSIZE, %ecx
        movl (%ebx), %eax
	movl %eax, (%ecx)
	movl WSIZE(%ebx), %eax
	movl %eax, WSIZE(%ecx)
	subl $WSIZE, %ecx
	movl %ecx, GlobalSp
	addl $2*WSIZE-1, %ebx
	movl %ebx, %ebp
	xor %eax, %eax
	NEXT
L_and:
	_AND
	NEXT
L_or:
	_OR
	NEXT
L_not:
	_NOT
	NEXT
L_xor:
	_XOR
	NEXT
L_lshift:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	movl (%ebx), %ecx
	movl WSIZE(%ebx), %eax
	shll %cl, %eax
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
	NEXT
L_rshift:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	movl (%ebx), %ecx
	movl WSIZE(%ebx), %eax
	shrl %cl, %eax
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
	NEXT
L_eq:
	REL_DYADIC sete
	NEXT
L_ne:
	REL_DYADIC setne
	NEXT
L_ult:
	REL_DYADIC setb
	NEXT
L_ugt:
	REL_DYADIC seta
	NEXT
L_lt:
	REL_DYADIC setl
	NEXT	
L_gt:
	REL_DYADIC setg
	NEXT
L_le:
	REL_DYADIC setle
	NEXT
L_ge:
	REL_DYADIC setge
	NEXT
L_zeroeq:
	REL_ZERO setz
	NEXT
L_zerone:
	REL_ZERO setnz
	NEXT
L_zerolt:
	REL_ZERO setl
	NEXT
L_zerogt:
	REL_ZERO setg
	NEXT
L_deq:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx
	addl $WSIZE, %ebx
	movl %ebx, GlobalSp
	movl (%ebx), %eax
	subl %edx, %eax
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	subl %ecx, %edx
	orl %edx, %eax
	cmpl $0, %eax
	jz deq1
	movl $0, (%ebx)
	xor %eax, %eax
	ret
deq1:	movl $-1, (%ebx)
	xor %eax, %eax	
	ret
L_dzeroeq:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, GlobalSp
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	orl (%ebx), %eax
	cmpl $0, %eax
	jz deq1
	movl $0, (%ebx)
	xor %eax, %eax
	ret
L_dlt:
	DMINUS
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, GlobalSp
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	cmpl $0, %eax
	jl deq1
	movl $0, (%ebx)
	xor %eax, %eax	
	ret
L_dult:	
	ret 	
L_querydup:
	movl GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	cmpl $0, %eax
	je L_querydupexit
	movl %eax, (%ebx)
	movl $WSIZE, %eax
	subl %eax, GlobalSp
	xor %eax, %eax
L_querydupexit:
	ret	
L_drop:
        DROP
        NEXT
L_dup:
	DUP
	NEXT
L_swap:
	SWAP
        NEXT
L_over:
	OVER
        NEXT
L_rot:
	pushl %ebp
	movl GlobalSp, %ebx
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, %ebp
	addl %eax, %ebx
	addl %eax, %ebx
	movl (%ebx), %ecx
	movl (%ebp), %edx
	movl %ecx, (%ebp)
	addl %eax, %ebp
	movl (%ebp), %ecx
	movl %edx, (%ebp)
	movl %ecx, (%ebx)
	xor %eax, %eax
	popl %ebp
	NEXT
L_minusrot:
	movl GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	movl %eax, (%ebx)
	addl $WSIZE, %ebx
	movl WSIZE(%ebx), %eax
	movl %eax, (%ebx)
	addl $WSIZE, %ebx
	movl WSIZE(%ebx), %eax
	movl %eax, (%ebx)
	movl -2*WSIZE(%ebx), %eax
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
	NEXT
L_nip:
        SWAP
	movl $WSIZE, %eax
        addl %eax, GlobalSp
	xor %eax, %eax
        NEXT
L_tuck:
        SWAP
        OVER
        NEXT
L_pick:
	movl GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	addl $2, %eax
	imul $WSIZE, %eax
	addl %eax, %ebx
	movl (%ebx), %eax
	movl GlobalSp, %ebx
	movl %eax, (%ebx)
	movl WSIZE(%ebx), %eax
	addl $2, %eax
	movl GlobalSp, %ebx
	movl (%ebx), %eax
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
	NEXT
L_roll:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx 
	movl (%ebx), %eax
	incl %eax
	pushl %eax
	pushl %eax
	pushl %eax
	pushl %ebx
	imul $WSIZE, %eax
	addl %eax, %ebx		# addr of item to roll
	movl (%ebx), %eax
	popl %ebx
	movl %eax, (%ebx)
	popl %eax		# number of cells to copy
	movl %eax, %ecx
	imul $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, %edx		# dest addr
	subl $WSIZE, %ebx	# src addr
rollloop:
	movl (%ebx), %eax
	sub $WSIZE, %ebx
	xchgl %ebx, %edx
	movl %eax, (%ebx)
	sub $WSIZE, %ebx
	xchgl %ebx, %edx
	loop rollloop

	popl %eax		# now we have to roll the typestack
	popl %eax

	xor %eax, %eax
	ret
L_depth:
	movl GlobalSp, %ebx
	movl BottomOfStack, %eax
	subl %ebx, %eax
	movl $WSIZE, (%ebx)
	movl $0, %edx
	idivl (%ebx)
	movl %eax, (%ebx)
	movl $WSIZE, %eax
	subl %eax, GlobalSp
	xor %eax, %eax
        ret
L_2drop:
	movl $WSIZE, %eax
	addl %eax, %eax
	addl %eax, GlobalSp
	xor %eax, %eax
        NEXT
L_2dup:
	movl GlobalSp, %ebx
	movl %ebx, %ecx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	movl %ecx, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax
        NEXT
L_2swap:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	xchgl %edx, (%ebx)
	addl $WSIZE, %ebx
	xchgl %eax, (%ebx)
	subl $2*WSIZE, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	xor %eax, %eax	
        NEXT
L_2over:
	FOVER
        NEXT
L_2rot:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, %ecx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	xchgl %edx, (%ebx)
	addl $WSIZE, %ebx
	xchgl %eax, (%ebx)
	addl $WSIZE, %ebx
	xchgl %edx, (%ebx)
	addl $WSIZE, %ebx
	xchgl %eax, (%ebx)
	movl %ecx, %ebx
	movl %edx, (%ebx)
	addl $WSIZE, %ebx
	movl %eax, (%ebx)
	xor %eax, %eax
        NEXT
L_question:
	FETCH
	cmpl $0, %eax
	jnz questionexit
	call CPP_dot
questionexit:	
	ret	
L_fetch:
	FETCH
	NEXT
L_store:
	STORE
	NEXT
L_afetch:
	FETCH
	NEXT
L_cfetch:
	xor %eax, %eax
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx
	movb (%ecx), %al
	movl %eax, (%ebx)
	xor %eax, %eax
        NEXT
L_cstore:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx	# address to store
	addl $WSIZE, %ebx
	movl (%ebx), %eax	# value to store
	movb %al, (%ecx)
	movl %ebx, GlobalSp
	xor %eax, %eax
	NEXT	
L_wfetch:
	movl GlobalSp, %ebx
	movl WSIZE(%ebx), %ebx
	movw (%ebx), %ax
	cwde
	movl GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
        NEXT
L_wstore:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	movl (%ebx), %eax
	pushl %eax
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	pop %ebx
	movw %ax, (%ebx)
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	xor %eax, %eax
        NEXT
L_sffetch:
	movl $WSIZE, %eax
        addl %eax, GlobalSp
        movl GlobalSp, %ebx
        movl (%ebx), %ebx
        flds (%ebx)
	movl $WSIZE, %eax
        subl %eax, GlobalSp
        movl GlobalSp, %ebx
        fstpl (%ebx)
        subl %eax, GlobalSp
	xor %eax, %eax
        NEXT
L_sfstore:
	movl $WSIZE, %eax
        addl %eax, GlobalSp
        movl GlobalSp, %ebx
        addl $WSIZE, %ebx
        fldl (%ebx)              # load the f number into NDP
        subl $WSIZE, %ebx
        movl (%ebx), %ebx          # load the dest address
        fstps (%ebx)             # store as single precision float
	movl $WSIZE, %eax
	sall $1, %eax
        addl %eax, GlobalSp
	xor %eax, %eax
        NEXT
L_dffetch:	
	movl GlobalSp, %ebx
	movl %ebx, %edx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx
	movl (%ecx), %eax
	movl %eax, (%edx)
	addl $WSIZE, %ecx
	movl (%ecx), %eax
	movl %eax, (%ebx)
	subl $WSIZE, %edx
	movl %edx, GlobalSp
	xor %eax, %eax
	NEXT
L_dfstore:
	movl GlobalSp, %ebx
	movl $WSIZE, %edx
	addl %edx, %ebx
	movl %ebx, %eax
	movl (%ebx), %ebx  # address to store
	addl %edx, %eax
	movl (%eax), %ecx
	movl %ecx, (%ebx)
	addl %edx, %eax
	addl %edx, %ebx
	movl (%eax), %ecx
	movl %ecx, (%ebx)
	movl %eax, GlobalSp
	xor %eax, %eax
	NEXT
L_inc:
        movl GlobalSp, %ebx
        incl WSIZE(%ebx)
        NEXT
L_dec:
        movl GlobalSp, %ebx
        decl WSIZE(%ebx)
        NEXT
L_twoplus:
	movl GlobalSp, %ebx
	incl WSIZE(%ebx)
	incl WSIZE(%ebx)
	NEXT
L_twominus:
	movl GlobalSp, %ebx
	decl WSIZE(%ebx)
	decl WSIZE(%ebx)
	NEXT
L_abs:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	cmpl $0, %eax
	jl abs1
	xor %eax, %eax
	ret
abs1:	negl %eax
	movl %eax, (%ebx)
	xor %eax, %eax
        ret
L_neg:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	negl %eax
	movl %eax, (%ebx)
	xor %eax, %eax
        NEXT
L_max:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	cmpl %eax, %ebx
	jl max1
	movl %ebx, %eax
	movl GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
	jmp maxexit
max1:
	movl GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
maxexit:
	xor %eax, %eax
        ret
L_min:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	cmpl %eax, %ebx
	jg min1
	movl %ebx, %eax
	movl GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
	jmp minexit
min1:
	movl GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
minexit:
	xor %eax, %eax
        ret
L_dmax:
	ret
L_dmin:
	ret
L_twostar:
	movl GlobalSp, %ebx
	sall $1, WSIZE(%ebx)
	NEXT
L_twodiv:
	movl GlobalSp, %ebx
	sarl $1, WSIZE(%ebx)
	NEXT
L_add:
	movl $WSIZE, %eax
	movl GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %eax
	addl %eax, WSIZE(%ebx)
	movl %ebx, GlobalSp
        xor %eax, %eax
        NEXT
L_sub:
	movl $WSIZE, %eax
	movl GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %eax
	subl %eax, WSIZE(%ebx)
	movl %ebx, GlobalSp
        xor %eax, %eax
        NEXT
L_mul:
	movl $WSIZE, %ecx
	movl GlobalSp, %ebx
	addl %ecx, %ebx
	movl %ebx, GlobalSp
	movl (%ebx), %eax
	addl %ecx, %ebx
	imull (%ebx)
	movl %eax, (%ebx)
	xor %eax, %eax
        NEXT
L_div:
	movl $WSIZE, %eax
        addl %eax, GlobalSp
        movl GlobalSp, %ebx
        movl (%ebx), %eax
        cmpl $0, %eax
        jnz div1
        movl $E_DIV_ZERO, %eax
        jmp divexit
div1:	
	addl $WSIZE, %ebx
        movl (%ebx), %eax
	cdq
        idivl -WSIZE(%ebx)
        movl %eax, (%ebx)
	xor %eax, %eax
divexit:
        ret
L_mod:
	call L_div
	movl %edx, (%ebx)
	NEXT
L_slashmod:
	call L_div
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	SWAP
	NEXT
L_starslash:
	STARSLASH	
	NEXT
L_starslashmod:
	STARSLASH
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	SWAP
	ret
L_plusstore:
	movl GlobalSp, %ebx
	push %ebx
	push %ebx
	push %ebx
	movl WSIZE(%ebx), %ebx
	movl (%ebx), %eax
	pop %ebx
	movl 2*WSIZE(%ebx), %ebx
	addl %ebx, %eax
	pop %ebx
	movl WSIZE(%ebx), %ebx
	movl %eax, (%ebx)
	pop %ebx
	movl $WSIZE, %eax
	sall $1, %eax
	addl %eax, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax
	NEXT
L_dabs:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx
	movl %ecx, %eax
	cmpl $0, %eax
	jl dabs_go
	xor %eax, %eax
	ret
dabs_go:
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	clc
	subl $1, %eax
	notl %eax
	movl %eax, (%ebx)
	movl %ecx, %eax
	sbbl $0, %eax
	notl %eax
	movl %eax, -WSIZE(%ebx)
	xor %eax, %eax
	ret
L_dnegate:
	DNEGATE
	NEXT	
L_dplus:
	DPLUS
	NEXT
L_dminus:
	DMINUS
	ret
L_umstar:
	movl $WSIZE, %eax
	movl GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl %ecx, %eax
	mull (%ebx)
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	xor %eax, %eax				
	ret
L_dsstar:
	ret
L_umslashmod:
	movl GlobalSp, %ebx
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, GlobalSp
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl $0, %edx
	movl (%ebx), %eax
	divl %ecx
	cmpl $0, %eax
	jne umslashmod_ovflow
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	divl %ecx
	jmp umslashmod_exit
umslashmod_ovflow:
	addl $WSIZE, %ebx
	movl $-1, %edx
	movl $-1, %eax
umslashmod_exit:	
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %eax, (%ebx)
	xor %eax, %eax		
	ret
L_mstar:
	movl $WSIZE, %eax
	movl GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl %ecx, %eax
	imull (%ebx)
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	xor %eax, %eax		
	ret
L_mplus:
	movl $WSIZE, %eax
	movl GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %ecx
	movl %ebx, GlobalSp
	addl %eax, %ebx
	movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %eax
	clc
	addl %ecx, %eax
	js mplus1
	adcl $0, %edx
mplus1:	movl %eax, (%ebx)
	movl %edx, -WSIZE(%ebx)
	xor %eax, %eax
	ret
L_mslash:
	movl $WSIZE, %eax
        movl GlobalSp, %ebx
	addl %eax, %ebx
        movl (%ebx), %ecx
	addl %eax, %ebx
	movl %ebx, GlobalSp
        cmpl $0, %ecx
        je mslash1
	# addl %eax, %ebx
        movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %eax
        idivl %ecx
        movl %eax, (%ebx)
	xor %eax, %eax		
	ret
mslash1:			
        movl $E_DIV_ZERO, %eax
        ret
L_udmstar:
	# multiply unsigned double and unsigned to give triple length product
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	mull %ecx
	movl %edx, -WSIZE(%ebx)
	movl %eax, (%ebx)
	addl $WSIZE, %ebx
	movl %ecx, %eax
	mull (%ebx)
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl (%ebx), %eax
	subl $WSIZE, %ebx
	clc
	addl %edx, %eax
	movl %eax, WSIZE(%ebx)
	# subl $WSIZE, %ebx
	movl (%ebx), %eax
	adcl $0, %eax
	movl %eax, (%ebx)
	xor %eax, %eax 		
	ret
L_utsslashmod:
	ret
L_stsslashrem:
	ret
L_utmslash:
	# divide unsigned triple length by unsigned to give ud quotient
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx		# divisor in ecx
	addl $WSIZE, %ebx
	movl (%ebx), %eax		# ut3
	movl $0, %edx
	divl %ecx			# ut3/u
	pushl %ebx			# keep local stack ptr
	movl GlobalSp, %ebx
	movl %eax, -4*WSIZE(%ebx)	# q3
	movl %edx, -5*WSIZE(%ebx)	# r3
	popl %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax		# ut2
	movl $0, %edx
	divl %ecx			# ut2/u
	pushl %ebx
	movl GlobalSp, %ebx
	movl %eax, -2*WSIZE(%ebx)	# q2
	movl %edx, -3*WSIZE(%ebx)	# r2
	popl %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax		# ut1
	movl $0, %edx
	divl %ecx			# ut1/u
	pushl %ebx
	movl GlobalSp, %ebx
	movl %eax, (%ebx)		# q1
	movl %edx, -WSIZE(%ebx)		# r1
	movl -5*WSIZE(%ebx), %edx	# r3 << 32
	movl $0, %eax
	divl %ecx			# (r3 << 32)/u
	movl %eax, -6*WSIZE(%ebx)	# q4
	movl %edx, -7*WSIZE(%ebx)	# r4
	movl -3*WSIZE(%ebx), %edx	# r2 << 32
	movl $0, %eax
	divl %ecx			# (r2 << 32)/u
	movl %eax, -8*WSIZE(%ebx)	# q5
	movl %edx, -9*WSIZE(%ebx)	# r5
	movl -7*WSIZE(%ebx), %edx	# r4 << 32
	movl $0, %eax
	divl %ecx			# (r4 << 32)/u
	movl %eax, -10*WSIZE(%ebx)	# q6
	movl %edx, -11*WSIZE(%ebx)	# r6
	movl -WSIZE(%ebx), %eax		# r1
	addl -9*WSIZE(%ebx), %eax	# r1 + r5
	addl -11*WSIZE(%ebx), %eax	# r1 + r5 + r6
	movl $0, %edx
	divl %ecx
	addl -10*WSIZE(%ebx), %eax	# q7 + q6
	addl -8*WSIZE(%ebx), %eax	# q7 + q6 + q5
	addl (%ebx), %eax		# q7 + q6 + q5 + q1
	popl %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	pushl %ebx
	movl GlobalSp, %ebx
	movl -2*WSIZE(%ebx), %eax	# q2
	addl -6*WSIZE(%ebx), %eax	# q2 + q4
	popl %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax
	ret
L_mstarslash:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	xorl (%ebx), %eax
	shrl $31, %eax
	pushl %eax	# keep sign of result -- negative is nonzero
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, GlobalSp
	call L_abs
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, GlobalSp
	call L_dabs
	movl GlobalSp, %ebx
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	call L_udmstar
	movl GlobalSp, %ebx
	subl $WSIZE, %ebx
	movl %ebx, GlobalSp
	call L_utmslash	
	popl %eax
	cmpl $0, %eax
	jnz mstarslash_neg
	xor %eax, %eax
	ret
mstarslash_neg:
	DNEGATE
	xor %eax, %eax
	ret		
L_fmslashmod:
	movl GlobalSp, %ebx
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, GlobalSp
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %eax
	idivl %ecx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %eax, (%ebx)
	cmpl $0, %ecx
	jg fmslashmod2
	cmpl $0, %edx
	jg fmslashmod3
	xor %eax, %eax
	ret
fmslashmod2:		
	cmpl $0, %edx
	jge fmslashmodexit
fmslashmod3:	
	decl %eax		# floor the result
	movl %eax, (%ebx)
	addl $WSIZE, %ebx
	addl %ecx, (%ebx)
fmslashmodexit:
	xor %eax, %eax
	ret
L_smslashrem:
	movl GlobalSp, %ebx
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, GlobalSp
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %eax
	idivl %ecx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %eax, (%ebx)
	xor %eax, %eax		
	ret
L_stod:
	movl GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	pushl %edx
	cdq
	movl %edx, (%ebx)
	popl %edx
	movl $WSIZE, %eax
	subl %eax, %ebx
	movl %ebx, GlobalSp
	xor %eax, %eax	
	NEXT
L_stof:
	movl $WSIZE, %eax
        addl %eax, GlobalSp
        movl GlobalSp, %ebx
        fildl (%ebx)
        movl GlobalSp, %ebx
	movl $WSIZE, %eax
        subl %eax, %ebx
        fstpl (%ebx)
	sall $1, %eax
        subl %eax, GlobalSp
	xor %eax, %eax
        NEXT
L_dtof:
	movl $WSIZE, %eax
        movl GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %eax
	xchgl WSIZE(%ebx), %eax
	movl %eax, (%ebx)
        fildq (%ebx)
        fstpl (%ebx)
	xor %eax, %eax	
	NEXT	
L_froundtos:
	movl $WSIZE, %eax
        addl %eax, GlobalSp
        movl GlobalSp, %ebx
        fldl (%ebx)
        addl %eax, %ebx
        fistpl (%ebx)
	xor %eax, %eax
        NEXT
L_ftrunctos:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	fldl (%ebx)
	fnstcw (%ebx)
	movl (%ebx), %ecx	# save NDP control word		
	movl %ecx, %edx	
	movb $12, %dh
	movl %edx, (%ebx)
	fldcw (%ebx)
	addl %eax, %ebx	
	fistpl (%ebx)
	subl %eax, %ebx
	movl %ecx, (%ebx)
	fldcw (%ebx)		# restore NDP control word
	xor %eax, %eax	
	NEXT	
L_ftod:
	movl $WSIZE, %eax
	movl GlobalSp, %ebx
	addl %eax, %ebx
	fldl (%ebx)
	subl %eax, %ebx
	fnstcw (%ebx)
	movl (%ebx), %ecx	# save NDP control word	
	movl %ecx, %edx
	movb $12, %dh		
	movl %edx, (%ebx)
	fldcw (%ebx)
	addl %eax, %ebx	
	fistpq (%ebx)
	subl %eax, %ebx
	movl %ecx, (%ebx)
	fldcw (%ebx)		# restore NDP control word
	addl %eax, %ebx 
	movl (%ebx), %eax
	xchgl WSIZE(%ebx), %eax
	movl %eax, (%ebx)
	xor %eax, %eax	
	NEXT	
L_degtorad:
	fldl FCONST_180
        movl GlobalSp, %ebx
        addl $WSIZE, %ebx
        fldl (%ebx)
        fdivp %st, %st(1)
        fldpi
        fmulp %st, %st(1)
        fstpl (%ebx)
        NEXT
L_radtodeg:
        movl GlobalSp, %ebx
        addl $WSIZE, %ebx
        fldl (%ebx)
        fldpi
	fxch
        fdivp %st, %st(1)
        fldl FCONST_180
        fmulp %st, %st(1)
        fstpl (%ebx)
        NEXT
L_fne:
	call L_feq
	movl GlobalSp, %ebx
	notl WSIZE(%ebx)
	NEXT
L_feq:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	fldl (%ebx)
	addl %eax, GlobalSp
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	fldl (%ebx)
	fucompp
	fnstsw %ax
	andb $69, %ah
	xorb $64, %ah
	jne flt2
	jmp flt1
L_flt:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	fldl (%ebx)
	addl %eax, GlobalSp
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	fcompl (%ebx)
	fnstsw %ax
	andb $69, %ah
	jne flt2
flt1:	movl GlobalSp, %ebx
	movl $-1, WSIZE(%ebx)
	jmp fltexit
flt2:   movl GlobalSp, %ebx
	movl $0, WSIZE(%ebx)
fltexit:
	xor %eax, %eax
	ret
L_fgt:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	fldl (%ebx)
	addl %eax, GlobalSp
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	fcompl (%ebx)
	fnstsw %ax
	andb $69, %ah
	cmpb $1, %ah
	jne flt2
	jmp flt1	
L_fle:
	FOVER
	FOVER
	call L_feq
	_PUSH
	call L_flt
	_POP
	_OR
	ret
L_fge:
	FOVER
	FOVER
	call L_feq
	_PUSH
	call L_fgt
	_POP
	_OR
	ret
L_fzeroeq:
	movl $WSIZE, %eax
	movl GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %ecx
	movl %ebx, GlobalSp
	addl %eax, %ebx
	movl (%ebx), %eax
	shll $1, %eax
	orl %ecx, %eax
	jnz fzeroeq2
fzeroeq1:	
	movl $-1, (%ebx)
	jmp fzeroeqexit
fzeroeq2:
	movl $0, (%ebx)
fzeroeqexit:	
	xor %eax, %eax
	ret
L_fzerolt:
	movl $WSIZE, %eax
	addl %eax, GlobalSp
	movl GlobalSp, %ebx
	fldl (%ebx)
	add %eax, %ebx
	fldz
	fcompp	
	fnstsw %ax
	andb $69, %ah
	jne fzeroeq2		
	jmp fzeroeq1
L_fzerogt:
	ret				
L_fadd:
	movl $WSIZE, %eax
        addl %eax, GlobalSp
        movl GlobalSp, %ebx
        fldl (%ebx)
	sall $1, %eax
        addl %eax, %ebx
        faddl (%ebx)
        fstpl (%ebx)
	movl $WSIZE, %eax
        addl %eax, GlobalSp
	xor %eax, %eax
        NEXT
L_fsub:
	movl $WSIZE, %eax
	sall $1, %eax
	addl $WSIZE, %eax
        addl %eax, GlobalSp
        movl GlobalSp, %ebx
        fldl (%ebx)
	subl $WSIZE, %eax
        subl %eax, %ebx
        fsubl (%ebx)
        addl %eax, %ebx
        fstpl (%ebx)
	movl $WSIZE, %eax
        subl %eax, GlobalSp
	xor %eax, %eax
        NEXT
L_fmul:
	movl $WSIZE, %eax
        addl %eax, GlobalSp
        movl GlobalSp, %ebx
        fldl (%ebx)
	sall $1, %eax
        addl %eax, %ebx
        fmull (%ebx)
        fstpl (%ebx)
	sarl $1, %eax
        addl %eax, GlobalSp
	xor %eax, %eax
        NEXT
L_fdiv:
	movl $WSIZE, %eax
        addl %eax, GlobalSp
        movl GlobalSp, %ebx
        fldl (%ebx)
        ftst
        jnz fdiv1
        subl %eax, GlobalSp
        movl $E_DIV_ZERO, %eax
        jmp fdivexit
fdiv1:  addl $WSIZE, %ebx
	addl $WSIZE, %ebx
        fdivrl (%ebx)
        fstpl (%ebx)
        addl %eax, GlobalSp
fdivexit:
	xor %eax, %eax
	ret
L_fabs:
        movl GlobalSp, %ebx
        addl $WSIZE, %ebx
        fldl (%ebx)
        fabs
        fstpl (%ebx)
        NEXT
L_fneg:
        movl GlobalSp, %ebx
        addl $WSIZE, %ebx
        fldl (%ebx)
        fchs
        fstpl (%ebx)
        NEXT
L_floor:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl WSIZE(%ebx), %eax
	pushl %eax
	movl (%ebx), %eax
	pushl %eax
	call floor
	addl $8, %esp
	fstpl (%ebx)
	xor %eax, %eax		
	NEXT
L_fround:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	fldl (%ebx)
	frndint
	fstpl (%ebx)
	NEXT
L_ftrunc:
	movl GlobalSp, %ebx
	addl $WSIZE, %ebx
	fldl (%ebx)
	fnstcw (%ebx)
	movl (%ebx), %ecx	# save NDP control word
	movl %ecx, %edx
	movb $12, %dh
	movl %edx, (%ebx)
	fldcw (%ebx)
	frndint
	movl %ecx, (%ebx)
	fldcw (%ebx)		# restore NDP control word	
	fstpl (%ebx)		
	xor %eax, %eax
	NEXT
L_fsqrt:
	movl GlobalSp, %ebx
	fldl WSIZE(%ebx)
	fsqrt
	fstpl WSIZE(%ebx)
	NEXT
L_fcos:
	movl GlobalSp, %ebx
	fldl WSIZE(%ebx)
	fcos
	fstpl WSIZE(%ebx)
	NEXT
L_fsin:
	movl GlobalSp, %ebx
	fldl WSIZE(%ebx)
	fsin
	fstpl WSIZE(%ebx)
	NEXT
L_fatan2:
	movl GlobalSp, %ebx
	addl $2*WSIZE, %ebx
	fldl WSIZE(%ebx)
	fldl -WSIZE(%ebx)
	fpatan
	fstpl WSIZE(%ebx)
	movl %ebx, GlobalSp
	NEXT
L_fsincos:
	movl GlobalSp, %ebx
	fldl WSIZE(%ebx)
	fsincos
	fstpl -WSIZE(%ebx)
	fstpl WSIZE(%ebx)
	subl $2*WSIZE, %ebx
	movl %ebx, GlobalSp	
	NEXT
#

	.comm GlobalSp,4,4
	.comm GlobalIp,4,4
	.comm GlobalRp,4,4
	.comm BottomOfStack,4,4
	.comm BottomOfReturnStack,4,4
	.comm vmEntryRp,4,4
	.comm Base,4,4
	.comm State,4,4
	.comm pTIB,4,4
	.comm TIB,256,1
	.comm WordBuf,256,1
	.comm NumberCount,4,4
	.comm NumberBuf,256,1

	
