\ Regula Falsi -- ANS compatible version V1.1  12/14/2005
\ Finds roots of real transcendental functions by hybrid
\       secant/binary search method

\ Forth Scientific Library Algorithm #7
\ 
\ ================================================================
\ This version is adapted for kForth which uses an integrated
\ data/fp stack. The stack comments have been modified accordingly.
\ Under kForth, this file requires:
\
\	ans-words.4th
\
\ Krishna Myneni 10/14/2003
\ Revised 12/14/2005 -- changed all instances of SF@ and SF! to F@ and F!
\                       and use common words DEFER and IS for vectoring.
\ ================================================================
\
\ Usage example:
\       : F1    ( x -- [x-e**-x])   FDUP  FNEGATE  FEXP  F-  ;
\       USE( F1  0e 1e 1.E-6 )FALSI  5.671432E-1  ok

\ Environmental dependencies:
\       ANS FLOAT and FLOAT EXT wordsets
\       Common usage words [UNDEFINED]  DEFER  IS

\     (c) Copyright 1994  Julian V. Noble.  Permission is granted
\     by the author to use this software for any application provided
\     the copyright notice is preserved.



\ Non STANDARD words

[UNDEFINED] F0>  [IF]  : F0>  0e F> ;  [THEN]
[UNDEFINED] F2/  [IF]  : F2/  2e F/ ;  [THEN]
[UNDEFINED] USE( [IF]  : USE(    '  ;  [THEN]

\ tested in F-PC  10/6/1994

\ Data structures

FVARIABLE A                       \ f(xa)
FVARIABLE B                       \ f(xb)
FVARIABLE XA                      \ lower end of interval
FVARIABLE XB                      \ upper end of interval
FVARIABLE EPSILON                 \ precision

DEFER DUMMY                       \ vectored function name

\ End data structures

: X'    ( -- x')               \ secant extrapolation
\         F" XA + (XA - XB) * A / (B - A) "    ;
          XA F@  FDUP   XB F@  F-       ( xa xa-xb )
          A  F@  B F@   FOVER  F-  F/  F*  F+   ;

: <X'>  ( -- <x'>)             \ binary search extrapolation
\         F" (XA + XB) / 2 "  ;
          XA F@  XB F@  F+  F2/  ;

: SAME-SIGN?   ( x y -- flag)    F*   F0>  ;

: !END    ( x --)    FDUP  DUMMY   FDUP  ( -- x f[x] f[x] )
          A F@  SAME-SIGN?
          IF   A F!  XA F!   ELSE   B F!  XB F!   THEN   ;

: SHRINK   X'  !END   <X'>  !END  ;     \ combine extrapolations

: INITIALIZE    ( xt lower upper precision --)
        EPSILON F!    XB F!    XA F!       \ store parameters
        IS DUMMY                           \ xt -> DUMMY
        XA F@  DUMMY  A F!                 \ compute fn at endpts
        XB F@  DUMMY  B F!
         A F@  B F@
        SAME-SIGN?  ABORT" EVEN # OF ROOTS IN INTERVAL!"  ;

: CONVERGED?    ( -- f)
\       F" ABS( XA - XB ) < EPSILON "   ;
        XA F@   XB F@  F-  FABS   EPSILON F@  F<   ;

: )FALSI        ( xt lower upper precision --)
        INITIALIZE
        BEGIN    SHRINK  CONVERGED?    UNTIL
        <X'>  ( FE.) F.  ;
