\ lists-test.4th
\
\ Demonstration of basic list manipulation facilities provided by lists.4th.
\
\ Copyright (c) 2003, 2006 Creative Consulting for Research and Education
\ Provided under the GNU General Public License.
\
\ Revisions:
\
\	2003-04-14  km
\       2006-04-19  extensive revision using Hayes' style tests;
\                     renamed from test-lists.4th to lists-test.4th  km
\ Notes:
\
\ 1. Some simple examples are borrowed from P. Norvig's,
\    "Paradigms of Artificial Intelligence Programming:
\     Case Studies in Common Lisp"
\
\ 2. Some list functions are not exemplified here.
\

include ans-words
include strings
include lists
include tester
DECIMAL

\ Hayes' style notation for testing equality of list results

: }list  ( ... -- | Compare the stack [expected] contents with the saved [actual] contents)  
  depth actual-depth @  = IF        \ if depths match
    depth ?dup IF                   \ if there is something on the stack
      0 DO                          \ for each stack item
        actual-results i CELLS + a@      \ compare actual with expected
        equal NOT IF S" INCORRECT RESULT: " ERROR LEAVE THEN
      LOOP
    THEN 
  ELSE                                  \ depth mismatch
    s" WRONG NUMBER OF RESULTS: " ERROR 
  THEN
;


COMMENT Defining lists
TESTING '( 
nil  ptr  x
nil  ptr  y
nil  ptr  z 
{ '( a b c )  to  x -> }
{ '( 1 2 3 )  to  y -> }
{ '( @x @y )  to  z -> }
{ x -> '( a b c ) }list
{ y -> '( 1 2 3 ) }list
{ z -> '( ( a b c ) ( 1 2 3 ) ) }list

COMMENT List Element Retrieval
TESTING FIRST CAR REST CDR SECOND LAST
{ x  first  -> quote a  }list
{ x  car    -> quote a  }list
{ x  rest   -> '( b c ) }list
{ x  cdr    -> '( b c ) }list
{ x  second -> quote b  }list
{ x  last   -> '( c )   }list

COMMENT List Properties
TESTING LENGTH #ATOMS REVERSE CONS
{ x  length -> 3 }
{ x  #atoms -> 3 }
{ z  length -> 2 }
{ z  #atoms -> 6 }
{ x  reverse -> '( c b a ) }list
{ x  -> '( a b c ) }list
{ quote 0  y cons -> '( 0 1 2 3 ) }list

COMMENT Predicate Functions
TESTING NULL LISTP QUOTE NUMBERP ATOMP
{ nil  null -> TRUE }
{ x  null -> FALSE }
{ x  listp -> TRUE }
{ quote 3  listp -> FALSE }
{ quote 3  atomp -> TRUE }
{ quote 3  numberp -> TRUE }
{ quote a  numberp -> FALSE }
{ quote a  atomp -> TRUE }
{ z  car  listp -> TRUE }
{ x  car  listp -> FALSE }

COMMENT Equality Operators
TESTING EQ EQUAL
{ x x  eq -> TRUE }
{ x y  eq -> FALSE }
{ '( a b c )  x  eq -> FALSE }
{ '( a b c )  x  equal -> TRUE }
{ quote a  x car  eq -> TRUE }

COMMENT List Element Location
TESTING MEMBER NTH POSITION
{ quote 2 y  member  -> '( 2 3 ) }list
{ 2 x  nth   -> quote c }list
{ quote c x  position -> 2 }
{ quote d x  position -> -1 }

COMMENT List Element Removal, Deletion, Substitution
TESTING REMOVE SUBSTITUTE MAPCAR DELETE
{ quote 2 y  remove  -> '( 1 3 ) }list
{ y  -> '( 1 2 3 ) }list
{ quote 4 quote 2 y substitute -> '( 1 4 3 ) }list

: negate-number ( ^val1 -- ^val2 )
    dup numberp if car negate 0 cons then ;

{ y ' negate-number mapcar  -> '( -1 -4 -3 ) }list
{ quote 4 y  delete  -> '( 1 3 ) }list
{ y  -> '( 1 3 ) }list

COMMENT Construction of Lists from other Lists
TESTING LIST APPEND NCONC 
nil  ptr  r
nil  ptr  s 
{ '( a b c d e )  to  r -> }
{ '( f g h )  to  s -> }
{ r s  cons   -> '( ( a b c d e ) f g h )     }list
{ r s  list   -> '( ( a b c d e ) ( f g h ) ) }list 
{ r s  append -> '( a b c d e f g h )         }list
{ r           -> '( a b c d e )               }list
{ r s  nconc  -> '( a b c d e f g h )         }list
{ r           -> '( a b c d e f g h )         }list
{ s           -> '( f g h )                   }list


TESTING MEMBER:TEST
{ '( ( a b ) c )  to  r -> }
{ '( a b )  to  s -> }
{ s  r  member  -> nil }list
{ s  r  ' equal  member:test  -> '( ( a b ) c ) }list
{ s  r  reverse  ' equal member:test -> '( ( a b ) ) }list
{ r  -> '( ( a b ) c ) }list
{ r  reverse  to  r -> }
{ r  ->	'( c ( a b ) ) }list


COMMENT Set Operations
TESTING INTERSECTION UNION SET-DIFFERENCE SUBSETP ADJOIN
{ '( a b c d ) to r -> }
{ '( c d e )   to s -> }
{ r s  intersection	-> '( d c ) }list
{ r s  union		-> '( e a b c d ) }list
{ r s  set-difference	-> '( b a ) }list
{ s r  subsetp		-> FALSE }
{ quote b s  adjoin	-> '( b c d e ) }list
{ quote c s  adjoin	-> '( c d e ) }list


COMMENT Lists that span multiple lines	 

nil ptr fruits

{ '( apple pear banana grape kiwi papaya mango tomato 
   orange grapefruit tangerine watermelon cantaloupe )  to fruits  -> }

nil ptr vegetables

{ '( potato corn onion carrot bean pea )  to vegetables -> }

nil ptr produce

{ fruits vegetables union to produce -> }
produce print


cr .stat

