\ expint     Real Exponential Integral         ACM Algorithm #20

\ Forth Scientific Library Algorithm #1

\ Evaluates the Real Exponential Integral,
\     E1(x) = - Ei(-x) =   int_x^\infty exp^{-u}/u du      for x > 0
\ using a rational approximation

\ This code conforms with ANS requiring:
\      1. The Floating-Point word set
\ 

\ Collected Algorithms from ACM, Volume 1 Algorithms 1-220,
\ 1980; Association for Computing Machinery Inc., New York,
\ ISBN 0-89791-017-6

\ (c) Copyright 1994 Everett F. Carter.  Permission is granted by the
\ author to use this software for any application provided the
\ copyright notice is preserved.
\
\ Ported to kForth with minor revisions by K. Myneni, 2006-11-07

CR .( EXPINT     V1.1b                 21 September 1994   EFC )


: expint ( fx -- expint[x] )
    
    FDUP
    1.0e F< IF
	FDUP 0.00107857e F* 0.00976004e F-
	FOVER F*
	0.05519968e F+
	FOVER F*
        0.24991055e F-
        FOVER F*
        0.99999193e F+
        FOVER F*
        0.57721566e F-
        FSWAP FLN F-
    ELSE
	FDUP 8.5733287401e F+
        FOVER F*
        18.059016973e F+
        FOVER F*
        8.6347608925e F+
        FOVER F*
        0.2677737343e F+

        FOVER
        FDUP 9.5733223454e F+
        FOVER F*
        25.6329561486e F+
        FOVER F*
        21.0996530827e F+
        FOVER F*
        3.9584969228e F+

        FSWAP FDROP
        F/
        FOVER F/
        FSWAP -1.0e F* FEXP
        F*

    THEN
;

TEST-CODE? [IF]
\ test code generates a small table of selected E1 values.
\ most comparison values are from Abramowitz & Stegun,
\ Handbook of Mathematical Functions, Table 5.1

: expint_test ( -- )

    CR
    ."   x    E1(x) exact      ExpInt[x] " CR


    ."  0.5   0.5597736      "
    0.5e expint  F. CR

    ."  1.0   0.2193839      "
    1.0e expint   F. CR

    ."  2.0   0.0489005      "
    2.0e expint    F. CR

    ."  5.0   0.001148296    "
    5.0e expint    F. CR

    ." 10.0   0.4156969e-5   "
    10.0e expint   F. CR

;

[THEN]

