\ quadratic.4th
\
\ Solve for the two complex solutions of the quadratic equation:
\
\   a*x^2 + b*x + c = 0
\
\ Assume a, b, and c are real. The two complex roots are 
\ x1r + i*x1i and x2r + i*x2i
\
\ Copyright (c) 2002 K. Myneni. Permission is granted by the
\  author to use this software for any application provided this
\  copyright notice is preserved.
\
\ Revisions:
\   2002-11-02  km; first version.
\   2003-10-25  Christopher Brannon; fixed problem with calculation
\                 of complex roots, 
\   2007-11-04  km; revised comments; added test code; save and restore base. 
\
CR .( QUADRATIC         V1.0b         11 February  2007   KM )
BASE @ DECIMAL

fvariable qa
fvariable qb
fvariable qc
fvariable square_root_term
fvariable first_term
fvariable |second_term|

: solve_quadratic ( qa qb qc -- x1r x1i x2r x2i )
    qc F! qb F! qa F!
    qb F@ fdup F* 4e qa F@ F* qc F@ F* F- square_root_term F!
    qb F@ fnegate qa F@ 2e F* F/ first_term F!
    square_root_term F@ fdup F0< 
    IF fabs THEN fsqrt 2e qa F@ F* F/ |second_term| F!
	
    square_root_term F@ F0<
    IF
	\ complex conjugate roots 
	first_term F@ |second_term| F@
	first_term F@ |second_term| F@ fnegate
    ELSE
	\ two real roots
	first_term F@ |second_term| F@ F+ 0e
	first_term F@ |second_term| F@ F- 0e
    THEN
;

BASE !

TEST-CODE? [IF]   \ test code ==============================================
[undefined] T{      [IF]  include ttester  [THEN]    
BASE @ DECIMAL

1e-15 rel-near F!
1e-15 abs-near F!
set-near

\ Examples from:
\
\    http://www.purplemath.com/modules/quadform.htm

-2e 3e F/       fconstant -2/3
-3e 2e F/       fconstant -3/2
 5e fsqrt       fconstant sqrt{5}
 2e fsqrt 3e F/ fconstant sqrt{2}/3
 3e fsqrt 2e F/ fconstant sqrt{3}/2
10e fsqrt 2e F/ fconstant sqrt{10}/2

CR
TESTING SOLVE_QUADRATIC
t{ 1e  3e -4e solve_quadratic ->  1e 0e -4e 0e     rrrr}t
t{ 2e -4e -3e solve_quadratic ->  1e sqrt{10}/2 F+ 0e  1e sqrt{10}/2 F- 0e rrrr}t
t{ 1e -2e -4e solve_quadratic ->  1e sqrt{5} F+ 0e  1e sqrt{5} F- 0e  rrrr}t
t{ 9e 12e  4e solve_quadratic ->  -2/3 0e  -2/3 0e rrrr}t
t{ 3e  4e  2e solve_quadratic ->  -2/3 sqrt{2}/3  -2/3 sqrt{2}/3 fnegate rrrr}t
t{ 1e  3e  3e solve_quadratic ->  -3/2 sqrt{3}/2  -3/2 sqrt{3}/2 fnegate rrrr}t

BASE !
[THEN]

