\ fcalls.4th
\
\ Forth to C function Calling Interface for kForth
\ 
\ Requires asm-x86.4th
\
\ Krishna Myneni, Creative Consulting for Research & Education
\ krishna.myneni@ccreweb.org
\
\ Requires:
\   ans-words
\   strings
\
\ Revisions:
\   2009-10-02  km  created
\   2009-10-04  km  implemented generic fcall


\ The fcallx words all assume that the arguments are 
\ single or double cell values, with no double precision
\ floating point args.

\ In order to use the generic fcall, we make an assumption
\ about the stack usage of the function we are calling. Below, we
\ assume that the called function does not require > 64K of stack.
65536 constant MAX_FUNC_STACK

CODE fcall ( ncells addr -- val)
   0 [ebx] eax mov,
   TCELL # ebx add,
   0 [ebx] ecx mov,
   TCELL # ebx add,
           ebx push,
       ecx edx mov,
    DO,
       0 [ebx] push,
       TCELL # ebx add,
    LOOP,
         MAX_FUNC_STACK # esp sub,
	 edx push,
	 MAX_FUNC_STACK TCELL + # esp add,
         eax call,
	 MAX_FUNC_STACK TCELL + # esp sub,
	 ecx pop,
	 MAX_FUNC_STACK # esp add,

	 ecx edx mov,	
	 2 # ecx shl,
	 ecx esp add,

         ebx pop,

         edx ecx mov,
	 ecx dec,
	 2 # ecx shl,
	 ecx ebx add,

     eax 0 [ebx] mov,
     0 # eax mov,
END-CODE


CODE fcall0 ( addr -- val )
    0 [ebx] eax mov,
    ebx push,
    eax call,
    ebx pop,          \ restore Forth stack pointer
    eax 0 [ebx] mov,
    0 # eax mov,
END-CODE


CODE fcall1 ( arg addr -- val ) 
    0 [ebx] eax mov,
    TCELL # ebx add,
    ebx push,
    0 [ebx] push,
    eax call,
    ecx pop,
    ebx pop,
    eax 0 [ebx] mov,
    0 # eax mov,
END-CODE


CODE fcall1b ( arg addr -- ) 
    0 [ebx] eax mov,
    TCELL # ebx add,
    ebx push,
    0 [ebx] push,
    eax call,
    eax pop,
    ebx pop,
    TCELL # ebx add,
    0 # eax mov,
END-CODE




CODE fcall2b ( arg1 arg2 addr -- )
    0 [ebx] eax mov,
    TCELL # ebx add,
    ebx push,
    2 0 DO I TCELL * [ebx] push, LOOP
    eax call,
    TCELL 2* # esp add,
    ebx pop,
    TCELL 2* # ebx add,
    0 # eax mov,
END-CODE


[undefined] ptr [if] : ptr create 1 cells ?allot ! does> a@ ; [then]

\ High-level interface to C functions
\
\ Arguments:
\    addr  -- function address
\    aargs -- address of counted string describing the arguments
\    u     -- length of aargs string
\    nargs -- number of arguments, for verification of parsing aargs
\
\ Example:
\
\    pcos s" r -- r " 1 cfunc cos
\
\ The string describing the arguments to the C function is a series
\ of symbols separated by one or more spaces, which indicates the Forth
\ data type being passed to or returned from the C function. The symbols 
\ are chosen according to the following conventions:
\
\     a   an address:               void*, char*, int*, double*, char**, etc.
\     n   a single cell value:      int, unsigned, etc.
\     u   a single cell value:      unsigned, etc.
\     d   a double cell integer:    long long
\     s   a single precision float: float
\     r   a double precision float: double
\    --   separator between arguments and return values

: parse_token ( a u -- a2 u2 a3 u3)
	\ parse next token from the string; a3 u3 is the token string
	bl skip 2dup bl scan 2>r r@ - 2r> 2swap ;


0 value cfunc_ncells

: inc_ncells ( -- ) cfunc_ncells 1+ to cfunc_ncells ;

: cfunc_parse_args ( aargs u nargs -- )
     -rot
     0 to cfunc_ncells
     BEGIN
       parse_token
     WHILE
       c@ 
       CASE
         [char] a OF ( ... ) inc_ncells ENDOF
         [char] n OF ( ... ) inc_ncells ENDOF
         [char] u OF ( ... ) inc_ncells ENDOF
         [char] d OF ( ... ) inc_ncells inc_ncells ENDOF
         [char] s OF ( ... ) inc_ncells ENDOF
         [char] r OF ( ... ) inc_ncells inc_ncells ENDOF
         true ABORT" Unrecognized argument type!"
       ENDCASE
       rot 1- -rot .s
     REPEAT
     drop 2drop  
     ABORT" Number of arguments does not match!"
;

0 ptr cfunc_pfa

: cfunc ( addr aargs u nargs  "name" -- )
     create 2 cells ?allot to cfunc_pfa
        rot cfunc_pfa !
	cfunc_parse_args 
	cfunc_ncells cfunc_pfa cell+ !
     does> 
        2@ \ fcall
;





