// ForthCompiler.h
//
// Copyright (c) 1998--2009 Krishna Myneni, Creative Consulting for
//   Research and Education
//
// This software is provided under the terms of the GNU General Public License.
//
// Revisions: 
//   2005-09-19
//   2006-03-16  incremented NUMBER_OF_INTRINSIC_WORDS
//               ...
//   2006-04-07  ... by 8 dnw
//   2006-04-08  NUMBER_OF_INTRINSIC_WORDS replaced
//                           by calculation dnw
//   2008-03-17  added declaration of CompileWord()  km
//   2008-03-21  fixed declaration of IsFloat to use argument of type
//                 double* rather than float*  km
//   2008-07-05  changed declaration of WordTemplate member WordName
//                 from char* to const char* to avoid gcc 4.2.x warnings  km
//   2009-10-04  changed DictionaryEntry to WordListEntry; added SearchList  km
#ifndef __FORTHCOMPILER_H__
#define __FORTHCOMPILER_H__

#define byte unsigned char
#define word unsigned short int

#define PRECEDENCE_NONE         0
#define PRECEDENCE_IMMEDIATE    1
#define PRECEDENCE_NON_DEFERRED 2
#define EXECUTE_NONE            0
#define EXECUTE_UP_TO           1
#define EXECUTE_CURRENT_ONLY    2
#define TRUE -1
#define FALSE 0
#define MAX_C_ERR_MESSAGES 13

// Error codes; The corresponding error messages are given in
//   the const char* array C_ErrorMessages, in ForthCompiler.cpp

#define E_C_NOERROR         0x100
#define E_C_ENDOFSTREAM     0x101
#define E_C_ENDOFDEF        0x102
#define E_C_ENDOFSTRING     0x103
#define E_C_NOTINDEF        0x104
#define E_C_OPENFILE        0x105
#define E_C_INCOMPLETEIF    0x106
#define E_C_INCOMPLETEBEGIN 0x107
#define E_C_UNKNOWNWORD     0x108
#define E_C_NODO            0x109
#define E_C_INCOMPLETELOOP  0x10A
#define E_C_INCOMPLETECASE  0x10B
#define E_C_VMERROR         0x10C

struct WordListEntry
{
  char WordName[32];
  word WordCode;
  byte Precedence;
  void* Cfa;
  void* Pfa;
};

class WordList : public vector<WordListEntry> 
{
public:
   vector<WordListEntry>::iterator RetrieveFromName( char* );
   vector<WordListEntry>::iterator RetrieveFromCfa( void* );
};

class Vocabulary
{
public:
   const char* ListName;
   WordList* pList;
   Vocabulary (const char*, WordList*);
};

class SearchList : public vector<Vocabulary*>
{
public:
   vector<WordListEntry>::iterator LocateWord( char* );
   vector<WordListEntry>::iterator LocateCfa( void* cfa );
};

struct WordTemplate
{
    const char* WordName;
    word WordCode;
    byte Precedence;
};

char* ExtractName (char*, char*);
int IsForthWord (char*, WordListEntry*);
int IsFloat (char*, double*);
int isBaseDigit (int);
int IsInt (char*, int*);
int ForthCompiler (vector<byte>*, int*);
void CompileWord (WordListEntry);
void OutputForthByteCode (vector<byte>*);
void SetForthInputStream (istream&);
void SetForthOutputStream (ostream&);

#endif
