\ lib-interface.4th
\
\ High-Level Interface to functions in an external shared object 
\ library.
\
\ Copyright (c) 2009 Krishna Myneni, Creative Consulting for
\ Research & Education, http://ccreweb.org
\
\ This software is provided under the GNU Lesser General Public
\ License (LGPL).
\
\ Requires kForth (ver >= 1.5.x)
\
\ Also requires:
\   ans-words
\   strings
\   asm
\   fcalls
\
\
\ Revisions:
\   2009-10-07  km  created
\
[undefined] fcall [if] s" fcalls.4th" included [then]
[undefined] ptr   [if] : ptr create 1 cells ?allot ! does> a@ ; [then]

base @
decimal

1 constant RTLD_LAZY

\ Make a null-terminated string.
\ This particular implementation is kForth-specific; all strings 
\ are null-terminated in kForth
: z" ( "string" -- a )
    postpone s" 
    postpone drop ;  immediate   


: check-dlerror ( -- ) 
    dlerror dup IF  dup strlen cr type cr ABORT THEN drop ;

0 value hndLib

: open-lib ( ^name -- )
    RTLD_LAZY dlopen to hndLib
    hndLib 0= IF check-dlerror THEN    
;

: close-lib ( -- )
    hndLib dlclose
    dup IF cr ." dlclose returned " . ELSE drop THEN
;

\ Load symbol from library
: lsym ( ^symbol -- addr )
	hndLib swap dlsym check-dlerror ;

\ High-level interface to C functions
\
\ Arguments:
\    addr  -- function address
\    aargs -- address of counted string describing the arguments
\    u     -- length of aargs string
\    nargs -- number of arguments, for verification of parsing aargs
\
\ Example:
\
\    c" cos" lsym  C-word cos ( r -- r )
\
\ The string describing the arguments to the C function is a series
\ of symbols separated by one or more spaces, which indicates the Forth
\ data type being passed to or returned from the C function. The symbols 
\ are chosen according to the following conventions:
\
\     a   an address:               void*, char*, int*, double*, char**, etc.
\     n   a single cell value:      int, unsigned, etc.
\     u   a single cell value:      unsigned, etc.
\     d   a double cell integer:    long long
\     s   a single precision float: float
\     r   a double precision float: double
\    --   separator between arguments and return values

0 value cword_ncells
0 value cword_nargs
0 value cword_nret

: inc_ncells ( -- ) cword_ncells 1+ to cword_ncells ;
: inc_nargs  ( -- ) cword_nargs  1+ to cword_nargs ;

: cword_parse_args ( "args_ret" -- )
     0 to cword_nargs
     0 to cword_ncells
     bl word count 0= swap c@ [char] ( <> or 
     ABORT" Start of argument list not found!"
     BEGIN
       bl word count
     WHILE
       c@ 
       CASE
         [char] a OF ( ... ) inc_ncells ENDOF
         [char] n OF ( ... ) inc_ncells ENDOF
         [char] u OF ( ... ) inc_ncells ENDOF
         [char] d OF ( ... ) inc_ncells inc_ncells ENDOF
         [char] s OF ( ... ) inc_ncells ENDOF
         [char] r OF ( ... ) inc_ncells inc_ncells ENDOF
         [char] ) OF ( drop 2drop) EXIT    ENDOF
         true ABORT" Unrecognized argument type!"
       ENDCASE
       inc_nargs
     REPEAT
     ( drop 2drop)
;

0 ptr cword_pfa

: C-word ( ^Cname "name" "args_ret" -- )
     lsym
     create 2 cells ?allot to cword_pfa
        cword_pfa !
	cword_parse_args 
	cword_ncells cword_pfa cell+ !
     does> 
        2@ fcall
;


base !

