\ 
\ simple-drawing.4th - demonstrate drawing of pixels, lines, arcs, etc.
\                    on a window. All drawings are done in black color
\                    over a white background.
\
\ Adapted from the C program at the link below:
\
\ http://users.actcom.co.il/~choo/lupg/tutorials/xlib-programming/simple-drawing.c
\

include ans-words
include asm
include strings
include lib-interface
include libX11

\
\ function: create_simple_window. Creates a window with a white background
\           in the given size.
\ input:    display, size of the window (in pixels), and location of the window
\           (in pixels).
\ output:   the window's ID.
\ notes:    window is created with a black border, 2 pixels wide.
\           the window is automatically mapped after its creation.
\ 

0 value win_border_width
0 value nwidth
0 value nheight
0 value nx
0 value ny
0 value disp
0 value screen_num
0 value win

: create_simple_window ( adisplay  nwidth nheight nx ny -- nWindow )
    to ny  to nx  to nheight  to nwidth  to disp
    disp XDefaultScreen  to screen_num
    2 to win_border_width

    \ create a simple window, as a direct child of the screen's 
    \ root window. Use the screen's black and white colors as
    \ the foreground and background colors of the window
    \ respectively. Place the new window's top-left corner at
    \ the given 'x,y' coordinates.
  
    disp
    disp screen_num XRootWindow
    nx  ny  nwidth nheight win_border_width
    disp screen_num  XBlackPixel
    disp screen_num  XWhitePixel
    XCreateSimpleWindow  to win

    \ Obtain MapNotify events
    disp win StructureNotifyMask XSelectInput drop

    \ make the window actually appear on the screen. 
    disp win XMapWindow drop

    \ flush all pending requests to the X server.
    disp XFlush drop

    win
;

0 value gc                              \ handle of newly created GC. 
0 value valuemask                       \ which values in 'gcvalues' to 
                                        \   check when creating the GC.
variable gcvalues                       \ initial values for the GC.
2 value line_width                      \ line width for the GC.

0 value reverse_video
0 value win
0 value disp
0 value screen_num

: create_gc ( adisplay win reverse_video -- nGC )
  to reverse_video
  to win
  to disp
  
  disp XDefaultScreen to screen_num

  disp win valuemask gcvalues XCreateGC to gc
  gc 0< ABORT" Unable to create the graphics context!"

  \ allocate foreground and background colors for this GC.
  reverse_video IF
    disp gc  disp screen_num XWhitePixel  XSetForeground drop
    disp gc  disp screen_num XBlackPixel  XSetBackground drop
  ELSE
    disp gc  disp screen_num XBlackPixel  XSetForeground drop
    disp gc  disp screen_num XWhitePixel  XSetBackground drop
  THEN

  \ define the style of lines that will be drawn using this GC.
  disp gc line_width LineSolid CapButt JoinBevel XSetLineAttributes drop

  \ define the fill style for the GC to be 'solid filling'.
  disp gc FillSolid  XSetFillStyle drop

  gc
;


0 value screen_num     \ number of screen to place the window on.
0 value win            \ pointer to the newly created window.
0 value gc             \ GC (graphics context) used for drawing in our window.
0 value display_width
0 value display_height \ height and width of the X display.
0 value width
0 value height         \ height and width for the new window.

variable display       \ pointer to X Display structure.
create event 32 cells allot

: simple-drawing ( -- )
			
  \ open connection with the X server.
  s" :0.0" $>zstr XOpenDisplay dup display !
  0= IF
    cr ." cannot connect to X server" cr
    ABORT
  THEN

  \ get the geometry of the default screen for our display.
  display @ XDefaultScreen to screen_num
  display @ screen_num XDisplayWidth  to display_width 
  display @ screen_num XDisplayHeight to display_height

  \ make the new window occupy 1/9 of the screen's size.
  display_width  3 /  to width
  display_height 3 /  to height
  cr ." window width - " width . 2 spaces ." height - " height . cr

  \ create a simple window, as a direct child of the screen's
  \ root window. Use the screen's white color as the background
  \ color of the window. Place the new window's top-left corner 
  \ at the given 'x,y' coordinates.
  display @ width height 0 0 create_simple_window to  win

  \ allocate a new GC (graphics context) for drawing in the window.
  display @ win 0 create_gc to gc

  \ need to wait for map notify event
  BEGIN
    display @ event XNextEvent drop
    event @ MapNotify = 
  UNTIL

  display @ false XSync drop

  \ draw one pixel near each corner of the window
  display @ win gc 5 5 XDrawPoint drop
  display @ win gc 5 height 5 - XDrawPoint  drop
  display @ win gc width 5 - 5  XDrawPoint  drop
  display @ win gc width 5 - height 5 - XDrawPoint drop

  \ draw two intersecting lines, one horizontal and one vertical,
  \ which intersect at point "50,100".
  display @ win gc 50   0  50 200  XDrawLine drop
  display @ win gc  0 100 200 100  XDrawLine drop

  \ now use the XDrawArc() function to draw a circle whose diameter
  \ is 30 pixels, and whose center is at location '50,100'.
  display @ win gc  50 30 2/ -  100 30 2/ -  30 30 0 360 64 *  XDrawArc drop

  (  XPoint points[] = 
      {0, 0},
      {15, 15},
      {0, 15},
      {0, 0}
  )
  \  int npoints = sizeof(points)/sizeof(XPoint)

    \ draw a small triangle at the top-left corner of the window.
    \ the triangle is made of a set of consecutive lines, whose
    \ end-point pixels are specified in the 'points' array.
   \ display win @ gc @ points npoints CoordModeOrigin  XDrawLines

  \ draw a rectangle whose top-left corner is at '120,150', its width is
  \ 50 pixels, and height is 60 pixels.
  display @ win gc 120 150 50 60  XDrawRectangle drop

  \ draw a filled rectangle of the same size as above, to the left of the
  \ previous rectangle.
  display @ win gc 60 150 50 60  XFillRectangle drop

  \ flush all pending requests to the X server.
  display @ XFlush drop
  display @ false XSync drop

  \ make a delay for a short period.
  4000000 usleep

  \ close the connection to the X server.
  display @ XCloseDisplay drop
;

simple-drawing

