\ Regula Falsi -- ANS compatible version V1.1  12/14/2005
\ Finds roots of real transcendental functions by hybrid
\       secant/binary search method

\ Forth Scientific Library Algorithm #7

\ Usage example:
\       : F1    ( x -- [x-e**-x])   FDUP  FNEGATE  FEXP  F-  ;
\       use( F1  0e 1e 1.E-6 )falsi  5.671432E-1  ok

\ Environmental dependencies:
\       Integrated data/fp stack
\       ANS FLOAT and FLOAT EXT wordsets
\       Common usage words [UNDEFINED]  DEFER  IS

\     (c) Copyright 1994  Julian V. Noble.  Permission is granted
\     by the author to use this software for any application provided
\     the copyright notice is preserved.

\ Revisions:
\   2003-10-14  km; Adapted for kForth (integrated data/fp stack)
\   2005-12-14  km; changed all instances of SF@ and SF! 
\                   to F@ and F!, and use common words DEFER and
\                   IS for vectoring.
\   2007-10-10  km; added automated test code.
\   2007-10-27  km; save base, switch to decimal, and restore base
\   2007-10-31  km; changed fvariable names of A and B to YA and YB
\                   to avoid masking the flocals implementation in
\                   fsl-util.

CR .( REGFALSI          V1.1d          31 October 2007   JVN )
BASE @ DECIMAL

\ Non STANDARD words

[UNDEFINED] f0>   [IF]  : f0>  0e F> ;      [THEN]
[UNDEFINED] f2/   [IF]  : f2/  2e F/ ;      [THEN]
[UNDEFINED] f-rot [IF] : f-rot  FROT FROT ; [THEN]
[UNDEFINED] use(  [IF]  : use(    '  ;      [THEN]

\ tested in F-PC  10/6/1994

\ Data structures

FVARIABLE YA                      \ f(xa)
FVARIABLE YB                      \ f(xb)
FVARIABLE XA                      \ lower end of interval
FVARIABLE XB                      \ upper end of interval
FVARIABLE EPSILON                 \ precision

DEFER DUMMY                       \ vectored function name

\ End data structures

: X'    ( -- x')               \ secant extrapolation
\         F" XA + (XA - XB) * A / (B - A) "    ;
          XA F@  FDUP   XB F@  F-       ( xa xa-xb )
          YA F@  YB F@  FOVER  F-  F/  F*  F+   ;

: <X'>  ( -- <x'>)             \ binary search extrapolation
\         F" (XA + XB) / 2 "  ;
          XA F@  XB F@  F+  F2/  ;

: SAME-SIGN?   ( x y -- flag)    F*   F0>  ;

: !END    ( x --)    FDUP  DUMMY   FDUP  ( -- x f[x] f[x] )
          YA F@  SAME-SIGN?
          IF   YA F!  XA F!   ELSE   YB F!  XB F!   THEN   ;

: SHRINK   X'  !END   <X'>  !END  ;     \ combine extrapolations

: INITIALIZE    ( xt lower upper precision --)
        EPSILON F!    XB F!    XA F!       \ store parameters
        IS DUMMY                           \ xt -> DUMMY
        XA F@  DUMMY  YA F!                \ compute fn at endpts
        XB F@  DUMMY  YB F!
        YA F@  YB F@
        SAME-SIGN?  ABORT" EVEN # OF ROOTS IN INTERVAL!"  ;

: CONVERGED?    ( -- f)
\       F" ABS( XA - XB ) < EPSILON "   ;
        XA F@   XB F@  F-  FABS   EPSILON F@  F<   ;

: )FALSI        ( xt lower upper precision --)
        INITIALIZE
        BEGIN    SHRINK  CONVERGED?    UNTIL
        <X'>  ( FE.) F.  ;

BASE !
    
TEST-CODE? [IF]   \ test code ==============================================
[undefined] T{      [IF]  include ttester.4th  [THEN]
BASE @ DECIMAL

1e-15 rel-near F!
1e-15 abs-near F!
set-near

: )root  ( xt -- ) ( F: upper lower precision -- root )
    initialize BEGIN shrink converged? UNTIL <x'> ;

: f1  ( x -- [x-e**-x])   FDUP  FNEGATE  FEXP  F-  ;

CR
TESTING )ROOT
t{ use( FSIN 3e 3.2e abs-near F@ )root  FSIN -> 0e  r}t
t{ use( FSIN 3e 3.2e abs-near F@ )root       -> PI  r}t
t{ use( f1   0e 1e   abs-near F@ )root  f1   -> 0e  r}t
t{ use( f1   0e 1e   abs-near F@ )root       -> 5.671432904097833E-1  r}t

\ First order spherical Bessel function ( find its roots as a test case )
: j1  ( x -- j1[x] )
    FDUP  FDUP F* FSWAP FSINCOS FROT F/
    FNEGATE F-ROT FSWAP F/ F+ ;

BASE !
[THEN]
