\ demo02.4th 
\
\  Demonstrate refreshed drawing and event handling in X.
\
\ Copyright (c) 2009 Krishna Myneni, Creative Consulting for
\   Research & Education, krishna.myneni@ccreweb.org
\
\ This code is released under the GNU Lesser GPL (LGPL).
\
\ Revisions:
\
include ans-words
include asm
include strings
include lib-interface
include libX11


0 value screen_num     \ number of screen to place the window on.
0 value win            \ pointer to the newly created window.
0 value gc             \ GC (graphics context) used for drawing in our window.
0 value colormap
0 value width
0 value height         \ height and width for the new window.
0 value xc
0 value yc             \ center coordinates of new window
variable display       \ pointer to X Display structure.
variable gcvalues
0 value colormap

XColor col_exact
XColor col_screen
0 value red
0 value pink
0 value green
0 value yellow
0 value orange
0 value blue
0 value black
0 value white

false value smile
XEvent event

\ Allocate a color
: get-color ( addr u -- pixel )
    $>zstr display @ colormap rot col_exact XParseColor
    IF
      display @ colormap col_exact XAllocColor drop
      col_exact XColor->pixel @
    ELSE 
      cr ." Failed to lookup color"
      black
    THEN
;

\ Allocate our colors
: get-colors ( -- )
    s" red"  get-color to red
    s" pink" get-color to pink
    s" green" get-color to green
    s" yellow" get-color to yellow
    s" orange" get-color to orange
    s" blue"   get-color to blue
;

\ Show info on a mouse button press event
: button-press-info ( -- )
    cr ." Button "
    event XButtonEvent->button ? ." pressed at x = "
    event XButtonEvent->x ? ."  y = " event XButtonEvent->y ?
;

\ Show info on a keypress event
: key-press-info ( -- )
    cr ." Key pressed with keycode = " event XKeyEvent->keycode ? 
    ."  at time " event XKeyEvent->time ? ." ms"
;

\ Set the foreground color
: foreground ( pixelval -- )
    >r display @ gc r> XSetForeground drop ;

\ Draw a filled circle, centered at x, y, with radius r
: filled-circle ( nx ny nr -- )
    dup >r - swap r@ - swap 2>r 
    display @ win gc 2r> r> 2* dup 0 360 64 * XFillArc drop ;

\ Apply coordinate offsets: x' = x + dx, y' = y + dy
: offset-xy ( nx ny ndx ndy -- nx' ny' )
    rot + >r + r> ;

: rainbow ( -- )
    display @ gc 5 LineSolid CapRound JoinRound XSetLineAttributes drop
    display @ win gc red foreground   0 0  width height 0 180 64 * XDrawArc drop
    display @ win gc orange foreground 5 0  width 10 - height  5 - 0 180 64 * XDrawArc drop
    display @ win gc yellow foreground 10 0  width 20 - height 10 - 0 180 64 * XDrawArc drop
    display @ win gc green foreground 15 0  width 30 - height  15 - 0 180 64 * XDrawArc drop
    display @ win gc blue  foreground 20 0  width 40 - height  20 - 0 180 64 * XDrawArc drop
    display @ gc 1 LineSolid CapRound JoinRound XSetLineAttributes drop
;

: smiley-face ( x y -- )
    2>r
    yellow foreground 
    2r@ 15 filled-circle  \ head
    black  foreground
    2r@ -6 -4 offset-xy 5 filled-circle  \ left eye
    2r@  6 -4 offset-xy 5 filled-circle  \ right eye
    display @ win gc 2r@
    smile IF
       -10 -10 offset-xy 20 20 220 64 * 100 64 *   \ smiley mouth 
    ELSE
       -10   5 offset-xy 20 20 40 64 * 100 64 *    \ frowney mouth
    THEN
    XDrawArc drop
    2r> 2drop
;

: redraw-window ( -- )
    width 2/ to xc
    height 2/ to yc
    display @ win gc xc 50 - yc 20 - 100 40  XDrawRectangle drop
    display @ win gc xc 50 - yc 20 -  0  0   XDrawLine drop
    display @ win gc xc 50 + yc 20 -  width 0 XDrawLine drop
    display @ win gc xc 50 - yc 20 +  0  height XDrawLine drop
    display @ win gc xc 50 + yc 20 +  width height XDrawLine drop
    display @ win gc xc 40 - yc  s" Hello X World!"  XDrawString drop
    \ rainbow
    xc yc 70 - smiley-face
    xc 100 - yc smiley-face
    xc yc 70 + smiley-face
    xc 100 + yc smiley-face
;


: demo ( -- )
			
    \ open connection with the X server.
    s" :0.0" $>zstr XOpenDisplay dup display !
    0= IF
      cr ." cannot connect to X server" cr
      ABORT
    THEN

    \ get the default colormap and window size for our display.
    display @ XDefaultScreen to screen_num
    display @ screen_num XBlackPixel to black
    display @ screen_num XWhitePixel to white
    display @ screen_num XDefaultColormap to colormap
    get-colors

    display @ screen_num XDisplayWidth  4 / to width 
    display @ screen_num XDisplayHeight 4 / to height
    
    display @
    display @ screen_num XRootWindow
    0 0  width height  0  black  white
    XCreateSimpleWindow  to win

    \ Obtain MapNotify, Key press, Button press, Window resize, and
    \ Window destroy events
    display @ win
    ExposureMask KeyPressMask or ButtonPressMask or
    StructureNotifyMask or EnterWindowMask or LeaveWindowMask or
    XSelectInput drop

    \ make the window actually appear on the screen. 
    display @ win XMapWindow drop

    \ flush all pending requests to the X server.
    display @ XFlush drop

    \ allocate a new GC (graphics context) for drawing in the window.
    display @ win 0 gcvalues XCreateGC to gc
    gc 0< ABORT" Unable to create the graphics context!"
    display @ gc FillSolid  XSetFillStyle drop

    \ flush all pending requests to the X server.
    display @ XFlush drop
    display @ false XSync drop

    \ Event loop
    BEGIN
      display @ event XNextEvent drop
      event @ 
      CASE
        Expose        OF  redraw-window      ENDOF
        ButtonPress   OF  button-press-info  ENDOF
        KeyPress      OF  key-press-info     ENDOF
        ConfigureNotify OF 
                     event XConfigureRequestEvent->width @ to width
                     event XConfigureRequestEvent->height @ to height
                     ENDOF 
        EnterNotify   OF   true  to smile  redraw-window    ENDOF
        LeaveNotify   OF   false to smile  redraw-window    ENDOF
      ENDCASE
      event @ Keypress =
    UNTIL

    \ close the connection to the X server.
    display @ gc XFreeGC drop
    display @ XCloseDisplay drop
;

demo

