\ X11 interface
\
\ Forth words for interfacing to X Windows (ver. 11).
\ Provides words corresponding to the C interface (Xlib).
\ 
\ Krishna Myneni, Creative Consulting for Research and Education,
\ krishna.myneni@ccreweb.org
\
\ Notes:
\
\ 1. Much of this file was adapted from x11.fs in reference [1]
\
\ 2. For a lucid overview and introduction to X Windows programming,
\    see Ref. [3]. A detailed function reference in provided in [4].
\
\
\ References:
\
\ [1] bigForth, v.3.2, Bernd Paysan, http://
\       see the file, x11.fs
\
\ [2] Xorg, http://
\       Xlib header files: X.h, Xlib.h
\
\ [3] Xlib Programming Manual for version 11, by Adrian Nye, 
\     O'Reilly and Associates
\
\ [4] Xlib Reference Manual for version 11, ed. by Adrian Nye, 
\     O'Reilly and Associates
\
\ Revisions:
\   2009-10-27  km  continue to add and group functions

vocabulary X11
also X11 definitions

[undefined] struct [IF] s" struct.4th" included     [THEN]
[undefined] int16: [IF] s" struct-ext.4th" included [THEN]

include X.4th
include Xlib.4th

0 value hndl_X11
s" libX11.so.6" open-lib        \ change if library name is different 
dup 0= [IF] check-lib-error [THEN]
to hndl_X11
cr .( Openend the X11 library )

base @ hex

: noop ;
' noop alias has-utf8
' noop alias has-png
' noop alias has-xft

s" XSync"           C-word  XSync       ( adpy ndiscard -- n )
s" XFlush"          C-word  XFlush      ( adpy -- n )

s" XMaxRequestSize"      C-word  XMaxRequestSize      ( adpy -- n )
s" XExtendedMaxRequestSize" C-word XExtendedMaxRequestSize ( adpy -- n )
s" XResourceManagerString" C-word XResourceManagerString ( adpy -- astring )
s" XScreenResourceString" C-word XScreenResourceString ( ascreen -- astring )

s" XVisualIDFromVisual"  C-word  XVisualIDFromVisual  ( avisual -- nvisualID )

s" XAddExtension"        C-word  XAddExtension        ( adpy -- aextcodes )
s" XEHeadOfExtensionList" C-word XEHeadOfExtensionList ( nobject -- aaextdata )
s" XDefaultRootWindow"   C-word  XDefaultRootWindow   ( adpy -- nwindow )
s" XRootWindowOfScreen"  C-word  XRootWindowOfScreen  ( ascreen -- nwindow )
s" XDefaultVisualOfScreen" C-word XDefaultVisualOfScreen ( ascreen -- avisual )
s" XDefaultGCOfScreen"   C-word  XDefaultGCOfScreen   ( ascreen -- ngc )
s" XBlackPixelOfScreen"  C-word  XBlackPixelOfScreen  ( ascreen -- u )
s" XWhitePixelOfScreen"  C-word  XWhitePixelOfScreen  ( ascreen -- u )
s" XNextRequest"         C-word  XNextRequest         ( adpy -- u )
s" XLastKnownRequestProcessed" C-word XLastKnownRequestProcessed ( adpy -- u )
s" XServerVendor"        C-word  XServerVendor        ( adpy -- astring )
s" XDisplayString"       C-word  XDisplayString       ( adpy -- astring )
s" XDefaultColormapOfScreen" C-word XDefaultColormapOfScreen ( ascreen -- ncolormap )

s" XDefaultScreenOfDisplay" C-word XDefaultScreenOfDisplay ( adpy -- ascreen )
s" XEventMaskOfScreen"   C-word  XEventMaskOfScreen   ( ascreen -- nmask )
s" XScreenNumberOfScreen" C-word XScreenNumberOfScreen ( ascreen -- n )

s" XConnectionNumber"    C-word  XConnectionNumber    ( adpy -- n )
s" XDefaultDepthOfScreen" C-word XDefaultDepthOfScreen ( ascreen -- n )
s" XDefaultScreen"       C-word  XDefaultScreen       ( adpy -- n )
s" XDoesBackingStore"    C-word  XDoesBackingStore    ( ascreen -- n )
s" XDoesSaveUnders"      C-word  XDoesSaveUnders      ( ascreen -- nflag )


s" XFreeExtensionList"   C-word  XFreeExtensionList   ( aalist -- n )


s" XMaxCmapsOfScreen"    C-word  XMaxCmapsOfScreen    ( ascreen -- n )
s" XMinCmapsOfScreen"    C-word  XMinCmapsOfScreen    ( ascreen -- n )
s" XNoOp"                C-word  XNoOp                ( adpy -- n )
s" XPlanesOfScreen"      C-word  XPlanesOfScreen      ( ascreen -- n )

s" XRefreshKeyboardMapping" C-word XRefreshKeyboardMapping ( aevent_map -- n )
s" XResetScreenSaver"    C-word  XResetScreenSaver    ( adpy -- n )
s" XScreenCount"         C-word  XScreenCount         ( adpy -- n )

s" XVendorRelease"       C-word  XVendorRelease       ( adpy -- n )

s" XSetLocaleModifiers"  C-word  XSetLocaleModifiers  ( amodifier_list -- astring )
s" XCloseOM"             C-word  XCloseOM             ( nom -- nstatus )
\ s" XSetOMValues"         C-word  XSetOMValues         ( ...om -- astring )
\ s" XGetOMValues"         C-word  XGetOMValues         ( ...om -- astring )
\ s" XDisplayOfOM"         C-word  XDisplayOfOM         ( om -- dpy )
\ s" XLocaleOfOM"          C-word  XLocaleOfOM          ( om -- string )
\ s" XCreateOC"            C-word  XCreateOC            ( ...om -- XOC )
\ s" XDestroyOC"           C-word  XDestroyOC           ( oc -- r )
\ s" XOMOfOC"              C-word  XOMOfOC              ( oc -- XOM )
\ s" XSetOCValues"         C-word  XSetOCValues         ( ...oc -- string )
\ s" XGetOCValues"         C-word  XGetOCValues         ( ...oc -- string )


\ s" XContextualDrawing"   C-word  XContextualDrawing   ( fontset -- flag )



\ s" XSetICFocus"          C-word  XSetICFocus          ( ic -- r )
\ s" XUnsetICFocus"        C-word  XUnsetICFocus        ( ic -- r )
\ s" XwcResetIC"           C-word  XwcResetIC           ( ic -- wstring )
\ s" XmbResetIC"           C-word  XmbResetIC           ( ic -- string )

\ s" XwcFreeStringList"    C-word  XwcFreeStringList    ( list -- r )
\ s" Xpermalloc"           C-word  Xpermalloc           ( size -- addr )
\ s" XrmStringToQuark"     C-word  XrmStringToQuark     ( string -- quark )
\ s" XrmPermStringToQuark" C-word  XrmPermStringToQuark ( string -- quark )
\ s" XrmQuarkToString"     C-word  XrmQuarkToString     ( quark -- string )
\ s" XrmDestroyDatabase"   C-word  XrmDestroyDatabase   ( dbase -- r )
\ s" XrmGetDatabase"       C-word  XrmGetDatabase       ( dpy -- dbase )
\ s" XrmGetFileDatabase"   C-word  XrmGetFileDatabase   ( filename -- dbase )
\ s" XrmGetStringDatabase" C-word  XrmGetStringDatabase ( string -- dbase )
\ s" XrmLocaleOfDatabase"  C-word  XrmLocaleOfDatabase  ( dbase -- string )

\ s" XrmInitialize"        C-word  XrmInitialize        ( -- r )
\ s" XInitThreads"         C-word  XInitThreads         ( -- status )

s" XAllPlanes"           C-word  XAllPlanes           ( -- u )

\ s" XrmUniqueQuark"       C-word  XrmUniqueQuark       ( -- quark )

s" XSupportsLocale"      C-word  XSupportsLocale ( -- nflag )



\ libX11 XVaCreateNestedList_1 [ 4 ] ints (ptr) XVaCreateNestedList     ( ...unused -- \ vanestedlist )
\ libX11 XVaCreateNestedList_2 [ 6 ] ints (ptr) XVaCreateNestedList     ( ...unused -- vanestedlist )
\ libX11 XVaCreateNestedList_3 [ 8 ] ints (ptr) XVaCreateNestedList     ( ...unused -- vanestedlist )
\ libX11 XVaCreateNestedList_4 [ &10 ] ints (ptr) XVaCreateNestedList     ( ...unused -- vanestedlist )



s" XSetClipMask"       C-word  XSetClipMask ( adpy ngc npixmap -- )
\ s" XSetClipOrigin"     C-word  XSetClipOrigin ( dpy gc clip_x_orig clip_y_orig -- )


\ s" XQueryPointer" C-word  XQueryPointer ( dpy w root_r child_r root_x_r root_y_r win_x_r win_y_r mask_r -- flag )
\ s" XLookupString" C-word  XLookupString ( event buffer bufsize keysym_r composestatus* -- n )
\ s" XTextExtents"  C-word  XTextExtents    ( font_struct string nchars direction_r font_ascent_r font_descent_r overall_r -- )
\ s" XTextExtents16" C-word XTextExtents16    ( font_struct string nchars direction_r font_ascent_r font_descent_r overall_r -- )
\ s" XInternAtom"  C-word  XInternAtom     ( dpy atomname flag -- atom )
\ s" XSetTransientForHint"  C-word  XSetTransientForHint    ( dpy w propw -- )

s" XParseGeometry"   C-word  XParseGeometry  ( aparsestring ax_r ay_r \
     aw_r ah_r -- n )

\ Host Access functions

s" XAddHost"  C-word  XAddHost        ( adpy ahost -- n )
s" XAddHosts" C-word  XAddHosts       ( adpy ahosts nhosts -- n )
s" XDisableAccessControl" C-word XDisableAccessControl ( adpy -- n )
s" XEnableAccessControl" C-word  XEnableAccessControl ( adpy -- n )
s" XListHosts" C-word  XListHosts     ( adpy anhosts_r anstate_r -- alist )
\ s" XRemoveHost"    C-word  XRemoveHost     ( host dpy -- n )
\ s" XRemoveHosts"   C-word  XRemoveHosts    ( num_hosts hosts dpy -- n )


\ Client functions
s" XKillClient"     C-word  XKillClient  ( adpy nresource -- n )


\ Housekeeping functions

s" XActivateScreenSaver" C-word  XActivateScreenSaver ( adpy -- n )
s" XOpenDisplay"    C-word  XOpenDisplay    ( aname -- adpy )
s" XCloseDisplay"   C-word  XCloseDisplay   ( adpy -- n )
s" XForceScreenSaver" C-word  XForceScreenSaver  ( adpy nmode -- n )
s" XFree"           C-word  XFree           ( adata -- n )
s" XFreeStringList" C-word  XFreeStringList ( aalist --  )
s" XGetScreenSaver" C-word  XGetScreenSaver ( adpy atimeout_r \
     ainterval_r  aprefer_blanking_r  allow_exposures_r -- n )
s" XLastKnownRequestProcessed"  C-word XLastKnownRequestProcessed  ( adpy -- u )
s" XLockDisplay"    C-word  XLockDisplay     ( adpy --  )
s" XUnlockDisplay"  C-word  XUnlockDisplay   ( adpy --  )

\ Error-handling functions

s" XDisplayName"    C-word  XDisplayName   ( astring -- astring )
s" XGetErrorDatabaseText" C-word  XGetErrorDatabaseText ( adpy \
     aname amessage adefault_string  abuffer_r nlength -- n )  
s" XGetErrorText"   C-word  XGetErrorText  ( adpy ncode abuffer_r \
     nlength -- n )
s" XSetErrorHandler"   C-word  XSetErrorHandler     ( nhandler -- nerrorhandler )
s" XSetIOErrorHandler" C-word  XSetIOErrorHandler   ( nhandler -- nioerrorhandler )

\ Display functions

s" XBitmapBitOrder"  C-word  XBitmapBitOrder   ( adpy -- n )
s" XBitmapPad"       C-word  XBitmapPad        ( adpy -- n )
s" XBitmapUnit"      C-word  XBitmapUnit       ( adpy -- n )
s" XCellsOfScreen"   C-word  XCellsOfScreen    ( ascreen -- n )
s" XDisplayCells"    C-word  XDisplayCells     ( adpy nscreen -- n )
s" XDisplayHeight"   C-word  XDisplayHeight    ( adpy nscreen -- n )
s" XDisplayHeightMM" C-word  XDisplayHeightMM  ( adpy nscreen -- n )
s" XDisplayWidth"    C-word  XDisplayWidth     ( adpy nscreen -- n )
s" XDisplayWidthMM"  C-word  XDisplayWidthMM   ( adpy nscreen -- n )
s" XDisplayKeycodes" C-word  XDisplayKeycodes  ( adpy amin_keycodes_r \
     amax_keycodes_r -- n )
s" XDisplayPlanes"   C-word  XDisplayPlanes    ( adpy nscreen -- n )

s" XRootWindow"       C-word  XRootWindow     ( adpy nscreen -- nwindow )
s" XDefaultVisual"    C-word  XDefaultVisual  ( adpy nscreen -- avisual )
s" XDefaultGC"        C-word  XDefaultGC      ( adpy nscreen -- ngc )
s" XDefaultDepth"     C-word  XDefaultDepth   ( adpy nscreen -- n )
s" XDefaultString"    C-word  XDefaultString  ( -- achar )
s" XBlackPixel"       C-word  XBlackPixel     ( adpy nscreen -- u )
s" XWhitePixel"       C-word  XWhitePixel     ( adpy nscreen -- u )
s" XDefaultColormap"  C-word  XDefaultColormap  ( adpy nscreen -- ncolormap )
s" XDisplayOfScreen"  C-word  XDisplayOfScreen  ( ascreen -- adpy )
s" XScreenOfDisplay"  C-word  XScreenOfDisplay  ( adpy nscreen -- ascreen ) 
s" XGetDefault"    C-word  XGetDefault     ( adpy aprogram aoption -- astring )
s" XHeightMMOfScreen" C-word  XHeightMMOfScreen    ( ascreen -- n )
s" XHeightOfScreen"   C-word  XHeightOfScreen      ( ascreen -- n )
s" XListDepths"       C-word  XListDepths     ( adpy nscreen acount_r -- alist )
s" XProtocolRevision" C-word  XProtocolRevision    ( adpy -- n )
s" XProtocolVersion"  C-word  XProtocolVersion     ( adpy -- n )
s" XWidthMMOfScreen"  C-word  XWidthMMOfScreen     ( ascreen -- n )
s" XWidthOfScreen"    C-word  XWidthOfScreen       ( ascreen -- n )

\ Window functions

s" XChangeWindowAttributes" C-word XChangeWindowAttributes ( adpy nwin \
     uvaluemask  attributes -- n )
s" XConfigureWindow"  C-word  XConfigureWindow  ( adpy nw nvalue_mask  \
     avalues -- n )
s" XCreateSimpleWindow"  C-word  XCreateSimpleWindow   ( adpy nparent \
     nx ny nw nh nbwidth uborder ubg -- nwindow )
s" XCreateWindow"     C-word  XCreateWindow ( adpy nparent nx ny uw uh \
     uborderwidth ndepth uclass avisual uvaluemask attribs -- nwindow )
s" XDestroyWindow"   C-word  XDestroyWindow ( adpy nw -- n )
s" XFetchName"       C-word  XFetchName     ( adpy nw aaname_r -- nstatus )
s" XGeometry"        C-word  XGeometry      ( adpy nscreen apos \
     adefault_pos ubwidth ufwidth ufheight nxadder nyadder \
     ax_r  ay_r  aw_r  ah_r -- n )
s" XGetGeometry"     C-word  XGetGeometry  ( adpy nd awroot_r ax_r ay_r \
     aw_r  ah_r  aborderw_r  adepth_r -- nstatus )
s" XGetWindowAttributes" C-word  XGetWindowAttributes ( adpy nw \
     awindows_attributes_r -- nstatus )
s" XGetWindowProperty" C-word  XGetWindowProperty ( adpy nw nproperty noffset \
           nlength ndelete nreq_type aactual_type_r aactual_format_r \
           anitems_r abytes_after_r aprop_r -- n )
s" XMapRaised"       C-word  XMapRaised     ( adpy nw -- n )
s" XMapWindow"       C-word  XMapWindow     ( adpy nw -- n )
s" XMoveResizeWindow" C-word XMoveResizeWindow ( adpy nwin nx ny uw uh -- n )
s" XMoveWindow"      C-word  XMoveWindow    ( adpy nwin ux uy -- n )
s" XResizeWindow"    C-word  XResizeWindow  ( adpy nwin uw uh -- n )
s" XUnmapWindow"     C-word  XUnmapWindow   ( adpy nw -- )
s" XIconifyWindow"   C-word  XIconifyWindow ( adpy nw nscreen -- nstatus )
s" XWithdrawWindow"  C-word  XWithdrawWindow ( adpy nw nscreen -- nstatus )
s" XRaiseWindow"     C-word  XRaiseWindow   ( adpy nw -- n )
s" XLowerWindow"     C-word  XLowerWindow   ( adpy nw -- n )
\ s" XReconfigureWMWindow" C-word  XReconfigureWMWindow   ( changes mask screen_nr w dpy -- status )
s" XReparentWindow"  C-word  XReparentWindow ( adpy nw nparent nx ny -- n )
s" XSetWindowBackground" C-word XSetWindowBackground ( adpy nw ubg_pixel -- n )
s" XSetWindowBackgroundPixmap" 
C-word XSetWindowBackgroundPixmap ( adpy nw nbg_pixmap -- n )
s" XSetWindowBorder" C-word  XSetWindowBorder ( adpy nw uborder_pixel -- n )
s" XSetWindowBorderPixmap" C-word XSetWindowBorderPixmap ( adpy nw \
                                          nborder_pixmap -- n )
s" XSetWindowBorderWidth"  C-word  XSetWindowBorderWidth ( adpy nw uwidth -- n )
s" XSetWindowColormap"     C-word  XSetWindowColormap    ( adpy nw ncolormap -- n )

s" XCirculateSubwindows" C-word XCirculateSubwindows    ( adpy nw ndirection -- n )
s" XCirculateSubwindowsDown" C-word XCirculateSubwindowsDown  ( adpy nw -- n )
s" XCirculateSubwindowsUp" C-word   XCirculateSubwindowsUp    ( adpy nw -- n )
s" XMapSubwindows"       C-word  XMapSubwindows         ( adpy nw -- n )
s" XUnmapSubwindows"     C-word  XUnmapSubwindows       ( adpy nw -- n )
s" XDestroySubwindows"   C-word  XDestroySubwindows     ( adpy nw -- n )
s" XRestackWindows"      C-word  XRestackWindows   ( adpy awindows nwindows -- n )


\ Graphics context functions

s" XCreateGC"        C-word  XCreateGC  ( adpy nd nvaluemask avalues -- ngc )
s" XChangeGC"        C-word  XChangeGC  ( adpy ngc uvaluemask avalues -- n )
s" XCopyGC"          C-word  XCopyGC    ( adpy nsrc uvaluemask ndest -- n )
s" XFlushGC"         C-word  XFlushGC   ( adpy ngc -- )
s" XFreeGC"          C-word  XFreeGC    ( adpy ngc -- n )
s" XGContextFromGC"  C-word  XGContextFromGC ( ngc -- ngcontext )
s" XGetGCValues"     C-word  XGetGCValues  ( adpy ngc uvaluemask avalues_r -- nstatus )
s" XSetBackground"   C-word  XSetBackground  ( adpy ngc ubackground -- n )
s" XSetForeground"   C-word  XSetForeground  ( adpy ngc ufg -- n )
s" XSetFunction"     C-word  XSetFunction    ( adpy ngc nfunction -- n )
s" XSetGraphicsExposures" C-word XSetGraphicsExposures ( adpy ngc \
     ngraphics_exposures -- n )
s" XSetLineAttributes" C-word  XSetLineAttributes ( adpy ngc uline_width \
     nline_style ncap_style njoin_style -- n )
s" XSetPlaneMask"    C-word  XSetPlaneMask   ( adpy ngc uplane_mask -- n )
s" XSetState"        C-word  XSetState   ( adpy ngc ufg ubg nfunc uplane_mask -- n )


\ Drawing functions

s" XClearArea" C-word  XClearArea    ( adpy nwin nx ny uwidth \
     uheight nexposures -- n )
s" XClearWindow"  C-word  XClearWindow ( adpy nw  -- n )
s" XCopyArea"     C-word  XCopyArea  ( adpy nsrc ndest ngc nx ny uw uh \
     ndx ndy -- n )
s" XCopyPlane"    C-word  XCopyPlane ( adpy nsrc ndest ngc nx ny uw uh \
     ndx ndy uplane -- n )
s" XDrawArc"   C-word  XDrawArc    ( adpy nd ngc nx ny uw uh nangle1 nangle2 -- n )
s" XDrawArcs"  C-word  XDrawArcs   ( adpy nd ngc aarcs narcs -- n )
s" XDrawLine"  C-word  XDrawLine   ( adpy nd ngc nx1 ny1 nx2 ny2 -- n )
s" XDrawLines" C-word  XDrawLines  ( adpy nd ngc apoints npoints nmode -- n )
s" XDrawPoint" C-word  XDrawPoint  ( adpy nd ngc nx ny -- n )
s" XDrawPoints" C-word XDrawPoints ( adpy nd ngc apoints npoint nmode -- n )
s" XDrawRectangle" C-word  XDrawRectangle ( adpy nd ngc nx ny uw uh -- n )
s" XDrawRectangles" C-word XDrawRectangles ( adpy nd ngc arects nrects -- n )
s" XDrawSegments"   C-word XDrawSegments   ( adpy nd ngc asegments nsegments -- n )
s" XDrawString"     C-word XDrawString     ( adpy nd ngc nx ny astring nlength -- n )
s" XDrawString16"   C-word XDrawString16   ( adpy nd ngc nx ny astring nlength -- n )
s" XDrawImageString" C-word XDrawImageString ( adpy nd ngc nx ny astring nlength -- n )
s" XDrawImageString16" C-word  XDrawImageString16 ( adpy nd nx ny astring nlength -- n )
s" XDrawText"   C-word  XDrawText   ( adpy nd ngc nx ny aitems nitems -- n )
s" XDrawText16" C-word  XDrawText16 ( adpy nd ngc nx ny aitems nitems -- n )

s" XFillArc"     C-word  XFillArc   ( adpy nd ngc nx ny uw uh nangle1 nangle2 -- n )
s" XFillArcs"    C-word  XFillArcs  ( adpy nd ngc aarcs narcs -- n )
s" XFillRectangle"  C-word  XFillRectangle  ( adpy nd ngc nx ny uw uh -- n )
s" XFillRectangles" C-word  XFillRectangles ( adpy nd ngc arects nrects -- n )
s" XFillPolygon"    C-word  XFillPolygon    ( adpy nd ngc apoints \
                                          npoints nshape nmode -- n )
s" XSetArcMode"     C-word  XSetArcMode     ( adpy ngc narc_mode -- n )
s" XSetFillRule"    C-word  XSetFillRule    ( adpy ngc nfill_rule -- n )
s" XSetFillStyle"   C-word  XSetFillStyle   ( adpy ngc nfill_style -- n )



\ Pixmap and Tile functions

s" XCreateBitmapFromData"  C-word  XCreateBitmapFromData  ( adpy nd adata \
     nwidth nheight -- npixmap )
s" XCreatePixmap" C-word  XCreatePixmap   ( adpy nd nw nh ndepth -- npixmap )
s" XCreatePixmapCursor" C-word XCreatePixmapCursor  ( adpy nsource nmask \
     afgcol abgcol nx ny -- ncursor )
s" XCreatePixmapFromBitmapData"  
C-word  XCreatePixmapFromBitmapData   ( adpy nd adata uwidth uheight \
     ufg ubg udepth -- npixmap )
s" XFreePixmap"   C-word  XFreePixmap ( adpy npixmap -- n )
s" XListPixmapFormats" C-word  XListPixmapFormats   ( adpy acount_r \
     -- apixmapformatvalues )


\ Image functions

s" XAddPixel"     C-word  XAddPixel     ( aximage nvalue -- n )
s" XCreateImage"  C-word  XCreateImage  ( adpy avisual ndepth nformat noffset \
     adata nw nh nbmpad nb/line -- aimage )
s" XDestroyImage" C-word  XDestroyImage ( aximage -- n )
s" XGetPixel"     C-word  XGetPixel     ( aximage nx ny -- u )
s" XGetImage"     C-word  XGetImage     ( adpy nd nx ny uw uh uplanemask \
     nformat -- aimage )
s" XGetSubImage"  C-word  XGetSubImage  ( adpy nd nx ny uw uh uplanemask \
     nformat axdimage ndx ndy -- aximage )
s" XInitImage"    C-word  XInitImage    ( aimage -- nstatus )
s" XImageByteOrder" C-word  XImageByteOrder ( adpy -- n )
s" XPutImage"     C-word  XPutImage     ( adpy nd ngc aimage \
     ns_x ns_y nd_x nd_y uw uh -- n )
s" XPutPixel"     C-word  XPutPixel     ( aximage nx ny upixel -- n )
s" XSubImage"     C-word  XSubImage     ( aximage nx ny uw uh -- aximage )


\ Region functions

s" XClipBox"        C-word  XClipBox        ( nreg arect_r -- n )
s" XCreateRegion"   C-word  XCreateRegion   ( -- nreg )
s" XDestroyRegion"  C-word  XDestroyRegion  ( nreg -- n )
s" XEmptyRegion"    C-word  XEmptyRegion    ( nreg -- n )
s" XEqualRegion"    C-word  XEqualRegion    ( nreg1 nreg2 -- n )
s" XOffsetRegion"   C-word  XOffsetRegion   ( nreg ndx ndy -- n )
s" XPointInRegion"  C-word  XPointInRegion  ( nreg nx ny -- nflag )
s" XPolygonRegion"  C-word  XPolygonRegion  ( apoints n nfill_rule -- nreg )
s" XRectInRegion"   C-word  XRectInRegion   ( nreg nx ny uw uh -- n )
s" XSetRegion"      C-word  XSetRegion      ( adpy ngc nreg -- n )
s" XSubtractRegion" C-word  XSubtractRegion ( nsra nsrb ndr_r -- n )
s" XUnionRegion"    C-word  XUnionRegion    ( nsra nsrb ndr_r -- n )
s" XShrinkRegion"   C-word  XShrinkRegion   ( nreg ndx ndy -- n )
s" XXorRegion"      C-word  XXorRegion      ( nsra nsrb ndr_r -- n )
s" XIntersectRegion" C-word XIntersectRegion  ( nsra nsrb ndr_r -- n )
s" XUnionRectWithRegion" C-word XUnionRectWithRegion ( arect nsrc_reg ndest_reg_r -- n )


\ Device-independent Color functions

s" XcmsAddColorSpace"  C-word  XcmsAddColorSpace ( acolor_space -- nstatus )
s" XcmsAddFunctionSet" C-word  XcmsAddFunctionSet ( afunction_set -- nstatus )
s" XcmsAllocColor"     C-word  XcmsAllocColor  ( adpy ncolormap acolor_in_out \
     nresult_format -- nstatus )
s" XcmsAllocNamedColor" C-word XcmsAllocNamedColor ( adpy ncolormap \
     acolor_string acolor_screen_r acolor_exact_r nresult_format -- nstatus )

\ Colormap functions
s" XAllocColor"       C-word  XAllocColor   ( adpy ncolormap acolor -- nstatus )
s" XAllocColorCells"  C-word  XAllocColorCells  ( adpy ncolormap ncontig \
   aplanemasks_r nplanes apixels_r npixels -- nstatus ) 
s" XAllocColorPlanes" C-word  XAllocColorPlanes ( adpy ncolormap ncontig \
   apixels_r npixels nreds ngreens nblues armask_r agmask_r abmask_r -- nstatus )
s" XAllocNamedColor"  C-word  XAllocNamedColor  ( adpy ncolormap acolor_name \
   ascreendef_r aexactdef_r -- nstatus )
s" XCopyColormapAndFree" C-word  XCopyColormapAndFree ( adpy ncolormap1 -- ncolormap2 )
s" XCreateColormap"      C-word  XCreateColormap ( adpy nw avisual \
     nalloc -- ncolormap )
s" XFreeColormap"    C-word  XFreeColormap ( adpy ncolormap -- n )
s" XFreeColors"      C-word  XFreeColors  ( adpy ncolormap apixels \
     npixels nplanes -- n )
s" XInstallColormap" C-word  XInstallColormap  ( adpy ncolormap -- n )
s" XListInstalledColormaps"  C-word  XListInstalledColormaps ( adpy nw \
     anum_r -- alist )
s" XLookupColor"     C-word  XLookupColor ( adpy ncolormap acolor_name \
     aexact_def_r ascreen_def_r -- nstatus )
s" XParseColor"      C-word  XParseColor  ( adpy ncolormap aspec \
     aexact_def_r -- nstatus )
s" XGetStandardColormap" C-word  XGetStandardColormap  ( adpy nw acolormap_r \
     nproperty -- nstatus )
s" XSetStandardColormap" C-word  XSetStandardColormap  ( adpy nw acolormap \
     nproperty -- )


\ Font functions

s" XFreeFont"         C-word  XFreeFont       ( adpy afont_struct -- n )
s" XFreeFontInfo"     C-word  XFreeFontInfo   ( aanames afree_info nactual_count -- n )
s" XFreeFontNames"    C-word  XFreeFontNames  ( aalist -- n )
s" XFreeFontPath"     C-word  XFreeFontPath   ( aalist -- n )
s" XGetFontPath"      C-word  XGetFontPath    ( adpy anpaths_r -- aastring )
s" XGetFontProperty"  C-word  XGetFontProperty ( afontstruct natom avalue_r -- nflag )
s" XListFonts"        C-word  XListFonts      ( adpy apattern nmaxnames \
     actual_count_r -- aalist )
s" XListFontsWithInfo" C-word XListFontsWithInfo ( adpy apattern nmaxnames \
     acount_r aainfo_r -- aalist )
s" XLoadFont"         C-word  XLoadFont       ( adpy aname -- nfont )
s" XLoadQueryFont"    C-word  XLoadQueryFont  ( adpy aname -- afontstruct )
s" XSetFont"          C-word  XSetFont        ( adpy ngc nfont -- n )
s" XSetFontPath"      C-word  XSetFontPath    ( adpy aadirectories ndirs -- n )
s" XUnloadFont"       C-word  XUnloadFont     ( adpy nfont -- n )
s" XTextWidth"        C-word  XTextWidth      ( afont_struct astring ncount -- n )
s" XTextWidth16"      C-word  XTextWidth16    ( afont_struct astring ncount -- n )


\ Text functions

s" XBaseFontNameListOfFontSet" 
  C-word XBaseFontNameListOfFontSet ( nfontset -- astring )
s" XContextDependentDrawing"   
  C-word XContextDependentDrawing  ( nfontset -- nflag )
s" XCreateFontSet"    C-word  XCreateFontSet ( adpy abase_font_name_list \
     aamissing_charset_list amissing-charset_count aadef_string -- nfontset )
s" XExtentsOfFontSet" C-word  XExtentsOfFontSet  ( nfontset -- afontsetextents )
s" XFontsOfFontSet"   C-word  XFontsOfFontSet ( nfont_set \
     aafont_struct_list_r  aafont_name_list_r -- n )
s" XFreeFontSet"      C-word  XFreeFontSet    ( adpy nfontset -- n )
s" XLocaleOfFontSet"  C-word  XLocaleOfFontSet  ( nfontset -- astring )
\ s" XSetTextProperty" C-word XSetTextProperty  ( dpy w text_prop prop -- )

\ s" XDirectionalDependentDrawing" C-word XDirectionalDependentDrawing ( fontset -- flag )
s" XStringToKeysym"      C-word  XStringToKeysym      ( astring -- nkeysym )

\ Grabbing functions
s" XGrabButton"   C-word  XGrabButton  ( adpy nbutton nmodifiers ngrab_win \
     nowner_events uevent_mask npointer_mode nkeyboard_mode nconfine_to ncursor -- n )
s" XGrabKey"      C-word  XGrabKey     ( adpy nkeycode umodifiers ngrab_win \
     nowner_events npointer_mode nkeyboard_mode -- n )
s" XGrabKeyboard" C-word  XGrabKeyboard ( adpy nw nowner_events npointer_mode \
     nkeyboard_mode utime -- n )
s" XGrabPointer"  C-word  XGrabPointer  ( adpy ngrab_win nowner_events \
     uevent_mask npointer_mode nkeyboard_mode nconfine_to ncursor utime -- n )
s" XGrabServer"   C-word  XGrabServer    ( adpy -- n )
\ s" XUngrabButton"   C-word  XUngrabButton  ( grab_window modifiers button dpy -- n )
\ s" XUngrabKey"      C-word  XUngrabKey   ( grab_window modifiers keycode dpy -- n )
\ s" XUngrabKeyboard" C-word  XUngrabKeyboard ( time dpy -- n )
s" XUngrabPointer"    C-word  XUngrabPointer  ( adpy ntime -- n )
s" XUngrabServer"     C-word  XUngrabServer   ( adpy -- n )

\ Input Context functions

s" XCreateIC"     C-word  XCreateIC       ( nim -- nXIC )
s" XDestroyIC"    C-word  XDestroyIC      ( nic -- n )
\ libX11 XCreateIC_1 [ 4 ] ints (ptr) XCreateIC       ( im -- XIC )
\ libX11 XCreateIC_2 [ 6 ] ints (ptr) XCreateIC       ( im -- XIC )
\ libX11 XCreateIC_3 [ 8 ] ints (ptr) XCreateIC       ( im -- XIC )
\ libX11 XCreateIC_4 [ &10 ] ints (ptr) XCreateIC       ( im -- XIC )
\ libX11 XCreateIC_5 [ &12 ] ints (ptr) XCreateIC       ( im -- XIC )

\ Input Method functions

s" XCloseIM"      C-word  XCloseIM        ( nim -- nstatus )
s" XDisplayOfIM"  C-word  XDisplayOfIM    ( nim -- adpy )
s" XIMOfIC"       C-word  XIMOfIC         ( nic -- nim )
s" XLocaleOfIM"   C-word  XLocaleOfIM     ( nim -- astring )
s" XOpenIM"       C-word  XOpenIM  ( adpy ardb ares_name ares_class -- nim )
\ libX11 XGetIMValues_1 [ 4 ] ints (ptr) XGetIMValues    ( im ... -- string )
\ libX11 XGetIMValues_2 [ 6 ] ints (ptr) XGetIMValues    ( im ... -- string )
\ libX11 XGetIMValues_3 [ 8 ] ints (ptr) XGetIMValues  ( im ... -- string )
\ libX11 XSetIMValues_1 [ 4 ] ints (ptr) XSetIMValues    ( im ... -- string )
\ libX11 XSetIMValues_2 [ 6 ] ints (ptr) XSetIMValues    ( im ... -- string )
\ libX11 XSetIMValues_3 [ 8 ] ints (ptr) XSetIMValues    ( im ... -- string )


\ Input Handling functions

s" XAllowEvents"   C-word  XAllowEvents    ( adpy nevent_mode ntime -- n )
s" XFilterEvent"   C-word  XFilterEvent    ( axevent nw -- nflag )
s" XGetInputFocus" C-word  XGetInputFocus  ( adpy afocus_ret arevert -- n )
s" XSetInputFocus" C-word  XSetInputFocus  ( adpy nfocus nrevert_to ntime -- n )

\ Pointer functions

s" XChangeActivePointerGrab" C-word  XChangeActivePointerGrab  ( adpy \
     nevent_mask ncursor utime -- n )
s" XChangePointerControl"  C-word  XChangePointerControl ( adpy \
     ndo_accel ndo_threshold naccel_numerator naccel_denom nthreshold -- n )
s" XGetPointerControl"   C-word  XGetPointerControl   ( adpy \
     aaccel_num_r  aaccel_denom_r athreshold_r -- n )
s" XGetPointerMapping"   C-word  XGetPointerMapping   ( adpy amap_r nmap -- n )

\ Keyboard

s" XKeycodeToKeysym"  C-word  XKeycodeToKeysym  ( adpy nkeycode nindex \
     -- nkeysym )
s" XKeysymToKeycode"  C-word  XKeysymToKeycode ( adpy nkeysym -- nkeycode )
s" XKeysymToString"   C-word  XKeysymToString  ( nkeysym -- astring )
s" XLookupKeysym"  C-word  XLookupKeysym   ( akey_event nindex -- nkeysym )
s" XGetKeyboardMapping" C-word  XGetKeyboardMapping  ( adpy nkeycode \
     nkeycode_c akeysyms_r -- akeysym )
s" XGetKeyboardControl"  C-word  XGetKeyboardControl  ( adpy avalues_r -- n )
s" XChangeKeyboardMapping" C-word XChangeKeyboardMapping  ( adpy \
   nfirst_keycode nkeysyms_per_keycode akeysyms nkeycodes -- n )
s" XFreeModifiermap"     C-word  XFreeModifiermap     ( amodmap -- n )
s" XGetModifierMapping"  C-word  XGetModifierMapping  ( adpy -- amodmap )
s" XNewModifiermap"      C-word  XNewModifiermap      ( nmaxkeys -- amodmap )


\ Cursor functions

s" XCreateFontCursor"  C-word  XCreateFontCursor  ( adpy nshape -- ncursor )
s" XCreateGlyphCursor" C-word  XCreateGlyphCursor ( adpy nsource_font \
     nmask_font nsource_char nmask_char afgcol abgcol -- ncursor )
s" XDefineCursor"      C-word  XDefineCursor  ( adpy nwin ncursor -- n )
s" XFreeCursor"        C-word  XFreeCursor    ( adpy ncursor -- n )


\ Icon functions

s" XGetIconName"    C-word  XGetIconName   ( adpy nw aaicon_name_r -- nstatus )
\ s" XGetIconSizes"   C-word  XGetIconSizes   ( count_r size_list_r w dpy -- status )
\ s" XSetIconName" C-word  XSetIconName    ( adpy nw aicon_name -- )


\ s" XStoreName"   C-word  XStoreName      ( dpy win window_name -- )

\ s" XWarpPointer" C-word  XWarpPointer    ( dpy src_w dest_w src_x src_y src_w src_h dest_x dest_y -- )

s" XSelectInput"   C-word  XSelectInput    ( adpy nw nevent_mask -- n )


\ Event functions

s" XDisplayMotionBufferSize" C-word XDisplayMotionBufferSize ( adpy -- u )
s" XNextEvent"      C-word  XNextEvent  ( adpy aevent_r -- n )
s" XMaskEvent"   C-word  XMaskEvent     ( adpy nevent_mask aevent_r -- n )
s" XCheckMaskEvent" C-word  XCheckMaskEvent ( adpy nevent_mask aevent_r -- nflag )
s" XWindowEvent"  C-word XWindowEvent   ( adpy nw nevent_mask aevent_r -- n )
s" XCheckWindowEvent" C-word XCheckWindowEvent ( adpy nw nevent_mask aevent_r -- nflag )
s" XPeekEvent"   C-word  XPeekEvent     ( adpy aevent_r -- n )
s" XCheckTypedEvent" C-word XCheckTypedEvent ( adpy nevent_type aevent_r -- nflag )
s" XCheckTypedWindowEvent" C-word XCheckTypedWindowEvent ( adpy nw nevent_type \
                                                    aevent_r  -- nflag )
s" XEventsQueued"   C-word  XEventsQueued  ( adpy nmode -- n )
s" XPending"        C-word  XPending       ( adpy -- n )
s" XPutBackEvent"   C-word  XPutBackEvent  ( adpy aevent -- n )
s" XGetMotionEvents" C-word XGetMotionEvents  ( adpy nw nstart nstop \
                                              anevents_r -- atimecoord )
s" XQLength"        C-word  XQLength       ( adpy -- n )

s" XSendEvent"    C-word XSendEvent     ( adpy nw npropagate \
                             nevent_mask aevent_send -- nstatus )
s" XIfEvent"     C-word  XIfEvent       ( adpy aevent_r apredicate aarg -- nflag )
\ s" XCheckIfEvent" C-word XCheckIfEvent  ( arg predicate event_return dpy -- flag )
\ s" XPeekIfEvent" C-word  XPeekIfEvent   ( arg predicate event_r dpy -- r )


\ Selection functions

\ s" XSetSelectionOwner" C-word  XSetSelectionOwner ( adpy selection owner time -- r )

s" XConvertSelection"  C-word  XConvertSelection  ( adpy nselection ntarget \
     nproperty nrequestor utime -- n )
s" XGetSelectionOwner" C-word  XGetSelectionOwner ( adpy natom -- nwindow )

\ Cut-Buffer functions

s" XFetchBuffer"  C-word  XFetchBuffer  ( adpy anbytes_r nbuffer -- axfbuffer )
s" XFetchBytes"   C-word  XFetchBytes   ( adpy anbytesr -- abuffer )
s" XStoreBytes"   C-word  XStoreBytes   ( adpy abytes nbytes -- n )


\ Property functions

s" XChangeProperty"  C-word  XChangeProperty    ( adpy nw nproperty ntype \
     nformat nmode adata nelements -- n )
s" XDeleteProperty"  C-word XDeleteProperty ( adpy nw natom -- n )
s" XGetAtomName"     C-word  XGetAtomName   ( adpy natom -- aname )
\ s" XGetAtomNames" C-word  XGetAtomNames   ( namesr count atoms dpy -- status )
s" XInternAtom"     C-word  XInternAtom    ( adpy aproperty_name \
     nonly_if_exists -- n )
s" XListProperties" C-word  XListProperties ( adpy nw anum_prop_r -- aatom )


\ Save Set functions

s" XAddToSaveSet"   C-word  XAddToSaveSet   ( adpy nw -- n )
s" XChangeSaveSet"  C-word  XChangeSaveSet  ( adpy nw nchange_mode -- n )

\ User Preferences functions

s" XAutoRepeatOff"       C-word  XAutoRepeatOff       ( adpy -- n )
s" XAutoRepeatOn"        C-word  XAutoRepeatOn        ( adpy -- n )
s" XBell"                C-word  XBell   ( adpy npercent -- n )
s" XChangeKeyboardControl" C-word XChangeKeyboardControl  ( adpy \
  uvalue_mask avalues -- n )

\ Window Manager Hints functions

s" XAllocClassHint"   C-word  XAllocClassHint      ( -- axclasshint )
s" XAllocIconSize"    C-word  XAllocIconSize       ( -- axiconsize )
s" XAllocSizeHints"   C-word  XAllocSizeHints      ( -- axsizehints )
s" XAllocStandardColormap" C-word XAllocStandardColormap  ( -- axstdcmap )
s" XAllocWMHints"     C-word  XAllocWMHints        ( -- axwmhints )
s" XGetClassHint"     C-word  XGetClassHint  ( adpy nw \
     aclass_hints_r -- nstatus )
s" XGetCommand"       C-word  XGetCommand    ( adpy nw \
     aargv_r argc_r -- nstatus )
s" XGetNormalHints"   C-word  XGetNormalHints ( adpy nw ahints_r -- nstatus )
s" XGetRGBColormaps"  C-word  XGetRGBColormaps ( adpy nw \
     aastdcmap_r acount_r nprop   -- nstatus )
s" XGetSizeHints"     C-word  XGetSizeHints   ( adpy nw ahints_r nprop -- nstatus )
s" XGetTextProperty"  C-word  XGetTextProperty ( adpy nw \
     atext_prop_r nprop -- nstatus )
s" XGetTransientForHint" C-word  XGetTransientForHint ( adpy nw \
     aprop_w_r -- nstatus )
s" XGetVisualInfo"       C-word  XGetVisualInfo  ( adpy nvinfo_mask \
     avinfo_templ anitems_r -- axvinfo )
s" XGetWMClientMachine"  C-word  XGetWMClientMachine ( adpy nw \
     atext_prop_r -- nstatus )
s" XGetWMColormapWindows" C-word XGetWMColormapWindows ( adpy nw \
     awindows_r ncount_r -- nstatus )
s" XGetWMHints"       C-word  XGetWMHints  ( adpy nw -- aWMHints )
s" XGetWMIconName"    C-word  XGetWMIconName ( adpy nw atext_prop_r -- nstatus )
s" XGetWMName"        C-word  XGetWMName   ( adpy nw atext_prop_r -- nstatus )
s" XGetWMNormalHints" C-word  XGetWMNormalHints  ( adpy nw \
     ahints_r asupplied_r -- nstatus )
s" XGetWMProtocols"   C-word  XGetWMProtocols ( adpy nw \
     aaprotocols_r acount_r -- nstatus )
s" XGetWMSizeHints"   C-word  XGetWMSizeHints ( adpy nw \
     ahints_r asupplied_r nprop -- nstatus )
s" XGetZoomHints"     C-word  XGetZoomHints   ( adpy nw azhints_r -- \
     nstatus )

\ s" XSetRGBColormaps" C-word XSetRGBColormaps ( prop count stdcmaps w dpy -- r )

\ s" XSetNormalHints" C-word  XSetNormalHints ( hints w dpy -- r )
\ s" XSetZoomHints"   C-word  XSetZoomHints   ( zhints w dpy -- r )
\ s" XSetSizeHints"   C-word  XSetSizeHints   ( prop hints w dpy -- r )
\ s" XSetWMHints"     C-word  XSetWMHints     ( dpy w wm_hints -- )
\ s" XSetWMNormalHints" C-word XSetWMNormalHints  ( dpy w hints -- )
\ s" XSetClassHint"   C-word  XSetClassHint   ( dpy w class_hints -- )

\ s" XSetWMProtocols" C-word  XSetWMProtocols ( count protocols w dpy -- status )
\ s" XSetWMColormapWindows" C-word XSetWMColormapWindows ( count colormap_windows w dpy -- status )

\ s" XSetWMClientMachine" C-word XSetWMClientMachine  ( text_prop w dpy -- r )
\ s" XSetWMIconName"    C-word  XSetWMIconName  ( text_prop w dpy -- r )
\ s" XSetWMName"        C-word  XSetWMName      ( text_prop w dpy -- r )
\ s" XSetWMProperties"  C-word  XSetWMProperties ( class_hints wm_hints normal_hints argc argv icon_name window_name w dpy -- r )

\ Resource Manager functions

s" XDeleteModifiermapEntry" C-word  XDeleteModifiermapEntry ( amodmap \
     nkeycode nmodifier -- amodmap )
s" XInsertModifiermapEntry" C-word  XInsertModifiermapEntry ( amodmap \
     nkeycode nmodifier -- amodmap )

\ X Extensions functions

s" XListExtensions"   C-word  XListExtensions ( adpy anextensions_r -- aalist )
\ s" XInitExtension"  C-word  XInitExtension  ( name dpy -- extcodes )
\ s" XFindOnExtensionList" C-word XFindOnExtensionList ( number structure -- extdata )
\ s" XAddToExtensionList"  C-word XAddToExtensionList  ( extdata structure -- r )



\ libX11 XSetICValues_1 [ 4 ] ints (int) XSetICValues    ( ic ... -- string )
\ libX11 XSetICValues_2 [ 6 ] ints (int) XSetICValues    ( ic ... -- string )
\ libX11 XSetICValues_3 [ 8 ] ints (int) XSetICValues    ( ic ... -- string )


\ Unused stuff, machine converted
\ -1 libX11 _XPrivDisplay               _XPrivDisplay   ( -- addr )
\ -1 libX11 _Xdebug                       _Xdebug         ( -- addr )

\ libX11 XQueryFont [ 2 ] ints (int) XQueryFont      ( XID dpy -- fontstruct )
\ libX11 XQueryBestCursor [ 6 ] ints (int) XQueryBestCursor        ( h_r w_r h w d dpy -- status )
\ libX11 XQueryBestSize [ 7 ] ints (int) XQueryBestSize  ( h_r w_r h w which_screen class dpy -- status )
\ libX11 XQueryBestStipple [ 6 ] ints (int) XQueryBestStipple       ( h_r w_r h w which_screen dpy -- status )
\ libX11 XQueryBestTile [ 6 ] ints (int) XQueryBestTile  ( h_r w_r h w which_screen dpy -- status )
\ libX11 XQueryColor [ 3 ] ints (int) XQueryColor     ( def_in_out colormap dpy -- r )
\ libX11 XQueryColors [ 4 ] ints (int) XQueryColors    ( ncolors defs_in_out colormap dpy -- r )
\ libX11 XQueryExtension [ 5 ] ints (int) XQueryExtension ( first_error_r first_event_r major_opcode_r name dpy -- flag )
\ libX11 XQueryKeymap [ 2 ] ints (int) XQueryKeymap    ( keys_r dpy -- r )
\ libX11 XQueryTextExtents [ 8 ] ints (int) XQueryTextExtents       ( overall_r font_descent_r font_ascent_r direction_r nchars string font_id dpy -- r )
\ libX11 XQueryTextExtents16 [ 8 ] ints (int) XQueryTextExtents16     ( overall_r font_descent_r font_ascent_r direction_r nchars string font_id dpy -- r )
\ libX11 XQueryTree [ 6 ] ints (int) XQueryTree      ( nchildren_r children_r parent_r root_r w dpy -- status )


0 ( has-utf8) [IF]
\ libX11 Xutf8DrawImageString [ 8 ] ints (int) Xutf8DrawImageString        ( dpy d fontset gc x y string length -- r )
\ libX11 Xutf8DrawString [ 8 ] ints (int) Xutf8DrawString     ( dpy d fontset gc x y string length -- r )
\ libX11 Xutf8LookupString [ 6 ] ints (int) Xutf8LookupString ( ic event buffer_r wchars_buffer keysym_r status_r -- n )
\ libX11 Xutf8TextExtents [ 7 ] ints (void) Xutf8TextExtents    ( font_struct string nchars direction_r font_ascent_r font_descent_r overall_r -- )
\ libX11 Xutf8DrawText [ 7 ] ints (void) Xutf8DrawText       ( dpy d gc x y items nitems -- )
\ libX11 Xutf8TextEscapement [ 3 ] ints (int) Xutf8TextEscapement       ( num_wchars text fontset -- n )
\ libX11 Xutf8TextPerCharExtents [ 9 ] ints (int) Xutf8TextPerCharExtents   ( overal_logical_r overall_ink_r num_chars buffer_size logical_extents_buffer ink_extents_buffer num_wchars text fontset -- status )
\ libX11 Xutf8ResetIC int (int) Xutf8ResetIC      ( ic -- string )
[THEN]





\ s" XReadBitmapFile" C-word  XReadBitmapFile ( y_hot_r x_hot_r bitmap_r height_r width_r filename d dpy -- n )
\ s" XReadBitmapFileData" C-word  XReadBitmapFileData ( y_hot_r x_hot_r bitmap_r height_r width_r filename -- n )
\ s" XWriteBitmapFile"   C-word  XWriteBitmapFile     ( y_hot x_hot h w bitmap filename dpy -- n )
\ s" XRebindKeysym"  C-word  XRebindKeysym ( bytes_string string mod_count list keysym dpy -- n )
\ s" XRecolorCursor" C-word  XRecolorCursor  ( bg fg cursor dpy -- n )
\ s" XRemoveFromSaveSet" C-word  XRemoveFromSaveSet   ( w dpy -- n )

\ s" XRotateBuffers"  C-word  XRotateBuffers  ( rotate dpy -- n )
\ s" XRotateWindowProperties" 
\ C-word XRotateWindowProperties ( npositions num_prop properties w dpy -- n )
\ s" XSetAccessControl" C-word  XSetAccessControl    ( mode dpy -- n )

\ s" XSetClipRectangles" C-word XSetClipRectangles   ( ordering n rectangles clip_y_orig clip_x_orig gc dpy -- n )
\ s" XSetCloseDownMode" C-word  XSetCloseDownMode    ( close_mode dpy -- n )
\ s" XSetCommand"       C-word  XSetCommand          ( argc argv w dpy -- n )
\ s" XSetDashes"        C-word  XSetDashes    ( n dash_list dash_ofset gc dpy -- n )

\ s" XSetModifierMapping"  C-word  XSetModifierMapping     ( modmap dpy -- n )

\ s" XSetPointerMapping"   C-word  XSetPointerMapping     ( nmap map dpy -- n )
\ s" XSetScreenSaver"      C-word  XSetScreenSaver ( allow_exposures prefer_blanking interval timeout dpy -- n )

\ s" XSetStipple"          C-word  XSetStipple   ( stripple gc dpy -- n )
\ s" XSetSubwindowMode"    C-word  XSetSubwindowMode ( subwindow_mode gc dpy -- n )
\ s" XSetTSOrigin"         C-word  XSetTSOrigin  ( ts_y_orig ts_x_orig gc dpy -- n )
\ s" XSetTile"             C-word  XSetTile        ( tile gc dpy -- )


\ s" XStoreBuffer"       C-word  XStoreBuffer   ( buffer nbytes bytes dpy -- n )
\ s" XStoreColor"        C-word  XStoreColor    ( color colormap dpy -- n )
\ s" XStoreColors"       C-word  XStoreColors   ( ncolors color colormap dpy -- n )
\ s" XStoreNamedColor"   C-word  XStoreNamedColor  ( flags pixel color colormap dpy -- n )

\ s" XTranslateCoordinates" C-word  XTranslateCoordinates  ( child_r dest_y_r dest_x_r src_y src_x dest_w src_w dpy -- flag )
\ s" XUndefineCursor"    C-word  XUndefineCursor ( w dpy -- n )

\ s" XUninstallColormap" C-word  XUninstallColormap   ( colormap dpy -- n )

\ s" XOpenOM"         C-word  XOpenOM ( res_class res_name rdb dpy -- XOM )

\ s" XmbTextEscapement" C-word  XmbTextEscapement  ( bytes_text text fontset -- n )
\ s" XwcTextEscapement" C-word  XwcTextEscapement  ( num_wchars text fontset -- n )
\ s" XmbTextExtents"    C-word  XmbTextExtents  ( overall_logical_r overall_ink_r bytes_text text fontset -- n )
\ s" XwcTextExtents"    C-word  XwcTextExtents  ( overall_logical_r overall_ink_r num_wchars text fontset -- n )

\ libX11 XmbTextPerCharExtents [ 9 ] ints (int) XmbTextPerCharExtents   ( overal_logical_r overall_ink_r num_chars buffer_size logical_extents_buffer  ink_extents_buffer bytes_text text fontset -- status )
\ libX11 XwcTextPerCharExtents [ 9 ] ints (int) XwcTextPerCharExtents   ( overal_logical_r overall_ink_r num_chars buffer_size logical_extents_buffer ink_extents_buffer num_wchars text fontset -- status )
\ libX11 XmbDrawText [ 7 ] ints (int) XmbDrawText     ( nitems text_items y x gc d dpy -- r )
\ libX11 XwcDrawText [ 7 ] ints (int) XwcDrawText     ( nitems text_items y x gc d dpy -- r )
\ libX11 XmbDrawString [ 8 ] ints (int) XmbDrawString   ( bytes_text text y x gc fontset d dpy -- r )
\ libX11 XwcDrawString [ 8 ] ints (int) XwcDrawString   ( num_wchars text y x gc fontset d dpy -- r )
\ libX11 XmbDrawImageString [ 8 ] ints (int) XmbDrawImageString      ( bytes_text text y x gc fontset d dpy -- r )
\ libX11 XwcDrawImageString [ 8 ] ints (int) XwcDrawImageString      ( num_wchars text y x gc fontset d dpy -- r )
\ libX11 XGetICValues_1 [ 4 ] ints (int) XGetICValues    ( ... ic -- string )
\ libX11 XGetICValues_2 [ 6 ] ints (int) XGetICValues    ( ... ic -- string )
\ libX11 XGetICValues_3 [ 8 ] ints (int) XGetICValues    ( ... ic -- string )
\ libX11 XmbLookupString [ 6 ] ints (int) XmbLookupString ( status_r keysym_r bytes_buffer buffer_r event ic -- n )
\ libX11 XwcLookupString [ 6 ] ints (int) XwcLookupString ( status_r keysym_r wchars_buffer buffer_r event ic -- n )
\ libX11 XRegisterIMInstantiateCallback [ 6 ] ints (int) XRegisterIMInstantiateCallback  ( client_data callback res_class res_name rdb dpy -- flag )
\ libX11 XUnregisterIMInstantiateCallbac [ 6 ] ints (int) XUnregisterIMInstantiateCallback        ( client_data callback res_class res_name rdb dpy -- flag )
\ libX11 XInternalConnectionNumbers [ 3 ] ints (int) XInternalConnectionNumbers      ( count_r fd_r dpy -- status )
\ libX11 XProcessInternalConnection [ 2 ] ints (int) XProcessInternalConnection      ( fd dpy -- r )
\ libX11 XAddConnectionWatch [ 3 ] ints (int) XAddConnectionWatch     ( client_data callback dpy -- status )
\ libX11 XRemoveConnectionWatch [ 3 ] ints (int) XRemoveConnectionWatch  ( client_data callback dpy -- status )

\ Xutil.h
\ The following functions are not performed by the XServer, but in Xlib.

\ s" XMatchVisualInfo" C-word XMatchVisualInfo  ( vinfo_r class depth screen dpy -- status )
\ s" XSetStandardProperties" C-word  XSetStandardProperties  ( sizehints argc argv icon_pixmap icon_name window_name w dpy -- r )
\ s" XSetIconSizes"   C-word  XSetIconSizes   ( count size_list w dpy -- r )
\ s" XConvertCase"    C-word  XConvertCase    ( upper* lower* sym -- r )


s" XDeleteContext"  C-word  XDeleteContext  ( adpy nrid ncontext -- n )
s" XFindContext"    C-word  XFindContext    ( adpy nrid ncontext adata_r -- n )
\ s" XSaveContext"    C-word  XSaveContext    ( data context rid dpy -- n )


\ libX11 XmbSetWMProperties [ 9 ] ints (int) XmbSetWMProperties      ( class_hints wm_hints normal_hints argc argv icon_name window_name w dpy -- r )
\ libX11 XSetWMSizeHints [ 4 ] ints (int) XSetWMSizeHints ( prop hints w dpy -- r )


\ libX11 XStringListToTextProperty [ 3 ] ints (int) XStringListToTextProperty       ( text_prop_r count list -- status )
\ libX11 XmbTextListToTextProperty [ 5 ] ints (int) XmbTextListToTextProperty       ( text_prop_r style count list dpy -- n )
\ libX11 XwcTextListToTextProperty [ 5 ] ints (int) XwcTextListToTextProperty       ( text_prop_r style count list dpy -- n )
\ libX11 XTextPropertyToStringList [ 3 ] ints (int) XTextPropertyToStringList       ( count_r list_r text_prop -- status )
\ libX11 XmbTextPropertyToTextList [ 4 ] ints (int) XmbTextPropertyToTextList       ( count_r list_r text_prop dpy -- n )
\ libX11 XwcTextPropertyToTextList [ 4 ] ints (int) XwcTextPropertyToTextList       ( count_r list_r text_prop dpy -- n )

\ libX11 XWMGeometry [ &10 ] ints (int) XWMGeometry     ( gravity_r h_r w_r y_r x_r hints border_w default_geometry user_geometry screen_nr dpy -- n )

\ **************************************************************

\ Memory Management

\ ****************************************************************

\ libX11 XrmStringToQuarkList [ 2 ] ints (int) XrmStringToQuarkList    ( quarks string -- r )
\ libX11 XrmStringToBindingQuarkList [ 3 ] ints (int) XrmStringToBindingQuarkList     ( quarks bindings string -- r )
(
struct{
    cell size
    cell addr
} XrmValue
)
\ libX11 XrmQPutResource [ 5 ] ints (int) XrmQPutResource ( value type quarks bindings dbase -- r )
\ libX11 XrmPutResource [ 4 ] ints (int) XrmPutResource  ( value type spec dbase -- r )
\ libX11 XrmQPutStringResource [ 4 ] ints (int) XrmQPutStringResource   ( value quarks bindings dbase -- r )
\ libX11 XrmPutStringResource [ 3 ] ints (int) XrmPutStringResource    ( value spec dbase -- r )
\ libX11 XrmPutLineResource [ 2 ] ints (int) XrmPutLineResource      ( line dbase -- r )
\ libX11 XrmQGetResource [ 5 ] ints (int) XrmQGetResource ( value_r quark_t_r quark_c quark_n dbase -- r )
\ libX11 XrmGetResource [ 5 ] ints (int) XrmGetResource  ( value_r str_t_r str_c str_n dbase -- flag )
\ libX11 XrmQGetSearchList [ 5 ] ints (int) XrmQGetSearchList       ( list_len list_r classes names dbase -- flag )
\ libX11 XrmQGetSearchResource [ 5 ] ints (int) XrmQGetSearchResource   ( value_r type_r class name list -- flag )
\ libX11 XrmSetDatabase [ 2 ] ints (int) XrmSetDatabase  ( dbase dpy -- r )
\ libX11 XrmCombineFileDatabase [ 3 ] ints (int) XrmCombineFileDatabase  ( flag dbase* filename -- status )
\ libX11 XrmPutFileDatabase [ 2 ] ints (int) XrmPutFileDatabase      ( string dbase -- r )
\ libX11 XrmMergeDatabases [ 2 ] ints (int) XrmMergeDatabases       ( dbase* dbase -- r )
\ libX11 XrmCombineDatabase [ 3 ] ints (int) XrmCombineDatabase      ( flag dbase* dbase )
\ libX11 XrmEnumerateDatabase [ 6 ] ints (int) XrmEnumerateDatabase    ( closure callback mode class_prefix name_prefix db -- flag )
\ libX11 XrmParseCommand [ 6 ] ints (int) XrmParseCommand ( argv_io argc_io name table-count table dbase -- r )

\ X extensions

\ libXext XShmPutImage [ &11 ] ints (int) XShmPutImage   ( send_event h w dy dx sy sx im gc win dpy -- status )

also forth definitions

base !


\ include Xstring.fs

