\ syscalls386.4th
\
\ Selected system calls for kForth ver >= 1.4.x on 386 machines running Linux.
\   
\ Copyright (c) 2004--2009 Krishna Myneni,
\ Provided under the GNU General Public License
\
\ Notes:
\
\ 1)   System calls under Linux may be performed using a software
\        interrupt, $80, and placing the parameters in appropriate
\        registers. The corresponding C wrapper functions are another
\        way to do this, but the $80 int method is direct. The Forth
\        stack parameters are chosen to correspond to the C wrapper 
\        function argument list.
\ 
\ 2)   There are about 221 system calls under Linux, but this file
\        provides only a select few. The provided syscalls allow 
\        communication with device drivers, e.g. serial port drivers. 
\        Add others as needed following the examples below and using 
\        the man pages for the C wrapper functions. System call numbers 
\        are listed in /usr/include/asm/unistd.h 
\
\ 3)   Compatibility with low-level kForth words is maintained to allow 
\        kForth code to be used under gforth, e.g. serial.fs, terminal.fs.
\        Other driver interface examples from kForth should also
\        work, e.g. the National Instruments GPIB interface nigpib.4th.
\
\ 4)   The code should be readily adaptable to other Forths running
\        on the same platform (386/Linux). It also demonstrates why
\        an assembler can be an important component of a Forth system.
\
\ Revisions:
\ 	2004-09-16  created  KM 
\       2005-11-07  ported to kForth (requires asm-x86.4th)  KM
\       2009-09-26  the word SYSCALL has been intrinsic to kForth
\                   since v. 1.4.1; this file is now updated
\                   to use the intrinsic SYSCALL and no longer requires
\                   asm-x86.4th;  KM


DECIMAL
vocabulary SYSCALLS 
also SYSCALLS definitions

\ From /usr/include/asm-i386/unistd.h
 0  constant  NR_RESTART
 1  constant  NR_EXIT
 2  constant  NR_FORK
 3  constant  NR_READ
 4  constant  NR_WRITE
 5  constant  NR_OPEN
 6  constant  NR_CLOSE
 7  constant  NR_WAITPID
 8  constant  NR_CREAT
 9  constant  NR_LINK
10  constant  NR_UNLINK
11  constant  NR_EXECVE
12  constant  NR_CHDIR
13  constant  NR_TIME
14  constant  NR_MKNOD
15  constant  NR_CHMOD
16  constant  NR_LCHOWN
17  constant  NR_BREAK
18  constant  NR_OLDSTAT
19  constant  NR_LSEEK
20  constant  NR_GETPID
21  constant  NR_MOUNT
22  constant  NR_UMOUNT
23  constant  NR_SETUID
24  constant  NR_GETUID
25  constant  NR_STIME
26  constant  NR_PTRACE
27  constant  NR_ALARM
28  constant  NR_OLDFSTAT
29  constant  NR_PAUSE
30  constant  NR_UTIME
31  constant  NR_STTY
32  constant  NR_GTTY
33  constant  NR_ACCESS
34  constant  NR_NICE
35  constant  NR_FTIME
36  constant  NR_SYNC
37  constant  NR_KILL
38  constant  NR_RENAME
39  constant  NR_MKDIR
40  constant  NR_RMDIR
41  constant  NR_DUP
42  constant  NR_PIPE
43  constant  NR_TIMES
44  constant  NR_PROF
45  constant  NR_BRK
46  constant  NR_SETGID
47  constant  NR_GETGID
48  constant  NR_SIGNAL
49  constant  NR_GETEUID
50  constant  NR_GETEGID
51  constant  NR_ACCT
52  constant  NR_UMOUNT2
53  constant  NR_LOCK
54  constant  NR_IOCTL
55  constant  NR_FCNTL
56  constant  NR_MPX
57  constant  NR_SETPGID
58  constant  NR_ULIMIT
59  constant  NR_OLDOLDUNAME
60  constant  NR_UMASK
61  constant  NR_CHROOT
62  constant  NR_USTAT
63  constant  NR_DUP2
64  constant  NR_GETPPID
65  constant  NR_GETPGRP
66  constant  NR_SETSID
67  constant  NR_SIGACTION
68  constant  NR_SGETMASK
69  constant  NR_SSETMASK
70  constant  NR_SETREUID
71  constant  NR_SETREGID
72  constant  NR_SIGSUSPEND
73  constant  NR_SIGPENDING
74  constant  NR_SETHOSTNAME
75  constant  NR_SETRLIMIT
76  constant  NR_GETRLIMIT
77  constant  NR_GETRUSAGE



: syscall0 0 swap syscall ;
: syscall1 1 swap syscall ;
: syscall2 2 swap syscall ;
: syscall3 3 swap syscall ;


\ sysexit ( code --  | exit to system with code )
\   sysexit is NOT the recommended way to exit back to the 
\   system from Forth. It is provided here as a demo of a very 
\   simple syscall.

: sysexit  1 syscall1 ;

: fork ;

: waitpid ;

: creat ;

: link ;

: unlink ;

: getpid ( -- u | get process id )
	20 syscall0 ;

\ The words OPEN, CLOSE, READ, WRITE, LSEEK, and IOCTL are already
\   defined in kForth with the same behavior. Therefore, the following 
\   defs are not required in kForth.

0 [IF] 
: open ( ^zaddr  flags -- fd | file descriptor is returned)
\   Note zaddr points to a buffer containing the counted filename
\   string terminated with a null character.
        swap 1+ swap
	0 	\ set mode to zero  
	5 syscall3 ;

: close ( fd -- flag )  6 syscall1 ;

: read ( fd  buf  count --  n | read count byes into buf from file )
	3 syscall3 ;

: write ( fd  buf  count  --  n  | write count bytes from buf to file )
	4 syscall3 ;

: lseek ( fd  offset  type  --  offs  | reposition the file ptr )
	19 syscall3 ;

: ioctl ( fd  request argp -- error )
        54 syscall3 ;

[THEN]


