/* 
midicon10.c

  Convert a text representation of a MIDI 1 to a text representation of a
  MIDI 0 file. 


  A program for converting between standard MIDI formats 0 and 1
  is not readily available under Linux. Some musical instruments
  which can play MIDI files from external storage media, e.g. Casio
  WK3300 keyboard, require the files to be in MIDI 0 format.

  Copyright (c) 2007 Krishna Myneni 
  Creative Consulting for Research and Education
  http://ccreweb.org

  This program is provided under the GNU General Public License (GPL).

  
USE:  midicon10 textfile1 > textfile2

  With the use of the program midicomp, the unix sort utility, and
  this program, a MIDI 1 file may be converted to a MIDI 0 file, as follows:

     midicomp    file1.mid > file1.txt
     sort     -g file1.txt > file2.txt
     midicon10   file2.txt > file3.txt
     midicomp -c file3.txt   file4.mid

  where file1.mid is the original MIDI 1 format file, and file4.mid 
  is the new MIDI 0 format file. A shell script may be written to
  perform all of the necessary steps with a single shell command.


BUILD (if building without Makefile): gcc -o midicon10 midicon10.c


  Revisions:

    2007-07-25  created  km
*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

void print_fmt0_header (char []);


int main (int argc, char* argv[])
{
    FILE *f;
    char infile[256], line[256], trkend_line[256];
    int no_fmt = 1;

    // Parse the arg list for input and output files

    if (argc > 1) {
	strcpy (infile, argv[1]);
    }
    else
    {
	printf ("\nFilename: ");
	scanf ("%s", infile);
    }

    // Open the file
    if ((f = fopen(infile, "r")) == NULL)
    {
	printf ("\nUnable to open input file (%s)!\n", infile);
	exit(errno);
    }

    trkend_line[0] = 0;

    while (!feof(f))
    {
	fgets(line, 255, f);

	if (*line)
	{
	    if (no_fmt) {
		if (strstr(line, "MFile "))
		{
		    print_fmt0_header(line);
		    no_fmt = 0;
		}
		else
		{
		    printf ("\nNot a valid input file!\n");
		    exit(1);
		}
	    }
	    else
	    {
		if (strstr(line, "MTrk") == line)
		    ;
		else if (strstr(line, "TrkEnd") == line)
		    ;
		else if (strstr(line, " Meta TrkEnd"))
		    strcpy(trkend_line, line);
		else
		    printf ("%s", line);
	    }
	}
    }
    
    // Write the trailer
    printf ("%s", trkend_line);
    printf ("TrkEnd\n");

    return 0;
}


void print_fmt0_header (char line[])
{
    char *s, *token;
    int i;
    s = strtok(line, " ");
    for (i = 0; i < 3; i++) s = strtok(NULL, " ");
    printf ("MFile 0 1 %s", s);
    printf ("MTrk\n");
}
