C     RESCALE
C
C     Purpose: Rescale X or Y values within an array.                    
C     Syntax: CALL RESCALE (X, Y, JS, JE, RESCALE_EXPRESSION, IERR)
C     Input Parameters:
C        JS   -- starting index in X, Y arrays
C        JE   -- ending index in X, Y arrays
C        RESCALE_EXPRESSION -- character substring containing
C          rescale expression (e.g. 'X * 2.0', 'SIN Y', '1.0 / X')
C     Output Parameters:
C        X, Y -- arrays with rescaled data for given data set.  
C        IERR -- error code:
C                  0 -- no error
C                  1 -- rescale expression empty
C                  2 -- missing X or Y operand
C                  3 -- rescale expression not recognizable
C                  4 -- invalid operator
C                  5 -- attempt to take log(x); x <= 0.
C                  6 -- divide by zero
C     Author: K. Myneni
C     Date: 17 Jan 90
C     Updated: 14 Feb 90
C
      SUBROUTINE RESCALE (X, Y, JS, JE, RESCALE_EXPRESSION, IERR)

      DIMENSION X(*), Y(*)
      CHARACTER*(*) RESCALE_EXPRESSION
      CHARACTER*20 TEMP
      CHARACTER*20 OPERAND_VALUE
      CHARACTER AC
      CHARACTER OC

      LOGICAL X_OPERAND

      CALL STR_UPPER_CASE (RESCALE_EXPRESSION)

      L1 = NLTRIM (RESCALE_EXPRESSION)
      L2 = NRTRIM (RESCALE_EXPRESSION)

      IF (L2.EQ.1) THEN
         IERR = 1              !  rescale expression empty
         RETURN
      END IF

      I1 = INDEX ('SIN,COS,TAN,ATN,LOG,EXP,ABS,L10',
     1            RESCALE_EXPRESSION(L1:L1+2))
      IF (I1.NE.0) THEN
C
C        Evaluate functional expression
C
         IFN = (I1 - 1) / 4 + 1
         I2 = INDEX (RESCALE_EXPRESSION(L1+3:), 'X')
         IF (I2.NE.0) THEN
            X_OPERAND = .TRUE.  
         ELSE
            X_OPERAND = .FALSE.  
         END IF

         DO J = JS, JE

            IF (X_OPERAND) THEN
               C1 = X(J)
            ELSE
               C1 = Y(J)
            END IF

            GO TO (100, 200, 300, 400, 500, 600, 700, 800) IFN

 100        C2 = SIN (C1)
            GO TO 5000
 200        C2 = COS (C1)
            GO TO 5000
 300        C2 = TAN (C1)
            GO TO 5000
 400        C2 = ATAN (C1)
            GO TO 5000
 500        IF (C1.LE.0.) THEN
               IERR = 5
               RETURN
            END IF
            C2 = LOG (C1)
            GO TO 5000
 600        C2 = EXP (C1)
            GO TO 5000
 700        C2 = ABS (C1)
            GO TO 5000
 800        IF (C1.LE.0) THEN
               IERR = 5
               RETURN
            END IF
            C2 = ALOG10(C1)
            GO TO 5000

 5000       CONTINUE

            IF (X_OPERAND) THEN
               X(J) = C2
            ELSE
               Y(J) = C2
            END IF
         END DO
      ELSE
C
C        Evaluate algebraic expression
C
         I1 = INDEX (RESCALE_EXPRESSION, 'X')
         I2 = INDEX (RESCALE_EXPRESSION, 'Y')

         I = I1 + I2

         IF (I.EQ.0) THEN
            IERR = 2              ! missing operand
            RETURN
         END IF

         IF ((I1.NE.0).AND.(I2.NE.0)) THEN
            IERR = 3              ! rescale expression not recognized
            RETURN
         END IF

         IF ((I.NE.L1).AND.(I.NE.L2)) THEN
            IERR = 3              ! rescale expression not recognized
            RETURN
         END IF

         AC = RESCALE_EXPRESSION (I:I)

         IF (AC.EQ.'X') THEN
            X_OPERAND = .TRUE.                                           
         ELSE
            X_OPERAND = .FALSE.                                          
         END IF

         IF (I.EQ.L1) THEN
            TEMP(1:) = RESCALE_EXPRESSION(L1 + 1:)
            L3 = NLTRIM (TEMP)
            OC = TEMP(L3:L3)      ! operator character
            OPERAND_VALUE(1:) = TEMP (L3+1:)
         ELSE
            TEMP(1:) = RESCALE_EXPRESSION(1: L2 - 1)
            L3 = NRTRIM (TEMP)
            OC = TEMP (L3:L3)
            OPERAND_VALUE(1:) = TEMP(1: L3 - 1)
         END IF

         I3 = INDEX ('*/+-^', OC)

         IF (I3.EQ.0) THEN
            IERR = 4              ! operator not recognized
            RETURN
         END IF

         C3 = STRVAL (OPERAND_VALUE)

         DO J = JS, JE
            IF (X_OPERAND) THEN
               C1 = X(J)
            ELSE
               C1 = Y(J)
            END IF

            C2 = C3

            IF (I.NE.L1) THEN
               CSAVE = C1                   ! swap operands
               C1 = C2
               C2 = CSAVE
            END IF

            GO TO (6100, 6200, 6300, 6400, 6500) I3

 6100       C = C1 * C2
            GO TO 7000
 6200       IF (C2.EQ.0.) THEN
               IERR = 6
               RETURN
            END IF
            C = C1 / C2
            GO TO 7000
 6300       C = C1 + C2
            GO TO 7000
 6400       C = C1 - C2
            GO TO 7000
 6500       C = C1 ** C2
            GO TO 7000

 7000       CONTINUE

            IF (X_OPERAND) THEN
               X(J) = C
            ELSE
               Y(J) = C
            END IF

         END DO

      END IF

      IERR = 0                 ! no error
      RETURN
      END
C
C ----------------------------------------------------------------------
C
