C  ********************************************************************
C  *                                                                  *
C  *                             VAXPLOT                              *
C  *                                                                  *
C  *                            Version 2.2                           *
C  *                                                                  *
C  *                                                                  *
C  *            (C) 1990  John Kielkopf and Krishna Myneni            *
C  *                                                                  *
C  *     Multi-windowed plotting workpad for VAX 3200 Workstation     *
C  *                                                                  *
C  ********************************************************************
C  *                                                                  *
C  *  Version 2.2 Release Date: 17 Sep 90                             *
C  *                                                                  *
C  *  Source Files:                                                   *
C  *                                                                  *
C  *     VAXPLOT.FOR           VAXPLOT.INC                            *
C  *     VXPERROR.FOR          VXPERROR.INC                           *
C  *     VXPMENU.FOR           VXPMENU.INC                            *
C  *     VAXFILE.FOR           VXPDISP.INC                            *
C  *     MATHUTIL.FOR          VXPDBASE.INC                           *
C  *     RESCALE.FOR                                                  *
C  *                                                                  *
C  *  Miscellaneous Files:                                            *
C  *                                                                  *
C  *     MAKE_VAXPLOT.COM     -- command file to compile and link     *
C  *				 VAXPLOT                              *
C  *     VAXPLOT.HLP          -- help file                            *
C  *     VXPINI.DAT           -- example initialization file          *
C  *                                                                  *
C  *  Instructions for Compiling and Linking:                         *
C  *                                                                  *
C  *     You may use the command file MAKE_VAXPLOT.COM to perform     *
C  *     the compiling and linking. Type                              *
C  *                                                                  *
C  *         @MAKE_VAXPLOT                                            *
C  *                                                                  *
C  *     at the system prompt. All of the source files must be        *
C  *     present. The VAX FORTRAN Compiler Version 4.0 or higher      *
C  *     is required. In addition, the VAX WORKSTATION SOFTWARE,      *
C  *     VWS Version 3.3 or higher is required.                       *
C  *                                                                  *
C  *                                                                  *
C  *  Installation Notes:                                             *
C  *                                                                  *
C  *    The symbol 'VAXPLOT_HELP_FILE' must be defined with the       *
C  *    DCL DEFINE command to the full path and file name of the      *
C  *    help file (VAXPLOT.HLP, or you may use your own file).        *
C  *                                                                  *
C  *    To specify a different set of initialization parameters,      *
C  *    such as default window dimensions, and colors, define         *
C  *    the symbol 'VAXPLOT_INI_FILE' to be the full path and         *
C  *    file name for an initialization data file. The file format    *
C  *    is described in the user documentation (for an example,       *
C  *    see the file VXPINI.DAT).                                     *
C  *                                                                  *
C  ********************************************************************
C  *                                                                  *
C  * Miscellaneous Program Information:                               *
C  *                                                                  *
C  *								      *
C  *     ATTRIBUTE BLOCKS:                                            *
C  *                                                                  *
C  *       Graphics attribute blocks are reserved as follows:         *
C  *                                                                  *
C  *         BLOCK 1	  : graphics window background attributes     * 
C  *         BLOCK 2      : graphics window foreground attributes     *
C  *         BLOCK 3      : menu window background attributes         *
C  *         BLOCK 4      : menu window foreground attributes         *
C  *         BLOCKS 5-11  : attributes for plots 1 - 7                *
C  *         BLOCK 20     : subroutine PLOT_LIST                      *
C  *         BLOCK 21     : reserved by subroutine AST_BUTTON         * 
C  *         BLOCK 22     : help window                               *
C  *         BLOCKS 30-31 : VXPMENU modules                           *
C  *                                                                  *
C  *                                                                  *
C  ********************************************************************

      	PROGRAM VAXPLOT

      	INCLUDE 'VAXPLOT.INC'

      	CHARACTER*20 OPTION

      	CALL OPENING_MESSAGE (VERSION)
      
      	NW = 0                               
      	OPTION = ' NEW '
                             
      	CALL OPTION_SELECT (NW, OPTION)

      	CALL SYS$HIBER                     ! Go to sleep
              
      	END
C
C ---------------------------------------------------------------------
C
C     OPEN_GRAPHICS_WINDOW
C
C     Purpose: Create a new virtual display and open a graphics
C              window to fill entire display.  
C     Syntax: CALL OPEN_GRAPHICS_WINDOW (XMIN, YMIN, XMAX, YMAX,
C            +   HEIGHT, WIDTH, TITLE, NW, IERR)
C     Input Parameters:
C	 XMIN, YMIN -- lower leftt corner of plotting area
C	 XMAX, YMAX -- upper right corner of plotting area
C	 HEIGHT     -- window height in cm
C	 WIDTH      -- window width in cm
C        TITLE      -- character substring title of window
C                 
C     Output Parameters:
C        NW   -- index of window in record array GW
C        IERR -- error code has the following meanings:
C                  0 -- no error
C                  1 -- exceeded maximum number of displays
C     Authors: J. Kielkopf and K. Myneni
C     Date: 19 Jan 90
C
      	SUBROUTINE OPEN_GRAPHICS_WINDOW (XMIN, YMIN, XMAX, YMAX,
     +     HEIGHT, WIDTH, TITLE, NW, IERR)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'

      	INCLUDE 'SYS$LIBRARY:UISENTRY'
      	INCLUDE 'SYS$LIBRARY:UISUSRDEF'
                               
      	CHARACTER*(*) TITLE

      	INTEGER VD_ID, WD_ID, TR_ID, VCM_ID


C  Find the first available position in array GW to create window
      
      	NW = 0                                    

      	DO J = 1, MAX_DISPLAYS
          IF (GW(J).VD_ID.EQ.0) THEN
            NW = J
            GO TO 5
          END IF
        END DO
 
 5    	CONTINUE
	     
      	IF (NW.EQ.0) THEN       ! max number of displays exceeded
          IERR = 1
          RETURN
      	END IF
      
      
      	VCM_ID=UIS$CREATE_COLOR_MAP(N_COLORS)

      	GW(NW).XMIN = XMIN
      	GW(NW).XMAX = XMAX
      	GW(NW).YMIN = YMIN
      	GW(NW).YMAX = YMAX


      	GW(NW).TITLE = TITLE

      	CALL COMPUTE_DISPLAY_LIMITS (XMIN, YMIN, XMAX, YMAX,
     +     XDMIN, YDMIN, XDMAX, YDMAX)

      	VD_ID = UIS$CREATE_DISPLAY (XDMIN, YDMIN, XDMAX, YDMAX,
     +     WIDTH, HEIGHT, VCM_ID)

      	CALL UIS$SET_COLORS (VD_ID, 0, N_COLORS, R_MAP,
     +     G_MAP, B_MAP)

	ITR = NRTRIM(TITLE)

      	WD_ID = UIS$CREATE_WINDOW (VD_ID, 'SYS$WORKSTATION',
     +     TITLE(1:ITR))

      	GW(NW).VD_ID = VD_ID
      	GW(NW).TR_ID = VD_ID                            
      	GW(NW).WD_ID = WD_ID
      	GW(NW).VCM_ID = VCM_ID

C
C  Copy attribute block 0 into attribute blocks 1 through 11 and
C    set the writing color index for each attribute block.
C
      	DO J = 1, N_COLORS
          CALL UIS$SET_WRITING_INDEX (VD_ID, 0, J, J - 1)
      	END DO
C
C  Attribute block 2 is reserved for graphics screen foreground.
C
      	CALL UIS$SET_WRITING_MODE (VD_ID, 2, 2, UIS$C_MODE_REPL)
              
C
C  Set AST routines for current window
C
      	CALL SET_AST_ROUTINES (NW)	
     
      	IERR = 0

      	RETURN
      	END
C
C ---------------------------------------------------------------------
C
C     AST_FIGURE
C
C     Purpose: Asynchronous System Trap routine to track and display 
C              cursor x, y position in current window.
C
C     Syntax: NONE
C     Input Parameters: NONE
C     Output Parameters: NONE
C     Authors: J. Kielkopf and K. Myneni
C     Date: 1 Feb 90

        SUBROUTINE AST_FIGURE

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDISP.INC'

        INCLUDE 'SYS$LIBRARY:UISENTRY'
        INCLUDE 'SYS$LIBRARY:UISUSRDEF'

	INTEGER TR_ID
        INTEGER WD_ID
	INTEGER ABLK
        CHARACTER*13 X_LABEL
        CHARACTER*13 Y_LABEL
        CHARACTER*26 FLABEL 
        CHARACTER FORM_X_LABEL
        CHARACTER FORM_Y_LABEL
        

        ENTRY FIG1
           NW = 1
           GO TO 100
        ENTRY FIG2
	   NW = 2
	   GO TO 100
	ENTRY FIG3
	   NW = 3
	   GO TO 100
	ENTRY FIG4
	   NW = 4
	   GO TO 100
	ENTRY FIG5
	   NW = 5
	   GO TO 100
	ENTRY FIG6
	   NW = 6
	   GO TO 100
	ENTRY FIG7
	   NW = 7
	   GO TO 100

 100    CONTINUE

        TR_ID = GW(NW).TR_ID
        WD_ID = GW(NW).WD_ID
        ABLK = 2  ! graphics foreground block

        XLBL = (GW(NW).XMIN + GW(NW).XMAX)/2.
        YLBL = GW(NW).YMAX + (GW(NW).YMAX - GW(NW).YMIN)/12.

        CALL UIS$GET_POINTER_POSITION(TR_ID, WD_ID, XPTR, YPTR)
        
        CALL LABEL_FORMAT (GW(NW).XMIN, GW(NW).XMAX, LEN_X_LABEL,
     +      FORM_X_LABEL)

        CALL LABEL_FORMAT (GW(NW).YMIN, GW(NW).YMAX, LEN_Y_LABEL,
     +      FORM_Y_LABEL)

        IF (FORM_X_LABEL.EQ.'F') THEN
           CALL FFORMAT(XPTR, LEN_X_LABEL, X_LABEL)
        ELSE
           CALL EFORMAT(XPTR, LEN_X_LABEL, X_LABEL)
        END IF

        IF (FORM_Y_LABEL.EQ.'F') THEN
           CALL FFORMAT(YPTR, LEN_Y_LABEL, Y_LABEL)
        ELSE
           CALL EFORMAT(YPTR, LEN_Y_LABEL, Y_LABEL)
        END IF                                        

        CALL STRIP_LEADING_ZEROS (X_LABEL)
        CALL STRIP_LEADING_ZEROS (Y_LABEL)
        
        FLABEL(1:13) = X_LABEL(1:13)
        FLABEL(14:26) = Y_LABEL(1:13)

        CALL UIS$TEXT(TR_ID, ABLK, FLABEL, XLBL, YLBL)

        RETURN
        END
C
C ---------------------------------------------------------------------
C
C     CLOSE_ALL_WINDOWS
C
C     Purpose: Delete all graphics virtual displays.                  
C     Syntax: CALL CLOSE_ALL_WINDOWS
C     Input Parameters: NONE
C     Output Parameters: NONE
C     Author: K. Myneni and J. Kielkopf
C     Date: 15 Jan 90
C
      	SUBROUTINE CLOSE_ALL_WINDOWS
                                                           
      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'

      	INCLUDE 'SYS$LIBRARY:UISENTRY'
      	INCLUDE 'SYS$LIBRARY:UISUSRDEF'

      	DO J = MAX_DISPLAYS, 1, -1
          IF (GW(J).VD_ID.NE.0) THEN
            CALL UIS$DELETE_DISPLAY(GW(J).VD_ID)
            CALL INIT_WINDOW_PARAMETERS (J)
          END IF                
      	END DO

      	RETURN
      	END

C
C ---------------------------------------------------------------------
C
C     FILE_READ
C
C     Purpose: Request input filename and read data from file
C              into the database arrays.                                 
C     Syntax: CALL FILE_READ (JSET, IERR)
C     Input Parameters: NONE
C     Output Parameters:
C        JSET  -- data set number containing new data
C        IERR  -- error code has the following meaning
C		    0     -- no error
C	            1-10  -- error code returned by module XYREAD
C		    20    -- no filename entered  
C     Author: K. Myneni
C     Date: 14 Jan 90
C     Modified: 20 Aug 90; added IERR to arg list -- KM
C
      	SUBROUTINE FILE_READ (JSET, IERR)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDBASE.INC'

      	CHARACTER*(HEADER_LENGTH) HEADER
      	CHARACTER*80 FNAME

      	JSET = DB.NSETS + 1                   ! New data set number
      	IF (DB.NPTS.EQ.0) THEN                ! Determine starting index
           JSTART = 1
        ELSE
           JSTART = DS(DB.NSETS).JE + 1
      	END IF
      	JMAX = MAX_POINTS

      	WRITE(6, 1010)
      	CALL FILL_STRING (FNAME, ' ')
 1010 	FORMAT(' Enter file name >> '$)       ! Enter file name
      	READ(5, 1020) FNAME
 1020 	FORMAT(A)

      	IF (NRTRIM(FNAME).EQ.1) THEN
	   IERR = 20
	   RETURN                             ! no file name entered
      	END IF

      	CALL STR_UPPER_CASE (FNAME)

      	IF (INDEX(FNAME, '.').EQ.0) THEN      ! Add '.DAT' extension if
	   L = NRTRIM (FNAME)                 ! no extension is specified
           FNAME(L+1:L+4)= '.DAT'
      	END IF

      	CALL XYREAD (X, Y, JSTART, JMAX, NPTS, IERR, FNAME, HEADER)

      	IF (IERR.NE.0) RETURN                 ! error occured in XYREAD

      	DS(JSET).JS = JSTART
      	DS(JSET).JE = JSTART + NPTS - 1
      	DS(JSET).FILE_NAME = FNAME
      	DS(JSET).HEADER = HEADER

      	CALL DATA_SET_EXTREMA (JSET)          ! set extrema for data set

      	DB.NSETS = DB.NSETS + 1
      	DB.NPTS = DB.NPTS + NPTS

      	CALL DISPLAY_HEADER (JSET)
                    
      	RETURN
      	END
C
C ---------------------------------------------------------------------
C
C     DATA_SET_EXTREMA
C
C     Purpose: Find extrema for given data set.  
C     Syntax: CALL DATA_SET_EXTREMA (NS)
C     Input Parameters:
C        NS -- data set number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 14 Jan 90
C
      	SUBROUTINE DATA_SET_EXTREMA (NS)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDBASE.INC'

      	JS = DS(NS).JS
      	JE = DS(NS).JE

      	XMIN = X(JS)
      	XMAX = X(JS)
      	YMIN = Y(JS)
      	YMAX = Y(JS)

      	DO  J = JS + 1, JE
          IF (X(J).LT.XMIN) XMIN = X(J)
          IF (X(J).GT.XMAX) XMAX = X(J)
          IF (Y(J).LT.YMIN) YMIN = Y(J)
          IF (Y(J).GT.YMAX) YMAX = Y(J)
      	END DO

      	DS(NS).XMIN = XMIN
      	DS(NS).XMAX = XMAX
      	DS(NS).YMIN = YMIN
      	DS(NS).YMAX = YMAX

      	RETURN
      	END

C        
C ---------------------------------------------------------------------
C
C     WINDOW_EXTREMA
C
C     Purpose: Find extrema for plots in a given graphics window.        
C     Syntax: CALL WINDOW_EXTREMA (NW, XMIN, YMIN, XMAX, YMAX, IERR)
C     Input Parameters:
C        NW -- window number
C     Output Parameters:
C	 XMIN, YMIN -- lower left corner of plotting area
C	 XMAX, YMAX -- upper right corner of plotting area
C	 IERR       -- error number has the following meaning
C		         0  -- no error
C                        1  -- bad window number
C                        2  -- non-existent display
C     Author: K. Myneni
C     Date: 14 Jan 90
C
      	SUBROUTINE WINDOW_EXTREMA (NW, XMIN, YMIN, XMAX, YMAX, IERR)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'
      	INCLUDE 'VXPDBASE.INC'
      
      	IF (NW.GT.MAX_DISPLAYS) THEN
	  IERR = 1                             ! bad window number
      	  RETURN
      	END IF

      	IF (GW(NW).VD_ID.EQ.0) THEN
          IERR = 2                             ! non-existent display
	  RETURN
      	END IF
		
      	J = GW(NW).P(1).SET
                         
      	XMIN = DS(J).XMIN
      	XMAX = DS(J).XMAX
      	YMIN = DS(J).YMIN
      	YMAX = DS(J).YMAX

      	IF (GW(NW).NPLOTS.GT.1) THEN
          DO K = 2, GW(NW).NPLOTS
            J = GW(NW).P(K).SET
            IF (DS(J).XMIN.LT.XMIN) XMIN = DS(J).XMIN
            IF (DS(J).XMAX.GT.XMAX) XMAX = DS(J).XMAX
            IF (DS(J).YMIN.LT.YMIN) YMIN = DS(J).YMIN
            IF (DS(J).YMAX.GT.YMAX) YMAX = DS(J).YMAX
          END DO
      	END IF
      
      	IERR = 0

      	RETURN
      	END
C
C ---------------------------------------------------------------------
C
C     DRAW_WINDOW
C
C     Purpose: Draw all plots in given graphics window.                
C     Syntax: CALL DRAW_WINDOW (NW)
C     Input Parameters:
C        NW -- window number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 18 Jan 90
C
      	SUBROUTINE DRAW_WINDOW (NW)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'

      	INCLUDE 'SYS$LIBRARY:UISENTRY'
      	INCLUDE 'SYS$LIBRARY:UISUSRDEF'
C
C     Erase the window first
C
      	CALL UIS$ERASE (GW(NW).VD_ID)

      	CALL GRID (NW)                           ! draw the grid and
      	CALL COORDINATE_LABELS (NW)              ! coordinate labels
      	CALL PLOT_LIST (NW)                      ! display plot list

      	DO I = 1, GW(NW).NPLOTS
          CALL DRAW_PLOT (I, NW, IERR)              ! draw each plot
      	END DO  
                                 
      	RETURN
      	END
C
C ---------------------------------------------------------------------
C
C     GRID
C
C     Purpose: Draw a border with tic marks.  
C     Syntax: CALL GRID (NW)
C     Input Parameters:
C        NW -- window number in which grid is to be drawn.
C     Output Parameters: NONE
C     Author: J. Kielkopf and K. Myneni
C     Date: 16 Jan 90
C
        SUBROUTINE GRID (NW)

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDISP.INC'

        INCLUDE 'SYS$LIBRARY:UISENTRY'
        INCLUDE 'SYS$LIBRARY:UISUSRDEF'

        INTEGER VD_ID, TR_ID, WD_ID, ABLK

        VD_ID = GW(NW).VD_ID
        TR_ID = GW(NW).TR_ID
        WD_ID = GW(NW).WD_ID
        XMIN = GW(NW).XMIN
        XMAX = GW(NW).XMAX
        YMIN = GW(NW).YMIN
        YMAX = GW(NW).YMAX

        ABLK = 2  ! attribute block 2 is graphics foreground

C        Draw the box.  

        CALL UIS$PLOT (TR_ID, ABLK, XMIN, YMIN, XMAX, YMIN, 
     +      XMAX, YMAX, XMIN, YMAX, XMIN, YMIN)

C        Draw the grid tics.  

        XDEL = 0.1*(XMAX - XMIN)
        YDEL = 0.1*(YMAX - YMIN)

        DO I = 1, 9
          U = XMIN + XDEL*I
          V = YMIN + YDEL*I
          UBMIN = XMIN + 0.1*XDEL
          UBMAX = XMAX - 0.1*XDEL
          VBMIN = YMIN + 0.1*YDEL
          VBMAX = YMAX - 0.1*YDEL
          CALL UIS$PLOT (TR_ID, ABLK, U, YMIN, U, VBMIN)
          CALL UIS$PLOT (TR_ID, ABLK, U, YMAX, U, VBMAX)
          CALL UIS$PLOT (TR_ID, ABLK, XMIN, V, UBMIN, V)
          CALL UIS$PLOT (TR_ID, ABLK, XMAX, V, UBMAX, V)
        END DO

        RETURN
        END
C
C ---------------------------------------------------------------------
C
C     INIT_PARAMETERS
C
C     Purpose: Initialize database, and display variables.  
C     Syntax: CALL INIT_PARAMETERS
C     Input Parameters: NONE
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 14 Jan 90
C
      	SUBROUTINE INIT_PARAMETERS

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'
      	INCLUDE 'VXPDBASE.INC'
      	INCLUDE 'VXPMENU.INC'

C  Initialize data base variables

      	DB.NSETS = 0
      	DB.NPTS = 0  

C  Initialize display variables

        DO J = 1, MAX_DISPLAYS
          CALL INIT_WINDOW_PARAMETERS (J)
        END DO

C  Read the initialization data file if it exists.
      
      
      	CALL READ_PARAMETERS

      	HW.VD_ID = 0
      	MW.VD_ID = 0

      	RETURN
      	END
C
C ---------------------------------------------------------------------
C
C     COORDINATE_LABELS
C
C     Purpose: Place coordinate labels in given graphics window.  
C     Syntax: CALL COORDINATE_LABELS (NW)
C     Input Parameters:
C        NW -- window number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 17 Jan 90
C
      	SUBROUTINE COORDINATE_LABELS (NW)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'

      	INCLUDE 'SYS$LIBRARY:UISENTRY'
      	INCLUDE 'SYS$LIBRARY:UISUSRDEF'

      	CHARACTER*13 X_LABEL
      	CHARACTER*13 Y_LABEL
      	CHARACTER FORM_X_LABEL
      	CHARACTER FORM_Y_LABEL
      	INTEGER ABLK

      	ABLK = 2  ! attribute block 2 is graphics foreground.	

      	CALL LABEL_FORMAT (GW(NW).XMIN, GW(NW).XMAX, LEN_X_LABEL,
     +    FORM_X_LABEL)

      	CALL LABEL_FORMAT (GW(NW).YMIN, GW(NW).YMAX, LEN_Y_LABEL,
     +     FORM_Y_LABEL)
      
      	YDEL = GW(NW).YMAX - GW(NW).YMIN
      	XDEL = GW(NW).XMAX - GW(NW).XMIN
      	XD_AREA = XDEL/X_PLOT_AREA
      	YD_AREA = YDEL/Y_PLOT_AREA

C  Compute label positions

	CALL UIS$GET_VIEWPORT_SIZE (GW(NW).WD_ID, WIDTH, HEIGHT)

	W0 = 0.8 * WIDTH * (1. - X_PLOT_AREA)

	XL_POSN = GW(NW).XMIN
	X_OFFSET = -X_PLOT_OFFSET*XD_AREA

	IF (W0.GT.4.5) THEN
      	  XL_POSN = XL_POSN + X_OFFSET
        ELSE
	  XL_POSN = XL_POSN + 5.0*X_OFFSET/W0
	END IF

      	YL_POSN = GW(NW).YMIN - 0.4*Y_PLOT_OFFSET*YD_AREA

      	J_SKIP = 2

      	DO J = 0, 10, J_SKIP

           YL = GW(NW).YMIN + J*0.1*YDEL       
           YLP = YL + 0.03*YDEL
           IF (FORM_Y_LABEL.EQ.'F') THEN
              CALL FFORMAT (YL, LEN_Y_LABEL, Y_LABEL)
           ELSE
              CALL EFORMAT (YL, LEN_Y_LABEL, Y_LABEL)
           END IF
         
           CALL STRIP_LEADING_ZEROS (Y_LABEL)
           CALL RIGHT_TRIM (Y_LABEL)
                        
           CALL UIS$TEXT (GW(NW).TR_ID, ABLK, Y_LABEL,
     +        XL_POSN, YLP)
                        
        END DO

        IF (LEN_X_LABEL.LT.10) THEN
           J_SKIP = 2
        ELSE
           J_SKIP = 5
        END IF

	IF (WIDTH.LT.10.) THEN
	   IF (WIDTH.LT.7.) THEN
	      J_SKIP = 10
	   ELSE
	      J_SKIP = 5
	   END IF
	END IF	

        DO J = 0, 10, J_SKIP    

           XL = GW(NW).XMIN + J*0.1*XDEL
           XLP = XL - 0.02*XDEL
           IF (FORM_X_LABEL.EQ.'F') THEN
              CALL FFORMAT (XL, LEN_X_LABEL, X_LABEL)
           ELSE
              CALL EFORMAT (XL, LEN_X_LABEL, X_LABEL)
           END IF
         
	   CALL STRIP_LEADING_ZEROS (X_LABEL)
           CALL LEFT_TRIM (X_LABEL)
                       
           CALL UIS$TEXT (GW(NW).TR_ID, ABLK, X_LABEL, XLP, YL_POSN)

      	END DO

      	RETURN
      	END
C
C ---------------------------------------------------------------------
C
C     LABEL_FORMAT
C
C     Purpose: Determine the coordinate label format.  
C     Syntax: CALL LABEL_FORMAT (SMIN, SMAX, L, FORM)
C     Input Parameters:
C        SMIN, SMAX -- minimum and maximum values for labels
C                      (real numbers).  
C     Output Parameters:
C        L -- number of decimal places in label.  
C        FORM -- character asumming the values, 'F', or 'E' for
C                floating point or exponential format, respectively.  
C     Author: K. Myneni
C     Date: 17 Jan 90
C
      SUBROUTINE LABEL_FORMAT (SMIN, SMAX, L, FORM)

      CHARACTER FORM

      VMAX = AMAX1 (ABS (SMAX), ABS (SMIN))
      SDEL = SMAX - SMIN

      IF ((SDEL.LT.(.01)).OR.(SDEL.GE.10000)) THEN
         L = 6
         FORM = 'E'
         RETURN
      ELSE IF ((SDEL.GE.(.01)).AND.(SDEL.LT.(0.1))) THEN
         L = 5
         FORM = 'F'
      ELSE IF ((SDEL.GE.(0.1)).AND.(SDEL.LT.(1.))) THEN
         L = 4
         FORM = 'F'
      ELSE IF ((SDEL.GE.(1.)).AND.(SDEL.LT.(10.))) THEN
         L = 3
         FORM = 'F'                   
      ELSE IF ((SDEL.GE.(10.)).AND.(SDEL.LT.(100.))) THEN
         L = 2
         FORM = 'F'
      ELSE IF ((SDEL.GE.(100.)).AND.(SDEL.LT.(1000.))) THEN
         L = 1
         FORM = 'F'
      ELSE
         L = 0
         FORM = 'F'
      END IF

      A = VMAX / SDEL

      I = INT (ALOG10 (A)) - 1
      IF (I.LE.L) THEN
         L = L - I
      ELSE
         L = 6
         FORM = 'E'
      END IF
                                            
      RETURN
      END
C
C ---------------------------------------------------------------------
C
C     COMMAND_MODE
C
C     Purpose: Use command prompt to execute VAXPLOT commands.
C     Syntax: CALL COMMAND_MODE (NW)
C     Input Parameters:
C       NW -- active window number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 14 Feb 90
C
      SUBROUTINE COMMAND_MODE (NW)

      CHARACTER*20 OPTION
      
      TYPE *
      TYPE *, '<< Command Mode >>'
      TYPE *
      TYPE *, '  Do not use menus while in command mode.'
      TYPE *, '  Type RESUME to return to menu mode.'
      TYPE *
      TYPE *
	
      DO WHILE (.TRUE.)

         CALL FILL_STRING (OPTION, ' ')

         WRITE(6, 200)
         READ (5, 300) (OPTION(J:J), J = 2, 20)
 200     FORMAT(' COMMAND >> '$)
 300     FORMAT(20A1)

         CALL STR_UPPER_CASE (OPTION)
      
         IF ((NRTRIM(OPTION).EQ.1).OR.
     +       (INDEX(OPTION, 'RESUME').NE.0)) THEN
C
C          Exit command mode.
C
           TYPE *
           TYPE *, '<< Menu Mode resumed >>'
           TYPE *

           RETURN                         
                      
         ELSE

            CALL OPTION_SELECT (NW, OPTION)

         END IF
      END DO

      RETURN
      END
C
C ---------------------------------------------------------------------
C
C     OPENING_MESSAGE
C
C     Purpose: Display opening message.  
C     Syntax: CALL OPENING_MESSAGE (VERSION)
C     Input Parameters:
C        VERSION -- character substring containing version number.       
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 20 Jan 90
C
      	SUBROUTINE OPENING_MESSAGE (VERSION)

      	CHARACTER*(*) VERSION

      	TYPE *
      	WRITE(6,100) VERSION(1:4)
 100  	FORMAT(30X, 'VAXPLOT  Ver ', A4)
      	TYPE *
      	TYPE *
      	WRITE(6,200)
 200  	FORMAT(20X, '(C) 1990  John Kielkopf and Krishna Myneni')
      	TYPE *
      	TYPE *

      	RETURN
      	END
C
C ---------------------------------------------------------------------
C
C     DISPLAY_HEADER
C
C     Purpose: Display a data set header
C     Syntax: CALL DISPLAY_HEADER (NS)
C     Input Parameters:
C        NS -- data set number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 20 Jan 90
C     Modified: 19 July 90 -- KM
C
        SUBROUTINE DISPLAY_HEADER (NS)

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDBASE.INC'

	CHARACTER*80 CLINE

	CALL FILL_STRING (CLINE, '=')

	TYPE *
	IRF = NRTRIM (DS(NS).FILE_NAME)
	NPTS = DS(NS).JE - DS(NS).JS + 1
	WRITE (6, 10) NS, DS(NS).FILE_NAME(1:IRF), NPTS
 10	FORMAT (1X, 'DATA SET ', I2, 5X, 'FILE: ', A, 5X, I5,
     +     ' points.')

	TYPE *, CLINE(1:80)
        CALL TYPE_HEADER(DS(NS).HEADER)
	TYPE *, CLINE(1:80)

        RETURN
        END
C
C ---------------------------------------------------------------------
C
C     DRAW_PLOT
C            
C     Purpose: Draw the specified plot in the specified window.         
C     Syntax: CALL DRAW_PLOT (NP, NW, IERR)
C     Input Parameters:
C        NP  -- number of plot to be drawn
C        NW  -- number of window
C     Output Parameters:
C        IERR -- error code has the following meanings:
C                  0 -- no error
C                  1 -- non-existent plot in specified window
C     Author: K. Myneni
C     Date: 23 Jan 90
C                         
C
      	SUBROUTINE DRAW_PLOT (NP, NW, IERR)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'
      	INCLUDE 'VXPDBASE.INC'                                    
      
      	INCLUDE 'SYS$LIBRARY:UISENTRY'
      	INCLUDE 'SYS$LIBRARY:UISUSRDEF'

      	INTEGER VD_ID, TR_ID
      	INTEGER ABLK
      	INTEGER PIXEL_WIDTH
      	CHARACTER PSYM

      	IF (NP.GT.GW(NW).NPLOTS) THEN
          IERR = 1                           ! no such plot belongs to
          RETURN                             ! window
      	END IF

      	VD_ID = GW(NW).VD_ID
      	TR_ID = GW(NW).TR_ID

      	XMIN = GW(NW).XMIN
      	XMAX = GW(NW).XMAX
      	YMIN = GW(NW).YMIN
      	YMAX = GW(NW).YMAX

      	ISET = GW(NW).P(NP).SET
      	JS = DS(ISET).JS
      	JE = DS(ISET).JE                                       
      	ABLK = GW(NW).P(NP).BLOCK
      	PSYM = GW(NW).P(NP).SYMBOL 

!  Set clipping window

      	CALL UIS$SET_CLIP (TR_ID, ABLK, ABLK, XMIN, YMIN,
     +     XMAX, YMAX)


      	IF (PSYM.EQ.'L') THEN

!  Line plot
                         
          CALL UIS$PLOT_ARRAY (TR_ID, ABLK, JE - JS + 1, 
     +       X(JS), Y(JS))
      
      	ELSE IF (PSYM.EQ.'P') THEN

!  Point plot

          RADIUS = (XMAX - XMIN) / 200.
          DO J = JS, JE
            CALL UIS$CIRCLE (TR_ID, ABLK, X(J), Y(J), RADIUS)
          END DO

      	ELSE IF (PSYM.EQ.'S') THEN

!  Stick plot

          DO J = JS, JE
            CALL UIS$PLOT (TR_ID, ABLK, X(J), 0., X(J), Y(J))
          END DO

      	END IF

! Disable clipping

      	CALL UIS$SET_CLIP (TR_ID, ABLK, ABLK)
      	IERR = 0

      	RETURN
      	END
C
C ---------------------------------------------------------------------
C
C     RESET_WINDOW_LIMITS
C
C     Purpose: Create a transformation to new world coordinates
C     Syntax: CALL RESET_WINDOW_LIMITS (NW, XMIN, YMIN, XMAX, YMAX)
C     Input Parameters:
C	 NW 	-- window number
C	 XMIN, YMIN -- lower left corner of plotting area
C	 XMAX, YMAX -- upper right corner of plotting area
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 28 Jan 90
C
      	SUBROUTINE RESET_WINDOW_LIMITS (NW, XMIN, YMIN, XMAX, YMAX)


      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'

      	INCLUDE 'SYS$LIBRARY:UISENTRY'
      	INCLUDE 'SYS$LIBRARY:UISUSRDEF'

      	INTEGER TR_ID, VD_ID

      	IF (NW.GT.MAX_DISPLAYS) RETURN         ! bad window number

      	VD_ID = GW(NW).VD_ID     
      
      	IF (VD_ID.EQ.0) RETURN                 ! window not open
      
      	IF (GW(NW).FREEZE) RETURN              ! window limits frozen

      	GW(NW).XMIN = XMIN
      	GW(NW).XMAX = XMAX
      	GW(NW).YMIN = YMIN
      	GW(NW).YMAX = YMAX

      	CALL COMPUTE_DISPLAY_LIMITS (XMIN, YMIN, XMAX, YMAX,
     +     XDMIN, YDMIN, XDMAX, YDMAX)

      	TR_ID = UIS$CREATE_TRANSFORMATION (VD_ID, XDMIN,
     +          YDMIN, XDMAX, YDMAX)

      	GW(NW).TR_ID = TR_ID

      	RETURN
      	END
C
C -------------------------------------------------------------------
C
C     COMPUTE_DISPLAY_LIMITS
C
C     Purpose: Compute corner coordinates of display for given
C              plotting area limits.
C     Syntax: CALL COMPUTE_DISPLAY_LIMITS (XMIN, YMIN, XMAX, YMAX,
C            +  XDMIN, YDMIN, XDMAX, YDMAX)
C
C
      	SUBROUTINE COMPUTE_DISPLAY_LIMITS (XMIN, YMIN, XMAX, YMAX,
     +     XDMIN, YDMIN, XDMAX, YDMAX)

      	INCLUDE 'VAXPLOT.INC'

      	XDEL = XMAX - XMIN
      	YDEL = YMAX - YMIN

C  Compute world coordinates of virtual display corners

      	XD_AREA = XDEL / X_PLOT_AREA
      	YD_AREA = YDEL / Y_PLOT_AREA


      	XDMIN = XMIN - X_PLOT_OFFSET*XD_AREA
      	XDMAX = XMAX + 0.1*XD_AREA
      	YDMIN = YMIN - Y_PLOT_OFFSET*YD_AREA
      	YDMAX = YMAX + 0.1*YD_AREA

      	RETURN
      	END
C
C ------------------------------------------------------------------
C
C    DROP_PLOT (NP, NW)
C
C    Purpose: Drop selected plot in selected window.
C    Syntax: CALL DROP_PLOT (NP, NW)
C    Input Parameters:
C       NP  -- plot number
C       NW  -- window number
C    Output Parameters: NONE
C    Author: K. Myneni
C    Date: 24 Jan 90
C
      	SUBROUTINE DROP_PLOT (NP, NW)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'

      	INCLUDE 'SYS$LIBRARY:UISENTRY'
      	INCLUDE 'SYS$LIBRARY:UISUSRDEF'

      	NPLOTS = GW(NW).NPLOTS
                               
      	IF (NP.GT.NPLOTS) RETURN
      
      	IF (NPLOTS.EQ.1) THEN
          CALL UIS$DELETE_DISPLAY(GW(NW).VD_ID)      ! delete window
          CALL INIT_WINDOW_PARAMETERS (NW)
          RETURN
      	END IF

      	IF (NP.NE.NPLOTS) THEN
          DO J = NP, NPLOTS - 1
            GW(NW).P(J) = GW(NW).P(J + 1)
          END DO
      	END IF
      
      	GW(NW).P(NPLOTS).BLOCK = 0
      	GW(NW).NPLOTS = NPLOTS - 1
      	GW(NW).ACTIVE_PLOT = GW(NW).NPLOTS
                                          
      	RETURN
      	END
C
C -----------------------------------------------------------------------
C
C     VAXPLOT_HELP
C
C     Purpose: Open a help window and display the contents of the
C              help file. If the help window has been opened
C              previously, this subroutine simply pops the window.
C     Syntax: CALL VAXPLOT_HELP
C     Input Parameters: NONE
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 11 Feb 90
C     Notes: The VAXPLOT file path and name should be set with the DCL
C            DEFINE command as the symbol 'VAXPLOT_HELP_FILE'.
C     
C            Attribute block 22 is reserved by this module.
C                                                         
        SUBROUTINE VAXPLOT_HELP
      
        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDISP.INC'

        INCLUDE 'SYS$LIBRARY:UISENTRY'
        INCLUDE 'SYS$LIBRARY:UISUSRDEF'
                                                                   
        INTEGER VD_ID, WD_ID
        CHARACTER*80 CLINE
        CHARACTER*31 HELP_FONT
      
        IF (HW.VD_ID.NE.0) THEN

C  Help window already created, so no need to make another one;
C    just pop existing window, or expand icon.

           J = UIS$GET_VIEWPORT_ICON (HW.WD_ID)
           IF (J.EQ.1) THEN
              CALL UIS$EXPAND_ICON (HW.WD_ID)     ! expand help window icon
           ELSE
              CALL UIS$POP_VIEWPORT (HW.WD_ID)    ! pop help window
           END IF
           RETURN
        END IF

C       HELP_FONT(1:) = 'DTABER0003WK00GG0001QZZZZ02A000'
C
      
        ILU = NEXT_AVAILABLE_UNIT()
        OPEN (UNIT = ILU, FILE = 'VAXPLOT_HELP_FILE', READONLY,
     +     ERR = 1000, STATUS = 'OLD')


        VD_ID = UIS$CREATE_DISPLAY (0., 0., 10., 10., 18., 14.)
        WD_ID = UIS$CREATE_WINDOW (VD_ID, 'SYS$WORKSTATION',
     +    'VAXPLOT HELP')

        HW.VD_ID = VD_ID
        HW.WD_ID = WD_ID
      

C  Disable the window closing option.
                                                
        CALL UIS$SET_CLOSE_AST (WD_ID)


        CALL UIS$SET_WRITING_MODE (VD_ID, 0, 22, UIS$C_MODE_REPL)
C       CALL UIS$SET_FONT (VD_ID, 0, 22, HELP_FONT)
        CALL UIS$SET_ALIGNED_POSITION (VD_ID, 22, 0., 10.)

        DO WHILE (.TRUE.)
           READ (ILU, 100, END = 300) (CLINE(J:J), J = 1, 80)
           CALL UIS$TEXT (VD_ID, 22, CLINE)
           CALL UIS$NEW_TEXT_LINE (VD_ID, 22)
 100       FORMAT(80A1)
        END DO

 300    CLOSE (ILU)
        GO TO 1200

 1000   CONTINUE

        TYPE *, 'Cannot find help file.'

 1200   CONTINUE

        RETURN
        END
C
C ----------------------------------------------------------------------
C
C     AST_CLOSE_WINDOW
C
C     Purpose: Asynchronous System Trap routine for closing graphics
C              window.
C     Syntax: NONE
C     Input Parameters: NONE
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 29 Jan 90
C
      SUBROUTINE AST_CLOSE_WINDOW

      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPDISP.INC'

      INCLUDE 'SYS$LIBRARY:UISENTRY'
      INCLUDE 'SYS$LIBRARY:UISUSRDEF'

      INTEGER VD_ID

      ENTRY CW1
	 NW = 1                
	 GO TO 100
      ENTRY CW2
         NW = 2    
         GO TO 100
      ENTRY CW3
         NW = 3
         GO TO 100                         
      ENTRY CW4
         NW = 4
         GO TO 100
      ENTRY CW5
         NW = 5
         GO TO 100
      ENTRY CW6                             
         NW = 6
         GO TO 100
      ENTRY CW7
         NW = 7
         GO TO 100

 100  CONTINUE

      VD_ID = GW(NW).VD_ID
      CALL UIS$DELETE_DISPLAY (VD_ID)
      CALL INIT_WINDOW_PARAMETERS (NW)
C
C     If last window is closed, start over.
C      
      IF (N_WINDOWS_OPEN().EQ.0) THEN
          NW = 0
          CALL OPTION_SELECT (NW, ' NEW ')
      END IF                          

      RETURN
      END                                
C
C -------------------------------------------------------------------------
C
C     AST_OPTION      
C
C     Purpose: Asynchronous System Trap for Additional Options.
C     Syntax: NONE
C     Input Parameters: NONE
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 11 Feb 90
C
      SUBROUTINE AST_OPTION

      ENTRY OP1
	 NW = 1
	 GO TO 100
      ENTRY OP2
	 NW = 2
	 GO TO 100
      ENTRY OP3
	 NW = 3
	 GO TO 100
      ENTRY OP4
	 NW = 4
	 GO TO 100
      ENTRY OP5
	 NW = 5
	 GO TO 100
      ENTRY OP6
	 NW = 6
	 GO TO 100
      ENTRY OP7
	 NW = 7
	 GO TO 100

 100  CONTINUE

      CALL SHOW_OPTIONS_MENU (NW)

      RETURN
      END
C
C --------------------------------------------------------------------------
C
C     AST_BUTTON
C
C     Purpose: Asynchronous System Trap for button press.
C     Syntax: NONE
C     Input Parameters: NONE
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 31 Jan 90
C     Notes: Attribute block 21 is reserved for use here.
C     Updated: 15 Feb 90
C     

      SUBROUTINE AST_BUTTON

      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPDISP.INC'

      INCLUDE 'SYS$LIBRARY:UISENTRY'
      INCLUDE 'SYS$LIBRARY:UISUSRDEF'
      
      LOGICAL FLAG
      CHARACTER*20 OPTION

      INTEGER VD_ID, WD_ID, TR_ID
      INTEGER BUTTON_STATUS
      INTEGER BUTTON_STATE
      

      ENTRY BU1
         NW = 1
         GO TO 100
      ENTRY BU2                
         NW = 2
         GO TO 100
      ENTRY BU3
         NW = 3
         GO TO 100
      ENTRY BU4
         NW = 4
         GO TO 100
      ENTRY BU5
         NW = 5
         GO TO 100
      ENTRY BU6
         NW = 6
         GO TO 100
      ENTRY BU7
         NW = 7
         GO TO 100
     
 100  CONTINUE
      
      IF (KEYBUF.GT.0) RETURN       ! return if button is being released
      
      IF (BTEST(KEYBUF, 0)) THEN
C
C        Middle button was pressed; show options menu
C
         CALL SHOW_OPTIONS_MENU (NW)
         
         RETURN
      END IF
C
C     First button was pressed; set expansion corners.
C
      VD_ID = GW(NW).VD_ID
      WD_ID = GW(NW).WD_ID
      TR_ID = GW(NW).TR_ID
      XDEL = GW(NW).XMAX - GW(NW).XMIN
      YDEL = GW(NW).YMAX - GW(NW).YMIN

      CALL UIS$GET_POINTER_POSITION (TR_ID, WD_ID, XPTR, YPTR)
      
      IF (YPTR.GT.GW(NW).YMAX) THEN
         
          ! Set the active plot.

          JPLOT = INT((XPTR - GW(NW).XMIN)/(XDEL/15.)) + 1
          IF (JPLOT.LE.GW(NW).NPLOTS) THEN
             GW(NW).ACTIVE_PLOT = JPLOT
             CALL PLOT_LIST (NW)
             RETURN
          END IF
      END IF

      ! Set expansion window
      
      XMIN = XPTR
      YMIN = YPTR

      ! Loop util cursor is moved or button is released.

      CALL UIS$GET_POINTER_POSITION (TR_ID, WD_ID, XPTR, YPTR)

      DO WHILE ((XPTR.EQ.XMIN).OR.(YPTR.EQ.YMIN))
	 BUTTON_STATUS = UIS$GET_BUTTONS (WD_ID, BUTTON_STATE)
         IF (BUTTON_STATUS.EQ.0) RETURN
         IF (BTEST(BUTTON_STATE, UIS$M_POINTER_BUTTON_2)) GO TO 2000
         CALL UIS$GET_POINTER_POSITION (TR_ID, WD_ID, XPTR, YPTR)
      END DO
      
C
C     Set bitwise complement writing mode for drawing box. 
C 
      CALL UIS$SET_WRITING_MODE (VD_ID, 2, 21, UIS$C_MODE_COMP)
      CALL UIS$SET_WRITING_INDEX (VD_ID, 21, 21, 1)	

      ! Loop until opposite corner of expansion box is defined
                                   
      DO WHILE (.TRUE.)

         ! Draw a box

         CALL UIS$PLOT (TR_ID, 21, XMIN, YMIN, XPTR, YMIN, XPTR, YPTR,
     +       XMIN, YPTR, XMIN, YMIN)

         ! Loop until coordinates change or button is released

         CALL UIS$GET_POINTER_POSITION (TR_ID, WD_ID, XPTR2, YPTR2)

         DO WHILE ((XPTR2.EQ.XPTR).AND.(YPTR2.EQ.YPTR))	
            BUTTON_STATUS = UIS$GET_BUTTONS (WD_ID, BUTTON_STATE)
            IF (BUTTON_STATUS.EQ.0) RETURN
            IF (BTEST(BUTTON_STATE, UIS$M_POINTER_BUTTON_2)) GO TO 2000
            CALL UIS$GET_POINTER_POSITION (TR_ID, WD_ID, XPTR2, YPTR2)
         END DO   
       
         ! Coordinates have changed, so remove box by drawing its
         ! bitwise complement. 

         CALL UIS$PLOT (TR_ID, 21, XMIN, YMIN, XPTR, YMIN, XPTR, YPTR,
     +       XMIN, YPTR, XMIN, YMIN)

         XPTR = XPTR2
         YPTR = YPTR2
      END DO 

 2000 CONTINUE

      XMAX = XPTR
      YMAX = YPTR
      
      IF ((XMAX.EQ.XMIN).OR.(YMAX.EQ.YMIN)) RETURN

      IF (XMAX.LT.XMIN) THEN
         XSAVE = XMIN
         XMIN = XMAX
         XMAX = XSAVE
      END IF

      IF (YMAX.LT.YMIN) THEN
         YSAVE = YMIN
         YMIN = YMAX
         YMAX = YSAVE
      END IF
      
      EX.XMIN = XMIN
      EX.XMAX = XMAX
      EX.YMIN = YMIN
      EX.YMAX = YMAX
      EX.FLAG = .TRUE.

      CALL OPTION_SELECT (NW, 'EXPAND ')

      RETURN                                       
      END
C
C ---------------------------------------------------------------------------
C                                       
C     OPTION_SELECT
C
C
      SUBROUTINE OPTION_SELECT (NW, OPTION)

      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPDISP.INC'
      INCLUDE 'VXPDBASE.INC'
      INCLUDE 'VXPERROR.INC'

      INCLUDE 'SYS$LIBRARY:UISENTRY'
      INCLUDE 'SYS$LIBRARY:UISUSRDEF'
                                       
      CHARACTER*(*) OPTION

      LOGICAL RESET_EXTREMA    
      LOGICAL RE_DRAW
      LOGICAL SET_ALTERED
      LOGICAL ERROR_OCCURED

      LOGICAL SFLAG	
     
      IF (NRTRIM(OPTION).EQ.1) RETURN

 1000 CONTINUE

C				
C     Initialize flow control flags
C      
      RESET_EXTREMA = .TRUE.       
      RE_DRAW = .TRUE.             
      SET_ALTERED = .FALSE.
      ERROR_OCCURED = .FALSE.

C
C     Note the meanings of the following variables.
C
C     NW  -- active window number; index in GW
C     NP  -- active plot number; index in GW(NW).P
C     NS  -- data set number associated with plot NP; index in DS
C
      NP = GW(NW).ACTIVE_PLOT     
      NS = GW(NW).P(NP).SET        
C
C =========================== SELECT OPTION ===========================
C
      IF (INDEX(OPTION, 'NEW ').NE.0) THEN

         ! new plot; close any open windows and start again

         CALL CLOSE_ALL_WINDOWS

         ! initialize database, display, and graphics window records.     

         CALL INIT_PARAMETERS  

         CALL FILE_READ (JSET, IERR)
         IF (IERR.EQ.0) THEN
            XMIN = DS(JSET).XMIN
            XMAX = DS(JSET).XMAX
	    YMIN = DS(JSET).YMIN
	    YMAX = DS(JSET).YMAX
            CALL OPEN_GRAPHICS_WINDOW (XMIN, YMIN, XMAX, YMAX,
     +          WINDOW_HEIGHT, WINDOW_WIDTH, DS(JSET).FILE_NAME,
     +          NW, IERR)
            IF (IERR.EQ.0) THEN
               GW(NW).NPLOTS = 1
               GW(NW).ACTIVE_PLOT = 1
               GW(NW).P(1).SET = JSET
               CALL SET_PLOT_ATTRIBUTE (1, NW)
            ELSE
               CALL VXPERROR (E_OPEN_GW, IERR)
               ERROR_OCCURED = .TRUE.
            END IF            
            RESET_EXTREMA = .FALSE.
         ELSE
            IF (IERR.EQ.20) THEN
	       CALL QUIT_VAXPLOT
	    ELSE
               ERROR_OCCURED = .TRUE.
               CALL VXPERROR (E_XYREAD, IERR)
	    END IF
         END IF

      ELSE IF (INDEX(OPTION, 'OVERLAY ').NE.0) THEN
             
         ! overlay another data file and add to plot list in
         ! active window

         IF (GW(NW).NPLOTS.EQ.MAX_PLOTS) THEN
            IERR = 1
            CALL VXPERROR (E_OPTION_SELECT, IERR)
            ERROR_OCCURED = .TRUE.
         ELSE
            CALL FILE_READ (JSET, IERR)
            IF (IERR.EQ.0) THEN
               NP = GW(NW).NPLOTS + 1
               GW(NW).NPLOTS = NP
               GW(NW).ACTIVE_PLOT = NP
               GW(NW).P(NP).SET = JSET
               CALL SET_PLOT_ATTRIBUTE (NP, NW)
            ELSE
               IF (IERR.EQ.20) THEN
		  TYPE *, 'OVERLAY cancelled.'
	       ELSE
                  ERROR_OCCURED = .TRUE.
                  CALL VXPERROR (E_XYREAD, IERR)
	       END IF	
            END IF
         END IF

      ELSE IF (INDEX(OPTION, 'DROP ').NE.0) THEN

         ! drop active plot in active window

         CALL DROP_PLOT (NP, NW)
         IF (GW(NW).NPLOTS.EQ.0) THEN
            RESET_EXTREMA = .FALSE.
            RE_DRAW = .FALSE.
         ELSE
            RESET_EXTREMA = .TRUE.
            RE_DRAW = .TRUE.
         END IF

      ELSE IF (INDEX(OPTION, 'REMOVE ').NE.0) THEN

         CALL REMOVE_DATA_SET (NS)

      ELSE IF (INDEX(OPTION, 'INCLUDE ').NE.0) THEN

	 CALL INCLUDE_DATA_SET (NS)
         IF (NS.EQ.0) THEN
            RESET_EXTREMA = .FALSE.
            RE_DRAW = .FALSE.
         ELSE
            NP = GW(NW).NPLOTS + 1
            GW(NW).NPLOTS = NP
            GW(NW).ACTIVE_PLOT = NP
            GW(NW).P(NP).SET = NS
            CALL SET_PLOT_ATTRIBUTE (NP, NW)
            RESET_EXTREMA = .TRUE.
            RE_DRAW = .TRUE.
         END IF
      
      ELSE IF (INDEX(OPTION, 'SAVE ').NE.0) THEN

         CALL SAVE_DATA_SET (NS, NW, IERR)

         RESET_EXTREMA = .FALSE.
         RE_DRAW = .FALSE.
         IF (IERR.NE.0) THEN
            ERROR_OCCURED = .TRUE.
            CALL VXPERROR (E_SAVE_DATA, IERR)
         END IF

      ELSE IF (INDEX(OPTION, 'EXPAND ').NE.0) THEN

         ! open new graphics window with expanded limits
            
	 IF (EX.FLAG) THEN
	    XMIN = EX.XMIN       
	    XMAX = EX.XMAX
	    YMIN = EX.YMIN
	    YMAX = EX.YMAX
	    EX.FLAG = .FALSE.
         ELSE
	    XMIN = GW(NW).XMIN
	    XMAX = GW(NW).XMAX
	    YMIN = GW(NW).YMIN
	    YMAX = GW(NW).YMAX
            TYPE *, 'ENTER XMIN, YMIN, XMAX, YMAX: '
            READ(5, *) XMIN, YMIN, XMAX, YMAX
         END IF

	 ! Use width and height of parent window

	 CALL UIS$GET_VIEWPORT_SIZE (GW(NW).WD_ID, WIDTH, HEIGHT)
	
         CALL OPEN_GRAPHICS_WINDOW (XMIN, YMIN, XMAX, YMAX,
     +       HEIGHT, WIDTH, 'EXPANSION OF '//
     +       GW(NW).TITLE, JW, IERR)
            
         IF (IERR.EQ.0) THEN

            DO J = 1, GW(NW).NPLOTS
               GW(JW).P(J) = GW(NW).P(J)
            END DO

            GW(JW).NPLOTS = GW(NW).NPLOTS
            GW(JW).ACTIVE_PLOT = GW(NW).ACTIVE_PLOT
            GW(JW).FREEZE = .TRUE.

            NW = JW

            RESET_EXTREMA = .FALSE.
            RE_DRAW = .TRUE.
         ELSE
            CALL VXPERROR (E_OPEN_GW, IERR)
            ERROR_OCCURED = .TRUE.
         END IF                            

      ELSE IF (INDEX(OPTION, 'RESCALE ').NE.0) THEN

         CALL V_RESCALE (NS, IERR)

         IF (IERR.EQ.0) THEN
            SET_ALTERED = .TRUE.
         ELSE
            ERROR_OCCURED = .TRUE.
            CALL VXPERROR (E_RESCALE, IERR)
         END IF
      
      ELSE IF (INDEX(OPTION, 'SMOOTH ').NE.0) THEN

         CALL V_SMOOTH (NS)
         SET_ALTERED = .TRUE.

      ELSE IF (INDEX(OPTION, 'POLY ').NE.0) THEN

         CALL V_POLYFIT (NW, NS, IERR)

         IF (IERR.NE.0) THEN
            ERROR_OCCURED = .TRUE.
            CALL VXPERROR(E_V_POLYFIT, IERR)
         ELSE

            ! add fitted function to plot list

            IF (GW(NW).NPLOTS.EQ.MAX_PLOTS) THEN
               IERR = 1
               CALL VXPERROR (E_OPTION_SELECT, IERR)
               ERROR_OCCURED = .TRUE.
            ELSE
               NP = GW(NW).NPLOTS + 1
               GW(NW).NPLOTS = NP
               GW(NW).ACTIVE_PLOT = NP
               GW(NW).P(NP).SET = DB.NSETS
               CALL SET_PLOT_ATTRIBUTE (NP, NW)
            END IF
         END IF

      ELSE IF (INDEX(OPTION, 'AREA ').NE.0) THEN

	 CALL V_AREA (NW, NS, IERR)

	 IF (IERR.NE.0) THEN
	    ERROR_OCCURED = .TRUE.
	    CALL VXPERROR (E_V_AREA, IERR)
	 END IF

	 RE_DRAW = .FALSE.
	 RESET_EXTREMA = .FALSE.

      ELSE IF (INDEX(OPTION, 'DERIV ').NE.0) THEN

	 CALL V_DERIVATIVE (NW, NS, IERR)

	 IF (IERR.NE.0) THEN
	    ERROR_OCCURED = .TRUE.
	    CALL VXPERROR (E_V_DERIV, IERR)
	 ELSE

            ! add derivative to plot list

            IF (GW(NW).NPLOTS.EQ.MAX_PLOTS) THEN
               IERR = 1
               CALL VXPERROR (E_OPTION_SELECT, IERR)
               ERROR_OCCURED = .TRUE.
	       RE_DRAW = .FALSE.
            ELSE
               NP = GW(NW).NPLOTS + 1
               GW(NW).NPLOTS = NP
               GW(NW).ACTIVE_PLOT = NP
               GW(NW).P(NP).SET = DB.NSETS
               CALL SET_PLOT_ATTRIBUTE (NP, NW)
	       RESET_EXTREMA = .TRUE.
            END IF
         END IF
	    	
      ELSE IF (INDEX(OPTION, 'HEADER ').NE.0) THEN

         CALL DISPLAY_HEADER (NS)
         RESET_EXTREMA = .FALSE.
         RE_DRAW = .FALSE.
      
      ELSE IF (INDEX(OPTION, 'UN-FREEZE').NE.0) THEN

         GW(NW).FREEZE = .FALSE.
         TYPE *, 'Window limits will be automatically set.'
         RESET_EXTREMA = .TRUE.
         RE_DRAW = .TRUE.

      ELSE IF (INDEX(OPTION, ' FREEZE ').NE.0) THEN

         GW(NW).FREEZE = .TRUE.
         TYPE *, 'Window limits frozen.'
         RESET_EXTREMA = .FALSE.
         RE_DRAW = .FALSE.

      ELSE IF (INDEX(OPTION, 'POINT ').NE.0) THEN

         ! modify plotting symbol of current plot to a point

         GW(NW).P(NP).SYMBOL = 'P'
         RESET_EXTREMA = .FALSE.

      ELSE IF (INDEX(OPTION, 'LINE ').NE.0) THEN

         ! modify plotting symbol of active plot to a line
                                                                  
         GW(NW).P(NP).SYMBOL = 'L'
         RESET_EXTREMA = .FALSE.

      ELSE IF (INDEX(OPTION, 'STICK ').NE.0) THEN

         ! modify plotting symbol of active plot to a vertical stick
                                                                  
         GW(NW).P(NP).SYMBOL = 'S'
         RESET_EXTREMA = .FALSE.

      ELSE IF (INDEX(OPTION, 'OPEN ').NE.0) THEN

          ! read another file and put in new graphics window

          CALL FILE_READ (JSET, IERR)
          IF (IERR.EQ.0) THEN
             XMIN = DS(JSET).XMIN
	     XMAX = DS(JSET).XMAX
             YMIN = DS(JSET).YMIN
             YMAX = DS(JSET).YMAX
             CALL OPEN_GRAPHICS_WINDOW (XMIN, YMIN, XMAX, YMAX,
     +           WINDOW_HEIGHT, WINDOW_WIDTH, DS(JSET).FILE_NAME,
     +           NW, IERR)
             IF (IERR.EQ.0) THEN
                GW(NW).NPLOTS = 1
                GW(NW).ACTIVE_PLOT = 1
                GW(NW).P(1).SET = JSET
                CALL SET_PLOT_ATTRIBUTE (1, NW)
                RESET_EXTREMA = .FALSE.
                RE_DRAW = .TRUE.
             ELSE
                   CALL VXPERROR (E_OPEN_GW, IERR)
                   ERROR_OCCURED = .TRUE.
             END IF                       
          ELSE
	     IF (IERR.EQ.20) THEN
	        TYPE *, 'OPEN cancelled.'
	     ELSE                                 
                ERROR_OCCURED = .TRUE.
                CALL VXPERROR (E_XYREAD, IERR)
	     END IF
          END IF

      ELSE IF (INDEX(OPTION, 'QUIT ').NE.0) THEN

         CALL QUIT_VAXPLOT

      ELSE IF (INDEX(OPTION, 'HELP ').NE.0) THEN

         CALL VAXPLOT_HELP
         RESET_EXTREMA = .FALSE.
         RE_DRAW = .FALSE.

      ELSE

         ! unrecognized command

         TYPE *,'UNRECOGNIZED COMMAND -- Type HELP for command list.'
         ERROR_OCCURED = .TRUE.

      END IF
C
C =========================== END SELECTION ===========================
C
C
C     Are all graphics windows closed? If so, start over.
C
      IF (N_WINDOWS_OPEN().EQ.0) THEN
         OPTION = ' NEW '
         GO TO 1000
      END IF

      NP = GW(NW).ACTIVE_PLOT    ! reset active plot 
      NS = GW(NW).P(NP).SET      ! reset active data set
C
C     Has an error occured? If so, return to calling module without
C       doing anything.
C
      IF (.NOT.ERROR_OCCURED) THEN

         IF (NW.EQ.0) THEN
            RESET_EXTREMA = .FALSE.
            RE_DRAW = .FALSE.
         END IF
C                                     
C        Has a data set been altered? If so, redraw all windows
C          having plots associated with that data set.
C
         IF (SET_ALTERED) THEN
            CALL REDRAW_WINDOWS_WITH_SET (NS)
            RETURN
         END IF

         IF (RESET_EXTREMA) THEN
            CALL WINDOW_EXTREMA (NW, XMIN, YMIN, XMAX, YMAX, IERR)
            IF (IERR.EQ.0) THEN
               CALL RESET_WINDOW_LIMITS (NW, XMIN, YMIN, XMAX, YMAX)
            ELSE
               CALL VXPERROR (E_WINDOW_EXT, IERR)
               RE_DRAW = .FALSE.
            END IF
         END IF

         IF (RE_DRAW) THEN
               CALL DRAW_WINDOW (NW)                       
         END IF

      END IF
      
      RETURN
      END
C
C -------------------------------------------------------------------
C
C     PLOT_LIST
C
C     Purpose: Show symoblic plot list, highlighting the active plot.
C     Syntax: CALL PLOT_LIST (NW)
C     Input Parameters:
C       NW -- window number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 5 Feb 90
C     Notes: Attribute block 20 is reserved for use in this module.
C
      	SUBROUTINE PLOT_LIST (NW)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'

      	INCLUDE 'SYS$LIBRARY:UISENTRY'
      	INCLUDE 'SYS$LIBRARY:UISUSRDEF'

      	INTEGER VD_ID
      	INTEGER TR_ID
      	INTEGER ABLK
	                  
      	NPLOTS = GW(NW).NPLOTS
      	VD_ID = GW(NW).VD_ID
      	TR_ID = GW(NW).TR_ID

      	IF (NPLOTS.EQ.1) RETURN
       
      	YDEL = GW(NW).YMAX - GW(NW).YMIN
      	XDEL = GW(NW).XMAX - GW(NW).XMIN
      
      	X_SPACING = XDEL/15.
      	YLIST = GW(NW).YMAX + YDEL/15.
      	RADIUS = X_SPACING/7.
      
      	CALL UIS$ERASE(TR_ID, GW(NW).XMIN, GW(NW).YMAX + YDEL/50.,
     +     GW(NW).XMAX, YLIST + YDEL/20.)

      	DO J = 1, NPLOTS
          XLIST = GW(NW).XMIN + J*X_SPACING - RADIUS
          ABLK = GW(NW).P(J).BLOCK
                           
          CALL UIS$SET_FONT(VD_ID, ABLK, 20, 'UIS$FILL_PATTERNS')
          IF (J.EQ.GW(NW).ACTIVE_PLOT) THEN
            CALL UIS$SET_FILL_PATTERN (VD_ID, 20, 20, 
     +         PATT$C_GREY15_16)
            CALL UIS$CIRCLE (TR_ID, 20, XLIST, YLIST, RADIUS)
          ELSE
            CALL UIS$CIRCLE (TR_ID, ABLK, XLIST, YLIST, RADIUS)
          END IF
      	END DO

      	RETURN
      	END
C
C ------------------------------------------------------------------------
C
C     SET_PLOT_ATTRIBUTE
C
C     Purpose: Set the attribute block for a plot
C     Syntax: CALL SET_PLOT_ATTRIBUTE (NP, NW)
C     Input Parameters:
C       NP -- plot number
C       NW -- window number
C     Output Parameters: NONE                                  
C     Author: K. Myneni
C     Date: 5 Feb 90
C     Updated: 12 Sep 90; KM
C
      	SUBROUTINE SET_PLOT_ATTRIBUTE (NP, NW)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'

C  Attribute blocks 5 through 11 are reserved for plots 1 - 7

	DO J = 1, MAX_PLOTS
	  DO K = 1, GW(NW).NPLOTS
	    IF (GW(NW).P(K).BLOCK.EQ.(J + 4)) GO TO 100
	  END DO

C  Unused attribute block found

          GW(NW).P(NP).BLOCK = J + 4       ! set the attribute block
      	  RETURN

 100	  CONTINUE

C  Attribute block is used so loop to next one.

	END DO

C  No more unused attribute blocks; simply return

	RETURN
      	END
C     
C ----------------------------------------------------------------------
C
C     REMOVE_DATA_SET
C
C     Purpose: Remove specified data set from the data base and 
C              remove all plots attached to data set from all
C              graphics windows.
C     Syntax: CALL REMOVE_DATA_SET (NS)
C     Input Parameters:
C       NS -- data set number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 6 Feb 90
C
      SUBROUTINE REMOVE_DATA_SET (NS)

      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPDISP.INC'
      INCLUDE 'VXPDBASE.INC'

      LOGICAL RE_DRAW

      IF (NS.GT.DB.NSETS) RETURN       ! invalid data set number

      IF (NS.EQ.DB.NSETS) THEN
         DB.NPTS = DS(NS-1).JE
      ELSE
C
C        Move points down in data base array
C         
         JSS = DS(NS + 1).JS                  ! starting source index
         JSE = DB.NPTS                        ! ending source index
         JSHIFT = DS(NS).JE - DS(NS).JS + 1   ! index shift in arrays

         DO J = JSS, JSE
            JT = J - JSHIFT
            X(JT) = X(J)
            Y(JT) = Y(J)
         END DO
C
C        Update data set information
C
         DO K = NS, DB.NSETS - 1
            DS(K) = DS(K+1)
            DS(K).JS = DS(K).JS - JSHIFT
            DS(K).JE = DS(K).JE - JSHIFT
         END DO
      END IF
C
C     Update data base information
C
      DB.NSETS = DB.NSETS - 1
      DB.NPTS = DS(DB.NSETS).JE
C
C     Update all graphics windows information
C
      DO J = 1, MAX_DISPLAYS
         RE_DRAW = .FALSE.
         IF (GW(J).VD_ID.NE.0) THEN
            DO K = 1, GW(J).NPLOTS
               JSET = GW(J).P(K).SET

               IF (JSET.EQ.NS) THEN
C
C                 Drop plot associated with set NS
C
                  CALL DROP_PLOT (K, J)
                  IF (GW(J).NPLOTS.NE.0) RE_DRAW = .TRUE.

               ELSE IF (JSET.GT.NS) THEN
C
C                 All higher set numbers need to be decremented
C
                  GW(J).P(K).SET = JSET - 1

               END IF
            END DO         
C
C           Redraw each window in which plot was dropped.
C            
            IF (RE_DRAW) THEN
               CALL WINDOW_EXTREMA (J, XMIN, YMIN, XMAX, YMAX, IERR)
               IF (IERR.EQ.0) THEN
                  CALL RESET_WINDOW_LIMITS (J, XMIN, YMIN, XMAX,
     +                YMAX)
                  CALL DRAW_WINDOW (J)
               END IF
            END IF
         END IF
      END DO

      WRITE (6, 100) NS
 100  FORMAT(' DATA SET ', I2, ' REMOVED.')
		
      RETURN
      END
C
C ---------------------------------------------------------------------
C
C     V_RESCALE
C
C     Purpose: Interface VAXPLOT with RESCALE routine.
C     Syntax: CALL V_RESCALE (NS, IERR)
C     Input Parameters:
C       NS -- data set number.
C     Output Parameters:
C       IERR -- error code returned by module RESCALE
C     Author: K. Myneni
C     Date: 12 Feb 90
C     Modified: 20 Aug 90; use IERR in arg list -- KM
C
      	SUBROUTINE V_RESCALE (NS, IERR)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDBASE.INC'
      
      	CHARACTER*20 RESCALE_EXPRESSION
      	CHARACTER*80 CLINE

      	CALL FILL_STRING (RESCALE_EXPRESSION, ' ')
      	CALL FILL_STRING (CLINE, ' ')
      	JS = DS(NS).JS
      	JE = DS(NS).JE
      
      	WRITE (6, 100)
 100  	FORMAT(' RESCALE >> '$)
      	READ (5, 200) RESCALE_EXPRESSION
 200  	FORMAT(A)

      	CALL RESCALE (X, Y, JS, JE, RESCALE_EXPRESSION, IERR)

      	IF (IERR.NE.0) RETURN

      	CALL DATA_SET_EXTREMA (NS)
      	CLINE(1:) = '==VAXPLOT==> RESCALE '//RESCALE_EXPRESSION//
     +     CHAR(13)//CHAR(10)//CHAR(0)
      	CALL ADD_NTSTRINGS (DS(NS).HEADER, CLINE, K)
      	TYPE *, 'RESCALED.'

      	RETURN
      	END
C
C -----------------------------------------------------------------------
C
C     REDRAW_WINDOWS_WITH_SET
C                                                                        
C     Purpose: Redraw all windows having plots associated with
C              specified data set.
C     Syntax: CALL REDRAW_WINDOWS_WITH_SET (NS)
C     Input Parameters:
C       NS -- data set number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 12 Feb 90
C     
      SUBROUTINE REDRAW_WINDOWS_WITH_SET (NS)

      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPDISP.INC'

      LOGICAL REDRAW_WINDOW 

      DO J = 1, MAX_DISPLAYS
         IF (GW(J).VD_ID.NE.0) THEN
	    REDRAW_WINDOW = .FALSE.
            DO K = 1, GW(J).NPLOTS
               IF (GW(J).P(K).SET.EQ.NS) THEN
                  REDRAW_WINDOW = .TRUE.
               END IF
            END DO

            IF (REDRAW_WINDOW) THEN
               CALL WINDOW_EXTREMA (J, XMIN, YMIN, XMAX, YMAX, IERR)
               IF (IERR.EQ.0) THEN     
                  CALL RESET_WINDOW_LIMITS (J, XMIN, YMIN, XMAX, YMAX)
                  CALL DRAW_WINDOW (J)
               END IF
            END IF
         END IF
      END DO
      
      RETURN
      END
C
C -----------------------------------------------------------------------
C
C     DATA_SET_LIST				
C
C     Purpose: Display list of data sets.
C     Syntax: CALL DATA_SET_LIST
C     Input Parameters: NONE
C     Output Parameters: NONE 
C     Author: K. Myneni
C     Date: 12 Feb 90
C
      	SUBROUTINE DATA_SET_LIST

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDBASE.INC'
      

      	DO J = 1, DB.NSETS

	   IRF = NRTRIM (DS(J).FILE_NAME)
	   NPTS = DS(J).JE - DS(J).JS + 1
	   WRITE (6, 10) J, DS(J).FILE_NAME(1:IRF), NPTS
 10	   FORMAT (1X, 'DATA SET ', I2, 5X, 'FILE: ', A, 5X, I5,
     +        ' points.')

      	END DO
      
      	TYPE *
      	TYPE *

      	RETURN
      	END
C
C --------------------------------------------------------------------
C
C	INCLUDE_DATA_SET
C
C	Purpose: Allow user to choose data set to be included
C	          in current graphics window.
C	Syntax: CALL INCLUDE_DATA_SET (NS)
C	Input Parameters: NONE
C	Output Parameters: 
C	  NS  -- number of data set to be included; is 0 for none
C	Author: K. Myneni
C	Date: 13 Sep 90
C
	SUBROUTINE INCLUDE_DATA_SET (NS)

	INCLUDE 'VAXPLOT.INC'
	INCLUDE 'VXPDBASE.INC'

      	CHARACTER*5 RESPONSE
      
      	CALL FILL_STRING (RESPONSE, ' ')

	CALL DATA_SET_LIST

      	WRITE (6, 100)
 100  	FORMAT(' SET NUMBER >> '$)
      	READ (5, 200) RESPONSE
 200  	FORMAT(A)
      
      	NS = INT(STRVAL(RESPONSE))

      	IF ((NS.LT.1).OR.(NS.GT.DB.NSETS)) THEN
	  NS = 0
	  TYPE *, 'INCLUDE cancelled.'
	END IF
	
	RETURN
	END	      
C
C --------------------------------------------------------------------
C
C     SAVE_DATA_SET
C
C     Purpose: Save specified data set to a file.
C     Syntax: CALL SAVE_DATA_SET (NS, NW, IERR)
C     Input Parameters:
C       NS  -- data set number
C       NW  -- window number 
C     Output Parameters:
C       IERR -- integer error code has the following meaning:
C                 0 -- no error
C                 1 -- no data in window domain for current plot
C     Author: K. Myneni
C     Date: 12 Feb 90
C     Modified: 19 July 90 -- KM
C     Modified: 16 Aug 90 -- KM
C     Modified: 20 Aug 90; add IERR to arg list -- KM
C
        SUBROUTINE SAVE_DATA_SET (NS, NW, IERR)

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDISP.INC'
        INCLUDE 'VXPDBASE.INC'

        CHARACTER*80 CLINE
        CHARACTER*80 PROMPT
        CHARACTER*20 FNAME
        CHARACTER*5 RESPONSE

        CALL FILL_STRING (CLINE, ' ')
        JS = DS(NS).JS
        JE = DS(NS).JE
        X1 = GW(NW).XMIN
	X2 = GW(NW).XMAX

        CALL INDEX_LIMITS (X, X1, X2, JS, JE, J1, J2, IERR)

        IF (IERR.NE.0) RETURN   ! no data in window domain

        IF ((J1.GT.JS).OR.(J2.LT.JE)) THEN

           ! Save only portion of data set within window domain?

           WRITE (6, 100)
           READ (5, 200) (RESPONSE(J:J), J = 1, 5)
           CALL STR_UPPER_CASE (RESPONSE)

           IF (INDEX(RESPONSE, 'Y').NE.0) THEN

              ! Save truncated data set

              CLINE(1:) = '==VAXPLOT==> TRUNCATED DATA SET'//
     +           CHAR(13)//CHAR(10)//CHAR(0)
              CALL ADD_NTSTRINGS (DS(NS).HEADER, CLINE, K)
              CALL FILL_STRING (CLINE, ' ')
           ELSE

              ! Save entire data set

              J1 = JS
              J2 = JE
           END IF
        END IF

 100    FORMAT(' Save only data in window domain? '$)
 200    FORMAT(5A1)

        CLINE(1:) = '==VAXPLOT==> SOURCE FILE IS '//DS(NS).FILE_NAME//
     +     CHAR(13)//CHAR(10)//CHAR(0)

        CALL ADD_NTSTRINGS (DS(NS).HEADER, CLINE, K)

        PROMPT(1:) = 'Enter the output filename >> '

        CALL WRITE_XYFILE (X, Y, J1, J2, DS(NS).HEADER, PROMPT)

	TYPE *, 'Data stored.'

        RETURN
        END
C
C -----------------------------------------------------------------------
C
C     V_SMOOTH
C
C     Purpose: Interface routine between VAXPLOT and SMOOTH routine.
C     Syntax: CALL V_SMOOTH (NS)
C     Input Parameters:
C       NS -- data set number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 12 Feb 90
C
      	SUBROUTINE V_SMOOTH (NS)

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDBASE.INC'
      
      	CHARACTER*80 CLINE
              
      	CALL FILL_STRING (CLINE, ' ')
      	JS = DS(NS).JS
      	JE = DS(NS).JE

      	CALL SMOOTH (X, Y, JS, JE)
      
      	CALL DATA_SET_EXTREMA (NS)

      	CLINE(1:) = '==VAXPLOT==> SMOOTHED '//CHAR(13)//CHAR(10)//
     +     CHAR(0)

      	CALL ADD_NTSTRINGS (DS(NS).HEADER, CLINE, K)
      	TYPE *, 'SMOOTHED.'

      	RETURN
      	END
C
C --------------------------------------------------------------------
C		
C     N_WINDOWS_OPEN
C
C     Purpose: Determine how many graphics windows are open.
C     Syntax: N = N_WINDOWS_OPEN ()
C     Input Parameters: NONE
C     Returns:
C       -- number of open windows
C     Author: K. Myneni
C     Date: 13 Feb 90
C
      	INTEGER FUNCTION N_WINDOWS_OPEN ()

      	INCLUDE 'VAXPLOT.INC'
      	INCLUDE 'VXPDISP.INC'
      
      	N = 0

      	DO J = 1, MAX_DISPLAYS
	   IF (GW(J).VD_ID.NE.0) N = N + 1
      	END DO

      	N_WINDOWS_OPEN = N

      	RETURN
      	END
C
C ----------------------------------------------------------------------
C
C     QUIT_VAXPLOT
C
C     Purpose: Prompt user to exit program.
C     Syntax: CALL QUIT_VAXPLOT
C     Input Parameters: NONE                                             
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 13 Feb 90
C
      	SUBROUTINE QUIT_VAXPLOT
      
      	CHARACTER*5 RESPONSE

      	CALL FILL_STRING (RESPONSE, ' ')
      
      	WRITE(6, 100)
      	READ(5, 200)RESPONSE
 100  	FORMAT(' Quit? '$)
 200  	FORMAT(A)
      	CALL STR_UPPER_CASE (RESPONSE)

      	IF (INDEX(RESPONSE, 'Y').EQ.0) RETURN

      	CALL CLOSE_ALL_WINDOWS
      	TYPE *, 'Exiting VAXPLOT.'
      	CALL EXIT                        ! normal program termination

      	END
C
C -------------------------------------------------------------------
C
C     READ_PARAMETERS
C
C     Purpose: Read a vaxplot parameter file to override default	
C              values of window dimensions, colors, etc.
C     Syntax: CALL READ_PARAMETERS
C     Input Parameters: NONE
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 14 Feb 90
C     Updated: 12 Sep 90; routine was READ_COLORS formerly -- KM
C     Notes: The symbol VAXPLOT_INI_FILE should be defined with
C            the DCL DEFINE command to the full path and file name
C            of the parameter data file.
C
        SUBROUTINE READ_PARAMETERS

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDISP.INC'
      
        CHARACTER*80 CLINE
	REAL REDS(7), GREENS(7), BLUES(7)

        ILU = NEXT_AVAILABLE_UNIT()
        OPEN (UNIT = ILU, FILE = 'VAXPLOT_INI_FILE', READONLY,
     +     ERR = 1000, STATUS = 'OLD')

        DO WHILE (.TRUE.)

          READ (ILU, 100, END = 500) (CLINE(J:J), J = 1, 80)
 100      FORMAT(80A1)
          IF (CLINE(1:1).NE.'!') THEN
             CALL STR_UPPER_CASE (CLINE)
	     IF (INDEX(CLINE, 'WIDTH').NE.0) THEN
	        J = INDEX(CLINE, '=')
	        IF (J.NE.0) THEN
	           WINDOW_WIDTH = STRVAL(CLINE(J + 1:))
 110		   FORMAT (F)
	        END IF
	     ELSE IF (INDEX(CLINE, 'HEIGHT').NE.0) THEN
	        J = INDEX(CLINE, '=')
	        IF (J.NE.0) THEN
	           WINDOW_HEIGHT = STRVAL(CLINE(J + 1:))
	        END IF
	     ELSE IF (INDEX(CLINE, 'GRAPH').NE.0) THEN
	        IF (INDEX(CLINE, 'BACK').NE.0) THEN
	           J = INDEX(CLINE, '=')
	           IF (J.NE.0) THEN
	              DECODE (16, 120, CLINE(J + 1:))
     +                   R_MAP(1), G_MAP(1), B_MAP(1)
 120	              FORMAT(3F)
	           END IF
	        ELSE IF (INDEX(CLINE, 'FORE').NE.0) THEN
	           J = INDEX(CLINE, '=')
	           IF (J.NE.0) THEN
		      DECODE (16, 120, CLINE(J + 1:))
     +		         R_MAP(2), G_MAP(2), B_MAP(2)
	           END IF
	        END IF
	     ELSE IF (INDEX(CLINE, 'MENU').NE.0) THEN
	        IF (INDEX(CLINE, 'BACK').NE.0) THEN
	           J = INDEX(CLINE, '=')
	           IF (J.NE.0) THEN
	              DECODE (16, 120, CLINE(J + 1:))
     +			 R_MAP(3), G_MAP(3), B_MAP(3)
	           END IF
	        ELSE IF (INDEX(CLINE, 'FORE').NE.0) THEN
	           J = INDEX(CLINE, '=')
	           IF (J.NE.0) THEN
	              DECODE (16, 120, CLINE(J + 1:))
     +			 R_MAP(4), G_MAP(4), B_MAP(4)
	           END IF
		END IF
	     ELSE IF (INDEX(CLINE, 'PLOT').NE.0) THEN
	        IF (INDEX(CLINE, 'RED').NE.0) THEN
	           J = INDEX(CLINE, '=')
	           IF (J.NE.0) THEN
		      DECODE (40, 130, CLINE(J + 1:)) REDS 
 130		      FORMAT(7F)
		      DO J = 1, 7
			 R_MAP(J + 4) = REDS(J)
		      END DO
		   END IF
		ELSE IF (INDEX(CLINE, 'GREEN').NE.0) THEN
		   J = INDEX(CLINE, '=')
		   IF (J.NE.0) THEN
		      DECODE (40, 130, CLINE(J + 1:)) GREENS
		      DO J = 1, 7
			 G_MAP(J + 4) = GREENS(J)
		      END DO
		   END IF
		ELSE IF (INDEX(CLINE, 'BLUE').NE.0) THEN
		   J = INDEX(CLINE, '=')
		   IF (J.NE.0) THEN
		      DECODE (40, 130, CLINE(J + 1:)) BLUES
		      DO J = 1, 7
			 B_MAP(J + 4) = BLUES(J)
		      END DO
		   END IF				
	        END IF
	     END IF
          END IF
        END DO

 500    CONTINUE

        CLOSE (ILU)

 1000   CONTINUE
C
C     Parameter data file not found, so just return.
C
        RETURN
        END
C
C --------------------------------------------------------------------
C                                         
C     INIT_WINDOW_PARAMETERS
C
C     Purpose: Initialize graphics window record.
C     Syntax: CALL INIT_WINDOW_PARAMETERS (NW)
C     Input Parameters:
C       NW -- window number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 15 Feb 90
C
        SUBROUTINE INIT_WINDOW_PARAMETERS (NW)

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDISP.INC'


	WINDOW_WIDTH = DEFAULT_WIDTH
	WINDOW_HEIGHT = DEFAULT_HEIGHT

        GW(NW).VD_ID = 0
        GW(NW).TR_ID = 0
        GW(NW).WD_ID = 0
        GW(NW).VCM_ID = 0
        GW(NW).FREEZE = .FALSE.
        GW(NW).SCALE = 0
        GW(NW).NPLOTS = 0

        DO J = 1, MAX_PLOTS
           GW(NW).P(J).SET = 0
           GW(NW).P(J).BLOCK = 0
           GW(NW).P(J).SYMBOL = 'L'
        END DO           

        RETURN
        END
C
C --------------------------------------------------------------------
C
C       V_POLYFIT
C
C       Purpose: Interface routine for polynomial fitting module,
C                POLFIT
C       Syntax: CALL V_POLYFIT (NW, NS, IERR)
C       Input Parameters:
C         NW -- window number
C         NS -- data set number
C       Output Parameters:
C         IERR -- error code has the following meaning
C                   0 -- no error
C                   1 -- no data in window domain for current plot
C                   2 -- insufficient number of points for fit
C                   3 -- number of terms out of range
C                   4 -- no room in data base for fitted function
C       Author: K. Myneni
C       Date: 18 Aug 90
C
        SUBROUTINE V_POLYFIT (NW, NS, IERR)

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDBASE.INC'
        INCLUDE 'VXPDISP.INC'

        DIMENSION A(MAX_TERMS)
	DIMENSION AP(MAX_TERMS)
        CHARACTER*(HEADER_LENGTH) HDR
	LOGICAL X_TRANSFORMED
      
        IERR = 0

        JS = DS(NS).JS
        JE = DS(NS).JE
        X1 = GW(NW).XMIN
        X2 = GW(NW).XMAX

        ! Determine interval over which to fit data

        CALL INDEX_LIMITS (X, X1, X2, JS, JE, J1, J2, IERR)

        IF (IERR.NE.0) RETURN           ! no data in window

        WRITE (6, 100)
        READ (5, *) NORDER

 100    FORMAT (' Enter the order of the fitting polynomial: '$)

        NTERMS = NORDER + 1

        IF ((NTERMS.LT.1).OR.(NTERMS.GT.MAX_TERMS)) THEN
           IERR = 3                     ! number of terms out of range
           RETURN
        END IF

        NPTS = J2 - J1 + 1

        IF (NPTS.LT.NTERMS) THEN
           IERR = 2                     ! insufficient number of points
           RETURN
        END IF

	XMIN = X1                       ! set defaults for fitted function
	XMAX = X2
	DX = 0.

        WRITE (6, 200)
        READ (5, *) XMIN, XMAX, DX

 200    FORMAT (' Enter XMIN, XMAX, and DELTA X for fitted function: '$)

C  Will there be a singularity problem in the calculation?

	S = ABS(XMAX / (XMAX - XMIN))

	IF (S.GT.1.) THEN

C  Perform a linear transformation to bring x values within
C  interval [-2, 2].

	  CALL LINEAR_TRANSFORMATION (X, JS, JE, -2., 2., C1, C2)

	  X_TRANSFORMED = .TRUE.
	ELSE
	  X_TRANSFORMED = .FALSE.
	END IF

C       Perform fit

        CALL POLFIT (X, Y, J1, J2, NTERMS, AP, CHISQR)

C  Compute fitted function

        JSP = DB.NPTS + 1

        IF (DX.EQ.0.) THEN

           ! Compute function at same x values as data

           DO J = J1, J2
              XX = X(J)
              JN = JSP + J - J1
              X(JN) = XX
              Y(JN) = HORNER(AP, NTERMS, XX)
           END DO

        ELSE

           ! Compute function over specified interval

           NPTS = INT((XMAX - XMIN)/DX) + 1
           DO J = 1, NPTS
              JN = JSP + J - 1
              XX = XMIN + (J - 1)*DX
              X(JN) = XX
              Y(JN) = HORNER(AP, NTERMS, XX)
           END DO
        END IF

        JEP = JN

C  Undo the transformation of x values.

	IF (X_TRANSFORMED) THEN
	  DO J = JS, JE
	    X(J) = (X(J) - C2)/C1
	  END DO
	  DO J = JSP, JEP
	    X(J) = (X(J) - C2)/C1
	  END DO
	END IF	

C  Obtain coefficients in normal space.

	IF (X_TRANSFORMED) THEN
	   CALL TRANSFORM_COEFFICIENTS (A, AP, NORDER, C1, C2)
	ELSE
	   DO J = 1, NTERMS
	     A(J) = AP(J)
	   END DO
	END IF

C  Create new data set in data base containing fitted polynomial

        NNS = DB.NSETS + 1              ! new data set number

        IF (NNS.GT.MAX_DATA_SETS) THEN
           IERR = 4                     ! exceeded max number of data sets
           RETURN
        ELSE
           DB.NSETS = NNS
        END IF

        DS(NNS).JS = JSP
        DS(NNS).JE = JEP
        DB.NPTS = JEP

        CALL DATA_SET_EXTREMA (NNS)

        DS(NNS).FILE_NAME = 'POLYNOMIAL_FIT'

C       Create header

	CALL FILL_STRING(HDR, ' ')
	LF = NRTRIM(DS(NS).FILE_NAME)
	XMIN = X(J1)                    
	XMAX = X(J2)
	IF (XMAX.LT.XMIN) THEN
	   XSAVE = XMIN
	   XMIN = XMAX
	   XMAX = XSAVE
	END IF
	
	HDR(1:) = '==VAXPLOT==> POLYNOMIAL FIT OF ORDER '
	L = NRTRIM(HDR) + 1
	ENCODE (4, 300, HDR(L:)) NORDER
 300	FORMAT (I2)
	L = NRTRIM(HDR) + 1
	HDR(L:) = ' TO ' // DS(NS).FILE_NAME(1:LF) //
     +     CHAR(13)//CHAR(10)//
     +	   '==VAXPLOT==> DOMAIN OF FIT IS ['

	L = NRTRIM(HDR) + 1

	ENCODE (45, 350, HDR(L:)) XMIN, XMAX
 350	FORMAT (F, ', ', F)
 400	FORMAT (F)

	L = NRTRIM(HDR) + 1

	HDR(L:) = ']'//CHAR(13)//CHAR(10)

	DO J = 1, NORDER + 1
	   L = NRTRIM(HDR) + 1
	   HDR(L:) = '==VAXPLOT==> '
	   L = NRTRIM(HDR) + 2
	   ENCODE (72, 500, HDR(L:)) J - 1, A(J)
 500	   FORMAT (3X, I2, 3X, E)
	   L = NRTRIM(HDR) + 1
	   HDR(L:) = CHAR(13)//CHAR(10)
	END DO

	L = NRTRIM(HDR) + 1
	HDR(L:) = '==VAXPLOT==> CHISQR = '
	L = NRTRIM(HDR) + 2
	ENCODE (20, 400, HDR(L:)) CHISQR

	L = NRTRIM(HDR) + 1
	HDR(L:) = CHAR(13)//CHAR(10)//CHAR(0)

	DS(NNS).HEADER = HDR

	CALL DISPLAY_HEADER (NNS)	

        RETURN
        END
C
C --------------------------------------------------------------------
C
C	V_AREA
C
C	Purpose: Interface routine between VAXPLOT and PARAB_AREA,
C	         area computing routine.
C	Syntax: CALL V_AREA (NW, NS, IERR)
C	Input Parameters:
C	  NW -- active window number
C	  NS -- data set number of active plot
C	Output Parameters:
C	  IERR -- integer error code has the following meaning:
C		    0  --  no error
C		    1  --  no data in window domain for plot
C		    2  --  only 1 point in window domain
C	Author: K. Myneni
C	Date: 29 Aug 90
C
	SUBROUTINE V_AREA (NW, NS, IERR)

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDBASE.INC'
        INCLUDE 'VXPDISP.INC'

	DOUBLE PRECISION AREA

        IERR = 0

        JS = DS(NS).JS
        JE = DS(NS).JE
        X1 = GW(NW).XMIN
        X2 = GW(NW).XMAX

        ! Determine interval over which to compute area

        CALL INDEX_LIMITS (X, X1, X2, JS, JE, J1, J2, IERR)

        IF (IERR.NE.0) RETURN           ! no data in window
	
	IF (J2.EQ.J1) THEN
	   IERR = 2                     ! only one point in window
	   RETURN
	END IF

		
        IF ((J2 - J1).EQ.1) THEN

	   ! Use trapezoid rule to compute area

	   AREA = DBLE((Y(J2) + Y(J1))*ABS(X(J2) - X(J1))/2.)

	ELSE

	   ! Use parabolic area finding routine

	   CALL PARAB_AREA (X, Y, J1, J2, AREA)

	END IF

	
	XMIN = X(J1)
	XMAX = X(J2)
	IF (XMAX.LT.XMIN) THEN
	   XSAVE = XMIN
	   XMIN = XMAX
	   XMAX = XSAVE
	END IF

	WRITE (6, 100) XMIN, XMAX, SNGL(AREA)
 100	FORMAT (' AREA(', F, ',', F, ') = ', F)

	RETURN
	END
C
C ------------------------------------------------------------------
C
C	V_DERIVATIVE
C
C	Purpose: Interface routine between VAXPLOT and DERIVATIVE
C		   routine.
C	Syntax: CALL V_DERIVATIVE (NW, NS, IERR)
C	Input Parameters:
C	  NW -- active window number
C	  NS -- data set number
C	Output Parameters:
C	  IERR -- integer error code has the following meaning:
C	            0 -- no error
C	            1 -- no data in window domain
C	            2 -- only 1 point in window domain
C	            3 -- two points have the same x value
C		    4 -- exceeding maximum number of data sets
C	Author: K. Myneni
C	Date: 31 Aug 90
C
	SUBROUTINE V_DERIVATIVE (NW, NS, IERR)

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDBASE.INC'
        INCLUDE 'VXPDISP.INC'

	IERR = 0
        JS = DS(NS).JS
        JE = DS(NS).JE
        X1 = GW(NW).XMIN
        X2 = GW(NW).XMAX

        ! Determine interval over which to compute derivative

        CALL INDEX_LIMITS (X, X1, X2, JS, JE, J1, J2, IERR)

        IF (IERR.NE.0) RETURN           ! no data in window
	
	IF (J2.EQ.J1) THEN
	   IERR = 2                     ! only one point in window
	   RETURN
	END IF

	J3 = DB.NPTS + 1

	CALL DERIVATIVE (X, Y, J1, J2, J3, IERR)

	IF (IERR.NE.0) THEN
	   IERR = 3               ! derivative routine returns error
	   RETURN
	END IF

	! Add new data set to data base.

	NNS = DB.NSETS + 1

        IF (NNS.GT.MAX_DATA_SETS) THEN
           IERR = 4                     ! exceeded max number of data sets
           RETURN
        ELSE
           DB.NSETS = NNS
        END IF

	DS(NNS).JS = J3
	JE = J3 + J2 - J1
	DS(NNS).JE = JE
	DB.NPTS = JE

        CALL DATA_SET_EXTREMA (NNS)

        DS(NNS).FILE_NAME = 'DERIVATIVE'
	XMIN = X(J1)
	XMAX = X(J2)
	IF (XMAX.LT.XMIN) THEN
	   XSAVE = XMIN
	   XMIN = XMAX
	   XMAX = XSAVE
	END IF

C       Create header

	LF = NRTRIM(DS(NS).FILE_NAME)
	DS(NNS).HEADER(1:) = '==VAXPLOT==> DERIVATIVE OF '//
     +     DS(NS).FILE_NAME(1:LF)//CHAR(13)//CHAR(10)//
     +     '==VAXPLOT==>   OVER INTERVAL ['

	L = NRTRIM(DS(NNS).HEADER) + 1

	ENCODE (45, 100, DS(NNS).HEADER(L:)) XMIN, XMAX
 100	FORMAT (F, ', ', F)

	L = NRTRIM(DS(NNS).HEADER) + 1

	DS(NNS).HEADER(L:) = ']'//CHAR(13)//CHAR(10)//CHAR(0)

	RETURN
	END
C
C ---------------------------------------------------------------------
C		
C	AST_RESIZE
C
C	Purpose: Resize the plotting area to take up default
C	         area of graphics window after window has been
C		 resized.
C	Syntax: NONE
C	Input Parameters: NONE
C	Output Parameters: NONE
C	Author: K. Myneni
C	Date: 7 Sep 90
C
	SUBROUTINE AST_RESIZE 

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDISP.INC'

        INCLUDE 'SYS$LIBRARY:UISENTRY'
        INCLUDE 'SYS$LIBRARY:UISUSRDEF'

	INTEGER VD_ID, WD_ID
	RECORD /WINDOW_PLACEMENT/ WP

	ENTRY RE1
	  NW = 1
	  GO TO 100
	ENTRY RE2
	  NW = 2
	  GO TO 100
	ENTRY RE3
	  NW = 3
	  GO TO 100
	ENTRY RE4
	  NW = 4
	  GO TO 100
	ENTRY RE5
	  NW = 5
	  GO TO 100
	ENTRY RE6
	  NW = 6
	  GO TO 100
	ENTRY RE7
	  NW = 7
	  GO TO 100

 100	CONTINUE

	VD_ID = GW(NW).VD_ID
        WD_ID = GW(NW).WD_ID

	CALL UIS$RESIZE_WINDOW (VD_ID, WD_ID)
	CALL UIS$GET_VIEWPORT_SIZE (WD_ID, WIDTH, HEIGHT)
	CALL UIS$GET_VIEWPORT_POSITION (WD_ID, RX, RY)
	CALL UIS$DELETE_WINDOW (WD_ID)

C  Create new window at the same position as the old one.

	WP.CODE_1 = WDPL$C_ABS_POS_X
	WP.ATTRIB_1 = RX
	WP.CODE_2 = WDPL$C_ABS_POS_Y
	WP.ATTRIB_2 = RY
	WP.CODE_3 = WDPL$C_ATTRIBUTES
	WP.ATTRIB_3 = 0
	WP.END = WDPL$C_END_OF_LIST
	
	ITR = NRTRIM(GW(NW).TITLE)

      	WD_ID = UIS$CREATE_WINDOW (VD_ID, 'SYS$WORKSTATION',
     +     GW(NW).TITLE(1:ITR),,,,, WIDTH, HEIGHT, WP)

	GW(NW).WD_ID = WD_ID
	CALL SET_AST_ROUTINES (NW)

C  Redraw window

	CALL DRAW_WINDOW (NW)

	RETURN
	END
C
C --------------------------------------------------------------------
C
C	SET_AST_ROUTINES
C
C	Purpose: Set the AST routines for specified window.
C	Syntax: CALL SET_AST_ROUTINES (NW)
C	Input Parameters:
C	  NW -- window number
C	Output Parameters: NONE
C	Author: K. Myneni
C	Date: 7 Sep 90
C
	SUBROUTINE SET_AST_ROUTINES (NW)

        INCLUDE 'VAXPLOT.INC'
        INCLUDE 'VXPDISP.INC'

        INCLUDE 'SYS$LIBRARY:UISENTRY'
        INCLUDE 'SYS$LIBRARY:UISUSRDEF'
                               

        INTEGER VD_ID, WD_ID

        EXTERNAL BU1, BU2, BU3, BU4, BU5, BU6, BU7
        EXTERNAL CW1, CW2, CW3, CW4, CW5, CW6, CW7
        EXTERNAL OP1, OP2, OP3, OP4, OP5, OP6, OP7
	EXTERNAL RE1, RE2, RE3, RE4, RE5, RE6, RE7
        EXTERNAL FIG1, FIG2, FIG3, FIG4, FIG5, FIG6, FIG7

	VD_ID = GW(NW).VD_ID
	WD_ID = GW(NW).WD_ID

        XC = (GW(NW).XMAX + GW(NW).XMIN)/2.
        YC = (GW(NW).YMAX + GW(NW).YMIN)/2.

        CALL UIS$SET_POINTER_POSITION (VD_ID, WD_ID, XC, YC)
        CALL UIS$SET_POINTER_PATTERN (VD_ID, WD_ID, CURSOR, 2, 8, 8)
	
        GO TO (10, 20, 30, 40, 50, 60, 70) NW

 10     CALL UIS$SET_CLOSE_AST (WD_ID, CW1, 0)
        GO TO 200
 20     CALL UIS$SET_CLOSE_AST (WD_ID, CW2, 0)
        GO TO 200
 30     CALL UIS$SET_CLOSE_AST (WD_ID, CW3, 0)
        GO TO 200
 40     CALL UIS$SET_CLOSE_AST (WD_ID, CW4, 0)
        GO TO 200
 50     CALL UIS$SET_CLOSE_AST (WD_ID, CW5, 0)
        GO TO 200
 60     CALL UIS$SET_CLOSE_AST (WD_ID, CW6, 0)
        GO TO 200      
 70     CALL UIS$SET_CLOSE_AST (WD_ID, CW7, 0)
        GO TO 200

 200    CONTINUE
      
        GO TO (210, 220, 230, 240, 250, 260, 270) NW

 210    CALL UIS$SET_POINTER_AST (VD_ID, WD_ID, FIG1, 0)
        GO TO 400
 220    CALL UIS$SET_POINTER_AST (VD_ID, WD_ID, FIG2, 0)
        GO TO 400
 230    CALL UIS$SET_POINTER_AST (VD_ID, WD_ID, FIG3, 0)
        GO TO 400
 240    CALL UIS$SET_POINTER_AST (VD_ID, WD_ID, FIG4, 0)
        GO TO 400
 250    CALL UIS$SET_POINTER_AST (VD_ID, WD_ID, FIG5, 0)
        GO TO 400
 260    CALL UIS$SET_POINTER_AST (VD_ID, WD_ID, FIG6, 0)
        GO TO 400
 270    CALL UIS$SET_POINTER_AST (VD_ID, WD_ID, FIG7, 0)
        GO TO 400

 400    CONTINUE

        GO TO (510, 520, 530, 540, 550, 560, 570) NW

 510    CALL UIS$SET_ADDOPT_AST (WD_ID, OP1, 0)
        GO TO 700
 520    CALL UIS$SET_ADDOPT_AST (WD_ID, OP2, 0)
        GO TO 700
 530    CALL UIS$SET_ADDOPT_AST (WD_ID, OP3, 0)
        GO TO 700
 540    CALL UIS$SET_ADDOPT_AST (WD_ID, OP4, 0)
        GO TO 700
 550    CALL UIS$SET_ADDOPT_AST (WD_ID, OP5, 0)
        GO TO 700
 560    CALL UIS$SET_ADDOPT_AST (WD_ID, OP6, 0)
        GO TO 700
 570    CALL UIS$SET_ADDOPT_AST (WD_ID, OP7, 0)
        GO TO 700

 700    CONTINUE
      
        GO TO (810, 820, 830, 840, 850, 860, 870) NW

 810    CALL UIS$SET_BUTTON_AST (VD_ID, WD_ID, BU1, , KEYBUF)
        GO TO 1000
 820    CALL UIS$SET_BUTTON_AST (VD_ID, WD_ID, BU2, , KEYBUF)
        GO TO 1000
 830    CALL UIS$SET_BUTTON_AST (VD_ID, WD_ID, BU3, , KEYBUF)
        GO TO 1000
 840    CALL UIS$SET_BUTTON_AST (VD_ID, WD_ID, BU4, , KEYBUF)
        GO TO 1000
 850    CALL UIS$SET_BUTTON_AST (VD_ID, WD_ID, BU5, , KEYBUF)
        GO TO 1000
 860    CALL UIS$SET_BUTTON_AST (VD_ID, WD_ID, BU6, , KEYBUF)
        GO TO 1000
 870    CALL UIS$SET_BUTTON_AST (VD_ID, WD_ID, BU7, , KEYBUF)
        GO TO 1000

 1000   CONTINUE

	GO TO (1110, 1120, 1130, 1140, 1150, 1160, 1170) NW

 1110	CALL UIS$SET_RESIZE_AST (VD_ID, WD_ID, RE1, 0)
	GO TO 1200
 1120	CALL UIS$SET_RESIZE_AST (VD_ID, WD_ID, RE2, 0)
	GO TO 1200
 1130	CALL UIS$SET_RESIZE_AST (VD_ID, WD_ID, RE3, 0)
	GO TO 1200
 1140	CALL UIS$SET_RESIZE_AST (VD_ID, WD_ID, RE4, 0)
	GO TO 1200
 1150	CALL UIS$SET_RESIZE_AST (VD_ID, WD_ID, RE5, 0)
	GO TO 1200
 1160	CALL UIS$SET_RESIZE_AST (VD_ID, WD_ID, RE6, 0)
	GO TO 1200
 1170	CALL UIS$SET_RESIZE_AST (VD_ID, WD_ID, RE7, 0)
	GO TO 1200
 
 1200	CONTINUE

	RETURN
	END
C
C -------------------------------------------------------------------
C
