C   VAXPLOT.INC
C
C   Include file for VAXPLOT.FOR and associated files.                   
C
C   Created:  8 January 90
C
C   Updated:  11 Feb 90
C   Updated:  18 Aug 90; added constant MAX_TERMS -- KM
C
C ---------------------------------------------------------------------
C             CONFIGURABLE CONSTANTS
C ---------------------------------------------------------------------

      CHARACTER*(*) VERSION
      INTEGER HEADER_LENGTH

      PARAMETER (MAX_POINTS = 32768)        ! maximum points in data base
      PARAMETER (MAX_DATA_SETS = 20)        ! number of allowed data sets
      PARAMETER (MAX_PLOTS = 7)             ! number of plots in 1 display
      PARAMETER (MAX_DISPLAYS = 7)          ! number of virtual displays
      PARAMETER (MAX_TERMS = 10)            ! max terms in fitting polynomial
      PARAMETER (HEADER_LENGTH = 4096)      ! length of header text
      PARAMETER (DEFAULT_WIDTH = 25.0)      ! width of virtual display
      PARAMETER (DEFAULT_HEIGHT = 12.5)     ! height of virtual display
      PARAMETER (VERSION = '2.2')           ! current version number
      PARAMETER (Y_PLOT_AREA = 0.8)         ! percent y plot area in window
      PARAMETER (X_PLOT_AREA = 0.75)        ! percent x plot area
      PARAMETER (Y_PLOT_OFFSET = 0.1)       ! y offset in window
      PARAMETER (X_PLOT_OFFSET = 0.15)      ! x offset in window

C ---------------------------------------------------------------------
C             VAXPLOT DATA STRUCTURES
C ---------------------------------------------------------------------

      STRUCTURE /DATA_SET/
         INTEGER JS                         ! starting index in X(), Y()
         INTEGER JE                         ! ending index in X(), Y()
         REAL XMIN                          ! extrema for data set
         REAL XMAX                          !    "
         REAL YMIN                          !    "
         REAL YMAX                          !    "
         CHARACTER*20 FILE_NAME             ! data file name
         CHARACTER*(HEADER_LENGTH) HEADER   ! header text
      END STRUCTURE

      STRUCTURE /DATA_BASE/
         INTEGER NSETS                      ! number of data sets
         INTEGER NPTS                       ! total number of points
      END STRUCTURE

      STRUCTURE /PLOT/
         INTEGER SET                        ! corresponding data set
         INTEGER BLOCK                      ! attribute block number
         CHARACTER*1 SYMBOL                 ! plotting symbol (e.g. 'L')
      END STRUCTURE

      STRUCTURE /DISPLAY/
         CHARACTER*20 TITLE
         INTEGER VD_ID
         INTEGER WD_ID
         INTEGER VCM_ID
      END STRUCTURE

      STRUCTURE /GRAPHICS_WINDOW/
         CHARACTER*20 TITLE                 ! window title
         INTEGER VD_ID                      ! virtual display ID
         INTEGER WD_ID                      ! window ID
         INTEGER TR_ID                      ! transformation ID
         INTEGER VCM_ID                     ! color map ID
         REAL XMIN                          ! plotting area limits
         REAL XMAX                          !    "
         REAL YMIN                          !    "
         REAL YMAX                          !    " 
         LOGICAL FREEZE                     ! freeze window limits flag
         INTEGER SCALE                      ! scale indicator
         INTEGER NPLOTS                     ! number of plots in window
         INTEGER ACTIVE_PLOT                ! currently active plot
         RECORD /PLOT/ P (MAX_PLOTS)        ! plots information
      END STRUCTURE

      STRUCTURE /EXPANSION/
         REAL XMIN                          ! expansion corner coordinates
         REAL XMAX                          !   "
         REAL YMIN                          !   "
         REAL YMAX                          !   "
         LOGICAL FLAG                       ! Is expansion box defined?
      END STRUCTURE

C ----------------------------------------------------------------------
