# vm.asm
#
# The kForth Virtual Machine
#
# Copyright (c) 1998--2001 Krishna Myneni, Creative Consulting for
#   Research and Education
#
# This software is provided under the terms of the General Public License.
#
# Usage from C++
#
#       extern "C" int vm (byte* ip);
#       ecode = vm(ip);
#
# Originally written for the A386 assembler
# Revisions:
#	3-21-98
#	8-25-1998 ported to GNU assembler under Linux
#	9-8-1998  additional functions added: c@, c!, f=, f<>
#	9-10-1998 added: and, or, not, xor
#	9-11-1998 added: = , <>, <, >, <=, >=
#	9-15-1998 added: +!, fsqrt, fsin, fcos, -rot, sp@, rp@
#	9-17-1998 added: a@
#	9-27-1998 added: mod, 2*, 2/
#	10-1-1998 added: pick
#	10-4-1998 fixed signed integer division and mod; added /MOD
#	10-6-1998 added ?dup
#	10-14-1998 added count
#	10-16-1998 fixed L_div error
#	10-19-1998 added 0<, 0=, 0>, true, false
#	10-20-1998 added 2+, 2-
#	02-09-1999 added execute
#	03-01-1999 added open, lseek, close, read, write
#	03-02-1999 added ioctl
#	03-03-1999 added usleep	
#	03-07-1999 added fill, cmove
#	03-09-1999 interchanged meaning of execute and call
#		   to be consistent with ANS Forth
#	03-27-1999 added +loop, unloop
#	03-29-1999 added roll
#	03-31-1999 added cmove>, key
#	05-05-1999 fixed +loop
#	05-06-1999 added fround
#	05-15-1999 added floor
#	05-26-1999 added fatan2, lshift, rshift
#	05-27-1999 added u<, quit, base
#	06-02-1999 added */, */mod
#	06-09-1999 call CPP functions from vm
#	07-18-1999 added find
#	09-06-1999 added pTIB, word, tick
#	09-12-1999 added system
#	10-04-1999 added create, variable, fvariable as intrinsic words
#	10-06-1999 added constant, fconstant as intrinsic words
#	10-07-1999 added chdir
#       10-08-1999 added erase, brackettick
#	10-09-1999 added time&date, ms, question, bl
#       10-10-1999 added char, forget, cold
#       10-20-1999 added >file, console
#	10-28-1999 added key?
#	12-24-1999 added hooks for f0=, f0<, f0>, u>, s>d, d>f, f>d, 
#	              um*, um/mod, m*, m+, m/, m*/
#	12-25-1999 added cells, cell+, dfloats, dfloat+
#	12-27-1999 added bye
#	01-08-2000 fixed f0<, increased vm loop efficiency
#	01-13-1999 added ?allot, fixed f0=
#	01-22-2000 modified + to remove ordering sensitivity for address arithmetic
#	01-23-2000 added 0<>, [char], .r, u.r, changed opcodes for 
#	             relational operators
#	01-24-2000 added CPP_literal, CPP_cquote, CPP_squote, CPP_dotquote
#	02-04-2000 implemented m*, um*
#	02-26-2000 fixed fm/mod, added sm/rem, um/mod
#	03-02-2000 modified QUIT to clear the return stacks, added CPP_do
#	03-05-2000 added CPP_begin, CPP_while, CPP_repeat, CPP_until, CPP_again,
#	             CPP_leave, CPP_if, CPP_else, CPP_then, CPP_lparen
#	05-17-2000 added CPP_does
#	05-18-2000 fix L_plusloop for negative increments
#	06-04-2000 fix L_roll to roll the typestack as well
#	06-11-2000 added CPP_case, CPP_endcase, CPP_of, and CPP_endof
#	06-15-2000 added CPP_querydo, CPP_abortquote
#	09-05-2000 handled overflow condition on um/mod; added code for m/;
#		     added m+	;  hooks for CPP_lbracket, CPP_rbracket, CPP_ddot
#	04-07-2001 fixed F0<; added L_dabs, L_dnegate, L_udmstar, L_mstarslash
#	04-22-2001 added L_dplus, L_dminus
#	04-24-2001 added L_twopush, L_twopop, L_tworfetch
#       05-07-2001 added L_definition
#	05-08-2001 improved vm execution efficiency by about 10%
#	05-13-2001 added CPP_dotparen, L_dlt, L_dzeroeq, L_deq
#	05-20-2001 added CPP_sharp, CPP_bracketsharp, CPP_sharpbracket, 
#	             CPP_sharps, CPP_sign, CPP_hold; fixed problem with L_dnegate
#
#	07-12-2001 fixed type-stack alignment problem with L_utmslash
#	08-26-2001 updated L_fval to avoid routing fp through NDP, changed all
#	             typestack settings of OP_FVAL to OP_IVAL
#	09-02-2001 added L_tobody
#

.equ WSIZE,	4

.equ OP_ADDR,	65
.equ OP_FVAL,	70
.equ OP_IVAL,	73
.equ OP_RET,	238
.equ SIGN_MASK,	0x80000000
	
# Error Codes

.equ E_NOT_ADDR,	1
.equ E_DIV_ZERO,	4
.equ E_RET_STK_CORRUPT,	5
.equ E_UNKNOWN_OP,	6

.data
FCONST_180: .double 180.
_JumpTable: .int L_false, L_true, L_cells, L_cellplus # 0 -- 3
           .int L_dfloats, L_dfloatplus, _CPP_case__Fv, _CPP_endcase__Fv  # 4 -- 7
           .int _CPP_of__Fv, _CPP_endof__Fv, _C_open, _C_lseek     # 8 -- 11
           .int _C_close, _C_read, _C_write, _C_ioctl # 12 -- 15
           .int L_usleep, L_ms, L_nop, L_nop      # 16 -- 19
           .int L_fill, L_cmove, L_cmovefrom, _CPP_dotparen__Fv # 20 -- 23
           .int _CPP_bracketsharp__Fv, _CPP_tofile__Fv, _CPP_console__Fv, _CPP_sharpbracket__Fv  # 24 -- 27
           .int _CPP_sharps__Fv, _CPP_squote__Fv, _CPP_cr__Fv, L_bl    # 28 -- 31
           .int _CPP_spaces__Fv, L_store, _CPP_cquote__Fv, _CPP_sharp__Fv # 32 -- 35
           .int _CPP_sign__Fv, L_mod, L_and, _L_tick       # 36 -- 39
           .int _CPP_lparen__Fv, _CPP_hold__Fv, L_mul, L_add  # 40 -- 43
           .int L_nop, L_sub, _CPP_dot__Fv, L_div  # 44 -- 47
           .int _L_dabs, L_dnegate, L_umstar, L_umslashmod   # 48 -- 51
           .int L_mstar, L_mplus, L_mslash, _L_mstarslash # 52 -- 55
           .int L_fmslashmod, L_smslashrem, L_nop, L_nop # 56 -- 59
           .int L_lt, L_eq, L_gt, L_question      # 60 -- 63
           .int L_fetch, L_addr, L_base, L_call   # 64 -- 67
           .int L_definition, L_erase, L_fval, _CPP_forget__Fv # 68 -- 71
           .int L_tobody, L_ival, L_nop, _C_key       # 72 -- 75
           .int L_lshift, L_slashmod, _C_numberquery, _CPP_dotr__Fv # 76 -- 79
           .int _CPP_ddot__Fv, _C_keyquery, L_rshift, _CPP_dots__Fv  # 80 -- 83
           .int _C_accept, _CPP_char__Fv, _CPP_bracketchar__Fv, _CPP_word__Fv  # 84 -- 87
           .int L_starslash, L_starslashmod, _CPP_udotr__Fv, _CPP_lbracket__Fv  # 88 -- 91
           .int L_nop, _CPP_rbracket__Fv, L_xor, _CPP_literal__Fv  # 92 -- 95
           .int _CPP_queryallot__Fv, _CPP_allot__Fv, L_binary, L_count # 96 -- 99
           .int L_decimal, _CPP_emit__Fv, _CPP_fdot__Fv, _CPP_cold__Fv # 100 -- 103
           .int L_hex, L_i, L_j, _CPP_brackettick__Fv         # 104 -- 107
           .int _CPP_fvariable__Fv, _C_timeanddate, _CPP_find__Fv, _CPP_constant__Fv # 108 -- 111
           .int L_nop, _CPP_fconstant__Fv, _CPP_create__Fv, _CPP_dotquote__Fv  # 112 -- 115
           .int _CPP_type__Fv, _CPP_udot__Fv, _CPP_variable__Fv, _CPP_words__Fv # 116 -- 119
           .int _CPP_does__Fv, _C_system, _C_chdir, L_nop   # 120 -- 123
           .int L_or, L_nop, L_not, L_nop         # 124 -- 127
           .int L_fsin, L_fcos, _C_ftan, _C_fasin   # 128 -- 131
           .int _C_facos, _C_fatan, _C_fexp, _C_fln   # 132 -- 135
           .int _C_flog, L_fatan2, L_nop, L_nop    # 136 -- 139
           .int _C_fmin, _C_fmax, L_floor, L_fround # 140 -- 143
           .int L_dlt, L_dzeroeq, L_deq, L_twopush        # 144 -- 147
           .int L_twopop, L_tworfetch, L_stod, L_stof      # 148 -- 151
           .int L_dtof, L_ftos, L_ftod, L_degtorad  # 152 -- 155
           .int L_radtodeg, L_dplus, _L_dminus, L_nop   # 156 -- 159
           .int L_inc, L_dec, L_abs, L_neg        # 160 -- 163
           .int L_min, L_max, L_twostar, L_twodiv # 164 -- 167
           .int L_twoplus, L_twominus, L_cfetch, L_cstore  # 168 -- 171
           .int L_wfetch, L_wstore, L_dffetch, L_dfstore  # 172 -- 175
           .int L_sffetch, L_sfstore, L_spfetch, L_plusstore # 176 -- 179
           .int L_fadd, L_fsub, L_fmul, L_fdiv    # 180 -- 183
           .int L_fabs, L_fneg, _C_fpow, L_fsqrt   # 184 -- 187
           .int L_nop, L_nop, L_feq, L_fne        # 188 -- 191
           .int L_flt, L_fgt, L_fle, L_fge        # 192 -- 195
           .int L_fzeroeq, L_fzerolt, L_fzerogt, L_nop # 196 -- 199
           .int L_drop, L_dup, L_swap, L_over     # 200 -- 203
           .int L_rot, L_minusrot, L_nip, L_tuck  # 204 -- 207
           .int L_pick, L_roll, _L_2drop, _L_2dup   # 208 -- 211
           .int L_2swap, L_2over, L_2rot, _L_depth # 212 -- 215
           .int L_querydup, _CPP_if__Fv, _CPP_else__Fv, _CPP_then__Fv # 216 -- 219
           .int L_push, L_pop, L_puship, L_rfetch # 220 -- 223
           .int L_rpfetch, L_afetch, _CPP_do__Fv, _CPP_leave__Fv # 224 -- 227
           .int _CPP_querydo__Fv, _CPP_abortquote__Fv, L_jz, L_jnz  # 228 -- 231
           .int L_jmp, L_loop, L_plusloop, L_unloop  # 232 -- 235
           .int L_execute, _CPP_recurse__Fv, _L_ret, _L_abort  # 236 -- 239
           .int _L_quit, L_ge, L_le, L_ne          # 240 -- 243
           .int L_zeroeq, L_zerone, L_zerolt, L_zerogt # 244 -- 247
           .int L_ult, L_ugt, _CPP_begin__Fv, _CPP_while__Fv    # 248 -- 251
           .int _CPP_repeat__Fv, _CPP_until__Fv, _CPP_again__Fv, _CPP_bye__Fv  # 252 -- 255



.text
	.align 4
.global _JumpTable
.global _L_depth, _L_tick, _L_quit, _L_abort, _L_ret, _L_dabs
.global _L_2dup, _L_2drop, _L_dminus, _L_mstarslash
.global _vm
#	.type	vm,@function
_vm:	
        pushl %ebp
        pushl %ebx
        pushl %ecx
        pushl %edx
        movl %esp, %ebp
        movl 20(%ebp), %ebx     # load the Forth instruction pointer
        movl %ebx, _GlobalIp
	xor %eax, %eax     
next:
	movb (%ebx), %al	# get the opcode
	movl _JumpTable(,%eax,4), %ebx	# machine code address of word
	xor %eax, %eax		# clear error code
	call *%ebx		# call the word
	movl _GlobalIp, %ebx
	incl %ebx		# increment the Forth instruction ptr
	movl %ebx, _GlobalIp
	cmpb $0, %al		# check for error
	jz next
exitloop:
        cmpl $OP_RET, %eax         # return from vm?
        jnz vmexit
        xor %eax, %eax            # clear the error
vmexit:
	pop %edx
        pop %ecx
        pop %ebx
        pop %ebp
        ret
L_nop:
        movl $E_UNKNOWN_OP, %eax   # unknown operation
        ret
_L_quit:
	movl _BottomOfReturnStack, %eax	# clear the return stacks
	movl %eax, _GlobalRp
	movl _BottomOfReturnTypeStack, %eax
	movl %eax, _GlobalRtp	
	movl $8, %eax		# exit the virtual machine
	ret
_L_abort:
	movl _BottomOfStack, %eax
	movl %eax, _GlobalSp
	movl _BottomOfTypeStack, %eax
	movl %eax, _GlobalTp
	jmp _L_quit
L_base:	
	movl _GlobalSp, %ebx
	movl $_Base, %eax
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalTp, %ebx
	movb $OP_ADDR, (%ebx)
	decl _GlobalTp
	xor %eax, %eax
	ret
L_binary:
	movl $_Base, %ebx
	movl $2, (%ebx)
	ret
L_decimal:	
	movl $_Base, %ebx
	movl $10, (%ebx)
	ret
L_hex:	
	movl $_Base, %ebx
	movl $16, (%ebx)
	ret		
L_false:
	movl _GlobalSp, %ebx
	movl $0, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalTp, %ebx
	movb $OP_IVAL, (%ebx)
	decl _GlobalTp
	ret
L_true:
	movl _GlobalSp, %ebx
	movl $-1, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalTp, %ebx
	movb $OP_IVAL, (%ebx)
	decl _GlobalTp
	ret
L_cells:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	sall $2, %eax
	movl %eax, (%ebx)
	xor %eax, %eax
	ret
L_cellplus:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %eax
	movl %eax, (%ebx)
	xor %eax, %eax
	ret
L_dfloats:	
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	sall $3, %eax
	movl %eax, (%ebx)
	xor %eax, %eax
	ret
L_dfloatplus:	
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %eax
	addl $WSIZE, %eax
	movl %eax, (%ebx)
	xor %eax, %eax
	ret				
L_bl:
	movl _GlobalSp, %ebx
	movl $32, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalTp, %ebx
	movb $OP_IVAL, (%ebx)
	decl _GlobalTp	
	ret
_L_ret:
	movl _GlobalRp, %ecx
	addl $WSIZE, %ecx
	movl %ecx, _GlobalRp
	incl _GlobalRtp
        cmpl _BottomOfReturnStack, %ecx
        jle ret1
        movl $OP_RET, %eax             # exhausted the return stack so exit vm
        ret
ret1:   movl _GlobalRtp, %ebx
	movb (%ebx), %al
        cmpb $OP_ADDR, %al
        jz ret2
        movl $E_RET_STK_CORRUPT, %eax   # indicate return stack corrupted
        jmp retexit
ret2:   movl %ecx, %ebx
        movl (%ebx), %eax
	movl %eax, _GlobalIp		# reset the instruction ptr
        xor %eax, %eax
retexit:
        ret
_L_tick:
	movl _GlobalSp, %ebx
	movl $32, (%ebx)
	movl $WSIZE, %eax
	subl %eax, _GlobalSp
	decl _GlobalTp
	call _CPP_word__Fv
	call _CPP_find__Fv
	call L_drop
	ret
L_tobody:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx	# code address
	incl %ecx		# the data address is offset by one
	movl (%ecx), %ecx
	movl %ecx, (%ebx)
	ret	
L_usleep:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	pushl %eax
	call _usleep
	popl %eax
	xor %eax, %eax
	ret
L_ms:
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	imull $1000, %eax
	movl %eax, WSIZE(%ebx)
	call L_usleep
	ret
L_fill:
	call L_swap
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalTp, %ebx
	movb (%ebx), %al
	cmpb $OP_ADDR, %al
	jz fill2
	popl %ebx
	popl %ebx
	movl $E_NOT_ADDR, %eax
	jmp fillexit
fill2:	movl _GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	call _memset
	addl $12, %esp
	xor %eax, %eax
fillexit:	
	ret
L_erase:
	movl _GlobalSp, %ebx
	movl $0, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	decl _GlobalTp
	call L_fill
	ret	
L_cmove:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	call L_swap
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalTp, %ebx
	movb (%ebx), %al
	cmpb $OP_ADDR, %al
	jz cmove2
	popl %ebx
	movl $E_NOT_ADDR, %eax
	ret
cmove2:	movl _GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalTp, %ebx
	movb (%ebx), %al
	cmpb $OP_ADDR, %al
	jz cmove3
	popl %ebx
	popl %ebx
	movl $E_NOT_ADDR, %eax
	ret
cmove3:	movl _GlobalSp, %ebx
	movl (%ebx), %ebx
	pushl %ebx
	call _memcpy
	addl $12, %esp
	xor %eax, %eax				
	ret		
L_cmovefrom:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %ecx	# load count register
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalTp, %ebx
	movb (%ebx), %al
	cmpb $OP_ADDR, %al
	jz cmovefrom2
	movl $E_NOT_ADDR, %eax						
	ret
cmovefrom2:
	movl _GlobalSp, %ebx
	movl (%ebx), %ebx
	movl %ecx, %eax
	decl %eax
	addl %eax, %ebx
	movl %ebx, %edx		# dest addr in %edx
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalTp, %ebx
	movb (%ebx), %al
	cmpb $OP_ADDR, %al
	jz cmovefrom3
	movl $E_NOT_ADDR, %eax
	ret
cmovefrom3:
	movl _GlobalSp, %ebx
	movl (%ebx), %ebx
	movl %ecx, %eax
	decl %eax
	addl %eax, %ebx		# src addr in %ebx
cmovefromloop:	
	movb (%ebx), %al
	decl %ebx
	xchgl %ebx, %edx
	movb %al, (%ebx)
	decl %ebx
	xchgl %ebx, %edx
	loop cmovefromloop	
	xor %eax, %eax
	ret
L_call:	
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	call *(%ebx)
	xor %eax, %eax
	ret 		
L_push:
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
	addl %eax, %ebx	
        movl %ebx, _GlobalSp
	movl (%ebx), %ecx
	movl _GlobalRp, %ebx
	movl %ecx, (%ebx)
	subl %eax, %ebx
	movl %ebx, _GlobalRp
	movl _GlobalTp, %ebx
	incl %ebx
	movl %ebx, _GlobalTp
	movb (%ebx), %al
	movl _GlobalRtp, %ebx
	movb %al, (%ebx)
	decl %ebx
	movl %ebx, _GlobalRtp
        xor %eax, %eax
        ret
L_pop:
	movl $WSIZE, %eax
	movl _GlobalRp, %ebx
	addl %eax, %ebx
	movl %ebx, _GlobalRp
	movl (%ebx), %ecx
	movl _GlobalSp, %ebx
	movl %ecx, (%ebx)
	subl %eax, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalRtp, %ebx
	incl %ebx
	movl %ebx, _GlobalRtp
	movb (%ebx), %al
	movl _GlobalTp, %ebx
	movb %al, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	xor %eax, %eax
	ret
L_twopush:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	movl %ebx, _GlobalSp
	movl _GlobalRp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalRp
	movl _GlobalTp, %ebx
	incl %ebx
	movw (%ebx), %ax
	incl %ebx
	movl %ebx, _GlobalTp
	movl _GlobalRtp, %ebx
	decl %ebx
	movw %ax, (%ebx)
	decl %ebx
	movl %ebx, _GlobalRtp
	xor %eax, %eax
	ret
L_twopop:
	movl _GlobalRp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	movl %ebx, _GlobalRp
	movl _GlobalSp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalRtp, %ebx
	incl %ebx
	movw (%ebx), %ax
	incl %ebx
	movl %ebx, _GlobalRtp
	movl _GlobalTp, %ebx
	decl %ebx
	movw %ax, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	xor %eax, %eax				
	ret
L_puship:
        movl _GlobalIp, %eax
        movl _GlobalRp, %ebx
        mov %eax, (%ebx)
	movl $WSIZE, %eax
        subl %eax, _GlobalRp
        movl _GlobalRtp, %ebx
	movb $OP_ADDR, %al
        movb %al, (%ebx)
        decl _GlobalRtp
        xor %eax, %eax
        ret
L_execute:	
        movl _GlobalIp, %ecx
        movl _GlobalRp, %ebx
        movl %ecx, (%ebx)
	movl $WSIZE, %eax 
        subl %eax, %ebx 
	movl %ebx, _GlobalRp
        movl _GlobalRtp, %ebx
	movb $OP_ADDR, (%ebx)
	decl %ebx
        movl %ebx, _GlobalRtp
	movl _GlobalSp, %ebx
        addl %eax, %ebx
	movl %ebx, _GlobalSp
        movl (%ebx), %eax
	decl %eax
	movl %eax, _GlobalIp
	incl _GlobalTp
        xor %eax, %eax
        ret
L_definition:
        movl _GlobalIp, %ebx
	movl $WSIZE, %eax
	incl %ebx
	movl (%ebx), %ecx # address to execute
	addl $3, %ebx
	movl %ebx, %edx
	movl _GlobalRp, %ebx
	movl %edx, (%ebx)
	subl %eax, %ebx
	movl %ebx, _GlobalRp
	movl _GlobalRtp, %ebx
	movb $OP_ADDR, (%ebx)
	decl %ebx
	movl %ebx, _GlobalRtp
	decl %ecx
	movl %ecx, _GlobalIp
        xor %eax, %eax	
	ret	
L_rfetch:
        movl _GlobalRp, %ebx
        addl $WSIZE, %ebx
        movl (%ebx), %eax
        movl _GlobalSp, %ebx
        movl %eax, (%ebx)
	movl $WSIZE, %eax
        subl %eax, _GlobalSp
        movl _GlobalRtp, %ebx
        incl %ebx
        movb (%ebx), %al
        movl _GlobalTp, %ebx
        movb %al, (%ebx)
        decl _GlobalTp
        xor %eax, %eax
	ret
L_tworfetch:
	movl _GlobalRp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	movl _GlobalSp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalRtp, %ebx
	incl %ebx
	movw (%ebx), %ax
	incl %ebx
	movl _GlobalTp, %ebx
	decl %ebx
	movw %ax, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	xor %eax, %eax				
	ret	
L_rpfetch:
	movl _GlobalRp, %eax
	addl $WSIZE, %eax
	movl _GlobalSp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalTp, %ebx
	movb $OP_ADDR, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	xor %eax, %eax
	ret
L_spfetch:
	movl _GlobalSp, %eax
	movl %eax, %ebx
	addl $WSIZE, %eax
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalTp, %ebx
	movb $OP_ADDR, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	xor %eax, %eax 
	ret
L_i:
        movl _GlobalRtp, %ebx
        movb 3(%ebx), %al
        movl _GlobalTp, %ebx
	movb %al, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
        movl _GlobalRp, %ebx
        movl 3*WSIZE(%ebx), %eax
        movl _GlobalSp, %ebx
        movl %eax, (%ebx)
	movl $WSIZE, %eax
        subl %eax, %ebx 
	movl %ebx, _GlobalSp
        xor %eax, %eax
        ret
L_j:
        movl _GlobalRtp, %ebx
        movb 6(%ebx), %al
	movl _GlobalTp, %ebx
	movb %al, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
        movl _GlobalRp, %ebx
        movl 6*WSIZE(%ebx), %eax
        movl _GlobalSp, %ebx
        movl %eax, (%ebx)
	movl $WSIZE, %eax
        subl %eax, %ebx
	movl %ebx, _GlobalSp
        xor %eax, %eax
        ret
L_loop:
        movl _GlobalRtp, %ebx
        incl %ebx
        movb (%ebx), %al
        cmpb $OP_ADDR, %al
        jnz loopbad
        movl _GlobalRp, %ebx	
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %ecx
	addl %eax, %ebx
        movl (%ebx), %eax
        incl %eax
	cmpl %ecx, %eax	
        jz L_unloop
loop1:	
        movl %eax, (%ebx)	# set loop counter to next value
	movl %edx, _GlobalIp	# set instruction ptr to start of loop
        xor %eax, %eax
        ret
L_unloop:  
	movl $WSIZE, %eax
	shll $1, %eax
	addl $WSIZE, %eax
#	imul $3, %eax
	addl %eax, _GlobalRp  # terminal count reached, discard top 3 items
	movl $3, %eax
        addl %eax, _GlobalRtp
        xor %eax, %eax
        ret
loopbad:
        movl $E_RET_STK_CORRUPT, %eax
        ret	
L_plusloop:
	movl _GlobalRtp, %ebx
        incl %ebx
        movb (%ebx), %al
        cmpb $OP_ADDR, %al
        jnz loopbad
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %ebp	# get loop increment 
	movl %ebx, _GlobalSp
	incl _GlobalTp		
        movl _GlobalRp, %ebx
	addl %eax, %ebx		# get ip and save in edx
	movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %ecx	# get terminal count in ecx
	addl %eax, %ebx
	movl (%ebx), %eax	# get current loop count 
	addl %ebp, %eax		# increment loop count
	cmpl $0, %ebp
	jl plusloop1
	cmpl %ecx, %eax
	jge L_unloop				
	movl %eax, (%ebx)	# set loop counter to incremented value
        movl %edx, _GlobalIp     # set instruction ptr to start of loop
        xor %eax, %eax
        ret
plusloop1:
	cmpl %ecx, %eax
	jl L_unloop
	movl %eax, (%ebx)
	movl %edx, _GlobalIp
	xor %eax, %eax
	ret
L_jz:
	movl $WSIZE, %eax
        movl _GlobalSp, %ebx
	addl %eax, %ebx
	movl %ebx, _GlobalSp
        incl _GlobalTp
        movl (%ebx), %eax
        cmpl $0, %eax
        jz jz1
	movl $4, %eax
        addl %eax, _GlobalIp       # do not jump
	xor %eax, %eax
	ret
jz1:    movl _GlobalIp, %ebx
        incl %ebx
        movl (%ebx), %eax          # get the relative jump count
        decl %eax
        addl %eax, _GlobalIp
jzexit: xor %eax, %eax
        ret
L_jnz:				# not implemented
	ret
L_jmp:
        movl _GlobalIp, %ebx
        incl %ebx
        movl (%ebx), %eax          # get the relative jump count
        addl %eax, %ebx
        subl $2, %ebx
        movl %ebx, _GlobalIp      # set instruction ptr
	xor %eax, %eax
        ret
L_count:
	movl _GlobalTp, %ebx
	movb 1(%ebx), %al
	cmpb $OP_ADDR, %al
	jnz counterror
	movb $OP_IVAL, (%ebx)
	decl _GlobalTp
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %ebx
	xor %eax, %eax
	movb (%ebx), %al
	movl _GlobalSp, %ebx
	incl WSIZE(%ebx)
	movl %eax, (%ebx)
	movl $WSIZE, %eax
	subl %eax, _GlobalSp
	xor %eax, %eax
	ret
counterror:
	movl $E_NOT_ADDR, %eax
	ret
L_ival:
        movl _GlobalIp, %ebx
        incl %ebx
        movl (%ebx), %eax
	movl %ebx, %ecx
        movl _GlobalSp, %ebx
        movl %eax, (%ebx)
	subl $WSIZE, %ebx
        movl %ebx, _GlobalSp    # decrement the stack ptr
        movl _GlobalTp, %ebx
	movb $OP_IVAL, %al
        movb %al, (%ebx)     # set type and dec
	decl %ebx
        movl %ebx, _GlobalTp
	movl %ecx, %eax
	decl %eax
	addl $WSIZE, %eax
        movl %eax, _GlobalIp    # advance the instruction ptr
        xor %eax, %eax            # no errors
        ret
L_addr:
	movl _GlobalIp, %ebx
	incl %ebx
	movl (%ebx), %eax
	movl %ebx, %ecx
	movl _GlobalSp, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalTp, %ebx
	movb $OP_ADDR, %al
	movb %al, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	movl %ecx, %eax
	decl %eax
	addl $WSIZE, %eax
	movl %eax, _GlobalIp
	xor %eax, %eax
        ret
L_fval:
        movl _GlobalIp, %ebx
        incl %ebx
        movl _GlobalSp, %ebp
        subl $WSIZE, %ebp
        movl (%ebx), %eax
	movl %eax, (%ebp)
	movl WSIZE(%ebx), %eax
	movl %eax, WSIZE(%ebp)
	subl $WSIZE, %ebp
	movl %ebp, _GlobalSp
	addl $2*WSIZE-1, %ebx
	movl %ebx, _GlobalIp
	movl _GlobalTp, %ebx
	movb $OP_IVAL, (%ebx)
	decl %ebx
	movb $OP_IVAL, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	xor %eax, %eax
	ret
L_and:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	pushl %ebx
	movl (%ebx), %ebx
	andl %ebx, %eax
	popl %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	incl _GlobalTp
	movl _GlobalTp, %ebx
	movb $OP_IVAL, 1(%ebx)
	xor %eax, %eax
	ret
L_or:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	orl %ebx, %eax
	movl _GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
	movl _GlobalTp, %ebx
	movb $OP_IVAL, 1(%ebx)
	xor %eax, %eax
	ret
L_not:
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	notl %eax
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
	ret
L_xor:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	xorl %ebx, %eax
	movl _GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
	movl _GlobalTp, %ebx
	movb $OP_IVAL, 1(%ebx)
	xor %eax, %eax
	ret
L_lshift:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %ecx
	movl WSIZE(%ebx), %eax
	shll %cl, %eax
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
	ret
L_rshift:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %ecx
	movl WSIZE(%ebx), %eax
	shrl %cl, %eax
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
	ret
L_eq:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	cmpl %ebx, %eax
	jne eq2
eq1:	movl _GlobalSp, %ebx
	movl $-1, WSIZE(%ebx)
	jmp eq3
eq2:    movl _GlobalSp, %ebx
	movl $0, WSIZE(%ebx)
eq3:    movl _GlobalTp, %ebx
	movb $OP_IVAL, 1(%ebx)
	xor %eax, %eax
	ret
L_ne:
	call L_eq
	call L_not
	ret
L_ult:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	cmpl %eax, %ebx
	jae eq2
	jmp eq1
	ret
L_ugt:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	cmpl %eax, %ebx
	jbe eq2
	jmp eq1	
	ret	
L_lt:
	movl _GlobalTp, %ebx
	incl %ebx
	movb $OP_IVAL, (%ebx)
	movl %ebx, _GlobalTp
	movl _GlobalSp, %ebx
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, _GlobalSp
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl (%ebx), %eax
	cmpl %ecx, %eax
	jge lt1
	movl $-1, (%ebx)
	xor %eax, %eax
	ret
lt1:
	movl $0, (%ebx)
	xor %eax, %eax
	ret	
L_gt:
	movl _GlobalTp, %ebx
	incl %ebx
	movb $OP_IVAL, (%ebx)
	movl %ebx, _GlobalTp
	movl _GlobalSp, %ebx
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, _GlobalSp
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl (%ebx), %eax
	cmpl %ecx, %eax
	jle gt1
	movl $-1, (%ebx)
	xor %eax, %eax
	ret
gt1:
	movl $0, (%ebx)
	xor %eax, %eax
	ret
L_le:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	cmpl %eax, %ebx
	jg eq2
	jmp eq1
L_ge:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	cmpl %eax, %ebx
	jl eq2
	jmp eq1
L_zerolt:
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	cmpl $0, %eax
	jl zerolt2
zerolt1:
	movl $0, WSIZE(%ebx)
	jmp zeroltexit
zerolt2:
	movl $-1, WSIZE(%ebx)
zeroltexit:
	movl _GlobalTp, %ebx
	movb $OP_IVAL, 1(%ebx)
	xor %eax, %eax
	ret
L_zeroeq:
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	cmpl $0, %eax
	je zerolt2
	jmp zerolt1
L_zerone:
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	cmpl $0, %eax
	je zerolt1
	jmp zerolt2
L_zerogt:
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	cmpl $0, %eax
	jg zerolt2
	jmp zerolt1
L_deq:
	movl _GlobalTp, %ebx
	addl $4, %ebx
	movb $OP_IVAL, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx
	addl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl (%ebx), %eax
	subl %edx, %eax
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	subl %ecx, %edx
	orl %edx, %eax
	cmpl $0, %eax
	jz deq1
	movl $0, (%ebx)
	xor %eax, %eax
	ret
deq1:	movl $-1, (%ebx)
	xor %eax, %eax	
	ret
L_dzeroeq:
	movl _GlobalTp, %ebx
	addl $2, %ebx
	movb $OP_IVAL, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	orl (%ebx), %eax
	cmpl $0, %eax
	jz deq1
	movl $0, (%ebx)
	xor %eax, %eax
	ret
L_dlt:
	call _L_dminus
	movl _GlobalTp, %ebx
	incl %ebx
	movl %ebx, _GlobalTp
	incl %ebx
	movb $OP_IVAL, (%ebx)
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	cmpl $0, %eax
	jl deq1
	movl $0, (%ebx)
	xor %eax, %eax	
	ret
L_dult:	
	ret
L_querydup:
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	cmpl $0, %eax
	je L_querydupexit
	movl %eax, (%ebx)
	movl _GlobalTp, %ebx
	movb 1(%ebx), %al
	movb %al, (%ebx)
	decl _GlobalTp
	movl $WSIZE, %eax
	subl %eax, _GlobalSp
	xor %eax, %eax
L_querydupexit:
	ret	
L_drop:
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        incl _GlobalTp
	xor %eax, %eax
        ret
L_dup:
        movl _GlobalSp, %ebx
        movl WSIZE(%ebx), %eax
        movl %eax, (%ebx)
	movl %ebx, %eax
	subl $WSIZE, %eax
	movl %eax, _GlobalSp
        movl _GlobalTp, %ebx
        movb 1(%ebx), %al
        movb %al, (%ebx)
        decl _GlobalTp
        xor %eax, %eax
        ret
L_swap:
        movl _GlobalSp, %ebx
        addl $WSIZE, %ebx
        movl WSIZE(%ebx), %eax
        xchgl (%ebx), %eax
        movl %eax, WSIZE(%ebx)
        movl _GlobalTp, %ebx
        incl %ebx
        movb 1(%ebx), %al
        xchgb (%ebx), %al
        movb %al, 1(%ebx)
        xor %eax, %eax
        ret
L_over:
        movl _GlobalSp, %ebx
        movl 2*WSIZE(%ebx), %eax
        movl %eax, (%ebx)
	movl %ebx, %eax
	subl $WSIZE, %eax
	movl %eax, _GlobalSp
        movl _GlobalTp, %ebx
        movb 2(%ebx), %al
        movb %al, (%ebx)
        decl _GlobalTp
        xor %eax, %eax
        ret
L_rot:
        call L_swap
        movl _GlobalSp, %ebx
        addl $WSIZE, %ebx
        movl 2*WSIZE(%ebx), %eax
        xchgl (%ebx), %eax
        xchgl 2*WSIZE(%ebx), %eax
        movl _GlobalTp, %ebx
        incl %ebx
        movb  2(%ebx), %al
        xchgb (%ebx), %al
        xchgb 2(%ebx), %al
        xor %eax, %eax
        ret
L_minusrot:
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	movl %eax, (%ebx)
	addl $WSIZE, %ebx
	movl WSIZE(%ebx), %eax
	movl %eax, (%ebx)
	addl $WSIZE, %ebx
	movl WSIZE(%ebx), %eax
	movl %eax, (%ebx)
	movl -2*WSIZE(%ebx), %eax
	movl %eax, WSIZE(%ebx)
	movl _GlobalTp, %ebx
	movb 1(%ebx), %al
	movb %al, (%ebx)
	incl %ebx
	movw 1(%ebx), %ax
	movw %ax, (%ebx)
	movb -1(%ebx), %al
	movb %al, 2(%ebx)
	xor %eax, %eax
	ret
L_nip:
        call L_swap
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        incl _GlobalTp
	xor %eax, %eax
        ret
L_tuck:
        call L_swap
        call L_over
        ret
L_pick:
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	addl $2, %eax
	imul $WSIZE, %eax
	addl %eax, %ebx
	movl (%ebx), %eax
	movl _GlobalSp, %ebx
	movl %eax, (%ebx)
	movl WSIZE(%ebx), %eax
	addl $2, %eax
	movl _GlobalTp, %ebx
	addl %eax, %ebx
	movb (%ebx), %al
	movl _GlobalTp, %ebx
	movb %al, 1(%ebx)
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
	ret
L_roll:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalSp, %ebx 
	movl (%ebx), %eax
	incl %eax
	pushl %eax
	pushl %eax
	pushl %eax
	pushl %ebx
	imul $WSIZE, %eax
	addl %eax, %ebx		# addr of item to roll
	movl (%ebx), %eax
	popl %ebx
	movl %eax, (%ebx)
	popl %eax		# number of cells to copy
	movl %eax, %ecx
	imul $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, %edx		# dest addr
	subl $WSIZE, %ebx	# src addr
rollloop:
	movl (%ebx), %eax
	sub $WSIZE, %ebx
	xchgl %ebx, %edx
	movl %eax, (%ebx)
	sub $WSIZE, %ebx
	xchgl %ebx, %edx
	loop rollloop

	popl %eax		# now we have to roll the typestack
	mov _GlobalTp, %ebx	
	addl %eax, %ebx
	movb (%ebx), %al
	movl _GlobalTp, %ebx
	movb %al, (%ebx)
	popl %eax
	movl %eax, %ecx
	addl %eax, %ebx
	movl %ebx, %edx
	decl %ebx
rolltloop:
	movb (%ebx), %al
	decl %ebx
	xchgl %ebx, %edx
	movb %al, (%ebx)
	decl %ebx
	xchgl %ebx, %edx
	loop rolltloop
	xor %eax, %eax
	ret
_L_depth:
	movl _GlobalSp, %ebx
	movl _BottomOfStack, %eax
	subl %ebx, %eax
	movl $WSIZE, (%ebx)
	movl $0, %edx
	idivl (%ebx)
	movl %eax, (%ebx)
	movl $WSIZE, %eax
	subl %eax, _GlobalSp
	movl _GlobalTp, %ebx
	movb $OP_IVAL, (%ebx)
	decl _GlobalTp
	xor %eax, %eax
        ret
_L_2drop:
	movl $WSIZE, %eax
	addl %eax, %eax
	addl %eax, _GlobalSp
        addl $2, _GlobalTp
	xor %eax, %eax
        ret
_L_2dup:
	movl _GlobalSp, %ebx
	movl %ebx, %ecx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	movl %ecx, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalTp, %ebx
	incl %ebx
	movw (%ebx), %ax
	subl $2, %ebx
	movw %ax, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	xor %eax, %eax
        ret
L_2swap:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	xchgl %edx, (%ebx)
	addl $WSIZE, %ebx
	xchgl %eax, (%ebx)
	subl $2*WSIZE, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	movl _GlobalTp, %ebx
	incl %ebx
	movw (%ebx), %ax
	addl $2, %ebx
	xchgw %ax, (%ebx)
	subl $2, %ebx
	movw %ax, (%ebx)
	xor %eax, %eax	
        ret
L_2over:
	movl _GlobalSp, %ebx
	movl %ebx, %ecx
	addl $3*WSIZE, %ebx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	movl %ecx, %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	movl _GlobalTp, %ebx
	movl %ebx, %ecx
	addl $3, %ebx
	movw (%ebx), %ax
	movl %ecx, %ebx
	decl %ebx
	movw %ax, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	xor %eax, %eax
        ret
L_2rot:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, %ecx
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	xchgl %edx, (%ebx)
	addl $WSIZE, %ebx
	xchgl %eax, (%ebx)
	addl $WSIZE, %ebx
	xchgl %edx, (%ebx)
	addl $WSIZE, %ebx
	xchgl %eax, (%ebx)
	movl %ecx, %ebx
	movl %edx, (%ebx)
	addl $WSIZE, %ebx
	movl %eax, (%ebx)
	movl _GlobalTp, %ebx
	incl %ebx
	movl %ebx, %ecx
	movw (%ebx), %ax
	addl $2, %ebx
	xchgw %ax, (%ebx)
	addl $2, %ebx
	xchgw %ax, (%ebx)
	movl %ecx, %ebx
	movw %ax, (%ebx)
	xor %eax, %eax
        ret
L_question:
	call L_fetch
	cmpl $0, %eax
	jnz questionexit
	call _CPP_dot__Fv
questionexit:	
	ret	
L_fetch:
	movl _GlobalSp, %edx
	movl _GlobalTp, %ebx
	incl %ebx
        movb (%ebx), %al
        cmpb $OP_ADDR, %al
        jnz fetcherror
        movb $OP_IVAL, (%ebx)
	addl $WSIZE, %edx
        movl %edx, %ebx	
        movl (%ebx), %ebx
        movl (%ebx), %eax
        movl %edx, %ebx
        movl %eax, (%ebx)
	xor %eax, %eax
	ret
fetcherror:
        movl $E_NOT_ADDR, %eax
        ret
L_store:
        movl _GlobalTp, %ebx
	movl %ebx, %ebp
	incl %ebx
        movb (%ebx), %al
        cmpb $OP_ADDR, %al
        jnz fetcherror
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
        addl %eax, %ebx
        movl (%ebx), %ecx	# address to store to in ecx
	addl %eax, %ebx
	movl (%ebx), %edx	# value to store in edx
	movl %ebx, _GlobalSp
	movl %ecx, %ebx
	movl %edx, (%ebx)
#	movl _GlobalTp, %ebx
	movl %ebp, %ebx
	incl %ebx
	incl %ebx
	movl %ebx, _GlobalTp
	xor %eax, %eax
	ret
L_afetch:
	movl _GlobalSp, %edx
	movl _GlobalTp, %ebx
	incl %ebx
        movb (%ebx), %al
        cmpb $OP_ADDR, %al
        jnz fetcherror
        movb $OP_ADDR, (%ebx)
	addl $WSIZE, %edx
        movl %edx, %ebx	
        movl (%ebx), %ebx
        movl (%ebx), %eax
        movl %edx, %ebx
        movl %eax, (%ebx)
	xor %eax, %eax
	ret
L_cfetch:
	movl _GlobalTp, %ebx
	movb 1(%ebx), %al
	cmpb $OP_ADDR, %al
	jnz fetcherror
	movb $OP_IVAL, 1(%ebx)
	xor %eax, %eax
	movl _GlobalSp, %ebx
	movl %ebx, %ecx
	movl WSIZE(%ebx), %ebx
	movb (%ebx), %al
	movl %ecx, %ebx
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
        ret
L_cstore:
	movl _GlobalSp, %eax
	movl %eax, %ecx
	incl _GlobalTp
	movl _GlobalTp, %ebx
	movb (%ebx), %al
	cmpb $OP_ADDR, %al
	jnz fetcherror
	movl %ecx, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	xchgl %ebx, %ecx
	movb %al, (%ebx)
	movl %ecx, %eax
	movl %eax, _GlobalSp
	incl _GlobalTp
	xor %eax, %eax
	ret	
L_wfetch:
	movl _GlobalTp, %ebx
	movb 1(%ebx), %al
	cmpb $OP_ADDR, %al
	jnz fetcherror
	movb $OP_IVAL, 1(%ebx)
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %ebx
	movw (%ebx), %ax
	cwde
	movl _GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
	xor %eax, %eax
        ret
L_wstore:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	movl _GlobalTp, %ebx
	movb (%ebx), %al
	cmpb $OP_ADDR, %al
	jnz fetcherror
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	pushl %eax
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	pop %ebx
	movw %ax, (%ebx)
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	incl _GlobalTp
	xor %eax, %eax
        ret
L_sffetch:
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        incl _GlobalTp
        movl _GlobalTp, %ebx
        movb (%ebx), %al
        cmpb $OP_ADDR, %al
        jnz fetcherror
        movb $OP_IVAL, (%ebx)
        decl %ebx
        movb $OP_IVAL, (%ebx)
        decl _GlobalTp
	decl _GlobalTp
        movl _GlobalSp, %ebx
        movl (%ebx), %ebx
        flds (%ebx)
	movl $WSIZE, %eax
        subl %eax, _GlobalSp
        movl _GlobalSp, %ebx
        fstpl (%ebx)
        subl %eax, _GlobalSp
	xor %eax, %eax
        ret
L_sfstore:
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        incl _GlobalTp
        movl _GlobalTp, %ebx
        movb (%ebx), %al
        cmpb $OP_ADDR, %al
        jnz fetcherror
        movl _GlobalSp, %ebx
        addl $WSIZE, %ebx
        fldl (%ebx)              # load the f number into NDP
        subl $WSIZE, %ebx
        movl (%ebx), %ebx          # load the dest address
        fstps (%ebx)             # store as single precision float
	movl $WSIZE, %eax
	sall $1, %eax
        addl %eax, _GlobalSp
	incl _GlobalTp
	incl _GlobalTp
	xor %eax, %eax
        ret
L_dffetch:
        movl _GlobalTp, %ebx
	incl %ebx
        movb (%ebx), %al
        cmpb $OP_ADDR, %al
        jnz fetcherror
	movb $OP_IVAL, (%ebx)
	decl %ebx
	movb $OP_IVAL, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp 
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %ecx
	xchgl %ebx, %ecx
	movl (%ebx), %eax	# first dword
	addl $WSIZE, %ebx
	movl (%ebx), %edx	# second dword
	movl %ecx, %ebx
	movl %eax, (%ebx)
	addl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $2*WSIZE, %ebx
	movl %ebx, _GlobalSp
	xor %eax, %eax
	ret
L_dfstore:
        movl _GlobalTp, %ebx
	incl %ebx
        movb (%ebx), %al
        cmpb $OP_ADDR, %al
        jnz fetcherror
	addl $2, %ebx
	movl %ebx, _GlobalTp
	movl _GlobalSp, %ebx
	movl $WSIZE, %edx
	addl %edx, %ebx
	movl %ebx, %eax
	movl (%ebx), %ebx  # address to store
	addl %edx, %eax
	movl (%eax), %ecx
	movl %ecx, (%ebx)
	addl %edx, %eax
	addl %edx, %ebx
	movl (%eax), %ecx
	movl %ecx, (%ebx)
	movl %eax, _GlobalSp
	xor %eax, %eax
	ret
L_inc:
        movl _GlobalSp, %ebx
        incl WSIZE(%ebx)
        ret
L_dec:
        movl _GlobalSp, %ebx
        decl WSIZE(%ebx)
        ret
L_twoplus:
	movl _GlobalSp, %ebx
	incl WSIZE(%ebx)
	incl WSIZE(%ebx)
	ret
L_twominus:
	movl _GlobalSp, %ebx
	decl WSIZE(%ebx)
	decl WSIZE(%ebx)
	ret
L_abs:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	cmpl $0, %eax
	jl abs1
	xor %eax, %eax
	ret
abs1:	negl %eax
	movl %eax, (%ebx)
	xor %eax, %eax
        ret
L_neg:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	negl %eax
	movl %eax, (%ebx)
	xor %eax, %eax
        ret
L_max:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	cmpl %eax, %ebx
	jl max1
	movl %ebx, %eax
	movl _GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
	jmp maxexit
max1:
	movl _GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
maxexit:
	incl _GlobalTp
	xor %eax, %eax
        ret
L_min:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	movl _GlobalSp, %ebx
	movl (%ebx), %eax
	movl WSIZE(%ebx), %ebx
	cmpl %eax, %ebx
	jg min1
	movl %ebx, %eax
	movl _GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
	jmp minexit
min1:
	movl _GlobalSp, %ebx
	movl %eax, WSIZE(%ebx)
minexit:
	incl _GlobalTp
	xor %eax, %eax
        ret
L_dmax:
	ret
L_dmin:
	ret
L_twostar:
	movl _GlobalSp, %ebx
	sall $1, WSIZE(%ebx)
	ret
L_twodiv:
	movl _GlobalSp, %ebx
	sarl $1, WSIZE(%ebx)
	ret
L_add:
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %eax
	addl %eax, WSIZE(%ebx)
	movl %ebx, _GlobalSp
	movl _GlobalTp, %ebx
	incl %ebx
	movl %ebx, _GlobalTp
	movw (%ebx), %ax
	andb %ah, %al		# and the two types to preserve addr type
	incl %ebx
	movb %al, (%ebx)
        xor %eax, %eax
        ret
L_sub:
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %eax
	subl %eax, WSIZE(%ebx)
	movl %ebx, _GlobalSp
        incl _GlobalTp		# result will have type of first operand
        xor %eax, %eax
        ret
L_mul:
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
	addl %eax, %ebx
	movl %ebx, _GlobalSp
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl %ecx, %eax
	imull (%ebx)
	movl %eax, (%ebx)
	incl _GlobalTp
	xor %eax, %eax
        ret
L_div:
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        incl _GlobalTp
        movl _GlobalSp, %ebx
        movl (%ebx), %eax
        cmpl $0, %eax
        jnz div1
        movl $E_DIV_ZERO, %eax
        jmp divexit
div1:	
	addl $WSIZE, %ebx
        movl (%ebx), %eax
	cdq
        idivl -WSIZE(%ebx)
        movl %eax, (%ebx)
	xor %eax, %eax
divexit:
        ret
L_mod:
	call L_div
	movl %edx, (%ebx)
	ret
L_slashmod:
	call L_div
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	decl _GlobalTp
	call L_swap
	ret
L_starslash:
	movl $WSIZE, %eax
	sall $1, %eax
        addl %eax, _GlobalSp
        movl _GlobalSp, %ebx
        movl WSIZE(%ebx), %eax
        imull (%ebx)
	idivl -WSIZE(%ebx)
	movl %eax, WSIZE(%ebx)
	incl _GlobalTp
	incl _GlobalTp
	xor %eax, %eax		
	ret
L_starslashmod:
	call L_starslash
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	decl _GlobalTp
	call L_swap
	ret
L_plusstore:
	movl _GlobalTp, %ebx
	movb 1(%ebx), %al
	cmpb $OP_ADDR, %al
	jnz fetcherror
	movl _GlobalSp, %ebx
	push %ebx
	push %ebx
	push %ebx
	movl WSIZE(%ebx), %ebx
	movl (%ebx), %eax
	pop %ebx
	movl 2*WSIZE(%ebx), %ebx
	addl %ebx, %eax
	pop %ebx
	movl WSIZE(%ebx), %ebx
	movl %eax, (%ebx)
	pop %ebx
	movl $WSIZE, %eax
	sall $1, %eax
	addl %eax, %ebx
	movl %ebx, _GlobalSp
	incl _GlobalTp
	incl _GlobalTp
	xor %eax, %eax
	ret
_L_dabs:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx
	movl %ecx, %eax
	cmpl $0, %eax
	jl dabs_go
	xor %eax, %eax
	ret
dabs_go:
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	clc
	subl $1, %eax
	notl %eax
	movl %eax, (%ebx)
	movl %ecx, %eax
	sbbl $0, %eax
	notl %eax
	movl %eax, -WSIZE(%ebx)
	xor %eax, %eax
	ret
L_dnegate:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, %ecx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	notl %eax
	clc
	addl $1, %eax
	movl %eax, (%ebx)
	movl %ecx, %ebx
	movl (%ebx), %eax
	notl %eax
	adcl $0, %eax
	movl %eax, (%ebx)
	xor %eax, %eax	
	ret	
L_dplus:
	movl _GlobalSp, %ebx
	addl $2*WSIZE, %ebx
	movl (%ebx), %eax
	clc
	addl 2*WSIZE(%ebx), %eax
	movl %eax, 2*WSIZE(%ebx)
	movl WSIZE(%ebx), %eax
	adcl -WSIZE(%ebx), %eax
	movl %eax, WSIZE(%ebx)
	movl %ebx, _GlobalSp
	incl _GlobalTp
	incl _GlobalTp
	xor %eax, %eax
	ret
_L_dminus:
	movl _GlobalSp, %ebx
	addl $2*WSIZE, %ebx
	movl 2*WSIZE(%ebx), %eax
	clc
	subl (%ebx), %eax
	movl %eax, 2*WSIZE(%ebx)
	movl WSIZE(%ebx), %eax
	sbbl -WSIZE(%ebx), %eax
	movl %eax, WSIZE(%ebx)
	movl %ebx, _GlobalSp
	incl _GlobalTp
	incl _GlobalTp
	xor %eax, %eax
	ret
L_umstar:
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl %ecx, %eax
	mull (%ebx)
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	xor %eax, %eax				
	ret
L_umslashmod:
	movl _GlobalSp, %ebx
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, _GlobalSp
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl $0, %edx
	movl (%ebx), %eax
	divl %ecx
	cmpl $0, %eax
	jne umslashmod_ovflow
	movl (%ebx), %edx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	divl %ecx
	jmp umslashmod_exit
umslashmod_ovflow:
	addl $WSIZE, %ebx
	movl $-1, %edx
	movl $-1, %eax
umslashmod_exit:	
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %eax, (%ebx)
	incl _GlobalTp
	xor %eax, %eax		
	ret
L_mstar:
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl %ecx, %eax
	imull (%ebx)
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %edx, (%ebx)
	xor %eax, %eax		
	ret
L_mplus:
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %ecx
	movl %ebx, _GlobalSp
	incl _GlobalTp
	addl %eax, %ebx
	movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %eax
	clc
	addl %ecx, %eax
	js mplus1
	adcl $0, %edx
mplus1:	movl %eax, (%ebx)
	movl %edx, -WSIZE(%ebx)
	xor %eax, %eax
	ret
L_mslash:
	movl $WSIZE, %eax
        incl _GlobalTp
        movl _GlobalSp, %ebx
	addl %eax, %ebx
        movl (%ebx), %ecx
	incl _GlobalTp
	addl %eax, %ebx
	movl %ebx, _GlobalSp
        cmpl $0, %ecx
        je mslash1
	addl %eax, %ebx
        movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %eax
        idivl %ecx
        movl %eax, (%ebx)
	xor %eax, %eax		
	ret
mslash1:			
        movl $E_DIV_ZERO, %eax
        ret
L_udmstar:
	# multiply unsigned double and unsigned to give triple length product
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	mull %ecx
	movl %edx, -WSIZE(%ebx)
	movl %eax, (%ebx)
	addl $WSIZE, %ebx
	movl %ecx, %eax
	mull (%ebx)
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl (%ebx), %eax
	subl $WSIZE, %ebx
	clc
	addl %edx, %eax
	movl %eax, WSIZE(%ebx)
	# subl $WSIZE, %ebx
	movl (%ebx), %eax
	adcl $0, %eax
	movl %eax, (%ebx)
	xor %eax, %eax 		
	ret
L_utmslash:
	# divide unsigned triple length by unsigned to give ud quotient
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %ecx		# divisor in ecx
	addl $WSIZE, %ebx
	movl (%ebx), %eax		# ut3
	movl $0, %edx
	divl %ecx			# ut3/u
	pushl %ebx			# keep local stack ptr
	movl _GlobalSp, %ebx
	movl %eax, -4*WSIZE(%ebx)	# q3
	movl %edx, -5*WSIZE(%ebx)	# r3
	popl %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax		# ut2
	movl $0, %edx
	divl %ecx			# ut2/u
	pushl %ebx
	movl _GlobalSp, %ebx
	movl %eax, -2*WSIZE(%ebx)	# q2
	movl %edx, -3*WSIZE(%ebx)	# r2
	popl %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax		# ut1
	movl $0, %edx
	divl %ecx			# ut1/u
	pushl %ebx
	movl _GlobalSp, %ebx
	movl %eax, (%ebx)		# q1
	movl %edx, -WSIZE(%ebx)		# r1
	movl -5*WSIZE(%ebx), %edx	# r3 << 32
	movl $0, %eax
	divl %ecx			# (r3 << 32)/u
	movl %eax, -6*WSIZE(%ebx)	# q4
	movl %edx, -7*WSIZE(%ebx)	# r4
	movl -3*WSIZE(%ebx), %edx	# r2 << 32
	movl $0, %eax
	divl %ecx			# (r2 << 32)/u
	movl %eax, -8*WSIZE(%ebx)	# q5
	movl %edx, -9*WSIZE(%ebx)	# r5
	movl -7*WSIZE(%ebx), %edx	# r4 << 32
	movl $0, %eax
	divl %ecx			# (r4 << 32)/u
	movl %eax, -10*WSIZE(%ebx)	# q6
	movl %edx, -11*WSIZE(%ebx)	# r6
	movl -WSIZE(%ebx), %eax		# r1
	addl -9*WSIZE(%ebx), %eax	# r1 + r5
	addl -11*WSIZE(%ebx), %eax	# r1 + r5 + r6
	movl $0, %edx
	divl %ecx
	addl -10*WSIZE(%ebx), %eax	# q7 + q6
	addl -8*WSIZE(%ebx), %eax	# q7 + q6 + q5
	addl (%ebx), %eax		# q7 + q6 + q5 + q1
	popl %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	pushl %ebx
	movl _GlobalSp, %ebx
	movl -2*WSIZE(%ebx), %eax	# q2
	addl -6*WSIZE(%ebx), %eax	# q2 + q4
	popl %ebx
	movl %eax, (%ebx)
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	addl $2, _GlobalTp
	xor %eax, %eax
	ret
_L_mstarslash:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	addl $WSIZE, %ebx
	movl (%ebx), %eax
	addl $WSIZE, %ebx
	xorl (%ebx), %eax
	shrl $31, %eax
	pushl %eax	# keep sign of result -- negative is nonzero
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	call L_abs
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	call _L_dabs
	movl _GlobalSp, %ebx
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	call L_udmstar
	movl _GlobalSp, %ebx
	subl $WSIZE, %ebx
	movl %ebx, _GlobalSp
	call L_utmslash	
	popl %eax
	cmpl $0, %eax
	jnz mstarslash_neg
	xor %eax, %eax
	ret
mstarslash_neg:
	call L_dnegate
	xor %eax, %eax
	ret
L_fmslashmod:
	movl _GlobalSp, %ebx
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, _GlobalSp
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %eax
	idivl %ecx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %eax, (%ebx)
	cmpl $0, %edx
	jge L_fmslashmodexit
	decl %eax		# floor the result
	movl %eax, (%ebx)
	addl $WSIZE, %ebx
	addl %ecx, (%ebx)
L_fmslashmodexit:		
	incl _GlobalTp
	xor %eax, %eax
	ret
L_smslashrem:
	movl _GlobalSp, %ebx
	movl $WSIZE, %eax
	addl %eax, %ebx
	movl %ebx, _GlobalSp
	movl (%ebx), %ecx
	addl %eax, %ebx
	movl (%ebx), %edx
	addl %eax, %ebx
	movl (%ebx), %eax
	idivl %ecx
	movl %edx, (%ebx)
	subl $WSIZE, %ebx
	movl %eax, (%ebx)
	incl _GlobalTp
	xor %eax, %eax		
	ret
L_stod:
	movl _GlobalSp, %ebx
	movl WSIZE(%ebx), %eax
	pushl %edx
	cdq
	movl %edx, (%ebx)
	popl %edx
	movl $WSIZE, %eax
	subl %eax, %ebx
	movl %ebx, _GlobalSp
	mov _GlobalTp, %ebx
	movb $OP_IVAL, (%ebx)
	decl _GlobalTp
	xor %eax, %eax	
	ret
L_stof:
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        incl _GlobalTp
        movl _GlobalSp, %ebx
        fildl (%ebx)
        movl _GlobalTp, %ebx
        movb $OP_IVAL, (%ebx)
        decl %ebx
        movb $OP_IVAL, (%ebx)
	decl _GlobalTp
	decl _GlobalTp
        movl _GlobalSp, %ebx
	movl $WSIZE, %eax
        subl %eax, %ebx
        fstpl (%ebx)
	sall $1, %eax
        subl %eax, _GlobalSp
	xor %eax, %eax
        ret
L_dtof:
	movl $WSIZE, %eax
        movl _GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %eax
	xchgl WSIZE(%ebx), %eax
	movl %eax, (%ebx)
        fildq (%ebx)
        fstpl (%ebx)
	xor %eax, %eax	
	ret	
L_ftos:
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        movl _GlobalSp, %ebx
        fldl (%ebx)
        addl %eax, %ebx
        fistpl (%ebx)
        incl _GlobalTp
        movl _GlobalTp, %ebx
        incl %ebx
        movb $OP_IVAL, (%ebx)
	xor %eax, %eax
        ret
L_ftod:
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
	addl %eax, %ebx
	fldl (%ebx)
	subl %eax, %ebx
	fnstcw (%ebx)
	movl (%ebx), %ecx	# save NDP control word	
	movl %ecx, %edx
	movb $12, %dh		
	movl %edx, (%ebx)
	fldcw (%ebx)
	addl %eax, %ebx	
	fistpq (%ebx)
	subl %eax, %ebx
	movl %ecx, (%ebx)
	fldcw (%ebx)		# restore NDP control word
	addl %eax, %ebx 
	movl (%ebx), %eax
	xchgl WSIZE(%ebx), %eax
	movl %eax, (%ebx)
	xor %eax, %eax	
	ret	
L_degtorad:
	fldl FCONST_180
        movl _GlobalSp, %ebx
        addl $WSIZE, %ebx
        fldl (%ebx)
        fdivp %st, %st(1)
        fldpi
        fmulp %st, %st(1)
        fstpl (%ebx)
        ret
L_radtodeg:
        movl _GlobalSp, %ebx
        addl $WSIZE, %ebx
        fldl (%ebx)
        fldpi
	fxch
        fdivp %st, %st(1)
        fldl FCONST_180
        fmulp %st, %st(1)
        fstpl (%ebx)
        ret
L_fne:
	call L_feq
	movl _GlobalSp, %ebx
	notl WSIZE(%ebx)
	ret
L_feq:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	movl _GlobalSp, %ebx
	fldl (%ebx)
	addl %eax, _GlobalSp
	addl %eax, _GlobalSp
	movl _GlobalSp, %ebx
	fldl (%ebx)
	fucompp
	fnstsw %ax
	andb $69, %ah
	xorb $64, %ah
	jne flt2
	jmp flt1
L_flt:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	movl _GlobalSp, %ebx
	fldl (%ebx)
	addl %eax, _GlobalSp
	addl %eax, _GlobalSp
	movl _GlobalSp, %ebx
	fcompl (%ebx)
	fnstsw %ax
	andb $69, %ah
	jne flt2
flt1:	movl _GlobalSp, %ebx
	movl $-1, WSIZE(%ebx)
	jmp fltexit
flt2:   movl _GlobalSp, %ebx
	movl $0, WSIZE(%ebx)
fltexit:
	movl $4, %eax
	addl %eax, _GlobalTp
	movl _GlobalTp, %ebx
	movb $OP_IVAL, (%ebx)
	decl _GlobalTp
	xor %eax, %eax
	ret
L_fgt:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	movl _GlobalSp, %ebx
	fldl (%ebx)
	addl %eax, _GlobalSp
	addl %eax, _GlobalSp
	movl _GlobalSp, %ebx
	fcompl (%ebx)
	fnstsw %ax
	andb $69, %ah
	cmpb $1, %ah
	jne flt2
	jmp flt1	
L_fle:
	call L_2over
	call L_2over
	call L_feq
	call L_push
	call L_flt
	call L_pop
	call L_or
	ret
L_fge:
	call L_2over
	call L_2over
	call L_feq
	call L_push
	call L_fgt
	call L_pop
	call L_or
	ret
L_fzeroeq:
	movl $WSIZE, %eax
	movl _GlobalSp, %ebx
	addl %eax, %ebx
	movl (%ebx), %ecx
	movl %ebx, _GlobalSp
	addl %eax, %ebx
	movl (%ebx), %eax
	shll $1, %eax
	orl %ecx, %eax
	jnz fzeroeq2
fzeroeq1:	
	movl $-1, (%ebx)
	jmp fzeroeqexit
fzeroeq2:
	movl $0, (%ebx)
fzeroeqexit:	
	movl _GlobalTp, %ebx
	incl %ebx
	incl %ebx
	movb $OP_IVAL, (%ebx)
	decl %ebx
	movl %ebx, _GlobalTp
	xor %eax, %eax
	ret
L_fzerolt:
	movl $WSIZE, %eax
	addl %eax, _GlobalSp
	movl _GlobalSp, %ebx
	fldl (%ebx)
	add %eax, %ebx
	fldz
	fcompp	
	fnstsw %ax
	andb $69, %ah
	jne fzeroeq2		
	jmp fzeroeq1
L_fzerogt:
	ret				
L_fadd:
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        movl _GlobalSp, %ebx
        fldl (%ebx)
	sall $1, %eax
        addl %eax, %ebx
        faddl (%ebx)
        fstpl (%ebx)
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        incl _GlobalTp
	incl _GlobalTp
	xor %eax, %eax
        ret
L_fsub:
	movl $WSIZE, %eax
	sall $1, %eax
	addl $WSIZE, %eax
        addl %eax, _GlobalSp
        movl _GlobalSp, %ebx
        fldl (%ebx)
	subl $WSIZE, %eax
        subl %eax, %ebx
        fsubl (%ebx)
        addl %eax, %ebx
        fstpl (%ebx)
	movl $WSIZE, %eax
        subl %eax, _GlobalSp
        incl _GlobalTp
	incl _GlobalTp
	xor %eax, %eax
        ret
L_fmul:
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        movl _GlobalSp, %ebx
        fldl (%ebx)
	sall $1, %eax
        addl %eax, %ebx
        fmull (%ebx)
        fstpl (%ebx)
	sarl $1, %eax
        addl %eax, _GlobalSp
        incl _GlobalTp
	incl _GlobalTp
	xor %eax, %eax
        ret
L_fdiv:
	movl $WSIZE, %eax
        addl %eax, _GlobalSp
        movl _GlobalSp, %ebx
        fldl (%ebx)
        ftst
        jnz fdiv1
        subl %eax, _GlobalSp
        movl $E_DIV_ZERO, %eax
        jmp fdivexit
fdiv1:  addl $WSIZE, %ebx
	addl $WSIZE, %ebx
        fdivrl (%ebx)
        fstpl (%ebx)
        addl %eax, _GlobalSp
        incl _GlobalTp
	incl _GlobalTp
fdivexit:
	xor %eax, %eax
	ret
L_fabs:
        movl _GlobalSp, %ebx
        addl $WSIZE, %ebx
        fldl (%ebx)
        fabs
        fstpl (%ebx)
        ret
L_fneg:
        movl _GlobalSp, %ebx
        addl $WSIZE, %ebx
        fldl (%ebx)
        fchs
        fstpl (%ebx)
        ret
L_floor:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	movl WSIZE(%ebx), %eax
	pushl %eax
	movl (%ebx), %eax
	pushl %eax
	call _floor
	addl $8, %esp
	fstpl (%ebx)
	xor %eax, %eax		
	ret
L_fround:
	movl _GlobalSp, %ebx
	addl $WSIZE, %ebx
	fldl (%ebx)
	frndint
	fstpl (%ebx)
	ret
L_fsqrt:
	movl _GlobalSp, %ebx
	fldl WSIZE(%ebx)
	fsqrt
	fstpl WSIZE(%ebx)
	ret
L_fcos:
	movl _GlobalSp, %ebx
	fldl WSIZE(%ebx)
	fcos
	fstpl WSIZE(%ebx)
	ret
L_fsin:
	movl _GlobalSp, %ebx
	fldl WSIZE(%ebx)
	fsin
	fstpl WSIZE(%ebx)
	ret
L_fatan2:
	movl _GlobalSp, %ebx
	addl $2*WSIZE, %ebx
	fldl WSIZE(%ebx)
	fldl -WSIZE(%ebx)
	fpatan
	fstpl WSIZE(%ebx)
	movl %ebx, _GlobalSp
	incl _GlobalTp
	incl _GlobalTp
	ret
L_fpow:
#        add _GlobalSp, WSIZE
#        mov ebx, _GlobalSp
#        FLD Q[ebx]
#        add ebx, 2*WSIZE
#        FLD Q[ebx]
#        FYL2X
#        # FLD1
#        # FSCALE
#        FSTP Q[ebx]
#        add _GlobalSp, WSIZE
#        add _GlobalTp, 2
        ret
#

	.comm _GlobalSp,4
	.comm _GlobalTp,4
	.comm _GlobalIp,4
	.comm _GlobalRp,4
	.comm _GlobalRtp,4
	.comm _BottomOfStack,4
	.comm _BottomOfReturnStack,4
	.comm _BottomOfTypeStack,4
	.comm _BottomOfReturnTypeStack,4
	.comm _Base,4
	.comm _State,4
	.comm _pTIB,4
	.comm _TIB,256
	.comm _WordBuf,256
	.comm _NumberCount,4
	.comm _NumberBuf,256

	