C       VXPERROR
C
C       Purpose: Display VAXPLOT error message.
C
C       Syntax: CALL VXPERROR (E_MODULE, IERR)
C
C       Input Parameters:
C         E_MODULE -- module number (see VXPERROR.INC for constants)
C         IERR     -- error number
C       Ouput Parameters: NONE
C
C       Author: K. Myneni
C       Date: 8 Jan 90
C       Updated: 12 Feb 90
C       Updated: 20 July 90 -- KM
C       Updated: 31 Aug 90  -- KM
C
        SUBROUTINE VXPERROR (E_MODULE, IERR)

        INCLUDE 'VXPERROR.INC'

        IF (IERR.EQ.0) RETURN       ! error code 0 always indicates no error.

        TYPE *, CHAR(7)             ! beep

        IF (E_MODULE.EQ.E_XYREAD) THEN

           CALL TYPE_XYREAD_ERROR (IERR)

        ELSE IF (E_MODULE.EQ.E_OPEN_GW) THEN
         
           GO TO (210) IERR

           TYPE *, 'Undiagnosed error in module OPEN_GRAPHICS_WINDOW.'
           GO TO 290
 
 210       TYPE *, 'Exceeding maximum number of windows.'
           GO TO 290
 
 290       CONTINUE
      

        ELSE IF (E_MODULE.EQ.E_DRAW_PLOT) THEN

           GO TO (310) IERR

           TYPE *, 'Undiagnosed error in module DRAW_PLOT.'
           GO TO 390

 310       TYPE *, 'Addressing non-existent plot.'
           GO TO 390

 390       CONTINUE
      
        ELSE IF (E_MODULE.EQ.E_WINDOW_EXT) THEN

           GO TO (410, 420) IERR

           TYPE *, 'Undiagnosed error in module WINDOW_EXTREMA.'
           GO TO 490

 410       TYPE *, 'Bad window number.'
           GO TO 490

 420       TYPE *, 'Non-existent window.'
           GO TO 490

 490       CONTINUE

        ELSE IF (E_MODULE.EQ.E_RESCALE) THEN

           GO TO (510, 520, 530, 540, 550, 560) IERR

           TYPE *, 'Undiagnosed error in module RESCALE.'
           GO TO 590

 510       TYPE *, 'Rescale expression is empty.'
           GO TO 590
 520       TYPE *, 'Missing X or Y operand.'
           GO TO 590
 530       TYPE *, 'Rescale expression not recognized.'
           GO TO 590
 540       TYPE *, 'Invalid operator.'
           GO TO 590
 550       TYPE *, 'Attempt to take log of number <= 0.'
           GO TO 590
 560       TYPE *, 'Divide by zero error.'
           GO TO 590

 590       CONTINUE
   
        ELSE IF (E_MODULE.EQ.E_XYWRITE) THEN

           GO TO (610) IERR

           TYPE *, 'Undiagnosed error in module XYWRITE.'
           GO TO 690

 610       TYPE *, 'File already exists.'
           GO TO 690

 690       CONTINUE

        ELSE IF (E_MODULE.EQ.E_OPTION_SELECT) THEN

           GO TO (710) IERR

           TYPE *, 'Undiagnosed error in module OPTION_SELECT.'
           GO TO 790

 710       TYPE *, 'Exceeding maximum number of plots in window.'
           GO TO 790

 790       CONTINUE

        ELSE IF (E_MODULE.EQ.E_SAVE_DATA) THEN

           GO TO (810) IERR

           TYPE *, 'Undiagnosed error in module SAVE_DATA_SET.'
           GO TO 890

 810       TYPE *, 'No data displayed in window for active plot.'
           GO TO 890

 890       CONTINUE

        ELSE IF (E_MODULE.EQ.E_V_POLYFIT) THEN

           GO TO (910, 920, 930) IERR

           TYPE *, 'Undiagnosed error in module V_POLYFIT.'
           GO TO 990

 910       TYPE *, 'No data in window domain for current plot.'
           GO TO 990

 920       TYPE *, 'Insufficient number of points for fit.'
           GO TO 990

 930       TYPE *, 'Exceeded maximum number of terms.'
           GO TO 990

 990       CONTINUE
           TYPE *, 'Fit aborted.'


	ELSE IF (E_MODULE.EQ.E_V_AREA) THEN

	   GO TO (1010, 1020) IERR

	   TYPE *, 'Undiagnosed error in module V_AREA.'
	   GO TO 1090

 1010	   TYPE *, 'No data in window domain for current plot.'
	   GO TO 1090

 1020	   TYPE *, 'Cannot compute area:'
	   TYPE *, '  Only 1 point in window domain.'
	   GO TO 1090

 1090	   CONTINUE

	ELSE IF (E_MODULE.EQ.E_V_DERIV) THEN

	   GO TO (1110, 1120, 1130) IERR

	   TYPE *, 'Undiagnosed error in module V_DERIVATIVE.'
	   GO TO 1190

 1110	   TYPE *, 'No data in window domain for current plot.'
	   GO TO 1190

 1120	   TYPE *, 'Cannot compute derivative:'
	   TYPE *, '  Only 1 point in window domain.'
	   GO TO 1190

 1130	   TYPE *, 'Two points have the same X value.'
	   TYPE *, 'Derivative computation aborted.'
	   GO TO 1190

 1190	   CONTINUE

        END IF


        RETURN
        END

