C     VXPMENU.FOR
C
C     This file contains menu modules for use with VAXPLOT.FOR
C
C     Created: 7 Feb 90
C
C     Updated: 12 Feb 90
C
C ---------------------------------------------------------------------
C
C     SHOW_OPTIONS_MENU
C
C     Purpose: Show the options menu.
C     Syntax: CALL SHOW_OPTIONS_MENU (NW)
C     Input Parameters: 
C       NW -- graphics window number
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 7 Feb 90
C
      SUBROUTINE SHOW_OPTIONS_MENU (NW)
      
      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPMENU.INC'

      JCW = NW                       ! set the current window number
      
      N_OPTS = N_MAIN_OPTS

      DO J = 1, N_OPTS
         OP(J) = MAIN_OP(J)
      END DO

      CALL MAKE_MENU(' OPTIONS ', DEF_MENU_WIDTH, DEF_MENU_HEIGHT)

      RETURN
      END
C 	
C ------------------------------------------------------------------
C
C    SHOW_FILE_MENU
C
C       
      SUBROUTINE SHOW_FILE_MENU
      
      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPMENU.INC'

      N_OPTS = N_FILE_OPTS

      DO J = 1, N_OPTS
         OP(J) = FILE_OP(J)
      END DO

      CALL MAKE_MENU (' FILE ', DEF_MENU_WIDTH, DEF_MENU_HEIGHT)

      RETURN
      END
C
C --------------------------------------------------------------------
C
C     SHOW_PLOT_MENU
C
      SUBROUTINE SHOW_PLOT_MENU

      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPDISP.INC'
      INCLUDE 'VXPMENU.INC'

      N_OPTS = N_PLOT_OPTS
C
C     The following DO loop is used to modify any of the plot menu
C     options based on the display variables (e.g. the FREEZE option).
C

      DO J = 1, N_OPTS
         IF (INDEX(PLOT_OP(J), 'FREEZE').NE.0) THEN
            IF (GW(JCW).FREEZE) THEN
               PLOT_OP(J) = ' UN-FREEZE '
            ELSE
               PLOT_OP(J) = ' FREEZE '
            END IF
         END IF
      END DO
C
C     Set up the options array
C
      DO J = 1, N_OPTS
         OP(J) = PLOT_OP(J)
      END DO

      CALL MAKE_MENU (' PLOT ', DEF_MENU_WIDTH, DEF_MENU_HEIGHT)

      RETURN
      END
C
C --------------------------------------------------------------------
C
C     SHOW_SYMBOL_MENU
C
      SUBROUTINE SHOW_SYMBOL_MENU

      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPDISP.INC'
      INCLUDE 'VXPMENU.INC'

      N_OPTS = N_SYMBOL_OPTS
C
C     Set up the options array
C
      DO J = 1, N_OPTS
         OP(J) = SYMBOL_OP(J)
      END DO

      CALL MAKE_MENU (' SYMBOL ', DEF_MENU_WIDTH, DEF_MENU_HEIGHT)

      RETURN
      END
C
C -------------------------------------------------------------------
C
C     SHOW_MATH_MENU
C
      SUBROUTINE SHOW_MATH_MENU

      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPMENU.INC'

      N_OPTS = N_MATH_OPTS

      DO J = 1, N_OPTS
         OP(J) = MATH_OP(J)
      END DO

      CALL MAKE_MENU (' MATH ', DEF_MENU_WIDTH, DEF_MENU_HEIGHT)

      RETURN
      END
C
C
C ---------------------------------------------------------------
C
C     MENU_POINTER
C
C
C
      SUBROUTINE MENU_POINTER

      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPMENU.INC'

      INCLUDE 'SYS$LIBRARY:UISENTRY'
      INCLUDE 'SYS$LIBRARY:UISUSRDEF'

      INTEGER STATUS

      STATUS = UIS$GET_POINTER_POSITION (MW.VD_ID, MW.WD_ID, RX, RY)
      
      IF (STATUS.EQ.0) RETURN

      K = 10 - INT(RY/2.)	  ! option number

      IF ((K.EQ.OPTION_NUMBER).OR.(K.LT.1).OR.(K.GT.N_OPTS)) THEN
         RETURN
      ELSE
         YP = 20. - FLOAT(2*OPTION_NUMBER) + 1.
         CALL UIS$TEXT (MW.VD_ID, 30, OP(OPTION_NUMBER), 1., YP)
        
         OPTION_NUMBER = K
         YP = 20. - FLOAT(2*OPTION_NUMBER) + 1.
         CALL UIS$TEXT (MW.VD_ID, 31, OP(OPTION_NUMBER), 1., YP)

      END IF

      RETURN
      END
C
C --------------------------------------------------------------------
C
C     MENU_BUTTON
C
C
C
      SUBROUTINE MENU_BUTTON 

      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPMENU.INC'
      
      CHARACTER*20 OPTION
      
      OPTION(1:) = OP(OPTION_NUMBER)(1:)

C
C     Delete the menu and return
C 
      CALL UIS$DELETE_DISPLAY (MW.VD_ID)
      MW.VD_ID = 0

      IF (INDEX(OPTION, 'FILE').NE.0) THEN

         CALL SHOW_FILE_MENU

      ELSE IF (INDEX(OPTION, 'PLOT').NE.0) THEN

         CALL SHOW_PLOT_MENU

      ELSE IF (INDEX(OPTION, 'SYMBOL').NE.0) THEN

	 CALL SHOW_SYMBOL_MENU

      ELSE IF (INDEX(OPTION, 'MATH').NE.0) THEN

         CALL SHOW_MATH_MENU

      ELSE IF (INDEX(OPTION, 'EXIT').NE.0) THEN

         RETURN
      
      ELSE IF (INDEX(OPTION, 'COMMAND').NE.0) THEN

         CALL COMMAND_MODE (JCW)

      ELSE

         CALL OPTION_SELECT (JCW, OPTION)
      
      END IF

      RETURN
      END
C
C --------------------------------------------------------------------
C
C     MAKE_MENU
C 
C     Purpose: Create a menu window at the top left of the active
C              window.
C     Syntax: CALL MAKE_MENU (NAME, WIDTH, HEIGHT)
C     Input Parameters:
C        NAME   -- title of menu window
C        WIDTH  -- width in cm
C        HEIGHT -- height in cm
C     Output Parameters: NONE
C     Author: K. Myneni
C     Date: 9 Feb 90
C     Notes: Attribute blocks 30 and 31 are reserved for use here.
C
      SUBROUTINE MAKE_MENU (MENU_NAME, WIDTH, HEIGHT)
      
      INCLUDE 'VAXPLOT.INC'
      INCLUDE 'VXPDISP.INC'
      INCLUDE 'VXPMENU.INC'

      INCLUDE 'SYS$LIBRARY:UISENTRY'
      INCLUDE 'SYS$LIBRARY:UISUSRDEF'
      
      CHARACTER*(*) MENU_NAME
      INTEGER BUTTON_STATE
      INTEGER BUTTON_STATUS
      REAL R_MENU(2), G_MENU(2), B_MENU(2)
      RECORD /WINDOW_PLACEMENT/ MWP		
      
      EXTERNAL MENU_POINTER
      EXTERNAL MENU_BUTTON

C
C     If another menu is open, delete it
C 
      IF (MW.VD_ID.NE.0) THEN

         CALL UIS$DELETE_DISPLAY (MW.VD_ID)

      END IF

      CALL UIS$GET_VIEWPORT_POSITION (GW(JCW).WD_ID, RX, RY)
      CALL UIS$GET_VIEWPORT_SIZE (GW(JCW).WD_ID, RW, RH)

      MW.VCM_ID = UIS$CREATE_COLOR_MAP (2) 

      MW.VD_ID = UIS$CREATE_DISPLAY (1., 1., 10., 20.,  
     +   WIDTH, HEIGHT, MW.VCM_ID)
      
      R_MENU(1) = R_MAP(3)
      R_MENU(2) = R_MAP(4)
      G_MENU(1) = G_MAP(3)
      G_MENU(2) = G_MAP(4)
      B_MENU(1) = B_MAP(3)
      B_MENU(2) = B_MAP(4)
		
      CALL UIS$SET_COLORS (MW.VD_ID, 0, 2,
     +    R_MENU, G_MENU, B_MENU)

      CALL UIS$SET_WRITING_MODE (MW.VD_ID, 0, 30, UIS$C_MODE_REPL)
C
C     Set character width and size, and writing
C     color.
C 
      CHW = 0.8
      CHH = 1.6

      CALL UIS$SET_CHAR_SIZE (MW.VD_ID, 30, 30, 'A', CHW, CHH)
      CALL UIS$SET_CHAR_SPACING (MW.VD_ID, 30, 30, 0.1, 0.0)
      CALL UIS$SET_WRITING_INDEX (MW.VD_ID, 30, 30, 1)
      CALL UIS$SET_WRITING_MODE (MW.VD_ID, 30, 31, UIS$C_MODE_REPLN)
      MWP.CODE_1 = WDPL$C_ABS_POS_X
      MWP.ATTRIB_1 = RX
      MWP.CODE_2 = WDPL$C_ABS_POS_Y
      MWP.ATTRIB_2 = RY + RH - HEIGHT
      MWP.CODE_3 = WDPL$C_ATTRIBUTES
      MWP.ATTRIB_3 = WDPL$M_NOKB_ICON .OR. WDPL$M_NOMENU_ICON
      MWP.END = WDPL$C_END_OF_LIST

      MW.WD_ID = UIS$CREATE_WINDOW (MW.VD_ID, 'SYS$WORKSTATION',
     +   MENU_NAME, , , , , , , MWP)

C
C     Display options list.
C
      DO J = 1, N_OPTS
        YP = 20. - FLOAT(2*J) + 1.
	CALL UIS$TEXT (MW.VD_ID, 30, OP(J), 1., YP)
      END DO
      
      OPTION_NUMBER = N_OPTS           ! initialize option number

      K = UIS$SET_POINTER_POSITION (MW.VD_ID, MW.WD_ID, 5., 10.)

C
C     Wait until the button is released.
C
      BUTTON_STATUS = UIS$GET_BUTTONS (MW.WD_ID, BUTTON_STATE)
      DO WHILE (.NOT.BTEST(BUTTON_STATE, UIS$M_POINTER_BUTTON_3))
	 BUTTON_STATUS = UIS$GET_BUTTONS (MW.WD_ID, BUTTON_STATE)
      END DO
      
      CALL UIS$SET_POINTER_AST (MW.VD_ID, MW.WD_ID, MENU_POINTER, 
     +    0)
      CALL UIS$SET_BUTTON_AST (MW.VD_ID, MW.WD_ID, MENU_BUTTON,
     +    0, MENU_KEYBUF)


      RETURN
      END
C
C
C							
			
