\ xyhpfile.4th
\
\ XYPLOT module to read binary data files written by
\   the Agilent 54624A scope data acquisition software (hp.4th)
\
\ Requires:
\
\	matrix.4th
\	strings.4th
\	files.4th
\
\ Revisions:
\
\	2002-12-18  created K. Myneni
\	2002-12-30  fixed xyplot problem for creating large datasets from
\			dsinfo structure  KM

2000000 constant MAX_HP_SAMPLES
4 constant MAX_CHANNELS

create hpbuf MAX_HP_SAMPLES 2* allot	\ transient buffer for i/o

MAX_HP_SAMPLES 2 fmatrix hp_waveform

\ The file header structure contains the following fields:
\
\	Number of samples			4-byte int
\	Number of averages			4-byte int
\	Number of channels			4-byte int
\	List of channel numbers			8 bytes
\	Channel vertical scales (volts/div)	64 bytes (8 dfloats)
\	Channel offsets (volts)			64 bytes (8 dfloats)
\	Channel couplings (AC, DC, GND)		32 bytes (8 4-byte strings)
\	Timebase range				8-byte dfloat
\	Timebase offset				8-byte dfloat
\		
  0  constant  HDR_NSAMPLES
  4  constant  HDR_NAVERAGES
  8  constant  HDR_NCHANNELS
 12  constant  HDR_CHANNELS
 20  constant  HDR_VSCALES
 84  constant  HDR_VOFFSETS
148  constant  HDR_COUPLINGS
180  constant  HDR_TIME_RANGE
188  constant  HDR_TIME_OFFSET
256  constant  HDR_SIZE

create   f_hdr HDR_SIZE allot
variable f_id
variable f_ns				\ number of samples
variable f_nc				\ number of channels
create   f_ch  MAX_CHANNELS allot	\ channel list

variable channel
variable ch_index
fvariable time_range
fvariable time_offset
MAX_CHANNELS 1 fmatrix vscales
MAX_CHANNELS 1 fmatrix voffsets

: read_hp_file ( ^file channel -- error | read specified channel from file)
	channel !
	count R/O open-file
	if drop -1 exit then		\ error opening file
	f_id !
	f_hdr HDR_SIZE f_id @ read-file
	drop HDR_SIZE <> 
	if f_id @ close-file drop -2 exit then	\ can't read header from file
	f_hdr HDR_NSAMPLES  + @ f_ns !
	f_hdr HDR_NCHANNELS + @ f_nc !
	f_ch MAX_CHANNELS erase
	f_hdr HDR_CHANNELS + f_ch MAX_CHANNELS cmove
	f_nc @ 0 ?do f_hdr HDR_VSCALES + i dfloats + f@
	  i 1+ 1 vscales fmat! loop
	f_nc @ 0 ?do f_hdr HDR_VOFFSETS + i dfloats + f@
	  i 1+ 1 voffsets fmat! loop
	f_hdr HDR_TIME_RANGE + f@ time_range f!
	f_hdr HDR_TIME_OFFSET + f@ time_offset f!
	
	\ Search the channel list in the file for the desired channel

	0 begin dup f_ch + c@ channel @ <> over f_nc @ < and while 1+ repeat
	dup ch_index !	\ channel list index of desired channel
	f_ch + c@ channel @ <> if 
	  f_id @ close-file drop -3 exit	\ desired channel not in list 
	then 

	\ Position the file pointer at the start of the desired channel data

	ch_index @ f_ns @ * 2* HDR_SIZE + s>d f_id @ reposition-file drop
	hpbuf f_ns @ 2* f_id @ read-file drop  \ read raw data into trans. buf
	f_ns @ 2* <> if 
	  f_id @ close-file drop -4 exit	\ error reading waveform
	then
	f_id @ close-file drop
	
	\ Scale and transfer the raw data into the waveform matrix

	f_ns @ 2 hp_waveform mat_size!	\ set size of the matrix
 
	hpbuf
	f_ns @ 1+ 1 ?do
	  dup w@ 128 - s>f 256e f/ 
	  ch_index @ 1+ 1 vscales fmat@ f* 
	  ch_index @ 1+ 1 voffsets fmat@ f+ 			\ y value
	  i 1- s>f time_range f@ f* f_ns @ s>f f/ time_offset f@ f+ \ x value
	  fswap 
	  2 i hp_waveform frow!
	  2+
	loop
	drop
	0	\ no error
;
 
create wfm_filename 256 allot
DatasetInfo dswfm
	  
: xy_read_hp ( -- | user interface )
	c" Enter the waveform filename:" get_input
	if wfm_filename strcpy else drop exit then
	c" Enter the channel number:" get_input
	if string>s channel ! else drop exit then
	
	wfm_filename channel @ read_hp_file
	dup if
	  ." Error " . ." while reading " wfm_filename count type cr
	  exit
	then
	drop

	\ Successfully read the waveform from the file. Now create
	\ a new xyplot dataset.

	c"  " 1+ dswfm DHEADER + !
	wfm_filename count
	s" (" strcat
	channel @ s>string count strcat
	s" )" strcat strpck 
	1+ dswfm DNAME + !
	256 dswfm DTYPE + !		\ double precision fp type
	hp_waveform mat_size@ drop dswfm DNPTS + !
	2 dswfm DSIZE + !
	hp_waveform 2 cells + dswfm DDATA + !

	dswfm make_ds


;

MN_FILE c" Read HP File" c" xy_read_hp" add_menu_item

	
	    	 
	  
	