\ anim3.4th
\
\  Copyright (c) 2005 Krishna Myneni
\  Provided under the GNU General Public License
\
\  Revisions:
\         2005-01-21  created  km

DatasetInfo dscube 
PlotInfo    plcube 

 8 3 fmatrix cube_data      \ 3D cube data
 8 2 fmatrix cube_xydata    \ Tranformed x,y data for cube
16 2 fmatrix cube_plotdata  \ directed line plot data for drawing cube

\ Fill out a dataset info structure on the new set

c" CubeAnimation" 1+ dscube DNAME ! \ name of new dataset
c" 3D Rotation of Cube" 1+ dscube DHEADER !
256 dscube DTYPE  !		\ real dataset (double precision)
 16 dscube DNPTS  !		\ 10 points in the set
2 dscube DSIZE  !		\ dimension is 2 (x, y)
cube_plotdata 2 cells + dscube DDATA  !	\ pointer to data


\ Fill out a plot info structure on the new plot

0 plcube PTYPE  !		\ plot type
1 plcube PSYM  !		\ plot symbol
2 plcube PCOLOR  !		\ plot color

\ Current view angles

fvariable theta       \ elevation angle in degrees
fvariable phi         \ azimuthal angle in degrees

fvariable sintheta
fvariable costheta
fvariable sinphi
fvariable cosphi

3 3 fmatrix R-Matrix   \ rotation matrix

: >R-Matrix ( ftheta fphi -- | compute rotation matrix given angles )
	    deg>rad  fsincos  cosphi   f!  sinphi   f!
	    deg>rad  fsincos  costheta f!  sintheta f!

	    costheta f@ cosphi f@  f*  	costheta f@ sinphi f@  f*  
	    sintheta f@ fnegate 3                     1 R-Matrix  frow!
	    sinphi   f@ fnegate   cosphi   f@  0e  3  2 R-Matrix  frow!
	    sintheta f@ cosphi f@  f* 	sintheta f@ sinphi f@  f*  
	    costheta f@  3                            3 R-Matrix  frow! ;


: Transform-Coords ( -- | apply rotation matrix to cube )
	9 1 DO
	  1 1 R-Matrix fmat@  i 1 cube_data fmat@ f*
	  1 2 R-Matrix fmat@  i 2 cube_data fmat@ f* f+
	  1 3 R-Matrix fmat@  i 3 cube_data fmat@ f* f+  \ x'
          i 1 cube_xydata fmat!
	  2 1 R-Matrix fmat@  i 1 cube_data fmat@ f*
	  2 2 R-Matrix fmat@  i 2 cube_data fmat@ f* f+  
	  2 3 R-Matrix fmat@  i 3 cube_data fmat@ f* f+  \ y'
	  i 2 cube_xydata fmat!
	LOOP ; 

: cubexy>plotxy ( n m -- | copy point n from cube_xydata to 
           point m of cube_plotdata )
	 >r cube_xydata frow@ r> cube_plotdata frow! ;
	  

: xydata>linedata ( -- | generate a directed line plot for the cube)
	5 1 DO  i i cubexy>plotxy  LOOP  1 5  cubexy>plotxy  \ Top square
	9 5 DO  i i 1+ cubexy>plotxy  LOOP  5 10  cubexy>plotxy
	6 11  cubexy>plotxy
	2 12  cubexy>plotxy
	3 13  cubexy>plotxy
	7 14  cubexy>plotxy
	8 15  cubexy>plotxy
	4 16  cubexy>plotxy
;

: project_cube ( -- | rotate and project cube onto x,y plane)
	theta f@ phi f@ >R-Matrix   \ compute rotation matrix
	Transform-Coords            \ transform cube coordinates
	xydata>linedata             \ make 2D line drawing
;


: rotate_cube ( -- )
	project_cube
	dscube ->npts 0 DO
	  i 1+ cube_plotdata frow@ drop
	  i dscube !xy
	LOOP
;


\ anim3 is the timer signal handler

: anim3 ( n -- )
        drop              \ drop the signal number
	false draw_plot
	rotate_cube
	true draw_plot
	theta f@ 1e f+ theta f!  \ increment by 1 degrees
	phi   f@ 1e f+ phi   f!  \ "
;

: start_anim3 ( -- | start the animation )
	?active dscube get_ds
	-2e -2e 2e 2e set_window_limits
	['] anim3  SIGALRM  forth-signal  drop
	100 100 set-timer    \ update drawing every 100 ms
;

: stop_anim3 ( -- | stop the animation )
	SIG_IGN  SIGALRM  forth-signal drop 
	?active set_ds_extrema ;


\ initialize the 8 vertices of the unit length cube
\
\   x     y     z
  -.5e  -.5e  0.5e   3   1  cube_data  frow!
  0.5e  -.5e  0.5e   3   2  cube_data  frow!
  0.5e  0.5e  0.5e   3   3  cube_data  frow!
  -.5e  0.5e  0.5e   3   4  cube_data  frow!
  -.5e  -.5e  -.5e   3   5  cube_data  frow!
  0.5e  -.5e  -.5e   3   6  cube_data  frow! 
  0.5e  0.5e  -.5e   3   7  cube_data  frow!
  -.5e  0.5e  -.5e   3   8  cube_data  frow!

\ Set initial view angles

0e theta f!
0e phi   f!

\ Compute the x, y plot data
project_cube
dscube make_ds  0 plcube PDSET  !  \ make a dataset in xyplot
plcube make_plot	\ make a plot in xyplot 
0 0 set_grid_lines	\ turn off x and y axes grid lines
-2e -2e 2e 2e set_window_limits

MN_FILE  c" Start Cube"   c" start_anim3"  add_menu_item
MN_FILE  c" Stop  Cube"   c" stop_anim3"   add_menu_item

start_anim3			\ start the animation

