\ anim1.4th
\
\  Simple animation demo of wavepacket propagation for xyplot.
\  This module demonstrates creating a new dataset, creating a 
\  plot associated with the set, and animation of the plot. 
\  The initial wavepacket moves to the left and, while doing so, 
\  becomes attenuated and broadened. A short optical pulse 
\  propagating in a dispersive and absorptive medium may show 
\  this kind of behavior.
\
\  Copyright (c) 1999--2005 Krishna Myneni
\  Provided under the GNU General Public License
\
\  Revisions:
\
\    1-19-2005  use signal handler to perform animation.  km
\    1-20-2005  updated for better efficiency of PROPAGATE1; drop
\                 the signal number passed to the handler.  km

DatasetInfo dsanim 
PlotInfo    planim 
create anim_xydata 200 2 * 4 * allot

\ Fill out a dataset info structure on the new set

c" Animation" 1+ dsanim DNAME ! \ name of new dataset
c" Example of creating and animating a dataset" 1+ dsanim DHEADER !
0 dsanim DTYPE  !		\ real dataset
200 dsanim DNPTS  !		\ 200 points in the set
2 dsanim DSIZE  !		\ dimension is 2 (x, y)
anim_xydata dsanim DDATA  !	\ pointer to data


\ Fill out a plot info structure on the new plot

	
0 planim PTYPE  !		\ plot type
1 planim PSYM  !		\ plot symbol
2 planim PCOLOR  !		\ plot color


: set_anim_data ( -- | initialize data buffer with 200 pts of y=x/10 )
	\ x goes from -10 to 10 in steps of 0.1 
	200 0 do
	  -10e .1e i s>f f* f+
	  fdup
	  anim_xydata i 2* SFSIZE * + 
	  dup >r sf! 
	  0.1e f* r> SFSIZE + sf!
	loop ;
	   
fvariable phase
fvariable deltax
fvariable atten
fvariable width

: initialize1 ( -- | set initial params for animation waveform )
     0e phase f!
     0e deltax f!
     1e atten f!
     1e width f!
;


: propagate1 ( -- | modify the animation dataset )
	dsanim ->npts 0 do
	  i dsanim @xy
	  fdrop fdup 
	  deltax f@ f+ 
	  fdup fdup f* fnegate width f@ f/ fexp fswap
	  10e f* phase f@ f+ fcos f*
	  atten f@ f*
	  i dsanim !xy
	loop
	\ ?active set_ds_extrema
	deltax f@ .03e f+ deltax f!
	atten f@ 0.99e f* atten f!
	width f@ .04e f+ width f!
	phase f@ 0.2e f+ phase f!
;


\ anim1 is the timer signal handler

: anim1 ( n -- | animate the active plot by modulating and translating it )
        drop              \ drop the signal number
	false draw_plot
	propagate1
	true draw_plot
;

: start_anim1 ( -- | start the animation )
	?active dsanim get_ds
	-10e -1e 10e 1e set_window_limits
	initialize1
	['] anim1  SIGALRM  forth-signal  drop
	100 100 set-timer    \ update waveform every 100 ms
;

: halt_anim1 ( -- | stop the animation )
	SIG_IGN  SIGALRM  forth-signal drop 
	?active set_ds_extrema ;

set_anim_data
dsanim make_ds  planim PDSET  !	\ make a dataset in xyplot
planim make_plot	\ make a plot in xyplot 
0 0 set_grid_lines	\ turn off x and y axes grid lines

MN_FILE  c" Start Animation"  c" start_anim1"  add_menu_item
MN_FILE  c" Stop Animation"   c" halt_anim1"   add_menu_item

start_anim1			\ start the animation




