\ xypolyfit.4th   
\                                                               
\  Polynomial fitting routine for xyplot                          
\
\ Adapted from the routine polfit in P.R. Bevington,
\ "Data Reduction and Error Analysis for the Physical Sciences"
\
\ Requires:
\
\       determ.4th
\
\ Revisions:
\              
\	1998-10-02  Adapted for LabForth KM             
\	1999-03-29  Adapted for kForth KM  
\	2000-03-15  Adapted for xyplot KM
\	2000-04-15  Added user query for order KM
\ 	2000-12-12  Indexing mods and use of fmatrix  KM
\       2005-01-13  use DatasetInfo structure creation word and update usage  KM
\       2007-06-02  use FSL arrays and matrices  KM

0  value  Nterms     \ number of terms (order - 1)               
0  value  Nmax                                                   
0  value  Npts
                                                   
fvariable chisq                                                 
fvariable xterm                                                 
fvariable yterm                                                 
fvariable delta                                                 
                                                                
\ Floating point matrices used by polfit                        
                                                                
FLOAT DMATRIX  apfit{{
FLOAT DARRAY   sumx{                                    
FLOAT DARRAY   sumy{

0 ptr params{     \ holds address of parameter array
0 ptr pDS


\ ds = address of dataset info structure                    
\ a = address of fmatrix for receiving fitted parameters        
\ n = order of fitting polynomial                               
                                                                
: polfit ( ds a n -- chi-square | perform polynomial fit )     
    1+ to Nterms                                                 
    to params{  to pDS                                                    
    & apfit{{ Nterms Nterms }}malloc
    & sumy{ Nterms }malloc                              
    Nterms 2* 1- to Nmax                                       
    & sumx{ Nmax }malloc                                    
    pDS DNPTS @ to Npts
    
    sumx{ }fzero  sumy{ }fzero  params{ }fzero
    0e chisq F!
    
    Npts 0 DO
	I pDS @xy	\ fetch the i^th point
	fswap		\ fy fx
	1e
	Nmax 0 DO
	    fdup sumx{ I } F@ F+ sumx{ I } F! fover F*
	LOOP  xterm F!

	fover
	Nterms 0 DO
	    fdup sumy{ I } F@ F+ sumy{ I } F! fover F*
	LOOP  yterm F!

	fover fdup F* chisq F@ F+ chisq F!
	fdrop fdrop
    LOOP

    Nterms 0 DO
	Nterms 0 DO
	    sumx{ I J + 1- } F@  apfit{{ J I }} F!
	LOOP
    LOOP

    apfit{{ determ  fdup delta F!
    F0= IF
	& sumx{ }free  & sumy{ }free  & apfit{{ }}free
	0e EXIT
    THEN

    Nterms 0 DO
	Nterms 0 DO
	    Nterms 0 DO
		sumx{ I J + 1- } F@  apfit{{ J I }} F!
	    LOOP
	    sumy{ I } F@  apfit{{ I J }} F!
	LOOP
	apfit{{ determ  delta F@ F/  params{ I } F!
    LOOP

\ Calculate chi-squared

    Nterms 0 DO
	chisq F@ params{ I } F@ sumy{ I } F@ F* 2e F* F- chisq F!
	Nterms 0 DO
	    sumx{ J I + 1- } F@  params{ I } F@ F*  params{ J } F@ F*
	    chisq F@ F+ chisq F!
	LOOP
    LOOP


    chisq F@
    Npts Nterms - s>f
    F/

    \ cleanup 
    & sumx{ }free  & sumy{ }free  & apfit{{ }}free
;



DatasetInfo  ds_pdata 
DatasetInfo  ds_pfit 
PlotInfo     pl_pfit

FLOAT DARRAY  poly_params{

0 value Norder

: xypoly ( n -- | fit a polynomial of order n to the active dataset )
    to Norder
    ?active dup 0 >= IF
	ds_pdata get_ds  0 >= IF
	    & poly_params{ Norder 1+ }malloc 
	    ds_pdata poly_params{ norder polfit
	    ." Reduced chi-square = " f. cr 
	    Nterms 0 DO
		I . 2 spaces poly_params{ I } F@ f. cr
	    LOOP
	    
	    ds_pdata DNPTS @ 100 > IF
		  \ Compute fitted curve at same abscissas as data
	    ELSE
		  \ Computed fitted curve on a grid of 100 points
	    THEN

	    & poly_params{ }free
	THEN
    THEN ;


: xypolyfit ( -- | prompt user and fit the active data set )
    ?active 0 >= IF
	c" Enter the order of the fitting polynomial:" get_input
	IF  string>s xypoly  ELSE  drop  THEN
    THEN ;

\ add "Poly Fit" as an item in the math menu

MN_MATH " Poly Fit" " xypolyfit draw_window" add_menu_item

