\ polyfit.4th   
\                                                               
\  Polynomial fitting routine in Forth                          
\
\ Adapted from the routine polfit in P.R. Bevington,
\ "Data Reduction and Error Analysis for the Physical Sciences"
\
\ Requires the following FSL modules:
\
\       lufact.4th
\       dets.4th
\
\ Revisions:
\              
\	10-02-98 Adapted for LabForth KM             
\	03-29-99 Adapted for kForth KM  
\       01-01-02 Cleaned up the code (no functional changes)  KM
\       2007-06-12  merged polfit from xypolyfit.4th to this module as polfit2;
\                     factor common code, and use FSL-style arrays and modules.

0  value  Nterms     \ number of terms (order - 1)               
0  value  Nmax                                                   
0  value  Npts

fvariable chisq                                                 
fvariable xterm                                                 
fvariable yterm                                                 
fvariable delta                                                 
                                                                
\ Floating point matrices used by polfit                        
                                                                
FLOAT DMATRIX  apfit{{
FLOAT DARRAY   sumx{                                    
FLOAT DARRAY   sumy{

0 ptr params{     \ holds address of parameter array

: }fzero ( 'array n -- | zero the n-element float array)
    FLOATS erase ;


LUMATRIX lmat

: determ ( 'mat N -- fdet | return the determinant of an NxN matrix )
    lmat swap lu-malloc
    lmat lufact
    lmat det
    lmat lu-free ;                                                 



\ Common code for versions of polfit is factored in polfit0; the
\ arrays sumx{ and sumy{ and the fvariable chisq must be set up
\ prior to calling polfit0

: polfit0 ( -- fdet | perform fitting and return determinant )
    Nterms 0 DO
	Nterms 0 DO  sumx{ I J + } F@  apfit{{ J I }} F!  LOOP
    LOOP

    apfit{{ Nterms determ  fdup delta F!
    F0= IF 0e EXIT  THEN

    Nterms 0 DO
	Nterms 0 DO
	    Nterms 0 DO  sumx{ I J + } F@  apfit{{ J I }} F!  LOOP
	    sumy{ I } F@  apfit{{ I J }} F!
	LOOP
	apfit{{ Nterms determ  delta F@ F/  params{ I } F!
    LOOP

\ Calculate chi-squared

    Nterms 0 DO
	chisq F@ params{ I } F@ sumy{ I } F@ F* 2e F* F- chisq F!
	Nterms 0 DO
	    sumx{ J I + } F@  params{ I } F@ F*  params{ J } F@ F*
	    chisq F@ F+ chisq F!
	LOOP
    LOOP

    chisq F@  Npts Nterms - s>f  F/ ;


\ The specialized variants below, polfit and polfit2, allow for data
\   to be passed to the polynomial fitter in either separate
\   x and y floating point arrays (polfit), or through an xyplot
\   dataset information structure (polfit2).

0  ptr  xd{
0  ptr  yd{

\ x = address of fmatrix containing x values                    
\ y = address of fmatrix containing y values                    
\ a = address of fmatrix for receiving fitted parameters        
\ n = order of fitting polynomial                               
\ np = number of points

: polfit ( 'x 'y 'a  n  np -- chi-square | perform polynomial fit )     
    to Npts  1+ to Nterms  to params{  to xd{  to yd{
    & apfit{{ Nterms Nterms }}malloc
    & sumy{ Nterms }malloc                              
    Nterms 2* 1- to Nmax                                       
    & sumx{ Nmax }malloc

    sumx{ Nmax }fzero  sumy{ Nterms }fzero  params{ Nterms }fzero
    0e chisq F!
    
    Npts 0 DO
	yd{ I } F@  xd{ I } F@  \ fy fx
	1e
	Nmax 0 DO  fdup sumx{ I } F@ F+ sumx{ I } F! fover F*  LOOP  xterm F!

	fover
	Nterms 0 DO  fdup sumy{ I } F@ F+ sumy{ I } F! fover F*  LOOP  yterm F!

	fover fdup F* chisq F@ F+ chisq F!
	fdrop fdrop
    LOOP

    polfit0

    \ cleanup
    & sumx{ }free  & sumy{ }free  & apfit{{ }}free
 ;
                                                               

0 ptr pDS

\ ds = address of dataset info structure                    
\ a = address of fmatrix for receiving fitted parameters        
\ n = order of fitting polynomial                               
                                                                
: polfit2 ( ds 'a n -- chi-square | perform polynomial fit )     
    1+ to Nterms                                                 
    to params{  to pDS                                                    
    & apfit{{ Nterms Nterms }}malloc
    & sumy{ Nterms }malloc                              
    Nterms 2* 1- to Nmax                                       
    & sumx{ Nmax }malloc                                    
    pDS DNPTS @ to Npts
    
    sumx{ Nmax }fzero  sumy{ Nterms }fzero  params{ Nterms }fzero
    0e chisq F!
    
    Npts 0 DO
	I pDS @xy	\ fetch the i^th point
	fswap		\ fy fx
	1e
	Nmax 0 DO  fdup sumx{ I } F@ F+ sumx{ I } F! fover F*  LOOP  xterm F!

	fover
	Nterms 0 DO  fdup sumy{ I } F@ F+ sumy{ I } F! fover F*  LOOP  yterm F!

	fover fdup F* chisq F@ F+ chisq F!
	fdrop fdrop
    LOOP

    polfit0

    \ cleanup 
    & sumx{ }free  & sumy{ }free  & apfit{{ }}free
;
