\ peak_search.4th
\
\  Purpose: Find all peak positions and heights in data contained
\             in x, y arrays. The argument ipw is used to
\             discriminate against minor fluctuations in data that
\             may be detected as a peak. ipw should be set to the
\             number of points over which the peak should maintain
\             a smooth shape.
\
\ Krishna Myneni, 04-01-2001
\
\ Requires:
\
\	derivative.4th
\	polyfit.4th
\
\ Revisions:
\       2007-06-12  rewritten for use with FSL-style arrays and modules
\

\ 1024 1024 * constant MAX_PEAKSEARCH_PTS
\ MAX_PEAKSEARCH_PTS 2 fmatrix pksin
FLOAT DMATRIX pksin{{

32768 constant MAX_PEAKS
\ MAX_PEAKS 2 fmatrix peaks
MAX_PEAKS 2 FLOAT MATRIX peaks{{

32 constant MAX_WIDTH

0 value Npeaks
0 ptr input_m{{

variable input_npts
variable ipw
variable ys1
variable ys2
variable ips
variable ipe

fvariable xpeak
fvariable ypeak
fvariable xpk2
fvariable xoffs

3          FLOAT ARRAY  xp{
3          FLOAT ARRAY  yp{
MAX_WIDTH  FLOAT ARRAY  xpp{
MAX_WIDTH  FLOAT ARRAY  ypp{
3          FLOAT ARRAY  pcoeffs{

: peak_search ( fm np ipw -- ierr)

	ipw !  input_npts ! to input_m{{

	\ Copy the input matrix into the pksin matrix

	input_m{{  pksin{{ }}fcopy 

	\ Compute first derivative of data.

	pksin{{ input_npts @  derivative  IF 1 EXIT THEN

	\  Search for X axis crossings.

	0 to Npeaks
	pksin{{ 0 1 }} F@ 0e F< ys1 !	\ is first value negative?

	input_npts @ 1 DO
	  pksin{{ I 1 }} F@ 0e F< ys2 !	
	  ys2 @ ys1 @ <> IF
	    \  X axis crossing found. Is it a peak or a valley?
	    
	    ys1 @ 0 >= IF
	      \ Went from positive to negative so we found a peak.
	      
	      Npeaks 1+ dup MAX_PEAKS <= IF to Npeaks THEN

	      \  Use linear interpolation to determine x value at crossing.

	      pksin{{ I 0 }}    F@  pksin{{ I 1- 1 }} F@  F*
	      pksin{{ I 1- 0 }} F@  pksin{{ I 1 }}    F@  F*
	      F-
	      pksin{{ I 1- 1 }} F@  pksin{{ I 1 }} F@ F- F/ xpeak F!

	      \  Use parabolic fit to original data to determine peak height.

	      pksin{{ I 1- 0 }} F@ fnegate xoffs F!	\ -x(i-1)
	      
	      0e                                xp{ 0 } F!
	      pksin{{ I 0 }}    F@ xoffs F@ F+  xp{ 1 } F!
	      pksin{{ I 1+ 0 }} F@ xoffs F@ F+  xp{ 2 } F!

	      input_m{{ I 1- 1 }} F@  yp{ 0 } F!
	      input_m{{ I 1 }}    F@  yp{ 1 } F!
	      input_m{{ I 1+ 1 }} F@  yp{ 2 } F!

	      xp{ yp{ pcoeffs{ 2 3 polfit fdrop

	      xpeak F@ xoffs F@ F+ xpk2 F!
	      xpk2 F@ fdup F* pcoeffs{ 2 } F@ F*
	      xpk2 F@ pcoeffs{ 1 } F@ F* F+
	      pcoeffs{ 0 } F@ F+  ypeak f!

	      xpeak F@  peaks{{ Npeaks 0 }} F!	\ store peak position
	      ypeak F@  peaks{{ Npeaks 1 }} F!	\ store peak height

	      \  Test for minimum width

	      i ipw @ 2/ - ips !
	      i ipw @ 2/ + ipe !

	      ips @ 1 < ipe @ input_npts @ > or
	      IF 
	        Npeaks 1- to Npeaks	\ reject the peak
	      ELSE
	        ips @ 1 pksin fmat@ xoffs f!

	        ipe @ 1+ ips @ do
	          i 1 pksin fmat@ xoffs f@ f-
	          i ips @ - 1+ 1 xpp fmat!
	          i 2 input_m a@ fmat@
	          i ips @ - 1+ 1 ypp fmat!
	        loop

	        xpp{ ypp{ pcoeffs{ 2 polfit fdrop
	        
	        pcoeffs{ 2 } F@ 0e F>=  IF Npeaks 1- to Npeaks THEN  \ reject the peak
	      THEN
	    THEN
	  THEN
	  ys2 @ ys1 !
	LOOP

	Npeaks 0> IF
	  Npeaks 2 peaks mat_size!	\ set the matrix size
	   0		\ no error
	ELSE
	  2		\ error: no peaks found
	THEN ;
